@@ -17,12 +17,14 @@ using namespace std;
 #include "Magick++/Image.h"
 #include "Magick++/Functions.h"
 #include "Magick++/Pixels.h"
+#include "Magick++/Options.h"
+#include "Magick++/ImageRef.h"
 
 #define AbsoluteValue(x)  ((x) < 0 ? -(x) : (x))
 
-const std::string Magick::borderGeometryDefault = "6x6+0+0";
-const std::string Magick::frameGeometryDefault  = "25x25+6+6";
-const std::string Magick::raiseGeometryDefault  = "6x6+0+0";
+MagickDLLDeclExtern const std::string Magick::borderGeometryDefault = "6x6+0+0";
+MagickDLLDeclExtern const std::string Magick::frameGeometryDefault  = "25x25+6+6";
+MagickDLLDeclExtern const std::string Magick::raiseGeometryDefault  = "6x6+0+0";
 
 //
 // Explicit template instantiations
@@ -32,7 +34,8 @@ const std::string Magick::raiseGeometryDefault  = "6x6+0+0";
 // Friend functions to compare Image objects
 //
 
-int Magick::operator == ( const Magick::Image& left_, const Magick::Image& right_ )
+MagickDLLDecl int Magick::operator == ( const Magick::Image& left_,
+                                        const Magick::Image& right_ )
 {
   // If image pixels and signature are the same, then the image is identical
   return ( ( left_.rows() == right_.rows() ) &&
@@ -40,26 +43,31 @@ int Magick::operator == ( const Magick::Image& left_, const Magick::Image& right
 	   ( left_.signature() == right_.signature() )
 	   );
 }
-int Magick::operator != ( const Magick::Image& left_, const Magick::Image& right_ )
+MagickDLLDecl int Magick::operator != ( const Magick::Image& left_,
+                                        const Magick::Image& right_ )
 {
   return ( ! (left_ == right_) );
 }
-int Magick::operator >  ( const Magick::Image& left_, const Magick::Image& right_ )
+MagickDLLDecl int Magick::operator >  ( const Magick::Image& left_,
+                                        const Magick::Image& right_ )
 {
   return ( !( left_ < right_ ) && ( left_ != right_ ) );
 }
-int Magick::operator <  ( const Magick::Image& left_, const Magick::Image& right_ )
+MagickDLLDecl int Magick::operator <  ( const Magick::Image& left_,
+                                        const Magick::Image& right_ )
 {
   // If image pixels are less, then image is smaller
   return ( ( left_.rows() * left_.columns() ) <
 	   ( right_.rows() * right_.columns() )
 	   );
 }
-int Magick::operator >= ( const Magick::Image& left_, const Magick::Image& right_ )
+MagickDLLDecl int Magick::operator >= ( const Magick::Image& left_,
+                                        const Magick::Image& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-int Magick::operator <= ( const Magick::Image& left_, const Magick::Image& right_ )
+MagickDLLDecl int Magick::operator <= ( const Magick::Image& left_,
+                                        const Magick::Image& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
 }
@@ -232,6 +240,22 @@ Magick::Image::~Image()
   _imgRef = 0;
 }
 
+// Local adaptive threshold image
+// http://www.dai.ed.ac.uk/HIPR2/adpthrsh.htm
+// Width x height define the size of the pixel neighborhood
+// offset = constant to subtract from pixel neighborhood mean
+void Magick::Image::adaptiveThreshold ( const unsigned int width_,
+                                        const unsigned int height_,
+                                        const unsigned int offset_ )
+{
+  ExceptionInfo exceptionInfo;
+  GetExceptionInfo( &exceptionInfo );
+  MagickLib::Image* newImage =
+    AdaptiveThresholdImage( constImage(), width_, height_, offset_, &exceptionInfo );
+  replaceImage( newImage );
+  throwException( exceptionInfo );
+}
+
 // Add noise to image
 void Magick::Image::addNoise( const NoiseType noiseType_ )
 {
@@ -430,6 +454,17 @@ void Magick::Image::colorize ( const unsigned int opacity_,
   colorize( opacity_, opacity_, opacity_, penColor_ );
 }
 
+// Compare current image with another image
+// Sets meanErrorPerPixel, normalizedMaxError, and normalizedMeanError
+// in the current image. False is returned if the images are identical.
+bool Magick::Image::compare ( const Image &reference_ )
+{
+  modifyImage();
+  Image ref = reference_;
+  ref.modifyImage();
+  return static_cast<bool>(IsImagesEqual(image(), ref.image()));
+}
+
 // Composite two images
 void Magick::Image::composite ( const Image &compositeImage_,
 				const int xOffset_,
@@ -556,7 +591,8 @@ void Magick::Image::contrast ( const unsigned int sharpen_ )
 // Convolve image.  Applies a general image convolution kernel to the image.
 //  order_ represents the number of columns and rows in the filter kernel.
 //  kernel_ is an array of doubles representing the convolution kernel.
-void Magick::Image::convolve ( const unsigned int order_, const double *kernel_ )
+void Magick::Image::convolve ( const unsigned int order_,
+                               const double *kernel_ )
 {
   ExceptionInfo exceptionInfo;
   GetExceptionInfo( &exceptionInfo );
@@ -715,7 +751,8 @@ void Magick::Image::flip ( void )
 // Flood-fill color across pixels that match the color of the
 // target pixel and are neighbors of the target pixel.
 // Uses current fuzz setting when determining color match.
-void Magick::Image::floodFillColor( const unsigned int x_, const unsigned int y_,
+void Magick::Image::floodFillColor( const unsigned int x_,
+                                    const unsigned int y_,
 				    const Magick::Color &fillColor_ )
 {
   floodFillTexture( x_, y_, Image( Geometry( 1, 1), fillColor_ ) );
@@ -729,22 +766,26 @@ void Magick::Image::floodFillColor( const Geometry &point_,
 // Flood-fill color across pixels starting at target-pixel and
 // stopping at pixels matching specified border color.
 // Uses current fuzz setting when determining color match.
-void Magick::Image::floodFillColor( const unsigned int x_, const unsigned int y_,
+void Magick::Image::floodFillColor( const unsigned int x_,
+                                    const unsigned int y_,
 				    const Magick::Color &fillColor_,
 				    const Magick::Color &borderColor_ )
 {
-  floodFillTexture( x_, y_, Image( Geometry( 1, 1), fillColor_), borderColor_ );
+  floodFillTexture( x_, y_, Image( Geometry( 1, 1), fillColor_),
+                    borderColor_ );
 }
 void Magick::Image::floodFillColor( const Geometry &point_,
 				    const Magick::Color &fillColor_,
 				    const Magick::Color &borderColor_ )
 {
-  floodFillTexture( point_, Image( Geometry( 1, 1), fillColor_), borderColor_ );
+  floodFillTexture( point_, Image( Geometry( 1, 1), fillColor_),
+                    borderColor_ );
 }
 
 // Floodfill pixels matching color (within fuzz factor) of target
 // pixel(x,y) with replacement opacity value using method.
-void Magick::Image::floodFillOpacity( const unsigned int x_, const unsigned int y_,
+void Magick::Image::floodFillOpacity( const unsigned int x_,
+                                      const unsigned int y_,
                                       const unsigned int opacity_,
                                       const PaintMethod method_ )
 {
@@ -752,14 +793,16 @@ void Magick::Image::floodFillOpacity( const unsigned int x_, const unsigned int
   MatteFloodfillImage ( image(),
                         static_cast<PixelPacket>(pixelColor(x_,y_)),
                         opacity_,
-			static_cast<long>(x_), static_cast<long>(y_), method_ );
+			static_cast<long>(x_), static_cast<long>(y_),
+                        method_ );
   throwImageException();
 }
 
 // Flood-fill texture across pixels that match the color of the
 // target pixel and are neighbors of the target pixel.
 // Uses current fuzz setting when determining color match.
-void Magick::Image::floodFillTexture( const unsigned int x_, const unsigned int y_,
+void Magick::Image::floodFillTexture( const unsigned int x_,
+                                      const unsigned int y_,
 				      const Magick::Image &texture_ )
 {
   modifyImage();
@@ -791,7 +834,8 @@ void Magick::Image::floodFillTexture( const Magick::Geometry &point_,
 // Flood-fill texture across pixels starting at target-pixel and
 // stopping at pixels matching specified border color.
 // Uses current fuzz setting when determining color match.
-void Magick::Image::floodFillTexture( const unsigned int x_, const unsigned int y_,
+void Magick::Image::floodFillTexture( const unsigned int x_,
+                                      const unsigned int y_,
 				      const Magick::Image &texture_,
 				      const Magick::Color &borderColor_ )
 {
@@ -800,12 +844,12 @@ void Magick::Image::floodFillTexture( const unsigned int x_, const unsigned int
   // Set drawing fill pattern
   options()->fillPattern(texture_.constImage());
 
-  ColorFloodfillImage ( image(), // Image *image
-                        options()->drawInfo(), // const DrawInfo *draw_info
-                        static_cast<PixelPacket>(borderColor_), // const PixelPacket target
-                        static_cast<long>(x_), // const long x_offset
-                        static_cast<long>(y_), // const long y_offset
-                        FillToBorderMethod // const PaintMethod method
+  ColorFloodfillImage ( image(),
+                        options()->drawInfo(),
+                        static_cast<PixelPacket>(borderColor_),
+                        static_cast<long>(x_),
+                        static_cast<long>(y_),
+                        FillToBorderMethod
                         );
 
   throwImageException();
@@ -847,7 +891,8 @@ void Magick::Image::frame ( const Geometry &geometry_ )
   replaceImage( newImage );
   throwException( exceptionInfo );
 }
-void Magick::Image::frame ( const unsigned int width_, const unsigned int height_,
+void Magick::Image::frame ( const unsigned int width_,
+                            const unsigned int height_,
 			    const int innerBevel_, const int outerBevel_ )
 {
   FrameInfo info;
@@ -1081,7 +1126,7 @@ void Magick::Image::quantize ( const bool measureError_  )
                  image() );
 
   if ( measureError_ )
-    QuantizationError( image() );
+    GetImageQuantizeError( image() );
 
   // Udate DirectClass representation of pixels
   SyncImage( image() );
@@ -1115,7 +1160,6 @@ void Magick::Image::read ( const std::string &imageSpec_ )
       // Destroy any extra image frames
       MagickLib::Image* next = image->next;
       image->next = 0;
-      image->orphan = true;
       next->previous = 0;
       DestroyImageList( next );
  
@@ -1211,7 +1255,8 @@ void Magick::Image::read ( const unsigned int width_,
   ExceptionInfo exceptionInfo;
   GetExceptionInfo( &exceptionInfo );
   MagickLib::Image* image =
-    ConstituteImage( width_, height_, map_.c_str(), type_, pixels_, &exceptionInfo );
+    ConstituteImage( width_, height_, map_.c_str(), type_, pixels_,
+                     &exceptionInfo );
   replaceImage( image );
   throwException( exceptionInfo );
   if ( image )
@@ -1246,7 +1291,8 @@ void Magick::Image::roll ( const Geometry &roll_ )
   replaceImage( newImage );
   throwException( exceptionInfo );
 }
-void Magick::Image::roll ( const unsigned int columns_, const unsigned int rows_ )
+void Magick::Image::roll ( const unsigned int columns_,
+                           const unsigned int rows_ )
 {
   ExceptionInfo exceptionInfo;
   GetExceptionInfo( &exceptionInfo );
@@ -1620,7 +1666,8 @@ void Magick::Image::write ( const int x_,
 {
   ExceptionInfo exceptionInfo;
   GetExceptionInfo( &exceptionInfo );
-  DispatchImage( image(), x_, y_, columns_, rows_, map_.c_str(), type_, pixels_,
+  DispatchImage( image(), x_, y_, columns_, rows_, map_.c_str(), type_,
+                 pixels_,
     &exceptionInfo);
   throwException( exceptionInfo );
 }
@@ -1921,7 +1968,7 @@ Magick::Image Magick::Image::clipMask ( void  ) const
                     &exceptionInfo);
 
       throwException( exceptionInfo );
-      return Magick::Image( image, new Magick::Options());
+      return Magick::Image( image );
 }
 
 void Magick::Image::colorFuzz ( const double fuzz_ )
@@ -1941,43 +1988,20 @@ void Magick::Image::colorMap ( const unsigned int index_,
 {
   MagickLib::Image* imageptr = image();
 
-  if (index_ > MaxRGB )
+  if (index_ > (MaxColormapSize-1) )
     throwExceptionExplicit( OptionError,
-                            "Index greater than MaxRGB" );
+                            "Colormap index must be less than MaxColormapSize" );
   
   if ( !color_.isValid() )
     throwExceptionExplicit( OptionError,
 			    "Color argument is invalid");
-
   modifyImage();
 
-  if ( !imageptr->colormap || index_ > imageptr->colors-1 )
-    {
+  // Ensure that colormap size is large enough
+  if ( colorMapSize() < (index_+1) )
+    colorMapSize( index_ + 1 );
 
-      if( !imageptr->colormap )
-        {
-          // Allocate colormap
-          imageptr->colormap =
-            static_cast<PixelPacket*>(AcquireMemory((index_+1)*sizeof(PixelPacket)));
-          imageptr->colors = 0;
-        }
-      else
-        {
-          // Re-allocate colormap
-          ReacquireMemory(reinterpret_cast<void **>(&(imageptr->colormap)),
-                          (index_+1)*sizeof(PixelPacket));
-        }
-
-      // Initialize new colormap entries as all black
-      Color black(0,0,0);
-      for( unsigned int i=imageptr->colors; i<index_; i++ )
-        (imageptr->colormap)[i] = black;
-
-      // Change number of colors
-      imageptr->colors = index_+1;
-    }
-
-  // Finally, set color at index in colormap
+  // Set color at index in colormap
   (imageptr->colormap)[index_] = color_;
 }
 // Return color in colormap at index
@@ -1987,7 +2011,7 @@ Magick::Color Magick::Image::colorMap ( const unsigned int index_ ) const
 
   if ( !imageptr->colormap )
     throwExceptionExplicit( OptionError,
-			    "Image does not support a colormap");
+			    "Image does not contain a colormap");
 
   if ( index_ > imageptr->colors-1 )
     throwExceptionExplicit( OptionError,
@@ -1996,6 +2020,49 @@ Magick::Color Magick::Image::colorMap ( const unsigned int index_ ) const
   return Magick::Color( (imageptr->colormap)[index_] );
 }
 
+// Colormap size (number of colormap entries)
+void Magick::Image::colorMapSize ( const unsigned int entries_ )
+{
+  if (entries_ >MaxColormapSize )
+    throwExceptionExplicit( OptionError,
+                            "Colormap entries must not exceed MaxColormapSize" );
+
+  modifyImage();
+
+  MagickLib::Image* imageptr = image();
+
+  if( !imageptr->colormap )
+    {
+      // Allocate colormap
+      imageptr->colormap =
+        static_cast<PixelPacket*>(AcquireMemory(entries_*sizeof(PixelPacket)));
+      imageptr->colors = 0;
+    }
+  else if ( entries_ > imageptr->colors )
+    {
+      // Re-allocate colormap
+      ReacquireMemory(reinterpret_cast<void **>(&(imageptr->colormap)),
+                      (entries_)*sizeof(PixelPacket));
+    }
+
+  // Initialize any new new colormap entries as all black
+  Color black(0,0,0);
+  for( unsigned int i=imageptr->colors; i<(entries_-1); i++ )
+    (imageptr->colormap)[i] = black;
+
+  imageptr->colors = entries_;
+}
+unsigned int Magick::Image::colorMapSize ( void )
+{
+  const MagickLib::Image* imageptr = constImage();
+
+  if ( !imageptr->colormap )
+    throwExceptionExplicit( OptionError,
+			    "Image does not contain a colormap");
+
+  return imageptr->colors;
+}
+
 // Image colorspace
 void Magick::Image::colorSpace( const ColorspaceType colorSpace_ )
 {
@@ -2034,18 +2101,18 @@ void Magick::Image::colorSpace( const ColorspaceType colorSpace_ )
        colorSpace_ == GRAYColorspace )
     {
       // Convert the image from an alternate colorspace representation
-      // In:				Out:
+      // In:			Out:
       // CMYKColorspace		RGBColorspace
-      // RGBColorspace		RGBColorspace (no conversion)
-      // GRAYColorspace		GRAYColorspace (no conversion)
+      // RGBColorspace          RGBColorspace (no conversion)
+      // GRAYColorspace         GRAYColorspace (no conversion)
       // TransparentColorspace	TransparentColorspace (no conversion)
       // OHTAColorspace		RGBColorspace
       // sRGBColorspace		RGBColorspace
       // XYZColorspace		RGBColorspace
-      // YCbCrColorspace		RGBColorspace
+      // YCbCrColorspace	RGBColorspace
       // YCCColorspace		RGBColorspace
       // YIQColorspace		RGBColorspace
-      // YPbPrColorspace		RGBColorspace
+      // YPbPrColorspace        RGBColorspace
       // YUVColorspace		RGBColorspace
       TransformRGBImage( image(), colorSpace_ );
       throwImageException();
@@ -2287,7 +2354,7 @@ void Magick::Image::fontTypeMetrics( const std::string &text_,
   drawInfo->text = const_cast<char *>(text_.c_str());
   GetTypeMetrics( image(), drawInfo, &(metrics->_typeMetric) );
   drawInfo->text = 0;
-};
+}
 
 // Image format string
 std::string Magick::Image::format ( void ) const
@@ -2329,7 +2396,7 @@ Magick::Geometry Magick::Image::geometry ( void ) const
 void Magick::Image::gifDisposeMethod ( const unsigned int disposeMethod_ )
 {
   modifyImage();
-  image()->dispose = disposeMethod_;
+  image()->dispose = (DisposeType) disposeMethod_;
 }
 unsigned int Magick::Image::gifDisposeMethod ( void ) const
 {
@@ -2454,7 +2521,7 @@ std::string Magick::Image::magick ( void ) const
   return constOptions()->magick( );
 }
 
-void Magick::Image::matte ( bool matteFlag_ )
+void Magick::Image::matte ( const bool matteFlag_ )
 {
   modifyImage();
   image()->matte = matteFlag_;
@@ -2499,7 +2566,7 @@ Magick::Color Magick::Image::matteColor ( void ) const
 
 double Magick::Image::meanErrorPerPixel ( void ) const
 {
-  return(constImage()->mean_error_per_pixel);
+  return(constImage()->error.mean_error_per_pixel);
 }
 
 void Magick::Image::monochrome ( const bool monochromeFlag_ )
@@ -2525,12 +2592,12 @@ Magick::Geometry Magick::Image::montageGeometry ( void ) const
 
 double Magick::Image::normalizedMaxError ( void ) const
 {
-  return(constImage()->normalized_maximum_error);
+  return(constImage()->error.normalized_maximum_error);
 }
 
 double Magick::Image::normalizedMeanError ( void ) const
 {
-  return constImage()->normalized_mean_error;
+  return constImage()->error.normalized_mean_error;
 }
 
 void Magick::Image::penColor ( const Color &penColor_ )
@@ -2647,6 +2714,45 @@ Magick::Geometry Magick::Image::page ( void ) const
                    constImage()->page.y < 0 ? true : false);
 }
 
+// Add a named profile to an an image or remove a named profile by
+// passing an empty Blob (use default Blob constructor).
+// Valid names are:
+// "*", "8BIM", "ICM", "IPTC", or a generic profile name.
+void Magick::Image::profile( const std::string name_,
+                             const Magick::Blob &profile_ )
+{
+  modifyImage();
+  int result = ProfileImage( image(), name_.c_str(),
+                             (unsigned char *)profile_.data(),
+                             profile_.length(), true);
+
+  if( !result )
+    throwImageException();
+}
+
+// Retrieve a named profile from the image.
+// Valid names are:
+// "8BIM", "8BIMTEXT", "APP1", "APP1JPEG", "ICC", "ICM", & "IPTC" or
+// an existing generic profile name.
+Magick::Blob Magick::Image::profile( const std::string name_ ) const
+{
+  const MagickLib::Image* image = constImage();
+
+  for (long i=0; i < (long) image->generic_profiles; i++)
+    {
+      if (!GlobExpression(image->generic_profile[i].name,name_.c_str()))
+        continue;      
+      
+      return Blob( (void*)image->generic_profile[i].info,
+                   image->generic_profile[i].length );
+    }
+  
+  Blob blob;
+  Image temp_image = *this;
+  temp_image.write( &blob, name_ );
+  return blob;
+}
+
 void Magick::Image::quality ( const unsigned int quality_ )
 {
   modifyImage();
@@ -2667,7 +2773,8 @@ unsigned int Magick::Image::quantizeColors ( void ) const
   return constOptions()->quantizeColors( );
 }
 
-void Magick::Image::quantizeColorSpace ( const Magick::ColorspaceType colorSpace_ )
+void Magick::Image::quantizeColorSpace
+  ( const Magick::ColorspaceType colorSpace_ )
 {
   modifyImage();
   options()->quantizeColorSpace( colorSpace_ );
@@ -2697,7 +2804,8 @@ unsigned int Magick::Image::quantizeTreeDepth ( void ) const
   return constOptions()->quantizeTreeDepth( );
 }
 
-void Magick::Image::renderingIntent ( const Magick::RenderingIntent renderingIntent_ )
+void Magick::Image::renderingIntent
+  ( const Magick::RenderingIntent renderingIntent_ )
 {
   modifyImage();
   image()->rendering_intent = renderingIntent_;
@@ -2707,7 +2815,8 @@ Magick::RenderingIntent Magick::Image::renderingIntent ( void ) const
   return static_cast<Magick::RenderingIntent>(constImage()->rendering_intent);
 }
 
-void Magick::Image::resolutionUnits ( const Magick::ResolutionType resolutionUnits_ )
+void Magick::Image::resolutionUnits
+  ( const Magick::ResolutionType resolutionUnits_ )
 {
   modifyImage();
   image()->units = resolutionUnits_;
@@ -2730,12 +2839,9 @@ unsigned int Magick::Image::scene ( void ) const
 
 std::string Magick::Image::signature ( const bool force_ ) const
 {
-  // Unlike most operations that modify image attributes, this the
-  // signature operation is not considered a modification of image
-  // data so we do not force a copy of the image if we calculate its
-  // signature.
+  Lock( &_imgRef->_mutexLock );
 
-  // Re-calculate image signature
+  // Re-calculate image signature if necessary
   if ( force_ ||
        !GetImageAttribute(constImage(), "Signature") ||
        constImage()->taint )
@@ -2912,6 +3018,17 @@ unsigned int Magick::Image::subRange ( void ) const
   return constOptions()->subRange( );
 }
 
+// Annotation text encoding (e.g. "UTF-16")
+void Magick::Image::textEncoding ( const std::string &encoding_ )
+{
+  modifyImage();
+  options()->textEncoding( encoding_ );
+}
+std::string Magick::Image::textEncoding ( void ) const
+{
+  return constOptions()->textEncoding( );
+}
+
 void Magick::Image::tileName ( const std::string &tileName_ )
 {
   modifyImage();
@@ -2979,13 +3096,16 @@ Magick::ImageType Magick::Image::type ( void ) const
 
   ExceptionInfo exceptionInfo;
   GetExceptionInfo( &exceptionInfo );
-  ImageType image_type = GetImageType(constImage(), &exceptionInfo);
+  ImageType image_type = constOptions()->type();
+  if ( image_type == UndefinedType )
+    image_type= GetImageType(constImage(), &exceptionInfo);
   throwException( exceptionInfo );
   return image_type;
 }
 void Magick::Image::type ( const Magick::ImageType type_)
 {
   modifyImage();
+  options()->type( type_ );
   SetImageType( image(), type_ );
 }
 
@@ -3080,9 +3200,10 @@ Magick::Image& Magick::Image::operator=( const Magick::Image &image_ )
 
 // Transfers read-only pixels from the image to the pixel cache as
 // defined by the specified region
-const Magick::PixelPacket* Magick::Image::getConstPixels ( const int x_, const int y_,
-                                                           const unsigned int columns_,
-                                                           const unsigned int rows_ ) const
+const Magick::PixelPacket* Magick::Image::getConstPixels
+  ( const int x_, const int y_,
+    const unsigned int columns_,
+    const unsigned int rows_ ) const
 {
   ExceptionInfo exceptionInfo;
   GetExceptionInfo( &exceptionInfo );
@@ -3187,16 +3308,56 @@ void Magick::Image::writePixels ( const Magick::QuantumType quantum_,
 //
 // Construct using Image and Magick::Options
 //
-Magick::Image::Image ( MagickLib::Image* image_,
-                       const Magick::Options* options_ )
-  : _imgRef(new ImageRef( image_, options_))
+Magick::Image::Image ( MagickLib::Image* image_ )
+  : _imgRef(new ImageRef( image_, new Magick::Options() ))
 {
 }
 
+// Get Magick::Options*
+Magick::Options* Magick::Image::options( void )
+{
+  return _imgRef->options();
+}
+const Magick::Options* Magick::Image::constOptions( void ) const
+{
+  return _imgRef->options();
+}
+
+// Get MagickLib::Image*
+MagickLib::Image*& Magick::Image::image( void )
+{
+  return _imgRef->image();
+}
+const MagickLib::Image* Magick::Image::constImage( void ) const
+{
+  return _imgRef->image();
+}
+
+// Get ImageInfo *
+MagickLib::ImageInfo* Magick::Image::imageInfo( void )
+{
+  return _imgRef->options()->imageInfo();
+}
+const MagickLib::ImageInfo * Magick::Image::constImageInfo( void ) const
+{
+  return _imgRef->options()->imageInfo();
+}
+
+// Get QuantizeInfo *
+MagickLib::QuantizeInfo* Magick::Image::quantizeInfo( void )
+{
+  return _imgRef->options()->quantizeInfo();
+}
+const MagickLib::QuantizeInfo * Magick::Image::constQuantizeInfo( void ) const
+{
+  return _imgRef->options()->quantizeInfo();
+}
+
 //
 // Replace current image
 //
-MagickLib::Image * Magick::Image::replaceImage( MagickLib::Image* replacement_ )
+MagickLib::Image * Magick::Image::replaceImage
+  ( MagickLib::Image* replacement_ )
 {
   MagickLib::Image* image;
   
@@ -3286,95 +3447,6 @@ void Magick::Image::unregisterId( void )
   _imgRef->id( -1 );
 }
 
-/////////////////////////////////////////////
-//
-// ImageRef image handle implementation
-//
-/////////////////////////////////////////////
-
-// // Construct with an image and default options
-Magick::ImageRef::ImageRef ( MagickLib::Image * image_ )
-  : _image(image_),
-    _options(new Options),
-    _id(-1),
-    _refCount(1),
-    _mutexLock()
-{
-}
-
-// Construct with an image and options
-// Inserts Image* in image, but copies Options into image.
-Magick::ImageRef::ImageRef ( MagickLib::Image * image_,
-			     const Options * options_ )
-  : _image(image_),
-    _options(0),
-    _id(-1),
-    _refCount(1),
-    _mutexLock()
-{
-  _options = new Options( *options_ );
-}
-
-// Default constructor
-Magick::ImageRef::ImageRef ( void )
-  : _image(0),
-    _options(new Options),
-    _id(-1),
-    _refCount(1),
-    _mutexLock()
-{
-  // Allocate default image
-  _image = AllocateImage( _options->imageInfo() );
-
-  // Test for error and throw exception (like throwImageException())
-  throwException(_image->exception);
-}
-
-// Destructor
-Magick::ImageRef::~ImageRef( void )
-{
-  // Unregister image (if still registered)
-  if( _id > -1 )
-    {
-      DeleteMagickRegistry( _id );
-      _id=-1;
-    }
-
-  // Deallocate image
-  if ( _image )
-    {
-      DestroyImageList( _image );
-      _image = 0;
-    }
-
-  // Deallocate image options
-  delete _options;
-  _options = 0;
-}
-
-// Assign image to reference
-void Magick::ImageRef::image ( MagickLib::Image * image_ )
-{
-  if(_image)
-    DestroyImageList( _image );
-  _image = image_;
-}
-
-// Assign options to reference
-void  Magick::ImageRef::options ( Magick::Options * options_ )
-{
-  delete _options;
-  _options = options_;
-}
-
-// Assign registration id to reference
-void Magick::ImageRef::id ( const long id_ )
-{
-  if( _id > -1 )
-    DeleteMagickRegistry( _id );
-  _id = id_;
-}
-
 //
 // Cleanup class to ensure that ImageMagick singletons are destroyed
 // so as to avoid any resemblence to a memory leak (which seems to
