@@ -11,40 +11,45 @@
 #include <string>
 #include <list>
 
+#include "Magick++/Include.h"
 #include "Magick++/Blob.h"
 #include "Magick++/Color.h"
 #include "Magick++/Drawable.h"
 #include "Magick++/Exception.h"
 #include "Magick++/Geometry.h"
-#include "Magick++/Options.h"
-#include "Magick++/Include.h"
-#include "Magick++/Thread.h"
 #include "Magick++/TypeMetric.h"
 
 namespace Magick
 {
   // Forward declarations
+  class Options;
   class ImageRef;
 
-  extern const std::string borderGeometryDefault;
-  extern const std::string frameGeometryDefault;
-  extern const std::string raiseGeometryDefault;
+  extern MagickDLLDecl const std::string borderGeometryDefault;
+  extern MagickDLLDecl const std::string frameGeometryDefault;
+  extern MagickDLLDecl const std::string raiseGeometryDefault;
 
   // Compare two Image objects regardless of LHS/RHS
   // Image sizes and signatures are used as basis of comparison
-  int operator == ( const Magick::Image& left_, const Magick::Image& right_ );
-  int operator != ( const Magick::Image& left_, const Magick::Image& right_ );
-  int operator >  ( const Magick::Image& left_, const Magick::Image& right_ );
-  int operator <  ( const Magick::Image& left_, const Magick::Image& right_ );
-  int operator >= ( const Magick::Image& left_, const Magick::Image& right_ );
-  int operator <= ( const Magick::Image& left_, const Magick::Image& right_ );
+  int MagickDLLDecl operator == ( const Magick::Image& left_,
+                                  const Magick::Image& right_ );
+  int MagickDLLDecl operator != ( const Magick::Image& left_,
+                                  const Magick::Image& right_ );
+  int MagickDLLDecl operator >  ( const Magick::Image& left_,
+                                  const Magick::Image& right_ );
+  int MagickDLLDecl operator <  ( const Magick::Image& left_,
+                                  const Magick::Image& right_ );
+  int MagickDLLDecl operator >= ( const Magick::Image& left_,
+                                  const Magick::Image& right_ );
+  int MagickDLLDecl operator <= ( const Magick::Image& left_,
+                                  const Magick::Image& right_ );
 
   //
   // Image is the representation of an image.  In reality, it actually
   // a handle object which contains a pointer to a shared reference
   // object (ImageRef). As such, this object is extremely space efficient.
   //
-  class Image
+  class MagickDLLDecl Image
   {
   public:
     // Construct from image file or image specification
@@ -60,13 +65,18 @@ namespace Magick
     Image ( const Blob &blob_, const Geometry &size_ );
 
     // Construct Image of specified size and depth from in-memory BLOB
-    Image ( const Blob &blob_, const Geometry &size, const unsigned int depth );
+    Image ( const Blob &blob_, const Geometry &size,
+            const unsigned int depth );
 
-    // Construct Image of specified size, depth, and format from in-memory BLOB
-    Image ( const Blob &blob_, const Geometry &size, const unsigned int depth_,
+    // Construct Image of specified size, depth, and format from
+    // in-memory BLOB
+    Image ( const Blob &blob_, const Geometry &size,
+            const unsigned int depth_,
 	    const std::string &magick_ );
-    // Construct Image of specified size, and format from in-memory BLOB
-    Image ( const Blob &blob_, const Geometry &size, const std::string &magick_ );
+    // Construct Image of specified size, and format from in-memory
+    // BLOB
+    Image ( const Blob &blob_, const Geometry &size,
+            const std::string &magick_ );
 
     // Construct an image based on an array of raw pixels, of
     // specified type and mapping, in memory
@@ -93,7 +103,15 @@ namespace Magick
     // Image operations
     //
     //////////////////////////////////////////////////////////////////////
-    
+
+    // Local adaptive threshold image
+    // http://www.dai.ed.ac.uk/HIPR2/adpthrsh.htm
+    // Width x height define the size of the pixel neighborhood
+    // offset = constant to subtract from pixel neighborhood mean
+    void            adaptiveThreshold ( const unsigned int width,
+                                        const unsigned int height,
+                                        const unsigned offset = 0 );
+
     // Add noise to image with specified noise type
     void            addNoise ( const NoiseType noiseType_ );
 
@@ -135,10 +153,12 @@ namespace Magick
     // The radius_ parameter specifies the radius of the Gaussian, in
     // pixels, not counting the center pixel.  The sigma_ parameter
     // specifies the standard deviation of the Laplacian, in pixels.
-    void            blur ( const double radius_ = 0.0, const double sigma_ = 1.0  );
+    void            blur ( const double radius_ = 0.0,
+                           const double sigma_ = 1.0  );
     
     // Border image (add border to image)
-    void            border ( const Geometry &geometry_ = borderGeometryDefault );
+    void            border ( const Geometry &geometry_
+                             = borderGeometryDefault );
 
     // Extract channel from image
     void            channel ( const ChannelType channel_ );
@@ -147,11 +167,13 @@ namespace Magick
     // The radius_ parameter specifies the radius of the Gaussian, in
     // pixels, not counting the center pixel.  The sigma_ parameter
     // specifies the standard deviation of the Laplacian, in pixels.
-    void            charcoal ( const double radius_ = 0.0, const double sigma_ = 1.0 );
-    
+    void            charcoal ( const double radius_ = 0.0,
+                               const double sigma_ = 1.0 );
+
     // Chop image (remove vertical or horizontal subregion of image)
-    // FIXME: describe how geometry argument is used to select either horizontal or
-    // vertical subregion of image.
+    // FIXME: describe how geometry argument is used to select either
+    // horizontal or vertical subregion of image.
+
     void            chop ( const Geometry &geometry_ );
     
     // Colorize image with pen color, using specified percent opacity
@@ -166,27 +188,36 @@ namespace Magick
     
     // Comment image (add comment string to image)
     void            comment ( const std::string &comment_ );
-    
+
+    // Compare current image with another image
+    // Sets meanErrorPerPixel, normalizedMaxError, and normalizedMeanError
+    // in the current image. False is returned if the images are identical.
+    bool            compare ( const Image &reference_ );
+
     // Compose an image onto another at specified offset and using
     // specified algorithm
     void            composite ( const Image &compositeImage_,
 				const int xOffset_,
 				const int yOffset_,
-				const CompositeOperator compose_ = InCompositeOp );
+				const CompositeOperator compose_
+                                = InCompositeOp );
     void            composite ( const Image &compositeImage_,
 				const Geometry &offset_,
-				const CompositeOperator compose_ = InCompositeOp );
+				const CompositeOperator compose_
+                                = InCompositeOp );
     void            composite ( const Image &compositeImage_,
 				const GravityType gravity_,
-				const CompositeOperator compose_ = InCompositeOp );
+				const CompositeOperator compose_
+                                = InCompositeOp );
     
     // Contrast image (enhance intensity differences in image)
     void            contrast ( const unsigned int sharpen_ );
 
-    // Convolve image.  Applies a user-specfied convolution to the image.
+    // Convolve image.  Applies a user-specified convolution to the image.
     //  order_ represents the number of columns and rows in the filter kernel.
     //  kernel_ is an array of doubles representing the convolution kernel.
-    void            convolve ( const unsigned int order_, const double *kernel_ );
+    void            convolve ( const unsigned int order_,
+                               const double *kernel_ );
 
     // Crop image (subregion of original image)
     void            crop ( const Geometry &geometry_ );
@@ -213,7 +244,8 @@ namespace Magick
     // The radius_ parameter specifies the radius of the Gaussian, in
     // pixels, not counting the center pixel.  The sigma_ parameter
     // specifies the standard deviation of the Laplacian, in pixels.
-    void            emboss ( const double radius_ = 0.0, const double sigma_ = 1.0);
+    void            emboss ( const double radius_ = 0.0,
+                             const double sigma_ = 1.0);
     
     // Enhance image (minimize noise)
     void            enhance ( void );
@@ -230,7 +262,8 @@ namespace Magick
     // Flood-fill color across pixels that match the color of the
     // target pixel and are neighbors of the target pixel.
     // Uses current fuzz setting when determining color match.
-    void            floodFillColor( const unsigned int x_, const unsigned int y_,
+    void            floodFillColor( const unsigned int x_,
+                                    const unsigned int y_,
 				    const Color &fillColor_ );
     void            floodFillColor( const Geometry &point_,
 				    const Color &fillColor_ );
@@ -238,7 +271,8 @@ namespace Magick
     // Flood-fill color across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    void            floodFillColor( const unsigned int x_, const unsigned int y_,
+    void            floodFillColor( const unsigned int x_,
+                                    const unsigned int y_,
 				    const Color &fillColor_,
 				    const Color &borderColor_ );
     void            floodFillColor( const Geometry &point_,
@@ -247,14 +281,16 @@ namespace Magick
 
     // Floodfill pixels matching color (within fuzz factor) of target
     // pixel(x,y) with replacement opacity value using method.
-    void            floodFillOpacity ( const unsigned int x_, const unsigned int y_,
+    void            floodFillOpacity ( const unsigned int x_,
+                                       const unsigned int y_,
                                        const unsigned int opacity_,
                                        const PaintMethod method_ );
 
     // Flood-fill texture across pixels that match the color of the
     // target pixel and are neighbors of the target pixel.
     // Uses current fuzz setting when determining color match.
-    void            floodFillTexture( const unsigned int x_, const unsigned int y_,
+    void            floodFillTexture( const unsigned int x_,
+                                      const unsigned int y_,
 				      const Image &texture_ );
     void            floodFillTexture( const Geometry &point_,
 				      const Image &texture_ );
@@ -262,7 +298,8 @@ namespace Magick
     // Flood-fill texture across pixels starting at target-pixel and
     // stopping at pixels matching specified border color.
     // Uses current fuzz setting when determining color match.
-    void            floodFillTexture( const unsigned int x_, const unsigned int y_,
+    void            floodFillTexture( const unsigned int x_,
+                                      const unsigned int y_,
 				      const Image &texture_,
 				      const Color &borderColor_ );
     void            floodFillTexture( const Geometry &point_,
@@ -274,8 +311,10 @@ namespace Magick
     
     // Frame image
     void            frame ( const Geometry &geometry_ = frameGeometryDefault );
-    void            frame ( const unsigned int width_, const unsigned int height_,
-			    const int innerBevel_ = 6, const int outerBevel_ = 6 );
+    void            frame ( const unsigned int width_,
+                            const unsigned int height_,
+			    const int innerBevel_ = 6,
+                            const int outerBevel_ = 6 );
     
     // Gamma correct image
     void            gamma ( const double gamma_ );
@@ -294,12 +333,13 @@ namespace Magick
     
     // Label image
     void            label ( const std::string &label_ );
-     
+
     // Magnify image by integral size
     void            magnify ( void );
     
     // Remap image colors with closest color from reference image
-    void            map ( const Image &mapImage_ , const bool dither_ = false );
+    void            map ( const Image &mapImage_ ,
+                          const bool dither_ = false );
     
     // Floodfill designated area with replacement opacity value
     void            matteFloodfill ( const Color &target_ ,
@@ -377,25 +417,28 @@ namespace Magick
     void            read ( const Blob        &blob_,
 			   const Geometry    &size_ );
 
-    // Read single image frame of specified size and depth from in-memory BLOB
+    // Read single image frame of specified size and depth from
+    // in-memory BLOB
     void            read ( const Blob         &blob_,
 			   const Geometry     &size_,
 			   const unsigned int depth_ );
 
-    // Read single image frame of specified size, depth, and format from in-memory BLOB
+    // Read single image frame of specified size, depth, and format
+    // from in-memory BLOB
     void            read ( const Blob         &blob_,
 			   const Geometry     &size_,
 			   const unsigned int depth_,
 			   const std::string  &magick_ );
 
-    // Read single image frame of specified size, and format from in-memory BLOB
+    // Read single image frame of specified size, and format from
+    // in-memory BLOB
     void            read ( const Blob         &blob_,
 			   const Geometry     &size_,
 			   const std::string  &magick_ );
 
-    // Read single image frame from an array of raw pixels, with specified storage
-    // type (ConstituteImage), e.g.
-    // image.read( 640, 480, "RGB", 0, pixels );
+    // Read single image frame from an array of raw pixels, with
+    // specified storage type (ConstituteImage), e.g.
+    //    image.read( 640, 480, "RGB", 0, pixels );
     void            read ( const unsigned int width_,
                            const unsigned int height_,
                            const std::string &map_,
@@ -422,9 +465,9 @@ namespace Magick
     void            scale ( const Geometry &geometry_ );
     
     // Segment (coalesce similar image components) by analyzing the
-    // histograms of the color components and identifying units that are
-    // homogeneous with the fuzzy c-means technique.
-    // Also uses QuantizeColorSpace and Verbose image attributes
+    // histograms of the color components and identifying units that
+    // are homogeneous with the fuzzy c-means technique.  Also uses
+    // QuantizeColorSpace and Verbose image attributes
     void            segment ( const double clusterThreshold_ = 1.0, 
 			      const double smoothingThreshold_ = 1.5 );
     
@@ -437,7 +480,8 @@ namespace Magick
     // The radius_ parameter specifies the radius of the Gaussian, in
     // pixels, not counting the center pixel.  The sigma_ parameter
     // specifies the standard deviation of the Laplacian, in pixels.
-    void            sharpen ( const double radius_ = 0.0, const double sigma_ = 1.0 );
+    void            sharpen ( const double radius_ = 0.0,
+                              const double sigma_ = 1.0 );
 
     // Shave pixels from image edges.
     void            shave ( const Geometry &geometry_ );
@@ -456,8 +500,8 @@ namespace Magick
     // Add a digital watermark to the image (based on second image)
     void            stegano ( const Image &watermark_ );
     
-    // Create an image which appears in stereo when viewed with red-blue glasses
-    // (Red image on left, blue on right)
+    // Create an image which appears in stereo when viewed with
+    // red-blue glasses (Red image on left, blue on right)
     void            stereo ( const Image &rightImage_ );
     
     // Swirl image (image pixels are rotated by degrees)
@@ -475,12 +519,20 @@ namespace Magick
     void            transform ( const Geometry &imageGeometry_,
 				const Geometry &cropGeometry_  );
 
-    // Add matte image to image, setting pixels matching color to transparent
+    // Add matte image to image, setting pixels matching color to
+    // transparent
     void            transparent ( const Color &color_ );
     
     // Trim edges that are the background color from the image
     void            trim ( void );
 
+    // Image representation type (also see type attribute)
+    //   Available types:
+    //    Bilevel        Grayscale       GrayscaleMatte
+    //    Palette        PaletteMatte    TrueColor
+    //    TrueColorMatte ColorSeparation ColorSeparationMatte
+    void            type ( const ImageType type_ );
+
     // Replace image with a sharpened version of the original image
     // using the unsharp mask algorithm.
     //  radius_
@@ -514,9 +566,9 @@ namespace Magick
 			    const std::string &magick_,
 			    const unsigned int depth_ );
 
-    // Write single image frame to an array of pixels with storage type specified
-    // by user (DispatchImage), e.g.
-    // image.write( 0, 0, 640, 1, "RGB", 0, pixels );
+    // Write single image frame to an array of pixels with storage
+    // type specified by user (DispatchImage), e.g.
+    //   image.write( 0, 0, 640, 1, "RGB", 0, pixels );
     void            write ( const int x_,
                             const int y_,
                             const unsigned int columns_,
@@ -542,8 +594,8 @@ namespace Magick
     void            antiAlias( const bool flag_ );
     bool            antiAlias( void );
     
-    // Time in 1/100ths of a second which must expire before displaying
-    // the next image in an animated sequence.
+    // Time in 1/100ths of a second which must expire before
+    // displaying the next image in an animated sequence.
     void            animationDelay ( const unsigned int delay_ );
     unsigned int    animationDelay ( void ) const;
     
@@ -557,7 +609,7 @@ namespace Magick
     Color           backgroundColor ( void ) const;
     
     // Name of texture image to tile onto the image background
-    void            backgroundTexture ( const std::string &backgroundTexture_ );
+    void            backgroundTexture (const std::string &backgroundTexture_ );
     std::string     backgroundTexture ( void ) const;
     
     // Base image width (before transformations)
@@ -582,9 +634,9 @@ namespace Magick
     void            boxColor ( const Color &boxColor_ );
     Color           boxColor ( void ) const;
 
-    // Pixel cache threshold.  Once this threshold is exceeded, all
-    // subsequent pixels cache operations are to/from disk.
-    // This setting is shared by all Image objects.
+    // Pixel cache threshold in megabytes.  Once this memory threshold
+    // is exceeded, all subsequent pixels cache operations are to/from
+    // disk.  This setting is shared by all Image objects.
     static void     cacheThreshold ( const unsigned int threshold_ );
     
     // Chromaticity blue primary point (e.g. x=0.15, y=0.06)
@@ -607,8 +659,8 @@ namespace Magick
     // NOTE: setting a DirectClass image to PseudoClass will result in
     // the loss of color information if the number of colors in the
     // image is greater than the maximum palette size (either 256 or
-    // 65536 entries depending on whether QuantumLeap was specified
-    // when ImageMagick was built).
+    // 65536 entries depending on the value of QuantumDepth when
+    // ImageMagick was built).
     void            classType ( const ClassType class_ );
     ClassType       classType ( void ) const;
 
@@ -623,9 +675,14 @@ namespace Magick
     double          colorFuzz ( void ) const;
     
     // Color at colormap position index_
-    void            colorMap ( const unsigned int index_, const Color &color_ );
+    void            colorMap ( const unsigned int index_,
+                               const Color &color_ );
     Color           colorMap ( const unsigned int index_ ) const;
 
+    // Colormap size (number of colormap entries)
+    void            colorMapSize ( const unsigned int entries_ );
+    unsigned int    colorMapSize ( void );
+
     // Image Color Space
     void            colorSpace( const ColorspaceType colorSpace_ );
     ColorspaceType  colorSpace ( void ) const;
@@ -748,7 +805,7 @@ namespace Magick
     // The mean error per pixel computed when an image is color reduced
     double          meanErrorPerPixel ( void ) const;
 
-   // Tile size and offset within an image montage
+    // Tile size and offset within an image montage
     Geometry        montageGeometry ( void ) const;
 
     // Transform image to black and white
@@ -763,6 +820,10 @@ namespace Magick
     // color reduced.
     double          normalizedMeanError ( void ) const;
 
+    // Preferred size and location of an image canvas.
+    void            page ( const Geometry &pageSize_ );
+    Geometry        page ( void ) const;
+
     // Pen color (deprecated, don't use any more)
     void            penColor ( const Color &penColor_ );
     Color           penColor ( void  ) const;
@@ -772,13 +833,22 @@ namespace Magick
     Image           penTexture ( void  ) const;
 
     // Get/set pixel color at location x & y.
-    void            pixelColor ( const unsigned int x_, const unsigned int y_,
+    void            pixelColor ( const unsigned int x_,
+                                 const unsigned int y_,
 				 const Color &color_ );
-    Color           pixelColor ( const unsigned int x_, const unsigned int y_ ) const;
+    Color           pixelColor ( const unsigned int x_,
+                                 const unsigned int y_ ) const;
 
-    // Preferred size and location of an image canvas.
-    void            page ( const Geometry &pageSize_ );
-    Geometry        page ( void ) const;
+    // Add or remove a named profile to/from the image. Remove the
+    // profile by passing an empty Blob (e.g. Blob()). Valid names are
+    // "*", "8BIM", "ICM", "IPTC", or a user/format-defined profile name.
+    void            profile( const std::string name_,
+                             const Blob &colorProfile_ );
+
+    // Retrieve a named profile from the image. Valid names are:
+    // "8BIM", "8BIMTEXT", "APP1", "APP1JPEG", "ICC", "ICM", & "IPTC"
+    // or an existing user/format-defined profile name.
+    Blob            profile( const std::string name_ ) const;
 
     // JPEG/MIFF/PNG compression level (default 75).
     void            quality ( const unsigned int quality_ );
@@ -824,7 +894,7 @@ namespace Magick
     void            size ( const Geometry &geometry_ );
     Geometry        size ( void ) const;
 
-     // enabled/disable stroke anti-aliasing
+    // enabled/disable stroke anti-aliasing
     void            strokeAntiAlias( const bool flag_ );
     bool            strokeAntiAlias( void ) const;
 
@@ -885,6 +955,10 @@ namespace Magick
     void            subRange ( const unsigned int subRange_ );
     unsigned int    subRange ( void ) const;
 
+    // Annotation text encoding (e.g. "UTF-16")
+    void            textEncoding ( const std::string &encoding_ );
+    std::string     textEncoding ( void ) const;
+
     // Tile name
     void            tileName ( const std::string &tileName_ );
     std::string     tileName ( void ) const;
@@ -910,13 +984,12 @@ namespace Magick
     // Skew to use in Y axis when annotating with text or drawing
     void            transformSkewY ( const double skewy_ );
 
-    // Image representation type.
+    // Image representation type (also see type operation)
     //   Available types:
     //    Bilevel        Grayscale       GrayscaleMatte
     //    Palette        PaletteMatte    TrueColor
     //    TrueColorMatte ColorSeparation ColorSeparationMatte
     ImageType       type ( void ) const;
-    void            type ( const ImageType type_ );
 
     // Print detailed information about the image
     void            verbose ( const bool verboseFlag_ );
@@ -961,16 +1034,18 @@ namespace Magick
 
     // Transfers pixels from the image to the pixel cache as defined
     // by the specified region. Modified pixels may be subsequently
-    // transferred back to the image via syncPixels.
-    // This method is valid for DirectClass images
+    // transferred back to the image via syncPixels.  This method is
+    // valid for DirectClass images.
     PixelPacket* getPixels ( const int x_, const int y_,
-			     const unsigned int columns_, const unsigned int rows_ );
+			     const unsigned int columns_,
+                             const unsigned int rows_ );
 
     // Allocates a pixel cache region to store image pixels as defined
     // by the region rectangle.  This area is subsequently transferred
     // from the pixel cache to the image via syncPixels.
     PixelPacket* setPixels ( const int x_, const int y_,
-			     unsigned int columns_, unsigned int rows_ );
+			     const unsigned int columns_,
+                             const unsigned int rows_ );
 
     // Transfers the image cache pixels to the image.
     void syncPixels ( void );
@@ -994,8 +1069,8 @@ namespace Magick
     //////////////////////////////////////////////////////////////////////
 
 
-    // Construct with image and options
-    Image ( MagickLib::Image* image_, const Magick::Options* options_ );
+    // Construct with MagickLib::Image and default options
+    Image ( MagickLib::Image* image_ );
 
     // Retrieve Image*
     MagickLib::Image*& image( void );
@@ -1009,10 +1084,13 @@ namespace Magick
     MagickLib::ImageInfo * imageInfo( void );
     const MagickLib::ImageInfo * constImageInfo( void ) const;
 
+    // Retrieve QuantizeInfo*
+    MagickLib::QuantizeInfo * quantizeInfo( void );
+    const MagickLib::QuantizeInfo * constQuantizeInfo( void ) const;
+
     // Replace current image (reference counted)
     MagickLib::Image* replaceImage ( MagickLib::Image* replacement_ );
 
-
     // Prepare to update image (copy if reference > 1)
     void            modifyImage ( void );
 
@@ -1029,75 +1107,18 @@ namespace Magick
     ImageRef *      _imgRef;
   };
 
-
-  //
-  // Reference counted access to Image *
-  // (Private implementation class)
-  //
-  class ImageRef {
-    friend class Image; 
-  private:
-    // Construct with an image pointer and default options
-    ImageRef ( MagickLib::Image * image_ );
-    // Construct with an image pointer and options
-    ImageRef ( MagickLib::Image * image_, const Options * options_ );
-    // Construct with null image and default options
-    ImageRef ( void );
-    // Destroy image and options
-    ~ImageRef( void );
-    
-    // Copy constructor and assignment are not supported
-    ImageRef(const ImageRef&);
-    ImageRef& operator=(const ImageRef&);
-    
-    void                 image ( MagickLib::Image * image_ );
-    MagickLib::Image *&  image ( void );
-    
-    void                 options ( Options * options_ );
-    Options *            options ( void );
-
-    void                 id ( const long id_ );
-    long                 id ( void ) const;
-    
-    MagickLib::Image *   _image;    // ImageMagick Image
-    Options *            _options;  // User-specified options
-    long                 _id;       // Registry ID (-1 if not registered)
-    int                  _refCount; // Reference count
-    MutexLock            _mutexLock;// Mutex lock
-  };
-
 } // end of namespace Magick
 
 //
 // Inlines
 //
 
-//
-// ImageRef
-//
-
-// Retrieve image from reference
-inline MagickLib::Image *& Magick::ImageRef::image ( void )
-{
-  return _image;
-}
-
-// Retrieve Options from reference
-inline Magick::Options * Magick::ImageRef::options ( void )
-{
-  return _options;
-}
-
-// Retrieve registration id from reference
-inline long Magick::ImageRef::id ( void ) const
-{
-  return _id;
-}
 
 //
 // Image
 //
 
+
 // Reduce noise in image using a noise peak elimination filter
 inline void Magick::Image::reduceNoise ( void )
 {
@@ -1105,7 +1126,7 @@ inline void Magick::Image::reduceNoise ( void )
 }
 
 // Stroke width for drawing vector objects (default one)
-inline void Magick::Image::lineWidth ( double lineWidth_ )
+inline void Magick::Image::lineWidth ( const double lineWidth_ )
 {
   strokeWidth( lineWidth_ );
 }
@@ -1114,36 +1135,6 @@ inline double Magick::Image::lineWidth ( void ) const
   return strokeWidth( );
 }
 
-// Get MagickLib::Image*
-inline MagickLib::Image*& Magick::Image::image( void )
-{
-  return _imgRef->image();
-}
-inline const MagickLib::Image* Magick::Image::constImage( void ) const
-{
-  return _imgRef->image();
-}
-
-// Get Magick::Options*
-inline Magick::Options* Magick::Image::options( void )
-{
-  return _imgRef->options();
-}
-inline const Magick::Options* Magick::Image::constOptions( void ) const
-{
-  return _imgRef->options();
-}
-
-// Get ImageInfo *
-inline MagickLib::ImageInfo* Magick::Image::imageInfo( void )
-{
-  return _imgRef->options()->imageInfo();
-}
-inline const MagickLib::ImageInfo * Magick::Image::constImageInfo( void ) const
-{
-  return _imgRef->options()->imageInfo();
-}
-
 // Get image storage class
 inline Magick::ClassType Magick::Image::classType ( void ) const
 {
