@@ -28,6 +28,10 @@
 # include <sys/types.h>
 #endif
 
+#if defined(HAVE_INTTYPES_H)
+#  include <inttypes.h>
+#endif
+
 #if defined(macintosh)
 #  include <stat.mac.h>  /* Needed for off_t */
 #endif
@@ -50,6 +54,41 @@ namespace MagickLib
 #undef class
 }
 
+//
+// Provide appropriate DLL imports/exports for Visual C++ and MinGW builds
+//
+#if defined(WIN32) && !defined(__CYGWIN__)
+# define MagickWIN32
+#endif
+#if defined(MagickWIN32)
+#  if defined(_MT) && defined(_DLL) && !defined(_LIB)
+#    define MagickDLLBuild
+#    pragma warning( disable: 4273 )      /* Disable the stupid dll linkage warnings */
+#    if !defined(MAGICK_IMPLEMENTATION)
+#      define MagickDLLDecl __declspec(dllimport)
+#      define MagickDLLDeclExtern extern __declspec(dllimport)
+#      if defined(_VISUALC_)
+#        pragma message( "Magick++ lib DLL import" )
+#      endif
+#    else
+#      define MagickDLLDecl __declspec(dllexport)
+#      define MagickDLLDeclExtern extern __declspec(dllexport)
+#      if defined(_VISUALC_)
+#        pragma message( "Magick++ lib DLL export" )
+#      endif
+#    endif
+#  else
+#    define MagickDLLDecl
+#    define MagickDLLDeclExtern
+#    if defined(_VISUALC_)
+#      pragma message( "Magick++ lib static interface" )
+#    endif
+#  endif
+#else
+#  define MagickDLLDecl
+#  define MagickDLLDeclExtern
+#endif
+
 //
 // Import ImageMagick symbols and types which are used as part of the
 // Magick++ API definition into namespace "Magick".
@@ -68,6 +107,19 @@ namespace Magick
   using MagickLib::DirectClass;
   using MagickLib::PseudoClass;
   
+  // Channel types
+  using MagickLib::ChannelType;
+  using MagickLib::UndefinedChannel;
+  using MagickLib::RedChannel;
+  using MagickLib::CyanChannel;
+  using MagickLib::GreenChannel;
+  using MagickLib::MagentaChannel;
+  using MagickLib::BlueChannel;
+  using MagickLib::YellowChannel;
+  using MagickLib::OpacityChannel;
+  using MagickLib::BlackChannel;
+  using MagickLib::MatteChannel;
+  
   // Color-space types
   using MagickLib::ColorspaceType;
   using MagickLib::UndefinedColorspace;
@@ -128,9 +180,15 @@ namespace Magick
   using MagickLib::Group4Compression;
   using MagickLib::JPEGCompression;
   using MagickLib::LZWCompression;
-  using MagickLib::RunlengthEncodedCompression;
+  using MagickLib::RLECompression;
   using MagickLib::ZipCompression;
 
+  using MagickLib::DisposeType;
+  using MagickLib::UndefinedDispose;
+  using MagickLib::NoneDispose;
+  using MagickLib::BackgroundDispose;
+  using MagickLib::PreviousDispose;
+
   // Endian options
   using MagickLib::EndianType;
   using MagickLib::UndefinedEndian;
@@ -197,14 +255,6 @@ namespace Magick
   using MagickLib::LineInterlace;
   using MagickLib::PlaneInterlace;
   using MagickLib::PartitionInterlace;
-  
-  // Channel types
-  using MagickLib::ChannelType;
-  using MagickLib::UndefinedChannel;
-  using MagickLib::RedChannel;
-  using MagickLib::GreenChannel;
-  using MagickLib::BlueChannel;
-  using MagickLib::MatteChannel;
 
   // Line cap types
   using MagickLib::LineCap;
@@ -350,6 +400,7 @@ namespace Magick
   using MagickLib::AcquireCacheView;
   using MagickLib::AcquireImagePixels;
   using MagickLib::AcquireMemory;
+  using MagickLib::AdaptiveThresholdImage;
   using MagickLib::AddNoiseImage;
   using MagickLib::AffineMatrix;
   using MagickLib::AllocateImage;
@@ -394,6 +445,7 @@ namespace Magick
   using MagickLib::DestroyQuantizeInfo;
   using MagickLib::DispatchImage;
   using MagickLib::DisplayImages;
+  using MagickLib::DrawAffine;
   using MagickLib::DrawAllocateContext;
   using MagickLib::DrawAnnotation;
   using MagickLib::DrawArc;
@@ -443,8 +495,9 @@ namespace Magick
   using MagickLib::DrawPushPattern;
   using MagickLib::DrawRectangle;
   using MagickLib::DrawRender;
+  using MagickLib::DrawRotate;
   using MagickLib::DrawRoundRectangle;
-  using MagickLib::DrawSetAffine;
+  using MagickLib::DrawScale;
   using MagickLib::DrawSetClipPath;
   using MagickLib::DrawSetClipRule;
   using MagickLib::DrawSetClipUnits;
@@ -460,11 +513,6 @@ namespace Magick
   using MagickLib::DrawSetFontStyle;
   using MagickLib::DrawSetFontWeight;
   using MagickLib::DrawSetGravity;
-  using MagickLib::DrawSetRotate;
-  using MagickLib::DrawSetScale;
-  using MagickLib::DrawSetSkewX;
-  using MagickLib::DrawSetSkewY;
-//   using MagickLib::DrawSetStopColor;
   using MagickLib::DrawSetStrokeAntialias;
   using MagickLib::DrawSetStrokeColor;
   using MagickLib::DrawSetStrokeColorString;
@@ -478,10 +526,13 @@ namespace Magick
   using MagickLib::DrawSetStrokeWidth;
   using MagickLib::DrawSetTextAntialias;
   using MagickLib::DrawSetTextDecoration;
+  using MagickLib::DrawSetTextEncoding;
   using MagickLib::DrawSetTextUnderColor;
   using MagickLib::DrawSetTextUnderColorString;
-  using MagickLib::DrawSetTranslate;
   using MagickLib::DrawSetViewbox;
+  using MagickLib::DrawSkewX;
+  using MagickLib::DrawSkewY;
+  using MagickLib::DrawTranslate;
   using MagickLib::EdgeImage;
   using MagickLib::EmbossImage;
   using MagickLib::EnhanceImage;
@@ -501,6 +552,7 @@ namespace Magick
   using MagickLib::GaussianBlurImage;
   using MagickLib::GetCacheView;
   using MagickLib::GetCacheViewIndexes;
+  using MagickLib::GetColorTuple;
   using MagickLib::GetDrawInfo;
   using MagickLib::GetExceptionInfo;
   using MagickLib::GetGeometry;
@@ -510,6 +562,7 @@ namespace Magick
   using MagickLib::GetImageDepth;
   using MagickLib::GetImageInfo;
   using MagickLib::GetImagePixels;
+  using MagickLib::GetImageQuantizeError;
   using MagickLib::GetImageType;
   using MagickLib::GetIndexes;
   using MagickLib::GetMagickGeometry;
@@ -520,6 +573,7 @@ namespace Magick
   using MagickLib::GetPixels;
   using MagickLib::GetQuantizeInfo;
   using MagickLib::GetTypeMetrics;
+  using MagickLib::GlobExpression;
   using MagickLib::GreaterValue;
   using MagickLib::HSLTransform;
   using MagickLib::HeightValue;
@@ -530,11 +584,14 @@ namespace Magick
   using MagickLib::ImageRegistryType;
   using MagickLib::ImageToBlob;
   using MagickLib::ImplodeImage;
+  using MagickLib::IsEventLogging;
   using MagickLib::IsGeometry;
+  using MagickLib::IsImagesEqual;
   using MagickLib::IsSubimage;
   using MagickLib::LessValue;
   using MagickLib::LiberateMemory;
   using MagickLib::LocaleCompare;
+  using MagickLib::LogMagickEvent;
   using MagickLib::MagickInfo;
   using MagickLib::MagickToMime;
   using MagickLib::MagnifyImage;
@@ -559,9 +616,9 @@ namespace Magick
   using MagickLib::PingImage;
   using MagickLib::PointInfo;
   using MagickLib::PopImagePixels;
+  using MagickLib::ProfileImage;
   using MagickLib::ProfileInfo;
   using MagickLib::PushImagePixels;
-  using MagickLib::QuantizationError;
   using MagickLib::QuantizeImage;
   using MagickLib::QuantizeInfo;
   using MagickLib::QueryColorDatabase;
@@ -592,6 +649,7 @@ namespace Magick
   using MagickLib::SetImageOpacity;
   using MagickLib::SetImagePixels;
   using MagickLib::SetImageType;
+  using MagickLib::SetLogEventMask;
   using MagickLib::SetMagickInfo;
   using MagickLib::SetMagickRegistry;
   using MagickLib::ShadeImage;
