@@ -25,8 +25,8 @@ int main( int /*argc*/, char ** argv)
   int failures=0;
 
   string srcdir("");
-  if(getenv("srcdir") != 0)
-    srcdir = getenv("srcdir") + string("/");
+  if(getenv("SRCDIR") != 0)
+    srcdir = getenv("SRCDIR");
 
   try {
 
