@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Typedef declaractions.
@@ -253,7 +259,8 @@ static unsigned int DecodeImage(Image *image,const unsigned int compression,
         }
       }
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,&image->exception))
+        break;
   }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
@@ -392,9 +399,9 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   (void) memset(&avi_info,0,sizeof(AVIInfo));
   (void) memset(&bmp_info,0,sizeof(BMPInfo));
   colormap=(PixelPacket *) NULL;
@@ -417,12 +424,11 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     if ((LocaleCompare(id,"00db") == 0) || (LocaleCompare(id,"00dc") == 0))
       {
-  if (LocaleCompare(bmp_info.compression,"MJPG") == 0)
+        if (LocaleCompare(bmp_info.compression,"MJPG") == 0)
           {
             FormatString(message,"AVI compression %.1024s not yet supported",
               bmp_info.compression);
-            ThrowException(exception,CorruptImageError,message,
-              image->filename);
+            ThrowException(exception,CorruptImageError,message,image->filename);
             for ( ; chunk_size != 0; chunk_size--)
               (void) ReadBlobByte(image);
             continue;
@@ -437,8 +443,8 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image->x_resolution=bmp_info.x_pixels/100.0;
         image->y_resolution=bmp_info.y_pixels/100.0;
         if (!AllocateImageColormap(image,number_colors ? number_colors : 256))
-          ThrowReaderException(ResourceLimitError,
-            "Memory allocation failed",image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+            image);
         if (number_colors != 0)
           (void) memcpy(image->colormap,colormap,
             number_colors*sizeof(PixelPacket));
@@ -449,7 +455,7 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         pixels=(unsigned char *)
           AcquireMemory(Max(bytes_per_line,image->columns+1)*image->rows);
         if (pixels == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         if (LocaleCompare(id,"00db") == 0)
           (void) ReadBlob(image,bytes_per_line*image->rows,(char *) pixels);
@@ -458,7 +464,7 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             status=DecodeImage(image,1,pixels);
             if (status == False)
               ThrowReaderException(CorruptImageError,
-                "unable to runlength decode",image);
+                "UnableToRunlengthDecodeImage",image);
           }
         /*
           Convert BMP raster image to pixel packets.
@@ -501,7 +507,12 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+                  {
+                    status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                      exception);
+                    if (status == False)
+                      break;
+                  }
             }
             break;
           }
@@ -538,7 +549,12 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+                  {
+                    status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                      exception);
+                    if (status == False)
+                      break;
+                  }
             }
             break;
           }
@@ -567,7 +583,12 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+                  {
+                    status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                      exception);
+                    if (status == False)
+                      break;
+                  }
             }
             break;
           }
@@ -578,9 +599,8 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
             /*
               Convert PseudoColor scanline.
-            if (bmp_info.compression == 1)
             */
-              bytes_per_line=image->columns << 1;
+            bytes_per_line=image->columns << 1;
             image->storage_class=DirectClass;
             for (y=(long) image->rows-1; y >= 0; y--)
             {
@@ -592,16 +612,21 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 word=(*p++);
                 word|=(*p++ << 8);
-                q->red=Upscale(ScaleColor5to8((word >> 11) & 0x1f));
-                q->green=Upscale(ScaleColor6to8((word >> 5) & 0x3f));
-                q->blue=Upscale(ScaleColor5to8(word & 0x1f));
+                q->red=ScaleCharToQuantum(ScaleColor5to8((word >> 11) & 0x1f));
+                q->green=ScaleCharToQuantum(ScaleColor6to8((word >> 5) & 0x3f));
+                q->blue=ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
                 q++;
               }
               if (!SyncImagePixels(image))
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+                  {
+                    status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                      exception);
+                    if (status == False)
+                      break;
+                  }
             }
             break;
           }
@@ -620,24 +645,28 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                q->blue=Upscale(*p++);
-                q->green=Upscale(*p++);
-                q->red=Upscale(*p++);
+                q->blue=ScaleCharToQuantum(*p++);
+                q->green=ScaleCharToQuantum(*p++);
+                q->red=ScaleCharToQuantum(*p++);
                 if (image->matte)
-                  q->opacity=Upscale(*p++);
+                  q->opacity=ScaleCharToQuantum(*p++);
                 q++;
               }
               if (!SyncImagePixels(image))
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+                  {
+                    status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                      exception);
+                    if (status == False)
+                      break;
+                  }
             }
             break;
           }
           default:
-            ThrowReaderException(CorruptImageError,"Not a avi image file",
-              image)
+            ThrowReaderException(CorruptImageError,"NotAnAVIImageFile",image)
         }
         LiberateMemory((void **) &pixels);
         if ((unsigned long) image->scene < (avi_info.total_frames-1))
@@ -651,8 +680,11 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 DestroyImageList(image);
                 return((Image *) NULL);
               }
-            image=image->next;
-            MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+            image=SyncNextImageInList(image);
+            status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+              exception);
+            if (status == False)
+              break;
           }
         continue;
       }
@@ -742,12 +774,12 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   AcquireMemory(number_colors*sizeof(PixelPacket));
                 if (colormap == (PixelPacket *) NULL)
                   ThrowReaderException(ResourceLimitError,
-                    "Memory allocation failed",image);
+                    "MemoryAllocationFailed",image);
                 for (i=0; i < (long) number_colors; i++)
                 {
-                  colormap[i].blue=Upscale(ReadBlobByte(image));
-                  colormap[i].green=Upscale(ReadBlobByte(image));
-                  colormap[i].red=Upscale(ReadBlobByte(image));
+                  colormap[i].blue=ScaleCharToQuantum(ReadBlobByte(image));
+                  colormap[i].green=ScaleCharToQuantum(ReadBlobByte(image));
+                  colormap[i].red=ScaleCharToQuantum(ReadBlobByte(image));
                   (void) ReadBlobByte(image);
                   chunk_size-=4;
                 }
@@ -848,9 +880,9 @@ ModuleExport void RegisterAVIImage(void)
     *entry;
 
   entry=SetMagickInfo("AVI");
-  entry->decoder=ReadAVIImage;
-  entry->magick=IsAVI;
-  entry->description=AcquireString("Audio/Visual Interleaved");
+  entry->decoder=(DecoderHandler) ReadAVIImage;
+  entry->magick=(MagickHandler) IsAVI;
+  entry->description=AcquireString("Microsoft Audio/Visual Interleaved");
   entry->module=AcquireString("AVI");
   (void) RegisterMagickInfo(entry);
 }
