@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -129,16 +135,16 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
-    Read AVS image.
+    Read AVS X image.
   */
   width=ReadBlobMSBLong(image);
   height=ReadBlobMSBLong(image);
   if ((width == (unsigned long) ~0) || (height == (unsigned long) ~0))
-    ThrowReaderException(CorruptImageError,"Not a AVS image file",image);
+    ThrowReaderException(CorruptImageError,"NotAnAVSImageFile",image);
   do
   {
     /*
@@ -152,24 +158,22 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
     pixels=(unsigned char *) AcquireMemory(4*image->columns);
     if (pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"Unable to allocate memory",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     for (y=0; y < (long) image->rows; y++)
     {
       count=ReadBlob(image,4*image->columns,pixels);
       if (count == 0)
-        ThrowReaderException(CorruptImageError,"Unable to read image data",
-          image);
+        ThrowReaderException(CorruptImageError,"UnableToReadImageData",image);
       p=pixels;
       q=SetImagePixels(image,0,y,image->columns,1);
       if (q == (PixelPacket *) NULL)
         break;
       for (x=0; x < (long) image->columns; x++)
       {
-        q->opacity=(Quantum) (MaxRGB-Upscale(*p++));
-        q->red=Upscale(*p++);
-        q->green=Upscale(*p++);
-        q->blue=Upscale(*p++);
+        q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
+        q->red=ScaleCharToQuantum(*p++);
+        q->green=ScaleCharToQuantum(*p++);
+        q->blue=ScaleCharToQuantum(*p++);
         image->matte|=(q->opacity != OpaqueOpacity);
         q++;
       }
@@ -177,11 +181,16 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
     }
     LiberateMemory((void **) &pixels);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -201,8 +210,11 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while ((width != (unsigned long) ~0) && (height != (unsigned long) ~0));
   while (image->previous != (Image *) NULL)
@@ -222,12 +234,11 @@ static Image *ReadAVSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterAVSImage adds attributes for the AVS image format to
-%  the list of supported formats.  The attributes include the image format
-%  tag, a method to read and/or write the format, whether the format
-%  supports the saving of more than one frame to the same file or blob,
-%  whether the format supports native in-memory I/O, and a brief
-%  description of the format.
+%  RegisterAVSImage() adds attributes for the AVS X image format to the list
+%  of supported formats.  The attributes include the image format tag, a
+%  method to read and/or write the format, whether the format supports the
+%  saving of more than one frame to the same file or blob, whether the format
+%  supports native in-memory I/O, and a brief description of the format.
 %
 %  The format of the RegisterAVSImage method is:
 %
@@ -240,8 +251,8 @@ ModuleExport void RegisterAVSImage(void)
     *entry;
 
   entry=SetMagickInfo("AVS");
-  entry->decoder=ReadAVSImage;
-  entry->encoder=WriteAVSImage;
+  entry->decoder=(DecoderHandler) ReadAVSImage;
+  entry->encoder=(EncoderHandler) WriteAVSImage;
   entry->description=AcquireString("AVS X image");
   entry->module=AcquireString("AVS");
   (void) RegisterMagickInfo(entry);
@@ -316,9 +327,11 @@ static unsigned int WriteAVSImage(const ImageInfo *image_info,Image *image)
     *pixels;
 
   unsigned int
-    scene,
     status;
 
+  unsigned long
+    scene;
+
   /*
     Open output image file.
   */
@@ -326,16 +339,16 @@ static unsigned int WriteAVSImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   scene=0;
   do
   {
     /*
       Write AVS header.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     (void) WriteBlobMSBLong(image,image->columns);
     (void) WriteBlobMSBLong(image,image->rows);
     /*
@@ -343,8 +356,7 @@ static unsigned int WriteAVSImage(const ImageInfo *image_info,Image *image)
     */
     pixels=(unsigned char *) AcquireMemory(image->columns*sizeof(PixelPacket));
     if (pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     /*
       Convert MIFF to AVS raster pixels.
     */
@@ -356,22 +368,27 @@ static unsigned int WriteAVSImage(const ImageInfo *image_info,Image *image)
       q=pixels;
       for (x=0; x < (long) image->columns; x++)
       {
-        *q++=MaxRGB-Downscale((image->matte ? p->opacity : OpaqueOpacity));
-        *q++=Downscale(p->red);
-        *q++=Downscale(p->green);
-        *q++=Downscale(p->blue);
+        *q++=ScaleQuantumToChar(
+          MaxRGB-(image->matte ? p->opacity : OpaqueOpacity));
+        *q++=ScaleQuantumToChar(p->red);
+        *q++=ScaleQuantumToChar(p->green);
+        *q++=ScaleQuantumToChar(p->blue);
         p++;
       }
       (void) WriteBlob(image,q-pixels,(char *) pixels);
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(SaveImageText,y,image->rows);
+          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+            break;
     }
     LiberateMemory((void **) &pixels);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
