@@ -18,7 +18,7 @@
 %                               February 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "render.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -103,7 +109,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     *p,
     *q;
 
-  register int
+  register long
     i;
 
   TypeMetric
@@ -121,7 +127,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if (image->columns == 0)
-    ThrowReaderException(OptionError,"Must specify image size",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
   if (*image_info->filename != '@')
     caption=AllocateString(image_info->filename);
   else
@@ -133,9 +139,9 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
         Read caption.
       */
       (void) strncpy(image->filename,image_info->filename+1,MaxTextExtent-2);
-      status=OpenBlob(image_info,image,ReadBinaryType,exception);
+      status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == False)
-        ThrowReaderException(FileOpenError,"Unable to open file",image);
+        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
       length=MaxTextExtent;
       caption=(char *) AcquireMemory(length);
       p=caption;
@@ -152,14 +158,14 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
           p=caption+strlen(caption);
         }
       if (caption == (char *) NULL)
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       CloseBlob(image);
     }
   /*
     Format caption.
   */
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+  draw_info->fill=image_info->pen;
   draw_info->text=AllocateString(caption);
   p=caption;
   q=draw_info->text;
@@ -169,7 +175,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     *q='\0';
     status=GetTypeMetrics(image,draw_info,&metrics);
     if (status == False)
-      ThrowReaderException(DelegateError,"Unable to get type metrics",image);
+      ThrowReaderException(TypeError,"UnableToGetTypeMetrics",image);
     if ((metrics.width+metrics.max_advance/2) < image->columns)
       continue;
     for (p--; !isspace((int) *p) && (p > caption); p--);
@@ -221,7 +227,7 @@ ModuleExport void RegisterCAPTIONImage(void)
     *entry;
 
   entry=SetMagickInfo("CAPTION");
-  entry->decoder=ReadCAPTIONImage;
+  entry->decoder=(DecoderHandler) ReadCAPTIONImage;
   entry->adjoin=False;
   entry->description=AcquireString("Image caption");
   entry->module=AcquireString("CAPTION");
