@@ -9,11 +9,11 @@
 %                                                                             %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
-%                                 July 1992                                   %
+%                             Leonard Rosenthol                               %
+%                                 May 2002                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -44,15 +44,24 @@
 %
 */
 
-
-#if defined(WIN32)
-#include <windows.h>
-#endif
-
 /*
   Include declarations.
 */
+
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "static.h"
+#include "utility.h"
+#if defined(HasWINGDI32)
+#  if defined(__CYGWIN__)
+#    include <windows.h>
+#  else
+     // All MinGW needs ...
+#    include <wingdi.h>
+#  endif
+#endif
 
 
 /*
@@ -86,6 +95,7 @@
 %
 %
 */
+#if defined(HasWINGDI32)
 static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
@@ -95,127 +105,119 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,ExceptionInfo *exce
     y;
 
   register long
-    i,
     x;
 
   register PixelPacket
     *q;
 
-  size_t
-    count;
-
-  unsigned int
-    packet_size,
-    status;
-
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
 
-#if defined(WIN32)
-{
-	HBITMAP	bitmapH;
+  {
+    HBITMAP
+      bitmapH;
 
-	OpenClipboard( NULL );
-	bitmapH = GetClipboardData( CF_BITMAP );
-	CloseClipboard();
+    OpenClipboard( NULL );
+    bitmapH = GetClipboardData( CF_BITMAP );
+    CloseClipboard();
 
-	if ( bitmapH != NULL )
-	{
-		BITMAPINFO
-			DIBinfo;
+    if ( bitmapH != NULL )
+      {
+        BITMAPINFO
+          DIBinfo;
 
-		BITMAP
-			bitmap;
+        BITMAP
+          bitmap;
 
-		HBITMAP
-			hBitmap,
-			hOldBitmap;
+        HBITMAP
+          hBitmap,
+          hOldBitmap;
 
-		HDC
-			hDC,
-			hMemDC;
+        HDC
+          hDC,
+          hMemDC;
 
-		RGBQUAD
-			*pBits,
-			*ppBits;
+        RGBQUAD
+          *pBits,
+          *ppBits;
 
-		// create an offscreen DC for the source
-		hMemDC = CreateCompatibleDC( NULL );
-		hOldBitmap = SelectObject( hMemDC, bitmapH );
-		GetObject( bitmapH, sizeof( BITMAP ), (LPSTR) &bitmap );
-		if ((image->columns == 0) || (image->rows == 0))
-		{
-			image->rows=bitmap.bmHeight;
-			image->columns=bitmap.bmWidth;
-		}
+        // create an offscreen DC for the source
+        hMemDC = CreateCompatibleDC( NULL );
+        hOldBitmap = SelectObject( hMemDC, bitmapH );
+        GetObject( bitmapH, sizeof( BITMAP ), (LPSTR) &bitmap );
+        if ((image->columns == 0) || (image->rows == 0))
+          {
+            image->rows=bitmap.bmHeight;
+            image->columns=bitmap.bmWidth;
+          }
 
-		/*
-			Initialize the bitmap header info.
-			Create a temp DC for the DIB to go into
-		*/
-		(void) memset(&DIBinfo,0,sizeof(BITMAPINFO));
-		DIBinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
-		DIBinfo.bmiHeader.biWidth=image->columns;
-		DIBinfo.bmiHeader.biHeight=(-1)*image->rows;
-		DIBinfo.bmiHeader.biPlanes=1;
-		DIBinfo.bmiHeader.biBitCount=32;
-		DIBinfo.bmiHeader.biCompression=BI_RGB;
-		hDC=GetDC(NULL);
-		if (!hDC)
-			ThrowReaderException(FatalException,"failed to create a DC",image);
-		hBitmap= CreateDIBSection(hDC,&DIBinfo,DIB_RGB_COLORS,(void **) &ppBits,NULL,0);
-		ReleaseDC(NULL,hDC);
-		if (!hBitmap)
-			ThrowReaderException(FatalException,"failed to create bitmap",image);
+        /*
+          Initialize the bitmap header info.
+          Create a temp DC for the DIB to go into
+        */
+        (void) memset(&DIBinfo,0,sizeof(BITMAPINFO));
+        DIBinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+        DIBinfo.bmiHeader.biWidth=image->columns;
+        DIBinfo.bmiHeader.biHeight=(-1)*image->rows;
+        DIBinfo.bmiHeader.biPlanes=1;
+        DIBinfo.bmiHeader.biBitCount=32;
+        DIBinfo.bmiHeader.biCompression=BI_RGB;
+        hDC=GetDC(NULL);
+        if (!hDC)
+          ThrowReaderException(CoderError,"UnableToCreateADC",image);
+        hBitmap= CreateDIBSection(hDC,&DIBinfo,DIB_RGB_COLORS,(void **) &ppBits,NULL,0);
+        ReleaseDC(NULL,hDC);
+        if (!hBitmap)
+          ThrowReaderException(CoderError,"UnableToCreateBitmap",image);
 
-		// create an offscreen DC
-		hDC=CreateCompatibleDC(NULL);
-		if (!hDC)
-		{
-			DeleteObject(hBitmap);
-			ThrowReaderException(FatalException,"failed to create a memory DC",image);
-		}
-		hOldBitmap=(HBITMAP) SelectObject(hDC,hBitmap);
-		if (!hOldBitmap)
-		{
-			DeleteDC(hDC);
-			DeleteObject(hBitmap);
-			ThrowReaderException(FatalException,"failed to create bitmap",image);
-		}
+        // create an offscreen DC
+        hDC=CreateCompatibleDC(NULL);
+        if (!hDC)
+          {
+            DeleteObject(hBitmap);
+            ThrowReaderException(CoderError,"UnableToCreateADC",image);
+          }
+        hOldBitmap=(HBITMAP) SelectObject(hDC,hBitmap);
+        if (!hOldBitmap)
+          {
+            DeleteDC(hDC);
+            DeleteObject(hBitmap);
+            ThrowReaderException(CoderError,"UnableToCreateBitmap",image);
+          }
 
-		// bitblt from the memory to the DIB-based one
-		BitBlt( hDC, 0, 0, image->columns, image->rows, hMemDC, 0, 0, SRCCOPY );
+        // bitblt from the memory to the DIB-based one
+        BitBlt( hDC, 0, 0, image->columns, image->rows, hMemDC, 0, 0, SRCCOPY );
 
-		// finally copy the pixels!
-		pBits=ppBits;
-		for (y=0; y < (long) image->rows; y++)
-		{
-			q=SetImagePixels(image,0,y,image->columns,1);
-			if (q == (PixelPacket *) NULL)
-				break;
-			for (x=0; x < (long) image->columns; x++)
-			{
-				q->red=Upscale(pBits->rgbRed);
-				q->green=Upscale(pBits->rgbGreen);
-				q->blue=Upscale(pBits->rgbBlue);
-				q->opacity=OpaqueOpacity;
-				pBits++;
-				q++;
-			}
-			if (!SyncImagePixels(image))
-			break;
-		}
-	} else
-		ThrowReaderException(FatalException,"no bitmap on clipboard",image);
-}
-#endif
+        // finally copy the pixels!
+        pBits=ppBits;
+        for (y=0; y < (long) image->rows; y++)
+          {
+            q=SetImagePixels(image,0,y,image->columns,1);
+            if (q == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (long) image->columns; x++)
+              {
+                q->red    = ScaleCharToQuantum(pBits->rgbRed);
+                q->green  = ScaleCharToQuantum(pBits->rgbGreen);
+                q->blue    = ScaleCharToQuantum(pBits->rgbBlue);
+                q->opacity  = OpaqueOpacity;
+                pBits++;
+                q++;
+              }
+            if (!SyncImagePixels(image))
+              break;
+          }
+      } else
+        ThrowReaderException(CoderError,"NoBitmapOnClipboard",image);
+  }
 
   CloseBlob(image);
   return(image);
 }
+#endif /* HasWINGDI32 */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -246,22 +248,9 @@ static Image *ReadCLIPBOARDImage(const ImageInfo *image_info,ExceptionInfo *exce
 %
 %
 */
+#if defined(HasWINGDI32)
 static unsigned int WriteCLIPBOARDImage(const ImageInfo *image_info,Image *image)
 {
-  int
-    y;
-
-  register const PixelPacket
-    *p;
-
-  unsigned char
-    *pixels;
-
-  unsigned int
-    packet_size,
-    scene,
-    status;
-
   /*
     Allocate memory for pixels.
   */
@@ -270,67 +259,24 @@ static unsigned int WriteCLIPBOARDImage(const ImageInfo *image_info,Image *image
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
 
-#if defined(WIN32)
   {
-    unsigned long 
-		nPixels,
-		nPixelCount;
-	long
-		memSize;
-	PixelPacket *pPixels;
-    BITMAP  bitmap;
-	HBITMAP	bitmapH;
-	HANDLE  theBitsH;
+    HBITMAP
+      bitmapH;
 
-	OpenClipboard( NULL );
+    OpenClipboard( NULL );
     EmptyClipboard();
 
-    nPixels = image->columns * image->rows;
+  bitmapH = ImageToHBITMAP(image);
+  SetClipboardData(CF_BITMAP, bitmapH);
 
-    bitmap.bmType = 0;
-    bitmap.bmWidth = image->columns;
-    bitmap.bmHeight = image->rows;
-    bitmap.bmWidthBytes = bitmap.bmWidth * 4;
-    bitmap.bmPlanes = 1;
-    bitmap.bmBitsPixel = 32;
-    bitmap.bmBits = NULL;
+  CloseClipboard();
 
-    memSize = nPixels * bitmap.bmBitsPixel;
-    theBitsH = (HANDLE) GlobalAlloc (GMEM_MOVEABLE | GMEM_DDESHARE, memSize);
-    if (theBitsH == NULL)
-	 return( False );	/* DoDisplayError( "OnEditCopy", GetLastError() ); */
-    else {
-		RGBQUAD * theBits = (RGBQUAD *) GlobalLock((HGLOBAL) theBitsH);
-		RGBQUAD *pDestPixel = theBits;
-		if ( bitmap.bmBits == NULL )
-			bitmap.bmBits = theBits;
-
-		pPixels = AcquireImagePixels(image,0,0,image->columns,image->rows,&image->exception);
-		for( nPixelCount = nPixels; nPixelCount ; nPixelCount-- )
-		{
-			pDestPixel->rgbRed	    = Downscale(pPixels->red);
-			pDestPixel->rgbGreen    = Downscale(pPixels->green);
-			pDestPixel->rgbBlue	    = Downscale(pPixels->blue);
-			pDestPixel->rgbReserved = 0;
-			++pDestPixel;
-			++pPixels;
-		}
-
-		bitmap.bmBits = theBits;
-		bitmapH = CreateBitmapIndirect( &bitmap );
-
-		GlobalUnlock((HGLOBAL) theBitsH);
-
-		SetClipboardData(CF_BITMAP, bitmapH);
-		CloseClipboard();
-    }
-
-	return(True);
+    return(True);
   }
-#endif
 
   return(False);
 }
+#endif /* HasWINGDI32 */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -357,16 +303,18 @@ static unsigned int WriteCLIPBOARDImage(const ImageInfo *image_info,Image *image
 */
 ModuleExport void RegisterCLIPBOARDImage(void)
 {
+#if defined(HasWINGDI32)
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("CLIPBOARD");
-  entry->decoder=ReadCLIPBOARDImage;
-  entry->encoder=WriteCLIPBOARDImage;
+  entry->decoder=(DecoderHandler) ReadCLIPBOARDImage;
+  entry->encoder=(EncoderHandler) WriteCLIPBOARDImage;
   entry->adjoin = False;
   entry->description=AcquireString("the system clipboard");
   entry->module=AcquireString("CLIPBOARD");
   (void) RegisterMagickInfo(entry);
+#endif
 }
 
 /*
@@ -390,6 +338,8 @@ ModuleExport void RegisterCLIPBOARDImage(void)
 */
 ModuleExport void UnregisterCLIPBOARDImage(void)
 {
+#if defined(HasWINGDI32)
   (void) UnregisterMagickInfo("CLIPBOARD");
+#endif
 }
 
