@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,7 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
-
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
+
 /*
   Forward declarations.
 */
@@ -115,24 +121,26 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
     *scanline;
 
   unsigned int
-    packet_size,
     status;
 
+  unsigned long
+    packet_size;
+
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"Must specify image size",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open image file.
       */
-      status=OpenBlob(image_info,image,ReadBinaryType,exception);
+      status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == False)
-        ThrowReaderException(FileOpenError,"Unable to open file",image);
+        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
       for (i=0; i < image->offset; i++)
         (void) ReadBlobByte(image);
     }
@@ -148,7 +156,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   scanline=(unsigned char *)
     AcquireMemory(packet_size*image->tile_info.width);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->subrange != 0)
     while (image->scene < image_info->subimage)
     {
@@ -194,7 +202,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (i=0; i < (long) count; i++)
@@ -233,7 +242,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (i=0; i < (long) count; i++)
@@ -252,9 +262,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         if (image_info->interlace == PartitionInterlace)
           {
             AppendImageFormat("C",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType,exception);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenError,"Unable to open file",image);
+              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
           }
         packet_size=image->depth > 8 ? 2 : 1;
         for (y=0; y < image->tile_info.y; y++)
@@ -273,7 +283,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                break;
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -283,9 +294,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             CloseBlob(image);
             AppendImageFormat("M",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType,exception);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenError,"Unable to open file",image);
+              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < image->tile_info.y; y++)
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
@@ -300,7 +311,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                break;
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -310,9 +322,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             CloseBlob(image);
             AppendImageFormat("Y",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType,exception);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenError,"Unable to open file",image);
+              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < image->tile_info.y; y++)
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
@@ -327,7 +339,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                break;
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -337,9 +350,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             CloseBlob(image);
             AppendImageFormat("K",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType,exception);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenError,"Unable to open file",image);
+              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < image->tile_info.y; y++)
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
@@ -354,7 +367,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                break;
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -369,10 +383,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               {
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
-                status=OpenBlob(image_info,image,ReadBinaryType,exception);
+                status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
                 if (status == False)
-                  ThrowReaderException(FileOpenError,"Unable to open file",
-                    image);
+                  ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
               }
             for (y=0; y < image->tile_info.y; y++)
               (void) ReadBlob(image,packet_size*image->tile_info.width,
@@ -389,7 +402,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(i,span))
-                  MagickMonitor(LoadImageText,i,span);
+                  if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                    break;
               i++;
             }
             count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -403,7 +417,11 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
       }
     }
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -424,8 +442,11 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (count != 0);
   LiberateMemory((void **) &scanline);
@@ -464,16 +485,16 @@ ModuleExport void RegisterCMYKImage(void)
     *entry;
 
   entry=SetMagickInfo("CMYK");
-  entry->decoder=ReadCMYKImage;
-  entry->encoder=WriteCMYKImage;
+  entry->decoder=(DecoderHandler) ReadCMYKImage;
+  entry->encoder=(EncoderHandler) WriteCMYKImage;
   entry->raw=True;
   entry->description=
     AcquireString("Raw cyan, magenta, yellow, and black samples");
   entry->module=AcquireString("CMYK");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("CMYKA");
-  entry->decoder=ReadCMYKImage;
-  entry->encoder=WriteCMYKImage;
+  entry->decoder=(DecoderHandler) ReadCMYKImage;
+  entry->encoder=(EncoderHandler) WriteCMYKImage;
   entry->raw=True;
   entry->description=
     AcquireString("Raw cyan, magenta, yellow, black, and opacity samples");
@@ -564,15 +585,15 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
     packet_size=image->depth > 8 ? 10 : 8;
   pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == False)
-        ThrowWriterException(FileOpenError,"Unable to open file",image);
+        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
     }
   scene=0;
   do
@@ -609,7 +630,8 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
             }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         break;
       }
@@ -637,7 +659,8 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlob(image,image->columns,pixels);
             }
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+              break;
         }
         break;
       }
@@ -650,9 +673,10 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
         if (image_info->interlace == PartitionInterlace)
           {
             AppendImageFormat("C",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=
+              OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -666,11 +690,13 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           {
             CloseBlob(image);
             AppendImageFormat("M",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
-        MagickMonitor(SaveImageText,100,400);
+        if (!MagickMonitor(SaveImageTag,100,400,&image->exception))
+          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -683,11 +709,13 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           {
             CloseBlob(image);
             AppendImageFormat("Y",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
-        MagickMonitor(SaveImageText,200,400);
+        if (!MagickMonitor(SaveImageTag,200,400,&image->exception))
+          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -700,11 +728,13 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           {
             CloseBlob(image);
             AppendImageFormat("K",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
-        MagickMonitor(SaveImageText,200,400);
+        if (!MagickMonitor(SaveImageTag,200,400,&image->exception))
+          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -715,16 +745,16 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
         }
         if (LocaleCompare(image_info->magick,"CMYKA") == 0)
           {
-            MagickMonitor(SaveImageText,300,400);
+            if (!MagickMonitor(SaveImageTag,300,400,&image->exception))
+              break;
             if (image_info->interlace == PartitionInterlace)
               {
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
-                status=OpenBlob(image_info,image,WriteBinaryType,
+                status=OpenBlob(image_info,image,WriteBinaryBlobMode,
                   &image->exception);
                 if (status == False)
-                  ThrowWriterException(FileOpenError,"Unable to open file",
-                    image);
+                  ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
               }
             for (y=0; y < (long) image->rows; y++)
             {
@@ -738,14 +768,18 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           }
         if (image_info->interlace == PartitionInterlace)
           (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-        MagickMonitor(SaveImageText,400,400);
+        if (!MagickMonitor(SaveImageTag,400,400,&image->exception))
+          break;
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   LiberateMemory((void **) &pixels);
   if (image_info->adjoin)
