@@ -49,13 +49,18 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "static.h"
+#include "utility.h"
 
 typedef struct
   {
   unsigned Width;
   unsigned Height;
   unsigned Reserved;
-  }CUTHeader;      /*Dr Hallo*/
+  }CUTHeader;      /*Dr Halo*/
 typedef struct
   {
   char FileId[2];
@@ -113,7 +118,7 @@ register IndexPacket *indexes;
                  break;
 /*            if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   ProgressMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
             break;
             }
       case 2:  /* Convert PseudoColor scanline. */
@@ -163,7 +168,7 @@ register IndexPacket *indexes;
                  break;
 /*         if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   ProgressMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
            break;
            }
       
@@ -194,7 +199,7 @@ register IndexPacket *indexes;
                  break;
 /*         if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   ProgressMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
            break;
      }
       case 8: /* Convert PseudoColor scanline. */
@@ -214,7 +219,7 @@ register IndexPacket *indexes;
                  break;
 /*           if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   ProgressMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
            }
            break;
 
@@ -225,10 +230,10 @@ static int GetCutColors(Image *image)
 {
 int MaxColor,x,y;
 PixelPacket *q;
-int Upscale16;
+int ScaleCharToQuantum16;
 
 /*This procedure computes number of colors in Grayed R[i]=G[i]=B[i] image*/
-Upscale16=Upscale(16);
+ScaleCharToQuantum16=ScaleCharToQuantum(16);
 MaxColor=0;
  for (y=0; y < (long)image->rows; y++)  
   {
@@ -236,12 +241,12 @@ MaxColor=0;
   for (x=0; x < (long)image->columns; x++)  
              {       
        if(MaxColor<q->red) MaxColor=q->red;
-       if(MaxColor>=Upscale16) return(255);  
+       if(MaxColor>=ScaleCharToQuantum16) return(255);  
        q++;  
        }  
   }
-if(MaxColor<(int) Upscale(2)) MaxColor=2;    
-else if(MaxColor<(int) Upscale(16)) MaxColor=16;    
+if(MaxColor<(int) ScaleCharToQuantum(2)) MaxColor=2;    
+else if(MaxColor<(int) ScaleCharToQuantum(16)) MaxColor=16;    
 return(MaxColor);
 }
 
@@ -298,9 +303,9 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read CUT image.
   */
@@ -311,7 +316,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
    Header.Reserved=ReadBlobLSBShort(image);
 
    if (Header.Width==0 || Header.Height==0 || Header.Reserved!=0)
-CUT_KO:  ThrowReaderException(CorruptImageError,"Not a CUT image file",image);
+CUT_KO:  ThrowReaderException(CorruptImageError,"NotACUTImageFile",image);
 
 /*---This code checks first line of image---*/
   EncodedByte=ReadBlobLSBShort(image);
@@ -381,7 +386,7 @@ CUT_KO:  ThrowReaderException(CorruptImageError,"Not a CUT image file",image);
      }
 
  if( (palette=AllocateImage(clone_info))==NULL ) goto NoPalette;
- status=OpenBlob(clone_info,palette,ReadBinaryType,exception);
+ status=OpenBlob(clone_info,palette,ReadBinaryBlobMode,exception);
  if (status == False)
      {
 ErasePalette:     
@@ -427,19 +432,19 @@ ErasePalette:
      if(MaxRGB!=PalHeader.MaxRed) 
          {
          image->colormap[i].red=(Quantum)
-           (((unsigned long)image->colormap[i].red*MaxRGB+(PalHeader.MaxRed>>1))/PalHeader.MaxRed);
+           (((double)image->colormap[i].red*MaxRGB+(PalHeader.MaxRed>>1))/PalHeader.MaxRed+0.5);
          }
      image->colormap[i].green=ReadBlobLSBShort(palette);
      if(MaxRGB!=PalHeader.MaxGreen) 
          {
          image->colormap[i].green=(Quantum)
-           (((unsigned long)image->colormap[i].green*MaxRGB+(PalHeader.MaxGreen>>1))/PalHeader.MaxGreen);
+           (((double)image->colormap[i].green*MaxRGB+(PalHeader.MaxGreen>>1))/PalHeader.MaxGreen+0.5);
          }
      image->colormap[i].blue=ReadBlobLSBShort(palette);       
      if(MaxRGB!=PalHeader.MaxBlue)  
          {
          image->colormap[i].blue=(Quantum)
-           (((unsigned long)image->colormap[i].blue*MaxRGB+(PalHeader.MaxBlue>>1))/PalHeader.MaxBlue);
+           (((double)image->colormap[i].blue*MaxRGB+(PalHeader.MaxBlue>>1))/PalHeader.MaxBlue+0.5);
          }
          
      }
@@ -454,15 +459,15 @@ NoPalette:
    image->colors=256;
    if (!AllocateImageColormap(image,image->colors))
      {
-NoMemory:  ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image)
+NoMemory:
+       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
            }     
    
    for (i=0; i < (long)image->colors; i++)
      {
-     image->colormap[i].red=Upscale(i);
-     image->colormap[i].green=Upscale(i);
-     image->colormap[i].blue=Upscale(i);
+     image->colormap[i].red=ScaleCharToQuantum(i);
+     image->colormap[i].green=ScaleCharToQuantum(i);
+     image->colormap[i].blue=ScaleCharToQuantum(i);
      }
    }
 
@@ -524,10 +529,12 @@ if(palette==NULL)
          {
    for (i=0; i < (long)image->colors; i++)
      {
-     j=Upscale(i);
-     if(image->colormap[i].red!=(Quantum) j) goto Finish;
-     if(image->colormap[i].green!=(Quantum) j) goto Finish;
-     if(image->colormap[i].blue!=(Quantum) j) goto Finish;
+     register Quantum
+       sample;
+     sample=ScaleCharToQuantum(i);
+     if(image->colormap[i].red!=sample) goto Finish;
+     if(image->colormap[i].green!=sample) goto Finish;
+     if(image->colormap[i].blue!=sample) goto Finish;
      }
      
    image->colormap[1].red=image->colormap[1].green=image->colormap[1].blue=MaxRGB;
@@ -536,7 +543,7 @@ if(palette==NULL)
      q=SetImagePixels(image,0,i,image->columns,1);  
      for (j=0; j < (long)image->columns; j++)  
              {       
-       if(q->red==Upscale(1))
+       if(q->red==ScaleCharToQuantum(1))
           {
           q->red=q->green=q->blue=MaxRGB;
           }
@@ -553,7 +560,7 @@ Finish:
   if(palette!=NULL) DestroyImage(palette);
   if(clone_info!=NULL) DestroyImageInfo(clone_info);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile",image);
   CloseBlob(image);
   return(image);       
 }
@@ -587,8 +594,9 @@ ModuleExport void RegisterCUTImage(void)
     *entry;
 
   entry=SetMagickInfo("CUT");
-  entry->decoder=ReadCUTImage;
-  entry->description=AcquireString("DR Hallo");
+  entry->decoder=(DecoderHandler) ReadCUTImage;
+  entry->seekable_stream=True;
+  entry->description=AcquireString("DR Halo");
   entry->module=AcquireString("CUT");
   (void) RegisterMagickInfo(entry);
 }
