@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "render.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Typedef declarations.
@@ -240,7 +247,8 @@ static unsigned int DecodeImage(Image *image,const unsigned long compression,
         }
       }
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,&image->exception))
+        break;
   }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
@@ -328,7 +336,8 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     *q++=0x00;
     *q++=0x00;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+        break;
   }
   /*
     End of bitmap.
@@ -371,7 +380,7 @@ static unsigned int IsDIB(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
     return(False);
-  if( (*magick==40) && (*(magick+1)==0))
+  if (memcmp(magick,"\050\000",2) == 0)
     return(True);
   return(False);
 }
@@ -457,17 +466,16 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Determine if this is a DIB file.
   */
   (void) memset(&dib_info,0,sizeof(DIBInfo));
   dib_info.size=ReadBlobLSBLong(image);
   if (dib_info.size!=40)
-    ThrowReaderException(CorruptImageError,"Not a Windows 3.X DIB image",
-      image);
+    ThrowReaderException(CorruptImageError,"NotADIBImageFile",image);
   /*
     Microsoft Windows 3.X DIB image file.
   */
@@ -501,20 +509,19 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   if(image_info->size)
     {
-      int
-        flags;
-
       RectangleInfo
         geometry;
 
-      flags=GetGeometry(image_info->size,&geometry.x,&geometry.y,
-        &geometry.width,&geometry.height);
-      if ((flags & WidthValue) && (geometry.width != 0)
-          && (geometry.width < image->columns))
-        image->columns=geometry.width;
-      if ((flags & HeightValue) && (geometry.height != 0)
-          && (geometry.height < image->rows))
-        image->rows=geometry.height;
+      unsigned int
+        flags;
+
+      flags=ParseAbsoluteGeometry(image_info->size,&geometry);
+      if (flags & WidthValue)
+        if ((geometry.width != 0) && (geometry.width < image->columns))
+          image->columns=geometry.width;
+      if (flags & HeightValue)
+        if ((geometry.height != 0) && (geometry.height < image->rows))
+          image->rows=geometry.height;
     }
   if (image->storage_class == PseudoClass)
     {
@@ -528,20 +535,18 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         Read DIB raster colormap.
       */
       if (!AllocateImageColormap(image,image->colors))
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       dib_colormap=(unsigned char *) AcquireMemory(4*image->colors);
       if (dib_colormap == (unsigned char *) NULL)
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       packet_size=4;
       (void) ReadBlob(image,packet_size*image->colors,(char *) dib_colormap);
       p=dib_colormap;
       for (i=0; i < (long) image->colors; i++)
       {
-        image->colormap[i].blue=Upscale(*p++);
-        image->colormap[i].green=Upscale(*p++);
-        image->colormap[i].red=Upscale(*p++);
+        image->colormap[i].blue=ScaleCharToQuantum(*p++);
+        image->colormap[i].green=ScaleCharToQuantum(*p++);
+        image->colormap[i].red=ScaleCharToQuantum(*p++);
         if (packet_size == 4)
           p++;
       }
@@ -557,8 +562,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   pixels=(unsigned char *)
     AcquireMemory(Max(bytes_per_line,image->columns+1)*image->rows);
   if (pixels == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-      image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if ((dib_info.compression == 0) || (dib_info.compression == 3))
     (void) ReadBlob(image,length,(char *) pixels);
   else
@@ -568,7 +572,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       status=DecodeImage(image,dib_info.compression,pixels);
       if (status == False)
-        ThrowReaderException(CorruptImageError,"runlength decoding failed",
+        ThrowReaderException(CorruptImageError,"UnableToRunlengthDecodeImage",
           image);
     }
   /*
@@ -618,7 +622,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+            {
+              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                exception);
+              if (status == False)
+                break;
+            }
       }
       break;
     }
@@ -655,7 +664,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+            {
+              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                exception);
+              if (status == False)
+                break;
+            }
       }
       break;
     }
@@ -685,7 +699,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+            {
+              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                exception);
+              if (status == False)
+                break;
+            }
       }
       break;
     }
@@ -712,15 +731,15 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           word|=(*p++ << 8);
           if (dib_info.red_mask == 0)
             {
-              q->red=Upscale(ScaleColor5to8((word >> 10) & 0x1f));
-              q->green=Upscale(ScaleColor5to8((word >> 5) & 0x1f));
-              q->blue=Upscale(ScaleColor5to8(word & 0x1f));
+              q->red=ScaleCharToQuantum(ScaleColor5to8((word >> 10) & 0x1f));
+              q->green=ScaleCharToQuantum(ScaleColor5to8((word >> 5) & 0x1f));
+              q->blue=ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
             }
           else
             {
-              q->red=Upscale(ScaleColor5to8((word >> 11) & 0x1f));
-              q->green=Upscale(ScaleColor6to8((word >> 5) & 0x3f));
-              q->blue=Upscale(ScaleColor5to8(word & 0x1f));
+              q->red=ScaleCharToQuantum(ScaleColor5to8((word >> 11) & 0x1f));
+              q->green=ScaleCharToQuantum(ScaleColor6to8((word >> 5) & 0x3f));
+              q->blue=ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
             }
           q++;
         }
@@ -728,7 +747,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+            {
+              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                exception);
+              if (status == False)
+                break;
+            }
       }
       break;
     }
@@ -746,27 +770,33 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         for (x=0; x < (long) image->columns; x++)
         {
-          q->blue=Upscale(*p++);
-          q->green=Upscale(*p++);
-          q->red=Upscale(*p++);
+          q->blue=ScaleCharToQuantum(*p++);
+          q->green=ScaleCharToQuantum(*p++);
+          q->red=ScaleCharToQuantum(*p++);
           if (image->matte)
-            q->opacity=Upscale(*p++);
+            q->opacity=ScaleCharToQuantum(*p++);
           q++;
         }
         if (!SyncImagePixels(image))
           break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+            {
+              status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                exception);
+              if (status == False)
+                break;
+            }
       }
       break;
     }
     default:
-      ThrowReaderException(CorruptImageError,"Not a DIB image file",image)
+      ThrowReaderException(CorruptImageError,"NotADIBImageFile",image)
   }
   LiberateMemory((void **) &pixels);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   if (dib_info.height < 0)
     {
       Image
@@ -817,9 +847,9 @@ ModuleExport void RegisterDIBImage(void)
     *entry;
 
   entry=SetMagickInfo("DIB");
-  entry->decoder=ReadDIBImage;
-  entry->encoder=WriteDIBImage;
-  entry->magick=IsDIB;
+  entry->decoder=(DecoderHandler) ReadDIBImage;
+  entry->encoder=(EncoderHandler) WriteDIBImage;
+  entry->magick=(MagickHandler) IsDIB;
   entry->adjoin=False;
   entry->stealth=True;
   entry->description=
@@ -920,13 +950,13 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   /*
     Initialize DIB raster file header.
   */
-  (void) TransformRGBImage(image,RGBColorspace);
+  (void) TransformRGBImage(image,image->colorspace);
   if (image->storage_class == DirectClass)
     {
       /*
@@ -970,8 +1000,7 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
   */
   pixels=(unsigned char *) AcquireMemory(dib_info.image_size);
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-      image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   switch (dib_info.bits_per_pixel)
   {
     case 1:
@@ -1009,7 +1038,8 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
          *q++=byte << (8-bit);
        if (image->previous == (Image *) NULL)
          if (QuantumTick(y,image->rows))
-           MagickMonitor(SaveImageText,y,image->rows);
+           if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+             break;
       }
       break;
     }
@@ -1032,7 +1062,8 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
         }
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+              break;
       }
       break;
     }
@@ -1050,16 +1081,17 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
         q=pixels+(image->rows-y-1)*bytes_per_line;
         for (x=0; x < (long) image->columns; x++)
         {
-          *q++=Downscale(p->blue);
-          *q++=Downscale(p->green);
-          *q++=Downscale(p->red);
+          *q++=ScaleQuantumToChar(p->blue);
+          *q++=ScaleQuantumToChar(p->green);
+          *q++=ScaleQuantumToChar(p->red);
           if (image->matte)
-            *q++=Downscale(p->opacity);
+            *q++=ScaleQuantumToChar(p->opacity);
           p++;
         }
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+              break;
       }
       break;
     }
@@ -1078,8 +1110,8 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
         if (pixels == (unsigned char *) NULL)
           {
             LiberateMemory((void **) &pixels);
-            ThrowWriterException(ResourceLimitError,
-              "Memory allocation failed",image)
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+              image)
           }
         dib_info.image_size=EncodeImage(image,bytes_per_line,pixels,dib_data);
         LiberateMemory((void **) &pixels);
@@ -1111,14 +1143,13 @@ static unsigned int WriteDIBImage(const ImageInfo *image_info,Image *image)
       dib_colormap=(unsigned char *)
         AcquireMemory(4*(1 << dib_info.bits_per_pixel));
       if (dib_colormap == (unsigned char *) NULL)
-        ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
       q=dib_colormap;
       for (i=0; i < (long) image->colors; i++)
       {
-        *q++=Downscale(image->colormap[i].blue);
-        *q++=Downscale(image->colormap[i].green);
-        *q++=Downscale(image->colormap[i].red);
+        *q++=ScaleQuantumToChar(image->colormap[i].blue);
+        *q++=ScaleQuantumToChar(image->colormap[i].green);
+        *q++=ScaleQuantumToChar(image->colormap[i].red);
         *q++=(Quantum) 0x0;
       }
       for ( ; i < (1L << dib_info.bits_per_pixel); i++)
