@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,8 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
-#if defined(HasDPS)
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 #include "xwindow.h"
+#if defined(HasDPS)
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -115,7 +121,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register int
+  register long
     i;
 
   register PixelPacket
@@ -167,7 +173,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
     return((Image *) NULL);
   /*
@@ -182,8 +188,8 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   map_info=XAllocStandardColormap();
   visual_info=(XVisualInfo *) NULL;
   if (map_info == (XStandardColormap *) NULL)
-    ThrowReaderException(ResourceLimitError,
-      "Unable to create standard colormap",image)
+    ThrowReaderException(ResourceLimitError,"UnableToCreateStandardColormap",
+      image)
   else
     {
       /*
@@ -366,17 +372,18 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             pixel=XGetPixel(dps_image,x,y);
             index=(pixel >> red_shift) & red_mask;
-            q->red=XDownscale(colors[index].red);
+            q->red=ScaleShortToQuantum(colors[index].red);
             index=(pixel >> green_shift) & green_mask;
-            q->green=XDownscale(colors[index].green);
+            q->green=ScaleShortToQuantum(colors[index].green);
             index=(pixel >> blue_shift) & blue_mask;
-            q->blue=XDownscale(colors[index].blue);
+            q->blue=ScaleShortToQuantum(colors[index].blue);
             q++;
           }
           if (!SyncImagePixels(image))
             break;
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,y,image->rows);
+            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+              break;
         }
       else
         for (y=0; y < (long) image->rows; y++)
@@ -388,17 +395,18 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             pixel=XGetPixel(dps_image,x,y);
             color=(pixel >> red_shift) & red_mask;
-            q->red=XDownscale((color*65535L)/red_mask);
+            q->red=ScaleShortToQuantum((color*65535L)/red_mask);
             color=(pixel >> green_shift) & green_mask;
-            q->green=XDownscale((color*65535L)/green_mask);
+            q->green=ScaleShortToQuantum((color*65535L)/green_mask);
             color=(pixel >> blue_shift) & blue_mask;
-            q->blue=XDownscale((color*65535L)/blue_mask);
+            q->blue=ScaleShortToQuantum((color*65535L)/blue_mask);
             q++;
           }
           if (!SyncImagePixels(image))
             break;
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,y,image->rows);
+            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+              break;
         }
       break;
     }
@@ -418,9 +426,9 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
       for (i=0; i < (long) image->colors; i++)
       {
-        image->colormap[colors[i].pixel].red=XDownscale(colors[i].red);
-        image->colormap[colors[i].pixel].green=XDownscale(colors[i].green);
-        image->colormap[colors[i].pixel].blue=XDownscale(colors[i].blue);
+        image->colormap[colors[i].pixel].red=ScaleShortToQuantum(colors[i].red);
+        image->colormap[colors[i].pixel].green=ScaleShortToQuantum(colors[i].green);
+        image->colormap[colors[i].pixel].blue=ScaleShortToQuantum(colors[i].blue);
       }
       /*
         Convert X image to PseudoClass packets.
@@ -436,7 +444,8 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (!SyncImagePixels(image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
       }
       break;
     }
@@ -500,7 +509,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #else
 static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  ThrowException(exception,MissingDelegateError,"Cannot read DPS images",
+  ThrowException(exception,CoderError,"DPSLibraryIsNotAvailable",
     image_info->filename);
   return((Image *) NULL);
 }
@@ -535,9 +544,9 @@ ModuleExport void RegisterDPSImage(void)
     *entry;
 
   entry=SetMagickInfo("DPS");
-  entry->decoder=ReadDPSImage;
+  entry->decoder=(DecoderHandler) ReadDPSImage;
   entry->blob_support=False;
-  entry->description=AcquireString("Display Postscript");
+  entry->description=AcquireString("Display Postscript Interpreter");
   entry->module=AcquireString("DPS");
   (void) RegisterMagickInfo(entry);
 }
