@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "delegate.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "render.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -60,84 +68,6 @@
 static unsigned int
   WriteEPTImage(const ImageInfo *,Image *);
 
-#if defined(HasGS)
-#include "ps/iapi.h"
-#include "ps/errors.h"
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   E x e c u t e P o s t I n t e r p r e t e r                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Method ExecutePostscriptInterpreter executes the postscript interpreter
-%  with the specified command.
-%
-%  The format of the ExecutePostscriptInterpreter method is:
-%
-%      unsigned int ExecutePostscriptInterpreter(const unsigned int verbose,
-%        const char *command)
-%
-%  A description of each parameter follows:
-%
-%    o status:  Method ExecutePostscriptInterpreter returns True is the command
-%      is successfully executed, otherwise False.
-%
-%    o verbose: A value other than zero displays the command prior to
-%      executing it.
-%
-%    o command: The address of a character string containing the command to
-%      execute.
-%
-%
-*/
-static unsigned int ExecutePostscriptInterpreter(const unsigned int verbose,
-  const char *command)
-{
-  char
-    **argv;
-
-  gs_main_instance
-    *interpreter;
-
-  int
-    argc,
-    code,
-    status;
-
-  register int
-    i;
-
-  if (verbose)
-    (void) fputs(command,stdout);
-  status=gsapi_new_instance(&interpreter,(void *) NULL);
-  if (status < 0)
-    return(False);
-  argv=StringToArgv(command,&argc);
-  status=gsapi_init_with_args(interpreter,argc-1,argv+1);
-  if (status == 0)
-    status=gsapi_run_string(interpreter,"systemdict /start get exec\n",0,&code);
-  gsapi_exit(interpreter);
-  gsapi_delete_instance(interpreter);
-  for (i=0; i < argc; i++)
-    LiberateMemory((void **) &argv[i]);
-  LiberateMemory((void **) &argv);
-  if ((status == 0) || (status == e_Quit))
-    return(False);
-  return(True);
-}
-#else
-static unsigned int ExecutePostscriptInterpreter(const unsigned int verbose,
-  const char *command)
-{
-  return(SystemCommand(verbose,command));
-}
-#endif
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -194,7 +124,7 @@ static unsigned int IsEPT(const unsigned char *magick,const size_t length)
 %  The format of the ReadEPTImage method is:
 %
 %      Image *ReadEPTImage(const ImageInfo *image_info,
-         ExceptionInfo *exception)
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -208,11 +138,12 @@ static unsigned int IsEPT(const unsigned char *magick,const size_t length)
 %
 %
 */
-static Image *ReadEPTImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
+static Image *ReadEPTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define BoundingBox  "%%BoundingBox:"
+#define BeginDocument  "%%BeginDocument:"
 #define DocumentMedia  "%%DocumentMedia:"
+#define EndDocument  "%%EndDocument:"
 #define PageBoundingBox  "%%PageBoundingBox:"
 #define PostscriptLevel  "%!PS-"
 #define RenderPostscriptText  "  Rendering postscript...  "
@@ -236,6 +167,9 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   FILE
     *file;
 
+  GeometryInfo
+    geometry_info;
+
   Image
     *image,
     *next_image;
@@ -244,11 +178,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
     *clone_info;
 
   int
-    c,
-    status;
-
-  off_t
-    filesize;
+    c;
 
   RectangleInfo
     box,
@@ -266,6 +196,11 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   size_t
     count;
 
+  unsigned int
+    flags,
+    skip,
+    status;
+
   unsigned long
     height,
     width;
@@ -290,16 +225,16 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
     Open image file.
   */
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Open temporary output file.
   */
   TemporaryFilename(postscript_filename);
-  file=fopen(postscript_filename,WriteBinaryType);
+  file=fopen(postscript_filename,"wb");
   if (file == (FILE *) NULL)
-    ThrowReaderException(FileOpenError,"Unable to write file",image);
+    ThrowReaderException(FileOpenError,"UnableToWriteFile",image);
   FormatString(translate_geometry,"%g %g translate\n              ",0.0,0.0);
   (void) fputs(translate_geometry,file);
   /*
@@ -309,44 +244,52 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   dy_resolution=72.0;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
-      (void) strcpy(density,PSDensityGeometry);
-      count=sscanf(density,"%lfx%lf",&image->x_resolution,
-        &image->y_resolution);
-      if (count != 2)
+      flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+      image->x_resolution=geometry_info.rho;
+      image->y_resolution=geometry_info.sigma;
+      if (!(flags & SigmaValue))
         image->y_resolution=image->x_resolution;
     }
   FormatString(density,"%gx%g",image->x_resolution,image->y_resolution);
-  SetGeometry(image,&page);
-  page.width=612;
-  page.height=792;
-  (void) GetGeometry(PSPageGeometry,&page.x,&page.y,&page.width,&page.height);
   /*
     Determine page geometry from the Postscript bounding box.
   */
   (void) ReadBlobLSBLong(image);
   count=ReadBlobLSBLong(image);
-  filesize=ReadBlobLSBLong(image);
+  (void) ReadBlobLSBLong(image);  // filesize
   for (i=0; i < (long) (count-12); i++)
     (void) ReadBlobByte(image);
   /*
     Copy Postscript to temporary file.
   */
+  memset(&page,0,sizeof(RectangleInfo));
+  skip=False;
   box.width=0;
   box.height=0;
-  p=command;
-  for (i=0; i < (long) filesize; i++)
+  for (p=command; (c=ReadBlobByte(image)) != EOF; )
   {
-    c=ReadBlobByte(image);
-    if (c == EOF)
-      break;
     (void) fputc(c,file);
+    if (image_info->page != (char *) NULL)
+      continue;
+    /*
+      Note document structuring comments.
+    */
     *p++=c;
     if ((c != '\n') && (c != '\r') && ((p-command) < (MaxTextExtent-1)))
       continue;
     *p='\0';
     p=command;
     /*
-      Parse a bounding box statement.
+      Skip %%BeginDocument thru %%EndDocument.
+    */
+    if (LocaleNCompare(BeginDocument,command,strlen(BeginDocument)) == 0)
+      skip=True;
+    if (LocaleNCompare(EndDocument,command,strlen(EndDocument)) == 0)
+      skip=False;
+    if (skip)
+      continue;
+    /*
+      Note region defined by bounding box.
     */
     count=0;
     if (LocaleNCompare(BoundingBox,command,strlen(BoundingBox)) == 0)
@@ -367,39 +310,37 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
       Set Postscript render geometry.
     */
     FormatString(translate_geometry,"%g %g translate\n",-bounds.x1,-bounds.y1);
-    width=(unsigned int) (bounds.x2-bounds.x1);
-    if ((float) ((int) bounds.x2) != bounds.x2)
-      width++;
-    height=(unsigned int) (bounds.y2-bounds.y1);
-    if ((float) ((int) bounds.y2) != bounds.y2)
-      height++;
+    width=(unsigned long) (bounds.x2-bounds.x1+0.5);
+    height=(unsigned long) (bounds.y2-bounds.y1+0.5);
     if ((width <= box.width) && (height <= box.height))
       continue;
     page.width=width;
     page.height=height;
     box=page;
   }
+  /*
+    Render postscript with the Ghostscript delegate.
+  */
+  if ((page.width == 0) || (page.height == 0))
+    (void) ParsePageGeometry(image,PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
-    (void) GetGeometry(image_info->page,&page.x,&page.y,&page.width,
-      &page.height);
-  FormatString(geometry,"%lux%lu",
-    (unsigned long) ceil(page.width*image->x_resolution/dx_resolution-0.5),
-    (unsigned long) ceil(page.height*image->y_resolution/dy_resolution-0.5));
+    (void) ParseAbsoluteGeometry(image_info->page,&page);
+  page.width=(unsigned long)
+    ceil(page.width*image->x_resolution/dx_resolution-0.5);
+  page.height=(unsigned long)
+    ceil(page.height*image->y_resolution/dy_resolution-0.5);
+  FormatString(geometry,"%lux%lu",page.width,page.height);
   if (ferror(file))
     {
       (void) fclose(file);
-      ThrowReaderException(FileOpenError,
-        "An error has occurred writing to file",image)
+      ThrowReaderException(CorruptImageError,"AnErrorHasOccurredWritingToFile",
+        image)
     }
   (void) rewind(file);
   (void) fputs(translate_geometry,file);
   (void) fclose(file);
   CloseBlob(image);
-  filesize=GetBlobSize(image);
   DestroyImage(image);
-  /*
-    Use Ghostscript to convert Postscript image.
-  */
   *options='\0';
   if (image_info->subrange != 0)
     FormatString(options,"-dFirstPage=%lu -dLastPage=%lu",
@@ -409,23 +350,23 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   FormatString(command,delegate_info->commands,image_info->antialias ? 4 : 1,
     image_info->antialias ? 4 : 1,geometry,density,options,image_info->filename,
     postscript_filename);
-  MagickMonitor(RenderPostscriptText,0,8);
-  status=ExecutePostscriptInterpreter(image_info->verbose,command);
+  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
+  status=InvokePostscriptDelegate(image_info->verbose,command);
   if (!IsAccessible(image_info->filename))
     {
       /*
         Ghostscript requires a showpage operator.
       */
-      file=fopen(postscript_filename,AppendBinaryType);
+      file=fopen(postscript_filename,"ab");
       if (file == (FILE *) NULL)
-        ThrowReaderException(FileOpenError,"Unable to write file",image);
+        ThrowReaderException(FileOpenError,"UnableToWriteFile",image);
       (void) fputs("showpage\n",file);
       (void) fclose(file);
-      status=ExecutePostscriptInterpreter(image_info->verbose,command);
+      status=InvokePostscriptDelegate(image_info->verbose,command);
     }
   (void) remove(postscript_filename);
-  MagickMonitor(RenderPostscriptText,7,8);
-  if (status)
+  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
+  if (!IsAccessible(image_info->filename))
     {
       /*
         Ghostscript has failed-- try the Display Postscript Extension.
@@ -434,8 +375,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
       image=ReadImage(image_info,exception);
       if (image != (Image *) NULL)
         return(image);
-      ThrowReaderException(CorruptImageError,"Postscript delegate failed",
-        image)
+      ThrowReaderException(CorruptImageError,"PostscriptDelegateFailed",image)
     }
   clone_info=CloneImageInfo(image_info);
   clone_info->blob=(void *) NULL;
@@ -444,13 +384,13 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   DestroyImageInfo(clone_info);
   (void) remove(image_info->filename);
   if (image == (Image *) NULL)
-    ThrowReaderException(CorruptImageError,"Postscript delegate failed",
-      image);
+    ThrowReaderException(CorruptImageError,"PostscriptDelegateFailed",image);
   do
   {
     (void) strcpy(image->magick,"PS");
     (void) strncpy(image->filename,filename,MaxTextExtent-1);
-    next_image=image->next;
+    image->page=page;
+    next_image=SyncNextImageInList(image);
     if (next_image != (Image *) NULL)
       image=next_image;
   } while (next_image != (Image *) NULL);
@@ -488,9 +428,9 @@ ModuleExport void RegisterEPTImage(void)
     *entry;
 
   entry=SetMagickInfo("EPT");
-  entry->decoder=ReadEPTImage;
-  entry->encoder=WriteEPTImage;
-  entry->magick=IsEPT;
+  entry->decoder=(DecoderHandler) ReadEPTImage;
+  entry->encoder=(EncoderHandler) WriteEPTImage;
+  entry->magick=(MagickHandler) IsEPT;
   entry->adjoin=False;
   entry->blob_support=False;
   entry->description=
@@ -593,10 +533,10 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
     Write EPT image.
   */
   (void) strncpy(image->filename,filename,MaxTextExtent-1);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  ps_file=fopen(ps_filename,ReadBinaryType);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  ps_file=fopen(ps_filename,"rb");
   status&=ps_file != (FILE *) NULL;
-  tiff_file=fopen(tiff_filename,ReadBinaryType);
+  tiff_file=fopen(tiff_filename,"rb");
   status&=tiff_file != (FILE *) NULL;
   if (status != False)
     {
@@ -632,6 +572,6 @@ static unsigned int WriteEPTImage(const ImageInfo *image_info,Image *image)
     (void) remove(ps_filename);
   (void) remove(tiff_filename);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   return(True);
 }
