@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -162,6 +168,9 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     scale,
     scaled_pixel;
 
+  ExtendedSignedIntegralType
+    number_pixels;
+
   FITSInfo
     fits_info;
 
@@ -172,7 +181,8 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     index;
 
   int
-    c;
+    c,
+    quantum;
 
   long
     exponent,
@@ -180,7 +190,6 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     k,
     l,
     packet_size,
-    quantum,
     scene,
     y;
 
@@ -210,9 +219,6 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     status,
     value_expected;
 
-  unsigned long
-    number_pixels;
-
   /*
     Open image file.
   */
@@ -221,9 +227,9 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Initialize image header.
   */
@@ -317,10 +323,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   /*
     Verify that required image information is defined.
   */
-  number_pixels=fits_info.columns*fits_info.rows;
+  number_pixels=(ExtendedSignedIntegralType) fits_info.columns*fits_info.rows;
   if ((!fits_info.simple) || (fits_info.number_axes < 1) ||
       (fits_info.number_axes > 4) || (number_pixels == 0))
-    ThrowReaderException(CorruptImageError,"image type not supported",image);
+    ThrowReaderException(CorruptImageError,"ImageTypeNotSupported",image);
   if (fits_info.bits_per_pixel == -32)
     {
       exponential[150]=1.0;
@@ -348,7 +354,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     image->storage_class=PseudoClass;
     image->scene=scene;
     if (!AllocateImageColormap(image,1 << image->depth))
-      ThrowReaderException(FileOpenError,"Unable to open file",image);
+      ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
     if (image_info->ping && (image_info->subrange != 0))
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
@@ -358,16 +364,19 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     packet_size=fits_info.bits_per_pixel/8;
     if (packet_size < 0)
       packet_size=(-packet_size);
-    number_pixels=image->columns*image->rows;
-    fits_pixels=(unsigned char *) AcquireMemory(packet_size*number_pixels);
+    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+    if ((packet_size*number_pixels) != (size_t) (packet_size*number_pixels))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    fits_pixels=(unsigned char *)
+      AcquireMemory(packet_size*image->columns*image->rows);
     if (fits_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     /*
       Convert FITS pixels to pixel packets.
     */
-    count=ReadBlob(image,packet_size*number_pixels,fits_pixels);
+    count=ReadBlob(image,packet_size*image->columns*image->rows,fits_pixels);
     if (count == 0)
-      ThrowReaderException(CorruptImageError,"Insufficient image data in file",
+      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile",
         image);
     if ((fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
       {
@@ -385,11 +394,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         pixel=(double) quantum;
         if (fits_info.bits_per_pixel == 16)
           if (pixel > 32767)
-            pixel-=65536;
+            pixel-=65536L;
         if (fits_info.bits_per_pixel == -32)
           {
-            j=((int) long_quantum[1] << 16) | ((int) long_quantum[2] << 8) |
-               (int) long_quantum[3];
+            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
+               (long) long_quantum[3];
             k=(int) *long_quantum;
             exponent=((k & 127) << 1) | (j >> 23);
             *(float *) long_quantum=
@@ -402,10 +411,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           }
         if (fits_info.bits_per_pixel == -64)
           {
-            j=((unsigned int) long_quantum[1] << 24) |
-              ((unsigned int) long_quantum[2] << 16) |
-              ((unsigned int) long_quantum[3] << 8) |
-               (unsigned int) long_quantum[4];
+            j=((long) long_quantum[1] << 24) |
+              ((long) long_quantum[2] << 16) |
+              ((long) long_quantum[3] << 8) |
+               (long) long_quantum[4];
             k=(int) *long_quantum;
             l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
                (int) long_quantum[7];
@@ -432,11 +441,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           pixel=(double) quantum;
           if (fits_info.bits_per_pixel == 16)
             if (pixel > 32767)
-              pixel-=65536;
+              pixel-=65536L;
           if (fits_info.bits_per_pixel == -32)
             {
-              j=((int) long_quantum[1] << 16) | ((int) long_quantum[2] << 8) |
-                 (int) long_quantum[3];
+              j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
+                 (long) long_quantum[3];
               k=(int) *long_quantum;
               exponent=((k & 127) << 1) | (j >> 23);
               *(float *) long_quantum=
@@ -449,10 +458,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
             }
           if (fits_info.bits_per_pixel == -64)
             {
-              j=((unsigned int) long_quantum[1] << 24) |
-                ((unsigned int) long_quantum[2] << 16) |
-                ((unsigned int) long_quantum[3] << 8) |
-                 (unsigned int) long_quantum[4];
+              j=((long) long_quantum[1] << 24) |
+                ((long) long_quantum[2] << 16) |
+                ((long) long_quantum[3] << 8) |
+                 (long) long_quantum[4];
               k=(int) *long_quantum;
               l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
                  (int) long_quantum[7];
@@ -498,11 +507,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         pixel=(double) quantum;
         if (fits_info.bits_per_pixel == 16)
           if (pixel > 32767)
-            pixel-=65536;
+            pixel-=65536L;
         if (fits_info.bits_per_pixel == -32)
           {
-            j=((int) long_quantum[1] << 16) | ((int) long_quantum[2] << 8) |
-               (int) long_quantum[3];
+            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
+               (long) long_quantum[3];
             k=(int) *long_quantum;
             exponent=((k & 127) << 1) | (j >> 23);
             *(float *) long_quantum=
@@ -515,13 +524,13 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           }
         if (fits_info.bits_per_pixel == -64)
           {
-            j=((unsigned int) long_quantum[1] << 24) |
-              ((unsigned int) long_quantum[2] << 16) |
-              ((unsigned int) long_quantum[3] << 8) |
-               (unsigned int) long_quantum[4];
+            j=((long) long_quantum[1] << 24) |
+              ((long) long_quantum[2] << 16) |
+              ((long) long_quantum[3] << 8) |
+               (long) long_quantum[4];
             k=(int) *long_quantum;
-            l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
-               (int) long_quantum[7];
+            l=((long) long_quantum[5] << 16) | ((long) long_quantum[6] << 8) |
+               (long) long_quantum[7];
             exponent=((k & 127) << 4) | (j >> 28);
             *(double *) long_quantum=exponential[exponent]*(16777216.0*
               (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
@@ -543,11 +552,16 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
       if (!SyncImagePixels(image))
         break;
       if (QuantumTick(y,image->rows))
-        MagickMonitor(LoadImageText,y,image->rows);
+        if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+          break;
     }
     LiberateMemory((void **) &fits_pixels);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -565,8 +579,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   }
   while (image->previous != (Image *) NULL)
@@ -604,9 +621,9 @@ ModuleExport void RegisterFITSImage(void)
     *entry;
 
   entry=SetMagickInfo("FITS");
-  entry->decoder=ReadFITSImage;
-  entry->encoder=WriteFITSImage;
-  entry->magick=IsFITS;
+  entry->decoder=(DecoderHandler) ReadFITSImage;
+  entry->encoder=(EncoderHandler) WriteFITSImage;
+  entry->magick=(MagickHandler) IsFITS;
   entry->adjoin=False;
   entry->description=AcquireString("Flexible Image Transport System");
   entry->module=AcquireString("FITS");
@@ -679,16 +696,18 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register int
+  register long
     i;
 
   unsigned char
     *pixels;
 
   unsigned int
-    packet_size,
     status;
 
+  unsigned long
+    packet_size;
+
   /*
     Open output image file.
   */
@@ -696,10 +715,10 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   /*
     Allocate image memory.
   */
@@ -707,7 +726,7 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
   fits_info=(char *) AcquireMemory(2880);
   pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
   if ((fits_info == (char *) NULL) || (pixels == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Initialize image header.
   */
@@ -744,7 +763,12 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
     (void) PopImagePixels(image,GrayQuantum,pixels);
     (void) WriteBlob(image,packet_size*image->columns,pixels);
     if (QuantumTick(image->rows-y-1,image->rows))
-      MagickMonitor(SaveImageText,image->rows-y-1,image->rows);
+      {
+        status=MagickMonitor(SaveImageTag,image->rows-y-1,image->rows,
+          &image->exception);
+        if (status == False)
+          break;
+      }
   }
   LiberateMemory((void **) &pixels);
   CloseBlob(image);
