@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 #if defined(HasFPX)
 #if !defined(vms) && !defined(macintosh) && !defined(WIN32)
 #include <fpxlib.h>
@@ -139,9 +146,6 @@ static unsigned int IsFPX(const unsigned char *magick,const size_t length)
 */
 static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  FILE
-    *file;
-
   FPXColorspace
     colorspace;
 
@@ -169,9 +173,6 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  int
-    c;
-
   long
     y;
 
@@ -213,33 +214,21 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
-  /*
-    Copy image to temporary file.
-  */
-  TemporaryFilename((char *) image_info->filename);
-  file=fopen(image_info->filename,WriteBinaryType);
-  if (file == (FILE *) NULL)
-    ThrowReaderException(FileOpenError,"Unable to write file",image);
-  c=ReadBlobByte(image);
-  while (c != EOF)
-  {
-    (void) fputc(c,file);
-    c=ReadBlobByte(image);
-  }
-  (void) fclose(file);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
+  FPX_InitSystem();
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    ThrowReaderException(DelegateError,"Unable to initialize FPX library",
-      image);
+    {
+      FPX_ClearSystem();
+      ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary",image);
+    }
   tile_width=64;
   tile_height=64;
   flashpix=(FPXImageHandle *) NULL;
@@ -258,13 +247,12 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (fpx_status == FPX_LOW_MEMORY_ERROR)
     {
       FPX_ClearSystem();
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     }
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(FileOpenError,"Unable to open FPX file",image);
+      ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
     }
   if (image_info->view == (char *) NULL)
     {
@@ -277,8 +265,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       aspect_ratio=(float) width/height;
       fpx_status=FPX_GetImageResultAspectRatio(flashpix,&aspect_ratio);
       if (fpx_status != FPX_OK)
-        ThrowReaderException(DelegateError,"Unable to read aspect ratio",
-          image);
+        ThrowReaderException(DelegateError,"UnableToReadAspectRatio",image);
       if (width != (unsigned long) ((aspect_ratio*height)+0.5))
         Swap(width,height);
     }
@@ -286,7 +273,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(FileOpenError,"Unable to read summary info",image);
+      ThrowReaderException(DelegateError,"UnableToReadSummaryInfo",image);
     }
   if (summary_info.title_valid)
     if ((summary_info.title.length != 0) &&
@@ -302,8 +289,8 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (label == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitError,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+              image);
           }
         (void) strncpy(label,(char *) summary_info.title.ptr,
           summary_info.title.length);
@@ -325,8 +312,8 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (comments == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitError,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+              image);
           }
         (void) strncpy(comments,(char *) summary_info.comments.ptr,
           summary_info.comments.length);
@@ -365,10 +352,10 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Create linear colormap.
       */
-      if (!AllocateImageColormap(image,MaxRGB+1))
+      if (!AllocateImageColormap(image,MaxColormapSize))
         {
           FPX_ClearSystem();
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         }
     }
@@ -376,7 +363,6 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
-      (void) remove(image->filename);
       return(image);
     }
   /*
@@ -388,8 +374,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       FPX_ClearSystem();
       (void) FPX_CloseImage(flashpix);
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     }
   /*
     Initialize FlashPix image description.
@@ -443,8 +428,8 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             LiberateMemory((void **) &scanline);
             (void) FPX_CloseImage(flashpix);
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitError,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+              image);
           }
       }
     /*
@@ -458,20 +443,20 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       if (fpx_info.numberOfComponents > 2)
         {
-          q->red=Upscale(*r);
-          q->green=Upscale(*g);
-          q->blue=Upscale(*b);
+          q->red=ScaleCharToQuantum(*r);
+          q->green=ScaleCharToQuantum(*g);
+          q->blue=ScaleCharToQuantum(*b);
         }
       else
         {
-          index=Upscale(*r);
+          index=ScaleCharToQuantum(*r);
           indexes[x]=index;
           q->red=index;
           q->green=index;
           q->blue=index;
         }
       if (image->matte)
-        q->opacity=MaxRGB-Upscale(*a);
+        q->opacity=ScaleCharToQuantum(255-*a);
       q++;
       r+=red_component->columnStride;
       g+=green_component->columnStride;
@@ -481,19 +466,19 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        break;
   }
   LiberateMemory((void **) &scanline);
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  (void) remove(image->filename);
   return(image);
 }
 #else
 static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  ThrowException(exception,MissingDelegateError,
-    "FPX library is not available",image_info->filename);
+  ThrowException(exception,CoderError,"FPXLibraryIsNotAvailable",
+    image_info->filename);
   return((Image *) NULL);
 }
 #endif
@@ -527,10 +512,11 @@ ModuleExport void RegisterFPXImage(void)
     *entry;
 
   entry=SetMagickInfo("FPX");
-  entry->decoder=ReadFPXImage;
-  entry->encoder=WriteFPXImage;
+  entry->decoder=(DecoderHandler) ReadFPXImage;
+  entry->encoder=(EncoderHandler) WriteFPXImage;
   entry->adjoin=False;
-  entry->magick=IsFPX;
+  entry->seekable_stream=True;
+  entry->magick=(MagickHandler) IsFPX;
   entry->description=AcquireString("FlashPix Format");
   entry->module=AcquireString("FPX");
   (void) RegisterMagickInfo(entry);
@@ -775,9 +761,6 @@ static void SetSaturation(double saturation,FPXColorTwistMatrix *color_twist)
 
 static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
 {
-  char
-    filename[MaxTextExtent];
-
   FPXBackground
     background_color;
 
@@ -836,19 +819,18 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   /*
     Initialize FPX toolkit.
   */
   image->depth=8;
-  (void) TransformRGBImage(image,RGBColorspace);
+  (void) TransformRGBImage(image,image->colorspace);
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(ResourceLimitError,
-      "Unable to initialize FPX library",image);
+    ThrowWriterException(DelegateError,"UnableToInitializeFPXLibrary",image);
   tile_width=64;
   tile_height=64;
   colorspace.numberOfComponents=3;
@@ -867,12 +849,6 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   compression=NONE;
   if (image_info->compression == JPEGCompression)
     compression=JPEG_UNSPECIFIED;
-  (void) strncpy(filename,image->filename,MaxTextExtent-1);
-  if ((image->blob->file == stdout) || image->blob->pipet ||
-      (image->blob->data != (unsigned char *) NULL))
-    TemporaryFilename(filename);
-  else
-    CloseBlob(image);
   {
 #if defined(macintosh)
     FSSpec
@@ -881,13 +857,13 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     FilenameToFSSpec(filename,&fsspec);
     fpx_status=FPX_CreateImageByFilename((const FSSpec &) fsspec,image->columns,
 #else
-    fpx_status=FPX_CreateImageByFilename(filename,image->columns,
+    fpx_status=FPX_CreateImageByFilename(image->filename,image->columns,
 #endif
       image->rows,tile_width,tile_height,colorspace,background_color,
       compression,&flashpix);
   }
   if (fpx_status != FPX_OK)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   if (image_info->compression == JPEGCompression)
     {
       /*
@@ -896,7 +872,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       fpx_status=
         FPX_SetJPEGCompression(flashpix,(unsigned short) (image_info->quality));
       if (fpx_status != FPX_OK)
-        ThrowWriterException(DelegateError,"Unable to set JPEG level",image);
+        ThrowWriterException(DelegateError,"UnableToSetJPEGLevel",image);
     }
   /*
     Set image summary info.
@@ -933,7 +909,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
         (void) strncpy((char *) summary_info.title.ptr,label->value,
           MaxTextExtent-1);
       else
-        ThrowWriterException(DelegateError,"Unable to set image title",image);
+        ThrowWriterException(DelegateError,"UnableToSetImageTitle",image);
     }
   comment=GetImageAttribute(image,"comment");
   if (comment != (ImageAttribute *) NULL)
@@ -948,12 +924,11 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       if (summary_info.comments.ptr != (unsigned char *) NULL)
         (void) strcpy((char *) summary_info.comments.ptr,comment->value);
       else
-        ThrowWriterException(DelegateError,"Unable to set image comments",
-          image);
+        ThrowWriterException(DelegateError,"UnableToSetImageComments",image);
     }
   fpx_status=FPX_SetSummaryInformation(flashpix,&summary_info);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(DelegateError,"Unable to set summary info",image);
+    ThrowWriterException(DelegateError,"UnableToSetSummaryInfo",image);
   /*
     Allocate pixels.
   */
@@ -963,8 +938,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     }
   /*
     Initialize FlashPix image description.
@@ -1003,7 +977,8 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     if (fpx_status != FPX_OK)
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+        break;
   }
   if (image_info->view != (char *) NULL)
     {
@@ -1095,75 +1070,51 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
         {
           fpx_status=FPX_SetImageAffineMatrix(flashpix,&affine);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"Unable to set affine matrix",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetAffineMatrix",image);
         }
       if (aspect_ratio_valid)
         {
           fpx_status=FPX_SetImageResultAspectRatio(flashpix,&aspect_ratio);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"Unable to set aspect ratio",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetAspectRatio",image);
         }
       if (color_twist_valid)
         {
           fpx_status=FPX_SetImageColorTwistMatrix(flashpix,&color_twist);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,
-              "Unable to set color color twist",image);
+            ThrowWriterException(DelegateError,"UnableToSetColorTwist",image);
         }
       if (contrast_valid)
         {
           fpx_status=FPX_SetImageContrastAdjustment(flashpix,&contrast);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"Unable to set contrast",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetContrast",image);
         }
       if (sharpen_valid)
         {
           fpx_status=FPX_SetImageFilteringValue(flashpix,&sharpen);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,
-              "Unable to set filtering value",image);
+            ThrowWriterException(DelegateError,"UnableToSetFilteringValue",
+              image);
         }
       if (view_rect_valid)
         {
           fpx_status=FPX_SetImageROI(flashpix,&view_rect);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,
-              "Unable to set region of interest",image);
+            ThrowWriterException(DelegateError,"UnableToSetRegionOfInterest",
+              image);
         }
     }
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
   LiberateMemory((void **) &pixels);
-  if ((image->blob->file == stdout) || image->blob->pipet ||
-      (image->blob->data != (unsigned char *) NULL))
-    {
-      FILE
-        *file;
-
-      int
-        c;
-
-      /*
-        Copy temporary file to image blob->
-      */
-      file=fopen(filename,ReadBinaryType);
-      if (file == (FILE *) NULL)
-        ThrowWriterException(FileOpenError,"Unable to open file",image);
-      for (c=fgetc(file); c != EOF; c=fgetc(file))
-        (void) WriteBlobByte(image,c);
-      (void) fclose(file);
-      (void) remove(filename);
-      CloseBlob(image);
-    }
+  CloseBlob(image);
   return(True);
 }
 #else
 static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
 {
-  ThrowBinaryException(MissingDelegateError,"FPX library is not available",
+  ThrowBinaryException(CoderError,"FPXLibraryIsNotAvailable",
     image->filename)
 }
 #endif
