@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "quantize.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -155,14 +163,14 @@ static unsigned int DecodeImage(Image *image,const long opacity)
       (prefix == (short *) NULL) ||
       (suffix == (unsigned char *) NULL) ||
       (pixel_stack == (unsigned char *) NULL))
-    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   /*
     Initialize GIF data stream decoder.
   */
   data_size=ReadBlobByte(image);
   if (data_size > 8)
-    ThrowBinaryException(CorruptImageError,"Corrupt GIF image",image->filename);
+    ThrowBinaryException(CorruptImageError,"CorruptGIFImage",image->filename);
   clear=1 << data_size;
   end_of_information=clear+1;
   available=clear+2;
@@ -281,7 +289,7 @@ static unsigned int DecodeImage(Image *image,const long opacity)
       if (index >= image->colors)
         {
           ThrowException(&image->exception,CorruptImageError,
-            "invalid colormap index",image->filename);
+            "InvalidColormapIndex",image->filename);
           index=0;
         }
       indexes[x]=index;
@@ -339,14 +347,15 @@ static unsigned int DecodeImage(Image *image,const long opacity)
       break;
     if (image->previous == (Image *) NULL)
       if (QuantumTick(y,image->rows))
-        MagickMonitor(LoadImageText,y,image->rows);
+        if (!MagickMonitor(LoadImageTag,y,image->rows,&image->exception))
+          break;
   }
-  if (y < (long) image->rows)
-    ThrowBinaryException(CorruptImageError,"Corrupt GIF image",image->filename);
   LiberateMemory((void **) &pixel_stack);
   LiberateMemory((void **) &suffix);
   LiberateMemory((void **) &prefix);
   LiberateMemory((void **) &packet);
+  if (y < (long) image->rows)
+    ThrowBinaryException(CorruptImageError,"CorruptGIFImage",image->filename);
   return(True);
 }
 
@@ -615,7 +624,8 @@ static unsigned int EncodeImage(const ImageInfo *image_info,Image *image,
       }
     if (image->previous == (Image *) NULL)
       if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+          break;
   }
   /*
     Flush out the buffered code.
@@ -803,7 +813,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     header[MaxTextExtent],
     magick[12];
 
-  unsigned int
+  unsigned long
     delay,
     dispose,
     global_colors,
@@ -818,16 +828,16 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Determine if this is a GIF file.
   */
   count=ReadBlob(image,6,(char *) magick);
   if ((count == 0) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
       (LocaleNCompare((char *) magick,"GIF89",5) != 0)))
-    ThrowReaderException(CorruptImageError,"Not a GIF image file",image);
+    ThrowReaderException(CorruptImageError,"NotAGIFImageFile",image);
   global_colors=0;
   global_colormap=(unsigned char *) NULL;
   page.width=ReadBlobLSBShort(image);
@@ -838,7 +848,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   global_colors=1 << ((flag & 0x07)+1);
   global_colormap=(unsigned char *) AcquireMemory(3*Max(global_colors,256));
   if (global_colormap == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (BitSet(flag,0x80))
     (void) ReadBlob(image,3*global_colors,(char *) global_colormap);
   delay=0;
@@ -860,8 +870,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         count=ReadBlob(image,1,(char *) &c);
         if (count == 0)
-          ThrowReaderException(CorruptImageError,
-            "Unable to read extension block",image);
+          ThrowReaderException(CorruptImageError,"UnableToReadExtensionBlock",
+            image);
         switch (c)
         {
           case 0xf9:
@@ -934,8 +944,11 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             LiberateMemory((void **) &global_colormap);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
     image_count++;
     /*
@@ -958,20 +971,19 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->page.y=page.y;
     image->page.x=page.x;
     image->delay=delay;
-    image->dispose=dispose;
+    image->dispose=(DisposeType) dispose;
     image->iterations=iterations;
     image->matte=opacity >= 0;
     delay=0;
     dispose=0;
     iterations=1;
     if ((image->columns == 0) || (image->rows == 0))
-      ThrowReaderException(CorruptImageError,"image size is 0",image);
+      ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize",image);
     /*
       Inititialize colormap.
     */
     if (!AllocateImageColormap(image,image->colors))
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     if (!BitSet(flag,0x80))
       {
         /*
@@ -980,9 +992,9 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         p=global_colormap;
         for (i=0; i < (long) image->colors; i++)
         {
-          image->colormap[i].red=Upscale(*p++);
-          image->colormap[i].green=Upscale(*p++);
-          image->colormap[i].blue=Upscale(*p++);
+          image->colormap[i].red=ScaleCharToQuantum(*p++);
+          image->colormap[i].green=ScaleCharToQuantum(*p++);
+          image->colormap[i].blue=ScaleCharToQuantum(*p++);
         }
         image->background_color=
           image->colormap[Min(background,image->colors-1)];
@@ -997,15 +1009,15 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         colormap=(unsigned char *) AcquireMemory(3*image->colors);
         if (colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         (void) ReadBlob(image,3*image->colors,(char *) colormap);
         p=colormap;
         for (i=0; i < (long) image->colors; i++)
         {
-          image->colormap[i].red=Upscale(*p++);
-          image->colormap[i].green=Upscale(*p++);
-          image->colormap[i].blue=Upscale(*p++);
+          image->colormap[i].red=ScaleCharToQuantum(*p++);
+          image->colormap[i].green=ScaleCharToQuantum(*p++);
+          image->colormap[i].blue=ScaleCharToQuantum(*p++);
         }
         LiberateMemory((void **) &colormap);
       }
@@ -1017,14 +1029,14 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     status=DecodeImage(image,opacity);
     if (!image_info->ping && (status == False))
-      ThrowReaderException(CorruptImageError,"Corrupt GIF image",image);
+      ThrowReaderException(CorruptImageError,"CorruptGIFImage",image);
     if (image_info->subrange != 0)
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
   }
   LiberateMemory((void **) &global_colormap);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(CorruptImageError,"image size is 0",image);
+    ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize",image);
   while (image->previous != (Image *) NULL)
     image=image->previous;
   CloseBlob(image);
@@ -1060,9 +1072,9 @@ ModuleExport void RegisterGIFImage(void)
     *entry;
 
   entry=SetMagickInfo("GIF");
-  entry->decoder=ReadGIFImage;
-  entry->encoder=WriteGIFImage;
-  entry->magick=IsGIF;
+  entry->decoder=(DecoderHandler) ReadGIFImage;
+  entry->encoder=(EncoderHandler) WriteGIFImage;
+  entry->magick=(MagickHandler) IsGIF;
   entry->description=AcquireString("CompuServe graphics interchange format");
 #if !defined(HasLZW)
   entry->version=AcquireString("LZW disabled");
@@ -1070,9 +1082,9 @@ ModuleExport void RegisterGIFImage(void)
   entry->module=AcquireString("GIF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("GIF87");
-  entry->decoder=ReadGIFImage;
-  entry->encoder=WriteGIFImage;
-  entry->magick=IsGIF;
+  entry->decoder=(DecoderHandler) ReadGIFImage;
+  entry->encoder=(EncoderHandler) WriteGIFImage;
+  entry->magick=(MagickHandler) IsGIF;
   entry->adjoin=False;
   entry->description=AcquireString("CompuServe graphics interchange format");
   entry->version=AcquireString("version 87a");
@@ -1158,7 +1170,7 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register int
+  register long
     x;
 
   register long
@@ -1178,9 +1190,11 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
 
   unsigned int
     interlace,
-    scene,
     status;
 
+  unsigned long
+    scene;
+
   /*
     Open output image file.
   */
@@ -1188,9 +1202,9 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   /*
     Determine image bounding box.
   */
@@ -1215,7 +1229,7 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
   colormap=(unsigned char *) AcquireMemory(768);
   if ((global_colormap == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   for (i=0; i < 768; i++)
     colormap[i]=0;
   /*
@@ -1229,12 +1243,12 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlob(image,6,"GIF87a");
     else
       (void) WriteBlob(image,6,"GIF89a");
+  page.x=image->page.x;
+  page.y=image->page.y;
   if ((image->page.width != 0) && (image->page.height != 0))
     page=image->page;
   (void) WriteBlobLSBShort(image,page.width);
   (void) WriteBlobLSBShort(image,page.height);
-  page.x=0;
-  page.y=0;
   /*
     Write images to file.
   */
@@ -1245,7 +1259,7 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     if ((image->storage_class == DirectClass) || (image->colors > 256))
       {
         /*
@@ -1268,7 +1282,7 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
                 LiberateMemory((void **) &global_colormap);
                 LiberateMemory((void **) &colormap);
                 ThrowWriterException(ResourceLimitError,
-                  "Memory allocation failed",image)
+                  "MemoryAllocationFailed",image)
               }
             image->colormap[opacity]=image->background_color;
             for (y=0; y < (long) image->rows; y++)
@@ -1305,7 +1319,7 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
             {
               if (p->opacity == TransparentOpacity)
                 {
-                  opacity=indexes[x];
+                  opacity=(long) indexes[x];
                   break;
                 }
               p++;
@@ -1322,9 +1336,9 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
     q=colormap;
     for (i=0; i < (long) image->colors; i++)
     {
-      *q++=Downscale(image->colormap[i].red);
-      *q++=Downscale(image->colormap[i].green);
-      *q++=Downscale(image->colormap[i].blue);
+      *q++=ScaleQuantumToChar(image->colormap[i].red);
+      *q++=ScaleQuantumToChar(image->colormap[i].green);
+      *q++=ScaleQuantumToChar(image->colormap[i].blue);
     }
     for ( ; i < (1L << bits_per_pixel); i++)
     {
@@ -1358,7 +1372,7 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobByte(image,0x21);
         (void) WriteBlobByte(image,0xf9);
         (void) WriteBlobByte(image,0x04);
-        c=(unsigned char) (image->dispose << 2);
+        c=(unsigned char) ((int) image->dispose << 2);
         if (opacity >= 0)
           c|=0x01;
         (void) WriteBlobByte(image,c);
@@ -1412,16 +1426,10 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
     /*
       Write the image header.
     */
+    page.x=image->page.x;
+    page.y=image->page.y;
     if ((image->page.width != 0) && (image->page.height != 0))
-      {
-        page=image->page;
-        if ((image->previous == (Image *) NULL) &&
-            (image->next == (Image *) NULL))
-          {
-            page.x=0;
-            page.y=0;
-          }
-      }
+      page=image->page;
     (void) WriteBlobLSBShort(image,page.x);
     (void) WriteBlobLSBShort(image,page.y);
     (void) WriteBlobLSBShort(image,image->columns);
@@ -1451,14 +1459,16 @@ static unsigned int WriteGIFImage(const ImageInfo *image_info,Image *image)
       {
         LiberateMemory((void **) &global_colormap);
         LiberateMemory((void **) &colormap);
-        ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-          image)
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image)
       }
     (void) WriteBlobByte(image,0x0);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   (void) WriteBlobByte(image,';'); /* terminator */
   LiberateMemory((void **) &global_colormap);
