@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -116,9 +122,11 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     *scanline;
 
   unsigned int
-    packet_size,
     status;
 
+  unsigned long
+    packet_size;
+
   /*
     Open image file.
   */
@@ -128,10 +136,10 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"Must specify image size",image);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   /*
@@ -140,7 +148,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   packet_size=image->depth > 8 ? 2 : 1;
   scanline=(unsigned char *) AcquireMemory(packet_size*image->tile_info.width);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->subrange != 0)
     while (image->scene < image_info->subimage)
     {
@@ -158,8 +166,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       Convert raster image to pixel packets.
     */
     if (!AllocateImageColormap(image,1 << image->depth))
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     if (image_info->ping && (image_info->subrange != 0))
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
@@ -177,13 +184,18 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
     }
     count=image->tile_info.height-image->rows-image->tile_info.y;
     for (j=0; j < (long) count; j++)
       (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -202,8 +214,11 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (count != 0);
   LiberateMemory((void **) &scanline);
@@ -242,71 +257,71 @@ ModuleExport void RegisterGRAYImage(void)
     *entry;
 
   entry=SetMagickInfo("GRAY");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->raw=True;
   entry->description=AcquireString("Raw gray samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("R");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw red samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("C");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw cyan samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("G");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw green samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw magenta samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("B");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw blue samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("Y");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw yellow samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("O");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw opacity samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("K");
-  entry->decoder=ReadGRAYImage;
-  entry->encoder=WriteGRAYImage;
+  entry->decoder=(DecoderHandler) ReadGRAYImage;
+  entry->encoder=(EncoderHandler) WriteGRAYImage;
   entry->stealth=True;
   entry->raw=True;
   entry->description=AcquireString("Raw black samples");
@@ -399,9 +414,9 @@ static unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   /*
     Convert image to gray scale PseudoColor class.
   */
@@ -411,12 +426,11 @@ static unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
     /*
       Allocate memory for scanline.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     packet_size=image->depth > 8 ? 2: 1;
     scanline=(unsigned char *) AcquireMemory(packet_size*image->columns);
     if (scanline == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     /*
       Convert MIFF to GRAY raster scanline.
     */
@@ -429,13 +443,17 @@ static unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlob(image,packet_size*image->columns,scanline);
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(SaveImageText,y,image->rows);
+          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+            break;
     }
     LiberateMemory((void **) &scanline);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
