@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "magick.h"
+#include "paint.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -127,23 +133,23 @@ ModuleExport void RegisterHTMLImage(void)
     *entry;
 
   entry=SetMagickInfo("HTM");
-  entry->encoder=WriteHTMLImage;
-  entry->magick=IsHTML;
+  entry->encoder=(EncoderHandler) WriteHTMLImage;
+  entry->magick=(MagickHandler) IsHTML;
   entry->adjoin=False;
   entry->description=
     AcquireString("Hypertext Markup Language and a client-side image map");
   entry->module=AcquireString("HTML");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("HTML");
-  entry->encoder=WriteHTMLImage;
-  entry->magick=IsHTML;
+  entry->encoder=(EncoderHandler) WriteHTMLImage;
+  entry->magick=(MagickHandler) IsHTML;
   entry->adjoin=False;
   entry->description=
     AcquireString("Hypertext Markup Language and a client-side image map");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SHTML");
-  entry->encoder=WriteHTMLImage;
-  entry->magick=IsHTML;
+  entry->encoder=(EncoderHandler) WriteHTMLImage;
+  entry->magick=(MagickHandler) IsHTML;
   entry->adjoin=False;
   entry->description=
     AcquireString("Hypertext Markup Language and a client-side image map");
@@ -237,11 +243,11 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   CloseBlob(image);
-  (void) TransformRGBImage(image,RGBColorspace);
+  (void) TransformRGBImage(image,image->colorspace);
   *url='\0';
   if ((LocaleCompare(image_info->magick,"FTP") == 0) ||
       (LocaleCompare(image_info->magick,"HTTP") == 0))
@@ -280,9 +286,9 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == False)
-        ThrowWriterException(FileOpenError,"Unable to open file",image);
+        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
       /*
         Write the HTML image file.
       */
@@ -314,12 +320,8 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
       */
       SetGeometry(image,&geometry);
       if (image->montage != (char *) NULL)
-        {
-          (void) GetGeometry(image->montage,&geometry.x,&geometry.y,
-            &geometry.width,&geometry.height);
-          (void) GetMagickGeometry(image->montage,&geometry.x,&geometry.y,
-            &geometry.width,&geometry.height);
-        }
+        (void) ParseMetaGeometry(image->montage,&geometry.x,&geometry.y,
+          &geometry.width,&geometry.height);
       /*
         Write an image map.
       */
@@ -390,21 +392,17 @@ static unsigned int WriteHTMLImage(const ImageInfo *image_info,Image *image)
   /*
     Open image map.
   */
-  status=OpenBlob(clone_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(clone_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   DestroyImageInfo(clone_info);
   /*
     Determine the size and location of each image tile.
   */
   SetGeometry(image,&geometry);
   if (image->montage != (char *) NULL)
-    {
-      (void) GetGeometry(image->montage,&geometry.x,&geometry.y,
-        &geometry.width,&geometry.height);
-      (void) GetMagickGeometry(image->montage,&geometry.x,&geometry.y,
-        &geometry.width,&geometry.height);
-    }
+    (void) ParseMetaGeometry(image->montage,&geometry.x,&geometry.y,
+      &geometry.width,&geometry.height);
   /*
     Write an image map.
   */
