@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -179,15 +185,15 @@ static Image *ReadIconImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   icon_file.reserved=ReadBlobLSBShort(image);
   icon_file.resource_type=ReadBlobLSBShort(image);
   icon_file.count=ReadBlobLSBShort(image);
   if ((icon_file.reserved != 0) || (icon_file.resource_type != 1) ||
       (icon_file.count > MaxIcons))
-    ThrowReaderException(CorruptImageError,"Not a ICO image file",image);
+    ThrowReaderException(CorruptImageError,"NotAnICOImageFile",image);
   for (i=0; i < icon_file.count; i++)
   {
     icon_file.directory[i].width=ReadBlobByte(image);
@@ -239,19 +245,17 @@ static Image *ReadIconImage(const ImageInfo *image_info,
         Read Icon raster colormap.
       */
       if (!AllocateImageColormap(image,1 << icon_info.bits_per_pixel))
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       icon_colormap=(unsigned char *) AcquireMemory(4*image->colors);
       if (icon_colormap == (unsigned char *) NULL)
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       (void) ReadBlob(image,4*image->colors,(char *) icon_colormap);
       p=icon_colormap;
       for (i=0; i < (long) image->colors; i++)
       {
-        image->colormap[i].blue=(Quantum) Upscale(*p++);
-        image->colormap[i].green=(Quantum) Upscale(*p++);
-        image->colormap[i].red=(Quantum) Upscale(*p++);
+        image->colormap[i].blue=(Quantum) ScaleCharToQuantum(*p++);
+        image->colormap[i].green=(Quantum) ScaleCharToQuantum(*p++);
+        image->colormap[i].red=(Quantum) ScaleCharToQuantum(*p++);
         p++;
       }
       LiberateMemory((void **) &icon_colormap);
@@ -293,7 +297,12 @@ static Image *ReadIconImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+              {
+                status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                  &image->exception);
+                if (status == False)
+                  break;
+              }
         }
         break;
       }
@@ -323,7 +332,12 @@ static Image *ReadIconImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+              {
+                status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                  &image->exception);
+                if (status == False)
+                  break;
+              }
         }
         break;
       }
@@ -347,7 +361,12 @@ static Image *ReadIconImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+              {
+                status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                  &image->exception);
+                if (status == False)
+                  break;
+              }
         }
         break;
       }
@@ -373,7 +392,12 @@ static Image *ReadIconImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+              {
+                status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                  &image->exception);
+                if (status == False)
+                  break;
+              }
         }
         break;
       }
@@ -390,23 +414,28 @@ static Image *ReadIconImage(const ImageInfo *image_info,
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->blue=(Quantum) Upscale(ReadBlobByte(image));
-            q->green=(Quantum) Upscale(ReadBlobByte(image));
-            q->red=(Quantum) Upscale(ReadBlobByte(image));
+            q->blue=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
+            q->green=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
+            q->red=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
             if (image->matte)
-              q->opacity=(Quantum) Upscale(ReadBlobByte(image));
+              q->opacity=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
             q++;
           }
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+              {
+                status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+                  &image->exception);
+                if (status == False)
+                  break;
+              }
         }
         break;
       }
       default:
-        ThrowReaderException(CorruptImageError,"Not a ICO image file",image)
+        ThrowReaderException(CorruptImageError,"NotAnICOImageFile",image)
     }
     SyncImage(image);
     /*
@@ -440,10 +469,19 @@ static Image *ReadIconImage(const ImageInfo *image_info,
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,image->rows-y-1,image->rows);
+          {
+            status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
+              &image->exception);
+            if (status == False)
+              break;
+          }
     }
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -461,8 +499,11 @@ static Image *ReadIconImage(const ImageInfo *image_info,
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   }
   while (image->previous != (Image *) NULL)
@@ -500,14 +541,16 @@ ModuleExport void RegisterICONImage(void)
     *entry;
 
   entry=SetMagickInfo("ICO");
-  entry->decoder=ReadIconImage;
+  entry->decoder=(DecoderHandler) ReadIconImage;
   entry->adjoin=False;
+  entry->seekable_stream=True;
   entry->description=AcquireString("Microsoft icon");
   entry->module=AcquireString("ICON");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ICON");
-  entry->decoder=ReadIconImage;
+  entry->decoder=(DecoderHandler) ReadIconImage;
   entry->adjoin=False;
+  entry->seekable_stream=True;
   entry->description=AcquireString("Microsoft icon");
   entry->module=AcquireString("ICON");
   (void) RegisterMagickInfo(entry);
