@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,12 +53,20 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
 */
+#if defined(HasJBIG)
 static unsigned int
   WriteJBIGImage(const ImageInfo *,Image *);
+#endif
 
 #if defined(HasJBIG)
 #include "jbig.h"
@@ -145,9 +153,9 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Initialize JBIG toolkit.
   */
@@ -164,7 +172,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   */
   buffer=(unsigned char *) AcquireMemory(MaxBufferSize);
   if (buffer == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   status=JBG_EAGAIN;
   do
   {
@@ -188,8 +196,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   if (!AllocateImageColormap(image,2))
     {
       LiberateMemory((void **) &buffer);
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
     }
   image->colormap[0].red=0;
   image->colormap[0].green=0;
@@ -231,7 +238,8 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        break;
   }
   /*
     Free scale resource.
@@ -241,14 +249,6 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   CloseBlob(image);
   return(image);
 }
-#else
-static Image *ReadJBIGImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
-{
-  ThrowException(exception,MissingDelegateError,
-    "JBIG library is not available",image_info->filename);
-  return((Image *) NULL);
-}
 #endif
 
 /*
@@ -279,7 +279,7 @@ ModuleExport void RegisterJBIGImage(void)
 #define JBIGDescription  "Joint Bi-level Image experts Group interchange format"
 
   char
-	  version[MaxTextExtent];
+    version[MaxTextExtent];
 
   MagickInfo
     *entry;
@@ -289,8 +289,10 @@ ModuleExport void RegisterJBIGImage(void)
   (void) strncpy(version,JBG_VERSION,MaxTextExtent-1);
 #endif
   entry=SetMagickInfo("BIE");
-  entry->decoder=ReadJBIGImage;
-  entry->encoder=WriteJBIGImage;
+#if defined(HasJBIG)
+  entry->decoder=(DecoderHandler) ReadJBIGImage;
+  entry->encoder=(EncoderHandler) WriteJBIGImage;
+#endif
   entry->adjoin=False;
   entry->description=AcquireString(JBIGDescription);
   if (*version != '\0')
@@ -298,16 +300,20 @@ ModuleExport void RegisterJBIGImage(void)
   entry->module=AcquireString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBG");
-  entry->decoder=ReadJBIGImage;
-  entry->encoder=WriteJBIGImage;
+#if defined(HasJBIG)
+  entry->decoder=(DecoderHandler) ReadJBIGImage;
+  entry->encoder=(EncoderHandler) WriteJBIGImage;
+#endif
   entry->description=AcquireString(JBIGDescription);
   if (*version != '\0')
     entry->version=AcquireString(version);
   entry->module=AcquireString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBIG");
-  entry->decoder=ReadJBIGImage;
-  entry->encoder=WriteJBIGImage;
+#if defined(HasJBIG)
+  entry->decoder=(DecoderHandler) ReadJBIGImage;
+  entry->encoder=(EncoderHandler) WriteJBIGImage;
+#endif
   entry->description=AcquireString(JBIGDescription);
   if (*version != '\0')
     entry->version=AcquireString(version);
@@ -403,16 +409,16 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
 
   unsigned char
     bit,
+    byte,
     *pixels,
     polarity;
 
   unsigned int
-    byte,
-    scene,
     status;
 
   unsigned long
-    number_packets;
+    number_packets,
+    scene;
 
   /*
     Open image file.
@@ -421,30 +427,28 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   scene=0;
   do
   {
     /*
       Allocate pixel data.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     number_packets=((image->columns+7) >> 3)*image->rows;
     pixels=(unsigned char *) AcquireMemory(number_packets);
     if (pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     /*
       Convert pixels to a bitmap.
     */
-    if ((image->storage_class == DirectClass) ||
-        !IsMonochromeImage(image,&image->exception))
-      SetImageType(image,BilevelType);
-    polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+    SetImageType(image,BilevelType);
+    polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
     if (image->colors == 2)
-      polarity=Intensity(&image->colormap[0]) > Intensity(&image->colormap[1]);
+      polarity=PixelIntensityToQuantum(&image->colormap[0]) >
+        PixelIntensityToQuantum(&image->colormap[1]);
     q=pixels;
     for (y=0; y < (long) image->rows; y++)
     {
@@ -470,7 +474,8 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
       if (bit != 0)
         *q++=byte << (8-bit);
       if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+          break;
     }
     /*
       Initialize JBIG info structure.
@@ -492,8 +497,19 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
         y_resolution=480;
         sans_offset=0;
         if (image_info->density != (char *) NULL)
-          (void) GetGeometry(image_info->density,&sans_offset,&sans_offset,
-            &x_resolution,&y_resolution);
+          {
+            GeometryInfo
+              geometry_info;
+
+            unsigned int
+              flags;
+
+            flags=ParseGeometry(image_info->density,&geometry_info);
+            x_resolution=geometry_info.rho;
+            y_resolution=geometry_info.sigma;
+            if (!(flags & SigmaValue))
+              y_resolution=x_resolution;
+          }
         (void) jbg_enc_lrlmax(&jbig_info,x_resolution,y_resolution);
       }
     (void) jbg_enc_lrange(&jbig_info,-1,-1);
@@ -507,8 +523,11 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
     LiberateMemory((void **) &pixels);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
@@ -516,10 +535,4 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
   CloseBlob(image);
   return(True);
 }
-#else
-static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateError,"JBIG library is not available",
-    image->filename);
-}
 #endif
