@@ -49,6 +49,11 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "static.h"
+#include "utility.h"
 
 /* Auto coloring method, sorry this creates some artefact inside data
 MinReal+j*MaxComplex = red  MaxReal+j*MaxComplex = black
@@ -102,7 +107,7 @@ register IndexPacket *indexes;
                  break;
 /*           if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   ProgressMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
            }
            return;
      
@@ -113,9 +118,9 @@ register IndexPacket *indexes;
        break;
            for (x=0; x < (long) image->columns; x++)
               {
-              q->red=XDownscale(*(unsigned short *)p);
-              q->green=XDownscale(*(unsigned short *)p);
-              q->blue=XDownscale(*(unsigned short *)p);
+              q->red=ScaleShortToQuantum(*(unsigned short *)p);
+              q->green=ScaleShortToQuantum(*(unsigned short *)p);
+              q->blue=ScaleShortToQuantum(*(unsigned short *)p);
         p+=2;
               q++;
               }
@@ -123,7 +128,7 @@ register IndexPacket *indexes;
               break;
 /*          if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+              MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
           return;     
     } 
        }
@@ -144,9 +149,9 @@ register PixelPacket *q;
    for (x=0; x < (long) image->columns; x++)
           {
     f=(double)MaxRGB* (*p-Min)/(Max-Min);
-          q->red=XDownscale(f);
-    q->green=XDownscale(f);
-    q->blue=XDownscale(f);
+          q->red=ScaleShortToQuantum(f);
+    q->green=ScaleShortToQuantum(f);
+    q->blue=ScaleShortToQuantum(f);
           p++;
           q++;
           }
@@ -154,7 +159,7 @@ register PixelPacket *q;
               return;
 /*          if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+              MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
    return;     
 }
 
@@ -176,7 +181,7 @@ register PixelPacket *q;
     if(*p>0)
       {
       f=(*p/Max)*(MaxRGB-q->red);
-      if((int) f+q->red>MaxRGB) q->red=MaxRGB;
+      if(f+q->red>MaxRGB) q->red=MaxRGB;
                      else q->red+=(int) f;
             if((int) f/2.0>q->green) q->green=q->blue=0;
          else q->green=q->blue-=(int) (f/2.0);    
@@ -184,7 +189,7 @@ register PixelPacket *q;
     if(*p<0)
       {
       f=(*p/Max)*(MaxRGB-q->blue);
-      if((int) f+q->blue>MaxRGB) q->blue=MaxRGB;
+      if(f+q->blue>MaxRGB) q->blue=MaxRGB;
                       else q->blue+=(int) f;
             if((int) f/2.0>q->green) q->green=q->red=0;
         else q->green=q->red-=(int) (f/2.0);    
@@ -196,13 +201,13 @@ register PixelPacket *q;
               return;
 /*          if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+              MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
    return;     
 }
 
 
 /*This function reads one block of unsigned shortS*/
-void ReadBlobWordLSB(Image *I,size_t len,unsigned short *data)
+static void ReadBlobWordLSB(Image *I,size_t len,unsigned short *data)
 {
 while(len>=2)
    {
@@ -239,7 +244,7 @@ static double ReadBlobLSBdouble(Image *image)
 }
 
 
-void ReadBlobDoublesLSB(Image *I,size_t len,double *data)
+static void ReadBlobDoublesLSB(Image *I,size_t len,double *data)
 {
 while(len>=8)
    {
@@ -287,7 +292,8 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image *image,*rotated_image;
   unsigned int status;
   MATHeader MATLAB_HDR;
-  unsigned long size,filepos;
+  unsigned long size;
+  ExtendedSignedIntegralType filepos;
   unsigned long CellType;
   int i,x;
   long ldblk;
@@ -299,9 +305,9 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   image=AllocateImage(image_info);
   
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read MATLAB image.
   */
@@ -321,11 +327,12 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MATLAB_HDR.NameFlag=ReadBlobLSBShort(image);
 
   if (strncmp(MATLAB_HDR.identific,"MATLAB",6))
-MATLAB_KO:  ThrowReaderException(CorruptImageError,"Not a MATLAB image file!",image);
+MATLAB_KO:  ThrowReaderException(CorruptImageError,"NotAMATLABImageFile",image);
   if (strncmp(MATLAB_HDR.idx,"\1IM",3)) goto MATLAB_KO;
   if(MATLAB_HDR.unknown0!=0x0E) goto MATLAB_KO;
   if(MATLAB_HDR.DimFlag!=8) 
-            ThrowReaderException(CorruptImageError,"Multi-dimensional matrices are not supported!",image);
+    ThrowReaderException(CoderError,"MultidimensionalMatricesAreNotSupported",
+      image);
 
 /*printf("MATLAB_HDR.StructureFlag %ld\n",MATLAB_HDR.StructureFlag);*/
   if(MATLAB_HDR.StructureFlag!=6 && MATLAB_HDR.StructureFlag!=0x806) goto MATLAB_KO;
@@ -351,21 +358,23 @@ MATLAB_KO:  ThrowReaderException(CorruptImageError,"Not a MATLAB image file!",im
    switch(CellType)
       {
       case 2:image->depth=8;    /*Byte type cell*/
-             ldblk=MATLAB_HDR.SizeX;
+             ldblk=(long) MATLAB_HDR.SizeX;
         if(MATLAB_HDR.StructureFlag==0x806) goto MATLAB_KO;    
              break;
       case 4:image->depth=16;      /*Word type cell*/
-             ldblk=2*MATLAB_HDR.SizeX;
+             ldblk=(long) (2*MATLAB_HDR.SizeX);
        if(MATLAB_HDR.StructureFlag==0x806) goto MATLAB_KO;    
              break; 
       case 9:image->depth=24;      /*double type cell*/
-             if(sizeof(double)!=8) ThrowReaderException(CorruptImageError,"Incompatible size of double!",image);
+             if(sizeof(double)!=8) ThrowReaderException(CoderError,
+               "IncompatibleSizeOfDouble",image);
        if(MATLAB_HDR.StructureFlag==0x806) 
           {      /*complex double type cell*/
     }
-       ldblk=8*MATLAB_HDR.SizeX;
+       ldblk=(long) (8*MATLAB_HDR.SizeX);
              break; 
-      default:ThrowReaderException(CorruptImageError,"Unsupported cell type in the matrix!",image)
+      default:ThrowReaderException(CoderError,"UnsupportedCellTypeInTheMatrix",
+        image)
       }    
 
    image->columns= MATLAB_HDR.SizeX;
@@ -381,15 +390,15 @@ MATLAB_KO:  ThrowReaderException(CorruptImageError,"Not a MATLAB image file!",im
      image->colors=256;
      if (!AllocateImageColormap(image,image->colors))
            {
-NoMemory:  ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-                                image)
+NoMemory:  ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+             image)
            }           
    
      for (i=0; i < (long)image->colors; i++)
            {
-           image->colormap[i].red=Upscale(i);
-           image->colormap[i].green=Upscale(i);
-           image->colormap[i].blue=Upscale(i);
+           image->colormap[i].red=ScaleCharToQuantum(i);
+           image->colormap[i].green=ScaleCharToQuantum(i);
+           image->colormap[i].blue=ScaleCharToQuantum(i);
            }
      }      
 
@@ -509,7 +518,8 @@ ModuleExport void RegisterMATImage(void)
     *entry;
 
   entry=SetMagickInfo("MAT");
-  entry->decoder=ReadMATImage;
+  entry->decoder=(DecoderHandler) ReadMATImage;
+  entry->seekable_stream=True;
   entry->description=AcquireString("MATLAB image format");
   entry->module=AcquireString("MAT");
   (void) RegisterMagickInfo(entry);
