@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,7 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
-
+#include "blob.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
+
 /*
   Forward declarations.
 */
@@ -89,7 +94,7 @@ ModuleExport void RegisterMATTEImage(void)
     *entry;
 
   entry=SetMagickInfo("MATTE");
-  entry->encoder=WriteMATTEImage;
+  entry->encoder=(EncoderHandler) WriteMATTEImage;
   entry->raw=True;
   entry->description=AcquireString("MATTE format");
   entry->module=AcquireString("MATTE");
@@ -171,8 +176,7 @@ static unsigned int WriteMATTEImage(const ImageInfo *image_info,Image *image)
     status;
 
   if (!image->matte)
-    ThrowWriterException(ResourceLimitError,
-      "Image does not have a matte channel",image);
+    ThrowWriterException(CoderError,"ImageDoesNotHaveAMatteChannel",image);
   matte_image=
     CloneImage(image,image->columns,image->rows,True,&image->exception);
   if (matte_image == (Image *) NULL)
@@ -200,7 +204,8 @@ static unsigned int WriteMATTEImage(const ImageInfo *image_info,Image *image)
       break;
     if (image->previous == (Image *) NULL)
       if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+          break;
   }
   (void) FormatString(matte_image->filename,"MIFF:%.1024s",image->filename);
   status=WriteImage(image_info,matte_image);
