@@ -18,7 +18,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,6 +54,14 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "cache.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -140,26 +148,34 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     keyword[MaxTextExtent],
     *values;
 
+  ExtendedSignedIntegralType
+    offset;
+
+  GeometryInfo
+    geometry_info;
+
   Image
     *image;
 
   int
     c;
 
-  off_t
-    length,
-    offset;
-
   register long
     i;
 
   register unsigned char
     *p;
 
+  size_t
+    length;
+
   unsigned int
-    quantum_depth,
+    flags,
     status;
 
+  unsigned long
+    quantum_depth;
+
   /*
     Open image file.
   */
@@ -168,9 +184,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   (void) strncpy(cache_filename,image->filename,MaxTextExtent-1);
   AppendImageFormat("cache",cache_filename);
   c=ReadBlobByte(image);
@@ -212,7 +228,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             c=ReadBlobByte(image);
             if ((c == EOF) || (c == '}'))
               break;
-            if ((p-comment+1) >= (int) length)
+            if ((p-comment+1) >= (long) length)
               {
                 *p='\0';
                 length<<=1;
@@ -224,8 +240,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             *p=c;
           }
           if (comment == (char *) NULL)
-            ThrowReaderException(ResourceLimitError,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+              image);
           *p='\0';
           (void) SetImageAttribute(image,"comment",comment);
           LiberateMemory((void **) &comment);
@@ -261,7 +277,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 }
               if (values == (char *) NULL)
                 ThrowReaderException(ResourceLimitError,
-                  "Memory allocation failed",image);
+                  "MemoryAllocationFailed",image);
               *p++=c;
               c=ReadBlobByte(image);
               if (*values != '{')
@@ -285,9 +301,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"blue-primary") == 0)
                   {
-                    (void) sscanf(values,"%lf,%lf",
-                      &image->chromaticity.blue_primary.x,
-                      &image->chromaticity.blue_primary.y);
+                    flags=ParseGeometry(values,&geometry_info);
+                    image->chromaticity.blue_primary.x=geometry_info.rho;
+                    image->chromaticity.blue_primary.y=geometry_info.sigma;
+                    if (!(flags & SigmaValue))
+                      image->chromaticity.blue_primary.y=
+                        image->chromaticity.blue_primary.x;
                     break;
                   }
                 if (LocaleCompare(keyword,"border-color") == 0)
@@ -305,13 +324,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"class") == 0)
                   {
+                    image->storage_class=UndefinedClass;
                     if (LocaleCompare(values,"PseudoClass") == 0)
                       image->storage_class=PseudoClass;
                     else
                       if (LocaleCompare(values,"DirectClass") == 0)
                         image->storage_class=DirectClass;
-                      else
-                        image->storage_class=UndefinedClass;
                     break;
                   }
                 if (LocaleCompare(keyword,"colors") == 0)
@@ -321,6 +339,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"colorspace") == 0)
                   {
+                    image->colorspace=UndefinedColorspace;
                     if (LocaleCompare(values,"CMYK") == 0)
                       image->colorspace=CMYKColorspace;
                     else
@@ -349,7 +368,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     if (LocaleCompare("LZW",values) == 0)
                       compression=LZWCompression;
                     if (LocaleCompare("RLE",values) == 0)
-                      compression=RunlengthEncodedCompression;
+                      compression=RLECompression;
                     if (LocaleCompare("Zip",values) == 0)
                       compression=ZipCompression;
                     image->compression=compression;
@@ -374,12 +393,20 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"depth") == 0)
                   {
-                    image->depth=atol(values) <= 8 ? 8 : 16;
+                    image->depth=atol(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"dispose") == 0)
                   {
-                    image->dispose=atol(values);
+                    image->dispose=(DisposeType) atol(values);
+                    if (LocaleCompare(values,"Background") == 0)
+                      image->dispose=BackgroundDispose;
+                    else
+                      if (LocaleCompare(values,"None") == 0)
+                        image->dispose=NoneDispose;
+                      else
+                        if (LocaleCompare(values,"Previous") == 0)
+                          image->dispose=PreviousDispose;
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -391,7 +418,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"error") == 0)
                   {
-                    image->mean_error_per_pixel=atof(values);
+                    image->error.mean_error_per_pixel=atof(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -408,9 +435,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"green-primary") == 0)
                   {
-                    (void) sscanf(values,"%lf,%lf",
-                      &image->chromaticity.green_primary.x,
-                      &image->chromaticity.green_primary.y);
+                    flags=ParseGeometry(values,&geometry_info);
+                    image->chromaticity.green_primary.x=geometry_info.rho;
+                    image->chromaticity.green_primary.y=geometry_info.sigma;
+                    if (!(flags & SigmaValue))
+                      image->chromaticity.green_primary.y=
+                        image->chromaticity.green_primary.x;
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -451,12 +481,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"maximum-error") == 0)
                   {
-                    image->normalized_maximum_error=atof(values);
+                    image->error.normalized_maximum_error=atof(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"mean-error") == 0)
                   {
-                    image->normalized_mean_error=atof(values);
+                    image->error.normalized_mean_error=atof(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"montage") == 0)
@@ -477,8 +507,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       *geometry;
 
                     geometry=GetPageGeometry(values);
-                    (void) GetGeometry(geometry,&image->page.x,&image->page.y,
-                      &image->page.width,&image->page.height);
+                    (void) ParseAbsoluteGeometry(geometry,&image->page);
                     LiberateMemory((void **) &geometry);
                     break;
                   }
@@ -503,7 +532,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         (i+1)*sizeof(ProfileInfo));
                     if (image->generic_profile == (ProfileInfo *) NULL)
                       ThrowReaderException(ResourceLimitError,
-                        "Memory allocation failed",image);
+                        "MemoryAllocationFailed",image);
                     image->generic_profile[i].name=AllocateString(keyword+8);
                     image->generic_profile[i].length=atol(values);
                     image->generic_profile[i].info=(unsigned char *) NULL;
@@ -532,7 +561,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"quantum-depth") == 0)
                   {
-                    quantum_depth=atol(values) <= 8 ? 8 : 16;
+                    quantum_depth=atol(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -544,13 +573,15 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"red-primary") == 0)
                   {
-                    (void) sscanf(values,"%lf,%lf",
-                      &image->chromaticity.red_primary.x,
-                      &image->chromaticity.red_primary.y);
+                    flags=ParseGeometry(values,&geometry_info);
+                    image->chromaticity.red_primary.x=geometry_info.rho;
+                    if (flags & SigmaValue)
+                      image->chromaticity.red_primary.y=geometry_info.sigma;
                     break;
                   }
                 if (LocaleCompare(keyword,"rendering-intent") == 0)
                   {
+                    image->rendering_intent=UndefinedIntent;
                     if (LocaleCompare(values,"Saturation") == 0)
                       image->rendering_intent=SaturationIntent;
                     else
@@ -562,14 +593,15 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         else
                           if (LocaleCompare(values,"relative") == 0)
                             image->rendering_intent=RelativeIntent;
-                          else
-                            image->rendering_intent=UndefinedIntent;
                     break;
                   }
                 if (LocaleCompare(keyword,"resolution") == 0)
                   {
-                    (void) sscanf(values,"%lfx%lf",&image->x_resolution,
-                      &image->y_resolution);
+                    flags=ParseGeometry(values,&geometry_info);
+                    image->x_resolution=geometry_info.rho;
+                    image->y_resolution=geometry_info.sigma;
+                    if (!(flags & SigmaValue))
+                      image->y_resolution=image->x_resolution;
                     break;
                   }
                 if (LocaleCompare(keyword,"rows") == 0)
@@ -598,14 +630,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"units") == 0)
                   {
-                    if (LocaleCompare(values,"undefined") == 0)
-                      image->units=UndefinedResolution;
+                    image->units=UndefinedResolution;
+                    if (LocaleCompare(values,"pixels-per-inch") == 0)
+                      image->units=PixelsPerInchResolution;
                     else
-                      if (LocaleCompare(values,"pixels-per-inch") == 0)
-                        image->units=PixelsPerInchResolution;
-                      else
-                        if (LocaleCompare(values,"pixels-per-centimeter") == 0)
-                          image->units=PixelsPerCentimeterResolution;
+                      if (LocaleCompare(values,"pixels-per-centimeter") == 0)
+                        image->units=PixelsPerCentimeterResolution;
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -617,9 +647,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"white-point") == 0)
                   {
-                    (void) sscanf(values,"%lf,%lf",
-                      &image->chromaticity.white_point.x,
-                      &image->chromaticity.white_point.y);
+                    flags=ParseGeometry(values,&geometry_info);
+                    image->chromaticity.white_point.x=geometry_info.rho;
+                    image->chromaticity.white_point.y=geometry_info.sigma;
+                    if (!(flags & SigmaValue))
+                      image->chromaticity.white_point.y=
+                        image->chromaticity.white_point.x;
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -648,11 +681,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (image->storage_class == UndefinedClass) ||
         (image->compression == UndefinedCompression) || (image->columns == 0) ||
         (image->rows == 0))
-      ThrowReaderException(CorruptImageError,"Incorrect image header in file",
-        image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader",image);
     if (quantum_depth != QuantumDepth)
-      ThrowReaderException(CacheError,"Inconsistent peristent cache depth",
-        image);
+      ThrowReaderException(CacheError,"InconsistentPersistentCacheDepth",image);
     if (image->montage != (char *) NULL)
       {
         register char
@@ -663,8 +694,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         image->directory=AllocateString((char *) NULL);
         if (image->directory == (char *) NULL)
-          ThrowReaderException(CorruptImageError,"Unable to read image data",
-            image);
+          ThrowReaderException(CorruptImageError,"UnableToReadImageData",image);
         p=image->directory;
         do
         {
@@ -677,8 +707,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               ReacquireMemory((void **) &image->directory,
                 (strlen(image->directory)+MaxTextExtent+1));
               if (image->directory == (char *) NULL)
-                ThrowReaderException(CorruptImageError,
-                  "Unable to read image data",image);
+                ThrowReaderException(CorruptImageError,"UnableToReadImageData",
+                  image);
               p=image->directory+strlen(image->directory);
             }
           c=ReadBlobByte(image);
@@ -693,8 +723,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image->color_profile.info=(unsigned char *)
           AcquireMemory(image->color_profile.length);
         if (image->color_profile.info == (unsigned char *) NULL)
-          ThrowReaderException(CorruptImageError,
-            "Unable to read color profile",image);
+          ThrowReaderException(CorruptImageError,"UnableToReadColorProfile",
+            image);
         (void) ReadBlob(image,image->color_profile.length,
           image->color_profile.info);
       }
@@ -706,8 +736,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image->iptc_profile.info=(unsigned char *)
           AcquireMemory(image->iptc_profile.length);
         if (image->iptc_profile.info == (unsigned char *) NULL)
-          ThrowReaderException(CorruptImageError,
-            "Unable to read IPTC profile",image);
+          ThrowReaderException(CorruptImageError,"UnableToReadIPTCProfile",
+            image);
         (void) ReadBlob(image,image->iptc_profile.length,
           image->iptc_profile.info);
       }
@@ -723,8 +753,8 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->generic_profile[i].info=(unsigned char *)
             AcquireMemory(image->generic_profile[i].length);
           if (image->generic_profile[i].info == (unsigned char *) NULL)
-            ThrowReaderException(CorruptImageError,
-              "Unable to read generic profile",image);
+            ThrowReaderException(CorruptImageError,"UnableToReadGenericProfile",
+              image);
           (void) ReadBlob(image,image->generic_profile[i].length,
             image->generic_profile[i].info);
         }
@@ -735,14 +765,14 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Create image colormap.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         if (image->colors == 0)
           for (i=0; i < 256; i++)
           {
-            image->colormap[i].red=Upscale(i);
-            image->colormap[i].green=Upscale(i);
-            image->colormap[i].blue=Upscale(i);
+            image->colormap[i].red=ScaleCharToQuantum(i);
+            image->colormap[i].green=ScaleCharToQuantum(i);
+            image->colormap[i].blue=ScaleCharToQuantum(i);
             image->colors++;
           }
         else
@@ -759,16 +789,16 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             packet_size=image->depth > 8 ? 6 : 3;
             colormap=(unsigned char *) AcquireMemory(packet_size*image->colors);
             if (colormap == (unsigned char *) NULL)
-              ThrowReaderException(ResourceLimitError,
-                "Memory allocation failed",image);
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+                image);
             (void) ReadBlob(image,packet_size*image->colors,colormap);
             p=colormap;
             if (image->depth <= 8)
               for (i=0; i < (long) image->colors; i++)
               {
-                image->colormap[i].red=Upscale(*p++);
-                image->colormap[i].green=Upscale(*p++);
-                image->colormap[i].blue=Upscale(*p++);
+                image->colormap[i].red=ScaleCharToQuantum(*p++);
+                image->colormap[i].green=ScaleCharToQuantum(*p++);
+                image->colormap[i].blue=ScaleCharToQuantum(*p++);
               }
             else
               for (i=0; i < (long) image->colors; i++)
@@ -784,21 +814,23 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
       }
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
-    if (image_info->ping)
-      break;
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
+    if (image_info->ping && (image_info->subrange != 0))
+      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+        break;
     /*
       Attach persistent pixel cache.
     */
     status=PersistCache(image,cache_filename,True,&offset,exception);
     if (status == False)
-      ThrowReaderException(CacheError,"Unable to perist pixel cache",image);
+      ThrowReaderException(CacheError,"UnableToPeristPixelCache",image);
     /*
       Proceed to next image.
     */
-    if (image_info->subrange != 0)
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
-        break;
     do
     {
       c=ReadBlobByte(image);
@@ -814,8 +846,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (c != EOF);
   while (image->previous != (Image *) NULL)
@@ -852,10 +887,14 @@ ModuleExport void RegisterMPCImage(void)
   MagickInfo
     *entry;
 
+  entry=SetMagickInfo("CACHE");
+  entry->description=AcquireString("Magick Persistent Cache image format");
+  entry->module=AcquireString("CACHE");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPC");
-  entry->decoder=ReadMPCImage;
-  entry->encoder=WriteMPCImage;
-  entry->magick=IsMPC;
+  entry->decoder=(DecoderHandler) ReadMPCImage;
+  entry->encoder=(EncoderHandler) WriteMPCImage;
+  entry->magick=(MagickHandler) IsMPC;
   entry->description=AcquireString("Magick Persistent Cache image format");
   entry->module=AcquireString("MPC");
   (void) RegisterMagickInfo(entry);
@@ -882,6 +921,7 @@ ModuleExport void RegisterMPCImage(void)
 */
 ModuleExport void UnregisterMPCImage(void)
 {
+  (void) UnregisterMagickInfo("CACHE");
   (void) UnregisterMagickInfo("MPC");
 }
 
@@ -923,7 +963,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  off_t
+  ExtendedSignedIntegralType
     offset;
 
   register long
@@ -942,9 +982,9 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   (void) strncpy(cache_filename,image->filename,MaxTextExtent-1);
   AppendImageFormat("cache",cache_filename);
   scene=0;
@@ -1006,7 +1046,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,"compression=LZW\n");
         break;
       }
-      case RunlengthEncodedCompression:
+      case RLECompression:
       {
         (void) WriteBlobString(image,"compression=RLE\n");
         break;
@@ -1046,12 +1086,11 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
     if ((image->next != (Image *) NULL) || (image->previous != (Image *) NULL))
       {
         if (image->scene == 0)
-          FormatString(buffer,"iterations=%lu  delay=%lu  Dispose=%lu\n",
-            image->iterations,image->delay,image->dispose);
-				else
-          FormatString(buffer,
-            "scene=%lu  iterations=%lu  delay=%lu  Dispose=%lu\n",
-            image->scene,image->iterations,image->delay,image->dispose);
+          FormatString(buffer,"iterations=%lu  delay=%lu\n",image->iterations,
+            image->delay);
+        else
+          FormatString(buffer,"scene=%lu  iterations=%lu  delay=%lu\n",
+            image->scene,image->iterations,image->delay);
         (void) WriteBlobString(image,buffer);
       }
     else
@@ -1061,7 +1100,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
             FormatString(buffer,"scene=%lu\n",image->scene);
             (void) WriteBlobString(image,buffer);
           }
-        if (image->iterations != 1)
+        if (image->iterations != 0)
           {
             FormatString(buffer,"iterations=%lu\n",image->iterations);
             (void) WriteBlobString(image,buffer);
@@ -1071,17 +1110,23 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
             FormatString(buffer,"delay=%lu\n",image->delay);
             (void) WriteBlobString(image,buffer);
           }
-        if (image->dispose != 0)
-          {
-            FormatString(buffer,"dispose=%lu\n",image->dispose);
-            (void) WriteBlobString(image,buffer);
-          }
       }
-    if (image->mean_error_per_pixel != 0.0)
+    if (image->dispose != UndefinedDispose)
+      {
+        if (image->dispose == BackgroundDispose)
+          (void) strcpy(buffer,"dispose=background\n");
+        else
+          if (image->dispose == NoneDispose)
+            (void) strcpy(buffer,"dispose=none\n");
+          else
+            (void) strcpy(buffer,"dispose=previous\n");
+        (void) WriteBlobString(image,buffer);
+      }
+    if (image->error.mean_error_per_pixel != 0.0)
       {
         FormatString(buffer,"error=%g  mean-error=%g  maximum-error=%g\n",
-          image->mean_error_per_pixel,image->normalized_mean_error,
-          image->normalized_maximum_error);
+          image->error.mean_error_per_pixel,image->error.normalized_mean_error,
+          image->error.normalized_maximum_error);
         (void) WriteBlobString(image,buffer);
       }
     if (image->rendering_intent != UndefinedIntent)
@@ -1247,11 +1292,14 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
     */
     status=PersistCache(image,cache_filename,False,&offset,&image->exception);
     if (status == False)
-      ThrowWriterException(CacheError,"Unable to perist pixel cache",image);
+      ThrowWriterException(CacheError,"UnableToPeristPixelCache",image);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
