@@ -18,7 +18,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -52,6 +52,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "delegate.h"
+#include "list.h"
+#include "log.h"
+#include "magick.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -158,9 +165,9 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   CloseBlob(image);
   /*
     Convert MPEG to PPM with delegate.
@@ -243,23 +250,26 @@ ModuleExport void RegisterMPEGImage(void)
     *entry;
 
   entry=SetMagickInfo("MPEG");
-  entry->decoder=ReadMPEGImage;
-  entry->encoder=WriteMPEGImage;
-  entry->magick=IsMPEG;
+  entry->decoder=(DecoderHandler) ReadMPEGImage;
+  entry->encoder=(EncoderHandler) WriteMPEGImage;
+  entry->magick=(MagickHandler) IsMPEG;
+  entry->blob_support=False;
   entry->description=AcquireString("MPEG Video Stream");
   entry->module=AcquireString("MPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPG");
-  entry->decoder=ReadMPEGImage;
-  entry->encoder=WriteMPEGImage;
-  entry->magick=IsMPEG;
+  entry->decoder=(DecoderHandler) ReadMPEGImage;
+  entry->encoder=(EncoderHandler) WriteMPEGImage;
+  entry->magick=(MagickHandler) IsMPEG;
+  entry->blob_support=False;
   entry->description=AcquireString("MPEG Video Stream");
   entry->module=AcquireString("MPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M2V");
-  entry->decoder=ReadMPEGImage;
-  entry->encoder=WriteMPEGImage;
-  entry->magick=IsMPEG;
+  entry->decoder=(DecoderHandler) ReadMPEGImage;
+  entry->encoder=(EncoderHandler) WriteMPEGImage;
+  entry->magick=(MagickHandler) IsMPEG;
+  entry->blob_support=False;
   entry->description=AcquireString("MPEG Video Stream");
   entry->module=AcquireString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -341,7 +351,7 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
   register Image
     *p;
 
-  register int
+  register long
     i;
 
   static int
@@ -462,14 +472,19 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
   vertical_factor=2;
   if (image_info->sampling_factor != (char *) NULL)
     {
+      GeometryInfo
+        geometry_info;
+
       long
-        count,
         horizontal_factor;
 
-      horizontal_factor=2;
-      count=sscanf(image_info->sampling_factor,"%ldx%ld",&horizontal_factor,
-        &vertical_factor);
-      if (count != 2)
+      unsigned int
+        flags;
+
+      flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
+      horizontal_factor=(long) geometry_info.rho;
+      vertical_factor=(long) geometry_info.sigma;
+      if (!(flags & SigmaValue))
         vertical_factor=horizontal_factor;
       if (mpeg)
         {
@@ -525,10 +540,8 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
 {
   char
     basename[MaxTextExtent],
-    *blob,
     filename[MaxTextExtent];
 
-
   Image
     *coalesce_image,
     *next_image;
@@ -536,9 +549,6 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   ImageInfo
     *clone_info;
 
-  int
-    file;
-
   register Image
     *p;
 
@@ -548,7 +558,11 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   size_t
     length;
 
+  unsigned char
+    *blob;
+
   unsigned int
+    logging,
     status;
 
   unsigned long
@@ -562,9 +576,10 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  logging=LogMagickEvent(CoderEvent,GetMagickModule(),"enter");
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   CloseBlob(image);
   /*
     Determine if the sequence of images have identical page info.
@@ -606,44 +621,65 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
           FormatString(filename,"%.1024s.niq",basename);
           (void) remove(filename);
         }
-      ThrowWriterException(DelegateError,"Unable to write MPEG parameters",
-        image)
+      ThrowWriterException(CoderError,"UnableToWriteMPEGParameters",image)
     }
   count=0;
   clone_info->interlace=PlaneInterlace;
   for (p=coalesce_image; p != (Image *) NULL; p=p->next)
   {
-    blob=(char *) NULL;
+    char
+      previous_image[MaxTextExtent];
+
+    blob=(unsigned char *) NULL;
     length=0;
     scene=p->scene;
     for (i=0; i < (long) Max((p->delay+1)/3,1); i++)
     {
-      p->scene=count++;
-      FormatString(p->filename,"%.1024s.%%d.yuv",basename);
-      switch ((int) i)
+      p->scene=count;
+      count++;
+      status=False;
+      switch (i)
       {
         case 0:
         {
-          status=WriteImage(clone_info,p);
+          Image
+            *frame;
+
+          FormatString(p->filename,"%.1024s.%lu.yuv",basename,p->scene);
           FormatString(filename,"%.1024s.%lu.yuv",basename,p->scene);
+          FormatString(previous_image,"%.1024s.%lu.yuv",basename,p->scene);
+          frame=CloneImage(p,0,0,True,&p->exception);
+          if (frame == (Image *) NULL)
+            break;
+          status=WriteImage(clone_info,frame);
+          DestroyImage(frame);
           break;
         }
         case 1:
-          blob=(char *) FileToBlob(filename,&length,&image->exception);
+          blob=(unsigned char *)
+            FileToBlob(previous_image,&length,&image->exception);
         default:
         {
           FormatString(filename,"%.1024s.%lu.yuv",basename,p->scene);
-          file=open(filename,O_WRONLY | O_CREAT | O_BINARY,0777);
-          if (file == -1)
-            break;
-          (void) write(file,blob,length);
-          (void) close(file);
+          if (length > 0)
+            status=BlobToFile(filename,blob,length,&image->exception);
           break;
         }
       }
+      if (logging)
+        {
+          if (status)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+              "%lu. Wrote YUV file for scene %lu:",i,p->scene);
+          else
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+              "%lu. Failed to write YUV file for scene %lu:",i,p->scene);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%.1024s",
+            filename);
+        }
     }
     p->scene=scene;
-    if (blob != (char *) NULL)
+    if (blob != (unsigned char *) NULL)
       LiberateMemory((void **) &blob);
     if (status == False)
       break;
@@ -677,5 +713,7 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   (void) remove(filename);
   if (coalesce_image != image)
     DestroyImage(coalesce_image);
+  if (logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"exit");
   return(status);
 }
