@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,11 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "magick.h"
+#include "registry.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -146,17 +151,19 @@ ModuleExport void RegisterMPRImage(void)
     *entry;
 
   entry=SetMagickInfo("MPR");
-  entry->decoder=ReadMPRImage;
-  entry->encoder=WriteMPRImage;
+  entry->decoder=(DecoderHandler) ReadMPRImage;
+  entry->encoder=(EncoderHandler) WriteMPRImage;
   entry->adjoin=False;
+  entry->stealth=True;
   entry->description=AcquireString("Magick Persistent Registry");
   entry->module=AcquireString("MPR");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPRI");
-  entry->decoder=ReadMPRImage;
-  entry->encoder=WriteMPRImage;
+  entry->decoder=(DecoderHandler) ReadMPRImage;
+  entry->encoder=(EncoderHandler) WriteMPRImage;
   entry->stealth=True;
   entry->adjoin=False;
+  entry->stealth=True;
   entry->description=AcquireString("Magick Persistent Registry");
   entry->module=AcquireString("MPRI");
   (void) RegisterMagickInfo(entry);
@@ -198,7 +205,7 @@ ModuleExport void UnregisterMPRImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadMPRImage writes an image into the Magick Persistent Registry
+%  WriteMPRImage() writes an image into the Magick Persistent Registry
 %  image as a blob from memory.  It allocates the memory necessary for the
 %  new Image structure and returns a pointer to the new image.
 %
