@@ -17,7 +17,7 @@
 %                                 April 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -52,6 +52,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "render.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -90,9 +97,9 @@ static unsigned int
 */
 static unsigned int IsMVG(const unsigned char *magick,const size_t length)
 {
-  if (length < 7)
+  if (length < 20)
     return(False);
-  if (LocaleNCompare((char *) magick,"viewbox",7) == 0)
+  if (LocaleNCompare((char *) magick,"push graphic-context",20) == 0)
     return(True);
   return(False);
 }
@@ -131,10 +138,7 @@ static unsigned int IsMVG(const unsigned char *magick,const size_t length)
 */
 static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-#define BoundingBox  "ViewBox"
-
-  char
-    filename[MaxTextExtent];
+#define BoundingBox  "viewbox"
 
   DrawInfo
     *draw_info;
@@ -142,15 +146,23 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
+  size_t
+    length;
+
+  unsigned int
+    status;
+
   /*
-    Initialize Image structure.
+    Open image.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == False)
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   if ((image->columns == 0) || (image->rows == 0))
     {
       char
@@ -162,15 +174,9 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       SegmentInfo
         bounds;
 
-      unsigned int
-        status;
- 
       /*
         Determine size of image canvas.
       */
-      status=OpenBlob(image_info,image,ReadBinaryType,exception);
-      if (status == False)
-        ThrowReaderException(FileOpenError,"Unable to open file",image);
       while (ReadBlobString(image,primitive) != (char *) NULL)
       {
         for (p=primitive; (*p == ' ') || (*p == '\t'); p++);
@@ -184,14 +190,22 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     }
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"Must specify image size",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+  /*
+    Render drawing.
+  */
   SetImage(image,OpaqueOpacity);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  (void) strcpy(filename,"@");
-  (void) strcat(filename,image_info->filename);
-  (void) CloneString(&draw_info->primitive,filename);
+  draw_info->fill=image_info->pen;
+  if (GetBlobStreamType(image) == BlobStream)
+    draw_info->primitive=AllocateString((char *) GetBlobStreamData(image));
+  else
+    draw_info->primitive=(char *) FileToBlob(image->filename,&length,exception);
+  if (draw_info->primitive == (char *) NULL)
+    return((Image *) NULL);
   (void) DrawImage(image,draw_info);
   DestroyDrawInfo(draw_info);
+  CloseBlob(image);
   return(image);
 }
 
@@ -224,10 +238,11 @@ ModuleExport void RegisterMVGImage(void)
     *entry;
 
   entry=SetMagickInfo("MVG");
-  entry->decoder=ReadMVGImage;
-  entry->encoder=WriteMVGImage;
-  entry->magick=IsMVG;
+  entry->decoder=(DecoderHandler) ReadMVGImage;
+  entry->encoder=(EncoderHandler) WriteMVGImage;
+  entry->magick=(MagickHandler) IsMVG;
   entry->adjoin=False;
+  entry->seekable_stream=True;
   entry->description=AcquireString("Magick Vector Graphics");
   entry->module=AcquireString("MVG");
   (void) RegisterMagickInfo(entry);
@@ -303,10 +318,10 @@ static unsigned int WriteMVGImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickSignature);
   attribute=GetImageAttribute(image,"[MVG]");
   if (attribute == (ImageAttribute *) NULL)
-    ThrowWriterException(DelegateError,"no image vector graphics",image);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+    ThrowWriterException(OptionError,"NoImageVectorGraphics",image);
+  status=OpenBlob(image_info,image,WriteBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   (void) WriteBlob(image,strlen(attribute->value),attribute->value);
   CloseBlob(image);
   return(True);
