@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,7 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
-
+#include "blob.h"
+#include "gem.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
+
 /*
   Forward declarations.
 */
@@ -102,7 +109,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   unsigned char *chroma1,unsigned char *chroma2)
 {
 #define IsSync  ((sum & 0xffffff00) == 0xfffffe00)
-#define DecodeImageText  "  PCD decode image...  "
+#define DecodeImageTag  "Decode/Image"
 #define PCDGetBits(n) \
 {  \
   sum=(sum << n) & 0xffffffff; \
@@ -142,7 +149,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   PCDTable
     *pcd_table[3];
 
-  register int
+  register long
     i,
     j;
 
@@ -159,15 +166,12 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   unsigned char
     *buffer;
 
-  unsigned int
+  unsigned long
     bits,
     plane,
-    sum;
-
-  unsigned long
-    number_pixels,
     pcd_length[3],
-    row;
+    row,
+    sum;
 
   /*
     Initialize Huffman tables.
@@ -178,7 +182,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   assert(chroma2 != (unsigned char *) NULL);
   buffer=(unsigned char *) AcquireMemory(0x800);
   if (buffer == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       (char *) NULL);
   sum=0;
   bits=32;
@@ -191,7 +195,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
     if (pcd_table[i] == (PCDTable *) NULL)
       {
         LiberateMemory((void **) &buffer);
-        ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           (char *) NULL)
       }
     r=pcd_table[i];
@@ -207,7 +211,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
       PCDGetBits(16);
       r->sequence=(sum & 0xffff) << 16;
       PCDGetBits(8);
-      r->key=sum & 0xff;
+      r->key=(unsigned char) (sum & 0xff);
       r->mask=(~((((unsigned int) 1) << (32-r->length))-1));
       r++;
     }
@@ -216,8 +220,10 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   /*
     Search for Sync byte.
   */
-  do { PCDGetBits(16) } while (0);
-  do { PCDGetBits(16) } while (0);
+  for (i=0; i < 1; i++)
+    PCDGetBits(16);
+  for (i=0; i < 1; i++)
+    PCDGetBits(16);
   while ((sum & 0x00fff000) != 0x00fff000)
     PCDGetBits(8);
   while (!IsSync)
@@ -227,8 +233,8 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   */
   count=0;
   length=0;
-  number_pixels=image->columns*image->rows;
   plane=0;
+  row=0;
   q=luma;
   for ( ; ; )
   {
@@ -268,13 +274,14 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
           }
           default:
           {
-            ThrowBinaryException(CorruptImageError,"Corrupt PCD image",
+            ThrowBinaryException(CorruptImageError,"CorruptPCDImage",
               image->filename)
           }
         }
         length=pcd_length[plane];
         if (QuantumTick(row,image->rows))
-          MagickMonitor(DecodeImageText,row,image->rows);
+          if (!MagickMonitor(DecodeImageTag,row,image->rows,&image->exception))
+            break;
         continue;
       }
     /*
@@ -283,10 +290,10 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
     r=pcd_table[plane];
     for (i=0; ((i < (long) length) && ((sum & r->mask) != r->sequence)); i++)
       r++;
-    if (((unsigned long) (q-luma) > number_pixels) || (r == (PCDTable *) NULL))
+    if ((row > image->rows) || (r == (PCDTable *) NULL))
       {
-        ThrowException(&image->exception,CorruptImageError,
-          "Corrupt PCD image, skipping to sync byte",image->filename);
+        ThrowException(&image->exception,CorruptImageWarning,"SkipToSyncByte",
+          image->filename);
         while ((sum & 0x00fff000) != 0x00fff000)
           PCDGetBits(8);
         while (!IsSync)
@@ -342,9 +349,9 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
 */
 static unsigned int IsPCD(const unsigned char *magick,const size_t length)
 {
-  if (length < 4)
+  if (length < 2052)
     return(False);
-  if (LocaleNCompare((char *) magick,"PCD_",4) == 0)
+  if (LocaleNCompare((char *) magick+2048,"PCD_",4) == 0)
     return(True);
   return(False);
 }
@@ -414,7 +421,7 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   montage_image=MontageImages(image,montage_info,exception);
   DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   DestroyImage(image);
   return(montage_image);
 }
@@ -427,18 +434,17 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     x;
 
-  off_t
+  ExtendedSignedIntegralType
+    number_pixels,
     offset;
 
   register long
+    i,
     y;
 
   register PixelPacket
     *q;
 
-  register long
-    i;
-
   register unsigned char
     *c1,
     *c2,
@@ -461,7 +467,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned long
     height,
     number_images,
-    number_pixels,
     subimage,
     width;
 
@@ -473,28 +478,27 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Determine if this is a PCD file.
   */
   header=(unsigned char *) AcquireMemory(3*0x800);
   if (header == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   count=ReadBlob(image,3*0x800,(char *) header);
   overview=LocaleNCompare((char *) header,"PCD_OPA",7) == 0;
   if ((count == 0) ||
       ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && !overview))
-    ThrowReaderException(CorruptImageError,"Not a PCD image file",image);
+    ThrowReaderException(CorruptImageError,"NotAPCDImageFile",image);
   rotate=header[0x0e02] & 0x03;
   number_images=(header[10] << 8) | header[11];
   LiberateMemory((void **) &header);
   /*
     Determine resolution by subimage specification.
   */
-  number_pixels=image->columns*image->rows;
-  if (number_pixels == 0)
+  if ((image->columns == 0) || (image->rows == 0))
     subimage=3;
   else
     {
@@ -533,13 +537,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate luma and chroma memory.
   */
-  number_pixels=image->columns*image->rows;
-  chroma1=(unsigned char *) AcquireMemory(number_pixels+1);
-  chroma2=(unsigned char *) AcquireMemory(number_pixels+1);
-  luma=(unsigned char *) AcquireMemory(number_pixels+1);
+  number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+  if (number_pixels != (size_t) number_pixels)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+  chroma1=(unsigned char *) AcquireMemory(image->columns*image->rows+1);
+  chroma2=(unsigned char *) AcquireMemory(image->columns*image->rows+1);
+  luma=(unsigned char *) AcquireMemory(image->columns*image->rows+1);
   if ((chroma1 == (unsigned char *) NULL) ||
       (chroma2 == (unsigned char *) NULL) || (luma == (unsigned char *) NULL))
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Advance to image data.
   */
@@ -606,9 +612,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->red=Upscale(*yy++);
-            q->green=Upscale(*c1++);
-            q->blue=Upscale(*c2++);
+            q->red=ScaleCharToQuantum(*yy++);
+            q->green=ScaleCharToQuantum(*c1++);
+            q->blue=ScaleCharToQuantum(*c2++);
             q++;
           }
           if (!SyncImagePixels(image))
@@ -629,10 +635,11 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 DestroyImageList(image);
                 return((Image *) NULL);
               }
-            image=image->next;
+            image=SyncNextImageInList(image);
           }
         (void) SetMonitorHandler(handler);
-        MagickMonitor(LoadImageText,j-1,number_images);
+        if (!MagickMonitor(LoadImageTag,j-1,number_images,exception))
+          break;
       }
       LiberateMemory((void **) &chroma2);
       LiberateMemory((void **) &chroma1);
@@ -710,61 +717,60 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      q->red=Upscale(*yy++);
-      q->green=Upscale(*c1++);
-      q->blue=Upscale(*c2++);
+      q->red=ScaleCharToQuantum(*yy++);
+      q->green=ScaleCharToQuantum(*c1++);
+      q->blue=ScaleCharToQuantum(*c2++);
       q++;
     }
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        break;
   }
   LiberateMemory((void **) &chroma2);
   LiberateMemory((void **) &chroma1);
   LiberateMemory((void **) &luma);
-  if (LocaleCompare(image_info->magick,"PCDS") == 0)
-    (void) TransformRGBImage(image,sRGBColorspace);
-  else
-    (void) TransformRGBImage(image,YCCColorspace);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
-  if ((rotate == 1) || (rotate == 3))
-    {
-      double
-        degrees;
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
+  CloseBlob(image);
+  if (!image_info->ping)
+    if ((rotate == 1) || (rotate == 3))
+      {
+        double
+          degrees;
+  
+        Image
+          *rotate_image;
 
-      Image
-        *rotated_image;
-
-      /*
-        Rotate image.
-      */
-      degrees=rotate == 1 ? -90.0 : 90.0;
-      rotated_image=RotateImage(image,degrees,exception);
-      if (rotated_image != (Image *) NULL)
-        {
-          DestroyImage(image);
-          image=rotated_image;
-        }
-    }
+        /*
+          Rotate image.
+        */
+        degrees=rotate == 1 ? -90.0 : 90.0;
+        rotate_image=RotateImage(image,degrees,exception);
+        if (rotate_image != (Image *) NULL)
+          {
+            DestroyImage(image);
+            image=rotate_image;
+          }
+      }
   /*
     Set CCIR 709 primaries with a D65 white point.
   */
   image->chromaticity.red_primary.x=0.6400f;
   image->chromaticity.red_primary.y=0.3300f;
-  image->chromaticity.red_primary.z=1.0000f;
   image->chromaticity.green_primary.x=0.3000f;
   image->chromaticity.green_primary.y=0.6000f;
-  image->chromaticity.green_primary.z=1.0000f;
   image->chromaticity.blue_primary.x=0.1500f;
   image->chromaticity.blue_primary.y=0.0600f;
-  image->chromaticity.blue_primary.z=1.0000f;
   image->chromaticity.white_point.x=0.3127f;
   image->chromaticity.white_point.y=0.3290f;
-  image->chromaticity.white_point.z=1.0000f;
-  image->gamma=0.5;
-  CloseBlob(image);
+  image->gamma=1.000f/2.200f;
+  if (LocaleCompare(image_info->magick,"PCDS") == 0)
+    (void) TransformRGBImage(image,sRGBColorspace);
+  else
+    (void) TransformRGBImage(image,YCCColorspace);
   return(image);
 }
 
@@ -797,16 +803,16 @@ ModuleExport void RegisterPCDImage(void)
     *entry;
 
   entry=SetMagickInfo("PCD");
-  entry->decoder=ReadPCDImage;
-  entry->encoder=WritePCDImage;
-  entry->magick=IsPCD;
+  entry->decoder=(DecoderHandler) ReadPCDImage;
+  entry->encoder=(EncoderHandler) WritePCDImage;
+  entry->magick=(MagickHandler) IsPCD;
   entry->adjoin=False;
   entry->description=AcquireString("Photo CD");
   entry->module=AcquireString("PCD");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCDS");
-  entry->decoder=ReadPCDImage;
-  entry->encoder=WritePCDImage;
+  entry->decoder=(DecoderHandler) ReadPCDImage;
+  entry->encoder=(EncoderHandler) WritePCDImage;
   entry->adjoin=False;
   entry->description=AcquireString("Photo CD");
   entry->module=AcquireString("PCD");
@@ -872,6 +878,9 @@ ModuleExport void UnregisterPCDImage(void)
 static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
   char *page_geometry,char *tile_geometry)
 {
+  GeometryInfo
+    geometry_info;
+
   Image
     *downsample_image,
     *tile_image;
@@ -886,15 +895,18 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
     *p,
     *q;
 
-  register int
+  register long
     i,
     x;
 
+  unsigned int
+    flags;
+
   /*
     Scale image to tile size.
   */
   SetGeometry(image,&geometry);
-  (void) GetMagickGeometry(page_geometry,&geometry.x,&geometry.y,
+  (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
     &geometry.width,&geometry.height);
   if ((geometry.width % 2) != 0)
     geometry.width--;
@@ -904,7 +916,11 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
     1.0,&image->exception);
   if (tile_image == (Image *) NULL)
     return(False);
-  (void) sscanf(page_geometry,"%lux%lu",&geometry.width,&geometry.height);
+  flags=ParseGeometry(page_geometry,&geometry_info);
+  geometry.width=(unsigned long) geometry_info.rho;
+  geometry.height=(unsigned long) geometry_info.sigma;
+  if (!(flags & SigmaValue))
+    geometry.height=geometry.width;
   if ((tile_image->columns != geometry.width) ||
       (tile_image->rows != geometry.height))
     {
@@ -942,7 +958,7 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
       break;
     for (x=0; x < (long) (tile_image->columns << 1); x++)
     {
-      (void) WriteBlobByte(image,Downscale(p->red));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
       p++;
     }
     q=AcquireImagePixels(downsample_image,0,y >> 1,downsample_image->columns,
@@ -951,7 +967,7 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
       break;
     for (x=0; x < (long) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,Downscale(q->green));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(q->green));
       q++;
     }
     q=AcquireImagePixels(downsample_image,0,y >> 1,downsample_image->columns,
@@ -960,11 +976,12 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
       break;
     for (x=0; x < (long) downsample_image->columns; x++)
     {
-      (void) WriteBlobByte(image,Downscale(q->blue));
+      (void) WriteBlobByte(image,ScaleQuantumToChar(q->blue));
       q++;
     }
     if (QuantumTick(y,tile_image->rows))
-      MagickMonitor(SaveImageText,y,tile_image->rows);
+      if (!MagickMonitor(SaveImageTag,y,tile_image->rows,&image->exception))
+        break;
   }
   for (i=0; i < 0x800; i++)
     (void) WriteBlobByte(image,'\0');
@@ -978,7 +995,7 @@ static unsigned int WritePCDImage(const ImageInfo *image_info,Image *image)
   Image
     *pcd_image;
 
-  register int
+  register long
     i;
 
   unsigned int
@@ -992,22 +1009,22 @@ static unsigned int WritePCDImage(const ImageInfo *image_info,Image *image)
   if (image->columns < image->rows)
     {
       Image
-        *rotated_image;
+        *rotate_image;
 
       /*
         Rotate portrait to landscape.
       */
-      rotated_image=RotateImage(image,90.0,&image->exception);
-      if (rotated_image == (Image *) NULL)
+      rotate_image=RotateImage(image,90.0,&image->exception);
+      if (rotate_image == (Image *) NULL)
         return(False);
-      pcd_image=rotated_image;
+      pcd_image=rotate_image;
     }
   /*
     Open output image file.
   */
-  status=OpenBlob(image_info,pcd_image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,pcd_image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",pcd_image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",pcd_image);
   (void) TransformRGBImage(pcd_image,RGBColorspace);
   /*
     Write PCD image header.
@@ -1040,7 +1057,7 @@ static unsigned int WritePCDImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(pcd_image,'\1');
   else
     (void) WriteBlobByte(pcd_image,'\0');
-  for (i=0; i < 3*0x800-1539; i++)
+  for (i=0; i < (3*0x800-1539); i++)
     (void) WriteBlobByte(pcd_image,'\0');
   /*
     Write PCD tiles.
