@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Typedef declarations.
@@ -205,6 +211,9 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
+  ExtendedSignedIntegralType
+    *page_table;
+
   int
     bits,
     id,
@@ -244,9 +253,6 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned int
     status;
 
-  off_t
-    *page_table;
-
   unsigned long
     pcx_packets;
 
@@ -258,13 +264,13 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Determine if this is a PCX file.
   */
-  page_table=(off_t *) NULL;
+  page_table=(ExtendedSignedIntegralType *) NULL;
   if (LocaleCompare(image_info->magick,"DCX") == 0)
     {
       unsigned long
@@ -275,20 +281,20 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       magic=ReadBlobLSBLong(image);
       if (magic != 987654321)
-        ThrowReaderException(CorruptImageError,"Not a DCX image file",image);
-      page_table=(off_t *) AcquireMemory(1024*sizeof(off_t));
-      if (page_table == (off_t *) NULL)
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(CorruptImageError,"NotADCXImageFile",image);
+      page_table=(ExtendedSignedIntegralType *)
+         AcquireMemory(1024*sizeof(ExtendedSignedIntegralType));
+      if (page_table == (ExtendedSignedIntegralType *) NULL)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       for (id=0; id < 1024; id++)
       {
-        page_table[id]=(off_t) ReadBlobLSBLong(image);
+        page_table[id]=(ExtendedSignedIntegralType) ReadBlobLSBLong(image);
         if (page_table[id] == 0)
           break;
       }
     }
-  if (page_table != (off_t *) NULL)
-    (void) SeekBlob(image,(off_t) page_table[0],SEEK_SET);
+  if (page_table != (ExtendedSignedIntegralType *) NULL)
+    (void) SeekBlob(image,(ExtendedSignedIntegralType) page_table[0],SEEK_SET);
   count=ReadBlob(image,1,(char *) &pcx_info.identifier);
   for (id=1; id < 1024; id++)
   {
@@ -297,7 +303,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     pcx_info.version=ReadBlobByte(image);
     if ((count == 0) || (pcx_info.identifier != 0x0a))
-      ThrowReaderException(CorruptImageError,"Not a PCX image file",image);
+      ThrowReaderException(CorruptImageError,"NotAPCXImageFile",image);
     pcx_info.encoding=ReadBlobByte(image);
     pcx_info.bits_per_pixel=ReadBlobByte(image);
     pcx_info.left=ReadBlobLSBShort(image);
@@ -318,8 +324,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->colors=16;
     pcx_colormap=(unsigned char *) AcquireMemory(3*256);
     if (pcx_colormap == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     (void) ReadBlob(image,3*image->colors,(char *) pcx_colormap);
     pcx_info.reserved=ReadBlobByte(image);
     pcx_info.planes=ReadBlobByte(image);
@@ -328,16 +333,15 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           ((pcx_info.bits_per_pixel*pcx_info.planes) == 1))
         image->colors=1 << (pcx_info.bits_per_pixel*pcx_info.planes);
     if (!AllocateImageColormap(image,image->colors))
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     if ((pcx_info.bits_per_pixel >= 8) && (pcx_info.planes != 1))
       image->storage_class=DirectClass;
     p=pcx_colormap;
     for (i=0; i < (long) image->colors; i++)
     {
-      image->colormap[i].red=Upscale(*p++);
-      image->colormap[i].green=Upscale(*p++);
-      image->colormap[i].blue=Upscale(*p++);
+      image->colormap[i].red=ScaleCharToQuantum(*p++);
+      image->colormap[i].green=ScaleCharToQuantum(*p++);
+      image->colormap[i].blue=ScaleCharToQuantum(*p++);
     }
     pcx_info.bytes_per_line=ReadBlobLSBShort(image);
     pcx_info.palette_info=ReadBlobLSBShort(image);
@@ -354,8 +358,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     scanline=(unsigned char *) AcquireMemory(image->columns*pcx_info.planes);
     if ((pcx_pixels == (unsigned char *) NULL) ||
         (scanline == (unsigned char *) NULL))
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     /*
       Uncompress image data.
     */
@@ -388,8 +391,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             Initialize image colormap.
           */
           if (image->colors > 256)
-            ThrowReaderException(CorruptImageError,
-              "PCX colormap exceeded 256 colors",image);
+            ThrowReaderException(CorruptImageError,"ColormapExceeds256Colors",
+              image);
           if ((pcx_info.bits_per_pixel*pcx_info.planes) == 1)
             {
               /*
@@ -413,9 +416,9 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 p=pcx_colormap;
                 for (i=0; i < (long) image->colors; i++)
                 {
-                  image->colormap[i].red=Upscale(*p++);
-                  image->colormap[i].green=Upscale(*p++);
-                  image->colormap[i].blue=Upscale(*p++);
+                  image->colormap[i].red=ScaleCharToQuantum(*p++);
+                  image->colormap[i].green=ScaleCharToQuantum(*p++);
+                  image->colormap[i].blue=ScaleCharToQuantum(*p++);
                 }
             }
           LiberateMemory((void **) &pcx_colormap);
@@ -441,23 +444,23 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               case 0:
               {
-                *r=Upscale(*p++);
+                *r=ScaleCharToQuantum(*p++);
                 break;
               }
               case 1:
               {
-                *r=Upscale(*p++);
+                *r=ScaleCharToQuantum(*p++);
                 break;
               }
               case 2:
               {
-                *r=Upscale(*p++);
+                *r=ScaleCharToQuantum(*p++);
                 break;
               }
               case 3:
               default:
               {
-                *r=Upscale(*p++);
+                *r=ScaleCharToQuantum(*p++);
                 break;
               }
             }
@@ -489,7 +492,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             case 1:
             {
-              register int
+              register long
                 bit;
 
               for (x=0; x < ((long) image->columns-7); x+=8)
@@ -554,11 +557,11 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           indexes[x]=(*r++);
         else
           {
-            q->red=Upscale(*r++);
-            q->green=Upscale(*r++);
-            q->blue=Upscale(*r++);
+            q->red=ScaleCharToQuantum(*r++);
+            q->green=ScaleCharToQuantum(*r++);
+            q->blue=ScaleCharToQuantum(*r++);
             if (image->matte)
-              q->opacity=(Quantum) (MaxRGB-Upscale(*r++));
+              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*r++));
           }
         q++;
       }
@@ -566,25 +569,30 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
     }
     if (image->storage_class == PseudoClass)
       SyncImage(image);
     LiberateMemory((void **) &scanline);
     LiberateMemory((void **) &pcx_pixels);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
     if (image_info->subrange != 0)
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
-    if (page_table == (off_t *) NULL)
+    if (page_table == (ExtendedSignedIntegralType *) NULL)
       break;
     if (page_table[id] == 0)
       break;
-    (void) SeekBlob(image,(off_t) page_table[id],SEEK_SET);
+    (void) SeekBlob(image,(ExtendedSignedIntegralType) page_table[id],SEEK_SET);
     count=ReadBlob(image,1,(char *) &pcx_info.identifier);
     if ((count != 0) && (pcx_info.identifier == 0x0a))
       {
@@ -597,11 +605,14 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   }
-  if (page_table != (off_t *) NULL)
+  if (page_table != (ExtendedSignedIntegralType *) NULL)
     LiberateMemory((void **) &page_table);
   while (image->previous != (Image *) NULL)
     image=image->previous;
@@ -638,17 +649,19 @@ ModuleExport void RegisterPCXImage(void)
     *entry;
 
   entry=SetMagickInfo("DCX");
-  entry->decoder=ReadPCXImage;
-  entry->encoder=WritePCXImage;
-  entry->magick=IsDCX;
+  entry->decoder=(DecoderHandler) ReadPCXImage;
+  entry->encoder=(EncoderHandler) WritePCXImage;
+  entry->seekable_stream=True;
+  entry->magick=(MagickHandler) IsDCX;
   entry->description=AcquireString("ZSoft IBM PC multi-page Paintbrush");
   entry->module=AcquireString("PCX");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCX");
-  entry->decoder=ReadPCXImage;
-  entry->encoder=WritePCXImage;
-  entry->magick=IsPCX;
+  entry->decoder=(DecoderHandler) ReadPCXImage;
+  entry->encoder=(EncoderHandler) WritePCXImage;
+  entry->magick=(MagickHandler) IsPCX;
   entry->adjoin=False;
+  entry->seekable_stream=True;
   entry->description=AcquireString("ZSoft IBM PC Paintbrush");
   entry->module=AcquireString("PCX");
   (void) RegisterMagickInfo(entry);
@@ -743,7 +756,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   unsigned int
     status;
 
-  off_t
+  ExtendedSignedIntegralType
     *page_table;
 
   unsigned long
@@ -756,28 +769,28 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
-  page_table=(off_t *) NULL;
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
+  page_table=(ExtendedSignedIntegralType *) NULL;
   if (image_info->adjoin)
     {
       /*
         Write the DCX page table.
       */
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
-      page_table=(off_t *) AcquireMemory(1024*sizeof(off_t));
-      if (page_table == (off_t *) NULL)
-        ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-          image);
+      page_table=(ExtendedSignedIntegralType *)
+        AcquireMemory(1024*sizeof(ExtendedSignedIntegralType));
+      if (page_table == (ExtendedSignedIntegralType *) NULL)
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
       for (scene=0; scene < 1024; scene++)
         (void) WriteBlobLSBLong(image,0x00000000L);
     }
   scene=0;
   do
   {
-    if (page_table != (off_t *) NULL)
+    if (page_table != (ExtendedSignedIntegralType *) NULL)
       page_table[scene]=TellBlob(image);
     /*
       Initialize PCX raster file header.
@@ -836,17 +849,16 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     */
     pcx_colormap=(unsigned char *) AcquireMemory(3*256);
     if (pcx_colormap == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     for (i=0; i < (3*256); i++)
       pcx_colormap[i]=0;
     q=pcx_colormap;
     if (image->storage_class == PseudoClass)
       for (i=0; i < (long) image->colors; i++)
       {
-        *q++=Downscale(image->colormap[i].red);
-        *q++=Downscale(image->colormap[i].green);
-        *q++=Downscale(image->colormap[i].blue);
+        *q++=ScaleQuantumToChar(image->colormap[i].red);
+        *q++=ScaleQuantumToChar(image->colormap[i].green);
+        *q++=ScaleQuantumToChar(image->colormap[i].blue);
       }
     (void) WriteBlob(image,3*16,(char *) pcx_colormap);
     (void) WriteBlobByte(image,pcx_info.reserved);
@@ -858,8 +870,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     length=image->rows*pcx_info.bytes_per_line*pcx_info.planes;
     pcx_pixels=(unsigned char *) AcquireMemory(length);
     if (pcx_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     q=pcx_pixels;
     if (image->storage_class == DirectClass)
       {
@@ -880,23 +891,23 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
               {
                 case 0:
                 {
-                  *q++=Downscale(p->red);
+                  *q++=ScaleQuantumToChar(p->red);
                   break;
                 }
                 case 1:
                 {
-                  *q++=Downscale(p->green);
+                  *q++=ScaleQuantumToChar(p->green);
                   break;
                 }
                 case 2:
                 {
-                  *q++=Downscale(p->blue);
+                  *q++=ScaleQuantumToChar(p->blue);
                   break;
                 }
                 case 3:
                 default:
                 {
-                  *q++=MaxRGB-Downscale(p->opacity);
+                  *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
                   break;
                 }
               }
@@ -904,7 +915,8 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             }
           }
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+              break;
         }
       }
     else
@@ -920,7 +932,8 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             *q++=indexes[x];
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
       else
         {
@@ -932,10 +945,10 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
           /*
             Convert PseudoClass image to a PCX monochrome image.
           */
-          polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+          polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
           if (image->colors == 2)
-            polarity=Intensity(&image->colormap[0]) <
-              Intensity(&image->colormap[1]);
+            polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+              PixelIntensityToQuantum(&image->colormap[1]);
           for (y=0; y < (long) image->rows; y++)
           {
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -963,7 +976,8 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
               *q++=byte << (8-bit);
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                MagickMonitor(SaveImageText,y,image->rows);
+                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                  break;
           }
         }
     /*
@@ -1000,7 +1014,8 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
           }
         (void) WriteBlobByte(image,previous);
         if (QuantumTick(y,image->rows))
-          MagickMonitor(SaveImageText,y,image->rows);
+          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+            break;
       }
     }
     (void) WriteBlobByte(image,pcx_info.colormap_signature);
@@ -1009,15 +1024,18 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     LiberateMemory((void **) &pcx_colormap);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
     if (scene >= 1023)
       break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
       image=image->previous;
-  if (page_table != (off_t *) NULL)
+  if (page_table != (ExtendedSignedIntegralType *) NULL)
     {
       /*
         Write the DCX page table.
@@ -1030,7 +1048,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
       LiberateMemory((void **) &page_table);
     }
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to write file",image);
+    ThrowWriterException(FileOpenError,"UnableToWriteFile",image);
   CloseBlob(image);
   return(True);
 }
