@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -52,8 +52,14 @@
 /*
   Include declarations.
 */
-#include <time.h>
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Typedef declarations.
@@ -155,7 +161,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
     count,
     pixel;
 
-  register int
+  register long
     i;
 
   register unsigned char
@@ -213,7 +219,7 @@ static unsigned int IsPDB(const unsigned char *magick,const size_t length)
 {
   if (length < 68)
     return(False);
-  if (LocaleNCompare((char *) (magick+60),"vIMGView",8) == 0)
+  if (memcmp(magick+60,"vIMGView",8) == 0)
     return(True);
   return(False);
 }
@@ -290,10 +296,10 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *pixels;
 
   unsigned int
-    bits_per_pixel,
     status;
 
   unsigned long
+    bits_per_pixel,
     packets;
 
   /*
@@ -304,9 +310,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Determine if this is a PDB image file.
   */
@@ -323,14 +329,12 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) ReadBlob(image,4,pdb_info.id);
   if ((count == 0) || (memcmp(pdb_info.type,"vIMG",4) != 0) ||
       (memcmp(pdb_info.id,"View",4) != 0))
-    ThrowReaderException(CorruptImageWarning,"Not a supported PDB image file",
-      image);
+    ThrowReaderException(CoderError,"NotASupportedImageFile",image);
   pdb_info.seed=ReadBlobMSBLong(image);
   pdb_info.next_record=ReadBlobMSBLong(image);
   pdb_info.number_records=ReadBlobMSBShort(image);
   if (pdb_info.next_record != 0)
-    ThrowReaderException(CorruptImageWarning,
-      "Multiple Record List PDB Unsupported",image);
+    ThrowReaderException(CoderError,"MultipleRecordListNotSupported",image);
   /*
     Read record header.
   */
@@ -339,7 +343,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   record_type=ReadBlobByte(image);
   if (((record_type != 0x00) && (record_type != 0x01)) ||
       (memcmp(tag,"\x40\x6f\x80",3) != 0))
-    ThrowReaderException(CorruptImageError,"Corrupt PDB image file",image);
+    ThrowReaderException(CorruptImageError,"CorruptPDBImageFile",image);
   if ((offset-TellBlob(image)) == 6)
     {
       (void) ReadBlobByte(image);
@@ -352,7 +356,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       record_type=ReadBlobByte(image);
       if (((record_type != 0x00) && (record_type != 0x01)) ||
           (memcmp(tag,"\x40\x6f\x80",3) != 0))
-        ThrowReaderException(CorruptImageError,"Corrupt PDB image file",image);
+        ThrowReaderException(CorruptImageError,"CorruptPDBImageFile",image);
       if ((offset-TellBlob(image)) == 6)
         {
           (void) ReadBlobByte(image);
@@ -383,7 +387,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->storage_class=PseudoClass;
   bits_per_pixel=pdb_image.type == 0 ? 2 : pdb_image.type == 2 ? 4 : 1;
   if (!AllocateImageColormap(image,1 << bits_per_pixel))
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->ping)
     {
       CloseBlob(image);
@@ -392,9 +396,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   packets=(bits_per_pixel*image->columns/8)*image->rows;
   pixels=(unsigned char *) AcquireMemory(packets+256);
   if (pixels == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitWarning,"MemoryAllocationFailed",image);
   switch (pdb_image.version)
-	{
+  {
     case 0:
     {
       image->compression=NoCompression;
@@ -403,12 +407,12 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     case 1:
     {
-      image->compression=RunlengthEncodedCompression;
+      image->compression=RLECompression;
       (void) DecodeImage(image,pixels,packets);
       break;
     }
     default:
-      ThrowReaderException(CorruptImageWarning,"Unknown compression type",image)
+      ThrowReaderException(CorruptImageWarning,"UnknownCompressionType",image)
   }
   p=pixels;
   switch (bits_per_pixel)
@@ -440,7 +444,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (!SyncImagePixels(image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
       }
       break;
     }
@@ -474,7 +479,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (!SyncImagePixels(image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
       }
       break;
     }
@@ -502,16 +508,18 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (!SyncImagePixels(image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
       }
       break;
     }
     default:
-      ThrowReaderException(CorruptImageError,"Not a PDB image file",image)
+      ThrowReaderException(CorruptImageError,"NotAPDBImageFile",image)
   }
   LiberateMemory((void **) &pixels);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   if ((offset-TellBlob(image)) == 0)
     {
       char
@@ -550,8 +558,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           c=ReadBlobByte(image);
         }
       if (comment == (char *) NULL)
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       (void) SetImageAttribute(image,"comment",comment);
       LiberateMemory((void **) &comment);
     }
@@ -588,9 +595,9 @@ ModuleExport void RegisterPDBImage(void)
     *entry;
 
   entry=SetMagickInfo("PDB");
-  entry->decoder=ReadPDBImage;
-  entry->encoder=WritePDBImage;
-  entry->magick=IsPDB;
+  entry->decoder=(DecoderHandler) ReadPDBImage;
+  entry->encoder=(EncoderHandler) WritePDBImage;
+  entry->magick=(MagickHandler) IsPDB;
   entry->description=AcquireString("Palm Database ImageViewer Format");
   entry->module=AcquireString("PDB");
   (void) RegisterMagickInfo(entry);
@@ -667,7 +674,7 @@ static unsigned char *EncodeRLE(unsigned char *destination,
 
 static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
 {
-  char
+  int
     bits;
 
   long
@@ -689,14 +696,14 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
     *scanline;
 
   unsigned int
-    bits_per_pixel,
-    status,
-    literal,
-    repeat;
+    status;
 
   unsigned long
+    bits_per_pixel,
+    literal,
     packet_size,
-    packets;
+    packets,
+    repeat;
 
   const ImageAttribute
     *comment;
@@ -708,17 +715,17 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   bits_per_pixel=image_info->depth;
   if (GetImageType(image,&image->exception) == BilevelType)
     bits_per_pixel=1;
   if ((bits_per_pixel != 1) && (bits_per_pixel != 2))
     bits_per_pixel=4;
   memset(pdb_info.name,0,32);
-  strncpy(pdb_info.name,image_info->filename,32);
+  (void) strncpy(pdb_info.name,image_info->filename,32);
   pdb_info.attributes=0;
   pdb_info.version=0;
   pdb_info.create_time=time(NULL);
@@ -747,10 +754,10 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBLong(image,pdb_info.seed);
   (void) WriteBlobMSBLong(image,pdb_info.next_record);
   (void) WriteBlobMSBShort(image,pdb_info.number_records);
-  (void) strncpy(pdb_image.name, pdb_info.name, 32);
+  (void) strncpy(pdb_image.name,pdb_info.name,32);
   pdb_image.version=1;  /* RLE Compressed */
   switch(bits_per_pixel)
-	{
+  {
     case 1: pdb_image.type=(char) 0xff; break;  /* monochrome */
     case 2: pdb_image.type=(char) 0x00; break;  /* 2 bit gray */
     default: pdb_image.type=(char) 0x02;  /* 4 bit gray */
@@ -762,26 +769,26 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   pdb_image.reserved_2=0;
   pdb_image.x_anchor=(short) 0xffff;
   pdb_image.y_anchor=(short) 0xffff;
-  pdb_image.width = image->columns;
+  pdb_image.width=(short) image->columns;
   if (image->columns % 16)
-    pdb_image.width=16*(image->columns/16+1);
-  pdb_image.height=image->rows;
+    pdb_image.width=(short) (16*(image->columns/16+1));
+  pdb_image.height=(short) image->rows;
   packets=(bits_per_pixel*image->columns/8)*image->rows;
   p=(unsigned char *) AcquireMemory(packets+packets/128);
   if (p == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitWarning,"MemoryAllocationFailed",image);
   buffer=(unsigned char *) AcquireMemory(256);
   if (buffer == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitWarning,"MemoryAllocationFailed",image);
   packet_size=image->depth > 8 ? 2: 1;
   scanline=(unsigned char *) AcquireMemory(packet_size*image->columns);
   if (scanline == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(ResourceLimitWarning,"MemoryAllocationFailed",image);
+  (void) TransformRGBImage(image,image->colorspace);
   /*
     Convert to GRAY raster scanline.
   */
-  bits=8/bits_per_pixel-1;  /* start at most significant bits */
+  bits=8/(long) bits_per_pixel-1;  /* start at most significant bits */
   literal=0;
   repeat=0;
   q=p;
@@ -793,7 +800,7 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
     (void) PopImagePixels(image,GrayQuantum,scanline);
     for (x=0; x < pdb_image.width; x++)
     {
-      if (x < image->columns)
+      if (x < (long) image->columns)
         buffer[literal+repeat]|=(0xff-scanline[x*packet_size]) >>
           (8-bits_per_pixel) << bits*bits_per_pixel;
       bits--;
@@ -834,12 +841,13 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
                   literal-=0x80;
                 }
             }
-        bits=8/bits_per_pixel-1;
+        bits=8/(long) bits_per_pixel-1;
         buffer[literal+repeat]=0x00;
       }
     }
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+        break;
   }
   q=EncodeRLE(q,buffer,literal,repeat);
   LiberateMemory((void **) &scanline);
@@ -847,7 +855,8 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   /*
     Write the Image record header.
   */
-  (void) WriteBlobMSBLong(image,TellBlob(image)+8*pdb_info.number_records);
+  (void) WriteBlobMSBLong(image,(unsigned long)
+    (TellBlob(image)+8*pdb_info.number_records));
   (void) WriteBlobByte(image,0x40);
   (void) WriteBlobByte(image,0x6f);
   (void) WriteBlobByte(image,0x80);
@@ -864,7 +873,7 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobByte(image,1);
     }
   /*
-	 Write the Image data.
+    Write the Image data.
   */
   (void) WriteBlob(image,32,pdb_image.name);
   (void) WriteBlobByte(image,pdb_image.version);
