@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "log.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   ImageMagick Macintosh PICT Methods.
@@ -491,7 +498,7 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,Image *blob,
     if (scanline_length >= row_bytes)
       {
         ThrowException(&image->exception,CorruptImageError,
-          "Unable to uncompress image","scanline length exceeds row bytes");
+          "UnableToUncompressImage","scanline length exceeds row bytes");
         break;
       }
     (void) ReadBlob(blob,scanline_length,(char *) scanline);
@@ -562,7 +569,7 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 #define MaxCount  128
 #define MaxPackbitsRunlength  128
 
-  int
+  long
     count,
     repeat_count,
     runlength;
@@ -602,27 +609,27 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
         if (runlength < 3)
           while (runlength > 0)
           {
-            *q++=index;
+            *q++=(unsigned char) index;
             runlength--;
             count++;
             if (count == MaxCount)
               {
-                *q++=MaxCount-1;
+                *q++=(unsigned char) (MaxCount-1);
                 count-=MaxCount;
               }
           }
         else
           {
             if (count > 0)
-              *q++=count-1;
+              *q++=(unsigned char) (count-1);
             count=0;
             while (runlength > 0)
             {
               repeat_count=runlength;
               if (repeat_count > MaxPackbitsRunlength)
                 repeat_count=MaxPackbitsRunlength;
-              *q++=index;
-              *q++=257-repeat_count;
+              *q++=(unsigned char) index;
+              *q++=(unsigned char) (257-repeat_count);
               runlength-=repeat_count;
             }
           }
@@ -634,27 +641,27 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   if (runlength < 3)
     while (runlength > 0)
     {
-      *q++=index;
+      *q++=(unsigned char) index;
       runlength--;
       count++;
       if (count == MaxCount)
         {
-          *q++=MaxCount-1;
+          *q++=(unsigned char) (MaxCount-1);
           count-=MaxCount;
         }
     }
   else
     {
       if (count > 0)
-        *q++=count-1;
+        *q++=(unsigned char) (count-1);
       count=0;
       while (runlength > 0)
       {
         repeat_count=runlength;
         if (repeat_count > MaxPackbitsRunlength)
           repeat_count=MaxPackbitsRunlength;
-        *q++=index;
-        *q++=257-repeat_count;
+        *q++=(unsigned char) index;
+        *q++=(unsigned char) (257-repeat_count);
         runlength-=repeat_count;
       }
     }
@@ -682,6 +689,46 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   return(length);
 }
 
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   I s P I C T                                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsPICT() returns True if the image format type, identified by the magick
+%  string, is PICT.
+%
+%  The format of the ReadPICTImage method is:
+%
+%      unsigned int IsPICT(const unsigned char *magick,const size_t length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsPICT returns True if the image format type is PICT.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+static unsigned int IsPICT(const unsigned char *magick,const size_t length)
+{
+  if (length < 528)
+    return(False);
+  if (memcmp(magick+522,"\000\021\002\377\014\000",6) == 0)
+    return(True);
+  if (memcmp(magick+522,"\021\001",2) == 0)
+    return(True);
+  return(False);
+}
+
 #if !defined(macintosh)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -729,12 +776,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
 
   int
     c,
-    code,
-    flags,
-    j,
-    version;
+    code;
 
   long
+    flags,
+    j,
+    version,
     y;
 
   PICTRectangle
@@ -770,9 +817,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read PICT header.
   */
@@ -782,17 +829,17 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   ReadRectangle(frame);
   while ((c=ReadBlobByte(image)) == 0);
   if (c != 0x11)
-    ThrowReaderException(CorruptImageError,"Not a PICT image file",image);
+    ThrowReaderException(CorruptImageError,"NotAPICTImageFile",image);
   version=ReadBlobByte(image);
   if (version == 2)
     {
       c=ReadBlobByte(image);
       if (c != 0xff)
-        ThrowReaderException(CorruptImageError,"Not a PICT image file",image);
+        ThrowReaderException(CorruptImageError,"NotAPICTImageFile",image);
     }
   else
     if (version != 1)
-      ThrowReaderException(CorruptImageError,"Not a PICT image file",image);
+      ThrowReaderException(CorruptImageError,"NotAPICTImageFile",image);
   /*
     Create black canvas.
   */
@@ -815,13 +862,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       code=ReadBlobMSBShort(image);
     if (code > 0xa1)
       {
-        if (image_info->debug)
-          (void) fprintf(stderr,"%04x:\n",code);
+        if (IsEventLogging())
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%04X:",code);
       }
     else
       {
-        if (image_info->debug)
-          (void) fprintf(stderr,"  %04x %.1024s: %.1024s\n",code,
+        if (IsEventLogging())
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  %04X %.1024s: %.1024s",code,
             codes[code].name,codes[code].description);
         switch (code)
         {
@@ -849,10 +896,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           case 0x13:
           case 0x14:
           {
-            int
+            long
               pattern;
 
-            unsigned int
+            unsigned long
               height,
               width;
 
@@ -869,7 +916,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (pattern != 1)
-              ThrowReaderException(CorruptImageError,"Unknown pattern type",
+              ThrowReaderException(CorruptImageError,"UnknownPatternType",
                 image);
             length=ReadBlobMSBShort(image);
             ReadRectangle(frame);
@@ -911,11 +958,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               Initialize image background color.
             */
             image->background_color.red=(Quantum)
-              XDownscale(ReadBlobMSBShort(image));
+              ScaleShortToQuantum(ReadBlobMSBShort(image));
             image->background_color.green=(Quantum)
-              XDownscale(ReadBlobMSBShort(image));
+              ScaleShortToQuantum(ReadBlobMSBShort(image));
             image->background_color.blue=(Quantum)
-              XDownscale(ReadBlobMSBShort(image));
+              ScaleShortToQuantum(ReadBlobMSBShort(image));
             break;
           }
           case 0x70:
@@ -942,7 +989,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           case 0x9a:
           case 0x9b:
           {
-            int
+            long
               bytes_per_line;
 
             PICTRectangle
@@ -1006,7 +1053,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   {
                     DestroyImage(tile_image);
                     ThrowReaderException(ResourceLimitError,
-                      "Memory allocation failed",image)
+                      "MemoryAllocationFailed",image)
                   }
                 if (bytes_per_line & 0x8000)
                   {
@@ -1016,11 +1063,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       if (flags & 0x8000)
                         j=i;
                       tile_image->colormap[j].red=(Quantum)
-                        XDownscale(ReadBlobMSBShort(image));
+                        ScaleShortToQuantum(ReadBlobMSBShort(image));
                       tile_image->colormap[j].green=(Quantum)
-                        XDownscale(ReadBlobMSBShort(image));
+                        ScaleShortToQuantum(ReadBlobMSBShort(image));
                       tile_image->colormap[j].blue=(Quantum)
-                        XDownscale(ReadBlobMSBShort(image));
+                        ScaleShortToQuantum(ReadBlobMSBShort(image));
                     }
                   }
                 else
@@ -1058,7 +1105,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 DestroyImage(tile_image);
                 ThrowReaderException(ResourceLimitError,
-                  "Memory allocation failed",image)
+                  "MemoryAllocationFailed",image)
               }
             /*
               Convert PICT tile image to pixel packets.
@@ -1086,24 +1133,27 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       {
                         i=(*p++);
                         j=(*p);
-                        q->red=Upscale((i & 0x7c) << 1);
-                        q->green=Upscale(((i & 0x03) << 6) | ((j & 0xe0) >> 2));
-                        q->blue=Upscale((j & 0x1f) << 3);
+                        q->red=ScaleCharToQuantum((i & 0x7c) << 1);
+                        q->green=ScaleCharToQuantum(((i & 0x03) << 6) |
+                          ((j & 0xe0) >> 2));
+                        q->blue=ScaleCharToQuantum((j & 0x1f) << 3);
                       }
                     else
                       if (!tile_image->matte)
                         {
-                          q->red=Upscale(*p);
-                          q->green=Upscale(*(p+tile_image->columns));
-                          q->blue=Upscale(*(p+2*tile_image->columns));
+                          q->red=ScaleCharToQuantum(*p);
+                          q->green=
+                            ScaleCharToQuantum(*(p+tile_image->columns));
+                          q->blue=ScaleCharToQuantum(*(p+2*tile_image->columns));
                         }
                       else
                         {
-                          q->opacity=(Quantum) (MaxRGB-Upscale(*p));
-                          q->red=Upscale(*(p+tile_image->columns));
+                          q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p));
+                          q->red=ScaleCharToQuantum(*(p+tile_image->columns));
                           q->green=(Quantum)
-                            Upscale(*(p+2*tile_image->columns));
-                          q->blue=Upscale(*(p+3*tile_image->columns));
+                            ScaleCharToQuantum(*(p+2*tile_image->columns));
+                          q->blue=
+                           ScaleCharToQuantum(*(p+3*tile_image->columns));
                         }
                   }
                 p++;
@@ -1116,7 +1166,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 p+=(pixmap.component_count-1)*tile_image->columns;
               if (destination.bottom == (long) image->rows)
                 if (QuantumTick(y,tile_image->rows))
-                  MagickMonitor(LoadImageText,y,tile_image->rows);
+                  {
+                    status=MagickMonitor(LoadImageTag,y,tile_image->rows,
+                      &image->exception);
+                    if (status == False)
+                      break;
+                  }
             }
             (void) LiberateMemory((void **) &pixels);
             if (tile_image != image)
@@ -1129,7 +1184,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 DestroyImage(tile_image);
               }
             if (destination.bottom != (long) image->rows)
-              MagickMonitor(LoadImageText,destination.bottom,image->rows);
+              {
+                status=MagickMonitor(LoadImageTag,destination.bottom,image->rows,
+                  &image->exception);
+                if (status == False)
+                  break;
+              }
             break;
           }
           case 0xa1:
@@ -1137,7 +1197,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             unsigned char
               *info;
 
-            unsigned short
+            unsigned long
               type;
 
             /*
@@ -1163,7 +1223,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   AcquireMemory(length);
                 if (image->color_profile.info == (unsigned char *) NULL)
                   ThrowReaderException(ResourceLimitError,
-                    "Memory allocation failed",image);
+                    "MemoryAllocationFailed",image);
                 image->color_profile.length=length;
                 (void) memcpy(image->color_profile.info,info,length);
                 break;
@@ -1174,7 +1234,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   AcquireMemory(length);
                 if (image->iptc_profile.info == (unsigned char *) NULL)
                   ThrowReaderException(ResourceLimitError,
-                    "Memory allocation failed",image);
+                    "MemoryAllocationFailed",image);
                 image->iptc_profile.length=length;
                 (void) memcpy(image->iptc_profile.info,info,length);
                 break;
@@ -1229,9 +1289,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         clone_info->blob=(void *) NULL;
         clone_info->length=0;
         TemporaryFilename(clone_info->filename);
-        file=fopen(clone_info->filename,WriteBinaryType);
+        file=fopen(clone_info->filename,"wb");
         if (file == (FILE *) NULL)
-          ThrowReaderException(FileOpenError,"Unable to write file",image);
+          ThrowReaderException(FileOpenError,"UnableToWriteFile",image);
         length=ReadBlobMSBLong(image);
         for (i=0; i < 6; i++)
           (void) ReadBlobMSBLong(image);
@@ -1282,7 +1342,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       }
   }
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
   return(image);
 }
@@ -1317,17 +1378,19 @@ ModuleExport void RegisterPICTImage(void)
     *entry;
 
   entry=SetMagickInfo("PCT");
-  entry->decoder=ReadPICTImage;
-  entry->encoder=WritePICTImage;
+  entry->decoder=(DecoderHandler) ReadPICTImage;
+  entry->encoder=(EncoderHandler) WritePICTImage;
   entry->adjoin=False;
   entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
+  entry->magick=(MagickHandler) IsPICT;
   entry->module=AcquireString("PICT");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PICT");
-  entry->decoder=ReadPICTImage;
-  entry->encoder=WritePICTImage;
+  entry->decoder=(DecoderHandler) ReadPICTImage;
+  entry->encoder=(EncoderHandler) WritePICTImage;
   entry->adjoin=False;
   entry->description=AcquireString("Apple Macintosh QuickDraw/PICT");
+  entry->magick=(MagickHandler) IsPICT;
   entry->module=AcquireString("PICT");
   (void) RegisterMagickInfo(entry);
 }
@@ -1406,6 +1469,9 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  ExtendedSignedIntegralType
+    offset;
+
   PICTPixmap
     pixmap;
 
@@ -1428,8 +1494,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     x;
 
   size_t
-    count,
-    offset;
+    count;
 
   unsigned char
     *buffer,
@@ -1437,11 +1502,11 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     *scanline;
 
   unsigned int
-    status,
-    storage_class;
+    status;
 
   unsigned long
-    bytes_per_line;
+    bytes_per_line,
+    storage_class;
 
   unsigned short
     base_address,
@@ -1456,12 +1521,11 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if ((image->columns > 65535L) || (image->rows > 65535L))
-    ThrowWriterException(ResourceLimitError,
-      "Width or height limit exceeded",image);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit",image);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   /*
     Initialize image info.
   */
@@ -1516,12 +1580,11 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   if ((buffer == (unsigned char *) NULL) ||
       (packed_scanline == (unsigned char *) NULL) ||
       (scanline == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Write header, header size, size bounding box, version, and reserved.
   */
-  for (i=0; i < PictInfoSize; i++)
-    buffer[i]=0;
+  memset(buffer,0,PictInfoSize);
   (void) WriteBlob(image,PictInfoSize,buffer);
   (void) WriteBlobMSBShort(image,0);
   (void) WriteBlobMSBShort(image,size_rectangle.top);
@@ -1677,29 +1740,21 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBLong(image,(unsigned long) pixmap.reserved);
   if (storage_class == PseudoClass)
     {
-      unsigned short
-        red,
-        green,
-        blue;
-
       /*
         Write image colormap.
       */
       (void) WriteBlobMSBLong(image,0x00000000L);  /* color seed */
       (void) WriteBlobMSBShort(image,0L);  /* color flags */
-      (void) WriteBlobMSBShort(image,(unsigned short) Max(image->colors-1,1));
+      (void) WriteBlobMSBShort(image,(unsigned short) (image->colors-1));
       for (i=0; i < (long) image->colors; i++)
       {
-        red=(unsigned short)
-          ((unsigned long) (image->colormap[i].red*65535L)/MaxRGB);
-        green=(unsigned short)
-          ((unsigned long) (image->colormap[i].green*65535L)/MaxRGB);
-        blue=(unsigned short)
-          ((unsigned long) (image->colormap[i].blue*65535L)/MaxRGB);
         (void) WriteBlobMSBShort(image,i);
-        (void) WriteBlobMSBShort(image,red);
-        (void) WriteBlobMSBShort(image,green);
-        (void) WriteBlobMSBShort(image,blue);
+        (void) WriteBlobMSBShort(image,
+          ScaleQuantumToShort(image->colormap[i].red));
+        (void) WriteBlobMSBShort(image,
+          ScaleQuantumToShort(image->colormap[i].green));
+        (void) WriteBlobMSBShort(image,
+          ScaleQuantumToShort(image->colormap[i].blue));
       }
     }
   /*
@@ -1729,7 +1784,8 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
         scanline[x]=indexes[x];
       count+=EncodeImage(image,scanline,row_bytes & 0x7FFF,packed_scanline);
       if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+          break;
     }
   else
     if (image->compression == JPEGCompression)
@@ -1767,16 +1823,17 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
             }
           for (x=0; x < (long) image->columns; x++)
           {
-            *red++=Downscale(p->red);
-            *green++=Downscale(p->green);
-            *blue++=Downscale(p->blue);
+            *red++=ScaleQuantumToChar(p->red);
+            *green++=ScaleQuantumToChar(p->green);
+            *blue++=ScaleQuantumToChar(p->blue);
             if (image->matte)
-              *opacity++=MaxRGB-Downscale(p->opacity);
+              *opacity++=ScaleQuantumToChar(MaxRGB-p->opacity);
             p++;
           }
           count+=EncodeImage(image,scanline,row_bytes & 0x7FFF,packed_scanline);
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+              break;
         }
       }
   if (count & 0x1)
@@ -1784,7 +1841,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBShort(image,PictEndOfPictureOp);
   offset=TellBlob(image);
   (void) SeekBlob(image,512,SEEK_SET);
-  (void) WriteBlobMSBShort(image,offset);
+  (void) WriteBlobMSBShort(image,(unsigned long) offset);
   LiberateMemory((void **) &scanline);
   LiberateMemory((void **) &packed_scanline);
   LiberateMemory((void **) &buffer);
