@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -111,10 +117,10 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *q;
 
   unsigned int
-    bits_per_pixel,
     status;
 
   unsigned long
+    bits_per_pixel,
     height,
     length,
     width;
@@ -127,9 +133,9 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read PIX image.
   */
@@ -140,7 +146,7 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   bits_per_pixel=ReadBlobMSBShort(image);
   if ((width == (unsigned long) ~0) || (height == (unsigned long) ~0) ||
       ((bits_per_pixel != 8) && (bits_per_pixel != 24)))
-    ThrowReaderException(CorruptImageError,"Not a PIX image file",image);
+    ThrowReaderException(CorruptImageError,"NotAPIXImageFile",image);
   do
   {
     /*
@@ -149,9 +155,8 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->columns= width;
     image->rows= height;
     if (bits_per_pixel == 8)
-      if (!AllocateImageColormap(image,MaxRGB+1))
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+      if (!AllocateImageColormap(image,256))
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     if (image_info->ping && (image_info->subrange != 0))
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
@@ -175,12 +180,12 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             length=ReadBlobByte(image);
             if (bits_per_pixel == 8)
-              index=Upscale(ReadBlobByte(image));
+              index=ScaleCharToQuantum(ReadBlobByte(image));
             else
               {
-                blue=Upscale(ReadBlobByte(image));
-                green=Upscale(ReadBlobByte(image));
-                red=Upscale(ReadBlobByte(image));
+                blue=ScaleCharToQuantum(ReadBlobByte(image));
+                green=ScaleCharToQuantum(ReadBlobByte(image));
+                red=ScaleCharToQuantum(ReadBlobByte(image));
               }
           }
         if (image->storage_class == PseudoClass)
@@ -194,12 +199,17 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (!SyncImagePixels(image))
         break;
       if (image->previous == (Image *) NULL)
-        MagickMonitor(LoadImageText,y,image->rows);
+        if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+          break;
     }
     if (image->storage_class == PseudoClass)
       SyncImage(image);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -224,8 +234,11 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (status == True);
   while (image->previous != (Image *) NULL)
@@ -263,7 +276,7 @@ ModuleExport void RegisterPIXImage(void)
     *entry;
 
   entry=SetMagickInfo("PIX");
-  entry->decoder=ReadPIXImage;
+  entry->decoder=(DecoderHandler) ReadPIXImage;
   entry->description=AcquireString("Alias/Wavefront RLE image format");
   entry->module=AcquireString("PIX");
   (void) RegisterMagickInfo(entry);
