@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "magick.h"
+#include "monitor.h"
+#include "signature.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -86,24 +92,24 @@
 %
 */
 
-static void PlasmaPixel(Image *image,double x,double y)
+static void inline PlasmaPixel(Image *image,double x,double y)
 {
   register PixelPacket
     *q;
 
-  q=GetImagePixels(image,(long) (x+0.5),(long) (y+0.5),1,1);
+  q=GetImagePixels(image,(long) ceil(x-0.5),(long) ceil(y-0.5),1,1);
   if (q == (PixelPacket *) NULL)
     return;
-  q->red=(Quantum) ((double) MaxRGB*rand()/RAND_MAX);
-  q->green=(Quantum) ((double) MaxRGB*rand()/RAND_MAX);
-  q->blue=(Quantum) ((double) MaxRGB*rand()/RAND_MAX);
+  q->red=(Quantum) ((double) MaxRGB*GetRandomValue()+0.5);
+  q->green=(Quantum) ((double) MaxRGB*GetRandomValue()+0.5);
+  q->blue=(Quantum) ((double) MaxRGB*GetRandomValue()+0.5);
   (void) SyncImagePixels(image);
 }
 
 static Image *ReadPlasmaImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define PlasmaImageText  "  Applying image plasma...  "
+#define PlasmaImageTag  "Plasma/Image"
 
   Image
     *image;
@@ -124,7 +130,7 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
   SegmentInfo
     segment_info;
 
-  unsigned int
+  unsigned long
     depth,
     max_depth;
 
@@ -148,7 +154,7 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      q->opacity=MaxRGB >> 1;
+      q->opacity=(Quantum) (MaxRGB/2);
       q++;
     }
     if (!SyncImagePixels(image))
@@ -158,7 +164,6 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
   segment_info.y1=0;
   segment_info.x2=image->columns-1;
   segment_info.y2=image->rows-1;
-  srand((unsigned int) time(0));
   if (LocaleCompare(image_info->filename,"fractal") == 0)
     {
       /*
@@ -175,12 +180,13 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
       PlasmaPixel(image,segment_info.x2,(segment_info.y1+segment_info.y2)/2);
       PlasmaPixel(image,segment_info.x2,segment_info.y2);
     }
-  i=(long) (Max(image->columns,image->rows) >> 1);
+  i=(long) Max(image->columns,image->rows)/2;
   for (max_depth=0; i != 0; max_depth++)
     i>>=1;
   for (depth=1; ; depth++)
   {
-    MagickMonitor(PlasmaImageText,depth,max_depth);
+    if (!MagickMonitor(PlasmaImageTag,depth,max_depth,&image->exception))
+      break;
     if (PlasmaImage(image,&segment_info,0,depth))
       break;
   }
@@ -216,13 +222,13 @@ ModuleExport void RegisterPLASMAImage(void)
     *entry;
 
   entry=SetMagickInfo("PLASMA");
-  entry->decoder=ReadPlasmaImage;
+  entry->decoder=(DecoderHandler) ReadPlasmaImage;
   entry->adjoin=False;
   entry->description=AcquireString("Plasma fractal image");
   entry->module=AcquireString("PLASMA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FRACTAL");
-  entry->decoder=ReadPlasmaImage;
+  entry->decoder=(DecoderHandler) ReadPlasmaImage;
   entry->adjoin=False;
   entry->description=AcquireString("Plasma fractal image");
   entry->module=AcquireString("PLASMA");
