@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,7 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
-
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
+
 /*
   Forward declarations.
 */
@@ -137,7 +144,7 @@ static unsigned int PNMInteger(Image *image,const unsigned int base)
   int
     c;
 
-  unsigned int
+  unsigned long
     value;
 
   /*
@@ -153,7 +160,7 @@ static unsigned int PNMInteger(Image *image,const unsigned int base)
         char
           *comment;
 
-        off_t
+        ExtendedSignedIntegralType
           offset;
 
         register char
@@ -222,17 +229,12 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  IndexPacket
-    index;
-
-  int
-    blue,
-    green,
-    red;
-
   long
     y;
 
+  LongPixelPacket
+    pixel;
+
   MonitorHandler
     handler;
 
@@ -252,13 +254,13 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *p;
 
   size_t
-    count,
-    number_pixels;
+    count;
 
   unsigned char
     *pixels;
 
   unsigned int
+    index,
     status;
 
   unsigned long
@@ -266,7 +268,6 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     packets,
     *scale;
 
-
   /*
     Open image file.
   */
@@ -275,23 +276,20 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read PNM image.
   */
   count=ReadBlob(image,1,(char *) &format);
   do
   {
-    /*
-      Verify PNM identifier.
-    */
-    if ((count == 0) || (format != 'P'))
-      ThrowReaderException(CorruptImageError,"Not a PNM image file",image);
     /*
       Initialize image structure.
     */
+    if ((count == 0) || (format != 'P'))
+      ThrowReaderException(CorruptImageError,"NotAPNMImageFile",image);
     format=ReadBlobByte(image);
     if (format == '7')
       (void) PNMInteger(image,10);
@@ -305,12 +303,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((format != '3') && (format != '6'))
       {
         image->storage_class=PseudoClass;
-        image->colors=max_value > MaxRGB ? MaxRGB+1 : max_value+1;
+        image->colors=
+          max_value >= MaxColormapSize ? MaxColormapSize : max_value+1;
       }
-    number_pixels=image->columns*image->rows;
-    if (number_pixels == 0)
-      ThrowReaderException(CorruptImageError,
-        "Unable to read image: image dimensions are zero",image);
+    if ((image->columns == 0) || (image->rows == 0))
+      ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize",image);
     scale=(unsigned long *) NULL;
     if (image->storage_class == PseudoClass)
       {
@@ -318,7 +315,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Create colormap.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         if (format == '7')
           {
@@ -326,16 +323,16 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
               Initialize 332 colormap.
             */
             i=0;
-            for (red=0; red < 8; red++)
-              for (green=0; green < 8; green++)
-                for (blue=0; blue < 4; blue++)
+            for (pixel.red=0; pixel.red < 8; pixel.red++)
+              for (pixel.green=0; pixel.green < 8; pixel.green++)
+                for (pixel.blue=0; pixel.blue < 4; pixel.blue++)
                 {
                   image->colormap[i].red=(Quantum)
-                    ((unsigned long) (MaxRGB*red)/0x07);
+                    (((double) MaxRGB*pixel.red)/0x07+0.5);
                   image->colormap[i].green=(Quantum)
-                    ((unsigned long) (MaxRGB*green)/0x07);
+                    (((double) MaxRGB*pixel.green)/0x07+0.5);
                   image->colormap[i].blue=(Quantum)
-                    ((unsigned long) (MaxRGB*blue)/0x03);
+                    (((double) MaxRGB*pixel.blue)/0x03+0.5);
                   i++;
                 }
           }
@@ -348,10 +345,10 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         scale=(unsigned long *)
           AcquireMemory((max_value+1)*sizeof(unsigned long));
         if (scale == (unsigned long *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (i=0; i <= (long) max_value; i++)
-          scale[i]=(unsigned long) ((MaxRGB*(double) i)/(double) max_value);
+          scale[i]=(unsigned long) (((double) MaxRGB*i)/max_value);
       }
     if (image_info->ping && (image_info->subrange != 0))
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
@@ -378,7 +375,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (index >= image->colors)
               {
                 ThrowException(&image->exception,CorruptImageError,
-                  "invalid colormap index",image->filename);
+                  "InvalidColormapIndex",image->filename);
                 index=0;
               }
             indexes[x]=index;
@@ -388,7 +385,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         break;
       }
@@ -415,7 +413,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (index >= image->colors)
               {
                 ThrowException(&image->exception,CorruptImageError,
-                  "invalid colormap index",image->filename);
+                  "InvalidColormapIndex",image->filename);
                 index=0;
               }
             indexes[x]=index;
@@ -425,7 +423,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         break;
       }
@@ -441,25 +440,26 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            red=PNMInteger(image,10);
-            green=PNMInteger(image,10);
-            blue=PNMInteger(image,10);
+            pixel.red=PNMInteger(image,10);
+            pixel.green=PNMInteger(image,10);
+            pixel.blue=PNMInteger(image,10);
             if (scale != (unsigned long *) NULL)
               {
-                red=scale[red];
-                green=scale[green];
-                blue=scale[blue];
+                pixel.red=scale[pixel.red];
+                pixel.green=scale[pixel.green];
+                pixel.blue=scale[pixel.blue];
               }
-            q->red=(Quantum) red;
-            q->green=(Quantum) green;
-            q->blue=(Quantum) blue;
+            q->red=(Quantum) pixel.red;
+            q->green=(Quantum) pixel.green;
+            q->blue=(Quantum) pixel.blue;
             q++;
           }
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         break;
       }
@@ -496,11 +496,12 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         if (EOFBlob(image))
-          ThrowReaderException(CorruptImageError,"Unexpected end-of-file",
-            image);
+          ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+            image->filename);
         break;
       }
       case '5':
@@ -512,47 +513,57 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         packets=image->depth <= 8 ? 1 : 2;
         pixels=(unsigned char *) AcquireMemory(packets*image->columns);
         if (pixels == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Unable to allocate memory",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (y=0; y < (long) image->rows; y++)
         {
           count=ReadBlob(image,packets*image->columns,pixels);
           if (count == 0)
-            ThrowReaderException(CorruptImageError,
-              "Unable to read image data",image);
+            ThrowReaderException(CorruptImageError,"UnableToReadImageData",
+              image);
           p=pixels;
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
           indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
-          {
-            if (image->depth <= 8)
+          if (image->depth <= 8)
+            for (x=0; x < (long) image->columns; x++)
+            {
               index=(*p++);
-            else
-              {
-                index=(*p << 8) | *(p+1);
-                p+=2;
-              }
-            if (index >= image->colors)
-              {
-                ThrowException(&image->exception,CorruptImageError,
-                  "invalid colormap index",image->filename);
-                index=0;
-              }
-            indexes[x]=index;
-            *q++=image->colormap[index];
-          }
+              if (index >= image->colors)
+                {
+                  ThrowException(&image->exception,CorruptImageError,
+                    "invalid colormap index",image->filename);
+                  index=0;
+                }
+              indexes[x]=index;
+              *q++=image->colormap[index];
+            }
+          else
+            for (x=0; x < (long) image->columns; x++)
+            {
+              index=(*p << 8) | *(p+1);
+              p+=2;
+              if (index >= image->colors)
+                {
+                  ThrowException(&image->exception,CorruptImageError,
+                    "InvalidColormapIndex",image->filename);
+                  index=0;
+                }
+              indexes[x]=index;
+              *q++=image->colormap[index];
+            }
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         LiberateMemory((void **) &pixels);
         if (EOFBlob(image))
-          ThrowReaderException(CorruptImageError,"Unexpected end-of-file",
-            image);
+          ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+            image->filename);
         break;
       }
       case '6':
@@ -563,62 +574,72 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         packets=image->depth <= 8 ? 3 : 6;
         pixels=(unsigned char *) AcquireMemory(packets*image->columns);
         if (pixels == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Unable to allocate memory",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (y=0; y < (long) image->rows; y++)
         {
           count=ReadBlob(image,packets*image->columns,pixels);
           if (count == 0)
-            ThrowReaderException(CorruptImageError,
-              "Unable to read image data",image);
+            ThrowReaderException(CorruptImageError,"UnableToReadImageData",
+              image);
           p=pixels;
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
-          {
-            if (image->depth <= 8)
-              {
-                red=(*p++);
-                green=(*p++);
-                blue=(*p++);
-              }
-            else
-              {
-                red=(*p++) << 8;
-                red|=(*p++);
-                green=(*p++) << 8;
-                green|=(*p++);
-                blue=(*p++) << 8;
-                blue|=(*p++);
-              }
-            if (scale != (unsigned long *) NULL)
-              {
-                red=scale[red];
-                green=scale[green];
-                blue=scale[blue];
-              }
-            q->red=(Quantum) red;
-            q->green=(Quantum) green;
-            q->blue=(Quantum) blue;
-            q++;
-          }
+          if (image->depth <= 8)
+            for (x=0; x < (long) image->columns; x++)
+            {
+              pixel.red=(*p++);
+              pixel.green=(*p++);
+              pixel.blue=(*p++);
+              if (scale != (unsigned long *) NULL)
+                {
+                  pixel.red=scale[pixel.red];
+                  pixel.green=scale[pixel.green];
+                  pixel.blue=scale[pixel.blue];
+                }
+              q->red=(Quantum) pixel.red;
+              q->green=(Quantum) pixel.green;
+              q->blue=(Quantum) pixel.blue;
+              q++;
+            }
+          else
+            for (x=0; x < (long) image->columns; x++)
+            {
+              pixel.red=(*p << 8) | *(p+1);
+              p+=2;
+              pixel.green=(*p << 8) | *(p+1);
+              p+=2;
+              pixel.blue=(*p << 8) | *(p+1);
+              p+=2;
+              if (scale != (unsigned long *) NULL)
+                {
+                  pixel.red=scale[pixel.red];
+                  pixel.green=scale[pixel.green];
+                  pixel.blue=scale[pixel.blue];
+                }
+              q->red=(Quantum) pixel.red;
+              q->green=(Quantum) pixel.green;
+              q->blue=(Quantum) pixel.blue;
+              q++;
+            }
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         LiberateMemory((void **) &pixels);
         handler=SetMonitorHandler((MonitorHandler) NULL);
         (void) SetMonitorHandler(handler);
         if (EOFBlob(image))
-          ThrowReaderException(CorruptImageError,"Unexpected end-of-file",
-            image);
+          ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+            image->filename);
         break;
       }
       default:
-        ThrowReaderException(CorruptImageError,"Not a PNM image file",image)
+        ThrowReaderException(CorruptImageError,"NotAPNMImageFile",image)
     }
     if (scale != (unsigned long *) NULL)
       LiberateMemory((void **) &scale);
@@ -650,8 +671,11 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while ((count != 0) && (format == 'P'));
   while (image->previous != (Image *) NULL)
@@ -689,33 +713,33 @@ ModuleExport void RegisterPNMImage(void)
     *entry;
 
   entry=SetMagickInfo("P7");
-  entry->decoder=ReadPNMImage;
-  entry->encoder=WritePNMImage;
+  entry->decoder=(DecoderHandler) ReadPNMImage;
+  entry->encoder=(EncoderHandler) WritePNMImage;
   entry->description=AcquireString("Xv thumbnail format");
   entry->module=AcquireString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PBM");
-  entry->decoder=ReadPNMImage;
-  entry->encoder=WritePNMImage;
+  entry->decoder=(DecoderHandler) ReadPNMImage;
+  entry->encoder=(EncoderHandler) WritePNMImage;
   entry->description=AcquireString("Portable bitmap format (black and white)");
   entry->module=AcquireString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PGM");
-  entry->decoder=ReadPNMImage;
-  entry->encoder=WritePNMImage;
+  entry->decoder=(DecoderHandler) ReadPNMImage;
+  entry->encoder=(EncoderHandler) WritePNMImage;
   entry->description=AcquireString("Portable graymap format (gray scale)");
   entry->module=AcquireString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PNM");
-  entry->decoder=ReadPNMImage;
-  entry->encoder=WritePNMImage;
-  entry->magick=IsPNM;
+  entry->decoder=(DecoderHandler) ReadPNMImage;
+  entry->encoder=(EncoderHandler) WritePNMImage;
+  entry->magick=(MagickHandler) IsPNM;
   entry->description=AcquireString("Portable anymap");
   entry->module=AcquireString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PPM");
-  entry->decoder=ReadPNMImage;
-  entry->encoder=WritePNMImage;
+  entry->decoder=(DecoderHandler) ReadPNMImage;
+  entry->encoder=(EncoderHandler) WritePNMImage;
   entry->description=AcquireString("Portable pixmap format (color)");
   entry->module=AcquireString("PNM");
   (void) RegisterMagickInfo(entry);
@@ -787,6 +811,9 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
+  IndexPacket
+    index;
+
   int
     format;
 
@@ -808,9 +835,6 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
     scene,
     status;
 
-  unsigned short
-    index;
-
   /*
     Open output image file.
   */
@@ -818,34 +842,32 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   scene=0;
   do
   {
     /*
       Write PNM file header.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     format=6;
     if (LocaleCompare(image_info->magick,"PGM") == 0)
       format=5;
     else
       if (LocaleCompare(image_info->magick,"PBM") == 0)
-        {
-          format=4;
-          SetImageType(image,BilevelType);
-        }
+        format=4;
       else
-        if ((image_info->type != TrueColorType) &&
+        if ((LocaleCompare(image_info->magick,"PNM") == 0) &&
+            (image_info->type != TrueColorType) &&
             IsGrayImage(image,&image->exception))
           {
             format=5;
             if (IsMonochromeImage(image,&image->exception))
               format=4;
           }
-    if ((image_info->compression == NoCompression) || (image->depth > 8))
+    if (image_info->compression == NoCompression)
       format-=3;
     if (LocaleCompare(image_info->magick,"P7") != 0)
       FormatString(buffer,"P%d\n",format);
@@ -885,16 +907,17 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
     {
       case 1:
       {
-        register unsigned char
+        unsigned int
           polarity;
 
         /*
           Convert image to a PBM image.
         */
-        polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+        SetImageType(image,BilevelType);
+        polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
         if (image->colors == 2)
-          polarity=
-            Intensity(&image->colormap[0]) > Intensity(&image->colormap[1]);
+          polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+            PixelIntensityToQuantum(&image->colormap[1]);
         i=0;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -904,7 +927,7 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           indexes=GetIndexes(image);
           for (x=0; x < (long) image->columns; x++)
           {
-            FormatString(buffer,"%d ",(int) (indexes[x] == polarity));
+            FormatString(buffer,"%u ",indexes[x] == polarity ? 0x00 : 0x01);
             (void) WriteBlobString(image,buffer);
             i++;
             if (i == 36)
@@ -915,7 +938,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         if (i != 0)
           (void) WriteBlobByte(image,'\n');
@@ -926,9 +950,10 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PGM image.
         */
-        FormatString(buffer,"%d\n",
-          image->depth > 8 ? MaxRGB : Downscale(MaxRGB));
-        (void) WriteBlobString(image,buffer);
+        if (image->depth <= 8)
+          (void) WriteBlobString(image,"255\n");
+        else
+          (void) WriteBlobString(image,"65535\n");
         i=0;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -937,9 +962,11 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            index=Intensity(p);
-            FormatString(buffer," %d",
-              image->depth > 8 ? index : Downscale(index));
+            index=PixelIntensityToQuantum(p);
+            if (image->depth <= 8)
+              FormatString(buffer," %u",ScaleQuantumToChar(index));
+            else
+              FormatString(buffer," %u",ScaleQuantumToShort(index));
             (void) WriteBlobString(image,buffer);
             i++;
             if (i == 12)
@@ -951,7 +978,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         if (i != 0)
           (void) WriteBlobByte(image,'\n');
@@ -962,9 +990,10 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PNM image.
         */
-        FormatString(buffer,"%d\n",
-          image->depth > 8 ? MaxRGB : Downscale(MaxRGB));
-        (void) WriteBlobString(image,buffer);
+        if (image->depth <= 8)
+          (void) WriteBlobString(image,"255\n");
+        else
+          (void) WriteBlobString(image,"65535\n");
         i=0;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -973,10 +1002,12 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            FormatString(buffer," %d %d %d",
-              image->depth > 8 ? p->red : Downscale(p->red),
-              image->depth > 8 ? p->green : Downscale(p->green),
-              image->depth > 8 ? p->blue : Downscale(p->blue));
+            if (image->depth <= 8)
+              FormatString(buffer,"%u %u %u ",ScaleQuantumToChar(p->red),
+                ScaleQuantumToChar(p->green),ScaleQuantumToChar(p->blue));
+            else
+              FormatString(buffer,"%u %u %u ",ScaleQuantumToShort(p->red),
+                ScaleQuantumToShort(p->green),ScaleQuantumToShort(p->blue));
             (void) WriteBlobString(image,buffer);
             i++;
             if (i == 4)
@@ -988,7 +1019,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         if (i != 0)
           (void) WriteBlobByte(image,'\n');
@@ -1004,10 +1036,11 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PBM image.
         */
-        polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+        SetImageType(image,BilevelType);
+        polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
         if (image->colors == 2)
-          polarity=
-            Intensity(&image->colormap[0]) > Intensity(&image->colormap[1]);
+          polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+            PixelIntensityToQuantum(&image->colormap[1]);
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -1019,7 +1052,7 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
           {
             byte<<=1;
-            if (indexes[x] == polarity)
+            if (indexes[x] != polarity)
               byte|=0x01;
             bit++;
             if (bit == 8)
@@ -1034,7 +1067,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlobByte(image,byte << (8-bit));
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         break;
       }
@@ -1043,22 +1077,33 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Convert image to a PGM image.
         */
-        FormatString(buffer,"%u\n",Downscale(MaxRGB));
-        (void) WriteBlobString(image,buffer);
+        if (image->depth <= 8)
+          (void) WriteBlobString(image,"255\n");
+        else
+          (void) WriteBlobString(image,"65535\n");
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
-          {
-            index=Downscale(Intensity(p));
-            (void) WriteBlobByte(image,index);
-            p++;
-          }
+          if (image->depth <= 8)
+            for (x=0; x < (long) image->columns; x++)
+            {
+              (void) WriteBlobByte(image,
+                ScaleQuantumToChar(PixelIntensityToQuantum(p)));
+              p++;
+            }
+          else
+            for (x=0; x < (long) image->columns; x++)
+            {
+              (void) WriteBlobMSBShort(image,
+                ScaleQuantumToShort(PixelIntensityToQuantum(p)));
+              p++;
+            }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         break;
       }
@@ -1068,7 +1113,7 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           *q;
 
         size_t
-          length;
+          packets;
 
         unsigned char
           *pixels;
@@ -1076,33 +1121,48 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
         /*
           Allocate memory for pixels.
         */
-        length=image->columns*sizeof(PixelPacket);
-        pixels=(unsigned char *) AcquireMemory(length);
+        packets=image->depth <= 8 ? 3 : 6;
+        pixels=(unsigned char *) AcquireMemory(packets*image->columns);
         if (pixels == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"Memory allocation failed",
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         /*
           Convert image to a PNM image.
         */
-        FormatString(buffer,"%u\n",Downscale(MaxRGB));
-        (void) WriteBlobString(image,buffer);
+        if (image->depth <= 8)
+          (void) WriteBlobString(image,"255\n");
+        else
+          (void) WriteBlobString(image,"65535\n");
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
           q=pixels;
-          for (x=0; x < (long) image->columns; x++)
-          {
-            *q++=Downscale(p->red);
-            *q++=Downscale(p->green);
-            *q++=Downscale(p->blue);
-            p++;
-          }
+          if (image->depth <= 8)
+            for (x=0; x < (long) image->columns; x++)
+            {
+              *q++=ScaleQuantumToChar(p->red);
+              *q++=ScaleQuantumToChar(p->green);
+              *q++=ScaleQuantumToChar(p->blue);
+              p++;
+            }
+          else
+            for (x=0; x < (long) image->columns; x++)
+            {
+              *q++=(unsigned char) (ScaleQuantumToShort(p->red) >> 8);
+              *q++=(unsigned char) ScaleQuantumToShort(p->red);
+              *q++=(unsigned char) (ScaleQuantumToShort(p->green) >> 8);
+              *q++=(unsigned char) ScaleQuantumToShort(p->green);
+              *q++=(unsigned char) (ScaleQuantumToShort(p->blue) >> 8);
+              *q++=(unsigned char) ScaleQuantumToShort(p->blue);
+              p++;
+            }
           (void) WriteBlob(image,q-pixels,(char *) pixels);
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         LiberateMemory((void **) &pixels);
         break;
@@ -1152,8 +1212,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
             if ((red_map[i][j] == (unsigned short *) NULL) ||
                 (green_map[i][j] == (unsigned short *) NULL) ||
                 (blue_map[i][j] == (unsigned short *) NULL))
-              ThrowWriterException(ResourceLimitError,
-                "Memory allocation failed",image);
+              ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+                image);
           }
         /*
           Initialize dither tables.
@@ -1197,13 +1257,14 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
           {
             if (!image_info->dither)
-              pixel=(Quantum) ((Downscale(p->red) & 0xe0) |
-                ((Downscale(p->green) & 0xe0) >> 3) |
-                ((Downscale(p->blue) & 0xc0) >> 6));
+              pixel=(Quantum) ((ScaleQuantumToChar(p->red) & 0xe0) |
+                ((ScaleQuantumToChar(p->green) & 0xe0) >> 3) |
+                ((ScaleQuantumToChar(p->blue) & 0xc0) >> 6));
             else
-              pixel=(Quantum) ((red_map[i][j][Downscale(p->red)] & 0xe0) |
-                ((green_map[i][j][Downscale(p->green)] & 0xe0) >> 3) |
-                ((blue_map[i][j][Downscale(p->blue)] & 0xc0) >> 6));
+              pixel=(Quantum)
+                ((red_map[i][j][ScaleQuantumToChar(p->red)] & 0xe0) |
+                ((green_map[i][j][ScaleQuantumToChar(p->green)] & 0xe0) >> 3) |
+                ((blue_map[i][j][ScaleQuantumToChar(p->blue)] & 0xc0) >> 6));
             (void) WriteBlobByte(image,pixel);
             p++;
             j++;
@@ -1214,7 +1275,8 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
           if (i == 2)
             i=0;
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+              break;
         }
         /*
           Free allocated memory.
@@ -1231,8 +1293,11 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
     }
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
