@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Constant declarations.
@@ -95,7 +102,7 @@ ModuleExport void RegisterPREVIEWImage(void)
     *entry;
 
   entry=SetMagickInfo("PREVIEW");
-  entry->encoder=WritePreviewImage;
+  entry->encoder=(EncoderHandler) WritePreviewImage;
   entry->adjoin=False;
   entry->description=
     AcquireString("Show a preview an image enhancement, effect, or f/x");
@@ -161,7 +168,7 @@ ModuleExport void UnregisterPREVIEWImage(void)
 static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
 {
 #define NumberTiles  9
-#define PreviewImageText  "  Creating image preview...  "
+#define PreviewImageTag  "Preview/Image"
 
   char
     *commands[NumberTiles+6],
@@ -177,7 +184,7 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
     threshold;
 
   Image
-		*images,
+    *images,
     *montage_image,
     *preview_image;
 
@@ -216,11 +223,11 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   CloseBlob(image);
-  (void) TransformRGBImage(image,RGBColorspace);
+  (void) TransformRGBImage(image,image->colorspace);
   clone_info=CloneImageInfo(image_info);
   clone_info->quality=0;
   colors=2;
@@ -228,7 +235,7 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
   degrees=0;
   gamma=(-0.2f);
   SetGeometry(image,&geometry);
-  (void) GetMagickGeometry(DefaultPreviewGeometry,&geometry.x,&geometry.y,
+  (void) ParseMetaGeometry(DefaultPreviewGeometry,&geometry.x,&geometry.y,
     &geometry.width,&geometry.height);
   images=NewImageList();
   percentage=12.5;
@@ -248,8 +255,7 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
       {
         (void) QueryColorDatabase("#dfdfdf",&preview_image->matte_color,
           &image->exception);
-        PushImageList(&images,preview_image,&image->exception);
-        DestroyImage(preview_image);
+        AppendImageToList(&images,preview_image);
         continue;
       }
     argc=1;
@@ -275,8 +281,8 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
       }
       case RollPreview:
       {
-        x=((i+1)*preview_image->columns)/NumberTiles;
-        y=((i+1)*preview_image->rows)/NumberTiles;
+        x=(long) ((i+1)*preview_image->columns)/NumberTiles;
+        y=(long) ((i+1)*preview_image->rows)/NumberTiles;
         FormatString(factor,"%+ld%+ld",x,y);
         FormatString(label,"roll %.1024s",factor);
         commands[argc++]=(char *) "-roll";
@@ -400,7 +406,8 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
       }
       case ThresholdPreview:
       {
-        FormatString(factor,"%d",(int) ((percentage*(MaxRGB+1))/100));
+        FormatString(factor,"%lu",(unsigned long)
+          ((percentage*((double) MaxRGB+1.0))/100));
         FormatString(label,"threshold %.1024s",factor);
         commands[argc++]=(char *) "-threshold";
         commands[argc++]=factor;
@@ -530,13 +537,13 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
                 preview_image=quality_image;
               }
           }
-        if (GetBlobSize(preview_image) >= (1 << 24))
-          FormatString(label,"quality %.1024s\n%lumb ",factor,
-            (unsigned long) (GetBlobSize(preview_image)/1024/1024));
+        if ((GetBlobSize(preview_image)/1024) >= 1024)
+          FormatString(label,"quality %.1024s\n%gmb ",factor,
+            (double) GetBlobSize(preview_image)/1024.0/1024.0);
         else
-          if (GetBlobSize(preview_image) >= (1 << 16))
-            FormatString(label,"quality %.1024s\n%lukb ",factor,
-              (unsigned long) (GetBlobSize(preview_image)/1024));
+          if (GetBlobSize(preview_image) >= 1024)
+            FormatString(label,"quality %.1024s\n%gkb ",factor,
+              (double) GetBlobSize(preview_image)/1024.0);
           else
             FormatString(label,"quality %.1024s\n%lub ",factor,
               (unsigned long) GetBlobSize(preview_image));
@@ -550,13 +557,13 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
     commands[argc++]=label;
     (void) MogrifyImage(clone_info,argc,commands,&preview_image);
     (void) SetMonitorHandler(handler);
-    PushImageList(&images,preview_image,&image->exception);
-    DestroyImage(preview_image);
-    MagickMonitor(PreviewImageText,i,NumberTiles);
+    AppendImageToList(&images,preview_image);
+    if (!MagickMonitor(PreviewImageTag,i,NumberTiles,&image->exception))
+      break;
   }
   DestroyImageInfo(clone_info);
   if (images == (Image *) NULL)
-		return(False);
+    return(False);
   /*
     Create the montage.
   */
@@ -570,8 +577,7 @@ static unsigned int WritePreviewImage(const ImageInfo *image_info,Image *image)
   DestroyMontageInfo(montage_info);
   DestroyImageList(images);
   if (montage_image == (Image *) NULL)
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-      image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (montage_image->montage != (char *) NULL)
     {
       /*
