@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "delegate.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -60,84 +68,6 @@
 static unsigned int
   WritePSImage(const ImageInfo *,Image *);
 
-#if defined(HasGS)
-#include "ps/iapi.h"
-#include "ps/errors.h"
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   E x e c u t e P o s t I n t e r p r e t e r                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Method ExecutePostscriptInterpreter executes the postscript interpreter
-%  with the specified command.
-%
-%  The format of the ExecutePostscriptInterpreter method is:
-%
-%      unsigned int ExecutePostscriptInterpreter(const unsigned int verbose,
-%        const char *command)
-%
-%  A description of each parameter follows:
-%
-%    o status:  Method ExecutePostscriptInterpreter returns True is the command
-%      is successfully executed, otherwise False.
-%
-%    o verbose: A value other than zero displays the command prior to
-%      executing it.
-%
-%    o command: The address of a character string containing the command to
-%      execute.
-%
-%
-*/
-static unsigned int ExecutePostscriptInterpreter(const unsigned int verbose,
-  const char *command)
-{
-  char
-    **argv;
-
-  gs_main_instance
-    *interpreter;
-
-  int
-    argc,
-    code,
-    status;
-
-  register int
-    i;
-
-  if (verbose)
-    (void) fputs(command,stdout);
-  status=gsapi_new_instance(&interpreter,(void *) NULL);
-  if (status < 0)
-    return(False);
-  argv=StringToArgv(command,&argc);
-  status=gsapi_init_with_args(interpreter,argc-1,argv+1);
-  if (status == 0)
-    status=gsapi_run_string(interpreter,"systemdict /start get exec\n",0,&code);
-  gsapi_exit(interpreter);
-  gsapi_delete_instance(interpreter);
-  for (i=0; i < argc; i++)
-    LiberateMemory((void **) &argv[i]);
-  LiberateMemory((void **) &argv);
-  if ((status == 0) || (status == e_Quit))
-    return(False);
-  return(True);
-}
-#else
-static unsigned int ExecutePostscriptInterpreter(const unsigned int verbose,
-  const char *command)
-{
-  return(SystemCommand(verbose,command));
-}
-#endif
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -171,10 +101,12 @@ static unsigned int IsPS(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
     return(False);
-  if (LocaleNCompare((char *) magick,"%!",2) == 0)
+  if (memcmp(magick,"%!",2) == 0)
     return(True);
   if (memcmp(magick,"\004%!",3) == 0)
     return(True);
+  if (memcmp(magick,"\305\320\323\306",4) == 0)
+    return(True);
   return(False);
 }
 
@@ -212,14 +144,16 @@ static unsigned int IsPS(const unsigned char *magick,const size_t length)
 static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define BoundingBox  "%%BoundingBox:"
+#define BeginDocument  "%%BeginDocument:"
 #define DocumentMedia  "%%DocumentMedia:"
+#define EndDocument  "%%EndDocument:"
 #define PageBoundingBox  "%%PageBoundingBox:"
 #define PostscriptLevel  "%!PS-"
 #define RenderPostscriptText  "  Rendering postscript...  "
 
   char
-    density[MaxTextExtent],
     command[MaxTextExtent],
+    density[MaxTextExtent],
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
     options[MaxTextExtent],
@@ -236,6 +170,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   FILE
     *file;
 
+  GeometryInfo
+    geometry_info;
+
   Image
     *image,
     *next_image;
@@ -244,8 +181,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *clone_info;
 
   int
-    c,
-    status;
+    c;
 
   RectangleInfo
     page;
@@ -262,13 +198,15 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     count;
 
+  unsigned int
+    flags,
+    skip,
+    status;
+
   unsigned long
     height,
     width;
 
-  off_t
-    filesize;
-
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
@@ -289,16 +227,16 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Open temporary output file.
   */
   TemporaryFilename(postscript_filename);
-  file=fopen(postscript_filename,WriteBinaryType);
+  file=fopen(postscript_filename,"wb");
   if (file == (FILE *) NULL)
-    ThrowReaderException(FileOpenError,"Unable to write file",image);
+    ThrowReaderException(FileOpenError,"UnableToWriteFile",image);
   FormatString(translate_geometry,"%g %g translate\n              ",0.0,0.0);
   (void) fputs(translate_geometry,file);
   /*
@@ -308,9 +246,10 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   dy_resolution=72.0;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
-      (void) strcpy(density,PSDensityGeometry);
-      count=sscanf(density,"%lfx%lf",&image->x_resolution,&image->y_resolution);
-      if (count != 2)
+      flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+      image->x_resolution=geometry_info.rho;
+      image->y_resolution=geometry_info.sigma;
+      if (!(flags & SigmaValue))
         image->y_resolution=image->x_resolution;
     }
   FormatString(density,"%gx%g",image->x_resolution,image->y_resolution);
@@ -318,7 +257,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Determine page geometry from the Postscript bounding box.
   */
   memset(&page,0,sizeof(RectangleInfo));
-  filesize=0;
   if (LocaleCompare(image_info->magick,"EPT") == 0)
     {
       /*
@@ -326,25 +264,35 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       (void) ReadBlobLSBLong(image);
       count=ReadBlobLSBLong(image);
-      filesize=(off_t) ReadBlobLSBLong(image);
+      (void) ReadBlobLSBLong(image);  /* filesize */
       for (i=0; i < (long) (count-12); i++)
         (void) ReadBlobByte(image);
     }
-  p=command;
-  for (i=0; (LocaleCompare(image_info->magick,"EPT") != 0) ||
-    (i < (long) filesize); i++)
+  skip=False;
+  for (p=command; (c=ReadBlobByte(image)) != EOF; )
   {
-    c=ReadBlobByte(image);
-    if (c == EOF)
-      break;
     (void) fputc(c,file);
+    if (image_info->page != (char *) NULL)
+      continue;
+    /*
+      Note document structuring comments.
+    */
     *p++=c;
     if ((c != '\n') && (c != '\r') && ((p-command) < (MaxTextExtent-1)))
       continue;
     *p='\0';
     p=command;
     /*
-      Parse a bounding box statement.
+      Skip %%BeginDocument thru %%EndDocument.
+    */
+    if (LocaleNCompare(BeginDocument,command,strlen(BeginDocument)) == 0)
+      skip=True;
+    if (LocaleNCompare(EndDocument,command,strlen(EndDocument)) == 0)
+      skip=False;
+    if (skip)
+      continue;
+    /*
+      Note region defined by bounding box.
     */
     count=0;
     if (LocaleNCompare(BoundingBox,command,strlen(BoundingBox)) == 0)
@@ -364,44 +312,36 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Set Postscript render geometry.
     */
     FormatString(translate_geometry,"%g %g translate\n",-bounds.x1,-bounds.y1);
-    width=(unsigned int) (bounds.x2-bounds.x1);
-    if ((float) ((int) bounds.x2) != bounds.x2)
-      width++;
-    height=(unsigned int) (bounds.y2-bounds.y1);
-    if ((float) ((int) bounds.y2) != bounds.y2)
-      height++;
+    width=(unsigned long) floor(bounds.x2-bounds.x1+0.5);
+    height=(unsigned long) floor(bounds.y2-bounds.y1+0.5);
     if (width > page.width)
       page.width=width;
     if (height > page.height)
       page.height=height;
   }
+  /*
+    Render postscript with the Ghostscript delegate.
+  */
   if ((page.width == 0) || (page.height == 0))
-    {
-      SetGeometry(image,&page);
-      (void) GetGeometry(PSPageGeometry,&page.x,&page.y,&page.width,
-        &page.height);
-    }
+    (void) ParsePageGeometry(image,PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
-    (void) GetGeometry(image_info->page,&page.x,&page.y,&page.width,
-      &page.height);
-  FormatString(geometry,"%lux%lu",
-    (unsigned long) ceil(page.width*image->x_resolution/dx_resolution-0.5),
-    (unsigned long) ceil(page.height*image->y_resolution/dy_resolution-0.5));
+    (void) ParseAbsoluteGeometry(image_info->page,&page);
+  page.width=(unsigned long)
+    ceil(page.width*image->x_resolution/dx_resolution-0.5);
+  page.height=(unsigned long)
+    ceil(page.height*image->y_resolution/dy_resolution-0.5);
+  FormatString(geometry,"%lux%lu",page.width,page.height);
   if (ferror(file))
     {
       (void) fclose(file);
-      ThrowReaderException(FileOpenError,
-        "An error has occurred writing to file",image)
+      ThrowReaderException(CorruptImageError,"AnErrorHasOccurredWritingToFile",
+        image)
     }
   (void) rewind(file);
   (void) fputs(translate_geometry,file);
   (void) fclose(file);
   CloseBlob(image);
-  filesize=GetBlobSize(image);
   DestroyImage(image);
-  /*
-    Use Ghostscript to convert Postscript image.
-  */
   *options='\0';
   if (image_info->subrange != 0)
     FormatString(options,"-dFirstPage=%lu -dLastPage=%lu",
@@ -411,23 +351,23 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   FormatString(command,delegate_info->commands,image_info->antialias ? 4 : 1,
     image_info->antialias ? 4 : 1,geometry,density,options,image_info->filename,
     postscript_filename);
-  MagickMonitor(RenderPostscriptText,0,8);
-  status=ExecutePostscriptInterpreter(image_info->verbose,command);
+  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
+  status=InvokePostscriptDelegate(image_info->verbose,command);
   if (!IsAccessible(image_info->filename))
     {
       /*
         Ghostscript requires a showpage operator.
       */
-      file=fopen(postscript_filename,AppendBinaryType);
+      file=fopen(postscript_filename,"ab");
       if (file == (FILE *) NULL)
-        ThrowReaderException(FileOpenError,"Unable to write file",image);
+        ThrowReaderException(FileOpenError,"UnableToWriteFile",image);
       (void) fputs("showpage\n",file);
       (void) fclose(file);
-      status=ExecutePostscriptInterpreter(image_info->verbose,command);
+      status=InvokePostscriptDelegate(image_info->verbose,command);
     }
   (void) remove(postscript_filename);
-  MagickMonitor(RenderPostscriptText,7,8);
-  if (status)
+  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
+  if (!IsAccessible(image_info->filename))
     {
       /*
         Ghostscript has failed-- try the Display Postscript Extension.
@@ -436,8 +376,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=ReadImage(image_info,exception);
       if (image != (Image *) NULL)
         return(image);
-      ThrowReaderException(CorruptImageError,"Postscript delegate failed",
-        image)
+      ThrowReaderException(DelegateError,"PostscriptDelegateFailed",image)
     }
   clone_info=CloneImageInfo(image_info);
   clone_info->blob=(void *) NULL;
@@ -446,13 +385,13 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   DestroyImageInfo(clone_info);
   (void) remove(image_info->filename);
   if (image == (Image *) NULL)
-    ThrowReaderException(CorruptImageError,"Postscript delegate failed",
-      image);
+    ThrowReaderException(DelegateError,"PostscriptDelegateFailed",image);
   do
   {
     (void) strcpy(image->magick,"PS");
     (void) strncpy(image->filename,filename,MaxTextExtent-1);
-    next_image=image->next;
+    image->page=page;
+    next_image=SyncNextImageInList(image);
     if (next_image != (Image *) NULL)
       image=next_image;
   } while (next_image != (Image *) NULL);
@@ -490,43 +429,43 @@ ModuleExport void RegisterPSImage(void)
     *entry;
 
   entry=SetMagickInfo("EPI");
-  entry->decoder=ReadPSImage;
-  entry->encoder=WritePSImage;
-  entry->magick=IsPS;
+  entry->decoder=(DecoderHandler) ReadPSImage;
+  entry->encoder=(EncoderHandler) WritePSImage;
+  entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
   entry->description=
     AcquireString("Adobe Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EPS");
-  entry->decoder=ReadPSImage;
-  entry->encoder=WritePSImage;
-  entry->magick=IsPS;
+  entry->decoder=(DecoderHandler) ReadPSImage;
+  entry->encoder=(EncoderHandler) WritePSImage;
+  entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
   entry->description=AcquireString("Adobe Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EPSF");
-  entry->decoder=ReadPSImage;
-  entry->encoder=WritePSImage;
-  entry->magick=IsPS;
+  entry->decoder=(DecoderHandler) ReadPSImage;
+  entry->encoder=(EncoderHandler) WritePSImage;
+  entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
   entry->description=AcquireString("Adobe Encapsulated PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EPSI");
-  entry->decoder=ReadPSImage;
-  entry->encoder=WritePSImage;
-  entry->magick=IsPS;
+  entry->decoder=(DecoderHandler) ReadPSImage;
+  entry->encoder=(EncoderHandler) WritePSImage;
+  entry->magick=(MagickHandler) IsPS;
   entry->adjoin=False;
   entry->description=
     AcquireString("Adobe Encapsulated PostScript Interchange format");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS");
-  entry->decoder=ReadPSImage;
-  entry->encoder=WritePSImage;
-  entry->magick=IsPS;
+  entry->decoder=(DecoderHandler) ReadPSImage;
+  entry->encoder=(EncoderHandler) WritePSImage;
+  entry->magick=(MagickHandler) IsPS;
   entry->description=AcquireString("Adobe PostScript");
   entry->module=AcquireString("PS");
   (void) RegisterMagickInfo(entry);
@@ -597,15 +536,15 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
 #define WriteRunlengthPacket(image,pixel,length,p) \
 { \
   if (image->matte && (p->opacity == TransparentOpacity)) \
-    FormatString(buffer,"ffffff%02x",(unsigned int) Min(length,0xff)); \
+    FormatString(buffer,"ffffff%02lX",Min(length,0xff)); \
   else \
-    FormatString(buffer,"%02x%02x%02x%02x",Downscale(pixel.red), \
-      Downscale(pixel.green),Downscale(pixel.blue),(unsigned int) \
+    FormatString(buffer,"%02X%02X%02X%02lX",ScaleQuantumToChar(pixel.red), \
+      ScaleQuantumToChar(pixel.green),ScaleQuantumToChar(pixel.blue), \
       Min(length,0xff)); \
   (void) WriteBlobString(image,buffer); \
 }
 
-  static const char 
+  static const char
     *PostscriptProlog[]=
     {
       "%%BeginProlog",
@@ -866,7 +805,6 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
   char
     buffer[MaxTextExtent],
     date[MaxTextExtent],
-    density[MaxTextExtent],
     **labels,
     page_geometry[MaxTextExtent];
 
@@ -884,6 +822,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
     y_resolution,
     y_scale;
 
+  GeometryInfo
+    geometry_info;
+
   IndexPacket
     index;
 
@@ -916,15 +857,18 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
   time_t
     timer;
 
-  unsigned int
+  unsigned char
     bit,
-    byte,
-    polarity,
+    byte;
+
+  unsigned int
+    flags,
     status;
 
   unsigned long
     count,
     page,
+    polarity,
     scene,
     text_size;
 
@@ -936,9 +880,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   page=1;
   scene=0;
   do
@@ -946,7 +890,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
     /*
       Scale image to size of Postscript page.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     text_size=0;
     attribute=GetImageAttribute(image,"label");
     if (attribute != (const ImageAttribute *) NULL)
@@ -964,7 +908,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
       else
         if (LocaleCompare(image_info->magick,"PS") == 0)
           (void) strcpy(page_geometry,PSPageGeometry);
-    (void) GetMagickGeometry(page_geometry,&geometry.x,&geometry.y,
+    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     /*
       Scale relative to dots-per-inch.
@@ -972,20 +916,24 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
     dx_resolution=72.0;
     dy_resolution=72.0;
     x_resolution=72.0;
-    (void) strcpy(density,PSDensityGeometry);
-    count=sscanf(density,"%lfx%lf",&x_resolution,&y_resolution);
-    if (count != 2)
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    x_resolution=geometry_info.rho;
+    y_resolution=geometry_info.sigma;
+    if (!(flags & SigmaValue))
       y_resolution=x_resolution;
     if (image_info->density != (char *) NULL)
       {
-        count=sscanf(image_info->density,"%lfx%lf",&x_resolution,&y_resolution);
-        if (count != 2)
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        x_resolution=geometry_info.rho;
+        y_resolution=geometry_info.sigma;
+        if (!(flags & SigmaValue))
           y_resolution=x_resolution;
       }
     x_scale=(geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
     y_scale=(geometry.height*dy_resolution)/y_resolution;
     geometry.height=(unsigned long) (y_scale+0.5);
+    count=0;
     if (page == 1)
       {
         /*
@@ -1032,7 +980,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             FormatString(buffer,"%%%%Pages: %lu\n", image_info->adjoin ?
-              (unsigned long) GetImageListSize(image) : 1L);
+              (unsigned long) GetImageListLength(image) : 1L);
             (void) WriteBlobString(image,buffer);
           }
         (void) WriteBlobString(image,"%%EndComments\n");
@@ -1056,19 +1004,17 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             */
             preview_image=CloneImage(image,0,0,True,&image->exception);
             if (preview_image == (Image *) NULL)
-              ThrowWriterException(ResourceLimitError,
-                "Memory allocation failed",image);
+              ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+                image);
             /*
               Dump image as bitmap.
             */
-            if ((preview_image->storage_class == DirectClass) ||
-                !IsMonochromeImage(preview_image,&preview_image->exception))
-              SetImageType(preview_image,BilevelType);
-            polarity=(Quantum)
-              Intensity(&preview_image->colormap[0]) < (0.5*MaxRGB);
+            SetImageType(preview_image,BilevelType);
+            polarity=
+              PixelIntensityToQuantum(&preview_image->colormap[0]) < (MaxRGB/2);
             if (preview_image->colors == 2)
-              polarity=Intensity(&preview_image->colormap[0]) >
-                Intensity(&preview_image->colormap[1]);
+              polarity=PixelIntensityToQuantum(&preview_image->colormap[0]) >
+                PixelIntensityToQuantum(&preview_image->colormap[1]);
             FormatString(buffer,"%%%%BeginPreview: %lu %lu %lu %lu\n%%  ",
               preview_image->columns,preview_image->rows,1L,
               (((preview_image->columns+7) >> 3)*preview_image->rows+35)/36);
@@ -1091,7 +1037,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 bit++;
                 if (bit == 8)
                   {
-                    FormatString(buffer,"%02x",byte & 0xff);
+                    FormatString(buffer,"%02X",byte & 0xff);
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1106,7 +1052,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               if (bit != 0)
                 {
                   byte<<=(8-bit);
-                  FormatString(buffer,"%02x",byte & 0xff);
+                  FormatString(buffer,"%02X",byte & 0xff);
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1124,7 +1070,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
         */
         for (q=PostscriptProlog; *q; q++)
         {
-          FormatString(buffer,"%.255s\n",*q);
+          FormatString(buffer,"%.1024s\n",*q);
           (void) WriteBlobString(image,buffer);
         }
         attribute=GetImageAttribute(image,"label");
@@ -1139,7 +1085,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
           }
         for (q=PostscriptEpilog; *q; q++)
         {
-          FormatString(buffer,"%.255s\n",*q);
+          FormatString(buffer,"%.1024s\n",*q);
           (void) WriteBlobString(image,buffer);
         }
         if (LocaleCompare(image_info->magick,"PS") == 0)
@@ -1212,7 +1158,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                FormatString(buffer,"%02x",Downscale(Intensity(p)));
+                FormatString(buffer,"%02X",
+                  ScaleQuantumToChar(PixelIntensityToQuantum(p)));
                 (void) WriteBlobString(image,buffer);
                 i++;
                 if (i == 36)
@@ -1224,7 +1171,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               }
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(SaveImageText,y,image->rows);
+                  {
+                    status=MagickMonitor(SaveImageTag,y,image->rows,
+                      &image->exception);
+                    if (status == False)
+                      break;
+                  }
             }
           }
         else
@@ -1235,13 +1187,11 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             /*
               Dump image as bitmap.
             */
-            if ((image->storage_class == DirectClass) ||
-                !IsMonochromeImage(image,&image->exception))
-              SetImageType(image,BilevelType);
-            polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+            SetImageType(image,BilevelType);
+            polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
             if (image->colors == 2)
-              polarity=Intensity(&image->colormap[1]) >
-                Intensity(&image->colormap[0]);
+              polarity=PixelIntensityToQuantum(&image->colormap[1]) <
+                PixelIntensityToQuantum(&image->colormap[0]);
             count=0;
             for (y=0; y < (long) image->rows; y++)
             {
@@ -1255,12 +1205,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               for (x=0; x < (long) image->columns; x++)
               {
                 byte<<=1;
-                if (indexes[x] == polarity)
+                if (indexes[x] != polarity)
                   byte|=0x01;
                 bit++;
                 if (bit == 8)
                   {
-                    FormatString(buffer,"%02x",byte & 0xff);
+                    FormatString(buffer,"%02X",byte & 0xff);
                     (void) WriteBlobString(image,buffer);
                     count++;
                     if (count == 36)
@@ -1276,7 +1226,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
               if (bit != 0)
                 {
                   byte<<=(8-bit);
-                  FormatString(buffer,"%02x",byte & 0xff);
+                  FormatString(buffer,"%02X",byte & 0xff);
                   (void) WriteBlobString(image,buffer);
                   count++;
                   if (count == 36)
@@ -1287,7 +1237,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 };
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(SaveImageText,y,image->rows);
+                  {
+                    status=MagickMonitor(SaveImageTag,y,image->rows,
+                      &image->exception);
+                    if (status == False)
+                      break;
+                  }
             }
           }
         if (count != 0)
@@ -1300,11 +1255,11 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
             Dump DirectClass image.
           */
           FormatString(buffer,"%lu %lu\n0\n%d\n",image->columns,image->rows,
-            (int) (image_info->compression == RunlengthEncodedCompression));
+            (int) (image_info->compression == RLECompression));
           (void) WriteBlobString(image,buffer);
           switch (image_info->compression)
           {
-            case RunlengthEncodedCompression:
+            case RLECompression:
             {
               /*
                 Dump runlength-encoded DirectColor packets.
@@ -1343,7 +1298,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 WriteRunlengthPacket(image,pixel,length,p);
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               break;
             }
@@ -1365,8 +1325,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   if (image->matte && (p->opacity == TransparentOpacity))
                     (void) strcpy(buffer,"ffffff");
                   else
-                    FormatString(buffer,"%02x%02x%02x",
-                      Downscale(p->red),Downscale(p->green),Downscale(p->blue));
+                    FormatString(buffer,"%02X%02X%02X",
+                      ScaleQuantumToChar(p->red),ScaleQuantumToChar(p->green),
+                      ScaleQuantumToChar(p->blue));
                   (void) WriteBlobString(image,buffer);
                   i++;
                   if (i == 12)
@@ -1378,7 +1339,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 }
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               break;
             }
@@ -1392,7 +1358,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
           */
           FormatString(buffer,"%lu %lu\n%d\n%d\n0\n",image->columns,image->rows,
             (int) (image->storage_class == PseudoClass),
-            (int) (image_info->compression == RunlengthEncodedCompression));
+            (int) (image_info->compression == RLECompression));
           (void) WriteBlobString(image,buffer);
           /*
             Dump number of colors and colormap.
@@ -1401,15 +1367,15 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobString(image,buffer);
           for (i=0; i < (long) image->colors; i++)
           {
-            FormatString(buffer,"%02x%02x%02x\n",
-              Downscale(image->colormap[i].red),
-              Downscale(image->colormap[i].green),
-              Downscale(image->colormap[i].blue));
+            FormatString(buffer,"%02X%02X%02X\n",
+              ScaleQuantumToChar(image->colormap[i].red),
+              ScaleQuantumToChar(image->colormap[i].green),
+              ScaleQuantumToChar(image->colormap[i].blue));
             (void) WriteBlobString(image,buffer);
           }
           switch (image_info->compression)
           {
-            case RunlengthEncodedCompression:
+            case RLECompression:
             {
               /*
                 Dump runlength-encoded PseudoColor packets.
@@ -1433,8 +1399,8 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                     {
                       if (x > 0)
                         {
-                          FormatString(buffer,"%02x%02x",(unsigned int)
-                            index,(unsigned int) Min(length,0xff));
+                          FormatString(buffer,"%02X%02lX",index,
+                            Min(length,0xff));
                           (void) WriteBlobString(image,buffer);
                           i++;
                           if (i == 18)
@@ -1449,12 +1415,16 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   pixel=(*p);
                   p++;
                 }
-                FormatString(buffer,"%02x%02x",(unsigned int) index,
-                  (unsigned int) Min(length,0xff));
+                FormatString(buffer,"%02X%02lX",index,Min(length,0xff));
                 (void) WriteBlobString(image,buffer);
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               break;
             }
@@ -1474,7 +1444,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 indexes=GetIndexes(image);
                 for (x=0; x < (long) image->columns; x++)
                 {
-                  FormatString(buffer,"%02x",indexes[x]);
+                  FormatString(buffer,"%02X",indexes[x]);
                   (void) WriteBlobString(image,buffer);
                   i++;
                   if (i == 36)
@@ -1486,7 +1456,12 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 }
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               break;
             }
@@ -1498,8 +1473,11 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"%%PageTrailer\n");
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
