@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "compress.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 #if defined(HasTIFF)
 #define CCITTParam  "-1"
 #else
@@ -128,8 +136,10 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
     *buffer;
 
   unsigned int
+    status;
+
+  unsigned long
     *byte_count,
-    status,
     strip_size;
 
   /*
@@ -142,9 +152,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   huffman_image=CloneImage(image,0,0,True,&image->exception);
   if (huffman_image == (Image *) NULL)
     return(False);
-  if ((huffman_image->storage_class == DirectClass) ||
-       !IsMonochromeImage(huffman_image,&image->exception))
-    SetImageType(huffman_image,BilevelType);
+  SetImageType(huffman_image,BilevelType);
   TemporaryFilename(filename);
   FormatString(huffman_image->filename,"tiff:%s",filename);
   clone_info=CloneImageInfo(image_info);
@@ -154,11 +162,11 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   DestroyImage(huffman_image);
   if (status == False)
     return(False);
-  tiff=TIFFOpen(filename,ReadBinaryType);
+  tiff=TIFFOpen(filename,"rb");
   if (tiff == (TIFF *) NULL)
     {
       (void) remove(filename);
-      ThrowBinaryException(FileOpenError,"Unable to open file",
+      ThrowBinaryException(FileOpenError,"UnableToOpenFile",
         image_info->filename)
     }
   /*
@@ -174,7 +182,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
     {
       TIFFClose(tiff);
       (void) remove(filename);
-      ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         (char *) NULL)
     }
   /*
@@ -188,7 +196,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
     if (fillorder == FILLORDER_LSB2MSB)
       TIFFReverseBits(buffer,count);
     for (j=0; j < count; j++)
-      Ascii85Encode(image,(unsigned int) buffer[j]);
+      Ascii85Encode(image,(unsigned long) buffer[j]);
     Ascii85Flush(image);
   }
   LiberateMemory((void **) &buffer);
@@ -202,8 +210,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  ThrowBinaryException(MissingDelegateError,"TIFF library is not available",
-    image->filename);
+  ThrowBinaryException(CoderError,"TIFFLibraryIsNotAvailable",image->filename);
 }
 #endif
 
@@ -236,13 +243,15 @@ ModuleExport void RegisterPS2Image(void)
     *entry;
 
   entry=SetMagickInfo("EPS2");
-  entry->encoder=WritePS2Image;
+  entry->encoder=(EncoderHandler) WritePS2Image;
   entry->adjoin=False;
+  entry->seekable_stream=True;
   entry->description=AcquireString("Adobe Level II Encapsulated PostScript");
   entry->module=AcquireString("PS2");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS2");
-  entry->encoder=WritePS2Image;
+  entry->encoder=(EncoderHandler) WritePS2Image;
+  entry->seekable_stream=True;
   entry->description=AcquireString("Adobe Level II PostScript");
   (void) RegisterMagickInfo(entry);
 }
@@ -395,6 +404,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       "    %%",
       "    /colormap colors 3 mul string def",
       "    currentfile colormap readhexstring pop pop",
+      "    currentfile buffer readline pop",
       "    [ /Indexed /DeviceRGB colors 1 sub colormap ] setcolorspace",
       "    <<",
       "      /ImageType 1",
@@ -423,7 +433,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       "  %%   image columns & rows.",
       "  %%   class: 0-DirectClass or 1-PseudoClass.",
       "  %%   colorspace: 0-RGB or 1-CMYK.",
-      "  %%   compression: 0-RunlengthEncodedCompression or 1-NoCompression.",
+      "  %%   compression: 0-RLECompression or 1-NoCompression.",
       "  %%   hex color packets.",
       "  %%",
       "  gsave",
@@ -462,7 +472,6 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
   char
     buffer[MaxTextExtent],
     date[MaxTextExtent],
-    density[MaxTextExtent],
     page_geometry[MaxTextExtent],
     **labels;
 
@@ -483,18 +492,24 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     y_resolution,
     y_scale;
 
+  ExtendedSignedIntegralType
+    number_pixels,
+    start,
+    stop;
+
+  GeometryInfo
+    geometry_info;
+
+  Image
+    *jpeg_image;
+
   int
-    count,
     status;
 
   long
     j,
     y;
 
-  off_t
-    start,
-    stop;
-
   RectangleInfo
     geometry;
 
@@ -522,12 +537,17 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
   unsigned char
     *pixels;
 
+  unsigned int
+    flags;
+
   unsigned long
-    number_pixels,
     page,
     scene,
     text_size;
 
+  void
+    *blob;
+
   /*
     Open output image file.
   */
@@ -535,9 +555,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   compression=image->compression;
   if (image_info->compression != UndefinedCompression)
     compression=image_info->compression;
@@ -546,27 +566,27 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
 #if !defined(HasJPEG)
     case JPEGCompression:
     {
-      compression=RunlengthEncodedCompression;
+      compression=RLECompression;
       ThrowException(&image->exception,MissingDelegateError,
-        "JPEG compression is not available",image->filename);
+        "JPEGLibraryIsNotAvailable",image->filename);
       break;
     }
 #endif
 #if !defined(HasLZW)
     case LZWCompression:
     {
-      compression=RunlengthEncodedCompression;
+      compression=RLECompression;
       ThrowException(&image->exception,MissingDelegateError,
-        "LZW compression is not available",image->filename);
+        "LZWEncodingNotEnabled",image->filename);
       break;
     }
 #endif
 #if !defined(HasZLIB)
     case ZipCompression:
     {
-      compression=RunlengthEncodedCompression;
+      compression=RLECompression;
       ThrowException(&image->exception,MissingDelegateError,
-        "ZLIB compression is not available",image->filename);
+        "ZipLibraryIsNotAvailable",image->filename);
       break;
     }
 #endif
@@ -597,7 +617,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       else
         if (LocaleCompare(image_info->magick,"PS2") == 0)
           (void) strcpy(page_geometry,PSPageGeometry);
-    (void) GetMagickGeometry(page_geometry,&geometry.x,&geometry.y,
+    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
        &geometry.width,&geometry.height);
     /*
       Scale relative to dots-per-inch.
@@ -605,16 +625,18 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     dx_resolution=72.0;
     dy_resolution=72.0;
     x_resolution=72.0;
-    (void) strcpy(density,PSDensityGeometry);
-    count=sscanf(density,"%lfx%lf",&x_resolution,&y_resolution);
-    if (count != 2)
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    x_resolution=geometry_info.rho;
+    y_resolution=geometry_info.sigma;
+    if (!(flags & SigmaValue))
       y_resolution=x_resolution;
     if (image_info->density != (char *) NULL)
       {
-        count=sscanf(image_info->density,"%lfx%lf",&x_resolution,
-          &y_resolution);
-        if (count != 2)
-          y_resolution=x_resolution;
+        flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+        image->x_resolution=geometry_info.rho;
+        image->y_resolution=geometry_info.sigma;
+        if (!(flags & SigmaValue))
+          image->y_resolution=image->x_resolution;
       }
     x_scale=(geometry.width*dx_resolution)/x_resolution;
     geometry.width=(unsigned long) (x_scale+0.5);
@@ -665,7 +687,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               (void) strcpy(buffer,"%%Pages: 1\n");
             else
               FormatString(buffer,"%%%%Pages: %lu\n",(unsigned long)
-                GetImageListSize(image));
+                GetImageListLength(image));
             (void) WriteBlobString(image,buffer);
           }
         (void) WriteBlobString(image,"%%EndComments\n");
@@ -699,7 +721,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           }
         for (q=PostscriptEpilog; *q; q++)
           {
-            FormatString(buffer,"%.255s\n",*q);
+            FormatString(buffer,"%.1024s\n",*q);
             (void) WriteBlobString(image,buffer);
           }
         if (LocaleCompare(image_info->magick,"PS2") == 0)
@@ -728,8 +750,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,"userdict begin\n");
     start=TellBlob(image);
     FormatString(buffer,"%%%%BeginData:%13ld %s Bytes\n",0L,
-      compression == NoCompression ? "ASCII" : "BINARY");
+      compression == NoCompression ? "ASCII" : "Binary");
     (void) WriteBlobString(image,buffer);
+    stop=TellBlob(image);
     (void) WriteBlobString(image,"DisplayImage\n");
     /*
       Output image data.
@@ -751,7 +774,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
         }
         LiberateMemory((void **) &labels);
       }
-    number_pixels=image->columns*image->rows;
+    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+    if (number_pixels != (size_t) number_pixels)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     if ((compression == FaxCompression) ||
         ((image_info->type != TrueColorType) &&
          IsGrayImage(image,&image->exception)))
@@ -778,28 +803,20 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           }
           case JPEGCompression:
           {
-            Image
-              *jpeg_image;
-
-            size_t
-              length;
-
-            void
-              *blob;
-
             /*
               Write image in JPEG format.
             */
             jpeg_image=CloneImage(image,0,0,True,&image->exception);
             if (jpeg_image == (Image *) NULL)
-              ThrowWriterException(DelegateError,"Unable to clone image",image);
+              ThrowWriterException(CoderError,image->exception.reason,image);
+            (void) strcpy(jpeg_image->magick,"JPEG");
             blob=ImageToBlob(image_info,jpeg_image,&length,&image->exception);
             (void) WriteBlob(image,length,blob);
             DestroyImage(jpeg_image);
             LiberateMemory((void **) &blob);
             break;
           }
-          case RunlengthEncodedCompression:
+          case RLECompression:
           default:
           {
             register unsigned char
@@ -811,8 +828,8 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
             length=number_pixels;
             pixels=(unsigned char *) AcquireMemory(length);
             if (pixels == (unsigned char *) NULL)
-              ThrowWriterException(ResourceLimitError,
-                "Memory allocation failed",image);
+              ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+                image);
             /*
               Dump Runlength encoded pixels.
             */
@@ -825,12 +842,17 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                *q++=Downscale(Intensity(p));
+                *q++=ScaleQuantumToChar(PixelIntensityToQuantum(p));
                 p++;
               }
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(SaveImageText,y,image->rows);
+                  {
+                    status=MagickMonitor(SaveImageTag,y,image->rows,
+                      &image->exception);
+                    if (status == False)
+                      break;
+                  }
             }
             if (compression == LZWCompression)
               status=LZWEncodeImage(image,length,pixels);
@@ -858,12 +880,18 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                Ascii85Encode(image,Downscale(Intensity(p)));
+                Ascii85Encode(image,
+                  ScaleQuantumToChar(PixelIntensityToQuantum(p)));
                 p++;
               }
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(SaveImageText,y,image->rows);
+                  {
+                    status=MagickMonitor(SaveImageTag,y,image->rows,
+                      &image->exception);
+                    if (status == False)
+                      break;
+                  }
             }
             Ascii85Flush(image);
             break;
@@ -871,7 +899,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
         }
       }
     else
-      if ((image->storage_class == DirectClass) ||
+      if ((image->storage_class == DirectClass) || (image->colors > 256) ||
           (compression == JPEGCompression))
         {
           FormatString(buffer,"%lu %lu\n0\n%d\n",image->columns,image->rows,
@@ -883,29 +911,20 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           {
             case JPEGCompression:
             {
-              Image
-                *jpeg_image;
-
-              size_t
-                length;
-
-              void
-                *blob;
-
               /*
                 Write image in JPEG format.
               */
               jpeg_image=CloneImage(image,0,0,True,&image->exception);
               if (jpeg_image == (Image *) NULL)
-                ThrowWriterException(DelegateError,"Unable to clone image",
-                  image);
+                ThrowWriterException(CoderError,image->exception.reason,image);
+              (void) strcpy(jpeg_image->magick,"JPEG");
               blob=ImageToBlob(image_info,jpeg_image,&length,&image->exception);
               (void) WriteBlob(image,length,blob);
               DestroyImage(jpeg_image);
               LiberateMemory((void **) &blob);
               break;
             }
-            case RunlengthEncodedCompression:
+            case RLECompression:
             default:
             {
               register unsigned char
@@ -919,7 +938,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               pixels=(unsigned char *) AcquireMemory(length);
               if (pixels == (unsigned char *) NULL)
                 ThrowWriterException(ResourceLimitError,
-                  "Memory allocation failed",image);
+                  "MemoryAllocationFailed",image);
               /*
                 Dump Packbit encoded pixels.
               */
@@ -934,29 +953,34 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 {
                   if (image->matte && (p->opacity == TransparentOpacity))
                     {
-                      *q++=Downscale(MaxRGB);
-                      *q++=Downscale(MaxRGB);
-                      *q++=Downscale(MaxRGB);
+                      *q++=ScaleQuantumToChar(MaxRGB);
+                      *q++=ScaleQuantumToChar(MaxRGB);
+                      *q++=ScaleQuantumToChar(MaxRGB);
                     }
                   else
                     if (image->colorspace != CMYKColorspace)
                       {
-                        *q++=Downscale(p->red);
-                        *q++=Downscale(p->green);
-                        *q++=Downscale(p->blue);
+                        *q++=ScaleQuantumToChar(p->red);
+                        *q++=ScaleQuantumToChar(p->green);
+                        *q++=ScaleQuantumToChar(p->blue);
                       }
                     else
                       {
-                        *q++=Downscale(p->red);
-                        *q++=Downscale(p->green);
-                        *q++=Downscale(p->blue);
-                        *q++=Downscale(p->opacity);
+                        *q++=ScaleQuantumToChar(p->red);
+                        *q++=ScaleQuantumToChar(p->green);
+                        *q++=ScaleQuantumToChar(p->blue);
+                        *q++=ScaleQuantumToChar(p->opacity);
                       }
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
@@ -986,29 +1010,34 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 {
                   if (image->matte && (p->opacity == TransparentOpacity))
                     {
-                      Ascii85Encode(image,Downscale(MaxRGB));
-                      Ascii85Encode(image,Downscale(MaxRGB));
-                      Ascii85Encode(image,Downscale(MaxRGB));
+                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
+                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
+                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
                     }
                   else
                     if (image->colorspace != CMYKColorspace)
                       {
-                        Ascii85Encode(image,Downscale(p->red));
-                        Ascii85Encode(image,Downscale(p->green));
-                        Ascii85Encode(image,Downscale(p->blue));
+                        Ascii85Encode(image,ScaleQuantumToChar(p->red));
+                        Ascii85Encode(image,ScaleQuantumToChar(p->green));
+                        Ascii85Encode(image,ScaleQuantumToChar(p->blue));
                       }
                     else
                       {
-                        Ascii85Encode(image,Downscale(p->red));
-                        Ascii85Encode(image,Downscale(p->green));
-                        Ascii85Encode(image,Downscale(p->blue));
-                        Ascii85Encode(image,Downscale(p->opacity));
+                        Ascii85Encode(image,ScaleQuantumToChar(p->red));
+                        Ascii85Encode(image,ScaleQuantumToChar(p->green));
+                        Ascii85Encode(image,ScaleQuantumToChar(p->blue));
+                        Ascii85Encode(image,ScaleQuantumToChar(p->opacity));
                       }
                   p++;
                 }
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               Ascii85Flush(image);
               break;
@@ -1029,15 +1058,15 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           (void) WriteBlobString(image,buffer);
           for (i=0; i < (long) image->colors; i++)
           {
-            FormatString(buffer,"%02x%02x%02x\n",
-              Downscale(image->colormap[i].red),
-              Downscale(image->colormap[i].green),
-              Downscale(image->colormap[i].blue));
+            FormatString(buffer,"%02X%02X%02X\n",
+              ScaleQuantumToChar(image->colormap[i].red),
+              ScaleQuantumToChar(image->colormap[i].green),
+              ScaleQuantumToChar(image->colormap[i].blue));
             (void) WriteBlobString(image,buffer);
           }
           switch (compression)
           {
-            case RunlengthEncodedCompression:
+            case RLECompression:
             default:
             {
               register unsigned char
@@ -1050,7 +1079,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               pixels=(unsigned char *) AcquireMemory(length);
               if (pixels == (unsigned char *) NULL)
                 ThrowWriterException(ResourceLimitError,
-                  "Memory allocation failed",image);
+                  "MemoryAllocationFailed",image);
               /*
                 Dump Runlength encoded pixels.
               */
@@ -1066,7 +1095,12 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                   *q++=indexes[x];
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
@@ -1097,7 +1131,12 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                   Ascii85Encode(image,indexes[x]);
                 if (image->previous == (Image *) NULL)
                   if (QuantumTick(y,image->rows))
-                    MagickMonitor(SaveImageText,y,image->rows);
+                    {
+                      status=MagickMonitor(SaveImageTag,y,image->rows,
+                        &image->exception);
+                      if (status == False)
+                        break;
+                    }
               }
               Ascii85Flush(image);
               break;
@@ -1105,10 +1144,11 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
           }
         }
     (void) WriteBlobByte(image,'\n');
+    length=TellBlob(image)-stop;
     stop=TellBlob(image);
     (void) SeekBlob(image,start,SEEK_SET);
-    FormatString(buffer,"%%%%BeginData:%13ld %s Bytes\n",(long) (stop-start+1),
-      compression == NoCompression ? "ASCII" : "BINARY");
+    FormatString(buffer,"%%%%BeginData:%13ld %s Bytes\n",(long) length,
+      compression == NoCompression ? "ASCII" : "Binary");
     (void) WriteBlobString(image,buffer);
     (void) SeekBlob(image,stop,SEEK_SET);
     (void) WriteBlobString(image,"%%EndData\n");
@@ -1117,8 +1157,11 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"%%PageTrailer\n");
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
