@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,14 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "compress.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 #if defined(HasTIFF)
 #define CCITTParam  "-1"
 #else
@@ -129,8 +137,10 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
     *buffer;
 
   unsigned int
+    status;
+
+  unsigned long
     *byte_count,
-    status,
     strip_size;
 
   /*
@@ -143,9 +153,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   huffman_image=CloneImage(image,0,0,True,&image->exception);
   if (huffman_image == (Image *) NULL)
     return(False);
-  if ((huffman_image->storage_class == DirectClass) ||
-       !IsMonochromeImage(huffman_image,&image->exception))
-    SetImageType(huffman_image,BilevelType);
+  SetImageType(huffman_image,BilevelType);
   TemporaryFilename(filename);
   FormatString(huffman_image->filename,"tiff:%s",filename);
   clone_info=CloneImageInfo(image_info);
@@ -155,11 +163,11 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   DestroyImage(huffman_image);
   if (status == False)
     return(False);
-  tiff=TIFFOpen(filename,ReadBinaryType);
+  tiff=TIFFOpen(filename,"rb");
   if (tiff == (TIFF *) NULL)
     {
       (void) remove(filename);
-      ThrowBinaryException(FileOpenError,"Unable to open file",
+      ThrowBinaryException(FileOpenError,"UnableToOpenFile",
         image_info->filename)
     }
   /*
@@ -175,7 +183,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
     {
       TIFFClose(tiff);
       (void) remove(filename);
-      ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         (char *) NULL)
     }
   /*
@@ -189,7 +197,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
     if (fillorder == FILLORDER_LSB2MSB)
       TIFFReverseBits(buffer,count);
     for (j=0; j < count; j++)
-      Ascii85Encode(image,(unsigned int) buffer[j]);
+      Ascii85Encode(image,(unsigned long) buffer[j]);
     Ascii85Flush(image);
   }
   LiberateMemory((void **) &buffer);
@@ -203,8 +211,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  ThrowBinaryException(MissingDelegateError,"TIFF library is not available",
-    image->filename);
+  ThrowBinaryException(CoderError,"TIFFLibraryIsNotAvailable",image->filename)
 }
 #endif
 
@@ -237,12 +244,12 @@ ModuleExport void RegisterPS3Image(void)
     *entry;
 
   entry=SetMagickInfo("EPS3");
-  entry->encoder=WritePS3Image;
+  entry->encoder=(EncoderHandler) WritePS3Image;
   entry->description=AcquireString("Adobe Level III Encapsulated PostScript");
   entry->module=AcquireString("PS3");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS3");
-  entry->encoder=WritePS3Image;
+  entry->encoder=(EncoderHandler) WritePS3Image;
   entry->description=AcquireString("Adobe Level III PostScript");
   entry->module=AcquireString("PS3");
   (void) RegisterMagickInfo(entry);
@@ -312,7 +319,6 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   char
     buffer[MaxTextExtent],
     date[MaxTextExtent],
-    density[MaxTextExtent],
     page_geometry[MaxTextExtent];
 
   CompressionType
@@ -329,9 +335,8 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     y_resolution,
     y_scale;
 
-  int
-    count,
-    status;
+  GeometryInfo
+    geometry_info;
 
   long
     y;
@@ -348,6 +353,9 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   SegmentInfo
     bounds;
 
+  size_t
+    length;
+
   time_t
     timer;
 
@@ -355,6 +363,10 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     *pixels;
 
   unsigned int
+    flags,
+    status;
+
+  unsigned long
     page,
     scene,
     text_size;
@@ -366,9 +378,9 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   compression=image->compression;
   if (image_info->compression != UndefinedCompression)
     compression=image_info->compression;
@@ -377,27 +389,27 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
 #if !defined(HasJPEG)
     case JPEGCompression:
     {
-      compression=RunlengthEncodedCompression;
+      compression=RLECompression;
       ThrowException(&image->exception,MissingDelegateError,
-        "JPEG compression is not available",image->filename);
+        "JPEGLibraryIsNotAvailable",image->filename);
       break;
     }
 #endif
 #if !defined(HasLZW)
     case LZWCompression:
     {
-      compression=RunlengthEncodedCompression;
+      compression=RLECompression;
       ThrowException(&image->exception,MissingDelegateError,
-        "LZW compression is not available",image->filename);
+        "LZWEncodingNotEnabled",image->filename);
       break;
     }
 #endif
 #if !defined(HasZLIB)
     case ZipCompression:
     {
-      compression=RunlengthEncodedCompression;
-      ThrowException(&image->exception,MissingDelegateError,
-        "ZLIB compression is not available",image->filename);
+      compression=RLECompression;
+      ThrowException(&image->exception,CoderError,
+        "ZipLibraryIsNotAvailable",image->filename);
       break;
     }
 #endif
@@ -409,12 +421,10 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
   do
   {
     if ((compression != NoCompression) &&
-        (compression != RunlengthEncodedCompression))
+        (compression != RLECompression))
       image->storage_class=DirectClass;
     if (compression == FaxCompression)
-      if ((image->storage_class == DirectClass) ||
-          !IsMonochromeImage(image,&image->exception))
-        SetImageType(image,BilevelType);
+      SetImageType(image,BilevelType);
     /*
       Scale image to size of Postscript page.
     */
@@ -435,7 +445,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
       else
         if (LocaleCompare(image_info->magick,"PS3") == 0)
           (void) strcpy(page_geometry,PSPageGeometry);
-    (void) GetMagickGeometry(page_geometry,&geometry.x,&geometry.y,
+    (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     /*
       Scale relative to dots-per-inch.
@@ -443,15 +453,17 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     dx_resolution=72.0;
     dy_resolution=72.0;
     x_resolution=72.0;
-    (void) strcpy(density,PSDensityGeometry);
-    count=sscanf(density,"%lfx%lf",&x_resolution,&y_resolution);
-    if (count != 2)
+    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+    x_resolution=geometry_info.rho;
+    y_resolution=geometry_info.sigma;
+    if (!(flags & SigmaValue))
       y_resolution=x_resolution;
     if (image_info->density != (char *) NULL)
       {
-        count=sscanf(image_info->density,"%lfx%lf",&x_resolution,
-          &y_resolution);
-        if (count != 2)
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        x_resolution=geometry_info.rho;
+        y_resolution=geometry_info.sigma;
+        if (!(flags & SigmaValue))
           y_resolution=x_resolution;
       }
     x_scale=(geometry.width*dx_resolution)/x_resolution;
@@ -467,6 +479,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
           (void) strcpy(buffer,"%!PS-Adobe-3.0 Resource-ProcSet\n");
         else
           (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0 Resource-ProcSet\n");
+        (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"%%Creator: (ImageMagick)\n");
         FormatString(buffer,"%%%%Title: (%.1024s)\n",image->filename);
         (void) WriteBlobString(image,buffer);
@@ -502,12 +515,12 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
               (void) strcpy(buffer,"%%Pages: 1\n");
             else
               FormatString(buffer,"%%%%Pages: %lu\n",(unsigned long)
-                GetImageListSize(image));
+                GetImageListLength(image));
             (void) WriteBlobString(image,buffer);
           }
         (void) WriteBlobString(image,"%%EndComments\n");
       }
-    FormatString(buffer,"%%%%Page:  1 %u\n",page++);
+    FormatString(buffer,"%%%%Page:  1 %lu\n",page++);
     (void) WriteBlobString(image,buffer);
     FormatString(buffer,"%%%%PageBoundingBox: %ld %ld %ld %ld\n",geometry.x,
       geometry.y,geometry.x+(long) geometry.width,geometry.y+(long)
@@ -553,9 +566,6 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         Image
           *jpeg_image;
 
-        size_t
-          length;
-
         void
           *blob;
 
@@ -564,33 +574,34 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
         */
         jpeg_image=CloneImage(image,0,0,True,&image->exception);
         if (jpeg_image == (Image *) NULL)
-          ThrowWriterException(DelegateError,"Unable to clone image",image);
+          ThrowWriterException(CoderError,image->exception.reason,image);
+        (void) strcpy(jpeg_image->magick,"JPEG");
         blob=ImageToBlob(image_info,jpeg_image,&length,&image->exception);
         (void) WriteBlob(image,length,blob);
         DestroyImage(jpeg_image);
         LiberateMemory((void **) &blob);
         break;
       }
-      case RunlengthEncodedCompression:
+      case RLECompression:
       default:
       {
+        ExtendedSignedIntegralType
+          number_pixels;
+
         register unsigned char
           *q;
 
-        size_t
-          length;
-
-        unsigned long
-          number_pixels;
-
         /*
           Allocate pixel array.
         */
-        number_pixels=image->columns*image->rows;
+        number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+        if ((4*number_pixels*4) != (size_t) (4*number_pixels))
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+            image);
         length=4*number_pixels;
         pixels=(unsigned char *) AcquireMemory(length);
         if (pixels == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"Memory allocation failed",
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         /*
           Dump Packbit encoded pixels.
@@ -603,15 +614,16 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            *q++=MaxRGB-Downscale(p->opacity);
-            *q++=Downscale(p->red);
-            *q++=Downscale(p->green);
-            *q++=Downscale(p->blue);
+            *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
+            *q++=ScaleQuantumToChar(p->red);
+            *q++=ScaleQuantumToChar(p->green);
+            *q++=ScaleQuantumToChar(p->blue);
             p++;
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         if (compression == ZipCompression)
           status=ZLIBEncodeImage(image,length,image_info->quality,pixels);
@@ -649,15 +661,16 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            Ascii85Encode(image,MaxRGB-Downscale(p->opacity));
-            Ascii85Encode(image,Downscale(p->red));
-            Ascii85Encode(image,Downscale(p->green));
-            Ascii85Encode(image,Downscale(p->blue));
+            Ascii85Encode(image,ScaleQuantumToChar(MaxRGB-p->opacity));
+            Ascii85Encode(image,ScaleQuantumToChar(p->red));
+            Ascii85Encode(image,ScaleQuantumToChar(p->green));
+            Ascii85Encode(image,ScaleQuantumToChar(p->blue));
             p++;
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         Ascii85Flush(image);
         break;
@@ -685,7 +698,7 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,buffer);
     if (image->colorspace != CMYKColorspace)
       (void) WriteBlobString(image,"  /Decode [ 0 1 0 1 0 1 ] def\n");
-		else
+    else
       (void) WriteBlobString(image,"  /Decode [ 1 0 1 0 1 0 1 0 ] def\n");
     (void) WriteBlobString(image,"end\n");
     (void) WriteBlobByte(image,'\n');
@@ -717,8 +730,14 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,buffer);
     FormatString(buffer,"%g %g scale\n",x_scale,y_scale);
     (void) WriteBlobString(image,buffer);
-    (void) WriteBlobString(image,"ImageMaskDictionary /Decode [ 1 0 ] put\n");
-    (void) WriteBlobString(image,"MaskedImageDictionary image\n");
+    if (compression == JPEGCompression)
+      (void) WriteBlobString(image,"ImageDataDictionary image\n");
+    else
+      {
+        (void) WriteBlobString(image,
+          "ImageMaskDictionary /Decode [ 1 0 ] put\n");
+        (void) WriteBlobString(image,"MaskedImageDictionary image\n");
+      }
     (void) WriteBlobString(image,"grestore                    \n");
     (void) WriteBlobByte(image,'\n');
     if (LocaleCompare(image_info->magick,"PS3") == 0)
@@ -726,8 +745,11 @@ static unsigned int WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"%%EndData\n");
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
@@ -805,7 +827,7 @@ static unsigned int ZLIBEncodeImage(Image *image,const size_t length,
   compressed_packets=(unsigned long) (1.001*length+12);
   compressed_pixels=(unsigned char *) AcquireMemory(compressed_packets);
   if (compressed_pixels == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       (char *) NULL);
   stream.next_in=pixels;
   stream.avail_in=(unsigned int) length;
@@ -825,8 +847,7 @@ static unsigned int ZLIBEncodeImage(Image *image,const size_t length,
       compressed_packets=stream.total_out;
     }
   if (status)
-    ThrowBinaryException(DelegateError,"Unable to Zip compress image",
-      (char *) NULL)
+    ThrowBinaryException(CoderError,"UnableToZipCompressImage",(char *) NULL)
   else
     for (i=0; i < (long) compressed_packets; i++)
       (void) WriteBlobByte(image,compressed_pixels[i]);
@@ -839,8 +860,7 @@ static unsigned int ZLIBEncodeImage(Image *image,const size_t length,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  ThrowBinaryException(MissingDelegateError,"ZLIB library is not available",
-    image->filename);
+  ThrowBinaryException(CoderError,"ZipLibraryIsNotAvailable",image->filename);
   return(False);
 }
 #endif
