@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -168,12 +174,12 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   pwp_image=AllocateImage(image_info);
-  status=OpenBlob(image_info,pwp_image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,pwp_image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",pwp_image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",pwp_image);
   count=ReadBlob(pwp_image,5,(char *) magick);
   if ((count == 0) || (LocaleNCompare((char *) magick,"SFW95",5) != 0))
-    ThrowReaderException(CorruptImageError,"Not a PWP image file",pwp_image);
+    ThrowReaderException(CorruptImageError,"NotAPWPImageFile",pwp_image);
   clone_info=CloneImageInfo(image_info);
   clone_info->blob=(void *) NULL;
   clone_info->length=0;
@@ -192,14 +198,13 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (c == EOF)
       break;
     if (LocaleNCompare((char *) (magick+12),"SFW94A",6) != 0)
-      ThrowReaderException(CorruptImageError,"Not a PWP image file",
-        pwp_image);
+      ThrowReaderException(CorruptImageError,"NotAPWPImageFile",pwp_image);
     /*
       Dump SFW image to a temporary file.
     */
-    file=fopen(clone_info->filename,WriteBinaryType);
+    file=fopen(clone_info->filename,"wb");
     if (file == (FILE *) NULL)
-      ThrowReaderException(FileOpenError,"Unable to write file",image);
+      ThrowReaderException(FileOpenError,"UnableToWriteFile",image);
     (void) fwrite("SFW94A",1,6,file);
     filesize=65535L*magick[2]+256L*magick[1]+magick[0];
     for (i=0; i < (long) filesize; i++)
@@ -229,14 +234,18 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->subrange != 0)
       if (next_image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
-    MagickMonitor(LoadImagesText,TellBlob(pwp_image),GetBlobSize(image));
+    status=MagickMonitor(LoadImagesTag,TellBlob(pwp_image),GetBlobSize(image),
+      &image->exception);
+    if (status == False)
+      break;
   }
   (void) remove(clone_info->filename);
   DestroyImageInfo(clone_info);
   CloseBlob(pwp_image);
   DestroyImage(pwp_image);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
   return(image);
 }
@@ -270,8 +279,8 @@ ModuleExport void RegisterPWPImage(void)
     *entry;
 
   entry=SetMagickInfo("PWP");
-  entry->decoder=ReadPWPImage;
-  entry->magick=IsPWP;
+  entry->decoder=(DecoderHandler) ReadPWPImage;
+  entry->magick=(MagickHandler) IsPWP;
   entry->description=AcquireString("Seattle Film Works");
   entry->module=AcquireString("PWP");
   (void) RegisterMagickInfo(entry);
