@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,7 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
-
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
+
 /*
   Forward declarations.
 */
@@ -113,24 +119,26 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *scanline;
 
   unsigned int
-    packet_size,
     status;
 
+  unsigned long
+    packet_size;
+
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"Must specify image size",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open image file.
       */
-      status=OpenBlob(image_info,image,ReadBinaryType,exception);
+      status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == False)
-        ThrowReaderException(FileOpenError,"Unable to open file",image);
+        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
       for (i=0; i < image->offset; i++)
         (void) ReadBlobByte(image);
     }
@@ -146,7 +154,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   scanline=(unsigned char *)
     AcquireMemory(packet_size*image->tile_info.width);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->subrange != 0)
     while (image->scene < image_info->subimage)
     {
@@ -191,7 +199,8 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (i=0; i < (long) count; i++)
@@ -228,7 +237,8 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
         for (i=0; i < (long) count; i++)
@@ -247,9 +257,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (image_info->interlace == PartitionInterlace)
           {
             AppendImageFormat("R",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType,exception);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenError,"Unable to open file",image);
+              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
           }
         packet_size=image->depth > 8 ? 2 : 1;
         for (y=0; y < image->tile_info.y; y++)
@@ -268,7 +278,8 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                break;
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -278,9 +289,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             CloseBlob(image);
             AppendImageFormat("G",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType,exception);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenError,"Unable to open file",image);
+              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < image->tile_info.y; y++)
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
@@ -295,7 +306,8 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                break;
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -305,9 +317,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             CloseBlob(image);
             AppendImageFormat("B",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType,exception);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenError,"Unable to open file",image);
+              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < image->tile_info.y; y++)
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
@@ -322,7 +334,8 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                break;
           i++;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -337,10 +350,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
-                status=OpenBlob(image_info,image,ReadBinaryType,exception);
+                status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
                 if (status == False)
-                  ThrowReaderException(FileOpenError,"Unable to open file",
-                    image);
+                  ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
               }
             for (y=0; y < image->tile_info.y; y++)
               (void) ReadBlob(image,packet_size*image->tile_info.width,
@@ -357,7 +369,8 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(i,span))
-                  MagickMonitor(LoadImageText,i,span);
+                  if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+                    break;
               i++;
             }
             count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -371,7 +384,11 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     }
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -392,8 +409,11 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (count != 0);
   LiberateMemory((void **) &scanline);
@@ -432,15 +452,15 @@ ModuleExport void RegisterRGBImage(void)
     *entry;
 
   entry=SetMagickInfo("RGB");
-  entry->decoder=ReadRGBImage;
-  entry->encoder=WriteRGBImage;
+  entry->decoder=(DecoderHandler) ReadRGBImage;
+  entry->encoder=(EncoderHandler) WriteRGBImage;
   entry->raw=True;
   entry->description=AcquireString("Raw red, green, and blue samples");
   entry->module=AcquireString("RGB");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RGBA");
-  entry->decoder=ReadRGBImage;
-  entry->encoder=WriteRGBImage;
+  entry->decoder=(DecoderHandler) ReadRGBImage;
+  entry->encoder=(EncoderHandler) WriteRGBImage;
   entry->raw=True;
   entry->description=AcquireString("Raw red, green, blue, and matte samples");
   entry->module=AcquireString("RGB");
@@ -514,10 +534,12 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     *pixels;
 
   unsigned int
-    packet_size,
-    scene,
     status;
 
+  unsigned long
+    packet_size,
+    scene;
+
   /*
     Allocate memory for pixels.
   */
@@ -530,15 +552,15 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     packet_size=image->depth > 8 ? 8 : 4;
   pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == False)
-        ThrowWriterException(FileOpenError,"Unable to open file",image);
+        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
     }
   scene=0;
   do
@@ -546,7 +568,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to RGB raster pixels.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     if (LocaleCompare(image_info->magick,"RGBA") == 0)
       if (!image->matte)
         SetImageOpacity(image,OpaqueOpacity);
@@ -575,7 +597,8 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         break;
       }
@@ -601,7 +624,8 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlob(image,image->columns,pixels);
             }
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+              break;
         }
         break;
       }
@@ -614,9 +638,9 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         if (image_info->interlace == PartitionInterlace)
           {
             AppendImageFormat("R",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -630,11 +654,13 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           {
             CloseBlob(image);
             AppendImageFormat("G",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
-        MagickMonitor(SaveImageText,100,400);
+        if (!MagickMonitor(SaveImageTag,100,400,&image->exception))
+          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -647,11 +673,13 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           {
             CloseBlob(image);
             AppendImageFormat("B",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
-        MagickMonitor(SaveImageText,200,400);
+        if (!MagickMonitor(SaveImageTag,200,400,&image->exception))
+          break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -662,16 +690,16 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         }
         if (LocaleCompare(image_info->magick,"RGBA") == 0)
           {
-            MagickMonitor(SaveImageText,300,400);
+            if (!MagickMonitor(SaveImageTag,300,400,&image->exception))
+              break;
             if (image_info->interlace == PartitionInterlace)
               {
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
-                status=OpenBlob(image_info,image,WriteBinaryType,
+                status=OpenBlob(image_info,image,WriteBinaryBlobMode,
                   &image->exception);
                 if (status == False)
-                  ThrowWriterException(FileOpenError,"Unable to open file",
-                    image);
+                  ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
               }
             for (y=0; y < (long) image->rows; y++)
             {
@@ -685,14 +713,18 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           }
         if (image_info->interlace == PartitionInterlace)
           (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-        MagickMonitor(SaveImageText,400,400);
+        if (!MagickMonitor(SaveImageTag,400,400,&image->exception))
+          break;
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   LiberateMemory((void **) &pixels);
   if (image_info->adjoin)
