@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -196,9 +203,9 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   rla_info.window.left=ReadBlobMSBShort(image);
   rla_info.window.right=ReadBlobMSBShort(image);
   rla_info.window.bottom=ReadBlobMSBShort(image);
@@ -255,7 +262,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   scanlines=(long *) AcquireMemory(image->rows*sizeof(long));
   if (scanlines == (long *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (*rla_info.description != '\0')
     (void) SetImageAttribute(image,"comment",rla_info.description);
   /*
@@ -298,24 +305,24 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 case 0:
                 {
-                  q->red=Upscale(byte);
+                  q->red=ScaleCharToQuantum(byte);
                   *indexes=0;
                   break;
                 }
                 case 1:
                 {
-                  q->green=Upscale(byte);
+                  q->green=ScaleCharToQuantum(byte);
                   break;
                 }
                 case 2:
                 {
-                  q->blue=Upscale(byte);
+                  q->blue=ScaleCharToQuantum(byte);
                   break;
                 }
                 case 3:
                 default:
                 {
-                  q->opacity=(Quantum) (MaxRGB-Upscale(byte));
+                  q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(byte));
                   break;
                 }
               }
@@ -339,24 +346,24 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             case 0:
             {
-              q->red=Upscale(byte);
+              q->red=ScaleCharToQuantum(byte);
               *indexes=0;
               break;
             }
             case 1:
             {
-              q->green=Upscale(byte);
+              q->green=ScaleCharToQuantum(byte);
               break;
             }
             case 2:
             {
-              q->blue=Upscale(byte);
+              q->blue=ScaleCharToQuantum(byte);
               break;
             }
             case 3:
             default:
             {
-              q->opacity=(Quantum) (MaxRGB-Upscale(byte));
+              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(byte));
               break;
             }
           }
@@ -369,10 +376,12 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     }
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        break;
   }
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
   return(image);
 }
@@ -406,7 +415,7 @@ ModuleExport void RegisterRLAImage(void)
     *entry;
 
   entry=SetMagickInfo("RLA");
-  entry->decoder=ReadRLAImage;
+  entry->decoder=(DecoderHandler) ReadRLAImage;
   entry->adjoin=False;
   entry->description=AcquireString("Alias/Wavefront image");
   entry->module=AcquireString("RLA");
