@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Typedef declaractions.
@@ -158,14 +164,14 @@ static unsigned int IsSGI(const unsigned char *magick,const size_t length)
 static void SGIDecode(const unsigned long bytes_per_pixel,
   unsigned char *max_packets,unsigned char *pixels)
 {
-  int
+  long
     count;
 
   register unsigned char
     *p,
     *q;
 
-  unsigned int
+  unsigned long
     pixel;
 
   p=max_packets;
@@ -176,7 +182,7 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
       {
         pixel=(*p++) << 8;
         pixel|=(*p++);
-        count=pixel & 0x7f;
+        count=(long) (pixel & 0x7f);
         if (count == 0)
           break;
         if (pixel & 0x80)
@@ -192,8 +198,8 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
             pixel|=(*p++);
             for ( ; count != 0; count--)
             {
-              *q=pixel >> 8;
-              *(q+1)=pixel;
+              *q=(unsigned char) (pixel >> 8);
+              *(q+1)=(unsigned char) pixel;
               q+=8;
             }
           }
@@ -203,7 +209,7 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
   for ( ; ; )
   {
     pixel=(*p++);
-    count=pixel & 0x7f;
+    count=(long) (pixel & 0x7f);
     if (count == 0)
       break;
     if (pixel & 0x80)
@@ -217,7 +223,7 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
         pixel=(*p++);
         for ( ; count != 0; count--)
         {
-          *q=pixel;
+          *q=(unsigned char) pixel;
           q+=4;
         }
       }
@@ -226,6 +232,9 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
 
 static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  ExtendedSignedIntegralType
+    number_pixels;
+
   Image
     *image;
 
@@ -256,8 +265,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status;
 
   unsigned long
-    bytes_per_pixel,
-    number_pixels;
+    bytes_per_pixel;
 
   /*
     Open image file.
@@ -267,9 +275,9 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read SGI raster header.
   */
@@ -280,7 +288,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Verify SGI identifier.
     */
     if (iris_info.magic != 0x01DA)
-      ThrowReaderException(CorruptImageError,"Not a SGI RGB image",image);
+      ThrowReaderException(CorruptImageError,"NotASGIImage",image);
     iris_info.storage=ReadBlobByte(image);
     iris_info.bytes_per_pixel=ReadBlobByte(image);
     iris_info.dimension=ReadBlobMSBShort(image);
@@ -306,12 +314,14 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Allocate SGI pixels.
     */
     bytes_per_pixel=iris_info.bytes_per_pixel;
-    number_pixels=iris_info.columns*iris_info.rows;
+    number_pixels=(ExtendedSignedIntegralType) iris_info.columns*iris_info.rows;
+    if ((4*bytes_per_pixel*number_pixels) != (size_t)
+        (4*bytes_per_pixel*number_pixels))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     iris_pixels=(unsigned char *)
-      AcquireMemory(4*bytes_per_pixel*number_pixels);
+      AcquireMemory(4*bytes_per_pixel*iris_info.columns*iris_info.rows);
     if (iris_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     if (iris_info.storage != 0x01)
       {
         unsigned char
@@ -323,7 +333,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         scanline=(unsigned char *)
           AcquireMemory(bytes_per_pixel*iris_info.columns);
         if (scanline == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (z=0; z < (int) iris_info.depth; z++)
         {
@@ -373,7 +383,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if ((offsets == (unsigned long *) NULL) ||
             (max_packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
           offsets[i]=ReadBlobMSBLong(image);
@@ -460,11 +470,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                q->red=XDownscale((*(p+0) << 8) | (*(p+1)));
-                q->green=XDownscale((*(p+2) << 8) | (*(p+3)));
-                q->blue=XDownscale((*(p+4) << 8) | (*(p+5)));
+                q->red=ScaleShortToQuantum((*(p+0) << 8) | (*(p+1)));
+                q->green=ScaleShortToQuantum((*(p+2) << 8) | (*(p+3)));
+                q->blue=ScaleShortToQuantum((*(p+4) << 8) | (*(p+5)));
                 q->opacity=(Quantum)
-                  (MaxRGB-XDownscale((*(p+6) << 8) | (*(p+7))));
+                  (MaxRGB-ScaleShortToQuantum((*(p+6) << 8) | (*(p+7))));
                 p+=8;
                 q++;
               }
@@ -472,7 +482,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,y,image->rows);
+                  if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                    break;
             }
           }
         else
@@ -484,10 +495,10 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             for (x=0; x < (long) image->columns; x++)
             {
-              q->red=Upscale(*p);
-              q->green=Upscale(*(p+1));
-              q->blue=Upscale(*(p+2));
-              q->opacity=(Quantum) (MaxRGB-Upscale(*(p+3)));
+              q->red=ScaleCharToQuantum(*p);
+              q->green=ScaleCharToQuantum(*(p+1));
+              q->blue=ScaleCharToQuantum(*(p+2));
+              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*(p+3)));
               p+=4;
               q++;
             }
@@ -495,7 +506,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                MagickMonitor(LoadImageText,y,image->rows);
+                if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                  break;
           }
       }
     else
@@ -504,7 +516,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Create grayscale map.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         /*
           Convert SGI image to PseudoClass pixel packets.
@@ -529,7 +541,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(LoadImageText,y,image->rows);
+                  if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                    break;
             }
           }
         else
@@ -550,13 +563,18 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                MagickMonitor(LoadImageText,y,image->rows);
+                if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                  break;
           }
         SyncImage(image);
       }
     LiberateMemory((void **) &iris_pixels);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -575,8 +593,11 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (iris_info.magic == 0x01DA);
   while (image->previous != (Image *) NULL)
@@ -614,11 +635,12 @@ ModuleExport void RegisterSGIImage(void)
     *entry;
 
   entry=SetMagickInfo("SGI");
-  entry->decoder=ReadSGIImage;
-  entry->encoder=WriteSGIImage;
-  entry->magick=IsSGI;
+  entry->decoder=(DecoderHandler) ReadSGIImage;
+  entry->encoder=(EncoderHandler) WriteSGIImage;
+  entry->magick=(MagickHandler) IsSGI;
   entry->description=AcquireString("Irix RGB image");
   entry->module=AcquireString("SGI");
+  entry->seekable_stream=True;
   (void) RegisterMagickInfo(entry);
 }
 
@@ -731,6 +753,9 @@ static size_t SGIEncode(unsigned char *pixels,size_t count,
 
 static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
 {
+  ExtendedSignedIntegralType
+    number_pixels;
+
   long
     y,
     z;
@@ -753,11 +778,10 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     *packets;
 
   unsigned int
-    scene,
     status;
 
   unsigned long
-    number_pixels;
+    scene;
 
   /*
     Open output image file.
@@ -767,18 +791,17 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if ((image->columns > 65535L) || (image->rows > 65535L))
-    ThrowWriterException(ResourceLimitError,
-      "Width or height limit exceeded",image);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit",image);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   scene=0;
   do
   {
     /*
       Initialize SGI raster file header.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     iris_info.magic=0x01DA;
     if (image_info->compression == NoCompression)
       iris_info.storage=0x00;
@@ -796,7 +819,7 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         iris_info.depth=1;
       }
     iris_info.minimum_value=0;
-    iris_info.maximum_value=Downscale(MaxRGB);
+    iris_info.maximum_value=ScaleQuantumToChar(MaxRGB);
     for (i=0; i < (int) sizeof(iris_info.filler); i++)
       iris_info.filler[i]=0;
     /*
@@ -816,11 +839,12 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     /*
       Allocate SGI pixels.
     */
-    number_pixels=image->columns*image->rows;
-    iris_pixels=(unsigned char *) AcquireMemory(4*number_pixels);
+    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+    if ((4*number_pixels) != (size_t) (4*number_pixels))
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+    iris_pixels=(unsigned char *) AcquireMemory(4*image->columns*image->rows);
     if (iris_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     /*
       Convert image pixels to uncompressed SGI pixels.
     */
@@ -832,15 +856,16 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
       q=iris_pixels+((iris_info.rows-1)-y)*(iris_info.columns*4);
       for (x=0; x < (long) image->columns; x++)
       {
-        *q++=Downscale(p->red);
-        *q++=Downscale(p->green);
-        *q++=Downscale(p->blue);
-        *q++=MaxRGB-Downscale(p->opacity);
+        *q++=ScaleQuantumToChar(p->red);
+        *q++=ScaleQuantumToChar(p->green);
+        *q++=ScaleQuantumToChar(p->blue);
+        *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
         p++;
       }
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(SaveImageText,y,image->rows);
+          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+            break;
     }
     if (image_info->compression == NoCompression)
       {
@@ -852,7 +877,7 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         */
         scanline=(unsigned char *) AcquireMemory(iris_info.columns);
         if (scanline == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"Memory allocation failed",
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (z=0; z < (int) iris_info.depth; z++)
         {
@@ -890,7 +915,7 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         if ((offsets == (unsigned long *) NULL) ||
             (packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
-          ThrowWriterException(ResourceLimitError,"Memory allocation failed",
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         offset=512+4*2*(iris_info.rows*iris_info.depth);
         number_packets=0;
@@ -926,8 +951,11 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     LiberateMemory((void **) &iris_pixels);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
