@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -141,7 +147,7 @@ static unsigned int DecodeImage(const unsigned char *compressed_pixels,
   register const unsigned char
     *p;
 
-  register int
+  register long
     count;
 
   register unsigned char
@@ -278,9 +284,9 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read SUN raster header.
   */
@@ -291,7 +297,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Verify SUN identifier.
     */
     if (sun_info.magic != 0x59a66a95)
-      ThrowReaderException(CorruptImageError,"Not a SUN raster image",image);
+      ThrowReaderException(CorruptImageError,"NotASUNRasterImage",image);
     sun_info.width=ReadBlobMSBLong(image);
     sun_info.height=ReadBlobMSBLong(image);
     sun_info.depth=ReadBlobMSBLong(image);
@@ -321,8 +327,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
               Create linear color ramp.
             */
             if (!AllocateImageColormap(image,image->colors))
-              ThrowReaderException(ResourceLimitError,
-                "Memory allocation failed",image);
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+                image);
           }
         break;
       }
@@ -335,21 +341,21 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Read SUN raster colormap.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         sun_colormap=(unsigned char *) AcquireMemory(image->colors);
         if (sun_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         (void) ReadBlob(image,image->colors,(char *) sun_colormap);
         for (i=0; i < (long) image->colors; i++)
-          image->colormap[i].red=Upscale(sun_colormap[i]);
+          image->colormap[i].red=ScaleCharToQuantum(sun_colormap[i]);
         (void) ReadBlob(image,image->colors,(char *) sun_colormap);
         for (i=0; i < (long) image->colors; i++)
-          image->colormap[i].green=Upscale(sun_colormap[i]);
+          image->colormap[i].green=ScaleCharToQuantum(sun_colormap[i]);
         (void) ReadBlob(image,image->colors,(char *) sun_colormap);
         for (i=0; i < (long) image->colors; i++)
-          image->colormap[i].blue=Upscale(sun_colormap[i]);
+          image->colormap[i].blue=ScaleCharToQuantum(sun_colormap[i]);
         LiberateMemory((void **) &sun_colormap);
         break;
       }
@@ -363,15 +369,14 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         sun_colormap=(unsigned char *) AcquireMemory(sun_info.maplength);
         if (sun_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         (void) ReadBlob(image,sun_info.maplength,(char *) sun_colormap);
         LiberateMemory((void **) &sun_colormap);
         break;
       }
       default:
-        ThrowReaderException(CorruptImageError,
-          "Colormap type is not supported",image)
+        ThrowReaderException(CoderError,"ColormapTypeNotSupported",image)
     } 
     image->matte=(sun_info.depth == 32);
     image->columns=sun_info.width;
@@ -383,12 +388,10 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     sun_data=(unsigned char *) AcquireMemory(sun_info.length);
     if (sun_data == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     count=ReadBlob(image,sun_info.length,(char *) sun_data);
     if ((count == 0) && (sun_info.type != RT_ENCODED))
-      ThrowReaderException(CorruptImageError,"Unable to read image data",
-        image);
+      ThrowReaderException(CorruptImageError,"UnableToReadImageData",image);
     sun_pixels=sun_data;
     if (sun_info.type == RT_ENCODED)
       {
@@ -402,7 +405,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         bytes_per_line=2*(sun_info.width*sun_info.depth+15)/16;
         sun_pixels=(unsigned char *) AcquireMemory(bytes_per_line*height);
         if (sun_pixels == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         (void) DecodeImage(sun_data,sun_info.length,sun_pixels);
         LiberateMemory((void **) &sun_data);
@@ -421,13 +424,13 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (x=0; x < ((long) image->columns-7); x+=8)
         {
           for (bit=7; bit >= 0; bit--)
-            indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x00 : 0x01);
+            indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x01 : 0x00);
           p++;
         }
         if ((image->columns % 8) != 0)
           {
             for (bit=7; bit >= (long) (8-(image->columns % 8)); bit--)
-              indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x00 : 0x01);
+              indexes[x+7-bit]=((*p) & (0x01 << bit) ? 0x01 : 0x00);
             p++;
           }
         if ((((image->columns/8)+(image->columns % 8 ? 1 : 0)) % 2) != 0)
@@ -436,7 +439,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         if (image->previous == (Image *) NULL)
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,y,image->rows);
+            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+              break;
       }
     else
       if (image->storage_class == PseudoClass)
@@ -454,7 +458,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
       else
         for (y=0; y < (long) image->rows; y++)
@@ -465,18 +470,18 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           for (x=0; x < (long) image->columns; x++)
           {
             if (image->matte)
-              q->opacity=(Quantum) (MaxRGB-Upscale(*p++));
+              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
             if (sun_info.type == RT_STANDARD)
               {
-                q->blue=Upscale(*p++);
-                q->green=Upscale(*p++);
-                q->red=Upscale(*p++);
+                q->blue=ScaleCharToQuantum(*p++);
+                q->green=ScaleCharToQuantum(*p++);
+                q->red=ScaleCharToQuantum(*p++);
               }
             else
               {
-                q->red=Upscale(*p++);
-                q->green=Upscale(*p++);
-                q->blue=Upscale(*p++);
+                q->red=ScaleCharToQuantum(*p++);
+                q->green=ScaleCharToQuantum(*p++);
+                q->blue=ScaleCharToQuantum(*p++);
               }
             if (image->colors != 0)
               {
@@ -492,13 +497,18 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
     if (image->storage_class == PseudoClass)
       SyncImage(image);
     LiberateMemory((void **) &sun_pixels);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -517,8 +527,11 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (sun_info.magic == 0x59a66a95);
   while (image->previous != (Image *) NULL)
@@ -556,15 +569,15 @@ ModuleExport void RegisterSUNImage(void)
     *entry;
 
   entry=SetMagickInfo("RAS");
-  entry->decoder=ReadSUNImage;
-  entry->encoder=WriteSUNImage;
-  entry->magick=IsSUN;
+  entry->decoder=(DecoderHandler) ReadSUNImage;
+  entry->encoder=(EncoderHandler) WriteSUNImage;
+  entry->magick=(MagickHandler) IsSUN;
   entry->description=AcquireString("SUN Rasterfile");
   entry->module=AcquireString("SUN");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SUN");
-  entry->decoder=ReadSUNImage;
-  entry->encoder=WriteSUNImage;
+  entry->decoder=(DecoderHandler) ReadSUNImage;
+  entry->encoder=(EncoderHandler) WriteSUNImage;
   entry->description=AcquireString("SUN Rasterfile");
   entry->module=AcquireString("SUN");
   (void) RegisterMagickInfo(entry);
@@ -645,6 +658,9 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
       maplength;
   } SUNInfo;
 
+  ExtendedSignedIntegralType
+    number_pixels;
+
   long
     y;
 
@@ -664,11 +680,10 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
     sun_info;
 
   unsigned int
-    scene,
     status;
 
   unsigned long
-    number_pixels;
+    scene;
 
   /*
     Open output image file.
@@ -677,16 +692,16 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   scene=0;
   do
   {
     /*
       Initialize SUN raster file header.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     sun_info.magic=0x59a66a95;
     sun_info.width=image->columns;
     sun_info.height=image->rows;
@@ -694,7 +709,9 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
       (image->storage_class == DirectClass ? RT_FORMAT_RGB : RT_STANDARD);
     sun_info.maptype=RMT_NONE;
     sun_info.maplength=0;
-    number_pixels=image->columns*image->rows;
+    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+    if ((4*number_pixels) != (size_t) (4*number_pixels))
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     if (image->storage_class == DirectClass)
       {
         /*
@@ -759,7 +776,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
         length=image->columns*sizeof(PixelPacket);
         pixels=(unsigned char *) AcquireMemory(length);
         if (pixels == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"Memory allocation failed",
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         /*
           Convert DirectClass packet to SUN RGB pixel.
@@ -773,10 +790,10 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
           {
             if (image->matte)
-              *q++=MaxRGB-Downscale(p->opacity);
-            *q++=Downscale(p->red);
-            *q++=Downscale(p->green);
-            *q++=Downscale(p->blue);
+              *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
+            *q++=ScaleQuantumToChar(p->red);
+            *q++=ScaleQuantumToChar(p->green);
+            *q++=ScaleQuantumToChar(p->blue);
             p++;
           }
           if (image->columns & 0x01)
@@ -784,7 +801,8 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlob(image,q-pixels,(char *) pixels);
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         LiberateMemory((void **) &pixels);
       }
@@ -799,10 +817,11 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           /*
             Convert PseudoClass image to a SUN monochrome image.
           */
-          polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+          SetImageType(image,BilevelType);
+          polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
           if (image->colors == 2)
-            polarity=
-              Intensity(&image->colormap[0]) > Intensity(&image->colormap[1]);
+            polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+              PixelIntensityToQuantum(&image->colormap[1]);
           for (y=0; y < (long) image->rows; y++)
           {
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -832,7 +851,8 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                MagickMonitor(SaveImageText,y,image->rows);
+                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                  break;
           }
         }
       else
@@ -841,11 +861,12 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             Dump colormap to file.
           */
           for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,Downscale(image->colormap[i].red));
+            (void) WriteBlobByte(image,
+              ScaleQuantumToChar(image->colormap[i].red));
           for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,Downscale(image->colormap[i].green));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(image->colormap[i].green));
           for (i=0; i < (long) image->colors; i++)
-            (void) WriteBlobByte(image,Downscale(image->colormap[i].blue));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(image->colormap[i].blue));
           /*
             Convert PseudoClass packet to SUN colormapped pixel.
           */
@@ -864,13 +885,17 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                MagickMonitor(SaveImageText,y,image->rows);
+                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                  break;
           }
         }
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
