@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -176,9 +183,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read TGA header information.
   */
@@ -189,7 +196,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   {
     if ((count == 0) || (tga_info.image_type == 0) ||
         (tga_info.image_type > 11))
-      ThrowReaderException(CorruptImageError,"Not a TGA image file",image);
+      ThrowReaderException(CorruptImageError,"NotATGAImageFile",image);
     tga_info.colormap_index=ReadBlobLSBShort(image);
     tga_info.colormap_length=ReadBlobLSBShort(image);
     tga_info.colormap_size=ReadBlobByte(image);
@@ -223,7 +230,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         comment=(char *) AcquireMemory(tga_info.id_length+1);
         if (comment == (char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         (void) ReadBlob(image,tga_info.id_length,comment);
         comment[tga_info.id_length]='\0';
@@ -238,7 +245,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Read TGA raster colormap.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (i=0; i < (long) image->colors; i++)
         {
@@ -250,7 +257,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              pixel.red=Upscale(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               pixel.green=pixel.red;
               pixel.blue=pixel.red;
               break;
@@ -264,11 +271,11 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               j=ReadBlobByte(image);
               k=ReadBlobByte(image);
               pixel.red=(Quantum)
-                ((unsigned long) (MaxRGB*((int) (k & 0x7c) >> 2))/31);
-              pixel.green=(Quantum) ((unsigned long)
-                (MaxRGB*(((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31);
+                (((double) MaxRGB*((int) (k & 0x7c) >> 2))/31+0.5);
+              pixel.green=(Quantum) ((MaxRGB*
+                (((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31+0.5);
               pixel.blue=(Quantum)
-                ((unsigned long) (MaxRGB*((int) (j & 0x1f)))/31);
+                (((double) MaxRGB*((int) (j & 0x1f)))/31+0.5);
               break;
             }
             case 24:
@@ -277,9 +284,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue, green and red.
               */
-              pixel.blue=Upscale(ReadBlobByte(image));
-              pixel.green=Upscale(ReadBlobByte(image));
-              pixel.red=Upscale(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               break;
             }
           }
@@ -324,7 +331,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 count=ReadBlob(image,1,(char *) &runlength);
                 if (count == 0)
                   ThrowReaderException(CorruptImageError,
-                    "Unable to read image data",image);
+                    "UnableToReadImageData",image);
                 flag=runlength & 0x80;
                 if (flag != 0)
                   runlength-=128;
@@ -340,14 +347,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              index=ConstrainColormapIndex(image,ReadBlobByte(image));
+              index=ReadBlobByte(image);
               if (tga_info.colormap_type != 0)
-                pixel=image->colormap[index];
+                pixel=image->colormap[ConstrainColormapIndex(image,index)];
               else
                 {
-                  pixel.red=Upscale(index);
-                  pixel.green=Upscale(index);
-                  pixel.blue=Upscale(index);
+                  pixel.red=ScaleCharToQuantum(index);
+                  pixel.green=ScaleCharToQuantum(index);
+                  pixel.blue=ScaleCharToQuantum(index);
                 }
               break;
             }
@@ -359,10 +366,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               */
               j=ReadBlobByte(image);
               k=ReadBlobByte(image);
-              pixel.red=(Quantum) ((MaxRGB*((int) (k & 0x7c) >> 2))/31);
-              pixel.green=(Quantum)
-                ((MaxRGB*(((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31);
-              pixel.blue=(Quantum) ((MaxRGB*((int) (j & 0x1f)))/31);
+              pixel.red=(Quantum)
+                ((double) (MaxRGB*((int) (k & 0x7c) >> 2))/31+0.5);
+              pixel.green=(Quantum) (((double) MaxRGB*
+                (((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31+0.5);
+              pixel.blue=(Quantum)
+                (((double) MaxRGB*((int) (j & 0x1f)))/31+0.5);
               if (image->storage_class == PseudoClass)
                 index=ConstrainColormapIndex(image,((unsigned short) k << 8)+j);
               break;
@@ -373,17 +382,16 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue green and red.
               */
-              pixel.blue=Upscale(ReadBlobByte(image));
-              pixel.green=Upscale(ReadBlobByte(image));
-              pixel.red=Upscale(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               if (tga_info.bits_per_pixel == 32)
-                pixel.opacity=(Quantum) (MaxRGB-Upscale(ReadBlobByte(image)));
+                pixel.opacity=ScaleCharToQuantum(MaxRGB-ReadBlobByte(image));
               break;
             }
           }
         if (status == False)
-          ThrowReaderException(CorruptImageError,"Unable to read image data",
-            image);
+          ThrowReaderException(CorruptImageError,"UnableToReadImageData",image);
         if (image->storage_class == PseudoClass)
           indexes[x]=index;
         *q++=pixel;
@@ -404,10 +412,15 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            break;
     }
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -429,8 +442,11 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (status == True);
   while (image->previous != (Image *) NULL)
@@ -468,26 +484,26 @@ ModuleExport void RegisterTGAImage(void)
     *entry;
 
   entry=SetMagickInfo("ICB");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
+  entry->decoder=(DecoderHandler) ReadTGAImage;
+  entry->encoder=(EncoderHandler) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TGA");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
+  entry->decoder=(DecoderHandler) ReadTGAImage;
+  entry->encoder=(EncoderHandler) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VDA");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
+  entry->decoder=(DecoderHandler) ReadTGAImage;
+  entry->encoder=(EncoderHandler) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VST");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
+  entry->decoder=(DecoderHandler) ReadTGAImage;
+  entry->encoder=(EncoderHandler) WriteTGAImage;
   entry->description=AcquireString("Truevision Targa image");
   entry->module=AcquireString("TGA");
   (void) RegisterMagickInfo(entry);
@@ -613,9 +629,11 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     *targa_pixels;
 
   unsigned int
-    scene,
     status;
 
+  unsigned long
+    scene;
+
   /*
     Open output image file.
   */
@@ -623,16 +641,16 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
   scene=0;
   do
   {
     /*
       Initialize TGA raster file header.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     targa_info.id_length=0;
     attribute=GetImageAttribute(image,"comment");
     if (attribute != (const ImageAttribute *) NULL)
@@ -699,14 +717,14 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
         targa_colormap=(unsigned char *)
           AcquireMemory(3*targa_info.colormap_length);
         if (targa_colormap == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"Memory allocation failed",
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         q=targa_colormap;
         for (i=0; i < (long) image->colors; i++)
         {
-          *q++=Downscale(image->colormap[i].blue);
-          *q++=Downscale(image->colormap[i].green);
-          *q++=Downscale(image->colormap[i].red);
+          *q++=ScaleQuantumToChar(image->colormap[i].blue);
+          *q++=ScaleQuantumToChar(image->colormap[i].green);
+          *q++=ScaleQuantumToChar(image->colormap[i].red);
         }
         (void) WriteBlob(image,3*targa_info.colormap_length,
           (char *) targa_colormap);
@@ -718,8 +736,7 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     count=(long) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
     targa_pixels=(unsigned char *) AcquireMemory(count);
     if (targa_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     for (y=(long) (image->rows-1); y >= 0; y--)
     {
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -733,27 +750,31 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
           *q++=indexes[x];
         else
           {
-            *q++=Downscale(p->blue);
-            *q++=Downscale(p->green);
-            *q++=Downscale(p->red);
+            *q++=ScaleQuantumToChar(p->blue);
+            *q++=ScaleQuantumToChar(p->green);
+            *q++=ScaleQuantumToChar(p->red);
             if (image->colorspace == CMYKColorspace)
-              *q++=Downscale(p->opacity);
+              *q++=ScaleQuantumToChar(p->opacity);
             else
               if (image->matte)
-                *q++=MaxRGB-Downscale(p->opacity);
+                *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
           }
         p++;
       }
       (void) WriteBlob(image,q-targa_pixels,(char *) targa_pixels);
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(SaveImageText,y,image->rows);
+          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+            break;
     }
     LiberateMemory((void **) &targa_pixels);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
