@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -118,9 +124,9 @@ static Image *ReadTILEImage(const ImageInfo *image_info,
     return((Image *) NULL);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"Must specify image size",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
   if (*image_info->filename == '\0')
-    ThrowReaderException(OptionError,"must specify an image name",image);
+    ThrowReaderException(OptionError,"MustSpecifyAnImageName",image);
   /*
     Tile texture onto image.
   */
@@ -129,7 +135,8 @@ static Image *ReadTILEImage(const ImageInfo *image_info,
   {
     for (x=0; x < (long) image->columns; x+=tile_image->columns)
       (void) CompositeImage(image,CopyCompositeOp,tile_image,x,y);
-    MagickMonitor(LoadImageText,y,image->rows);
+    if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+      break;
   }
   DestroyImage(tile_image);
   return(image);
@@ -164,7 +171,7 @@ ModuleExport void RegisterTILEImage(void)
     *entry;
 
   entry=SetMagickInfo("TILE");
-  entry->decoder=ReadTILEImage;
+  entry->decoder=(DecoderHandler) ReadTILEImage;
   entry->raw=True;
   entry->description=AcquireString("Tile image with a texture");
   entry->module=AcquireString("TILE");
