@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,11 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -89,7 +94,7 @@ ModuleExport void RegisterUILImage(void)
     *entry;
 
   entry=SetMagickInfo("UIL");
-  entry->encoder=WriteUILImage;
+  entry->encoder=(EncoderHandler) WriteUILImage;
   entry->adjoin=False;
   entry->description=AcquireString("X-Motif UIL table");
   entry->module=AcquireString("UIL");
@@ -154,16 +159,15 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
 {
 #define MaxCixels  92
 
-  static const char
-    Cixel[MaxCixels+1] = " .XoO+@#$%&*=-;:>,<1234567890qwertyuipasdfghjk"
-                         "lzxcvbnmMNBVCZASDFGHJKLPIUYTREWQ!~^/()_`'][{}|";
-
   char
     basename[MaxTextExtent],
     buffer[MaxTextExtent],
     name[MaxTextExtent],
     symbol[MaxTextExtent];
 
+  ExtendedSignedIntegralType
+    number_pixels;
+
   int
     j;
 
@@ -181,14 +185,17 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
     i,
     x;
 
+  static const char
+    Cixel[MaxCixels+1] = " .XoO+@#$%&*=-;:>,<1234567890qwertyuipasdfghjk"
+                         "lzxcvbnmMNBVCZASDFGHJKLPIUYTREWQ!~^/()_`'][{}|";
+
   unsigned int
-    characters_per_pixel,
     status,
     transparent;
 
   unsigned long
-    colors,
-    number_pixels;
+    characters_per_pixel,
+    colors;
 
   /*
     Open output image file.
@@ -197,10 +204,10 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   transparent=False;
   i=0;
   p=(const PixelPacket *) NULL;
@@ -220,11 +227,15 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
           /*
             Map all the transparent pixels.
           */
-          number_pixels=image->columns*image->rows;
-          matte_image=(unsigned char *) AcquireMemory(number_pixels);
+          number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+          if (number_pixels != (size_t) number_pixels)
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+              image);
+          matte_image=(unsigned char *)
+            AcquireMemory(image->columns*image->rows);
           if (matte_image == (unsigned char *) NULL)
-            ThrowWriterException(ResourceLimitError,
-              "Memory allocation failed",image);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+              image);
           for (y=0; y < (long) image->rows; y++)
           {
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -299,9 +310,9 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
     if (LocaleCompare(name,"None") == 0)
       FormatString(buffer,"    background color = '%.1024s'",symbol);
     else
-      FormatString(buffer,"    color('%.1024s',%.1024s) = '%.1024s'",
-        name,Intensity(p) < (0.5*(MaxRGB+1)) ? "background" : "foreground",
-        symbol);
+      FormatString(buffer,"    color('%.1024s',%.1024s) = '%.1024s'",name,
+        PixelIntensityToQuantum(p) < (((double) MaxRGB+1.0)/2.0) ?
+        "background" : "foreground",symbol);
     (void) WriteBlobString(image,buffer);
     FormatString(buffer,"%.1024s",(i == (long) (colors-1) ? ");\n" : ",\n"));
     (void) WriteBlobString(image,buffer);
@@ -322,7 +333,7 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"    \"");
     for (x=0; x < (long) image->columns; x++)
     {
-      k=indexes[x] % MaxCixels;
+      k=(long) (indexes[x] % MaxCixels);
       symbol[0]=Cixel[k];
       for (j=1; j < (int) characters_per_pixel; j++)
       {
@@ -330,7 +341,7 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
         symbol[j]=Cixel[k];
       }
       symbol[j]='\0';
-      FormatString(buffer,"%.1024s",symbol);
+      (void) strncpy(buffer,symbol,MaxTextExtent-1);
       (void) WriteBlobString(image,buffer);
       p++;
     }
@@ -338,7 +349,8 @@ static unsigned int WriteUILImage(const ImageInfo *image_info,Image *image)
       (y == (long) (image->rows-1) ? ");" : ","));
     (void) WriteBlobString(image,buffer);
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+        break;
   }
   CloseBlob(image);
   return(True);
