@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -167,9 +173,9 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Decode image header.
   */
@@ -254,10 +260,10 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     count++;
   }
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(CorruptImageError,"image size is zero",image);
+    ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize",image);
   image->depth=8;
   if (!AllocateImageColormap(image,256))
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->ping)
     {
       CloseBlob(image);
@@ -268,7 +274,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   */
   scanline=(unsigned char *) AcquireMemory(image->columns);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(CorruptImageError,"Unable to read image data",image);
+    ThrowReaderException(CorruptImageError,"UnableToReadImageData",image);
   for (y=0; y < (long) image->rows; y++)
   {
     if (!SetImagePixels(image,0,y,image->columns,1))
@@ -278,11 +284,13 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        break;
   }
   LiberateMemory((void **) &scanline);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
   return(image);
 }
@@ -316,9 +324,9 @@ ModuleExport void RegisterVICARImage(void)
     *entry;
 
   entry=SetMagickInfo("VICAR");
-  entry->decoder=ReadVICARImage;
-  entry->encoder=WriteVICARImage;
-  entry->magick=IsVICAR;
+  entry->decoder=(DecoderHandler) ReadVICARImage;
+  entry->encoder=(EncoderHandler) WriteVICARImage;
+  entry->magick=(MagickHandler) IsVICAR;
   entry->adjoin=False;
   entry->description=AcquireString("VICAR rasterfile format");
   entry->module=AcquireString("VICAR");
@@ -406,10 +414,10 @@ static unsigned int WriteVICARImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   /*
     Write header.
   */
@@ -424,7 +432,7 @@ static unsigned int WriteVICARImage(const ImageInfo *image_info,Image *image)
   */
   scanline=(unsigned char *) AcquireMemory(image->columns);
   if (scanline == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Write VICAR scanline.
   */
@@ -437,7 +445,8 @@ static unsigned int WriteVICARImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlob(image,image->columns,scanline);
     if (image->previous == (Image *) NULL)
       if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+          break;
   }
   LiberateMemory((void **) &scanline);
   CloseBlob(image);
