@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -204,6 +211,9 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     scale_factor,
     value;
 
+  ExtendedSignedIntegralType
+    number_pixels;
+
   Image
     *image;
 
@@ -236,13 +246,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     *viff_pixels;
 
   unsigned int
-    bytes_per_pixel,
     status;
 
   unsigned long
+    bytes_per_pixel,
     lsb_first,
-    max_packets,
-    number_pixels;
+    max_packets;
 
   ViffInfo
     viff_info;
@@ -255,9 +264,9 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read VIFF header (1024 bytes).
   */
@@ -268,7 +277,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       Verify VIFF identifier.
     */
     if ((count == 0) || ((unsigned char) viff_info.identifier != 0xab))
-      ThrowReaderException(CorruptImageError,"Not a VIFF raster",image);
+      ThrowReaderException(CorruptImageError,"NotAVIFFImage",image);
     /*
       Initialize VIFF image.
     */
@@ -338,40 +347,36 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     /*
       Verify that we can read this VIFF image.
     */
-    number_pixels=viff_info.columns*viff_info.rows;
+    number_pixels=(ExtendedSignedIntegralType) viff_info.columns*viff_info.rows;
+    if (number_pixels != (size_t) number_pixels)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     if (number_pixels == 0)
-      ThrowReaderException(CorruptImageError,
-        "Image column or row size is not supported",image);
+      ThrowReaderException(CoderError,"ImageColumnOrRowSizeIsNotSupported",
+        image);
     if ((viff_info.data_storage_type != VFF_TYP_BIT) &&
         (viff_info.data_storage_type != VFF_TYP_1_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_2_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_4_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_FLOAT) &&
         (viff_info.data_storage_type != VFF_TYP_DOUBLE))
-      ThrowReaderException(CorruptImageError,
-        "Data storage type is not supported",image);
+      ThrowReaderException(CoderError,"DataStorageTypeIsNotSupported",image);
     if (viff_info.data_encode_scheme != VFF_DES_RAW)
-      ThrowReaderException(CorruptImageError,
-        "Data encoding scheme is not supported",image);
+      ThrowReaderException(CoderError,"DataEncodingSchemeIsNotSupported",image);
     if ((viff_info.map_storage_type != VFF_MAPTYP_NONE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_1_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_2_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_4_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_FLOAT) &&
         (viff_info.map_storage_type != VFF_MAPTYP_DOUBLE))
-      ThrowReaderException(CorruptImageError,
-        "Map storage type is not supported",image);
+      ThrowReaderException(CoderError,"MapStorageTypeIsNotSupported",image);
     if ((viff_info.color_space_model != VFF_CM_NONE) &&
         (viff_info.color_space_model != VFF_CM_ntscRGB) &&
         (viff_info.color_space_model != VFF_CM_genericRGB))
-      ThrowReaderException(CorruptImageError,
-        "Colorspace model is not supported",image);
+      ThrowReaderException(CoderError,"ColorspaceModelIsNotSupported",image);
     if (viff_info.location_type != VFF_LOC_IMPLICIT)
-      ThrowReaderException(CorruptImageError,
-        "Location type is not supported",image);
+      ThrowReaderException(CoderError,"LocationTypeIsNotSupported",image);
     if (viff_info.number_of_images != 1)
-      ThrowReaderException(CorruptImageError,
-        "Number of images is not supported",image);
+      ThrowReaderException(CoderError,"NumberOfImagesIsNotSupported",image);
     if (viff_info.map_rows == 0)
       viff_info.map_scheme=VFF_MS_NONE;
     switch ((int) viff_info.map_scheme)
@@ -383,13 +388,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             /*
               Create linear color ramp.
             */
+            image->colors=image->depth <= 8 ? 256 : 65536L;
             if (viff_info.data_storage_type == VFF_TYP_BIT)
               image->colors=2;
-            else
-              image->colors=1 << (viff_info.number_data_bands*QuantumDepth);
             if (!AllocateImageColormap(image,image->colors))
-              ThrowReaderException(ResourceLimitError,
-                "Memory allocation failed",image);
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
+                image);
           }
         break;
       }
@@ -413,12 +417,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         }
         image->colors=viff_info.map_columns;
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         viff_colormap=(unsigned char *)
           AcquireMemory(bytes_per_pixel*image->colors*viff_info.map_rows);
         if (viff_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         /*
           Read VIFF raster colormap.
@@ -458,25 +462,24 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           }
           if (i < (long) image->colors)
             {
-              image->colormap[i].red=Upscale((unsigned long) value);
-              image->colormap[i].green=Upscale((unsigned long) value);
-              image->colormap[i].blue=Upscale((unsigned long) value);
+              image->colormap[i].red=ScaleCharToQuantum((unsigned long) value);
+              image->colormap[i].green=ScaleCharToQuantum((unsigned long) value);
+              image->colormap[i].blue=ScaleCharToQuantum((unsigned long) value);
             }
           else
             if (i < (long) (2*image->colors))
               image->colormap[i % image->colors].green=
-                Upscale((unsigned long) value);
+                ScaleCharToQuantum((unsigned long) value);
             else
               if (i < (long) (3*image->colors))
                 image->colormap[i % image->colors].blue=
-                  Upscale((unsigned long) value);
+                  ScaleCharToQuantum((unsigned long) value);
         }
         LiberateMemory((void **) &viff_colormap);
         break;
       }
       default:
-        ThrowReaderException(CorruptImageError,
-          "Colormap type is not supported",image)
+        ThrowReaderException(CoderError,"ColormapTypeIsNotSupported",image)
     }
     /*
       Initialize image structure.
@@ -507,8 +510,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     viff_pixels=(unsigned char *)
       AcquireMemory(bytes_per_pixel*max_packets*sizeof(Quantum));
     if (viff_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     (void) ReadBlob(image,bytes_per_pixel*max_packets,(char *) viff_pixels);
     lsb_first=1;
     if (*(char *) &lsb_first &&
@@ -571,11 +573,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         else
           if (min_value == max_value)
             {
-              scale_factor=(double) MaxRGB/min_value;
+              scale_factor=MaxRGB/min_value;
               min_value=0;
             }
           else
-            scale_factor=(double) MaxRGB/(max_value-min_value);
+            scale_factor=MaxRGB/(max_value-min_value);
       }
     /*
       Convert pixels to Quantum size.
@@ -615,10 +617,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         /*
           Convert bitmap scanline.
         */
-        polarity=0;
+        SetImageType(image,BilevelType);
+        polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
         if (image->colors >= 2)
-          polarity=Intensity(&image->colormap[0]) <
-            Intensity(&image->colormap[1]);
+          polarity=PixelIntensityToQuantum(&image->colormap[0]) >
+            PixelIntensityToQuantum(&image->colormap[1]);
         for (y=0; y < (long) image->rows; y++)
         {
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -643,7 +646,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
       }
     else
@@ -660,14 +664,15 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
         }
       else
         {
           /*
             Convert DirectColor scanline.
           */
-          number_pixels=image->columns*image->rows;
+          number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
           for (y=0; y < (long) image->rows; y++)
           {
             q=SetImagePixels(image,0,y,image->columns,1);
@@ -675,17 +680,17 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
               break;
             for (x=0; x < (long) image->columns; x++)
             {
-              q->red=Upscale(*p);
-              q->green=Upscale(*(p+number_pixels));
-              q->blue=Upscale(*(p+2*number_pixels));
+              q->red=ScaleCharToQuantum(*p);
+              q->green=ScaleCharToQuantum(*(p+number_pixels));
+              q->blue=ScaleCharToQuantum(*(p+2*number_pixels));
               if (image->colors != 0)
                 {
                   q->red=image->colormap[q->red].red;
                   q->green=image->colormap[q->green].green;
                   q->blue=image->colormap[q->blue].blue;
                 }
-              q->opacity=Upscale(image->matte ?
-                MaxRGB-(*(p+number_pixels*3)) : OpaqueOpacity);
+              q->opacity=(Quantum) (image->matte ? MaxRGB-
+                ScaleCharToQuantum(*(p+number_pixels*3)) : OpaqueOpacity);
               p++;
               q++;
             }
@@ -693,14 +698,19 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
               break;
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                MagickMonitor(LoadImageText,y,image->rows);
+                if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                  break;
           }
         }
     LiberateMemory((void **) &viff_pixels);
     if (image->storage_class == PseudoClass)
       SyncImage(image);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -719,8 +729,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while ((count != 0) && (viff_info.identifier == 0xab));
   while (image->previous != (Image *) NULL)
@@ -758,15 +771,15 @@ ModuleExport void RegisterVIFFImage(void)
     *entry;
 
   entry=SetMagickInfo("VIFF");
-  entry->decoder=ReadVIFFImage;
-  entry->encoder=WriteVIFFImage;
-  entry->magick=IsVIFF;
+  entry->decoder=(DecoderHandler) ReadVIFFImage;
+  entry->encoder=(EncoderHandler) WriteVIFFImage;
+  entry->magick=(MagickHandler) IsVIFF;
   entry->description=AcquireString("Khoros Visualization image");
   entry->module=AcquireString("VIFF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XV");
-  entry->decoder=ReadVIFFImage;
-  entry->encoder=WriteVIFFImage;
+  entry->decoder=(DecoderHandler) ReadVIFFImage;
+  entry->encoder=(EncoderHandler) WriteVIFFImage;
   entry->description=AcquireString("Khoros Visualization image");
   (void) RegisterMagickInfo(entry);
 }
@@ -883,6 +896,9 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
+  ExtendedSignedIntegralType
+    number_pixels;
+
   long
     y;
 
@@ -906,12 +922,11 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     *viff_pixels;
 
   unsigned int
-    scene,
     status;
 
   unsigned long
-    number_pixels,
-    packets;
+    packets,
+    scene;
 
   ViffInfo
     viff_info;
@@ -923,16 +938,17 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  memset(&viff_info,0,sizeof(ViffInfo));
   scene=0;
   do
   {
     /*
       Initialize VIFF image structure.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     viff_info.identifier=(char) 0xab;
     viff_info.file_type=1;
     viff_info.release=1;
@@ -964,7 +980,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     viff_info.map_subrows=0;
     viff_info.map_enable=1;  /* no colormap */
     viff_info.maps_per_cycle=0;
-    number_pixels=image->columns*image->rows;
+    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
     if (image->storage_class == DirectClass)
       {
         /*
@@ -992,7 +1008,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             viff_info.map_columns=image->colors;
           }
         else
-          if (image->colors == 2)
+          if (image->colors <= 2)
             {
               /*
                 Monochrome VIFF raster.
@@ -1044,15 +1060,14 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     */
     viff_pixels=(unsigned char *) AcquireMemory(packets);
     if (viff_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
     q=viff_pixels;
     if (image->storage_class == DirectClass)
       {
         /*
           Convert DirectClass packet to VIFF RGB pixel.
         */
-        number_pixels=image->columns*image->rows;
+        number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -1060,17 +1075,18 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            *q=Downscale(p->red);
-            *(q+number_pixels)=Downscale(p->green);
-            *(q+number_pixels*2)=Downscale(p->blue);
+            *q=ScaleQuantumToChar(p->red);
+            *(q+number_pixels)=ScaleQuantumToChar(p->green);
+            *(q+number_pixels*2)=ScaleQuantumToChar(p->blue);
             if (image->matte)
-              *(q+number_pixels*3)=MaxRGB-Downscale(p->opacity);
+              *(q+number_pixels*3)=ScaleQuantumToChar(MaxRGB-p->opacity);
             p++;
             q++;
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
       }
     else
@@ -1084,15 +1100,15 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
           */
           viff_colormap=(unsigned char *) AcquireMemory(3*image->colors);
           if (viff_colormap == (unsigned char *) NULL)
-            ThrowWriterException(ResourceLimitError,
-              "Memory allocation failed",image);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
+              image);
           q=viff_colormap;
           for (i=0; i < (long) image->colors; i++)
-            *q++=Downscale(image->colormap[i].red);
+            *q++=ScaleQuantumToChar(image->colormap[i].red);
           for (i=0; i < (long) image->colors; i++)
-            *q++=Downscale(image->colormap[i].green);
+            *q++=ScaleQuantumToChar(image->colormap[i].green);
           for (i=0; i < (long) image->colors; i++)
-            *q++=Downscale(image->colormap[i].blue);
+            *q++=ScaleQuantumToChar(image->colormap[i].blue);
           (void) WriteBlob(image,3*image->colors,(char *) viff_colormap);
           LiberateMemory((void **) &viff_colormap);
           /*
@@ -1109,13 +1125,14 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
               *q++=indexes[x];
             if (image->previous == (Image *) NULL)
               if (QuantumTick(y,image->rows))
-                MagickMonitor(SaveImageText,y,image->rows);
+                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                  break;
           }
         }
       else
-        if (image->colors == 2)
+        if (image->colors <= 2)
           {
-            int
+            long
               x,
               y;
 
@@ -1127,10 +1144,11 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             /*
               Convert PseudoClass image to a VIFF monochrome image.
             */
-            polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+            SetImageType(image,BilevelType);
+            polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
             if (image->colors == 2)
-              polarity=
-                Intensity(&image->colormap[0]) > Intensity(&image->colormap[1]);
+              polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+                PixelIntensityToQuantum(&image->colormap[1]);
             for (y=0; y < (long) image->rows; y++)
             {
               p=AcquireImagePixels(image,0,y,image->columns,1,
@@ -1157,7 +1175,12 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
                 *q++=byte >> (8-bit);
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(SaveImageText,y,image->rows);
+                  {
+                    status=MagickMonitor(SaveImageTag,y,image->rows,
+                      &image->exception);
+                    if (status == False)
+                      break;
+                  }
             }
           }
         else
@@ -1173,20 +1196,28 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                *q++=(unsigned char) Intensity(p);
+                *q++=(unsigned char) PixelIntensityToQuantum(p);
                 p++;
               }
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(y,image->rows))
-                  MagickMonitor(SaveImageText,y,image->rows);
+                  {
+                    status=MagickMonitor(SaveImageTag,y,image->rows,
+                      &image->exception);
+                    if (status == False)
+                      break;
+                  }
             }
           }
     (void) WriteBlob(image,packets,(char *) viff_pixels);
     LiberateMemory((void **) &viff_pixels);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
