@@ -18,7 +18,7 @@
 %                               January 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -52,6 +52,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -152,27 +158,26 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   if (!ReadBlob(image,2,(char *) &header)) 
-    ThrowReaderException(CorruptImageError,"Not a WBMP image file",image);
+    ThrowReaderException(CorruptImageError,"NotAWBMPImageFile",image);
   if (header != 0)
-    ThrowReaderException(CorruptImageError,
-      "Only WBMP level 0 files supported",image);
+    ThrowReaderException(CoderError,"OnlyLevelZerofilesSupported",image);
   /*
     Initialize image structure.
   */
   if (WBMPReadInteger(image,&image->columns) == False) 
-    ThrowReaderException(CorruptImageError,"Corrupt WBMP image",image);
+    ThrowReaderException(CorruptImageError,"CorruptWBMPimage",image);
   if (WBMPReadInteger(image,&image->rows) == False) 
-    ThrowReaderException(CorruptImageError,"Corrupt WBMP image",image);
+    ThrowReaderException(CorruptImageError,"CorruptWBMPimage",image);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(CorruptImageError,"Not a WBMP image file",image);
+    ThrowReaderException(CorruptImageError,"NotAWBMPImageFile",image);
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   if (!AllocateImageColormap(image,2))
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   if (image_info->ping)
     {
       CloseBlob(image);
@@ -195,8 +200,7 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
         {
           byte=ReadBlobByte(image);
           if (byte == EOF)
-            ThrowReaderException(CorruptImageError,"Corrupt WBMP image",
-              image);
+            ThrowReaderException(CorruptImageError,"CorruptWBMPImage",image);
         }
       indexes[x]=(byte & (0x01 << (7-bit))) ? 1 : 0;
       bit++;
@@ -206,11 +210,13 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        break;
   }
   SyncImage(image);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
   return(image);
 }
@@ -244,8 +250,8 @@ ModuleExport void RegisterWBMPImage(void)
     *entry;
 
   entry=SetMagickInfo("WBMP");
-  entry->decoder=ReadWBMPImage;
-  entry->encoder=WriteWBMPImage;
+  entry->decoder=(DecoderHandler) ReadWBMPImage;
+  entry->encoder=(EncoderHandler) WriteWBMPImage;
   entry->adjoin=False;
   entry->description=AcquireString("Wireless Bitmap (level 0) image");
   entry->module=AcquireString("WBMP");
@@ -316,7 +322,7 @@ static void WBMPWriteInteger(Image *image,const unsigned long value)
     flag,
     n;
 
-  register int
+  register long
     i;
 
   unsigned char
@@ -369,27 +375,18 @@ static unsigned int WriteWBMPImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   /*
     Convert image to a bi-level image.
   */
-  if (!IsMonochromeImage(image,&image->exception))
-    {
-      QuantizeInfo
-        quantize_info;
-
-      GetQuantizeInfo(&quantize_info);
-      quantize_info.number_colors=2;
-      quantize_info.dither=image_info->dither;
-      quantize_info.colorspace=GRAYColorspace;
-      (void) QuantizeImage(&quantize_info,image);
-    }
-  polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+  SetImageType(image,BilevelType);
+  polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
   if (image->colors == 2)
-    polarity=Intensity(&image->colormap[0]) < Intensity(&image->colormap[1]);
+    polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+      PixelIntensityToQuantum(&image->colormap[1]);
   (void) WriteBlobMSBShort(image,0);
   WBMPWriteInteger(image,image->columns);
   WBMPWriteInteger(image,image->rows);
@@ -416,7 +413,8 @@ static unsigned int WriteWBMPImage(const ImageInfo *image_info,Image *image)
     if (bit != 0)
       (void) WriteBlobByte(image,byte);
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+        break;
   }
   CloseBlob(image);
   return(True);
