@@ -23,7 +23,7 @@
 %                            May 2001 - Oct 2001                              %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -56,12 +56,21 @@
  * Include declarations.
  */
 #include "studio.h"
+#include "blob.h"
+#include "draw.h"
+#include "list.h"
+#include "log.h"
+#include "magick.h"
+#include "monitor.h"
+#include "paint.h"
+#include "static.h"
+#include "utility.h"
+#include "version.h"
 #if defined(WIN32)
 #define M_PI MagickPI
 #endif
 
 #if defined(HasWMF) || defined(HasWMFlite)
-/* #if !defined(HasWIN32WMFAPI) */
 
 #define ERR(API)  ((API)->err != wmf_E_None)
 #define XC(x) ((double)x)
@@ -150,17 +159,10 @@ struct _wmf_magick_t
     translate_y,
     rotate;
 
-  /* MVG output */
+  /* Vector output */
   DrawContext
     draw_context;
 
-  char
-    *mvg;
-
-  size_t
-    mvg_alloc,
-    mvg_length;
-
   /* ImageMagick image */
     Image
       *image;
@@ -265,8 +267,16 @@ static void         util_set_pen(wmfAPI * API, wmfDC * dc);
 /* Progress callback */
 int magick_progress_callback (void* context,float quantum)
 {
-  MagickMonitor((char*)context,(off_t)floor(quantum*100),100);
-  return 0;
+  Image
+    *image;
+
+  unsigned int
+    status = 0;
+
+  image=(Image *) context;
+  if (MagickMonitor((char*)context,(ExtendedSignedIntegralType)floor(quantum*100),100,&image->exception) == False)
+    status = 1;
+  return status;
 }
 
 /* Set fill color */
@@ -275,9 +285,9 @@ static void draw_color_fill_rgb( wmfAPI* API, const wmfRGB* rgb )
   PixelPacket
     fill_color;
 
-  fill_color.red     = Upscale(rgb->r);
-  fill_color.green   = Upscale(rgb->g);
-  fill_color.blue    = Upscale(rgb->b);
+  fill_color.red     = ScaleCharToQuantum(rgb->r);
+  fill_color.green   = ScaleCharToQuantum(rgb->g);
+  fill_color.blue    = ScaleCharToQuantum(rgb->b);
   fill_color.opacity = OpaqueOpacity;
 
   DrawSetFillColor(WmfDrawContext,&fill_color);
@@ -290,9 +300,9 @@ static void draw_color_stroke_rgb( wmfAPI* API, const wmfRGB* rgb )
   PixelPacket
     stroke_color;
   
-  stroke_color.red     = Upscale(rgb->r);
-  stroke_color.green   = Upscale(rgb->g);
-  stroke_color.blue    = Upscale(rgb->b);
+  stroke_color.red     = ScaleCharToQuantum(rgb->r);
+  stroke_color.green   = ScaleCharToQuantum(rgb->g);
+  stroke_color.blue    = ScaleCharToQuantum(rgb->b);
   stroke_color.opacity = OpaqueOpacity;
   
   DrawSetStrokeColor(WmfDrawContext,&stroke_color);
@@ -310,17 +320,46 @@ static void draw_pattern_push( wmfAPI* API,
   DrawPushPattern(WmfDrawContext,pattern_id,0,0,columns,rows);
 }
 
+/* Pattern/Bit BLT with raster operation (ROP) support.  Invoked by
+   META_PATBLT, which is equivalent to Windows PatBlt() call, or by
+   META_DIBBITBLT which is equivalent to Windows BitBlt() call. */
+
+/* The BitBlt function transfers pixels from a rectangular area in one
+   device context called the 'source', to a rectangular area of the
+   same size in another device context, called the 'destination'. */
+
 static void ipa_rop_draw(wmfAPI * API, wmfROP_Draw_t * rop_draw)
 {
+/*   wmfBrush */
+/*     *brush = WMF_DC_BRUSH(rop_draw->dc); */
+  
+/*   wmfBMP */
+/*     *brush_bmp = WMF_BRUSH_BITMAP(brush); */
+
   if (!TO_FILL(rop_draw))
     return;
 
   /* Save graphic context */
   DrawPushGraphicContext(WmfDrawContext);
 
-  /* FIXME: finish implementing (once we know what it is supposed to do!)*/
+  /* FIXME: finish implementing (once we know what it is supposed to do!) */
 
-  util_set_brush(API, rop_draw->dc, BrushApplyFill);
+  /*
+  struct _wmfROP_Draw_t
+  {       wmfDC* dc;
+    
+    wmfD_Coord TL;
+    wmfD_Coord BR;
+    
+    U32 ROP;
+    
+    double pixel_width;
+    double pixel_height;
+  };
+  */
+
+/*   if(brush_bmp && brush_bmp->data != 0) */
+/*     printf("Have an image!\n"); */
 
   switch (rop_draw->ROP) /* Ternary raster operations */
     {
@@ -352,7 +391,7 @@ static void ipa_rop_draw(wmfAPI * API, wmfROP_Draw_t * rop_draw)
       printf("ipa_rop_draw MERGEPAINT ROP mode not implemented\n");
       break;
     case PATCOPY: /* dest = pattern */
-      printf("ipa_rop_draw PATCOPY ROP mode not implemented\n");
+      util_set_brush(API, rop_draw->dc, BrushApplyFill);
       break;
     case PATPAINT: /* dest = DPSnoo */
       printf("ipa_rop_draw PATPAINT ROP mode not implemented\n");
@@ -364,12 +403,13 @@ static void ipa_rop_draw(wmfAPI * API, wmfROP_Draw_t * rop_draw)
       printf("ipa_rop_draw DSTINVERT ROP mode not implemented\n");
       break;
     case BLACKNESS: /* dest = BLACK */
-      printf("ipa_rop_draw BLACKNESS ROP mode not implemented\n");
+      DrawSetFillColorString(WmfDrawContext,"black");
       break;
     case WHITENESS: /* dest = WHITE */
-      printf("ipa_rop_draw WHITENESS ROP mode not implemented\n");
+      DrawSetFillColorString(WmfDrawContext,"white");
       break;
     default:
+      printf("ipa_rop_draw 0x%x ROP mode not implemented\n", rop_draw->ROP);
       break;
     }
 
@@ -399,6 +439,9 @@ static void ipa_bmp_draw(wmfAPI *API, wmfBMP_Draw_t *bmp_draw)
   MonitorHandler
     handler;
 
+  PixelPacket
+    white;
+
   if (bmp_draw->bmp.data == 0)
     return;
 
@@ -407,7 +450,7 @@ static void ipa_bmp_draw(wmfAPI *API, wmfBMP_Draw_t *bmp_draw)
   if(!image)
     {
        ThrowException(&ddata->image->exception,exception.severity,
-                       exception.reason,exception.description);
+         exception.reason,exception.description);
        return;
     }
 
@@ -438,9 +481,22 @@ static void ipa_bmp_draw(wmfAPI *API, wmfBMP_Draw_t *bmp_draw)
         }
       else
         ThrowException(&ddata->image->exception,exception.severity,
-                       exception.reason,exception.description);
+          exception.reason,exception.description);
     }
 
+  QueryColorDatabase( "white", &white, &exception );
+
+  if ( ddata->image_info->texture ||
+       !(ColorMatch(&ddata->image_info->background_color, &white)) ||
+       ddata->image_info->background_color.opacity != OpaqueOpacity )
+  {
+    /*
+      Set image white background to transparent so that it may be
+      overlaid over non-white backgrounds.
+    */
+    TransparentImage( image, white, MaxRGB );
+  }
+
   width = AbsoluteValue(bmp_draw->pixel_width * (double) bmp_draw->crop.w);
   height = AbsoluteValue(bmp_draw->pixel_height * (double) bmp_draw->crop.h);
 
@@ -505,8 +561,8 @@ static void ipa_bmp_read(wmfAPI * API, wmfBMP_Read_t * bmp_read) {
         description[MaxTextExtent];
 
       FormatString(description,"packed DIB at offset %ld", bmp_read->offset);
-      ThrowException(&ddata->image->exception,CorruptImageError,exception.reason,
-                     exception.description);
+      ThrowException(&ddata->image->exception,CorruptImageError,
+        exception.reason,exception.description);
     }
   else
     {
@@ -540,7 +596,6 @@ static void ipa_device_open(wmfAPI * API)
     *draw_info;
 
   draw_info = CloneDrawInfo((ImageInfo*)NULL,(DrawInfo*)NULL);
-  draw_info->debug = ddata->image_info->debug;
 
   ddata->pattern_id = 0;
   ddata->clipping = False;
@@ -580,17 +635,17 @@ static void ipa_device_begin(wmfAPI * API)
   DrawSetViewbox(WmfDrawContext, 0, 0, ddata->image->columns, ddata->image->rows );
 
   FormatString(comment,"Created by ImageMagick %s (http://www.imagemagick.org/)",
-               MagickLibVersionText);
+               GetMagickVersion((unsigned long *) NULL));
   DrawComment(WmfDrawContext,comment);
 
   /* Scale width and height to image */
-  DrawSetScale(WmfDrawContext, ddata->scale_x, ddata->scale_y);
+  DrawScale(WmfDrawContext, ddata->scale_x, ddata->scale_y);
 
   /* Translate to TL corner of bounding box */
-  DrawSetTranslate(WmfDrawContext, ddata->translate_x, ddata->translate_y);
+  DrawTranslate(WmfDrawContext, ddata->translate_x, ddata->translate_y);
 
   /* Apply rotation */
-  DrawSetRotate(WmfDrawContext, ddata->rotate);
+  DrawRotate(WmfDrawContext, ddata->rotate);
 
   if(ddata->image_info->texture == NULL)
     {
@@ -616,6 +671,10 @@ static void ipa_device_begin(wmfAPI * API)
 
       image_info = CloneImageInfo((ImageInfo *) 0);
       strcpy(image_info->filename, ddata->image_info->texture);
+
+      if ( ddata->image_info->size )
+        CloneString(&image_info->size,ddata->image_info->size);
+
       image = ReadImage(image_info,&exception);
       DestroyImageInfo(image_info);
       if(image)
@@ -623,6 +682,7 @@ static void ipa_device_begin(wmfAPI * API)
           char
             pattern_id[30];
 
+          strcpy(image->magick,"MIFF");
           DrawPushDefs(WmfDrawContext);
           draw_pattern_push(API, ddata->pattern_id, image->columns, image->rows);
           DrawComposite(WmfDrawContext, CopyCompositeOp, 0, 0, image->columns, image->rows, image);
@@ -639,8 +699,9 @@ static void ipa_device_begin(wmfAPI * API)
         }
       else
         {
+          LogMagickEvent(CoderEvent,GetMagickModule(),"reading texture image failed!");
           ThrowException(&ddata->image->exception,exception.severity,
-                         exception.reason,exception.description);
+            exception.reason,exception.description);
         }
     }
 
@@ -1135,9 +1196,6 @@ static void ipa_functions(wmfAPI *API)
   /*
      Device data defaults
    */
-  ddata->mvg = 0;
-  ddata->mvg_alloc = 0;
-  ddata->mvg_length = 0;
   ddata->image = 0;
 }
 
@@ -1280,9 +1338,9 @@ static void ipa_draw_text(wmfAPI * API, wmfDrawText_t * draw_text)
           PixelPacket
             under_color;
 
-          under_color.red     = Upscale(box->r);
-          under_color.green   = Upscale(box->g);
-          under_color.blue    = Upscale(box->b);
+          under_color.red     = ScaleCharToQuantum(box->r);
+          under_color.green   = ScaleCharToQuantum(box->g);
+          under_color.blue    = ScaleCharToQuantum(box->b);
           under_color.opacity = OpaqueOpacity;
 
           DrawSetTextUnderColor(WmfDrawContext,&under_color);
@@ -1309,10 +1367,10 @@ static void ipa_draw_text(wmfAPI * API, wmfDrawText_t * draw_text)
   DrawSetFont(WmfDrawContext, WMF_FONT_PSNAME(font));
 
   /* Translate coordinates so target is 0,0 */
-  DrawSetTranslate(WmfDrawContext, XC(point.x), YC(point.y));
+  DrawTranslate(WmfDrawContext, XC(point.x), YC(point.y));
 
   /* Transform horizontal scale to draw text at 1:1 ratio */
-  DrawSetScale(WmfDrawContext, ddata->scale_y / ddata->scale_x, 1.0);
+  DrawScale(WmfDrawContext, ddata->scale_y / ddata->scale_x, 1.0);
 
   /* Apply rotation */
   /* ImageMagick's drawing rotation is clockwise from horizontal
@@ -1321,7 +1379,7 @@ static void ipa_draw_text(wmfAPI * API, wmfDrawText_t * draw_text)
   if (angle == 360)
     angle = 0;
   if (angle != 0)
-    DrawSetRotate(WmfDrawContext, angle);
+    DrawRotate(WmfDrawContext, angle);
 
   /*
    * Render text
@@ -1773,7 +1831,7 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
           dasharray[2] = 0;
 
           DrawSetStrokeAntialias(WmfDrawContext,False);
-          DrawSetStrokeDashArray(WmfDrawContext,dasharray);
+          DrawSetStrokeDashArray(WmfDrawContext,2,dasharray);
           break;
         }
       case PS_ALTERNATE:
@@ -1785,7 +1843,7 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
           dasharray[2] = 0;
 
           DrawSetStrokeAntialias(WmfDrawContext,False);
-          DrawSetStrokeDashArray(WmfDrawContext,dasharray);
+          DrawSetStrokeDashArray(WmfDrawContext,2,dasharray);
           break;
         }
       case PS_DASHDOT:    /* _._._._  */
@@ -1798,7 +1856,7 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
           dasharray[4] = 0;
 
           DrawSetStrokeAntialias(WmfDrawContext,False);
-          DrawSetStrokeDashArray(WmfDrawContext,dasharray);
+          DrawSetStrokeDashArray(WmfDrawContext,4,dasharray);
           break;
         }
       case PS_DASHDOTDOT:  /* _.._.._  */
@@ -1813,14 +1871,14 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
           dasharray[6] = 0;
 
           DrawSetStrokeAntialias(WmfDrawContext,False);
-          DrawSetStrokeDashArray(WmfDrawContext,dasharray);
+          DrawSetStrokeDashArray(WmfDrawContext,6,dasharray);
           break;
         }
       case PS_INSIDEFRAME:  /* There is nothing to do in this case... */
       case PS_SOLID:
       default:
         {
-          DrawSetStrokeDashArray(WmfDrawContext,(double *)NULL);
+          DrawSetStrokeDashArray(WmfDrawContext,0,(double *)NULL);
           break;
         }
       }
@@ -2064,7 +2122,7 @@ static void lite_font_map( wmfAPI* API, wmfFont* font)
   if(type_info_base == 0)
     {
       ThrowException(&ddata->image->exception,exception.severity,
-                     exception.reason,exception.description);
+        exception.reason,exception.description);
       return;
     }
 
@@ -2088,7 +2146,7 @@ static void lite_font_map( wmfAPI* API, wmfFont* font)
         target_weight = WMF_FONT_WEIGHT(font);
 
       /* printf("Desired weight  = %i\n", WMF_FONT_WEIGHT(font)); */
-      for ( type_info=type_info_base; type_info != 0; type_info=type_info->next )
+      for ( type_info=(TypeInfo *) type_info_base; type_info != 0; type_info=type_info->next )
         {
           if(LocaleCompare(wmf_font_name,type_info->family) == 0)
             {
@@ -2116,7 +2174,7 @@ static void lite_font_map( wmfAPI* API, wmfFont* font)
   /* Look for exact full match */
   if(!magick_font->ps_name)
     {
-      for ( type_info=type_info_base; type_info != 0; type_info=type_info->next)
+      for ( type_info=(TypeInfo *) type_info_base; type_info != 0; type_info=type_info->next)
         {
           if(LocaleCompare(wmf_font_name,type_info->description) == 0)
             {
@@ -2226,7 +2284,7 @@ static int ipa_blob_read(void* context)
 /* BLOB seek */
 static int ipa_blob_seek(void* context,long position)
 {
-  return (int)SeekBlob((Image*)context,(off_t)position,SEEK_SET);
+  return (int)SeekBlob((Image*)context,(ExtendedSignedIntegralType)position,SEEK_SET);
 }
 
 /* BLOB tell */
@@ -2255,6 +2313,9 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
     resolution_x,
     units_per_inch;
 
+  unsigned int
+    logging;
+
   unsigned long
     wmf_options_flags = 0;
 
@@ -2273,10 +2334,19 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
   wmfD_Rect
     bbox;
 
-  image = AllocateImage(image_info);
-  if (!OpenBlob(image_info,image,ReadBinaryType,exception))
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+  logging=LogMagickEvent(CoderEvent,GetMagickModule(),"enter ReadWMFImage()");
 
+  image = AllocateImage(image_info);
+  if (!OpenBlob(image_info,image,ReadBinaryBlobMode,exception))
+    {
+      if(logging)
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  OpenBlob failed");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+        }
+      ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+    }
+  
   /*
    * Create WMF API
    *
@@ -2295,7 +2365,12 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
     {
       if (API)
         wmf_api_destroy(API);
-      ThrowReaderException(DelegateFatalError, "Failed to intialize libwmf", image);
+      if(logging)
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_api_create failed");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+        }
+      ThrowReaderException(DelegateError,"UnableToInitializeWMFLibrary",image);
     }
 
   /* Register progress monitor */
@@ -2321,18 +2396,30 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
   if (wmf_error != wmf_E_None)
     {
       wmf_api_destroy(API);
-      ThrowReaderException(FileOpenFatalError, "Unable to open file", image);
+      if(logging)
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_bbuf_input failed");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+        }
+      ThrowReaderException(FileOpenError,"UnableToOpenFile", image);
     }
 
   /*
    * Scan WMF file
    *
    */
+  if (logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Scanning WMF to obtain bounding box");
   wmf_error = wmf_scan(API, 0, &bbox);
   if (wmf_error != wmf_E_None)
     {
       wmf_api_destroy(API);
-      ThrowReaderException(CorruptImageFatalError, "Failed to scan file", image);
+      if (logging)
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_scan failed with wmf_error %d", wmf_error);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+        }
+      ThrowReaderException(DelegateError,"FailedToScanFile",image);
     }
 
   /*
@@ -2364,8 +2451,12 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
   if (wmf_error != wmf_E_None)
     {
       wmf_api_destroy(API);
-      ThrowReaderException(CorruptImageFatalError,
-                           "Failed to compute output size", image);
+      if (logging)
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_size failed with wmf_error %d", wmf_error);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+        }
+      ThrowReaderException(DelegateError,"FailedToComputeOutputSize",image);
     }
 
   /* Obtain (or guess) metafile units */
@@ -2414,26 +2505,32 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
       ddata->translate_y = 0-bbox.BR.y;
     }
 
+  if (logging)
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Placeable metafile:          %s",
+                     (API)->File->placeable ? "Yes" : "No");
+      
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Size in metafile units:      %.4gx%.4g",
+                     wmf_width, wmf_height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Metafile units/inch:         %.4g",
+                     units_per_inch);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Size in inches:              %.4gx%.4g",
+                     image_width_inch,image_height_inch);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Bounding Box:                %.4g,%.4g %.4g,%.4g",
+                     bbox.TL.x, bbox.TL.y, bbox.BR.x, bbox.BR.y);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Bounding width x height:     %.4gx%.4g",
+                     bounding_width, bounding_height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Output resolution:           %.4gx%.4g",
+                     resolution_x, resolution_y);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Image size:                  %.4gx%.4g",
+                     image_width, image_height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Bounding box scale factor:   %.4g,%.4g",
+                     ddata->scale_x, ddata->scale_y);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Translation:                 %.4g,%.4g",
+                     ddata->translate_x, ddata->translate_y);
+    }
+
 #if 0
-  printf("\nPlaceable metafile:          ");
-  if ((API)->File->placeable)
-    printf("Yes\n");
-  else
-    printf("No\n");
-  printf("Size in metafile units:      %.4gx%.4g\n", wmf_width, wmf_height);
-  printf("Metafile units/inch:         %.4g\n", units_per_inch);
-  printf("Size in inches:              %.4gx%.4g\n",image_width_inch,image_height_inch);
-  printf("Bounding Box:                %.4g,%.4g %.4g,%.4g\n",
-         bbox.TL.x, bbox.TL.y, bbox.BR.x, bbox.BR.y);
-  printf("Bounding width x height:     %.4gx%.4g\n", bounding_width, bounding_height);
-  printf("Output resolution:           %.4gx%.4g\n", resolution_x, resolution_y);
-  printf("Image size:                  %.4gx%.4g\n", image_width, image_height);
-  printf("Bounding box scale factor:   %.4g,%.4g\n",
-         ddata->scale_x, ddata->scale_y);
-  printf("Translation:                 %.4g,%.4g\n",
-   ddata->translate_x, ddata->translate_y);
-
-
 #if 0
   {
     typedef struct _wmfPlayer_t wmfPlayer_t;
@@ -2475,7 +2572,6 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
    * Create canvas image
    *
    */
-
   image->rows = (unsigned long)ceil(image_height);
   image->columns = (unsigned long)ceil(image_width);
 
@@ -2483,9 +2579,16 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
     {
       wmf_api_destroy(API);
       CloseBlob(image);
+      if(logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
       return(image);
     }
 
+  if (logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "  Creating canvas image with size %ldx%ld",
+                          image->rows, image->columns);
+
   /*
    * Set solid background color
    */
@@ -2515,29 +2618,48 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
       }
   }
   /*
-   * Play file to generate MVG drawing commands
+   * Play file to generate Vector drawing commands
    *
    */
 
-  ddata->mvg = NULL;
+  if (logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Playing WMF to prepare vectors");
+
   wmf_error = wmf_play(API, 0, &bbox);
   if (wmf_error != wmf_E_None)
     {
       wmf_api_destroy(API);
-      ThrowReaderException(CorruptImageFatalError, "Failed to render file", image);
+      if (logging)
+        {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Playing WMF failed with wmf_error %d", wmf_error);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+        }
+      ThrowReaderException(DelegateError,"FailedToRenderFile",image);
     }
 
   /*
    * Scribble on canvas image
    *
    */
-  
+
+  if (logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Rendering WMF vectors");
   DrawRender(ddata->draw_context);
 
   /* Cleanup allocated data */
   wmf_api_destroy(API);
   CloseBlob(image);
 
+  // Check for and report any rendering error
+  if(image->exception.severity != UndefinedException)
+    ThrowException(exception,
+                   CoderWarning,
+                   ddata->image->exception.reason,
+                   ddata->image->exception.description);
+
+  if(logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+
   /* Return image */
   return image;
 }
@@ -2549,6 +2671,46 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s W M F                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsWMF returns True if the image format type, identified by the magick
+%  string, is WMF.
+%
+%  The format of the IsWMF method is:
+%
+%      unsigned int IsWMF(const unsigned char *magick,const size_t length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsWMF returns True if the image format type is WMF.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+static unsigned int IsWMF(const unsigned char *magick,const size_t length)
+{
+  if (length < 4)
+    return(False);
+  if (memcmp(magick,"\327\315\306\232",4) == 0)
+    return(True);
+  if (memcmp(magick,"\001\000\011\000",4) == 0)
+    return(True);
+  return(False);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e g i s t e r W M F I m a g e                                           %
 %                                                                             %
 %                                                                             %
@@ -2569,31 +2731,19 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
 */
 ModuleExport void RegisterWMFImage(void)
 {
-#if defined(HasWMF) || defined(HasWMFlite)
   MagickInfo
     *entry;
 
   entry = SetMagickInfo("WMF");
+#if defined(HasWMF) || defined(HasWMFlite)
   entry->decoder = ReadWMFImage;
-  entry->description = AcquireString("Windows Meta File");
-  entry->blob_support = False;
-  entry->module = AcquireString("WMF");
-  (void) RegisterMagickInfo(entry);
-#if defined(HasWIN32WMFAPI)
-  entry = SetMagickInfo("EMF");
-  entry->decoder = ReadEMFImage;
-  entry->description = AcquireString("Windows WIN32 API rendered Enhanced Meta File");
-  entry->blob_support = False;
-  entry->module = AcquireString("WMF");
-  (void) RegisterMagickInfo(entry);
-  entry = SetMagickInfo("WMFWIN32");
-  entry->decoder = ReadEMFImage;
-  entry->description = AcquireString("Windows WIN32 API rendered Meta File");
-  entry->blob_support = False;
-  entry->module = AcquireString("WMFWIN32");
-  (void) RegisterMagickInfo(entry);
-#endif
 #endif /* HasWMF || HasWMFlite */
+  entry->magick=(MagickHandler) IsWMF;
+  entry->description = AcquireString("Windows Meta File");
+  entry->blob_support = True;
+  entry->seekable_stream=True;
+  entry->module = AcquireString("WMF");
+  (void) RegisterMagickInfo(entry);
 }
 
 /*
@@ -2619,9 +2769,5 @@ ModuleExport void UnregisterWMFImage(void)
 {
 #if defined(HasWMF) || defined(HasWMFlite)
   (void) UnregisterMagickInfo("WMF");
-#if defined(HasWIN32WMFAPI)
-  (void) UnregisterMagickInfo("EMF");
-  (void) UnregisterMagickInfo("WMFWIN32");
-#endif
 #endif /* defined(HasWMF) */
 }
