@@ -49,6 +49,11 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -61,8 +66,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsWPG returns True if the image format type, identified by the
-%  magick string, is WPG.
+%  IsWPG() returns True if the image format type, identified by the magick
+%  string, is WPG.
 %
 %  The format of the IsWPG method is:
 %
@@ -152,7 +157,7 @@ register IndexPacket *indexes;
                  break;
 /*            if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
             break;
             }
       case 2:  /* Convert PseudoColor scanline. */
@@ -166,10 +171,10 @@ register IndexPacket *indexes;
                  index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
                  indexes[x]=index;
                  *q++=image->colormap[index];
-     index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
+                 index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
                  indexes[x]=index;
                  *q++=image->colormap[index];
-     index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
+                 index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
                  indexes[x]=index;
                  *q++=image->colormap[index];
                  index=ConstrainColormapIndex(image,(*p) & 0x3);
@@ -202,7 +207,7 @@ register IndexPacket *indexes;
                  break;
 /*         if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
            break;
            }
       
@@ -233,7 +238,7 @@ register IndexPacket *indexes;
                  break;
 /*         if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
            break;
            }
       case 8: /* Convert PseudoColor scanline. */
@@ -252,7 +257,7 @@ register IndexPacket *indexes;
                  break;
 /*           if (image->previous == (Image *) NULL)
                  if (QuantumTick(y,image->rows))
-                   MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+                   MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
            }
            break;
      
@@ -262,16 +267,16 @@ register IndexPacket *indexes;
       break;
           for (x=0; x < (long) image->columns; x++)
              {
-             q->red=Upscale(*p++);
-             q->green=Upscale(*p++);
-             q->blue=Upscale(*p++);
+             q->red=ScaleCharToQuantum(*p++);
+             q->green=ScaleCharToQuantum(*p++);
+             q->blue=ScaleCharToQuantum(*p++);
              q++;
              }
           if (!SyncImagePixels(image))
             break;
 /*          if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,image->rows-y-1,image->rows);*/
+              MagickMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
           break;     
        }
 }
@@ -372,23 +377,35 @@ char SampleSize=1;
 
      switch(bbuf)
        {
-       case 0x7D:SampleSize=ReadBlobByte(image);  
+       case 0x7D:SampleSize=ReadBlobByte(image);  /* DSZ */
      if(SampleSize>8) return(-2);
      if(SampleSize<1) return(-2);
      break;
-       case 0x7E:
+       case 0x7E:fprintf(stderr,"\nUnsupported WPG token XOR, please report!");  // XOR
      break;
-       case 0x7F:RunCount=ReadBlobByte(image);  
+       case 0x7F:RunCount=ReadBlobByte(image);   /* BLK */
      for(i=0;i<(int) SampleSize*((int)RunCount+1);i++)
       {
       InsertRByte(0);
       }
      break;
-       case 0xFD:
+       case 0xFD:fprintf(stderr,"\nUnsupported WPG2 token EXT, please report!");  // EXT
      break;
-       case 0xFE:
+       case 0xFE:RunCount=ReadBlobByte(image);  /* RST */
+     if(x!=0)
+      {
+      fprintf(stderr,"\nUnsupported WPG2 unaligned token RST x=%d, please report!\n",x);
+      return(-3);
+      }
+      {            
+      for(i=0;i<=RunCount;i++) /* duplicate the previous row RunCount x */
+      {      
+      InsertRow(BImgBuff,(long) (image->rows>y?image->rows-y-1:0),image);
+      y++;
+      }    
+                  }
      break;
-       case 0xFF:RunCount=ReadBlobByte(image);
+       case 0xFF:RunCount=ReadBlobByte(image);          /* WHT */
      for(i=0;i<(int) SampleSize*((int)RunCount+1);i++)
       {
       InsertRByte(0xFF);
@@ -421,7 +438,7 @@ char SampleSize=1;
 
 
 static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
-  off_t PS_Offset,long PS_Size)
+  ExtendedSignedIntegralType PS_Offset,long PS_Size)
 {
 /*char Filename[MaxTextExtent];*/
 FILE *f;
@@ -455,7 +472,7 @@ if(image2!=NULL)   /* Allocate next image structure. */
     image2->next=image;
     
     while(image->next!=NULL)
-       image=image->next;
+       image=SyncNextImageInList(image);
     }
     
 FINISH_UNL:    
@@ -502,7 +519,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 typedef struct
   {
   unsigned long FileId;
-  off_t DataOffset;
+  ExtendedSignedIntegralType DataOffset;
   unsigned int ProductType;
   unsigned int FileType;
   unsigned char MajorVersion;
@@ -581,14 +598,14 @@ typedef struct {
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read WPG image.
   */
    Header.FileId=ReadBlobLSBLong(image);
-   Header.DataOffset=(off_t) ReadBlobLSBLong(image);
+   Header.DataOffset=(ExtendedSignedIntegralType) ReadBlobLSBLong(image);
    Header.ProductType=ReadBlobLSBShort(image);
    Header.FileType=ReadBlobLSBShort(image);
    Header.MajorVersion=ReadBlobByte(image);
@@ -597,9 +614,9 @@ typedef struct {
    Header.Reserved=ReadBlobLSBShort(image);
 
   if (Header.FileId!=0x435057FF || (Header.ProductType>>8)!=0x16 )
-      ThrowReaderException(CorruptImageError,"Not a WPG image file",image);
-  if(Header.EncryptKey!=0 )
-      ThrowReaderException(CorruptImageError,"Encrypted WPG image file",image);
+    ThrowReaderException(CorruptImageError,"NotAWPGImageFile",image);
+  if (Header.EncryptKey!=0 )
+    ThrowReaderException(CoderError,"EncryptedWPGImageFileNotSupported",image);
 
   image->colors = 0;
 
@@ -649,9 +666,9 @@ typedef struct {
       goto NoMemory;
      for (i=WPG_Palette.StartIndex; i < (int)WPG_Palette.NumOfEntries; i++)
        {
-       image->colormap[i].red=Upscale(ReadBlobByte(image));
-       image->colormap[i].green=Upscale(ReadBlobByte(image));
-       image->colormap[i].blue=Upscale(ReadBlobByte(image));
+       image->colormap[i].red=ScaleCharToQuantum(ReadBlobByte(image));
+       image->colormap[i].green=ScaleCharToQuantum(ReadBlobByte(image));
+       image->colormap[i].blue=ScaleCharToQuantum(ReadBlobByte(image));
        }
      break;
      
@@ -696,7 +713,7 @@ UnpackRaster:
          image->colors=1 << image->depth;
          if (!AllocateImageColormap(image,image->colors))
       {
-NoMemory:    ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+NoMemory:    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
            image)
       }
          }
@@ -713,19 +730,20 @@ NoMemory:    ThrowReaderException(ResourceLimitError,"Memory allocation failed",
         {  /*fix crippled monochrome palette*/
         image->colormap[1].red =
         image->colormap[1].green =
-        image->colormap[1].blue = Upscale(255);
+        image->colormap[1].blue = ScaleCharToQuantum(255);
         }
            }      
 
      if(UnpackWPGRaster(image)<0) /* The raster cannot be unpacked */
          {
-DecompressionFailed: ThrowReaderException(ResourceLimitError,"Cannot decompress WPG raster",image)
+DecompressionFailed: ThrowReaderException(CoderError,"UnableToDecompressImage",
+  image)
          }
 
      /* Allocate next image structure. */
      AllocateNextImage(image_info,image);
      if (image->next == (Image *) NULL) goto Finish;
-     image=image->next;
+     image=SyncNextImageInList(image);
      image->columns=image->rows=0;
      image->colors=image->depth=0;
      break;
@@ -763,13 +781,12 @@ DecompressionFailed: ThrowReaderException(ResourceLimitError,"Cannot decompress
 
        image->colors=WPG_Palette.NumOfEntries;
        if (!AllocateImageColormap(image,image->colors))
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-           image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
        for (i=WPG_Palette.StartIndex; i < (int)WPG_Palette.NumOfEntries; i++)
        {
-       image->colormap[i].red=Upscale(ReadBlobByte(image));
-       image->colormap[i].green=Upscale(ReadBlobByte(image));
-       image->colormap[i].blue=Upscale(ReadBlobByte(image));
+       image->colormap[i].red=ScaleCharToQuantum(ReadBlobByte(image));
+       image->colormap[i].green=ScaleCharToQuantum(ReadBlobByte(image));
+       image->colormap[i].blue=ScaleCharToQuantum(ReadBlobByte(image));
        (void) ReadBlobByte(image);   /*Opacity??*/
        }
        break;
@@ -830,7 +847,7 @@ DecompressionFailed: ThrowReaderException(ResourceLimitError,"Cannot decompress
       /* Allocate next image structure. */
        AllocateNextImage(image_info,image);
        if (image->next == (Image *) NULL) goto Finish;
-       image=image->next;
+       image=SyncNextImageInList(image);
        image->columns=image->rows=0;
        image->colors=image->depth=0;
        break;
@@ -849,7 +866,7 @@ DecompressionFailed: ThrowReaderException(ResourceLimitError,"Cannot decompress
       break;
 
    default:
-     ThrowReaderException(CorruptImageError,"Unsupported level of WPG image",image)
+     ThrowReaderException(CoderError,"Unsupportedlevel",image)
    }
 
 
@@ -897,10 +914,11 @@ ModuleExport void RegisterWPGImage(void)
     *entry;
 
   entry=SetMagickInfo("WPG");
-  entry->decoder=ReadWPGImage;
-  entry->magick=IsWPG;
+  entry->decoder=(DecoderHandler) ReadWPGImage;
+  entry->magick=(MagickHandler) IsWPG;
   entry->description=AcquireString("Word Perfect Graphics");
   entry->module=AcquireString("WPG");
+  entry->seekable_stream=True;
   (void) RegisterMagickInfo(entry);
 }
 
