@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,9 +53,11 @@
   Include declarations.
 */
 #include "studio.h"
-#if defined(HasX11)
+#include "blob.h"
+#include "magick.h"
+#include "static.h"
+#include "utility.h"
 #include "xwindow.h"
-#endif
 
 /*
   Forward declarations.
@@ -100,8 +102,8 @@ static Image *ReadXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #else
 static Image *ReadXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  ThrowException(exception,MissingDelegateError,
-    "X11 library is not available",image_info->filename);
+  ThrowException(exception,MissingDelegateError,"XWindowLibraryIsNotAvailable",
+    image_info->filename);
   return((Image *) NULL);
 }
 #endif
@@ -135,8 +137,8 @@ ModuleExport void RegisterXImage(void)
     *entry;
 
   entry=SetMagickInfo("X");
-  entry->decoder=ReadXImage;
-  entry->encoder=WriteXImage;
+  entry->decoder=(DecoderHandler) ReadXImage;
+  entry->encoder=(EncoderHandler) WriteXImage;
   entry->adjoin=False;
   entry->description=AcquireString("X Image");
   entry->module=AcquireString("X");
@@ -219,8 +221,7 @@ static unsigned int WriteXImage(const ImageInfo *image_info,Image *image)
   */
   display=XOpenDisplay(image_info->server_name);
   if (display == (Display *) NULL)
-    ThrowWriterException(ResourceLimitError,"Unable to connect to X server",
-      image);
+    ThrowWriterException(ResourceLimitError,"UnableToOpenXServer",image);
   /*
     Set our forgiving error handler.
   */
@@ -243,7 +244,7 @@ static unsigned int WriteXImage(const ImageInfo *image_info,Image *image)
 #else
 static unsigned int WriteXImage(const ImageInfo *image_info,Image *image)
 {
-  ThrowBinaryException(MissingDelegateError,"X11 library is not available",
+  ThrowBinaryException(MissingDelegateError,"XWindowLibraryIsNotAvailable",
     image->filename);
 }
 #endif
