@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -93,7 +99,7 @@ static unsigned int IsXBM(const unsigned char *magick,const size_t length)
 {
   if (length < 7)
     return(False);
-  if (LocaleNCompare((char *) magick,"#define",7) == 0)
+  if (memcmp(magick,"#define",7) == 0)
     return(True);
   return(False);
 }
@@ -195,14 +201,14 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *data;
 
   unsigned int
-    byte,
-    padding,
-    status,
-    value,
-    version;
+    status;
 
   unsigned long
-    bytes_per_line;
+    byte,
+    bytes_per_line,
+    padding,
+    value,
+    version;
 
   /*
     Open image file.
@@ -212,9 +218,9 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read X bitmap header.
   */
@@ -256,29 +262,28 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
   }
   if ((image->columns == 0) || (image->rows == 0) || EOFBlob(image))
-    ThrowReaderException(CorruptImageError,
-      "XBM file is not in the correct format",image);
+    ThrowReaderException(CorruptImageError,"NotAXBMImageFile",image);
   /*
     Initialize image structure.
   */
   if (!AllocateImageColormap(image,image->colors))
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   padding=0;
   if ((image->columns % 16) && ((image->columns % 16) < 9)  && (version == 10))
     padding=1;
   bytes_per_line=(image->columns+7)/8+padding;
   data=(unsigned char *) AcquireMemory(bytes_per_line*image->rows);
   if (data == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Initialize colormap.
   */
-  image->colormap[0].red=0;
-  image->colormap[0].green=0;
-  image->colormap[0].blue=0;
-  image->colormap[1].red=MaxRGB;
-  image->colormap[1].green=MaxRGB;
-  image->colormap[1].blue=MaxRGB;
+  image->colormap[0].red=MaxRGB;
+  image->colormap[0].green=MaxRGB;
+  image->colormap[0].blue=MaxRGB;
+  image->colormap[1].red=0;
+  image->colormap[1].green=0;
+  image->colormap[1].blue=0;
   if (image_info->ping)
     {
       CloseBlob(image);
@@ -323,15 +328,15 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     for (i=0; i < (long) (bytes_per_line*image->rows); (i+=2))
     {
       value=XBMInteger(image,hex_digits);
-      *p++=value;
+      *p++=(unsigned char) value;
       if (!padding || ((i+2) % bytes_per_line))
-        *p++=value >> 8;
+        *p++=(unsigned char) (value >> 8);
     }
   else
     for (i=0; i < (long) (bytes_per_line*image->rows); i++)
     {
       value=XBMInteger(image,hex_digits);
-      *p++=value;
+      *p++=(unsigned char) value;
     }
   /*
     Convert X bitmap image to pixel packets.
@@ -349,7 +354,7 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       if (bit == 0)
         byte=(*p++);
-      indexes[x]=byte & 0x01 ? 0 : 1;
+      indexes[x]=byte & 0x01 ? 0x01 : 0x00;
       bit++;
       byte>>=1;
       if (bit == 8)
@@ -358,7 +363,8 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-       MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        break;
   }
   LiberateMemory((void **) &data);
   SyncImage(image);
@@ -395,9 +401,9 @@ ModuleExport void RegisterXBMImage(void)
     *entry;
 
   entry=SetMagickInfo("XBM");
-  entry->decoder=ReadXBMImage;
-  entry->encoder=WriteXBMImage;
-  entry->magick=IsXBM;
+  entry->decoder=(DecoderHandler) ReadXBMImage;
+  entry->encoder=(EncoderHandler) WriteXBMImage;
+  entry->magick=(MagickHandler) IsXBM;
   entry->adjoin=False;
   entry->description=
     AcquireString("X Windows system bitmap (black and white)");
@@ -470,7 +476,7 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register int
+  register long
     x;
 
   register IndexPacket
@@ -482,9 +488,11 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
     polarity;
 
   unsigned int
-    count,
     status;
 
+  unsigned long
+    count;
+
   /*
     Open output image file.
   */
@@ -492,10 +500,10 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   /*
     Write X bitmap header.
   */
@@ -511,12 +519,11 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
   /*
     Convert MIFF to X bitmap pixels.
   */
-  if ((image->storage_class == DirectClass) ||
-      !IsMonochromeImage(image,&image->exception))
-    SetImageType(image,BilevelType);
-  polarity=Intensity(&image->colormap[0]) > (0.5*MaxRGB);
+  SetImageType(image,BilevelType);
+  polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
   if (image->colors == 2)
-    polarity=Intensity(&image->colormap[0]) > Intensity(&image->colormap[1]);
+    polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+      PixelIntensityToQuantum(&image->colormap[1]);
   bit=0;
   byte=0;
   count=0;
@@ -533,7 +540,7 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
     for (x=0; x < (long) image->columns; x++)
     {
       byte>>=1;
-      if (indexes[x] == polarity)
+      if (indexes[x] != polarity)
         byte|=0x80;
       bit++;
       if (bit == 8)
@@ -541,7 +548,7 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
           /*
             Write a bitmap byte to the image file.
           */
-          FormatString(buffer,"0x%02x, ",(unsigned int) (byte & 0xff));
+          FormatString(buffer,"0x%02X, ",(unsigned int) (byte & 0xff));
           (void) WriteBlob(image,strlen(buffer),buffer);
           count++;
           if (count == 12)
@@ -561,7 +568,7 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
           Write a bitmap byte to the image file.
         */
         byte>>=(8-bit);
-        FormatString(buffer,"0x%02x, ",(unsigned int) (byte & 0xff));
+        FormatString(buffer,"0x%02X, ",(unsigned int) (byte & 0xff));
         (void) WriteBlob(image,strlen(buffer),buffer);
         count++;
         if (count == 12)
@@ -574,7 +581,8 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
         byte=0;
       };
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+        break;
   }
   (void) strcpy(buffer,"};\n");
   (void) WriteBlob(image,strlen(buffer),buffer);
