@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,11 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "static.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -91,6 +96,9 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
+  unsigned int
+    status;
+
   /*
     Initialize Image structure.
   */
@@ -104,16 +112,18 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image->rows == 0)
     image->rows=1;
   (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-  (void) QueryColorDatabase((char *) image_info->filename,
+  status=QueryColorDatabase((char *) image_info->filename,
     &image->background_color,exception);
-  if (image->background_color.opacity == OpaqueOpacity)
+  if (status == False)
     {
-      if (!AllocateImageColormap(image,1))
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
-      image->colormap[0]=image->background_color;
+      DestroyImage(image);
+      return((Image *) NULL);
     }
-  SetImage(image,OpaqueOpacity);
+  if (!AllocateImageColormap(image,1))
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+  image->colormap[0]=image->background_color;
+  SetImage(image,image->background_color.opacity);
+  image->depth=GetImageDepth(image,exception);
   return(image);
 }
 
@@ -146,7 +156,7 @@ ModuleExport void RegisterXCImage(void)
     *entry;
 
   entry=SetMagickInfo("XC");
-  entry->decoder=ReadXCImage;
+  entry->decoder=(DecoderHandler) ReadXCImage;
   entry->adjoin=False;
   entry->raw=True;
   entry->description=AcquireString("Constant image uniform color");
