@@ -18,7 +18,7 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "quantize.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Typedef declarations.
@@ -129,10 +135,10 @@ typedef struct {
           apply_mask,
           show_mask,
           edit_mask,
-          floating_offset,
-          offset_x,
-          offset_y,
-          mode,
+          floating_offset;
+  long offset_x,
+          offset_y;
+  unsigned long mode,
           tattoo;
   Image*      image;
 } XCFLayerInfo;
@@ -217,29 +223,28 @@ typedef enum
 static CompositeOperator GIMPBlendModeToCompositeOperator( unsigned long blendMode )
 {
   switch ( blendMode ) {
-    case GIMP_NORMAL_MODE:    return( OverCompositeOp );      break;
-    case GIMP_DISSOLVE_MODE:  return( DissolveCompositeOp );    break;
-    case GIMP_MULTIPLY_MODE:  return( MultiplyCompositeOp );    break;
-    case GIMP_SCREEN_MODE:    return( ScreenCompositeOp );    break;
-    case GIMP_OVERLAY_MODE:    return( OverlayCompositeOp );    break;
-    case GIMP_DIFFERENCE_MODE:  return( DifferenceCompositeOp );  break;
-    case GIMP_ADDITION_MODE:  return( AddCompositeOp );      break;
-    case GIMP_SUBTRACT_MODE:  return( SubtractCompositeOp );    break;
-    case GIMP_DARKEN_ONLY_MODE:  return( DarkenCompositeOp );    break;
-    case GIMP_LIGHTEN_ONLY_MODE:return( LightenCompositeOp );    break;
-    case GIMP_HUE_MODE:      return( HueCompositeOp );      break;
-    case GIMP_SATURATION_MODE:  return( SaturateCompositeOp );    break;
-    case GIMP_COLOR_MODE:    return( ColorizeCompositeOp );    break;
+    case GIMP_NORMAL_MODE:    return( OverCompositeOp );
+    case GIMP_DISSOLVE_MODE:  return( DissolveCompositeOp );
+    case GIMP_MULTIPLY_MODE:  return( MultiplyCompositeOp );
+    case GIMP_SCREEN_MODE:    return( ScreenCompositeOp );
+    case GIMP_OVERLAY_MODE:    return( OverlayCompositeOp );
+    case GIMP_DIFFERENCE_MODE:  return( DifferenceCompositeOp );
+    case GIMP_ADDITION_MODE:  return( AddCompositeOp );
+    case GIMP_SUBTRACT_MODE:  return( SubtractCompositeOp );
+    case GIMP_DARKEN_ONLY_MODE:  return( DarkenCompositeOp );
+    case GIMP_LIGHTEN_ONLY_MODE:return( LightenCompositeOp );
+    case GIMP_HUE_MODE:      return( HueCompositeOp );
+    case GIMP_SATURATION_MODE:  return( SaturateCompositeOp );
+    case GIMP_COLOR_MODE:    return( ColorizeCompositeOp );
     /* these are the ones we don't support...yet */
-    case GIMP_DODGE_MODE:    return( OverCompositeOp );    break;
-    case GIMP_BURN_MODE:    return( OverCompositeOp );    break;
-    case GIMP_HARDLIGHT_MODE:  return( OverCompositeOp );    break;
-    case GIMP_BEHIND_MODE:    return( OverCompositeOp );    break;
-    case GIMP_VALUE_MODE:    return( OverCompositeOp );    break;
-    case GIMP_DIVIDE_MODE:    return( OverCompositeOp );    break;
+    case GIMP_DODGE_MODE:    return( OverCompositeOp );
+    case GIMP_BURN_MODE:    return( OverCompositeOp );
+    case GIMP_HARDLIGHT_MODE:  return( OverCompositeOp );
+    case GIMP_BEHIND_MODE:    return( OverCompositeOp );
+    case GIMP_VALUE_MODE:    return( OverCompositeOp );
+    case GIMP_DIVIDE_MODE:    return( OverCompositeOp );
     default:
         return( OverCompositeOp );
-        break;
   }
 }
 
@@ -249,7 +254,7 @@ static CompositeOperator GIMPBlendModeToCompositeOperator( unsigned long blendMo
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   R e a d B l o b S t r i n g W i t h L o n g S i z e                                              %
++   R e a d B l o b S t r i n g W i t h L o n g S i z e                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -300,7 +305,7 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string)
 
 
 static int load_tile (Image* image, Image* tile_image, XCFDocInfo* inDocInfo, 
-            XCFLayerInfo*  inLayerInfo, int data_length)
+            XCFLayerInfo*  inLayerInfo, size_t data_length)
 {
   int i, nmemb_read_successfully;
   PixelPacket*  q;
@@ -319,17 +324,16 @@ static int load_tile (Image* image, Image* tile_image, XCFDocInfo* inDocInfo,
   */
   for( i=0; i<nmemb_read_successfully;i++ ) {
     if ( inDocInfo->image_type == GIMP_GRAY ) {
-      q->red    = Upscale(*graydata);
-      q->green  = Upscale(*graydata);
-      q->blue    = Upscale(*graydata);
-      q->opacity  = Upscale(255-inLayerInfo->opacity);
-
+      q->red    = ScaleCharToQuantum(*graydata);
+      q->green  = ScaleCharToQuantum(*graydata);
+      q->blue    = ScaleCharToQuantum(*graydata);
+      q->opacity  = ScaleCharToQuantum(255-inLayerInfo->opacity);
       graydata++;
     } else if ( inDocInfo->image_type == GIMP_RGB ) {
-      q->red    = Upscale(xcfdata->red);
-      q->green  = Upscale(xcfdata->green);
-      q->blue    = Upscale(xcfdata->blue);
-      q->opacity  = xcfdata->opacity==0 ? TransparentOpacity : Upscale(255-inLayerInfo->opacity);  
+      q->red    = ScaleCharToQuantum(xcfdata->red);
+      q->green  = ScaleCharToQuantum(xcfdata->green);
+      q->blue    = ScaleCharToQuantum(xcfdata->blue);
+      q->opacity  = (Quantum) (xcfdata->opacity==0 ? TransparentOpacity : ScaleCharToQuantum(255-inLayerInfo->opacity)); 
 
       xcfdata++;
     };
@@ -345,7 +349,7 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
               XCFLayerInfo*  inLayerInfo, int data_length)
 {
   unsigned char data, val;
-  int size;
+  ExtendedSignedIntegralType size;
   int count;
   int length;
   int bpp;    /* BYTES per pixel! */
@@ -369,114 +373,130 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
       count = 0;
 
       while (size > 0)
-  {
-    if (xcfdata > xcfdatalimit)
       {
-        goto bogus_rle;
-      }
-
-    val = *xcfdata++;
-
-    length = val;
-    if (length >= 128)
-      {
-        length = 255 - (length - 1);
-        if (length == 128)
-    {
-      if (xcfdata >= xcfdatalimit)
-        {
-          goto bogus_rle;
-        }
-
-      length = (*xcfdata << 8) + xcfdata[1];
-      xcfdata += 2;
-    }
-
-        count += length;
-        size -= length;
-
-        if (size < 0)
-    {
-      goto bogus_rle;
-    }
-
-        if (&xcfdata[length-1] > xcfdatalimit)
-    {
-      goto bogus_rle;
-    }
-
-        while (length-- > 0)
-    {
-      data = *xcfdata++;
-      switch (i) {
-        case 0:  
-        q->red    = Upscale(data);  
-        if ( inDocInfo->image_type == GIMP_GRAY ) {
-          q->green  = Upscale(data);
-          q->blue    = Upscale(data);
-          q->opacity  = Upscale(255-inLayerInfo->opacity);
-        }
-        break;
-        case 1:  q->green  = Upscale(data);  break;
-        case 2:  q->blue    = Upscale(data);  break;
-        case 3:  
-          q->opacity = data==0 ? TransparentOpacity : Upscale(255-inLayerInfo->opacity);  
-        break;
-      }
-      q++;
-    }
-      }
-    else
-      {
-        length += 1;
-        if (length == 128)
-    {
-      if (xcfdata >= xcfdatalimit)
-        {
-          goto bogus_rle;
-        }
-
-      length = (*xcfdata << 8) + xcfdata[1];
-      xcfdata += 2;
-    }
-
-        count += length;
-        size -= length;
-
-              if (size < 0)
-    {
-      goto bogus_rle;
-    }
-
         if (xcfdata > xcfdatalimit)
-    {
-      goto bogus_rle;
-    }
-
+          {
+            goto bogus_rle;
+          }
+    
         val = *xcfdata++;
-
-        for (j = 0; j < length; j++)
-    {
-      data = val;
-      switch (i) {
-        case 0:  
-          q->red    = Upscale(data);  
-        if ( inDocInfo->image_type == GIMP_GRAY ) {
-          q->green  = Upscale(data);
-          q->blue    = Upscale(data);
-          q->opacity  = Upscale(255-inLayerInfo->opacity);
-        }
-        break;
-        case 1:  q->green  = Upscale(data);  break;
-        case 2:  q->blue    = Upscale(data);  break;
-        case 3:  
-          q->opacity = data==0 ? TransparentOpacity : Upscale(255-inLayerInfo->opacity);  
-        break;
+    
+        length = val;
+        if (length >= 128)
+          {
+            length = 255 - (length - 1);
+            if (length == 128)
+              {
+                if (xcfdata >= xcfdatalimit)
+                  {
+                    goto bogus_rle;
+                  }
+    
+                length = (*xcfdata << 8) + xcfdata[1];
+                xcfdata += 2;
+              }
+    
+            count += length;
+            size -= length;
+    
+            if (size < 0)
+              {
+                goto bogus_rle;
+              }
+    
+            if (&xcfdata[length-1] > xcfdatalimit)
+              {
+                goto bogus_rle;
+              }
+    
+            while (length-- > 0)
+            {
+              data = *xcfdata++;
+              switch (i) {
+                case 0:  
+                q->red    = ScaleCharToQuantum(data);  
+                if ( inDocInfo->image_type == GIMP_GRAY )
+                  {
+                    q->green  = ScaleCharToQuantum(data);
+                    q->blue    = ScaleCharToQuantum(data);
+                    q->opacity  = ScaleCharToQuantum(255-inLayerInfo->opacity);
+                  }
+                else
+                  {
+                    q->green  = q->red;
+                    q->blue    = q->red;
+                    q->opacity  = ScaleCharToQuantum(255-inLayerInfo->opacity);
+                  }
+                break;
+                case 1:  q->green  = ScaleCharToQuantum(data);  break;
+                case 2:  q->blue    = ScaleCharToQuantum(data);  break;
+                case 3:  
+                  q->opacity = (Quantum) (data==0 ? TransparentOpacity :
+                      ScaleCharToQuantum(255-inLayerInfo->opacity));  
+                break;
+              }
+              q++;
+            }
+          }
+        else
+          {
+            length += 1;
+            if (length == 128)
+              {
+                if (xcfdata >= xcfdatalimit)
+                  {
+                    goto bogus_rle;
+                  }
+    
+                length = (*xcfdata << 8) + xcfdata[1];
+                xcfdata += 2;
+              }
+    
+            count += length;
+            size -= length;
+    
+            if (size < 0)
+              {
+                goto bogus_rle;
+              }
+    
+            if (xcfdata > xcfdatalimit)
+              {
+                goto bogus_rle;
+              }
+    
+            val = *xcfdata++;
+    
+            for (j = 0; j < length; j++)
+            {
+              data = val;
+              switch (i) {
+                case 0:  
+                  q->red    = ScaleCharToQuantum(data);  
+                  if ( inDocInfo->image_type == GIMP_GRAY )
+                    {
+                      q->green  = ScaleCharToQuantum(data);
+                      q->blue    = ScaleCharToQuantum(data);
+                      q->opacity  = ScaleCharToQuantum(255-inLayerInfo->opacity);
+                    }
+                  else
+                    {
+                      q->green  = q->red;
+                      q->blue    = q->red;
+                      q->opacity  = ScaleCharToQuantum(255-inLayerInfo->opacity);
+                    }
+                  break;
+                case 1:  q->green  = ScaleCharToQuantum(data);  break;
+                case 2:  q->blue    = ScaleCharToQuantum(data);  break;
+                case 3:  
+                  q->opacity = (Quantum) (data==0 ? TransparentOpacity :
+                     ScaleCharToQuantum(255-inLayerInfo->opacity));  
+                  break;
+              }
+              q++;
+            }
+          }
       }
-      q++;
-    }
-      }
-  }
     }
   LiberateMemory((void **) &xcfodata);
   return True;
@@ -487,9 +507,10 @@ static int load_tile_rle (Image* image, Image* tile_image, XCFDocInfo* inDocInfo
   return False;
 }
 
-static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* inLayerInfo)
+static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
+   inLayerInfo)
 {
-  unsigned long
+  ExtendedSignedIntegralType
     saved_pos,
     offset,
     offset2;
@@ -498,8 +519,8 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* inLaye
     ntiles,
     ntile_rows,
     ntile_cols,
-    width,
-    height;
+    junk_width,
+    junk_height;
 
   int
     i,
@@ -518,8 +539,8 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* inLaye
     *exception = inDocInfo->exception;
   
   /* start reading the data */
-  width = ReadBlobMSBLong(image);
-  height = ReadBlobMSBLong(image);
+  junk_width = ReadBlobMSBLong(image);
+  junk_height = ReadBlobMSBLong(image);
 
   /* read in the first tile offset.
    *  if it is '0', then this tile level is empty
@@ -539,7 +560,7 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* inLaye
       fail = False;
 
       if (offset == 0)
-      ThrowBinaryException(FileOpenError,"not enough tiles found in level",image->filename);
+        ThrowBinaryException(FileOpenError,"NotEnoughTiles",image->filename);
 
       /* save the current position as it is where the
        *  next tile offset is stored.
@@ -547,61 +568,66 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* inLaye
       saved_pos = TellBlob(image);
 
       /* read in the offset of the next tile so we can calculate the amount
-   of data needed for this tile*/
-    offset2 = ReadBlobMSBLong(image);
+         of data needed for this tile*/
+      offset2 = ReadBlobMSBLong(image);
       
       /* if the offset is 0 then we need to read in the maximum possible
-   allowing for negative compression */
+         allowing for negative compression */
       if (offset2 == 0)
-    offset2 = (unsigned long) (offset + TILE_WIDTH * TILE_WIDTH * 4* 1.5); 
+      offset2 = (unsigned long) (offset + TILE_WIDTH * TILE_WIDTH * 4* 1.5); 
                       /* 1.5 is probably more
                          than we need to allow */
 
       /* seek to the tile offset */
       SeekBlob(image, offset, SEEK_SET);
 
-    /* allocate the image for the tile 
-      NOTE: the last tile in a row or column may not be a full tile!
-    */
-    tile_image_width = destLeft==(int) ntile_cols-1 ? (int) image->columns % TILE_WIDTH :  TILE_WIDTH;
-    if (tile_image_width == 0)  tile_image_width = TILE_WIDTH;
-    tile_image_height = destTop==(int) ntile_rows-1 ? (int) image->rows % TILE_HEIGHT :  TILE_HEIGHT;
-    if (tile_image_height == 0)  tile_image_height = TILE_HEIGHT;
-      tile_image=CloneImage(inLayerInfo->image, tile_image_width, tile_image_height,True,exception);
+      /* allocate the image for the tile 
+        NOTE: the last tile in a row or column may not be a full tile!
+      */
+      tile_image_width = destLeft==(int) ntile_cols-1 ?
+        (int) image->columns % TILE_WIDTH :  TILE_WIDTH;
+      if (tile_image_width == 0)  tile_image_width = TILE_WIDTH;
+      tile_image_height = destTop==(int) ntile_rows-1 ?
+        (int) image->rows % TILE_HEIGHT :  TILE_HEIGHT;
+      if (tile_image_height == 0)  tile_image_height = TILE_HEIGHT;
+      tile_image=CloneImage(inLayerInfo->image, tile_image_width,
+      tile_image_height,True,exception);
 
       /* read in the tile */
       switch (inDocInfo->compression)
-  {
-    case COMPRESS_NONE:
-      if (!load_tile (image, tile_image, inDocInfo, inLayerInfo, offset2 - offset))
-      fail = True;
-      break;
-    case COMPRESS_RLE:
-      if (!load_tile_rle (image, tile_image, inDocInfo, inLayerInfo, offset2 - offset))
-      fail = True;
-      break;
-    case COMPRESS_ZLIB:
-      ThrowBinaryException(FileOpenError,"xcf: zlib compression unimplemented",image->filename)
-    case COMPRESS_FRACTAL:
-      ThrowBinaryException(FileOpenError,"xcf: fractal compression unimplemented",image->filename)
-  }
+      {
+        case COMPRESS_NONE:
+          if (!load_tile (image,tile_image,inDocInfo,inLayerInfo,(size_t) (offset2-offset)))
+            fail = True;
+          break;
+        case COMPRESS_RLE:
+          if (!load_tile_rle (image,tile_image,inDocInfo,inLayerInfo,
+              (size_t) (offset2-offset)))
+            fail = True;
+          break;
+        case COMPRESS_ZLIB:
+          ThrowBinaryException(CoderError,"ZipCompressionNotSupported",
+            image->filename)
+        case COMPRESS_FRACTAL:
+          ThrowBinaryException(CoderError,"FractalCompressionNotSupported",
+            image->filename)
+      }
 
-  /* composite the tile onto the layer's image, and then destroy it */
-    (void) CompositeImage(inLayerInfo->image,CopyCompositeOp,tile_image,
-              destLeft * TILE_WIDTH,destTop*TILE_HEIGHT);
-    DestroyImage(tile_image);
+      /* composite the tile onto the layer's image, and then destroy it */
+        (void) CompositeImage(inLayerInfo->image,CopyCompositeOp,tile_image,
+                  destLeft * TILE_WIDTH,destTop*TILE_HEIGHT);
+        DestroyImage(tile_image);
 
-  /* adjust tile position */
-  destLeft++;
-  if (destLeft >= (int) ntile_cols) {
-    destLeft = 0;
-    destTop++;
-  }
+      /* adjust tile position */
+      destLeft++;
+      if (destLeft >= (int) ntile_cols)
+        {
+          destLeft = 0;
+          destTop++;
+        }
 
       if (fail) 
-  {
-    return False;
-  }
+          return False;
       
       /* restore the saved position so we'll be ready to
        *  read the next offset.
@@ -614,26 +640,27 @@ static int load_level (Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* inLaye
 
 
   if (offset != 0)
-    ThrowBinaryException(FileOpenError,"encountered garbage after reading level",image->filename)
+    ThrowBinaryException(CorruptImageError,"CorruptXCFImage",image->filename)
 
   return 1;
 }
 
-static int load_hierarchy (Image *image, XCFDocInfo* inDocInfo, XCFLayerInfo* inLayer)
+static int load_hierarchy (Image *image, XCFDocInfo* inDocInfo, XCFLayerInfo*
+    inLayer)
 {
-  unsigned long 
+  ExtendedSignedIntegralType
   saved_pos,
   offset,
   junk;
 
   unsigned long
-  width,
-  height,
-  bpp;
+  junk_width,
+  junk_height,
+  junk_bpp;
 
-  width = ReadBlobMSBLong(image);
-  height = ReadBlobMSBLong(image);
-  bpp = inDocInfo->bpp = ReadBlobMSBLong(image);
+  junk_width = ReadBlobMSBLong(image);
+  junk_height = ReadBlobMSBLong(image);
+  junk_bpp = inDocInfo->bpp = ReadBlobMSBLong(image);
 
   /* load in the levels...we make sure that the number of levels
    *  calculated when the TileManager was created is the same
@@ -644,9 +671,9 @@ static int load_hierarchy (Image *image, XCFDocInfo* inDocInfo, XCFLayerInfo* in
   /* discard offsets for layers below first, if any.
    */
   do 
-    {
+  {
     junk = ReadBlobMSBLong(image);
-    }
+  }
   while (junk != 0);
 
   /* save the current position as it is where the
@@ -670,7 +697,8 @@ static int load_hierarchy (Image *image, XCFDocInfo* inDocInfo, XCFLayerInfo* in
 }
 
 
-static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* outLayer )
+static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo*
+   outLayer )
 {
   int
     i;
@@ -692,13 +720,10 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* outL
   ReadBlobStringWithLongSize(image, outLayer->name);
 
   /* allocate the image for this layer */
-  outLayer->image=CloneImage(image,outLayer->width,
-                             outLayer->height,True,
-                         &image->exception);
+  outLayer->image=CloneImage(image,outLayer->width, outLayer->height,True,
+     &image->exception);
   if (outLayer->image == (Image *) NULL)
-  {
     return False;
-  }
 
   /* read the layer properties! */
   foundPropEnd = 0;
@@ -739,8 +764,8 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* outL
       outLayer->show_mask = ReadBlobMSBLong(image);
       break;
     case PROP_OFFSETS:
-      outLayer->offset_x = ReadBlobMSBLong(image);
-      outLayer->offset_y = ReadBlobMSBLong(image);
+      outLayer->offset_x = (long) ReadBlobMSBLong(image);
+      outLayer->offset_y = (long) ReadBlobMSBLong(image);
       break;
     case PROP_MODE:
       outLayer->mode = ReadBlobMSBLong(image);
@@ -768,7 +793,8 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* outL
      }
      break;
     default:
-      /* g_message ("unexpected/unknown layer property: %d (skipping)",  prop_type); */
+      /* g_message ("unexpected/unknown layer property: %d (skipping)",
+         prop_type); */
 
       {
       int buf[16];
@@ -792,8 +818,11 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* outL
 
   /* set the compositing mode */
   outLayer->image->compose = GIMPBlendModeToCompositeOperator( outLayer->mode );
-  if ( outLayer->visible == False )  /* BOGUS: should really be separate member var! */
-    outLayer->image->compose = NoCompositeOp;  
+  if ( outLayer->visible == False )
+    {
+      /* BOGUS: should really be separate member var! */
+      outLayer->image->compose = NoCompositeOp;  
+    }
 
   /* read the hierarchy and layer mask offsets */
   hierarchy_offset = ReadBlobMSBLong(image);
@@ -873,6 +902,9 @@ static int ReadOneLayer( Image* image, XCFDocInfo* inDocInfo, XCFLayerInfo* outL
 */
 static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  char
+    magick[14];
+
   Image
     *image;
 
@@ -881,11 +913,14 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   unsigned long
     i,
-  image_type;
+    image_type;
 
   int
     foundPropEnd = 0;
 
+  size_t
+    count;
+
   XCFDocInfo
     doc_info;
 
@@ -897,14 +932,13 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
-
-  /* skip over "gimp xcf ..." */
-  for (i=0; i < 14; i++)
-    (void) ReadBlobByte(image);
-
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  count=ReadBlob(image,14,(char *) magick);
+  if ((count == 0) ||
+      (LocaleNCompare((char *) magick,"gimp xcf",8) != 0))
+    ThrowReaderException(CorruptImageError,"NotAXCFImageFile",image);
   /* clear the docinfo stuff */
   memset( &doc_info, 0, sizeof(XCFDocInfo));
   doc_info.exception = exception;
@@ -922,11 +956,10 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   } else if ( image_type == GIMP_GRAY ) {
     image->colorspace=GRAYColorspace;
   } else if ( image_type == GIMP_INDEXED )
-    ThrowReaderException(FileOpenError,"Indexed colors not currently supported",image);
+    ThrowReaderException(CoderError,"ColormapTypeNotSupported",image);
   SetImage(image,OpaqueOpacity);  /* until we know otherwise...*/
   image->matte=True;  /* XCF always has a matte! */
 
-
   /* read properties */
   while ( !foundPropEnd ) {
   PropType    prop_type = (PropType) ReadBlobMSBLong(image);
@@ -953,11 +986,11 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         gimage->cmap = g_new (guchar, gimage->num_cols*3);
         xcf_seek_pos (info, info->cp + gimage->num_cols);
         for (i = 0; i<gimage->num_cols; i++) 
-      {
-        gimage->cmap[i*3+0] = i;
-        gimage->cmap[i*3+1] = i;
-        gimage->cmap[i*3+2] = i;
-      }
+          {
+            gimage->cmap[i*3+0] = i;
+            gimage->cmap[i*3+1] = i;
+            gimage->cmap[i*3+2] = i;
+          }
       }
       else 
       {
@@ -979,7 +1012,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (doc_info.compression != COMPRESS_RLE) &&
         (doc_info.compression != COMPRESS_ZLIB) &&
         (doc_info.compression != COMPRESS_FRACTAL))
-          ThrowReaderException(CorruptImageError,"Unknown compression type",image);
+          ThrowReaderException(CorruptImageError,"CompressionNotValid",image);
       }
       break;
 
@@ -1034,9 +1067,9 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       while (info->cp - base < prop_size)
         {
-      p = xcf_load_parasite (info);
-      gimp_image_parasite_attach (gimage, p);
-      gimp_parasite_free (p);
+          p = xcf_load_parasite (info);
+          gimp_image_parasite_attach (gimage, p);
+          gimp_parasite_free (p);
         }
       if (info->cp - base != prop_size)
         g_message ("Error detected while loading an image's parasites");
@@ -1066,28 +1099,29 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
     case PROP_USER_UNIT:
       {
-		char  unit_string[1000];
+        char  unit_string[1000];
         /*BOGUS: ignored for now */
         /*float  factor = (float) */ (void) ReadBlobMSBLong(image);
         /* unsigned long digits =  */ (void) ReadBlobMSBLong(image);
-		for (i=0; i<5; i++)
-		 ReadBlobStringWithLongSize(image, unit_string);
+        for (i=0; i<5; i++)
+         ReadBlobStringWithLongSize(image, unit_string);
       }
      break;
 
     default:
-      /* g_message ("unexpected/unknown image property: %d (skipping)", prop_type); */
+      /* g_message ("unexpected/unknown image property: %d (skipping)",
+         prop_type); */
 
       {
       int buf[16];
-      int amount;
+      long amount;
 
       /* read over it... */
       while (prop_size > 0)
         {
-        amount = Min (16, prop_size);
+        amount = (long) Min (16, prop_size);
         for (i=0; i<(unsigned long) amount; i++)
-        amount = ReadBlob(image, amount, &buf);
+        amount = (long) ReadBlob(image, amount, &buf);
         prop_size -= Min (16, amount);
         }
       }
@@ -1109,9 +1143,9 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         we have to scan the layer offset list, and then reposition
         the read pointer
     */
-    unsigned long  oldPos = TellBlob(image);
+    ExtendedSignedIntegralType  oldPos = TellBlob(image);
     do {
-      int  offset = ReadBlobMSBLong(image);
+      long  offset = (long) ReadBlobMSBLong(image);
       if ( offset == 0 )
         foundAllLayers = True;
       else
@@ -1121,14 +1155,15 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
 
     /* allocate our array of layer info blocks */
-    layer_info=(XCFLayerInfo *) AcquireMemory(number_layers*sizeof(XCFLayerInfo));
+    layer_info=(XCFLayerInfo *)
+      AcquireMemory(number_layers*sizeof(XCFLayerInfo));
     if (layer_info == (XCFLayerInfo *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed", image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     (void) memset(layer_info,0,number_layers*sizeof(XCFLayerInfo));
 
-    while (True)
+    for ( ; ; )
     {
-      unsigned long
+      ExtendedSignedIntegralType
         offset,
         saved_pos;
       int
@@ -1157,7 +1192,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         int j;
         for (j=0; j < current_layer; j++)
           DestroyImage(layer_info[j].image);
-        ThrowReaderException(ResourceLimitError, "Memory allocation failed",image)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
       }
 
       /* restore the saved position so we'll be ready to
@@ -1182,7 +1217,8 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         QuantizeImage( &qi, image );
       }
     } else {
-      if ( 0 ) {
+#if 0
+        {
         /* NOTE: XCF layers are REVERSED from composite order! */
         signed int  j;
         for (j=number_layers-1; j>=0; j--) {
@@ -1202,8 +1238,9 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
           }
         }
-
-      } else {
+      }
+#else
+      {
         /* NOTE: XCF layers are REVERSED from composite order! */
         signed int  j;
 
@@ -1246,6 +1283,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         }
       }
+#endif
     }
 
     LiberateMemory((void **) &layer_info);
@@ -1322,10 +1360,11 @@ ModuleExport void RegisterXCFImage(void)
     *entry;
 
   entry=SetMagickInfo("XCF");
-  entry->decoder=ReadXCFImage;
-  entry->magick=IsXCF;
+  entry->decoder=(DecoderHandler) ReadXCFImage;
+  entry->magick=(MagickHandler) IsXCF;
   entry->description=AcquireString("GIMP image");
   entry->module=AcquireString("XCF");
+  entry->seekable_stream=True;
   (void) RegisterMagickInfo(entry);
 }
 
