@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "quantize.h"
+#include "static.h"
+#include "utility.h"
 
 
 /*
@@ -95,7 +102,7 @@ static unsigned int IsXPM(const unsigned char *magick,const size_t length)
 {
   if (length < 9)
     return(False);
-  if (LocaleNCompare((char *) magick,"/* XPM */",9) == 0)
+  if (LocaleNCompare((char *) magick+1,"* XPM *",7) == 0)
     return(True);
   return(False);
 }
@@ -146,7 +153,7 @@ static char *ParseColor(char *data)
   register const char
     *q;
 
-  register int
+  register long
     i;
 
   for (i=0; i < NumberTargets; i++)
@@ -223,9 +230,9 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
     Read XPM file.
   */
@@ -250,7 +257,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       p=xpm_buffer+strlen(xpm_buffer);
     }
   if (xpm_buffer == (char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Remove comments.
   */
@@ -266,8 +273,8 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   if ((count != 4) || (width > 2) || (image->columns == 0) ||
       (image->rows == 0) || (image->colors == 0))
-    ThrowReaderException(CorruptImageError,"Not a XPM image file",image)
-  image->depth=8;
+    ThrowReaderException(CorruptImageError,"NotAXPMImageFile",image)
+  image->depth=16;
   /*
     Remove unquoted characters.
   */
@@ -288,7 +295,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   textlist=StringToList(xpm_buffer);
   LiberateMemory((void **) &xpm_buffer);
   if (textlist == (char **) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Initialize image structure.
   */
@@ -298,8 +305,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (i=0; textlist[i] != (char *) NULL; i++)
         LiberateMemory((void **) &textlist[i]);
       LiberateMemory((void **) &textlist);
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
     }
   /*
     Read image colormap.
@@ -318,8 +324,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           LiberateMemory((void **) &textlist[i]);
         LiberateMemory((void **) &textlist);
         LiberateMemory((void **) &keys);
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
       }
     keys[j][width]='\0';
     (void) strncpy(keys[j],p,width);
@@ -345,14 +350,15 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         none=j;
         (void) strcpy(target,"black");
       }
-    (void) QueryColorDatabase(target,&image->colormap[j],exception);
+    if (!QueryColorDatabase(target,&image->colormap[j],exception))
+      break;
   }
   if (j < (long) image->colors)
     {
       for (i=0; textlist[i] != (char *) NULL; i++)
         LiberateMemory((void **) &textlist[i]);
       LiberateMemory((void **) &textlist);
-      ThrowReaderException(CorruptImageError,"Corrupt XPM image file",image)
+      ThrowReaderException(CorruptImageError,"CorruptXPMImage",image)
     }
   j=0;
   key[width]='\0';
@@ -389,7 +395,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
       }
       if (y < (long) image->rows)
-        ThrowReaderException(CorruptImageError,"Not enough pixel data",image);
+        ThrowReaderException(CorruptImageError,"NotenoughPixelData",image);
     }
   /*
     Free resources.
@@ -433,23 +439,23 @@ ModuleExport void RegisterXPMImage(void)
     *entry;
 
   entry=SetMagickInfo("PICON");
-  entry->decoder=ReadXPMImage;
-  entry->encoder=WritePICONImage;
+  entry->decoder=(DecoderHandler) ReadXPMImage;
+  entry->encoder=(EncoderHandler) WritePICONImage;
   entry->adjoin=False;
   entry->description=AcquireString("Personal Icon");
   entry->module=AcquireString("PICON");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PM");
-  entry->decoder=ReadXPMImage;
-  entry->encoder=WriteXPMImage;
+  entry->decoder=(DecoderHandler) ReadXPMImage;
+  entry->encoder=(EncoderHandler) WriteXPMImage;
   entry->adjoin=False;
   entry->description=AcquireString("X Windows system pixmap (color)");
   entry->module=AcquireString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XPM");
-  entry->decoder=ReadXPMImage;
-  entry->encoder=WriteXPMImage;
-  entry->magick=IsXPM;
+  entry->decoder=(DecoderHandler) ReadXPMImage;
+  entry->encoder=(EncoderHandler) WriteXPMImage;
+  entry->magick=(MagickHandler) IsXPM;
   entry->adjoin=False;
   entry->description=AcquireString("X Windows system pixmap (color)");
   entry->module=AcquireString("XPM");
@@ -600,12 +606,12 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   SetGeometry(image,&geometry);
-  (void) GetMagickGeometry(PiconGeometry,&geometry.x,&geometry.y,
+  (void) ParseMetaGeometry(PiconGeometry,&geometry.x,&geometry.y,
     &geometry.width,&geometry.height);
   picon=ResizeImage(image,geometry.width,geometry.height,TriangleFilter,1.0,
     &image->exception);
@@ -621,7 +627,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   transparent=False;
   if (picon->storage_class == PseudoClass)
     {
-      CompressColormap(picon);
+      CompressImageColormap(picon);
       if (picon->matte)
         transparent=True;
     }
@@ -689,8 +695,8 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   (void) FormatString(buffer,"static char *%.1024s[] = {\n",basename);
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,"/* columns rows colors chars-per-pixel */\n");
-  FormatString(buffer,"\"%lu %lu %lu %ld\",\n",picon->columns,
-    picon->rows,colors,characters_per_pixel);
+  FormatString(buffer,"\"%lu %lu %lu %ld\",\n",picon->columns,picon->rows,
+    colors,characters_per_pixel);
   (void) WriteBlobString(image,buffer);
   for (i=0; i < (long) colors; i++)
   {
@@ -698,7 +704,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
       Define XPM color.
     */
     picon->colormap[i].opacity=OpaqueOpacity;
-    (void) QueryColorname(picon,picon->colormap+i,X11Compliance,name,
+    (void) QueryColorname(picon,picon->colormap+i,XPMCompliance,name,
       &picon->exception);
     if (transparent)
       {
@@ -740,14 +746,15 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
         symbol[j]=Cixel[k];
       }
       symbol[j]='\0';
-      FormatString(buffer,"%.1024s",symbol);
+      (void) strncpy(buffer,symbol,MaxTextExtent-1);
       (void) WriteBlobString(image,buffer);
     }
     FormatString(buffer,"\"%.1024s\n",
       (y == (long) (picon->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
     if (QuantumTick(y,picon->rows))
-      MagickMonitor(SaveImageText,y,picon->rows);
+      if (!MagickMonitor(SaveImageTag,y,picon->rows,&image->exception))
+        break;
   }
   DestroyImage(picon);
   (void) WriteBlobString(image,"};\n");
@@ -833,14 +840,14 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   transparent=False;
   if (image->storage_class == PseudoClass)
     {
-      CompressColormap(image);
+      CompressImageColormap(image);
       if (image->matte)
         transparent=True;
     }
@@ -917,7 +924,7 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
       Define XPM color.
     */
     image->colormap[i].opacity=OpaqueOpacity;
-    (void) QueryColorname(image,image->colormap+i,X11Compliance,name,
+    (void) QueryColorname(image,image->colormap+i,XPMCompliance,name,
       &image->exception);
     if (transparent)
       {
@@ -959,14 +966,15 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
         symbol[j]=Cixel[k];
       }
       symbol[j]='\0';
-      FormatString(buffer,"%.1024s",symbol);
+      (void) strncpy(buffer,symbol,MaxTextExtent-1);
       (void) WriteBlobString(image,buffer);
     }
     FormatString(buffer,"\"%.1024s\n",
       (y == (long) (image->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+        break;
   }
   (void) WriteBlobString(image,"};\n");
   CloseBlob(image);
