@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -192,16 +199,15 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
   /*
      Read in header information.
   */
   count=ReadBlob(image,sz_XWDheader,(char *) &header);
   if (count == 0)
-    ThrowReaderException(CorruptImageError,"Unable to read dump file header",
-      image);
+    ThrowReaderException(CorruptImageError,"UnableToReadImageHeader",image);
   image->columns=header.pixmap_width;
   image->rows=header.pixmap_height;
   image->depth=8;
@@ -215,28 +221,26 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Check to see if the dump file is in the proper format.
   */
   if (header.file_version != XWD_FILE_VERSION)
-    ThrowReaderException(CorruptImageError,"XWD file format version mismatch",
-      image);
+    ThrowReaderException(CorruptImageError,"FileFormatVersionMismatch",image);
   if (header.header_size < sz_XWDheader)
-    ThrowReaderException(CorruptImageError,"XWD header size is too small",
-      image);
+    ThrowReaderException(CorruptImageError,"CorruptXWDImage",image);
   length=header.header_size-sz_XWDheader;
   comment=(char *) AcquireMemory(length+1);
   if (comment == (char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   count=ReadBlob(image,length,comment);
   comment[length]='\0';
   (void) SetImageAttribute(image,"comment",comment);
   LiberateMemory((void **) &comment);
   if (count == 0)
-    ThrowReaderException(CorruptImageError,
-      "Unable to read window name from dump file",image);
+    ThrowReaderException(CorruptImageError,"UnableToReadWindowNameFromDumpFile",
+      image);
   /*
     Initialize the X image.
   */
   ximage=(XImage *) AcquireMemory(sizeof(XImage));
   if (ximage == (XImage *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   ximage->depth=(int) header.pixmap_depth;
   ximage->format=(int) header.pixmap_format;
   ximage->xoffset=(int) header.xoffset;
@@ -254,7 +258,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ximage->blue_mask=header.blue_mask;
   status=XInitImage(ximage);
   if (status == False)
-    ThrowReaderException(CorruptImageError,"Invalid XWD header",image);
+    ThrowReaderException(CorruptImageError,"UnrecognizedXWDHeader",image);
   /*
     Read colormap.
   */
@@ -264,17 +268,15 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       XWDColor
         color;
 
-      colors=(XColor *)
-        AcquireMemory((unsigned int) header.ncolors*sizeof(XColor));
+      colors=(XColor *) AcquireMemory(header.ncolors*sizeof(XColor));
       if (colors == (XColor *) NULL)
-        ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
       for (i=0; i < (long) header.ncolors; i++)
       {
         count=ReadBlob(image,sz_XWDColor,(char *) &color);
         if (count == 0)
           ThrowReaderException(CorruptImageError,
-            "Unable to read color map from dump file",image);
+            "UnableToReadColormapFromDumpfile",image);
         colors[i].pixel=color.pixel;
         colors[i].red=color.red;
         colors[i].green=color.green;
@@ -303,10 +305,10 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     length=ximage->bytes_per_line*ximage->height*ximage->depth;
   ximage->data=(char *) AcquireMemory(length);
   if (ximage->data == (char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   count=ReadBlob(image,length,ximage->data);
   if (count == 0)
-    ThrowReaderException(CorruptImageError,"Unable to read dump pixmap",
+    ThrowReaderException(CorruptImageError,"UnableToReadPixmapFromDumpFile",
       image);
   /*
     Convert image to MIFF format.
@@ -373,17 +375,18 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
               index=(unsigned short) ((pixel >> red_shift) & red_mask);
-              q->red=XDownscale(colors[index].red);
+              q->red=ScaleShortToQuantum(colors[index].red);
               index=(unsigned short) ((pixel >> green_shift) & green_mask);
-              q->green=XDownscale(colors[index].green);
+              q->green=ScaleShortToQuantum(colors[index].green);
               index=(unsigned short) ((pixel >> blue_shift) & blue_mask);
-              q->blue=XDownscale(colors[index].blue);
+              q->blue=ScaleShortToQuantum(colors[index].blue);
               q++;
             }
             if (!SyncImagePixels(image))
               break;
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
           }
         else
           for (y=0; y < (long) image->rows; y++)
@@ -395,17 +398,18 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
               color=(pixel >> red_shift) & red_mask;
-              q->red=XDownscale((color*65535L)/red_mask);
+              q->red=ScaleShortToQuantum((color*65535L)/red_mask);
               color=(pixel >> green_shift) & green_mask;
-              q->green=XDownscale((color*65535L)/green_mask);
+              q->green=ScaleShortToQuantum((color*65535L)/green_mask);
               color=(pixel >> blue_shift) & blue_mask;
-              q->blue=XDownscale((color*65535L)/blue_mask);
+              q->blue=ScaleShortToQuantum((color*65535L)/blue_mask);
               q++;
             }
             if (!SyncImagePixels(image))
               break;
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+                break;
           }
         break;
       }
@@ -415,13 +419,13 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Convert X image to PseudoClass packets.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"Memory allocation failed",
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
             image);
         for (i=0; i < (long) image->colors; i++)
         {
-          image->colormap[i].red=XDownscale(colors[i].red);
-          image->colormap[i].green=XDownscale(colors[i].green);
-          image->colormap[i].blue=XDownscale(colors[i].blue);
+          image->colormap[i].red=ScaleShortToQuantum(colors[i].red);
+          image->colormap[i].green=ScaleShortToQuantum(colors[i].green);
+          image->colormap[i].blue=ScaleShortToQuantum(colors[i].blue);
         }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -439,7 +443,8 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (!SyncImagePixels(image))
             break;
           if (QuantumTick(y,image->rows))
-            MagickMonitor(LoadImageText,y,image->rows);
+            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+              break;
         }
         break;
       }
@@ -452,15 +457,16 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   LiberateMemory((void **) &ximage->data);
   LiberateMemory((void **) &ximage);
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+    ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
   return(image);
 }
 #else
 static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  ThrowException(exception,MissingDelegateError,
-    "X11 library is not available",image_info->filename);
+  ThrowException(exception,MissingDelegateError,"XWindowLibraryIsNotAvailable",
+    image_info->filename);
   return((Image *) NULL);
 }
 #endif
@@ -494,9 +500,9 @@ ModuleExport void RegisterXWDImage(void)
     *entry;
 
   entry=SetMagickInfo("XWD");
-  entry->decoder=ReadXWDImage;
-  entry->encoder=WriteXWDImage;
-  entry->magick=IsXWD;
+  entry->decoder=(DecoderHandler) ReadXWDImage;
+  entry->encoder=(EncoderHandler) WriteXWDImage;
+  entry->magick=(MagickHandler) IsXWD;
   entry->adjoin=False;
   entry->description=AcquireString("X Windows system window dump (color)");
   entry->module=AcquireString("XWD");
@@ -571,7 +577,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   register IndexPacket
     *indexes;
 
-  register int
+  register long
     x;
 
   register long
@@ -602,10 +608,10 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
+    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  (void) TransformRGBImage(image,image->colorspace);
   /*
     Initialize XWD file header.
   */
@@ -664,14 +670,13 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
       */
       colors=(XColor *) AcquireMemory(image->colors*sizeof(XColor));
       if (colors == (XColor *) NULL)
-        ThrowWriterException(ResourceLimitError,"Memory allocation failed",
-          image);
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
       for (i=0; i < (long) image->colors; i++)
       {
         colors[i].pixel=i;
-        colors[i].red=XUpscale(image->colormap[i].red);
-        colors[i].green=XUpscale(image->colormap[i].green);
-        colors[i].blue=XUpscale(image->colormap[i].blue);
+        colors[i].red=ScaleQuantumToShort(image->colormap[i].red);
+        colors[i].green=ScaleQuantumToShort(image->colormap[i].green);
+        colors[i].blue=ScaleQuantumToShort(image->colormap[i].blue);
         colors[i].flags=DoRed | DoGreen | DoBlue;
         colors[i].pad=0;
         if (*(char *) &lsb_first)
@@ -697,7 +702,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   pixels=(unsigned char *)
     AcquireMemory(image->columns*sizeof(PixelPacket));
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
   /*
     Convert MIFF to XWD raster pixels.
   */
@@ -715,9 +720,9 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
         *q++=indexes[x];
       else
         {
-          *q++=Downscale(p->red);
-          *q++=Downscale(p->green);
-          *q++=Downscale(p->blue);
+          *q++=ScaleQuantumToChar(p->red);
+          *q++=ScaleQuantumToChar(p->green);
+          *q++=ScaleQuantumToChar(p->blue);
         }
       p++;
     }
@@ -726,7 +731,8 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlob(image,q-pixels,(char *) pixels);
     if (image->previous == (Image *) NULL)
       if (QuantumTick(y,image->rows))
-        MagickMonitor(SaveImageText,y,image->rows);
+        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+          break;
   }
   LiberateMemory((void **) &pixels);
   CloseBlob(image);
@@ -735,7 +741,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
 #else
 static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
 {
-  ThrowBinaryException(MissingDelegateError,"X11 library is not available",
+  ThrowBinaryException(MissingDelegateError,"XWindowLibraryIsNotAvailable",
     image->filename);
 }
 #endif
