@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,6 +53,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "list.h"
+#include "magick.h"
+#include "monitor.h"
+#include "static.h"
+#include "utility.h"
 
 /*
   Forward declarations.
@@ -141,23 +147,27 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"Must specify image size",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
   image->depth=8;
   interlace=image_info->interlace;
   horizontal_factor=2;
   vertical_factor=2;
   if (image_info->sampling_factor != (char *) NULL)
     {
-      long
-        factors;
+      GeometryInfo
+        geometry_info;
 
-      factors=sscanf(image_info->sampling_factor,"%ldx%ld",&horizontal_factor,
-        &vertical_factor);
-      if (factors != 2)
+      unsigned int
+        flags;
+
+      flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
+      horizontal_factor=(long) geometry_info.rho;
+      vertical_factor=(long) geometry_info.sigma;
+      if (!(flags & SigmaValue))
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
           (vertical_factor != 1) && (vertical_factor != 2))
-        ThrowReaderException(CorruptImageError,"Unexpected sampling factor",
+        ThrowReaderException(CorruptImageError,"UnexpectedSamplingFactor",
           image);
     }
   if ((interlace == UndefinedInterlace) ||
@@ -172,9 +182,9 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Open image file.
       */
-      status=OpenBlob(image_info,image,ReadBinaryType,exception);
+      status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == False)
-        ThrowReaderException(FileOpenError,"Unable to open file",image);
+        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
       for (i=0; i < image->offset; i++)
         (void) ReadBlobByte(image);
     }
@@ -186,14 +196,13 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   else
     scanline=(unsigned char *) AcquireMemory(image->columns);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
   do
   {
     chroma_image=CloneImage(image,image->columns/horizontal_factor,
       image->rows/vertical_factor,True,exception);
     if (chroma_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     /*
       Convert raster image to pixel packets.
     */
@@ -203,9 +212,9 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (interlace == PartitionInterlace)
       {
         AppendImageFormat("Y",image->filename);
-        status=OpenBlob(image_info,image,ReadBinaryType,exception);
+        status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == False)
-          ThrowReaderException(FileOpenError,"Unable to open file",image);
+          ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
       }
     for (y=0; y < (long) image->rows; y++)
     {
@@ -223,15 +232,15 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           for (x=0; x < (long) image->columns; x+=2)
           {
             s->red=0;
-            s->green=Upscale(*p++);
-            q->red=Upscale(*p++);
+            s->green=ScaleCharToQuantum(*p++);
+            q->red=ScaleCharToQuantum(*p++);
             q->green=0;
             q->blue=0;
             q++;
             q->green=0;
             q->blue=0;
-            s->blue=Upscale(*p++);
-            q->red=Upscale(*p++);
+            s->blue=ScaleCharToQuantum(*p++);
+            q->red=ScaleCharToQuantum(*p++);
             s++;
             q++;
           }
@@ -246,7 +255,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->red=Upscale(*p++);
+            q->red=ScaleCharToQuantum(*p++);
             q->green=0;
             q->blue=0;
             q++;
@@ -258,15 +267,16 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (!SyncImagePixels(chroma_image))
           break;
       if (image->previous == (Image *) NULL)
-        MagickMonitor(LoadImageText,y,image->rows);
+        if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+          break;
     }
     if (interlace == PartitionInterlace)
       {
         CloseBlob(image);
         AppendImageFormat("U",image->filename);
-        status=OpenBlob(image_info,image,ReadBinaryType,exception);
+        status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == False)
-          ThrowReaderException(FileOpenError,"Unable to open file",image);
+          ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
       }
     if (interlace != NoInterlace)
       {
@@ -280,7 +290,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           for (x=0; x < (long) chroma_image->columns; x++)
           {
             q->red=0;
-            q->green=Upscale(*p++);
+            q->green=ScaleCharToQuantum(*p++);
             q->blue=0;
             q++;
           }
@@ -291,9 +301,9 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           CloseBlob(image);
           AppendImageFormat("V",image->filename);
-          status=OpenBlob(image_info,image,ReadBinaryType,exception);
+          status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
           if (status == False)
-            ThrowReaderException(FileOpenError,"Unable to open file",image);
+            ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
         }
       for (y=0; y < (long) chroma_image->rows; y++)
       {
@@ -304,7 +314,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         for (x=0; x < (long) chroma_image->columns; x++)
         {
-          q->blue=Upscale(*p++);
+          q->blue=ScaleCharToQuantum(*p++);
           q++;
         }
         if (!SyncImagePixels(chroma_image))
@@ -318,8 +328,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       TriangleFilter,1.0,exception);
     DestroyImage(chroma_image);
     if (resize_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
     for (y=0; y < (long) image->rows; y++)
     {
       q=GetImagePixels(image,0,y,image->columns,1);
@@ -342,7 +351,11 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (interlace == PartitionInterlace)
       (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
     if (EOFBlob(image))
-      ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+      {
+        ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
@@ -364,8 +377,11 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image));
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (count != 0);
   LiberateMemory((void **) &scanline);
@@ -404,8 +420,8 @@ ModuleExport void RegisterYUVImage(void)
     *entry;
 
   entry=SetMagickInfo("YUV");
-  entry->decoder=ReadYUVImage;
-  entry->encoder=WriteYUVImage;
+  entry->decoder=(DecoderHandler) ReadYUVImage;
+  entry->encoder=(EncoderHandler) WriteYUVImage;
   entry->adjoin=False;
   entry->raw=True;
   entry->description=AcquireString("CCIR 601 4:1:1 or 4:2:2");
@@ -474,6 +490,9 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     *chroma_image,
     *yuv_image;
 
+  InterlaceType
+    interlace;
+
   long
     horizontal_factor,
     vertical_factor,
@@ -494,9 +513,6 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     height,
     width;
 
-  InterlaceType
-    interlace;
-
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
@@ -506,16 +522,20 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
   vertical_factor=2;
   if (image_info->sampling_factor != (char *) NULL)
     {
-      long
-        factors;
+      GeometryInfo
+        geometry_info;
 
-      factors=sscanf(image_info->sampling_factor,"%ldx%ld",&horizontal_factor,
-        &vertical_factor);
-      if (factors != 2)
+      int
+        flags;
+
+      flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
+      horizontal_factor=(long) geometry_info.rho;
+      vertical_factor=(long) geometry_info.sigma;
+      if (!(flags & SigmaValue))
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
           (vertical_factor != 1) && (vertical_factor != 2))
-        ThrowWriterException(ResourceLimitError,"Unexpected sampling factor",
+        ThrowWriterException(CorruptImageError,"UnexpectedSamplingFactor",
           image);
     }
   if ((interlace == UndefinedInterlace) ||
@@ -530,16 +550,16 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == False)
-        ThrowWriterException(FileOpenError,"Unable to open file",image);
+        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
     }
   else
     {
       AppendImageFormat("Y",image->filename);
-      status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == False)
-        ThrowWriterException(FileOpenError,"Unable to open file",image);
+        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
     }
   scene=0;
   do
@@ -548,13 +568,13 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
       Sample image to an even width and height, if necessary.
     */
     image->depth=8;
-    (void) TransformRGBImage(image,RGBColorspace);
+    (void) TransformRGBImage(image,image->colorspace);
     width=image->columns+(image->columns & (horizontal_factor-1));
     height=image->rows+(image->rows & (vertical_factor-1));
     yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
       &image->exception);
     if (yuv_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,"Unable to resize image",image);
+      ThrowWriterException(ResourceLimitError,image->exception.reason,image);
     (void) RGBTransformImage(yuv_image,YCbCrColorspace);
     /*
       Downsample image.
@@ -562,11 +582,13 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     chroma_image=ResizeImage(image,width/horizontal_factor,
       height/vertical_factor,TriangleFilter,1.0,&image->exception);
     if (chroma_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,"Unable to resize image",image);
+      ThrowWriterException(ResourceLimitError,image->exception.reason,image);
     (void) RGBTransformImage(chroma_image,YCbCrColorspace);
     if (interlace == NoInterlace)
       {
-        /* Write noninterlaced YUV. */
+        /*
+          Write noninterlaced YUV.
+        */
         for (y=0; y < (long) yuv_image->rows; y++)
         {
           p=AcquireImagePixels(yuv_image,0,y,yuv_image->columns,1,
@@ -579,18 +601,19 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) yuv_image->columns; x++)
           {
-            (void) WriteBlobByte(image,Downscale(s->green));
-            (void) WriteBlobByte(image,Downscale(p->red));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(s->green));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
             p++;
-            (void) WriteBlobByte(image,Downscale(s->blue));
-            (void) WriteBlobByte(image,Downscale(p->red));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(s->blue));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
             p++;
             s++;
             x++;
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         DestroyImage(yuv_image);
       }
@@ -607,12 +630,13 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) yuv_image->columns; x++)
           {
-            (void) WriteBlobByte(image,Downscale(p->red));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
             p++;
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+                break;
         }
         DestroyImage(yuv_image);
         /*
@@ -622,9 +646,10 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
           {
             CloseBlob(image);
             AppendImageFormat("U",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < (long) chroma_image->rows; y++)
         {
@@ -634,7 +659,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) chroma_image->columns; x++)
           {
-            (void) WriteBlobByte(image,Downscale(p->green));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(p->green));
             p++;
           }
         }
@@ -645,9 +670,10 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
           {
             CloseBlob(image);
             AppendImageFormat("V",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenError,"Unable to open file",image);
+              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
           }
         for (y=0; y < (long) chroma_image->rows; y++)
         {
@@ -657,7 +683,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             break;
           for (x=0; x < (long) chroma_image->columns; x++)
           {
-            (void) WriteBlobByte(image,Downscale(p->blue));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(p->blue));
             p++;
           }
         }
@@ -667,8 +693,11 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
       (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
   } while (image_info->adjoin);
   if (image_info->adjoin)
     while (image->previous != (Image *) NULL)
