@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -56,9 +56,11 @@
   Include declarations.
 */
 #include "studio.h"
-#if defined(HasX11)
+#include "gem.h"
+#include "log.h"
+#include "render.h"
+#include "utility.h"
 #include "xwindow.h"
-#endif
 #if defined(HasTTF)
 #if defined(__MINGW32__)
 #  undef interface  /* Remove interface define */
@@ -72,14 +74,11 @@
   Forward declarations.
 */
 static unsigned int
-  RenderType(Image *,const DrawInfo *,const PointInfo *,const unsigned int,
-	  TypeMetric *),
-  RenderPostscript(Image *,const DrawInfo *,const PointInfo *,
-    const unsigned int,TypeMetric *),
-  RenderFreetype(Image *,const DrawInfo *,const PointInfo *,const unsigned int,
-	  TypeMetric *),
-  RenderX11(Image *,const DrawInfo *,const PointInfo *,const unsigned int,
-    TypeMetric *);
+  RenderType(Image *,const DrawInfo *,const PointInfo *,TypeMetric *),
+  RenderPostscript(Image *,const DrawInfo *,const PointInfo *,TypeMetric *),
+  RenderFreetype(Image *,const DrawInfo *,const char *,const PointInfo *,
+    TypeMetric *),
+  RenderX11(Image *,const DrawInfo *,const PointInfo *,TypeMetric *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -181,8 +180,8 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
     return(False);
   text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
   if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Unable to annotate image",
-      "Memory allocation failed");
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToAnnotateImage");
   textlist=StringToList(text);
   LiberateMemory((void **) &text);
   if (textlist == (char **) NULL)
@@ -194,12 +193,11 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   number_lines=i;
   text=(char *) AcquireMemory(length+MaxTextExtent);
   if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Unable to annotate image",
-      "Memory allocation failed");
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToAnnotateImage");
   SetGeometry(image,&geometry);
   if (draw_info->geometry != (char *) NULL)
-    (void) GetGeometry(draw_info->geometry,&geometry.x,&geometry.y,
-      &geometry.width,&geometry.height);
+    (void) ParsePageGeometry(image,draw_info->geometry,&geometry);
   annotate=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   matte=image->matte;
@@ -214,7 +212,7 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
     (void) CloneString(&annotate->text,textlist[i]);
     if ((i == 0) || (annotate->gravity != NorthWestGravity))
       (void) GetTypeMetrics(image,annotate,&metrics);
-    height=metrics.ascent-metrics.descent;
+    height=(unsigned long) (metrics.ascent-metrics.descent);
     switch (annotate->gravity)
     {
       case ForgetGravity:
@@ -274,31 +272,34 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
       }
       case SouthWestGravity:
       {
-        offset.x=geometry.x+i*draw_info->affine.ry*height-draw_info->affine.ry*
-          (number_lines-1)*height;
+        offset.x=geometry.x+i*draw_info->affine.ry*height+draw_info->affine.ry*
+          (metrics.ascent+metrics.descent-(number_lines-1)*height);
         offset.y=(geometry.height == 0 ? 1 : -1)*geometry.y+geometry.height+i*
           draw_info->affine.sy*height-draw_info->affine.sy*(number_lines-1)*
-          height;
+          height+draw_info->affine.sy*metrics.ascent;
         break;
       }
       case SouthGravity:
       {
-        offset.x=geometry.x+geometry.width/2+i*draw_info->affine.ry*
-          height-draw_info->affine.sx*metrics.width/2-
-          draw_info->affine.ry*(number_lines-1)*height;
+        offset.x=geometry.x+geometry.width/2+i*draw_info->affine.ry*height-
+          draw_info->affine.sx*metrics.width/2+draw_info->affine.ry*
+          (metrics.ascent+metrics.descent-(number_lines-1)*height);
         offset.y=(geometry.height == 0 ? 1 : -1)*geometry.y+geometry.height+i*
           draw_info->affine.sy*height-draw_info->affine.rx*
-          metrics.width/2-draw_info->affine.sy*(number_lines-1)*height;
+          metrics.width/2-draw_info->affine.sy*(number_lines-1)*height+
+          draw_info->affine.sy*metrics.ascent;
         break;
       }
       case SouthEastGravity:
       {
         offset.x=(geometry.width == 0 ? 1 : -1)*geometry.x+geometry.width+i*
-          draw_info->affine.ry*height-draw_info->affine.sx*metrics.width-
-          draw_info->affine.ry*(number_lines-1)*height;
+          draw_info->affine.ry*height-draw_info->affine.sx*metrics.width+
+          draw_info->affine.ry*(metrics.ascent+metrics.descent-(number_lines-1)*
+          height);
         offset.y=(geometry.height == 0 ? 1 : -1)*geometry.y+geometry.height+i*
           draw_info->affine.sy*height-draw_info->affine.rx*metrics.width-
-          draw_info->affine.sy*(number_lines-1)*height;
+          draw_info->affine.sy*(number_lines-1)*height+
+          draw_info->affine.sy*metrics.ascent;
         break;
       }
     }
@@ -349,28 +350,29 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
       metrics.underline_thickness,metrics.width);
     if (annotate->decorate == OverlineDecoration)
       {
-        clone_info->affine.ty-=draw_info->affine.sy*
-          (metrics.ascent+metrics.descent)-metrics.underline_position;
+        clone_info->affine.ty-=(draw_info->affine.sy*
+          (metrics.ascent+metrics.descent-metrics.underline_position+2));
         (void) CloneString(&clone_info->primitive,primitive);
         (void) DrawImage(image,clone_info);
       }
     else
       if (annotate->decorate == UnderlineDecoration)
         {
-          clone_info->affine.ty-=metrics.underline_position;
+          clone_info->affine.ty-=
+            (draw_info->affine.sy*metrics.underline_position);
           (void) CloneString(&clone_info->primitive,primitive);
           (void) DrawImage(image,clone_info);
         }
     /*
       Annotate image with text.
     */
-    status=RenderType(image,annotate,&offset,True,&metrics);
+    status=RenderType(image,annotate,&offset,&metrics);
     if (status == False)
       break;
     if (annotate->decorate == LineThroughDecoration)
       {
-        clone_info->affine.ty-=draw_info->affine.sy*height/2.0-
-          metrics.underline_position/2.0;
+        clone_info->affine.ty-=(draw_info->affine.sy*(height+
+          metrics.underline_position+metrics.descent)/2.0);
         (void) CloneString(&clone_info->primitive,primitive);
         (void) DrawImage(image,clone_info);
       }
@@ -408,7 +410,7 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
 %
 %  The format of the EncodeSJIS function is:
 %
-%      encoding=EncodeSJIS(text,count)
+%      encoding=EncodeSJIS(const char *text,size_t count)
 %
 %  A description of each parameter follows:
 %
@@ -433,7 +435,7 @@ static int GetOneCharacter(const unsigned char *text,size_t *length)
   if (!(c & 0x80))
     {
       *length=1;
-      return(c);
+      return((int) c);
     }
   if (*length < 2)
     {
@@ -443,7 +445,7 @@ static int GetOneCharacter(const unsigned char *text,size_t *length)
   *length=2;
   c=((int) (text[0]) << 8);
   c|=text[1];
-  return(c);
+  return((int) c);
 }
 
 static unsigned short *EncodeSJIS(const char *text,size_t *count)
@@ -469,8 +471,8 @@ static unsigned short *EncodeSJIS(const char *text,size_t *count)
   encoding=(unsigned short *)
     AcquireMemory((strlen(text)+MaxTextExtent)*sizeof(unsigned short));
   if (encoding == (unsigned short *) NULL)
-    MagickFatalError(ResourceLimitFatalError,
-      "Unable to convert text to Textcode","Memory allocation failed");
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToConvertText");
   q=encoding;
   for (p=text; *p != '\0'; p+=length)
   {
@@ -506,7 +508,7 @@ static unsigned short *EncodeSJIS(const char *text,size_t *count)
 %
 %  The format of the EncodeText function is:
 %
-%      encoding=EncodeText(text,count)
+%      encoding=EncodeText(const char *text,size_t count)
 %
 %  A description of each parameter follows:
 %
@@ -536,8 +538,8 @@ static unsigned short *EncodeText(const char *text,size_t *count)
   encoding=(unsigned short *)
     AcquireMemory((strlen(text)+MaxTextExtent)*sizeof(unsigned short));
   if (encoding == (unsigned short *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"Unable to convert text to Text",
-      "Memory allocation failed");
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToConvertText");
   q=encoding;
   for (p=text; *p != '\0'; p++)
     *q++=(unsigned char) *p;
@@ -562,7 +564,7 @@ static unsigned short *EncodeText(const char *text,size_t *count)
 %
 %  The format of the EncodeUnicode function is:
 %
-%      unicode=EncodeUnicode(text,count)
+%      unicode=EncodeUnicode(const unsigned char *text,size_t count)
 %
 %  A description of each parameter follows:
 %
@@ -576,9 +578,9 @@ static unsigned short *EncodeText(const char *text,size_t *count)
 %
 */
 
-static int GetUnicodeCharacter(const unsigned char *text,size_t *length)
+static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
 {
-  unsigned int
+  unsigned long
     c;
 
   if (*length < 1)
@@ -587,7 +589,7 @@ static int GetUnicodeCharacter(const unsigned char *text,size_t *length)
   if (!(c & 0x80))
     {
       *length=1;
-      return(c);
+      return((long) c);
     }
   if ((*length < 2) || ((text[1] & 0xc0) != 0x80))
     {
@@ -599,7 +601,7 @@ static int GetUnicodeCharacter(const unsigned char *text,size_t *length)
       *length=2;
       c=(text[0] & 0x1f) << 6;
       c|=text[1] & 0x3f;
-      return(c);
+      return((long) c);
     }
   if ((*length < 3) || ((text[2] & 0xc0) != 0x80))
     {
@@ -612,7 +614,7 @@ static int GetUnicodeCharacter(const unsigned char *text,size_t *length)
       c=(text[0] & 0xf) << 12;
       c|=(text[1] & 0x3f) << 6;
       c|=text[2] & 0x3f;
-      return(c);
+      return((long) c);
     }
   if ((*length < 4) || ((c & 0xf8) != 0xf0) || ((text[3] & 0xc0) != 0x80))
     {
@@ -624,7 +626,7 @@ static int GetUnicodeCharacter(const unsigned char *text,size_t *length)
   c|=(text[1] & 0x3f) << 12;
   c|=(text[2] & 0x3f) << 6;
   c|=text[3] & 0x3f;
-  return(c);
+  return((long) c);
 }
 
 static unsigned short *EncodeUnicode(const char *text,size_t *count)
@@ -650,8 +652,8 @@ static unsigned short *EncodeUnicode(const char *text,size_t *count)
   unicode=(unsigned short *)
     AcquireMemory((strlen(text)+MaxTextExtent)*sizeof(unsigned short));
   if (unicode == (unsigned short *) NULL)
-    MagickFatalError(ResourceLimitFatalError,
-      "Unable to convert text to Unicode","Memory allocation failed");
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToConvertText");
   q=unicode;
   for (p=text; *p != '\0'; p+=length)
   {
@@ -713,6 +715,9 @@ static unsigned short *EncodeUnicode(const char *text,size_t *count)
 MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
   TypeMetric *metrics)
 {
+  DrawInfo
+    *clone_info;
+
   PointInfo
     offset;
 
@@ -722,10 +727,13 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->text != (char *) NULL);
   assert(draw_info->signature == MagickSignature);
+  clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
+  clone_info->render=False;
   memset(metrics,0,sizeof(TypeMetric));
   offset.x=0.0;
   offset.y=0.0;
-  status=RenderType(image,draw_info,&offset,False,metrics);
+  status=RenderType(image,clone_info,&offset,metrics);
+  DestroyDrawInfo(clone_info);
   return(status);
 }
 
@@ -746,7 +754,7 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
 %  The format of the RenderType method is:
 %
 %      unsigned int RenderType(Image *image,DrawInfo *draw_info,
-%        const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+%        const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
@@ -759,52 +767,43 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
 %
 %    o offset: (x,y) location of text relative to image.
 %
-%    o render: a value other than zero renders the text otherwise just the
-%      font metric is returned.
-%
 %    o metrics: bounding box of text.
 %
 %
 */
 static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+  const PointInfo *offset,TypeMetric *metrics)
 {
   const TypeInfo
     *type_info;
 
   DrawInfo
-    *clone_info;
+   *clone_info;
 
   unsigned int
     status;
 
-  type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
-    draw_info->stretch,draw_info->weight,&image->exception);
+  type_info=(const TypeInfo *) NULL;
   if (draw_info->font != (char *) NULL)
-    type_info=GetTypeInfo(draw_info->font,&image->exception);
-  clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
-  if (type_info != (const TypeInfo *) NULL)
     {
-      if (type_info->glyphs != (char *) NULL)
-        (void) CloneString(&clone_info->font,type_info->glyphs);
-      if (type_info->encoding != (char *) NULL)
-        (void) CloneString(&clone_info->encoding,type_info->encoding);
-      status=RenderFreetype(image,clone_info,offset,render,metrics);
-    }
-  else
-    if (*draw_info->font == '@')
-      {
-        (void) CloneString(&clone_info->font,draw_info->font+1);
-        status=RenderFreetype(image,clone_info,offset,render,metrics);
-      }
-    else
+      if (*draw_info->font == '@')
+        return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
       if (*draw_info->font == '-')
-        status=RenderX11(image,clone_info,offset,render,metrics);
-      else
-        if (IsAccessible(clone_info->font))
-          status=RenderFreetype(image,clone_info,offset,render,metrics);
-        else
-          status=RenderPostscript(image,clone_info,offset,render,metrics);
+        return(RenderX11(image,draw_info,offset,metrics));
+      type_info=GetTypeInfo(draw_info->font,&image->exception);
+      if (type_info == (const TypeInfo *) NULL)
+        if (IsAccessible(draw_info->font))
+          return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
+    }
+  if (type_info == (const TypeInfo *) NULL)
+    type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
+      draw_info->stretch,draw_info->weight,&image->exception);
+  if (type_info == (const TypeInfo *) NULL)
+    return(RenderPostscript(image,draw_info,offset,metrics));
+  clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
+  if (type_info->glyphs != (char *) NULL)
+    (void) CloneString(&clone_info->font,type_info->glyphs);
+  status=RenderFreetype(image,clone_info,type_info->encoding,offset,metrics);
   DestroyDrawInfo(clone_info);
   return(status);
 }
@@ -826,7 +825,7 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
 %  The format of the RenderFreetype method is:
 %
 %      unsigned int RenderFreetype(Image *image,DrawInfo *draw_info,
-%        const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+%        const char *encoding,const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
@@ -837,10 +836,9 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
 %
 %    o draw_info: The draw info.
 %
-%    o offset: (x,y) location of text relative to image.
+%    o encoding: The font encoding.
 %
-%    o render: a value other than zero renders the text otherwise just the
-%      font metric is returned.
+%    o offset: (x,y) location of text relative to image.
 %
 %    o metrics: bounding box of text.
 %
@@ -850,21 +848,17 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
 static inline PixelPacket AlphaComposite(const PixelPacket *p,
   const double alpha,const PixelPacket *q,const double beta)
 {
-  register double
-    scale;
-
   PixelPacket
     composite;
 
-  scale=1.0/MaxRGB;
   composite.red=(Quantum)
-    (scale*((MaxRGB-alpha)*p->red+scale*alpha*(MaxRGB-beta)*q->red)+0.5);
+    (((MaxRGB-alpha)*p->red+alpha*(MaxRGB-beta)*q->red/MaxRGB)/MaxRGB+0.5);
   composite.green=(Quantum)
-    (scale*((MaxRGB-alpha)*p->green+scale*alpha*(MaxRGB-beta)*q->green)+0.5);
+    (((MaxRGB-alpha)*p->green+alpha*(MaxRGB-beta)*q->green/MaxRGB)/MaxRGB+0.5);
   composite.blue=(Quantum)
-    (scale*((MaxRGB-alpha)*p->blue+scale*alpha*(MaxRGB-beta)*q->blue)+0.5);
+    (((MaxRGB-alpha)*p->blue+alpha*(MaxRGB-beta)*q->blue/MaxRGB)/MaxRGB+0.5);
   composite.opacity=(Quantum)
-    (MaxRGB-((MaxRGB-alpha)+scale*alpha*(MaxRGB-beta))+0.5);
+    (MaxRGB-((MaxRGB-alpha)+alpha*(MaxRGB-beta)/MaxRGB)+0.5);
   return(composite);
 }
 
@@ -931,7 +925,7 @@ static int TraceQuadraticBezier(FT_Vector *control,FT_Vector *to,
 }
 
 static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+  const char *encoding,const PointInfo *offset,TypeMetric *metrics)
 {
   typedef struct _GlyphInfo
   {
@@ -945,8 +939,8 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
       image;
   } GlyphInfo;
 
-  char
-    font[MaxTextExtent];
+  double
+    opacity;
 
   DrawInfo
     *clone_info;
@@ -992,9 +986,6 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
     point,
     resolution;
 
-  Quantum
-    opacity;
-
   register long
     i,
     x;
@@ -1022,56 +1013,60 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
     active;
 
   unsigned short
-    *encoding;
+    *text;
 
   /*
     Initialize Truetype library.
   */
   status=FT_Init_FreeType(&library);
   if (status)
-    ThrowBinaryException(DelegateError,"Unable to open freetype library",
+    ThrowBinaryException(TypeError,"UnableToInitializeFreetypeLibrary",
       draw_info->font);
-  status=FT_New_Face(library,draw_info->font,0,&face);
+  if (*draw_info->font != '@')
+    status=FT_New_Face(library,draw_info->font,0,&face);
+  else
+    status=FT_New_Face(library,draw_info->font+1,0,&face);
   if (status != 0)
     {
       (void) FT_Done_FreeType(library);
-      ThrowBinaryException(DelegateError,"Unable to read font",
-        draw_info->font)
+      ThrowBinaryException(DelegateError,"UnableToReadFont",draw_info->font)
     }
-  (void) strncpy(font,draw_info->font,MaxTextExtent-1);
   if (face->num_charmaps != 0)
     status=FT_Set_Charmap(face,face->charmaps[0]);
-  encoding_type=ft_encoding_none;
-  if (draw_info->encoding != (char *) NULL)
+  encoding_type=ft_encoding_unicode;
+  status=FT_Select_Charmap(face,encoding_type);
+  if (status != 0)
     {
-      if (LocaleCompare(draw_info->encoding,"AdobeCustom") == 0)
+      encoding_type=ft_encoding_none;
+      status=FT_Select_Charmap(face,encoding_type);
+    }
+  if (encoding != (char *) NULL)
+    {
+      if (LocaleCompare(encoding,"AdobeCustom") == 0)
         encoding_type=ft_encoding_adobe_custom;
-      if (LocaleCompare(draw_info->encoding,"AdobeExpert") == 0)
+      if (LocaleCompare(encoding,"AdobeExpert") == 0)
         encoding_type=ft_encoding_adobe_expert;
-      if (LocaleCompare(draw_info->encoding,"AdobeStandard") == 0)
+      if (LocaleCompare(encoding,"AdobeStandard") == 0)
         encoding_type=ft_encoding_adobe_standard;
-      if (LocaleCompare(draw_info->encoding,"AppleRoman") == 0)
+      if (LocaleCompare(encoding,"AppleRoman") == 0)
         encoding_type=ft_encoding_apple_roman;
-      if (LocaleCompare(draw_info->encoding,"BIG5") == 0)
+      if (LocaleCompare(encoding,"BIG5") == 0)
         encoding_type=ft_encoding_big5;
-      if (LocaleCompare(draw_info->encoding,"GB2312") == 0)
+      if (LocaleCompare(encoding,"GB2312") == 0)
         encoding_type=ft_encoding_gb2312;
-      if (LocaleCompare(draw_info->encoding,"Latin 2") == 0)
-        encoding_type=ft_encoding_latin_2;
-      if (LocaleCompare(draw_info->encoding,"None") == 0)
+      if (LocaleCompare(encoding,"None") == 0)
         encoding_type=ft_encoding_none;
-      if (LocaleCompare(draw_info->encoding,"SJIScode") == 0)
+      if (LocaleCompare(encoding,"SJIScode") == 0)
         encoding_type=ft_encoding_sjis;
-      if (LocaleCompare(draw_info->encoding,"Symbol") == 0)
+      if (LocaleCompare(encoding,"Symbol") == 0)
         encoding_type=ft_encoding_symbol;
-      if (LocaleCompare(draw_info->encoding,"Unicode") == 0)
+      if (LocaleCompare(encoding,"Unicode") == 0)
         encoding_type=ft_encoding_unicode;
-      if (LocaleCompare(draw_info->encoding,"Wansung") == 0)
+      if (LocaleCompare(encoding,"Wansung") == 0)
         encoding_type=ft_encoding_wansung;
       status=FT_Select_Charmap(face,encoding_type);
       if (status != 0)
-        ThrowBinaryException(DelegateError,"Unrecognized encoding",
-          draw_info->encoding);
+        ThrowBinaryException(TypeError,"UnrecognizedFontEncoding",encoding);
     }
   /*
     Set text size.
@@ -1080,8 +1075,16 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   resolution.y=72.0;
   if (draw_info->density != (char *) NULL)
     {
-      i=sscanf(draw_info->density,"%lfx%lf",&resolution.x,&resolution.y);
-      if (i != 2)
+      GeometryInfo
+        geometry_info;
+
+      unsigned int
+        flags;
+
+      flags=ParseGeometry(draw_info->density,&geometry_info);
+      resolution.x=geometry_info.rho;
+      resolution.y=geometry_info.sigma;
+      if (!(flags & SigmaValue))
         resolution.y=resolution.x;
     }
   (void) FT_Set_Char_Size(face,(FT_F26Dot6) (64.0*draw_info->pointsize),
@@ -1107,48 +1110,65 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   {
     case ft_encoding_sjis:
     {
-      encoding=EncodeSJIS(draw_info->text,&length);
+      text=EncodeSJIS(draw_info->text,&length);
+      break;
+    }
+    case ft_encoding_unicode:
+    {
+      text=EncodeUnicode(draw_info->text,&length);
       break;
     }
     default:
     {
-      if (!draw_info->unicode)
+      if (draw_info->encoding != (char *) NULL)
         {
-          encoding=EncodeText(draw_info->text,&length);
-          break;
+          if (LocaleCompare(draw_info->encoding,"SJIS") == 0)
+            {
+              text=EncodeSJIS(draw_info->text,&length);
+              break;
+            }
+          if ((LocaleCompare(draw_info->encoding,"UTF-8") == 0) ||
+              (encoding_type != ft_encoding_none))
+            {
+              text=EncodeUnicode(draw_info->text,&length);
+              break;
+            }
         }
-    }
-    case ft_encoding_unicode:
-    {
-      encoding=EncodeUnicode(draw_info->text,&length);
+      text=EncodeText(draw_info->text,&length);
       break;
     }
   }
-  if (encoding == (unsigned short *) NULL)
+  if (text == (unsigned short *) NULL)
     {
       (void) FT_Done_Face(face);
       (void) FT_Done_FreeType(library);
-      ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
-        "Memory allocation failed")
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        draw_info->font)
     }
   /*
     Compute bounding box.
   */
+  (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),
+    "Font %.1024s; font-encoding %.1024s; text-encoding %.1024s; pointsize %g",
+    draw_info->font != (char *) NULL ? draw_info->font : "none",
+    encoding != (char *) NULL ? encoding : "none",
+    draw_info->encoding != (char *) NULL ? draw_info->encoding : "none",
+    draw_info->pointsize);
   glyph.id=0;
   last_glyph.id=0;
   origin.x=0;
   origin.y=0;
-  affine.xx=(FT_Fixed) (65536.0*draw_info->affine.sx);
-  affine.yx=(FT_Fixed) (-65536.0*draw_info->affine.rx);
-  affine.xy=(FT_Fixed) (-65536.0*draw_info->affine.ry);
-  affine.yy=(FT_Fixed) (65536.0*draw_info->affine.sy);
+  affine.xx=(FT_Fixed) (65536L*draw_info->affine.sx+0.5);
+  affine.yx=(FT_Fixed) (-65536L*draw_info->affine.rx+0.5);
+  affine.xy=(FT_Fixed) (-65536L*draw_info->affine.ry+0.5);
+  affine.yy=(FT_Fixed) (65536L*draw_info->affine.sy+0.5);
   clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   (void) QueryColorDatabase("#000000ff",&clone_info->fill,&image->exception);
   (void) CloneString(&clone_info->primitive,"path '");
   pattern=draw_info->fill_pattern;
   for (i=0; i < (long) length; i++)
   {
-    glyph.id=FT_Get_Char_Index(face,encoding[i]);
+    glyph.id=FT_Get_Char_Index(face,text[i]);
     if ((glyph.id != 0) && (last_glyph.id != 0) && FT_HAS_KERNING(face))
       {
         FT_Vector
@@ -1174,7 +1194,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
       metrics->bounds.x2=bounds.xMax;
     if ((i == 0) || (bounds.yMax > metrics->bounds.y2))
       metrics->bounds.y2=bounds.yMax;
-    if (render)
+    if (draw_info->render)
       if ((draw_info->stroke.opacity != TransparentOpacity) ||
           (draw_info->stroke_pattern != (Image *) NULL))
         {
@@ -1188,7 +1208,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
         }
     FT_Vector_Transform(&glyph.origin,&affine);
     (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
-    if (render)
+    if (draw_info->render)
       {
         if ((draw_info->fill.opacity != TransparentOpacity) ||
             (pattern != (Image *) NULL))
@@ -1226,18 +1246,16 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
                     continue;
                   }
                 if (draw_info->text_antialias)
-                  opacity=Upscale(*p);
+                  opacity=ScaleCharToQuantum(*p);
                 else
-                  opacity=(Quantum)
-                    ((*p) < 127 ? OpaqueOpacity : TransparentOpacity);
+                  opacity=((*p) < 127 ? OpaqueOpacity : TransparentOpacity);
                 fill_color=draw_info->fill;
                 if (pattern != (Image *) NULL)
                   fill_color=AcquireOnePixel(pattern,
                     (long) (point.x+x-pattern->tile_info.x) % pattern->columns,
                     (long) (point.y+y-pattern->tile_info.y) % pattern->rows,
-                    &image->exception);
-                opacity=(Quantum) ((unsigned long) ((MaxRGB-opacity)*
-                  (MaxRGB-fill_color.opacity))/MaxRGB);
+										&image->exception);
+                opacity=((MaxRGB-opacity)*(MaxRGB-fill_color.opacity))/MaxRGB;
                 if (!active)
                   q=GetImagePixels(image,(long) ceil(point.x+x-0.5),
                     (long) ceil(point.y+y-0.5),1,1);
@@ -1269,7 +1287,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y1/=64.0;
   metrics->bounds.x2/=64.0;
   metrics->bounds.y2/=64.0;
-  if (render)
+  if (draw_info->render)
     if ((draw_info->stroke.opacity != TransparentOpacity) ||
         (draw_info->stroke_pattern != (Image *) NULL))
       {
@@ -1286,7 +1304,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   /*
     Free resources.
   */
-  LiberateMemory((void **) &encoding);
+  LiberateMemory((void **) &text);
   DestroyDrawInfo(clone_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
@@ -1294,10 +1312,10 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
 }
 #else
 static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+  const char *encoding,const PointInfo *offset,TypeMetric *metrics)
 {
-  ThrowBinaryException(MissingDelegateError,
-    "FreeType library is not available",draw_info->font);
+  ThrowBinaryException(DelegateError,"FreeTypeLibraryIsNotAvailable",
+    draw_info->font)
 }
 #endif
 
@@ -1318,7 +1336,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
 %  The format of the RenderPostscript method is:
 %
 %      unsigned int RenderPostscript(Image *image,DrawInfo *draw_info,
-%        const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+%        const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
@@ -1331,9 +1349,6 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
 %
 %    o offset: (x,y) location of text relative to image.
 %
-%    o render: a value other than zero renders the text otherwise just the
-%      font metric is returned.
-%
 %    o metrics: bounding box of text.
 %
 %
@@ -1370,7 +1385,7 @@ static char *EscapeParenthesis(const char *text)
 }
 
 static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+  const PointInfo *offset,TypeMetric *metrics)
 {
   char
     filename[MaxTextExtent],
@@ -1408,10 +1423,13 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   /*
     Render label with a Postscript font.
   */
+  (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),
+    "Font %.1024s; pointsize %g",draw_info->font != (char *) NULL ?
+    draw_info->font : "none",draw_info->pointsize);
   TemporaryFilename(filename);
-  file=fopen(filename,WriteBinaryType);
+  file=fopen(filename,"wb");
   if (file == (FILE *) NULL)
-    ThrowBinaryException(FileOpenError,"Unable to open file",filename);
+    ThrowBinaryException(FileOpenError,"UnableToOpenFile",filename);
   (void) fprintf(file,"%%!PS-Adobe-3.0\n");
   (void) fprintf(file,"/ReencodeType\n");
   (void) fprintf(file,"{\n");
@@ -1480,11 +1498,16 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   resolution.y=72.0;
   if (draw_info->density != (char *) NULL)
     {
-      int
-        count;
+      GeometryInfo
+        geometry_info;
 
-      count=sscanf(draw_info->density,"%lfx%lf",&resolution.x,&resolution.y);
-      if (count != 2)
+      unsigned int
+        flags;
+
+      flags=ParseGeometry(draw_info->density,&geometry_info);
+      resolution.x=geometry_info.rho;
+      resolution.y=geometry_info.sigma;
+      if (!(flags & SigmaValue))
         resolution.y=resolution.x;
     }
   if (!identity)
@@ -1495,7 +1518,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
         crop_info;
 
       crop_info=GetImageBoundingBox(annotate_image,&annotate_image->exception);
-      crop_info.height=(unsigned int) ceil((resolution.y/72.0)*
+      crop_info.height=(unsigned long) ceil((resolution.y/72.0)*
         ExpandAffine(&draw_info->affine)*draw_info->pointsize-0.5);
       crop_info.y=(long) ceil((resolution.y/72.0)*extent.y/8.0-0.5);
       (void) FormatString(geometry,"%lux%lu%+ld%+ld",crop_info.width,
@@ -1516,7 +1539,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y2=metrics->ascent+metrics->descent;
   metrics->underline_position=(-2.0);
   metrics->underline_thickness=1.0;
-  if (!render)
+  if (!draw_info->render)
     {
       DestroyImage(annotate_image);
       return(True);
@@ -1541,10 +1564,12 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
         {
           if (pattern != (Image *) NULL)
             fill_color=AcquireOnePixel(pattern,
-              (x-pattern->tile_info.x) % pattern->columns,
-              (y-pattern->tile_info.y) % pattern->rows,&image->exception);
-          q->opacity=(Quantum) (MaxRGB-((unsigned long) ((MaxRGB-Intensity(q))*
-            (MaxRGB-fill_color.opacity))/MaxRGB));
+              (long) (x-pattern->tile_info.x) % pattern->columns,
+              (long) (y-pattern->tile_info.y) % pattern->rows,
+              &image->exception);
+          q->opacity=(Quantum) (MaxRGB-(((MaxRGB-(double)
+            PixelIntensityToQuantum(q))*(MaxRGB-fill_color.opacity))/
+            MaxRGB)+0.5);
           q->red=fill_color.red;
           q->green=fill_color.green;
           q->blue=fill_color.blue;
@@ -1578,7 +1603,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
 %  The format of the RenderX11 method is:
 %
 %      unsigned int RenderX11(Image *image,DrawInfo *draw_info,
-%        const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+%        const PointInfo *offset,TypeMetric *metrics)
 %
 %  A description of each parameter follows:
 %
@@ -1591,16 +1616,13 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
 %
 %    o offset: (x,y) location of text relative to image.
 %
-%    o render: a value other than zero renders the text otherwise just the
-%      font metric is returned.
-%
 %    o metrics: bounding box of text.
 %
 %
 */
 #if defined(HasX11)
 static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+  const PointInfo *offset,TypeMetric *metrics)
 {
   static DrawInfo
     cache_info;
@@ -1630,8 +1652,10 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
     *visual_info;
 
   unsigned int
+    status;
+
+  unsigned long
     height,
-    status,
     width;
 
   if (display == (Display *) NULL)
@@ -1644,7 +1668,7 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       */
       display=XOpenDisplay(draw_info->server_name);
       if (display == (Display *) NULL)
-        ThrowBinaryException(XServerError,"Unable to open X server",
+        ThrowBinaryException(XServerError,"UnableToOpenXServer",
           draw_info->server_name);
       /*
         Get user defaults from X resource database.
@@ -1656,22 +1680,21 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       resource_info.close_server=False;
       resource_info.colormap=PrivateColormap;
       resource_info.font=AllocateString(draw_info->font);
-      resource_info.background_color=AllocateString("#ffffff");
-      resource_info.foreground_color=AllocateString("#000000");
+      resource_info.background_color=AllocateString("#ffffffffffff");
+      resource_info.foreground_color=AllocateString("#000000000000");
       map_info=XAllocStandardColormap();
       if (map_info == (XStandardColormap *) NULL)
-        ThrowBinaryException(ResourceLimitError,"Unable to allocate colormap",
-          "Memory allocation failed");
+        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+          "UnableToAllocateColormap");
       /*
         Initialize visual info.
       */
       visual_info=XBestVisualInfo(display,map_info,&resource_info);
       if (visual_info == (XVisualInfo *) NULL)
-        ThrowBinaryException(XServerError,"Unable to get visual",
+        ThrowBinaryException(XServerError,"UnableToGetVisual",
           draw_info->server_name);
       map_info->colormap=(Colormap) NULL;
       pixel.pixels=(unsigned long *) NULL;
-      pixel.gamma_map=(XColor *) NULL;
       /*
         Initialize Standard Colormap info.
       */
@@ -1685,15 +1708,14 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       */
       font_info=XBestFont(display,&resource_info,False);
       if (font_info == (XFontStruct *) NULL)
-        ThrowBinaryException(XServerError,"Unable to load font",
-          draw_info->font);
+        ThrowBinaryException(XServerError,"UnableToLoadFont",draw_info->font);
       if ((map_info == (XStandardColormap *) NULL) ||
           (visual_info == (XVisualInfo *) NULL) ||
           (font_info == (XFontStruct *) NULL))
         {
           XFreeResources(display,visual_info,map_info,&pixel,font_info,
             &resource_info,(XWindowInfo *) NULL);
-          ThrowBinaryException(XServerError,"Unable to get X server font",
+          ThrowBinaryException(XServerError,"UnableToLoadFont",
             draw_info->server_name)
         }
       cache_info=(*draw_info);
@@ -1712,9 +1734,11 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       (void) CloneString(&resource_info.font,draw_info->font);
       font_info=XBestFont(display,&resource_info,False);
       if (font_info == (XFontStruct *) NULL)
-        ThrowBinaryException(ResourceLimitError,"Unable to load font",
-          draw_info->font);
+        ThrowBinaryException(XServerError,"UnableToLoadFont",draw_info->font);
     }
+  (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),
+    "Font %.1024s; pointsize %g",draw_info->font != (char *) NULL ?
+    draw_info->font : "none",draw_info->pointsize);
   cache_info=(*draw_info);
   annotate_info.font_info=font_info;
   annotate_info.text=(char *) draw_info->text;
@@ -1734,7 +1758,7 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y2=metrics->ascent+metrics->descent;
   metrics->underline_position=(-2.0);
   metrics->underline_thickness=1.0;
-  if (!render)
+  if (!draw_info->render)
     return(True);
   if (draw_info->fill.opacity == TransparentOpacity)
     return(True);
@@ -1750,23 +1774,23 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
         annotate_info.degrees=(180.0/MagickPI)*
           atan2(draw_info->affine.rx,draw_info->affine.sx);
     }
-  FormatString(annotate_info.geometry,"%ux%u+%d+%d",width,height,
-    (int) ceil(offset->x-0.5),
-    (int) ceil(offset->y-metrics->ascent-metrics->descent-0.5));
-  pixel.pen_color.red=XUpscale(draw_info->fill.red);
-  pixel.pen_color.green=XUpscale(draw_info->fill.green);
-  pixel.pen_color.blue=XUpscale(draw_info->fill.blue);
+  FormatString(annotate_info.geometry,"%lux%lu+%ld+%ld",width,height,
+    (long) ceil(offset->x-0.5),
+    (long) ceil(offset->y-metrics->ascent-metrics->descent-0.5));
+  pixel.pen_color.red=ScaleQuantumToShort(draw_info->fill.red);
+  pixel.pen_color.green=ScaleQuantumToShort(draw_info->fill.green);
+  pixel.pen_color.blue=ScaleQuantumToShort(draw_info->fill.blue);
   status=XAnnotateImage(display,&pixel,&annotate_info,image);
   if (status == 0)
-    ThrowBinaryException(ResourceLimitError,"Unable to annotate image",
-      "Memory allocation failed");
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToAnnotateImage");
   return(True);
 }
 #else
 static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,const unsigned int render,TypeMetric *metrics)
+  const PointInfo *offset,TypeMetric *metrics)
 {
-  ThrowBinaryException(MissingDelegateError,
-    "X11 library is not available",draw_info->font);
+  ThrowBinaryException(DelegateError,"XWindowLibraryIsNotAvailable",
+    draw_info->font);
 }
 #endif
