@@ -18,7 +18,7 @@
 %                              February 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -56,6 +56,11 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "blob.h"
+#include "list.h"
+#include "render.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -133,7 +138,7 @@ MagickExport void DestroyImageAttributes(Image *image)
 %
 */
 
-static int GenerateIPTCAttribute(Image *image,const char *key)
+static unsigned int GenerateIPTCAttribute(Image *image,const char *key)
 {
   char
     *attribute;
@@ -313,8 +318,8 @@ static char *TraceClippingPath(char *blob,size_t length,unsigned long columns,
             {
               y=ReadMSBLong(&blob,&length);
               x=ReadMSBLong(&blob,&length);
-              point[i].y=(double) y*rows/4096/4096;
               point[i].x=(double) x*columns/4096/4096;
+              point[i].y=(double) y*rows/4096/4096;
             }
             if (status)
               {
@@ -477,88 +482,88 @@ typedef struct _TagInfo
 static TagInfo
   tag_table[] =
   {
-    {  0x100, "ImageWidth"},
-    {  0x101, "ImageLength"},
-    {  0x102, "BitsPerSample"},
-    {  0x103, "Compression"},
-    {  0x106, "PhotometricInterpretation"},
-    {  0x10A, "FillOrder"},
-    {  0x10D, "DocumentName"},
-    {  0x10E, "ImageDescription"},
-    {  0x10F, "Make"},
-    {  0x110, "Model"},
-    {  0x111, "StripOffsets"},
-    {  0x112, "Orientation"},
-    {  0x115, "SamplesPerPixel"},
-    {  0x116, "RowsPerStrip"},
-    {  0x117, "StripByteCounts"},
-    {  0x11A, "XResolution"},
-    {  0x11B, "YResolution"},
-    {  0x11C, "PlanarConfiguration"},
-    {  0x128, "ResolutionUnit"},
-    {  0x12D, "TransferFunction"},
-    {  0x131, "Software"},
-    {  0x132, "DateTime"},
-    {  0x13B, "Artist"},
-    {  0x13E, "WhitePoint"},
-    {  0x13F, "PrimaryChromaticities"},
-    {  0x156, "TransferRange"},
-    {  0x200, "JPEGProc"},
-    {  0x201, "JPEGInterchangeFormat"},
-    {  0x202, "JPEGInterchangeFormatLength"},
-    {  0x211, "YCbCrCoefficients"},
-    {  0x212, "YCbCrSubSampling"},
-    {  0x213, "YCbCrPositioning"},
-    {  0x214, "ReferenceBlackWhite"},
-    {  0x828D, "CFARepeatPatternDim"},
-    {  0x828E, "CFAPattern"},
-    {  0x828F, "BatteryLevel"},
-    {  0x8298, "Copyright"},
-    {  0x829A, "ExposureTime"},
-    {  0x829D, "FNumber"},
-    {  0x83BB, "IPTC/NAA"},
-    {  0x8769, "ExifOffset"},
-    {  0x8773, "InterColorProfile"},
-    {  0x8822, "ExposureProgram"},
-    {  0x8824, "SpectralSensitivity"},
-    {  0x8825, "GPSInfo"},
-    {  0x8827, "ISOSpeedRatings"},
-    {  0x8828, "OECF"},
-    {  0x9000, "ExifVersion"},
-    {  0x9003, "DateTimeOriginal"},
-    {  0x9004, "DateTimeDigitized"},
-    {  0x9101, "ComponentsConfiguration"},
-    {  0x9102, "CompressedBitsPerPixel"},
-    {  0x9201, "ShutterSpeedValue"},
-    {  0x9202, "ApertureValue"},
-    {  0x9203, "BrightnessValue"},
-    {  0x9204, "ExposureBiasValue"},
-    {  0x9205, "MaxApertureValue"},
-    {  0x9206, "SubjectDistance"},
-    {  0x9207, "MeteringMode"},
-    {  0x9208, "LightSource"},
-    {  0x9209, "Flash"},
-    {  0x920A, "FocalLength"},
-    {  0x927C, "MakerNote"},
-    {  0x9286, "UserComment"},
-    {  0x9290, "SubSecTime"},
-    {  0x9291, "SubSecTimeOriginal"},
-    {  0x9292, "SubSecTimeDigitized"},
-    {  0xA000, "FlashPixVersion"},
-    {  0xA001, "ColorSpace"},
-    {  0xA002, "ExifImageWidth"},
-    {  0xA003, "ExifImageLength"},
-    {  0xA005, "InteroperabilityOffset"},
-    {  0xA20B, "FlashEnergy"},
-    {  0xA20C, "SpatialFrequencyResponse"},
-    {  0xA20E, "FocalPlaneXResolution"},
-    {  0xA20F, "FocalPlaneYResolution"},
-    {  0xA210, "FocalPlaneResolutionUnit"},
-    {  0xA214, "SubjectLocation"},
-    {  0xA215, "ExposureIndex"},
-    {  0xA217, "SensingMethod"},
-    {  0xA300, "FileSource"},
-    {  0xA301, "SceneType"},
+    {  0x100, (char *) "ImageWidth"},
+    {  0x101, (char *) "ImageLength"},
+    {  0x102, (char *) "BitsPerSample"},
+    {  0x103, (char *) "Compression"},
+    {  0x106, (char *) "PhotometricInterpretation"},
+    {  0x10A, (char *) "FillOrder"},
+    {  0x10D, (char *) "DocumentName"},
+    {  0x10E, (char *) "ImageDescription"},
+    {  0x10F, (char *) "Make"},
+    {  0x110, (char *) "Model"},
+    {  0x111, (char *) "StripOffsets"},
+    {  0x112, (char *) "Orientation"},
+    {  0x115, (char *) "SamplesPerPixel"},
+    {  0x116, (char *) "RowsPerStrip"},
+    {  0x117, (char *) "StripByteCounts"},
+    {  0x11A, (char *) "XResolution"},
+    {  0x11B, (char *) "YResolution"},
+    {  0x11C, (char *) "PlanarConfiguration"},
+    {  0x128, (char *) "ResolutionUnit"},
+    {  0x12D, (char *) "TransferFunction"},
+    {  0x131, (char *) "Software"},
+    {  0x132, (char *) "DateTime"},
+    {  0x13B, (char *) "Artist"},
+    {  0x13E, (char *) "WhitePoint"},
+    {  0x13F, (char *) "PrimaryChromaticities"},
+    {  0x156, (char *) "TransferRange"},
+    {  0x200, (char *) "JPEGProc"},
+    {  0x201, (char *) "JPEGInterchangeFormat"},
+    {  0x202, (char *) "JPEGInterchangeFormatLength"},
+    {  0x211, (char *) "YCbCrCoefficients"},
+    {  0x212, (char *) "YCbCrSubSampling"},
+    {  0x213, (char *) "YCbCrPositioning"},
+    {  0x214, (char *) "ReferenceBlackWhite"},
+    {  0x828D, (char *) "CFARepeatPatternDim"},
+    {  0x828E, (char *) "CFAPattern"},
+    {  0x828F, (char *) "BatteryLevel"},
+    {  0x8298, (char *) "Copyright"},
+    {  0x829A, (char *) "ExposureTime"},
+    {  0x829D, (char *) "FNumber"},
+    {  0x83BB, (char *) "IPTC/NAA"},
+    {  0x8769, (char *) "ExifOffset"},
+    {  0x8773, (char *) "InterColorProfile"},
+    {  0x8822, (char *) "ExposureProgram"},
+    {  0x8824, (char *) "SpectralSensitivity"},
+    {  0x8825, (char *) "GPSInfo"},
+    {  0x8827, (char *) "ISOSpeedRatings"},
+    {  0x8828, (char *) "OECF"},
+    {  0x9000, (char *) "ExifVersion"},
+    {  0x9003, (char *) "DateTimeOriginal"},
+    {  0x9004, (char *) "DateTimeDigitized"},
+    {  0x9101, (char *) "ComponentsConfiguration"},
+    {  0x9102, (char *) "CompressedBitsPerPixel"},
+    {  0x9201, (char *) "ShutterSpeedValue"},
+    {  0x9202, (char *) "ApertureValue"},
+    {  0x9203, (char *) "BrightnessValue"},
+    {  0x9204, (char *) "ExposureBiasValue"},
+    {  0x9205, (char *) "MaxApertureValue"},
+    {  0x9206, (char *) "SubjectDistance"},
+    {  0x9207, (char *) "MeteringMode"},
+    {  0x9208, (char *) "LightSource"},
+    {  0x9209, (char *) "Flash"},
+    {  0x920A, (char *) "FocalLength"},
+    {  0x927C, (char *) "MakerNote"},
+    {  0x9286, (char *) "UserComment"},
+    {  0x9290, (char *) "SubSecTime"},
+    {  0x9291, (char *) "SubSecTimeOriginal"},
+    {  0x9292, (char *) "SubSecTimeDigitized"},
+    {  0xA000, (char *) "FlashPixVersion"},
+    {  0xA001, (char *) "ColorSpace"},
+    {  0xA002, (char *) "ExifImageWidth"},
+    {  0xA003, (char *) "ExifImageLength"},
+    {  0xA005, (char *) "InteroperabilityOffset"},
+    {  0xA20B, (char *) "FlashEnergy"},
+    {  0xA20C, (char *) "SpatialFrequencyResponse"},
+    {  0xA20E, (char *) "FocalPlaneXResolution"},
+    {  0xA20F, (char *) "FocalPlaneYResolution"},
+    {  0xA210, (char *) "FocalPlaneResolutionUnit"},
+    {  0xA214, (char *) "SubjectLocation"},
+    {  0xA215, (char *) "ExposureIndex"},
+    {  0xA217, (char *) "SensingMethod"},
+    {  0xA300, (char *) "FileSource"},
+    {  0xA301, (char *) "SceneType"},
     {  0x0000, (char *) NULL}
   };
 
@@ -854,13 +859,13 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
           {
             case EXIF_FMT_SBYTE:
             {
-              FormatString(s,"%d",(int)(*(char *) pval));
+              FormatString(s,"%ld",(long) (*(char *) pval));
               value=AllocateString(s);
               break;
             }
             case EXIF_FMT_BYTE:
             {
-              FormatString(s,"%d",(int)(*(unsigned char *) pval));
+              FormatString(s,"%ld",(long) (*(unsigned char *) pval));
               value=AllocateString(s);
               break;
             }
@@ -904,7 +909,7 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
             }
             case EXIF_FMT_SINGLE:
             {
-              FormatString(s,"%f",(double)*(float *) pval);
+              FormatString(s,"%f",(double) *(float *) pval);
               value=AllocateString(s);
               break;
             }
@@ -951,7 +956,7 @@ static int GenerateEXIFAttribute(Image *image,const char *specification)
               {
                 case 1:
                 {
-                  description="unknown";
+                  description=(char *) "unknown";
                   for (i=0; ; i++)
                   {
                     if (tag_table[i].tag == 0)
@@ -1201,15 +1206,11 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
     {
       if (LocaleNCompare("size",key,2) == 0)
         {
-          if (GetBlobSize(image) >= (1 << 24))
-            FormatString(attribute,"%lumb",
-              (unsigned long) (GetBlobSize(image)/1024/1024));
-          else
-            if (GetBlobSize(image) >= (1 << 16))
-              FormatString(attribute,"%lukb",
-                (unsigned long) (GetBlobSize(image)/1024));
-            else
-              FormatString(attribute,"%lu",(unsigned long) GetBlobSize(image));
+          char
+            format[MaxTextExtent];
+
+          FormatSize(GetBlobSize(image),format);
+          FormatString(attribute,"%.1024s",format);
           break;
         }
       if (LocaleNCompare("scene",key,2) == 0)
@@ -1221,7 +1222,7 @@ MagickExport const ImageAttribute *GetImageInfoAttribute(
         }
       if (LocaleNCompare("scenes",key,6) == 0)
         {
-          FormatString(attribute,"%lu",(unsigned long) GetImageListSize(image));
+          FormatString(attribute,"%lu",(unsigned long) GetImageListLength(image));
           break;
         }
        break;
@@ -1394,9 +1395,10 @@ MagickExport unsigned int SetImageAttribute(Image *image,const char *key,
   if (attribute == (ImageAttribute *) NULL)
     return(False);
   attribute->key=AllocateString(key);
-  if ((LocaleNCompare(key,"IPTC",4) == 0) ||
-      (LocaleNCompare(key,"8BIM",4) == 0) ||
-      (LocaleNCompare(key,"EXIF",4) == 0))
+  if ((LocaleNCompare(key,"EXIF",4) == 0) ||
+      (LocaleNCompare(key,"IPTC",4) == 0) ||
+      (LocaleNCompare(key,"[Locale",7) == 0) ||
+      (LocaleNCompare(key,"8BIM",4) == 0))
     attribute->value=AllocateString(value);
   else
     attribute->value=TranslateText((ImageInfo *) NULL,image,value);
