@@ -18,7 +18,7 @@
 %                              May  1993                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,12 +54,16 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "compress.h"
+#include "monitor.h"
+#include "utility.h"
 
 /*
   Define declarations.
 */
-#define LoadImageText  "  Load image...  "
-#define SaveImageText  "  Save image...  "
+#define LoadImageTag  "  Load image...  "
+#define SaveImageTag  "  Save image...  "
 
 /*
   Typedef declarations.
@@ -224,11 +228,9 @@ static char *Ascii85Tuple(unsigned char *data)
     i,
     x;
 
-  unsigned int
-		code;
-
   unsigned long
-		quantum;
+    code,
+    quantum;
 
   code=(((data[0] << 8) | data[1]) << 16) | (data[2] << 8) | data[3];
   if (code == 0L)
@@ -240,12 +242,12 @@ static char *Ascii85Tuple(unsigned char *data)
   quantum=85L*85L*85L*85L;
   for (i=0; i < 4; i++)
   {
-    x=code/quantum;
+    x=(long) (code/quantum);
     code-=quantum*x;
-    tuple[i]='!'+x;
+    tuple[i]=(char) ('!'+x);
     quantum/=85L;
-	}
-  tuple[4]='!'+(code % 85L);
+  }
+  tuple[4]=(char) ('!'+(code % 85L));
   tuple[5]='\0';
   return(tuple);
 }
@@ -253,13 +255,13 @@ static char *Ascii85Tuple(unsigned char *data)
 MagickExport void Ascii85Initialize(Image *image)
 {
   /*
-	 Allocate image structure.
+    Allocate image structure.
   */
-	image->ascii85=(Ascii85Info *) AcquireMemory(sizeof(Ascii85Info));
-	if (image->ascii85 == (Ascii85Info *) NULL)
-		MagickFatalError(ResourceLimitFatalError,"Unable to allocate Ascii85",
-      "Memory allocation failed");
-	(void) memset(image->ascii85,0,sizeof(Ascii85Info));
+  image->ascii85=(Ascii85Info *) AcquireMemory(sizeof(Ascii85Info));
+  if (image->ascii85 == (Ascii85Info *) NULL)
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateAscii85Info");
+  (void) memset(image->ascii85,0,sizeof(Ascii85Info));
   image->ascii85->line_break=MaxLineExtent << 1;
   image->ascii85->offset=0;
 }
@@ -379,7 +381,7 @@ MagickExport unsigned int HuffmanDecodeImage(Image *image)
       mask=0x80;  \
     }  \
   runlength++;  \
-  bit=byte & mask ? 1 : 0; \
+  bit=byte & mask ? 0x01 : 0x00; \
   mask>>=1;  \
   if (bit)  \
     runlength=0;  \
@@ -437,7 +439,7 @@ MagickExport unsigned int HuffmanDecodeImage(Image *image)
   if ((mb_hash == (HuffmanTable **) NULL) ||
       (mw_hash == (HuffmanTable **) NULL) ||
       (scanline == (unsigned char *) NULL))
-    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       (char *) NULL);
   /*
     Initialize Huffman tables.
@@ -591,7 +593,8 @@ MagickExport unsigned int HuffmanDecodeImage(Image *image)
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (!MagickMonitor(LoadImageTag,y,image->rows,&image->exception))
+        break;
     y++;
   }
   image->rows=Max(y-3,1);
@@ -655,7 +658,7 @@ MagickExport unsigned int HuffmanEncodeImage(const ImageInfo *image_info,
       if (LocaleCompare(image_info->magick,"FAX") == 0) \
         (void) WriteBlobByte(image,byte);  \
       else \
-        Ascii85Encode(image,(unsigned int) byte); \
+        Ascii85Encode(image,(unsigned long) byte); \
       byte=0;  \
       bit=0x80;  \
     }  \
@@ -696,10 +699,8 @@ MagickExport unsigned int HuffmanEncodeImage(const ImageInfo *image_info,
     byte,
     *scanline;
 
-  unsigned int
-    mask;
-
   unsigned long
+    mask,
     width;
 
   /*
@@ -712,14 +713,12 @@ MagickExport unsigned int HuffmanEncodeImage(const ImageInfo *image_info,
     width=Max(image->columns,1728);
   scanline=(unsigned char *) AcquireMemory(width+1);
   if (scanline == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       (char *) NULL);
   huffman_image=CloneImage(image,0,0,True,&image->exception);
   if (huffman_image == (Image *) NULL)
     return(False);
-  if ((huffman_image->storage_class == DirectClass) ||
-       !IsMonochromeImage(huffman_image,&image->exception))
-    SetImageType(huffman_image,BilevelType);
+  SetImageType(huffman_image,BilevelType);
   byte=0;
   bit=0x80;
   if (LocaleCompare(image_info->magick,"FAX") != 0)
@@ -736,10 +735,10 @@ MagickExport unsigned int HuffmanEncodeImage(const ImageInfo *image_info,
   /*
     Compress runlength encoded to 1D Huffman pixels.
   */
-  polarity=0;
+  polarity=PixelIntensityToQuantum(&huffman_image->colormap[0]) < (MaxRGB/2);
   if (huffman_image->colors == 2)
-    polarity=(Intensity(&huffman_image->colormap[0]) >
-      Intensity(&huffman_image->colormap[1]) ? 0 : 1);
+    polarity=(PixelIntensityToQuantum(&huffman_image->colormap[0]) <
+      PixelIntensityToQuantum(&huffman_image->colormap[1]) ? 0x00 : 0x01);
   q=scanline;
   for (i=0; i < (long) width; i++)
     *q++=(unsigned char) polarity;
@@ -753,7 +752,7 @@ MagickExport unsigned int HuffmanEncodeImage(const ImageInfo *image_info,
     indexes=GetIndexes(huffman_image);
     for (x=0; x < (long) huffman_image->columns; x++)
     {
-      *q=(indexes[x] == polarity ? (int) polarity : (int) !polarity);
+      *q=(unsigned char) (indexes[x] == polarity ? !polarity : polarity);
       q++;
     }
     /*
@@ -811,7 +810,8 @@ MagickExport unsigned int HuffmanEncodeImage(const ImageInfo *image_info,
     q=scanline;
     if (huffman_image->previous == (Image *) NULL)
       if (QuantumTick(y,huffman_image->rows))
-        MagickMonitor(SaveImageText,y,huffman_image->rows);
+        if (!MagickMonitor(SaveImageTag,y,huffman_image->rows,&image->exception))
+          break;
   }
   /*
     End of page.
@@ -830,7 +830,7 @@ MagickExport unsigned int HuffmanEncodeImage(const ImageInfo *image_info,
       if (LocaleCompare(image_info->magick,"FAX") == 0)
         (void) WriteBlobByte(image,byte);
       else
-        Ascii85Encode(image,(unsigned int) byte);
+        Ascii85Encode(image,(unsigned long) byte);
     }
   if (LocaleCompare(image_info->magick,"FAX") != 0)
     Ascii85Flush(image);
@@ -885,7 +885,7 @@ MagickExport unsigned int LZWEncodeImage(Image *image,const size_t length,
     number_bits+=code_width; \
     while (number_bits >= 8) \
     { \
-        (void) WriteBlobByte(image,(unsigned int) (accumulator >> 24)); \
+        (void) WriteBlobByte(image,(unsigned long) (accumulator >> 24)); \
         accumulator=accumulator << 8; \
         number_bits-=8; \
     } \
@@ -1011,8 +1011,7 @@ MagickExport unsigned int LZWEncodeImage(Image *image,const size_t length,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  ThrowBinaryException(MissingDelegateError,"LZW library is not available",
-    (char *) NULL)
+  ThrowBinaryException(DelegateError,"LZWEncodingNotEnabled",(char *) NULL)
 }
 #endif
 
@@ -1071,7 +1070,7 @@ MagickExport unsigned int PackbitsEncodeImage(Image *image,const size_t length,
   assert(pixels != (unsigned char *) NULL);
   packbits=(unsigned char *) AcquireMemory(128);
   if (packbits == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       (char *) NULL);
   i=(long) length;
   while (i != 0)
@@ -1123,7 +1122,7 @@ MagickExport unsigned int PackbitsEncodeImage(Image *image,const size_t length,
                 break;
             }
             i-=count;
-            (void) WriteBlobByte(image,(int) ((256-count)+1));
+            (void) WriteBlobByte(image,(long) ((256-count)+1));
             (void) WriteBlobByte(image,*pixels);
             pixels+=count;
             break;
