@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,6 +54,7 @@
   Include declarations.
 */
 #include "studio.h"
+#include "monitor.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -112,7 +113,7 @@ MagickExport Image *BorderImage(const Image *image,
   border_image=FrameImage(clone_image,&frame_info,exception);
   DestroyImage(clone_image);
   if (border_image != (Image *) NULL)
-		border_image->matte_color=image->matte_color;
+    border_image->matte_color=image->matte_color;
   return(border_image);
 }
 
@@ -151,7 +152,7 @@ MagickExport Image *BorderImage(const Image *image,
 MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   ExceptionInfo *exception)
 {
-#define FrameImageText  "  Add frame to image...  "
+#define FrameImageTag  "Frame/Image"
 
   Image
     *frame_image;
@@ -187,14 +188,14 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   assert(image->signature == MagickSignature);
   assert(frame_info != (FrameInfo *) NULL);
   if ((frame_info->outer_bevel < 0) || (frame_info->inner_bevel < 0))
-    ThrowImageException(OptionError,"Unable to frame image",
-      "bevel width is negative");
+    ThrowImageException(OptionError,"UnableToFrameImage",
+      "BevelWidthIsNegative");
   bevel_width=frame_info->outer_bevel+frame_info->inner_bevel;
   width=(long) (frame_info->width-frame_info->x-bevel_width);
   height=(long) (frame_info->height-frame_info->y-bevel_width);
   if ((width < (long) image->columns) || (height < (long) image->rows))
-    ThrowImageException(OptionError,"Unable to frame image",
-      "frame is less than image size");
+    ThrowImageException(OptionError,"UnableToFrameImage",
+      "FrameIsLessThanImageSize");
   /*
     Initialize framed image attributes.
   */
@@ -208,32 +209,30 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     Initialize 3D effects color.
   */
   matte=image->matte_color;
-  accentuate.red=(Quantum) ((unsigned long) (matte.red*(MaxRGB-
-    AccentuateModulate)+MaxRGB*AccentuateModulate)/MaxRGB);
-  accentuate.green=(Quantum) ((unsigned long) (matte.green*(MaxRGB-
-    AccentuateModulate)+MaxRGB*AccentuateModulate)/MaxRGB);
-  accentuate.blue=(Quantum) ((unsigned long) (matte.blue*(MaxRGB-
-    AccentuateModulate)+MaxRGB*AccentuateModulate)/MaxRGB);
-  accentuate.opacity=(Quantum) ((unsigned long) (matte.opacity*(MaxRGB-
-    AccentuateModulate)+MaxRGB*AccentuateModulate)/MaxRGB);
-  highlight.red=(Quantum) ((unsigned long) (matte.red*(MaxRGB-
-    HighlightModulate)+MaxRGB*HighlightModulate)/MaxRGB);
-  highlight.green=(Quantum) ((unsigned long) (matte.green*(MaxRGB-
-    HighlightModulate)+MaxRGB*HighlightModulate)/MaxRGB);
-  highlight.blue=(Quantum) ((unsigned long) (matte.blue*(MaxRGB-
-    HighlightModulate)+MaxRGB*HighlightModulate)/MaxRGB);
-  highlight.opacity=(Quantum) ((unsigned long) (matte.opacity*(MaxRGB-
-    HighlightModulate)+MaxRGB*HighlightModulate)/MaxRGB);
-  shadow.red=(Quantum) ((unsigned long) (matte.red*ShadowModulate)/MaxRGB);
-  shadow.green=(Quantum) ((unsigned long) (matte.green*ShadowModulate)/MaxRGB);
-  shadow.blue=(Quantum) ((unsigned long) (matte.blue*ShadowModulate)/MaxRGB);
-  shadow.opacity=(Quantum)
-    ((unsigned long) (matte.opacity*ShadowModulate)/MaxRGB);
-  trough.red=(Quantum) ((unsigned long) (matte.red*TroughModulate)/MaxRGB);
-  trough.green=(Quantum) ((unsigned long) (matte.green*TroughModulate)/MaxRGB);
-  trough.blue=(Quantum) ((unsigned long) (matte.blue*TroughModulate)/MaxRGB);
-  trough.opacity=(Quantum)
-    ((unsigned long) (matte.opacity*TroughModulate)/MaxRGB);
+  accentuate.red=(Quantum) ((((double) MaxRGB-AccentuateModulate)*matte.red+
+    ((double) MaxRGB*AccentuateModulate))/MaxRGB+0.5);
+  accentuate.green=(Quantum) ((((double) MaxRGB-AccentuateModulate)*matte.green+
+    ((double) MaxRGB*AccentuateModulate))/MaxRGB+0.5);
+  accentuate.blue=(Quantum) ((((double) MaxRGB-AccentuateModulate)*matte.blue+
+    ((double) MaxRGB*AccentuateModulate))/MaxRGB+0.5);
+  accentuate.opacity=(Quantum) ((((double) MaxRGB-AccentuateModulate)*
+    matte.opacity+((double) MaxRGB*AccentuateModulate))/MaxRGB+0.5);
+  highlight.red=(Quantum) ((((double) MaxRGB-HighlightModulate)*matte.red+
+    ((double) MaxRGB*HighlightModulate))/MaxRGB+0.5);
+  highlight.green=(Quantum) ((((double) MaxRGB-HighlightModulate)*matte.green+
+    ((double) MaxRGB*HighlightModulate))/MaxRGB+0.5);
+  highlight.blue=(Quantum) ((((double) MaxRGB-HighlightModulate)*matte.blue+
+    ((double) MaxRGB*HighlightModulate))/MaxRGB+0.5);
+  highlight.opacity=(Quantum) ((((double) MaxRGB-HighlightModulate)*
+    matte.opacity+((double) MaxRGB*HighlightModulate))/MaxRGB+0.5);
+  shadow.red=(Quantum) (((double) matte.red*ShadowModulate)/MaxRGB+0.5);
+  shadow.green=(Quantum) (((double) matte.green*ShadowModulate)/MaxRGB+0.5);
+  shadow.blue=(Quantum) (((double) matte.blue*ShadowModulate)/MaxRGB+0.5);
+  shadow.opacity=(Quantum) (((double) matte.opacity*ShadowModulate)/MaxRGB+0.5);
+  trough.red=(Quantum) (((double) matte.red*TroughModulate)/MaxRGB+0.5);
+  trough.green=(Quantum) (((double) matte.green*TroughModulate)/MaxRGB+0.5);
+  trough.blue=(Quantum) (((double) matte.blue*TroughModulate)/MaxRGB+0.5);
+  trough.opacity=(Quantum) (((double) matte.opacity*TroughModulate)/MaxRGB+0.5);
   /*
     Draw top of ornamental border.
   */
@@ -256,7 +255,8 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
       {
         for (x=0; x < frame_info->outer_bevel; x++)
           *q++=highlight;
-        for (x=0; x < (long) (frame_image->columns-2*frame_info->outer_bevel); x++)
+        width=(long) (frame_image->columns-2*frame_info->outer_bevel);
+        for (x=0; x < (long) width; x++)
           *q++=matte;
         for (x=0; x < frame_info->outer_bevel; x++)
           *q++=shadow;
@@ -317,7 +317,8 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     if (!SyncImagePixels(frame_image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(FrameImageText,y,image->rows);
+      if (!MagickMonitor(FrameImageTag,y,image->rows,exception))
+        break;
   }
   /*
     Draw bottom of ornamental border.
@@ -406,18 +407,18 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
 MagickExport unsigned int RaiseImage(Image *image,
   const RectangleInfo *raise_info,const int raise)
 {
-#define AccentuateFactor  Upscale(135)
-#define HighlightFactor  Upscale(190)
-#define ShadowFactor  Upscale(190)
-#define RaiseImageText  "  Raise image...  "
-#define TroughFactor  Upscale(135)
+#define AccentuateFactor  ScaleCharToQuantum(135)
+#define HighlightFactor  ScaleCharToQuantum(190)
+#define ShadowFactor  ScaleCharToQuantum(190)
+#define RaiseImageTag  "Raise/Image"
+#define TroughFactor  ScaleCharToQuantum(135)
 
   Quantum
     foreground,
     background;
 
   long
-  	y;
+    y;
 
   register long
     x;
@@ -430,8 +431,8 @@ MagickExport unsigned int RaiseImage(Image *image,
   assert(raise_info != (RectangleInfo *) NULL);
   if ((image->columns <= (raise_info->width << 1)) ||
       (image->rows <= (raise_info->height << 1)))
-    ThrowBinaryException(OptionError,"Unable to raise image",
-      "image size must exceed bevel width");
+    ThrowBinaryException(OptionError,"UnableToRaiseImage",
+      "ImageSizeMustExceedBevelWidth");
   foreground=MaxRGB;
   background=0;
   if (!raise)
@@ -447,38 +448,39 @@ MagickExport unsigned int RaiseImage(Image *image,
       break;
     for (x=0; x < y; x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB);
+      q->green=(Quantum) (((double) q->green*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB);
+      q->blue=(Quantum) (((double) q->blue*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB);
       q++;
     }
     for ( ; x < (long) (image->columns-y); x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*AccentuateFactor+foreground*
-        (MaxRGB-AccentuateFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green* AccentuateFactor+
-        foreground*(MaxRGB-AccentuateFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*AccentuateFactor+foreground*
-        (MaxRGB-AccentuateFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*AccentuateFactor+
+        (double) foreground*(MaxRGB-AccentuateFactor))/MaxRGB);
+      q->green=(Quantum) (((double) q->green*AccentuateFactor+
+        (double) foreground*(MaxRGB-AccentuateFactor))/MaxRGB);
+      q->blue=(Quantum) (((double) q->blue*AccentuateFactor+
+        (double) foreground*(MaxRGB-AccentuateFactor))/MaxRGB);
       q++;
     }
     for ( ; x < (long) image->columns; x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB);
+      q->green=(Quantum) (((double) q->green*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB);
+      q->blue=(Quantum) (((double) q->blue*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB);
       q++;
     }
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(RaiseImageText,y,image->rows);
+      if (!MagickMonitor(RaiseImageTag,y,image->rows,&image->exception))
+        break;
   }
   for ( ; y < (long) (image->rows-raise_info->height); y++)
   {
@@ -487,30 +489,31 @@ MagickExport unsigned int RaiseImage(Image *image,
       break;
     for (x=0; x < (long) raise_info->width; x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB);
+      q->green=(Quantum) (((double) q->green*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB);
+      q->blue=(Quantum) (((double) q->blue*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB);
       q++;
     }
     for ( ; x < (long) (image->columns-raise_info->width); x++)
       q++;
     for ( ; x < (long) image->columns; x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB);
+      q->green=(Quantum) (((double) q->green*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB);
+      q->blue=(Quantum) (((double) q->blue*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB);
       q++;
     }
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(RaiseImageText,y,image->rows);
+      if (!MagickMonitor(RaiseImageTag,y,image->rows,&image->exception))
+        break;
   }
   for ( ; y < (long) image->rows; y++)
   {
@@ -519,38 +522,39 @@ MagickExport unsigned int RaiseImage(Image *image,
       break;
     for (x=0; x < (long) (image->rows-y); x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*HighlightFactor+foreground*
-        (MaxRGB-HighlightFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB+0.5);
+      q->green=(Quantum) (((double) q->green*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB+0.5);
+      q->blue=(Quantum) (((double) q->blue*HighlightFactor+
+        (double) foreground*(MaxRGB-HighlightFactor))/MaxRGB+0.5);
       q++;
     }
     for ( ; x < (long) (image->columns-(image->rows-y)); x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*TroughFactor+background*
-        (MaxRGB-TroughFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green*TroughFactor+background*
-        (MaxRGB-TroughFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*TroughFactor+background*
-        (MaxRGB-TroughFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*TroughFactor+
+        (double) background*(MaxRGB-TroughFactor))/MaxRGB+0.5);
+      q->green=(Quantum) (((double) q->green*TroughFactor+
+        (double) background*(MaxRGB-TroughFactor))/MaxRGB+0.5);
+      q->blue=(Quantum) (((double) q->blue*TroughFactor+
+        (double) background*(MaxRGB-TroughFactor))/MaxRGB+0.5);
       q++;
     }
     for ( ; x < (long) image->columns; x++)
     {
-      q->red=(Quantum) ((unsigned long) (q->red*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
-      q->green=(Quantum) ((unsigned long) (q->green*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
-      q->blue=(Quantum) ((unsigned long) (q->blue*ShadowFactor+background*
-        (MaxRGB-ShadowFactor))/MaxRGB);
+      q->red=(Quantum) (((double) q->red*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB+0.5);
+      q->green=(Quantum) (((double) q->green*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB+0.5);
+      q->blue=(Quantum) (((double) q->blue*ShadowFactor+
+        (double) background*(MaxRGB-ShadowFactor))/MaxRGB+0.5);
       q++;
     }
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(RaiseImageText,y,image->rows);
+      if (!MagickMonitor(RaiseImageTag,y,image->rows,&image->exception))
+        break;
   }
   return(True);
 }
