@@ -17,7 +17,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -59,6 +59,9 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "delegate.h"
+#include "utility.h"
 
 /*
   Define declarations.
@@ -69,7 +72,7 @@
   Declare delegate map.
 */
 static char
-  *DelegateMap =
+  *DelegateMap = (char *)
     "<?xml version=\"1.0\"?>"
     "<delegatemap>"
     "  <delegate stealth=\"True\" />"
@@ -78,17 +81,17 @@ static char
 /*
   Global declaractions.
 */
-static DelegateInfo
-  *delegate_list = (DelegateInfo *) NULL;
-
 static SemaphoreInfo
   *delegate_semaphore = (SemaphoreInfo *) NULL;
+
+static DelegateInfo
+  *delegate_list = (DelegateInfo *) NULL;
 
 /*
   Forward declaractions.
 */
 static unsigned int
-  ReadConfigurationFile(const char *,ExceptionInfo *);
+  ReadConfigureFile(const char *,const unsigned long,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -122,8 +125,8 @@ MagickExport void DestroyDelegateInfo(void)
   {
     delegate_info=p;
     p=p->next;
-    if (delegate_info->filename != (char *) NULL)
-      LiberateMemory((void **) &delegate_info->filename);
+    if (delegate_info->path != (char *) NULL)
+      LiberateMemory((void **) &delegate_info->path);
     if (delegate_info->decode != (char *) NULL)
       LiberateMemory((void **) &delegate_info->decode);
     if (delegate_info->encode != (char *) NULL)
@@ -194,20 +197,20 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
   delegate_info=GetDelegateInfo(decode,encode,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     {
-      ThrowException(exception,MissingDelegateError,"no tag found",
+      ThrowException(exception,DelegateError,"NoTagFound",
         decode ? decode : encode);
       return((char *) NULL);
     }
   commands=StringToList(delegate_info->commands);
   if (commands == (char **) NULL)
     {
-      ThrowException(exception,ResourceLimitError,"Memory allocation failed",
+      ThrowException(exception,ResourceLimitError,"MemoryAllocationFailed",
         decode ? decode : encode);
       return((char *) NULL);
     }
   command=TranslateText(image_info,image,commands[0]);
   if (command == (char *) NULL)
-    ThrowException(exception,ResourceLimitError,"Memory allocation failed",
+    ThrowException(exception,ResourceLimitError,"MemoryAllocationFailed",
       commands[0]);
   /*
     Free resources.
@@ -258,12 +261,15 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
   register DelegateInfo
     *p;
 
-  AcquireSemaphoreInfo(&delegate_semaphore);
   if (delegate_list == (DelegateInfo *) NULL)
-    (void) ReadConfigurationFile(DelegateFilename,exception);
-  LiberateSemaphoreInfo(&delegate_semaphore);
+    {
+      AcquireSemaphoreInfo(&delegate_semaphore);
+      if (delegate_list == (DelegateInfo *) NULL)
+        (void) ReadConfigureFile(DelegateFilename,0,exception);
+      LiberateSemaphoreInfo(&delegate_semaphore);
+    }
   if ((LocaleCompare(decode,"*") == 0) && (LocaleCompare(encode,"*") == 0))
-    return(delegate_list);
+    return((const DelegateInfo *) delegate_list);
   /*
     Search for requested delegate.
   */
@@ -308,7 +314,7 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
         delegate_list=p;
       }
   LiberateSemaphoreInfo(&delegate_semaphore);
-  return(p);
+  return((const DelegateInfo *) p);
 }
 
 /*
@@ -369,7 +375,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   delegate_info=GetDelegateInfo(decode,encode,exception);
   if (delegate_info == (DelegateInfo *) NULL)
     {
-      ThrowException(exception,MissingDelegateError,"no tag found",
+      ThrowException(exception,DelegateError,"NoTagFound",
         decode ? decode : encode);
       return(False);
     }
@@ -396,7 +402,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
           delegate_info->encode : delegate_info->decode);
         if (magick == (char *) NULL)
           {
-            ThrowException(exception,DelegateError,"delegate failed",
+            ThrowException(exception,DelegateError,"DelegateFailed",
               decode ? decode : encode);
             return(False);
           }
@@ -418,7 +424,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
           if (status == False)
             {
               DestroyImageInfo(clone_info);
-              ThrowException(exception,DelegateError,"delegate failed",
+              ThrowException(exception,DelegateError,"DelegateFailed",
                 decode ? decode : encode);
               return(False);
             }
@@ -434,7 +440,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   commands=StringToList(delegate_info->commands);
   if (commands == (char **) NULL)
     {
-      ThrowException(exception,ResourceLimitError,"Memory allocation failed",
+      ThrowException(exception,ResourceLimitError,"MemoryAllocationFailed",
         decode ? decode : encode);
       return(False);
     }
@@ -456,7 +462,7 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
     (void) remove(image_info->zero);
     (void) remove(image_info->unique);
     if (status != False)
-      ThrowException(exception,DelegateError,"delegate failed",commands[i]);
+      ThrowException(exception,DelegateError,"DelegateFailed",commands[i]);
     LiberateMemory((void **) &commands[i]);
   }
   for ( ; commands[i] != (char *) NULL; i++)
@@ -470,6 +476,107 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I n v o k e P o s t s r i p t D e l e g a t e                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  InvokePostscriptDelegate() executes the postscript interpreter with the
+%  specified command.
+%
+%  The format of the InvokePostscriptDelegate method is:
+%
+%      unsigned int InvokePostscriptDelegate(const unsigned int verbose,
+%        const char *command)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method InvokePostscriptDelegate returns True is the command
+%      is successfully executed, otherwise False.
+%
+%    o verbose: A value other than zero displays the command prior to
+%      executing it.
+%
+%    o command: The address of a character string containing the command to
+%      execute.
+%
+%
+*/
+MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
+  const char *command)
+{
+#if defined(HasGS) || defined(WIN32)
+  char
+    **argv;
+
+  gs_main_instance
+    *interpreter;
+
+  int
+    argc,
+    code,
+    status;
+
+  register long
+    i;
+
+#if defined(WIN32)
+  const GhostscriptVectors
+    *gs_func;
+
+  gs_func=NTGhostscriptDLLVectors();
+#elif defined(HasGS)
+  GhostscriptVectors
+    gs_func_struct;
+
+  const GhostscriptVectors
+    *gs_func;
+
+  gs_func=(&gs_func_struct);
+  gs_func_struct.exit=gsapi_exit;
+  gs_func_struct.init_with_args=gsapi_init_with_args;
+  gs_func_struct.new_instance=gsapi_new_instance;
+  gs_func_struct.run_string=gsapi_run_string;
+  gs_func_struct.delete_instance=gsapi_delete_instance;
+#endif
+  if (gs_func == (GhostscriptVectors *) NULL)
+    {
+      if (verbose)
+        (void) fputs(command,stdout);
+      return(SystemCommand(verbose,command));
+    }
+  if (verbose)
+    {
+      (void) fputs("gsdll32",stdout);
+      (void) fputs(strchr(command,' '),stdout);
+    }
+  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+  if (status < 0)
+    return(False);
+  argv=StringToArgv(command,&argc);
+  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+  if (status == 0)
+    status=(gs_func->run_string)
+     (interpreter,"systemdict /start get exec\n",0,&code);
+  (gs_func->exit)(interpreter);
+  (gs_func->delete_instance)(interpreter);
+  for (i=0; i < argc; i++)
+    LiberateMemory((void **) &argv[i]);
+  LiberateMemory((void **) &argv);
+  if ((status == 0) || (status == -101))
+    return(False);
+  return(True);
+#else
+  return(SystemCommand(verbose,command));
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  L i s t D e l e g a t e I n f o                                            %
 %                                                                             %
 %                                                                             %
@@ -496,12 +603,12 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
     **commands,
     delegate[MaxTextExtent];
 
-  register const DelegateInfo
-    *p;
-
   register long
     i;
 
+  register const DelegateInfo
+    *p;
+
   if (file == (const FILE *) NULL)
     file=stdout;
   (void) GetDelegateInfo("*","*",exception);
@@ -509,11 +616,12 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
   for (p=delegate_list; p != (const DelegateInfo *) NULL; p=p->next)
   {
     if ((p->previous == (DelegateInfo *) NULL) ||
-        (LocaleCompare(p->filename,p->previous->filename) != 0))
+        (LocaleCompare(p->path,p->previous->path) != 0))
       {
         if (p->previous != (DelegateInfo *) NULL)
           (void) fprintf(file,"\n");
-        (void) fprintf(file,"Filename: %.1024s\n\n",p->filename);
+        if (p->path != (char *) NULL)
+          (void) fprintf(file,"Path: %.1024s\n\n",p->path);
         (void) fprintf(file,"Delegate             Command\n");
         (void) fprintf(file,"-------------------------------------------------"
           "------------------------------\n");
@@ -544,38 +652,39 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   R e a d C o n f i g u r a t i o n F i l e                                 %
++   R e a d C o n f i g u r e F i l e                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadConfigurationFile reads the color configuration file which maps
+%  Method ReadConfigureFile reads the color configuration file which maps
 %  color strings with a particular image format.
 %
-%  The format of the ReadConfigurationFile method is:
+%  The format of the ReadConfigureFile method is:
 %
-%      unsigned int ReadConfigurationFile(const char *basename,
-%        ExceptionInfo *exception)
+%      unsigned int ReadConfigureFile(const char *basename,
+%        const unsigned long depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method ReadConfigurationFile returns True if at least one color
+%    o status: Method ReadConfigureFile returns True if at least one color
 %      is defined otherwise False.
 %
 %    o basename:  The color configuration filename.
 %
+%    o depth: depth of <include /> statements.
+%
 %    o exception: Return any errors or warnings in this structure.
 %
 %
 */
-static unsigned int ReadConfigurationFile(const char *basename,
-  ExceptionInfo *exception)
+static unsigned int ReadConfigureFile(const char *basename,
+  const unsigned long depth,ExceptionInfo *exception)
 {
   char
-    filename[MaxTextExtent],
     keyword[MaxTextExtent],
-    *path,
+    path[MaxTextExtent],
     *q,
     *token,
     *xml;
@@ -584,16 +693,13 @@ static unsigned int ReadConfigurationFile(const char *basename,
     length;
 
   /*
-    Read the delegates configuration file.
+    Read the delegates configure file.
   */
-  FormatString(filename,"%.1024s",basename);
-  path=GetMagickConfigurePath(basename,exception);
-  if (path != (char *) NULL)
-    {
-      FormatString(filename,"%.1024s",path);
-      LiberateMemory((void **) &path);
-    }
-  xml=(char *) FileToBlob(filename,&length,exception);
+  (void) strcpy(path,basename);
+  if (depth == 0)
+    xml=(char *) GetConfigureBlob(basename,path,&length,exception);
+  else
+    xml=(char *) FileToBlob(basename,&length,exception);
   if (xml == (char *) NULL)
     xml=AllocateString(DelegateMap);
   token=AllocateString(xml);
@@ -605,33 +711,48 @@ static unsigned int ReadConfigurationFile(const char *basename,
     GetToken(q,&q,token);
     if (*token == '\0')
       break;
-    FormatString(keyword,"%.1024s",token);
-    if (LocaleCompare(keyword,"<!") == 0)
+    (void) strncpy(keyword,token,MaxTextExtent-1);
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
-          Comment.
+          Comment element.
         */
-        while ((*token != '>') && (*q != '\0'))
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
           GetToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
       {
         /*
-          Include.
+          Include element.
         */
         while ((*token != '>') && (*q != '\0'))
         {
-          strncpy(keyword,token,MaxTextExtent-1);
+          (void) strncpy(keyword,token,MaxTextExtent-1);
           GetToken(q,&q,token);
           if (*token != '=')
             continue;
           GetToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              (void) ReadConfigurationFile(token,exception);
-              while (delegate_list->next != (DelegateInfo *) NULL)
-                delegate_list=delegate_list->next;
+              if (depth > 200)
+                ThrowException(exception,ConfigureError,
+                  "IncludeElementNestedTooDeeply",path);
+              else
+                {
+                  char
+                    filename[MaxTextExtent];
+
+                  GetPathComponent(path,HeadPath,filename);
+                  if (*filename != '\0')
+                    (void) strcat(filename,DirectorySeparator);
+                  (void) strncat(filename,token,MaxTextExtent-
+                    strlen(filename)-1);
+                  (void) ReadConfigureFile(filename,depth+1,exception);
+                }
+              if (delegate_list != (DelegateInfo *) NULL)
+                while (delegate_list->next != (DelegateInfo *) NULL)
+                  delegate_list=delegate_list->next;
             }
         }
         continue;
@@ -646,10 +767,10 @@ static unsigned int ReadConfigurationFile(const char *basename,
         */
         delegate_info=(DelegateInfo *) AcquireMemory(sizeof(DelegateInfo));
         if (delegate_info == (DelegateInfo *) NULL)
-          MagickFatalError(ResourceLimitFatalError,
-            "Unable to allocate delegate","Memory allocation failed");
+          MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+            "UnableToAllocateDelegateInfo");
         (void) memset(delegate_info,0,sizeof(DelegateInfo));
-        delegate_info->filename=AcquireString(filename);
+        delegate_info->path=AcquireString(path);
         delegate_info->signature=MagickSignature;
         if (delegate_list == (DelegateInfo *) NULL)
           {
@@ -675,7 +796,18 @@ static unsigned int ReadConfigurationFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"command") == 0)
           {
-            delegate_list->commands=AcquireString(token);
+            delegate_list->commands=AllocateString(token);
+#if defined(WIN32)
+            if (strchr(delegate_list->commands,'@') != (char *) NULL)
+              {
+                char
+                  path[MaxTextExtent];
+
+                NTGhostscriptEXE(path,MaxTextExtent-1);
+                SubstituteString((char **) &delegate_list->commands,
+                  "@PSDelegate@",path);
+              }
+#endif
             break;
           }
         break;
@@ -776,12 +908,12 @@ static unsigned int ReadConfigurationFile(const char *basename,
 */
 MagickExport DelegateInfo *SetDelegateInfo(DelegateInfo *delegate_info)
 {
-  DelegateInfo
-    *delegate;
-
   register DelegateInfo
     *p;
 
+  DelegateInfo
+    *delegate;
+
   /*
     Initialize new delegate.
   */
@@ -789,19 +921,19 @@ MagickExport DelegateInfo *SetDelegateInfo(DelegateInfo *delegate_info)
   assert(delegate_info->signature == MagickSignature);
   delegate=(DelegateInfo *) AcquireMemory(sizeof(DelegateInfo));
   if (delegate == (DelegateInfo *) NULL)
-    return(delegate_list);
+    return((DelegateInfo *) delegate_list);
   delegate->decode=AcquireString(delegate_info->decode);
   delegate->encode=AcquireString(delegate_info->encode);
   delegate->mode=delegate_info->mode;
   delegate->commands=(char *) NULL;
   if (delegate_info->commands != (char *) NULL)
-    delegate->commands=AcquireString(delegate_info->commands);
+    delegate->commands=AllocateString(delegate_info->commands);
   delegate->previous=(DelegateInfo *) NULL;
   delegate->next=(DelegateInfo *) NULL;
   if (delegate_list == (DelegateInfo *) NULL)
     {
       delegate_list=delegate;
-      return(delegate_list);
+      return((DelegateInfo *) delegate_list);
     }
   for (p=delegate_list; p != (DelegateInfo *) NULL; p=p->next)
   {
@@ -815,7 +947,7 @@ MagickExport DelegateInfo *SetDelegateInfo(DelegateInfo *delegate_info)
         LiberateMemory((void **) &p->commands);
         p->commands=delegate->commands;
         LiberateMemory((void **) &delegate);
-        return(delegate_list);
+        return((DelegateInfo *) delegate_list);
       }
     if (p->next == (DelegateInfo *) NULL)
       break;
@@ -825,5 +957,5 @@ MagickExport DelegateInfo *SetDelegateInfo(DelegateInfo *delegate_info)
   */
   delegate->previous=p;
   p->next=delegate;
-  return(delegate_list);
+  return((DelegateInfo *) delegate_list);
 }
