@@ -18,7 +18,7 @@
 %                                July 1993                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,14 +54,23 @@
   Include declarations.
 */
 #include "studio.h"
+#include "utility.h"
 
 /*
   Forward declarations.
 */
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
 static void
   DefaultErrorHandler(const ExceptionType,const char *,const char *),
   DefaultFatalErrorHandler(const ExceptionType,const char *,const char *),
   DefaultWarningHandler(const ExceptionType,const char *,const char *);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
 
 /*
   Global declarations.
@@ -80,6 +89,58 @@ static WarningHandler
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C a t c h E x c e p t i o n                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  CatchException() returns if no exceptions is found otherwise it reports
+%  the exception as a warning, error, or fatal depending on the severity.
+%
+%  The format of the CatchException method is:
+%
+%      CatchException(const ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o exception: The exception info.
+%
+%
+*/
+MagickExport void CatchException(const ExceptionInfo *exception)
+{
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  if (exception->severity == UndefinedException)
+    return;
+  errno=exception->error_number;
+  if ((exception->severity >= WarningException) &&
+      (exception->severity < ErrorException))
+    {
+      MagickWarning(exception->severity,exception->reason,
+        exception->description);
+      return;
+    }
+  if ((exception->severity >= ErrorException) &&
+      (exception->severity < FatalErrorException))
+    {
+      MagickError(exception->severity,exception->reason,exception->description);
+      return;
+    }
+  if (exception->severity >= FatalErrorException)
+    {
+      MagickFatalError(exception->severity,exception->reason,
+        exception->description);
+      return;
+    }
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   D e f a u l t E r r o r H a n d l e r                                     %
 %                                                                             %
 %                                                                             %
@@ -90,12 +151,12 @@ static WarningHandler
 %
 %  The format of the DefaultErrorHandler method is:
 %
-%      void MagickError(const ExceptionType error,const char *reason,
+%      void MagickError(const ExceptionType severity,const char *reason,
 %        const char *description)
 %
 %  A description of each parameter follows:
 %
-%    o exception: Specifies the numeric error category.
+%    o severity: Specifies the numeric error category.
 %
 %    o reason: Specifies the reason to display before terminating the
 %      program.
@@ -116,19 +177,17 @@ static const char *GetErrorMessageString(const int error_number)
   return(message);
 }
 
-static void DefaultErrorHandler(const ExceptionType error,const char *reason,
+static void DefaultErrorHandler(const ExceptionType severity,const char *reason,
   const char *description)
 {
   if (reason == (char *) NULL)
-    {
-      DestroyMagick();
-      Exit(error);
-    }
+    return;
   (void) fprintf(stderr,"%.1024s: %.1024s",SetClientName((char *) NULL),
-    reason);
+    GetLocaleExceptionMessage(severity,reason));
   if (description != (char *) NULL)
-    (void) fprintf(stderr," (%.1024s)",description);
-  if ((error != OptionError) && errno)
+    (void) fprintf(stderr," (%.1024s)",
+      GetLocaleExceptionMessage(severity,description));
+  if ((severity != OptionError) && errno)
     (void) fprintf(stderr," [%.1024s]",GetErrorMessageString(errno));
   (void) fprintf(stderr,".\n");
 }
@@ -149,12 +208,12 @@ static void DefaultErrorHandler(const ExceptionType error,const char *reason,
 %
 %  The format of the DefaultFatalErrorHandler method is:
 %
-%      void MagickFatalError(const ExceptionType error,const char *reason,
+%      void MagickFatalError(const ExceptionType severity,const char *reason,
 %        const char *description)
 %
 %  A description of each parameter follows:
 %
-%    o exception: Specifies the numeric error category.
+%    o severity: Specifies the numeric error category.
 %
 %    o reason: Specifies the reason to display before terminating the
 %      program.
@@ -163,20 +222,21 @@ static void DefaultErrorHandler(const ExceptionType error,const char *reason,
 %
 %
 */
-static void DefaultFatalErrorHandler(const ExceptionType error,
+static void DefaultFatalErrorHandler(const ExceptionType severity,
   const char *reason,const char *description)
 {
   if (reason == (char *) NULL)
     return;
   (void) fprintf(stderr,"%.1024s: %.1024s",SetClientName((char *) NULL),
-    reason);
+    GetLocaleExceptionMessage(severity,reason));
   if (description != (char *) NULL)
-    (void) fprintf(stderr," (%.1024s)",description);
-  if ((error != OptionError) && errno)
+    (void) fprintf(stderr," (%.1024s)",
+      GetLocaleExceptionMessage(severity,description));
+  if ((severity != OptionError) && errno)
     (void) fprintf(stderr," [%.1024s]",GetErrorMessageString(errno));
   (void) fprintf(stderr,".\n");
   DestroyMagick();
-  Exit(error);
+  Exit(severity);
 }
 
 /*
@@ -208,16 +268,17 @@ static void DefaultFatalErrorHandler(const ExceptionType error,
 %
 %
 */
-static void DefaultWarningHandler(const ExceptionType warning,
+static void DefaultWarningHandler(const ExceptionType severity,
   const char *reason,const char *description)
 {
   if (reason == (char *) NULL)
     return;
   (void) fprintf(stderr,"%.1024s: %.1024s",SetClientName((char *) NULL),
-    reason);
+    GetLocaleExceptionMessage(severity,reason));
   if (description != (char *) NULL)
-    (void) fprintf(stderr," (%.1024s)",description);
-  if ((warning != OptionWarning) && errno)
+    (void) fprintf(stderr," (%.1024s)",
+      GetLocaleExceptionMessage(severity,description));
+  if ((severity != OptionWarning) && errno)
     (void) fprintf(stderr," [%.1024s]",GetErrorMessageString(errno));
   (void) fprintf(stderr,".\n");
 }
@@ -266,7 +327,7 @@ MagickExport void DestroyExceptionInfo(ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetExceptionInfo() initializes exception to default values.
+%  GetExceptionInfo() initializes an exception to default values.
 %
 %  The format of the GetExceptionInfo method is:
 %
@@ -291,6 +352,115 @@ MagickExport void GetExceptionInfo(ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t L o c a l e E x c e p t i o n M e s s a g e                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetLocaleExceptionMessage() converts a enumerated exception severity and tag
+%  to a message in the current locale.
+%
+%  The format of the GetLocaleExceptionMessage method is:
+%
+%      const char *GetLocaleExceptionMessage(const ExceptionType severity,
+%        const char *tag)
+%
+%  A description of each parameter follows:
+%
+%    o severity: the severity of the exception.
+%
+%    o tag: the message tag.
+%
+%
+%
+*/
+
+static const char *ExceptionSeverityToTag(const ExceptionType severity)
+{
+  switch (severity)
+  {
+    case ResourceLimitWarning: return("Resource/Limit/Warning/");
+    case TypeWarning: return("Type/Warning/");
+    case OptionWarning: return("Option/Warning/");
+    case DelegateWarning: return("Delegate/Warning/");
+    case MissingDelegateWarning: return("Missing/Delegate/Warning/");
+    case CorruptImageWarning: return("Corrupt/Image/Warning/");
+    case FileOpenWarning: return("File/Open/Warning/");
+    case BlobWarning: return("Blob/Warning/");
+    case StreamWarning: return("Stream/Warning/");
+    case CacheWarning: return("Cache/Warning/");
+    case CoderWarning: return("Coder/Warning/");
+    case ModuleWarning: return("Module/Warning/");
+    case DrawWarning: return("Draw/Warning/");
+    case ImageWarning: return("Image/Warning/");
+    case XServerWarning: return("XServer/Warning/");
+    case MonitorWarning: return("Monitor/Warning/");
+    case RegistryWarning: return("Registry/Warning/");
+    case ConfigureWarning: return("Configure/Warning/");
+    case ResourceLimitError: return("Resource/Limit/Error/");
+    case TypeError: return("Type/Error/");
+    case OptionError: return("Option/Error/");
+    case DelegateError: return("Delegate/Error/");
+    case MissingDelegateError: return("Missing/Delegate/Error/");
+    case CorruptImageError: return("Corrupt/Image/Error/");
+    case FileOpenError: return("File/Open/Error/");
+    case BlobError: return("Blob/Error/");
+    case StreamError: return("Stream/Error/");
+    case CacheError: return("Cache/Error/");
+    case CoderError: return("Coder/Error/");
+    case ModuleError: return("Module/Error/");
+    case DrawError: return("Draw/Error/");
+    case ImageError: return("Image/Error/");
+    case XServerError: return("XServer/Error/");
+    case MonitorError: return("Monitor/Error/");
+    case RegistryError: return("Registry/Error/");
+    case ConfigureError: return("Configure/Error/");
+    case ResourceLimitFatalError: return("Resource/Limit/FatalError/");
+    case TypeFatalError: return("Type/FatalError/");
+    case OptionFatalError: return("Option/FatalError/");
+    case DelegateFatalError: return("Delegate/FatalError/");
+    case MissingDelegateFatalError: return("Missing/Delegate/FatalError/");
+    case CorruptImageFatalError: return("Corrupt/Image/FatalError/");
+    case FileOpenFatalError: return("File/Open/FatalError/");
+    case BlobFatalError: return("Blob/FatalError/");
+    case StreamFatalError: return("Stream/FatalError/");
+    case CacheFatalError: return("Cache/FatalError/");
+    case CoderFatalError: return("Coder/FatalError/");
+    case ModuleFatalError: return("Module/FatalError/");
+    case DrawFatalError: return("Draw/FatalError/");
+    case ImageFatalError: return("Image/FatalError/");
+    case XServerFatalError: return("XServer/FatalError/");
+    case MonitorFatalError: return("Monitor/FatalError/");
+    case RegistryFatalError: return("Registry/FatalError/");
+    case ConfigureFatalError: return("Configure/FatalError/");
+    default: break;
+  }
+  return("");
+}
+
+MagickExport const char *GetLocaleExceptionMessage(const ExceptionType severity,
+  const char *tag)
+{
+  char
+    message[MaxTextExtent];
+
+  const char
+    *locale_message;
+
+  FormatString(message,"Exception/%.1024s%.1024s",
+    ExceptionSeverityToTag(severity),tag);
+  locale_message=GetLocaleMessage(message);
+  if (locale_message == message)
+    return(tag);
+  return(locale_message);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k E r r o r                                                     %
 %                                                                             %
 %                                                                             %
@@ -357,7 +527,6 @@ MagickExport void MagickFatalError(const ExceptionType error,const char *reason,
 {
   if (fatal_error_handler != (ErrorHandler) NULL)
     (*fatal_error_handler)(error,reason,description);
-  errno=0;
 }
 
 /*
@@ -460,7 +629,6 @@ MagickExport void SetExceptionInfo(ExceptionInfo *exception,
 {
   assert(exception != (ExceptionInfo *) NULL);
   exception->severity=severity;
-  errno=0;
 }
 
 /*
@@ -487,13 +655,13 @@ MagickExport void SetExceptionInfo(ExceptionInfo *exception,
 %
 %
 */
-MagickExport FatalErrorHandler SetFatalErrorHandler(ErrorHandler handler)
+MagickExport FatalErrorHandler SetFatalErrorHandler(FatalErrorHandler handler)
 {
   FatalErrorHandler
     previous_handler;
 
-  previous_handler=error_handler;
-  error_handler=handler;
+  previous_handler=fatal_error_handler;
+  fatal_error_handler=handler;
   return(previous_handler);
 }
 
@@ -555,11 +723,11 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 %
 %    o exception: The exception.
 %
-%    o severity: Define the severity of the exception.
+%    o severity: The severity of the exception.
 %
-%    o reason: Define the reason of the exception.
+%    o reason: The reason of the exception.
 %
-%    o description: Describe the exception.
+%    o description: The exception description.
 %
 %
 */
@@ -567,7 +735,21 @@ MagickExport void ThrowException(ExceptionInfo *exception,
   const ExceptionType severity,const char *reason,const char *description)
 {
   assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
   exception->severity=(ExceptionType) severity;
-  (void) CloneString(&exception->reason,reason);
+  exception->error_number=errno;
+  if (reason == (char *) NULL)
+    {
+      if (exception->reason != (char *) NULL)
+        LiberateMemory((void **) &exception->reason);
+    }
+  else
+    (void) CloneString(&exception->reason,reason);
+  if (description == (char *) NULL)
+    {
+      if (exception->description != (char *) NULL)
+        LiberateMemory((void **) &exception->description);
+      return;
+    }
   (void) CloneString(&exception->description,description);
 }
