@@ -54,6 +54,8 @@
   Include declarations.
 */
 #include "studio.h"
+#include "gem.h"
+#include "signature.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -138,12 +140,8 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
 */
 MagickExport double ExpandAffine(const AffineMatrix *affine)
 {
-  double
-    expand;
-
   assert(affine != (const AffineMatrix *) NULL);
-  expand=fabs(affine->sx*affine->sy)-fabs(affine->rx*affine->ry);
-  return(sqrt(fabs(expand)));
+  return(sqrt(fabs(affine->sx*affine->sy-affine->rx*affine->ry)));
 }
 
 /*
@@ -190,7 +188,7 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
     sigma,
     value;
 
-  alpha=(double) rand()/RAND_MAX;
+  alpha=GetRandomValue();
   if (alpha == 0.0)
     alpha=1.0;
   switch (noise_type)
@@ -206,11 +204,11 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
       double
         tau;
 
-      beta=(double) rand()/RAND_MAX;
+      beta=GetRandomValue();
       sigma=sqrt(-2.0*log(alpha))*cos(2.0*MagickPI*beta);
       tau=sqrt(-2.0*log(alpha))*sin(2.0*MagickPI*beta);
-      value=(double) pixel+
-        (sqrt((double) pixel)*SigmaGaussian*sigma)+(TauGaussian*tau);
+      value=(double) pixel+sqrt((double) pixel)*SigmaGaussian*sigma+
+        TauGaussian*tau;
       break;
     }
     case MultiplicativeGaussianNoise:
@@ -219,9 +217,9 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
         sigma=MaxRGB;
       else
         sigma=sqrt(-2.0*log(alpha));
-      beta=(double) rand()/RAND_MAX;
-      value=(double) pixel+
-        pixel*SigmaMultiplicativeGaussian*sigma*cos(2.0*MagickPI*beta);
+      beta=GetRandomValue();
+      value=(double) pixel+pixel*SigmaMultiplicativeGaussian*sigma*
+        cos(2.0*MagickPI*beta);
       break;
     }
     case ImpulseNoise:
@@ -240,7 +238,7 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
       if (alpha <= 0.5)
         {
           if (alpha <= NoiseEpsilon)
-            value=(double) pixel-MaxRGB;
+            value=(double) pixel-(double) MaxRGB;
           else
             value=(double) pixel+SigmaLaplacian*log(2.0*alpha);
           break;
@@ -259,7 +257,7 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
 
       for (i=0; alpha > exp(-SigmaPoisson*pixel); i++)
       {
-        beta=(double) rand()/RAND_MAX;
+        beta=GetRandomValue();
         alpha=alpha*beta;
       }
       value=i/SigmaPoisson;
@@ -284,9 +282,9 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetOptimalKernelWidth computes the optimal kernel radius for a
-%  convolution filter.  Start with the minimum value of 3 pixels and walk out
-%  until we drop below the threshold of one pixel numerical accuracy,
+%  GetOptimalKernelWidth() computes the optimal kernel radius for a convolution
+%  filter.  Start with the minimum value of 3 pixels and walk out until we drop
+%  below the threshold of one pixel numerical accuracy.
 %
 %  The format of the GetOptimalKernelWidth method is:
 %
@@ -311,21 +309,22 @@ MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
     normalize,
     value;
 
-  int
+  long
     width;
 
-  register int
+  register long
     u;
 
+  assert(sigma != 0.0);
   if (radius > 0.0)
     return((int) (2.0*ceil(radius)+1.0));
   for (width=5; ;)
   {
     normalize=0.0;
     for (u=(-width/2); u <= (width/2); u++)
-      normalize+=exp(-(double) (u*u)/(2.0*sigma*sigma));
+      normalize+=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma);
     u=width/2;
-    value=exp(-(double) (u*u)/(2.0*sigma*sigma))/normalize;
+    value=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
     if ((long) (MaxRGB*value) <= 0)
       break;
     width+=2;
@@ -336,16 +335,18 @@ MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
 MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
 {
   double
+    alpha,
     normalize,
     value;
 
-  int
+  long
     width;
 
-  register int
+  register long
     u,
     v;
 
+  assert(sigma != 0.0);
   if (radius > 0.0)
     return((int) (2.0*ceil(radius)+1.0));
   for (width=5; ;)
@@ -354,10 +355,13 @@ MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
     for (v=(-width/2); v <= (width/2); v++)
     {
       for (u=(-width/2); u <= (width/2); u++)
-        normalize+=exp(-(double) (u*u+v*v)/(sigma*sigma));
+      {
+        alpha=exp(-((double) u*u+v*v)/(2.0*sigma*sigma));
+        normalize+=alpha/(2.0*MagickPI*sigma*sigma);
+      }
     }
     v=width/2;
-    value=exp(-(double) (v*v)/(sigma*sigma))/normalize;
+    value=exp(-((double) v*v)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
     if ((long) (MaxRGB*value) <= 0)
       break;
     width+=2;
@@ -482,7 +486,7 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
     r,
     v;
 
-  register int
+  register long
     i;
 
   /*
@@ -499,7 +503,7 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
       *blue=(Quantum) (MaxRGB*v+0.5);
       return;
     }
-  i=floor(hue);
+  i=(long) floor(hue);
   f=hue-i;
   if (i & 0x01)
     f=1.0-f;
@@ -560,7 +564,7 @@ MagickExport void Hull(const long x_offset,const long y_offset,
   const unsigned long columns,const unsigned long rows,Quantum *f,Quantum *g,
   const int polarity)
 {
-  long
+  double
     v,
     y;
 
@@ -587,8 +591,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
       for (x=0; x < (long) columns; x++)
       {
         v=(*p);
-        if (*r >= (v+Upscale(2)))
-          v+=Upscale(1);
+        if (*r >= (v+ScaleCharToQuantum(2)))
+          v+=ScaleCharToQuantum(1);
         *q=(Quantum) v;
         p++;
         q++;
@@ -598,8 +602,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
       for (x=0; x < (long) columns; x++)
       {
         v=(*p);
-        if (*r <= (v-(long) Upscale(2)))
-          v-=(long) Upscale(1);
+        if (*r <= (v-(long) ScaleCharToQuantum(2)))
+          v-=(long) ScaleCharToQuantum(1);
         *q=(Quantum) v;
         p++;
         q++;
@@ -623,8 +627,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
       for (x=0; x < (long) columns; x++)
       {
         v=(*q);
-        if ((*s >= (v+Upscale(2))) && (*r > v))
-          v+=Upscale(1);
+        if ((*s >= (v+ScaleCharToQuantum(2))) && (*r > v))
+          v+=ScaleCharToQuantum(1);
         *p=(Quantum) v;
         p++;
         q++;
@@ -635,8 +639,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
       for (x=0; x < (long) columns; x++)
       {
         v=(*q);
-        if ((*s <= (v-(long) Upscale(2))) && (*r < v))
-          v-=(long) Upscale(1);
+        if ((*s <= (v-(long) ScaleCharToQuantum(2))) && (*r < v))
+          v-=(long) ScaleCharToQuantum(1);
         *p=(Quantum) v;
         p++;
         q++;
@@ -718,45 +722,26 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
     beta;
 
   PixelPacket
-    color,
-    p,
-    q,
-    r,
-    s;
+    color;
 
-  register double
-    x,
-    y;
+  register const PixelPacket
+    *p;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  x=x_offset+0.5;
-  y=y_offset+0.5;
-  if ((x < -1.0) || (x >= image->columns) || (y < -1.0) || (y >= image->rows))
-    return(image->background_color);
-  p=image->background_color;
-  if ((x >= 0.0) && (y >= 0.0))
-    p=AcquireOnePixel(image,(int) x,(int) y,exception);
-  q=image->background_color;
-  if (((x+1.0) < image->columns) && (y >= 0.0))
-    q=AcquireOnePixel(image,(int) (x+1.0),(int) y,exception);
-  r=image->background_color;
-  if ((x >= 0.0) && ((y+1.0) < image->rows))
-    r=AcquireOnePixel(image,(int) x,(int) (y+1.0),exception);
-  s=image->background_color;
-  if (((x+1.0) < image->columns) && ((y+1.0) < image->rows))
-    s=AcquireOnePixel(image,(int) (x+1.0),(int) (y+1.0),exception);
-  x-=floor(x);
-  y-=floor(y);
-  alpha=1.0-x;
-  beta=1.0-y;
-  color.red=(Quantum) (beta*(alpha*p.red+x*q.red)+y*(alpha*r.red+x*s.red)+0.5);
-  color.green=(Quantum)
-    (beta*(alpha*p.green+x*q.green)+y*(alpha*r.green+x*s.green)+0.5);
-  color.blue=(Quantum)
-    (beta*(alpha*p.blue+x*q.blue)+y*(alpha*r.blue+x*s.blue)+0.5);
-  color.opacity=(Quantum)
-    (beta*(alpha*p.opacity+x*q.opacity)+y*(alpha*r.opacity+x*s.opacity)+0.5);
+  p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
+  if (p == (const PixelPacket *) NULL)
+    return(AcquireOnePixel(image,(long) x_offset,(long) y_offset,exception));
+  alpha=x_offset-floor(x_offset);
+  beta=y_offset-floor(y_offset);
+  color.red=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].red+
+    alpha*p[1].red)+beta*((1.0-alpha)*p[2].red+alpha*p[3].red)+0.5);
+  color.green=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].green+
+    alpha*p[1].green)+beta*((1.0-alpha)*p[2].green+alpha*p[3].green)+0.5);
+  color.blue=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].blue+
+    alpha*p[1].blue)+beta*((1.0-alpha)*p[2].blue+alpha*p[3].blue)+0.5);
+  color.opacity=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].opacity+
+    alpha*p[1].opacity)+beta*((1.0-alpha)*p[2].opacity+alpha*p[3].opacity)+0.5);
   return(color);
 }
 
@@ -774,7 +759,7 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 %  Method Modulate modulates the hue, saturation, and brightness of an
 %  image.
 %
-%  The format of the ModulateImage method is:
+%  The format of the Modulate method is:
 %
 %      void Modulate(const double percent_hue,const double percent_saturation,
 %        const double percent_brightness,Quantum *red,Quantum *green,
@@ -831,47 +816,6 @@ MagickExport void Modulate(const double percent_hue,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   P e r m u t a t e                                                         %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Method Permutate()
-%
-%  The format of the Permutate method is:
-%
-%      void Permutate(long n,long k)
-%
-%  A description of each parameter follows:
-%
-%    o n:
-%
-%    o k:
-%
-%
-*/
-MagickExport double Permutate(const long n,const long k)
-{
-  double
-    r;
-
-  register long
-    i;
-
-  r=1.0;
-  for (i=k+1; i <= n; i++)
-    r*=i;
-  for (i=1; i <= (n-k); i++)
-    r/=i;
-  return(r);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   T r a n s f o r m H S L                                                   %
 %                                                                             %
 %                                                                             %
@@ -920,7 +864,7 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   min=Min(r,Min(g,b));
   *hue=0.0;
   *saturation=0.0;
-  *luminosity=(0.5+MagickEpsilon)*(min+max);
+  *luminosity=(min+max)/2.0;
   delta=max-min;
   if (delta == 0.0)
     return;
@@ -968,33 +912,35 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
   const Quantum blue,double *hue,double *whiteness,double *blackness)
 {
   double
-    b,
-    f,
-    g,
-    r,
-    v;
+    f;
 
-  register int
+  register long
     i;
 
+  Quantum
+    v,
+    w;
+
   /*
     Convert RGB to HWB colorspace.
   */
   assert(hue != (double *) NULL);
   assert(whiteness != (double *) NULL);
   assert(blackness != (double *) NULL);
-  r=(double) red/MaxRGB;
-  g=(double) green/MaxRGB;
-  b=(double) blue/MaxRGB;
-  *hue=0.0;
-  *whiteness=Min(r,Min(g,b));
-  v=Max(r,Max(g,b));
-  *blackness=1.0-v;
-  if (v == *whiteness)
-    return;
-  f=(r == *whiteness) ? g-b : ((g == *whiteness) ? b-r : r-g);
-  i=(r == *whiteness) ? 3 : ((g == *whiteness) ? 5 : 1);
-  *hue=(double) i-f/(v-*whiteness);
+  w=Min(red,Min(green,blue));
+  v=Max(red,Max(green,blue));
+  *blackness=(double) (MaxRGB-v)/MaxRGB;
+  if (v == w)
+    {
+      *hue=0.0;
+      *whiteness=1.0-(*blackness);
+      return;
+    }
+  f=(red == w) ? (double) green-blue : ((green == w) ? (double) blue-red :
+    (double) red-green);
+  i=(red == w) ? 3 : ((green == w) ? 5 : 1);
+  *hue=i-f/(v-w);
+  *whiteness=(double) w/MaxRGB;
 }
 
 /*
@@ -1023,7 +969,7 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
 %    o scaled_width:  Specifies the final width of the upsampled pixel array.
 %
 %    o pixels:  An unsigned char containing the pixel data.  On output the
-%      upsampled pixels are returned here.
+%      upsampled pixels are returns here.
 %
 %
 */
@@ -1054,7 +1000,7 @@ MagickExport void Upsample(const unsigned long width,const unsigned long height,
       p--;
       q-=2;
       *q=(*p);
-      *(q+1)=(((int) *p)+((int) *(p+1))+1) >> 1;
+      *(q+1)=(unsigned char) ((((long) *p)+((long) *(p+1))+1) >> 1);
     }
   }
   for (y=0; y < (long) (height-1); y++)
@@ -1064,14 +1010,15 @@ MagickExport void Upsample(const unsigned long width,const unsigned long height,
     r=q+scaled_width;
     for (x=0; x < (long) (width-1); x++)
     {
-      *q=(((int) *p)+((int) *r)+1) >> 1;
-      *(q+1)=(((int) *p)+((int) *(p+2))+((int) *r)+((int) *(r+2))+2) >> 2;
+      *q=(unsigned char) ((((long) *p)+((long) *r)+1) >> 1);
+      *(q+1)=(unsigned char)
+        ((((long) *p)+((long) *(p+2))+((long) *r)+((long) *(r+2))+2) >> 2);
       q+=2;
       p+=2;
       r+=2;
     }
-    *q++=(((int) *p++)+((int) *r++)+1) >> 1;
-    *q++=(((int) *p++)+((int) *r++)+1) >> 1;
+    *q++=(unsigned char) ((((long) *p++)+((long) *r++)+1) >> 1);
+    *q++=(unsigned char) ((((long) *p++)+((long) *r++)+1) >> 1);
   }
   p=pixels+(2*height-2)*scaled_width;
   q=pixels+(2*height-1)*scaled_width;
