@@ -8,45 +8,412 @@
 extern "C" {
 #endif
 
+/*
+  Include declarations.
+*/
+#include "semaphore.h"
+#include "error.h"
+#include "timer.h"
+
+/*
+  Define declarations.
+*/
 #if !defined(QuantumDepth)
 #define QuantumDepth  16
 #define QuantumLeap
 #endif
 
-#if QuantumDepth == 8
-/*
-  Color quantum is [0..255].
-*/
-#define Downscale(quantum)  ((Quantum) (quantum))
-#define Upscale(quantum)  ((unsigned int) (quantum))
-#define XDownscale(color)  ((Quantum) ((color)/257))
-#define XUpscale(color)  ((unsigned int) (257*(color)))
+#if (QuantumDepth == 8)
+#define MaxColormapSize  256UL
+#define MaxMap  255UL
+#define MaxRGB  255UL
+#define ScaleCharToQuantum(value)  ((Quantum) (value))
+#define ScaleLongToQuantum(value)  ((Quantum) ((value)/16843009UL))
+#define ScaleMapToQuantum(value)  ((Quantum) (value))
+#define ScaleQuantum(quantum)  ((unsigned long) (quantum))
+#define ScaleQuantumToChar(quantum)  ((unsigned char) (quantum))
+#define ScaleQuantumToLong(quantum)  ((unsigned long) (16843009UL*(quantum)))
+#define ScaleQuantumToMap(quantum)  ((unsigned char) (quantum))
+#define ScaleQuantumToShort(quantum)  ((unsigned short) (257UL*(quantum)))
+#define ScaleShortToQuantum(value)  ((Quantum) ((value)/257UL))
+#define ScaleToQuantum(value)  ((unsigned long) (value))
 
 typedef unsigned char Quantum;
-#elif QuantumDepth == 16
-/*
-  Color quantum is [0..65535].
-*/
-#define Downscale(quantum)  ((Quantum) ((quantum)/257))
-#define Upscale(quantum)  ((unsigned int) (257*(quantum)))
-#define XDownscale(color)  ((Quantum) (color))
-#define XUpscale(color)  ((unsigned int) (color))
+#elif (QuantumDepth == 16)
+#define MaxColormapSize  65536UL
+#define MaxMap 65535UL
+#define MaxRGB  65535UL
+#define ScaleCharToQuantum(value)  ((Quantum) (257UL*(value)))
+#define ScaleLongToQuantum(value)  ((Quantum) ((value)/65537UL))
+#define ScaleMapToQuantum(value)  ((Quantum) (value))
+#define ScaleQuantum(quantum)  ((unsigned long) ((quantum)/257UL))
+#define ScaleQuantumToChar(quantum)  ((unsigned char) ((quantum)/257UL))
+#define ScaleQuantumToLong(quantum)  ((unsigned long) (65537UL*(quantum)))
+#define ScaleQuantumToMap(quantum)  ((unsigned short) (quantum))
+#define ScaleQuantumToShort(quantum)  ((unsigned short) (quantum))
+#define ScaleShortToQuantum(value)  ((Quantum) (value))
+#define ScaleToQuantum(value)  ((unsigned long) (257UL*(value)))
 
 typedef unsigned short Quantum;
+#elif (QuantumDepth == 32)
+#define MaxColormapSize  65536UL
+#define MaxMap 65535UL
+#define MaxRGB  4294967295UL
+#define ScaleCharToQuantum(value)  ((Quantum) (16843009UL*(value)))
+#define ScaleLongToQuantum(value)  ((Quantum) ((value)))
+#define ScaleMapToQuantum(value)  ((Quantum) (65537UL*(value)))
+#define ScaleQuantum(quantum)  ((unsigned long) ((quantum)/16843009UL))
+#define ScaleQuantumToChar(quantum)  ((unsigned char) ((quantum)/16843009UL))
+#define ScaleQuantumToLong(quantum)  ((unsigned long) (quantum))
+#define ScaleQuantumToMap(quantum)  ((unsigned short) ((quantum)/65537UL))
+#define ScaleQuantumToShort(quantum)  ((unsigned short) ((quantum)/65537UL))
+#define ScaleShortToQuantum(value)  ((Quantum) (65537UL*(value)))
+#define ScaleToQuantum(value)  ((unsigned long) (16843009UL*(value)))
+
+typedef unsigned int Quantum;
 #else
 # error "Specified value of QuantumDepth is not supported"
 #endif
 
 #define ColorMatch(p,q) (((p)->red == (q)->red) && \
   ((p)->green == (q)->green) && ((p)->blue == (q)->blue))
-#define Intensity(color)  ((Quantum) \
-  ((9798L*(color)->red+19235L*(color)->green+3735L*(color)->blue)/32768L))
-#define XIntensity(color)  ((unsigned short) \
-  ((9798L*(color)->red+19235L*(color)->green+3735L*(color)->blue)/32768L))
-#define MaxRGB  ((Quantum) ((1L << QuantumDepth)-1L))
-#define OpaqueOpacity  0
+#define OpaqueOpacity  0UL
+#define PixelIntensity(pixel) ((unsigned long) \
+  (0.299*(pixel)->red+0.587*(pixel)->green+0.114*(pixel)->blue+0.5))
+#define PixelIntensityToQuantum(pixel) ((Quantum) \
+  (0.299*(pixel)->red+0.587*(pixel)->green+0.114*(pixel)->blue+0.5))
 #define TransparentOpacity  MaxRGB
-
+/*
+  Enum declarations.
+*/
+typedef enum
+{
+  UndefinedChannel,
+  RedChannel,
+  CyanChannel,
+  GreenChannel,
+  MagentaChannel,
+  BlueChannel,
+  YellowChannel,
+  OpacityChannel,
+  BlackChannel,
+  MatteChannel
+} ChannelType;
+
+typedef enum
+{
+  UndefinedClass,
+  DirectClass,
+  PseudoClass
+} ClassType;
+
+typedef enum
+{
+  UndefinedColorspace,
+  RGBColorspace,
+  GRAYColorspace,
+  TransparentColorspace,
+  OHTAColorspace,
+  XYZColorspace,
+  YCbCrColorspace,
+  YCCColorspace,
+  YIQColorspace,
+  YPbPrColorspace,
+  YUVColorspace,
+  CMYKColorspace,
+  sRGBColorspace
+} ColorspaceType;
+
+typedef enum
+{
+  UndefinedCompliance = 0x0000,
+  NoCompliance = 0x0000,
+  SVGCompliance = 0x0001,
+  X11Compliance = 0x0002,
+  XPMCompliance = 0x0004,
+  AllCompliance = 0xffff
+} ComplianceType;
+
+typedef enum
+{
+  UndefinedCompositeOp = 0,
+  OverCompositeOp,
+  InCompositeOp,
+  OutCompositeOp,
+  AtopCompositeOp,
+  XorCompositeOp,
+  PlusCompositeOp,
+  MinusCompositeOp,
+  AddCompositeOp,
+  SubtractCompositeOp,
+  DifferenceCompositeOp,
+  MultiplyCompositeOp,
+  BumpmapCompositeOp,
+  CopyCompositeOp,
+  CopyRedCompositeOp,
+  CopyGreenCompositeOp,
+  CopyBlueCompositeOp,
+  CopyOpacityCompositeOp,
+  ClearCompositeOp,
+  DissolveCompositeOp,
+  DisplaceCompositeOp,
+  ModulateCompositeOp,
+  ThresholdCompositeOp,
+  NoCompositeOp,
+  DarkenCompositeOp,
+  LightenCompositeOp,
+  HueCompositeOp,
+  SaturateCompositeOp,
+  ColorizeCompositeOp,
+  LuminizeCompositeOp,
+  ScreenCompositeOp,
+  OverlayCompositeOp
+} CompositeOperator;
+
+typedef enum
+{
+  UndefinedCompression,
+  NoCompression,
+  BZipCompression,
+  FaxCompression,
+  Group4Compression,
+  JPEGCompression,
+  LosslessJPEGCompression,
+  LZWCompression,
+  RLECompression,
+  ZipCompression
+} CompressionType;
+
+typedef enum
+{
+  UndefinedDispose,
+  NoneDispose,
+  BackgroundDispose,
+  PreviousDispose
+} DisposeType;
+
+typedef enum
+{
+  UndefinedEndian,
+  LSBEndian,
+  MSBEndian
+} EndianType;
+
+typedef enum
+{
+  UndefinedFilter,
+  PointFilter,
+  BoxFilter,
+  TriangleFilter,
+  HermiteFilter,
+  HanningFilter,
+  HammingFilter,
+  BlackmanFilter,
+  GaussianFilter,
+  QuadraticFilter,
+  CubicFilter,
+  CatromFilter,
+  MitchellFilter,
+  LanczosFilter,
+  BesselFilter,
+  SincFilter
+} FilterTypes;
+
+typedef enum
+{
+#undef NoValue
+  NoValue = 0x0000,
+#undef XValue
+  PsiValue = 0x0001,
+  XValue = 0x0001,
+#undef YValue
+  XiValue = 0x0002,
+  YValue = 0x0002,
+#undef WidthValue
+  RhoValue = 0x0004,
+  WidthValue = 0x0004,
+#undef HeightValue
+  SigmaValue = 0x0008,
+  HeightValue = 0x0008,
+#undef AllValues
+  AllValues = 0x000F,
+#undef XNegative
+  XiNegative = 0x0010,
+  XNegative = 0x0010,
+#undef YNegative
+  PsiNegative = 0x0020,
+  YNegative = 0x0020,
+  PercentValue = 0x1000,
+  AspectValue = 0x2000,
+  LessValue = 0x4000,
+  GreaterValue = 0x8000,
+  AreaValue = 0x10000
+} GeometryFlags;
+
+typedef enum
+{
+#undef ForgetGravity
+  ForgetGravity,
+#undef NorthWestGravity
+  NorthWestGravity,
+#undef NorthGravity
+  NorthGravity,
+#undef NorthEastGravity
+  NorthEastGravity,
+#undef WestGravity
+  WestGravity,
+#undef CenterGravity
+  CenterGravity,
+#undef EastGravity
+  EastGravity,
+#undef SouthWestGravity
+  SouthWestGravity,
+#undef SouthGravity
+  SouthGravity,
+#undef SouthEastGravity
+  SouthEastGravity,
+#undef StaticGravity
+  StaticGravity
+} GravityType;
+
+typedef enum
+{
+  UndefinedType,
+  BilevelType,
+  GrayscaleType,
+  GrayscaleMatteType,
+  PaletteType,
+  PaletteMatteType,
+  TrueColorType,
+  TrueColorMatteType,
+  ColorSeparationType,
+  ColorSeparationMatteType,
+  OptimizeType
+} ImageType;
+
+typedef enum
+{
+  UndefinedInterlace,
+  NoInterlace,
+  LineInterlace,
+  PlaneInterlace,
+  PartitionInterlace
+} InterlaceType;
+
+typedef enum
+{
+  UndefinedMode,
+  FrameMode,
+  UnframeMode,
+  ConcatenateMode
+} MontageMode;
+
+typedef enum
+{
+  UniformNoise,
+  GaussianNoise,
+  MultiplicativeGaussianNoise,
+  ImpulseNoise,
+  LaplacianNoise,
+  PoissonNoise
+} NoiseType;
+
+typedef enum
+{
+  UndefinedProfile,
+  ICMProfile,
+  IPTCProfile
+} ProfileType;
+
+typedef enum
+{
+  UndefinedPreview = 0,
+  RotatePreview,
+  ShearPreview,
+  RollPreview,
+  HuePreview,
+  SaturationPreview,
+  BrightnessPreview,
+  GammaPreview,
+  SpiffPreview,
+  DullPreview,
+  GrayscalePreview,
+  QuantizePreview,
+  DespecklePreview,
+  ReduceNoisePreview,
+  AddNoisePreview,
+  SharpenPreview,
+  BlurPreview,
+  ThresholdPreview,
+  EdgeDetectPreview,
+  SpreadPreview,
+  SolarizePreview,
+  ShadePreview,
+  RaisePreview,
+  SegmentPreview,
+  SwirlPreview,
+  ImplodePreview,
+  WavePreview,
+  OilPaintPreview,
+  CharcoalDrawingPreview,
+  JPEGPreview
+} PreviewType;
+
+typedef enum
+{
+  IndexQuantum,
+  GrayQuantum,
+  IndexAlphaQuantum,
+  GrayAlphaQuantum,
+  RedQuantum,
+  CyanQuantum,
+  GreenQuantum,
+  YellowQuantum,
+  BlueQuantum,
+  MagentaQuantum,
+  AlphaQuantum,
+  BlackQuantum,
+  RGBQuantum,
+  RGBAQuantum,
+  CMYKQuantum,
+  CMYKAQuantum
+} QuantumType;
+
+typedef enum
+{
+  UndefinedIntent,
+  SaturationIntent,
+  PerceptualIntent,
+  AbsoluteIntent,
+  RelativeIntent
+} RenderingIntent;
+
+typedef enum
+{
+  UndefinedResolution,
+  PixelsPerInchResolution,
+  PixelsPerCentimeterResolution
+} ResolutionType;
+
+typedef enum
+{
+  CharPixel,
+  ShortPixel,
+  IntegerPixel,
+  LongPixel,
+  FloatPixel,
+  DoublePixel
+} StorageType;
+
+typedef enum
+{
+  UndefinedTransmitType,
+  FileTransmitType,
+  BlobTransmitType,
+  StreamTransmitType,
+  ImageTransmitType
+} TransmitType;
+
 /*
   Typedef declarations.
 */
@@ -106,7 +473,7 @@ typedef struct _PixelPacket
 typedef struct _ColorInfo
 {
   const char
-    *filename,
+    *path,
     *name;
 
   ComplianceType
@@ -126,6 +493,31 @@ typedef struct _ColorInfo
     *next;
 } ColorInfo;
 
+typedef struct _DoublePixelPacket
+{
+  double
+    red,
+    green,
+    blue,
+    opacity;
+} DoublePixelPacket;
+
+typedef struct _ErrorInfo
+{
+  double
+    mean_error_per_pixel,
+    normalized_mean_error,
+    normalized_maximum_error;
+} ErrorInfo;
+
+#if !defined(WIN32)
+typedef off_t ExtendedSignedIntegralType;
+typedef size_t ExtendedUnsignedIntegralType;
+#else
+typedef __int64 ExtendedSignedIntegralType;
+typedef unsigned __int64 ExtendedUnsignedIntegralType;
+#endif
+
 typedef struct _FrameInfo
 {
   unsigned long
@@ -139,8 +531,26 @@ typedef struct _FrameInfo
     outer_bevel;
 } FrameInfo;
 
+typedef struct _GeometryInfo
+{
+  double
+    rho,
+    sigma,
+    xi,
+    psi;
+} GeometryInfo;
+
 typedef Quantum IndexPacket;
 
+typedef struct _LongPixelPacket
+{
+  unsigned long
+    red,
+    green,
+    blue,
+    opacity;
+} LongPixelPacket;
+
 typedef struct _MontageInfo
 {
   char
@@ -227,7 +637,6 @@ typedef struct _Image
     compression;
 
   unsigned int
-    orphan,  /* deprecated */
     taint,
     matte;
 
@@ -302,12 +711,14 @@ typedef struct _Image
   CompositeOperator
     compose;
 
+  DisposeType
+    dispose;
+
   struct _Image
     *clip_mask;
 
   unsigned long
     scene,
-    dispose,
     delay,
     iterations,
     total_colors;
@@ -315,10 +726,8 @@ typedef struct _Image
   long
     start_loop;
 
-  double
-    mean_error_per_pixel,
-    normalized_mean_error,
-    normalized_maximum_error;
+  ErrorInfo
+    error;
 
   TimerInfo
     timer;
@@ -365,6 +774,9 @@ typedef struct _Image
     *next;
 } Image;
 
+typedef unsigned int
+  (*StreamHandler)(const Image *,const void *,const size_t);
+
 typedef struct _ImageInfo
 {
   CompressionType
@@ -435,11 +847,11 @@ typedef struct _ImageInfo
 
   unsigned int
     ping,
-    verbose,
-    debug;
+    verbose;
 
   char
-    *view;
+    *view,
+    *authenticate;
 
   Image
     *attributes;
@@ -447,8 +859,11 @@ typedef struct _ImageInfo
   void
     *client_data;
 
-  int
-    (*fifo)(const Image *,const void *,const size_t);
+  void
+    *cache;
+
+  StreamHandler
+    stream;
 
   FILE
     *file;
@@ -473,25 +888,19 @@ typedef struct _ImageInfo
   Image const declarations.
 */
 extern MagickExport const char
-  *Alphabet,
-  *AppendBinaryType,
   *BackgroundColor,
   *BorderColor,
-  *DefaultPointSize,
   *DefaultTileFrame,
   *DefaultTileGeometry,
   *DefaultTileLabel,
   *ForegroundColor,
   *MatteColor,
-  *LoadImageText,
-  *LoadImagesText,
+  *LoadImageTag,
+  *LoadImagesTag,
   *PSDensityGeometry,
   *PSPageGeometry,
-  *ReadBinaryType,
-  *ReadBinaryUnbufferedType,
-  *SaveImageText,
-  *SaveImagesText,
-  *WriteBinaryType;
+  *SaveImageTag,
+  *SaveImagesTag;
 
 extern MagickExport const unsigned long
   DefaultCompressionQuality;
@@ -500,12 +909,11 @@ extern MagickExport const unsigned long
   Image utilities methods.
 */
 extern MagickExport char
-  **GetColorList(const char *,int *),
-  *GetMagickConfigurePath(const char *,ExceptionInfo *);
+  **GetColorList(const char *,unsigned long *),
+  *GetPageGeometry(const char *);
 
 extern MagickExport const char
-  *GetImageMagick(const unsigned char *,const size_t),
-  *GetMagickVersion(unsigned int *);
+  *GetImageMagick(const unsigned char *,const size_t);
 
 extern MagickExport const ColorInfo
   *GetColorInfo(const char *,ExceptionInfo *);
@@ -518,6 +926,8 @@ extern MagickExport const PixelPacket
     const unsigned long,ExceptionInfo *);
 
 extern MagickExport Image
+  *AdaptiveThresholdImage(const Image *,const unsigned long,const unsigned long,
+    const long,ExceptionInfo *),
   *AddNoiseImage(const Image *,const NoiseType,ExceptionInfo *),
   *AffineTransformImage(const Image *,const AffineMatrix *,ExceptionInfo *),
   *AllocateImage(const ImageInfo *),
@@ -533,8 +943,7 @@ extern MagickExport Image
   *ColorizeImage(const Image *,const char *,const PixelPacket,ExceptionInfo *),
   *ConstituteImage(const unsigned long,const unsigned long,const char *,
     const StorageType,const void *,ExceptionInfo *),
-  *ConvolveImage(const Image *,const unsigned int,const double *,
-    ExceptionInfo *),
+  *ConvolveImage(const Image *,const unsigned int,const double *,ExceptionInfo *),
   *CropImage(const Image *,const RectangleInfo *,ExceptionInfo *),
   *DeconstructImages(const Image *,ExceptionInfo *),
   *DespeckleImage(const Image *,ExceptionInfo *),
@@ -559,8 +968,7 @@ extern MagickExport Image
   *PingImage(const ImageInfo *,ExceptionInfo *),
   *ReadImage(const ImageInfo *,ExceptionInfo *),
   *ReadInlineImage(const ImageInfo *,const char *,ExceptionInfo *),
-  *ReadStream(const ImageInfo *,
-    int (*)(const Image *,const void *,const size_t),ExceptionInfo *),
+  *ReadStream(const ImageInfo *,StreamHandler,ExceptionInfo *),
   *ReduceNoiseImage(const Image *,const double,ExceptionInfo *),
   *ReferenceImage(Image *),
   *ResizeImage(const Image *,const unsigned long,const unsigned long,
@@ -579,6 +987,8 @@ extern MagickExport Image
   *SteganoImage(const Image *,const Image *,ExceptionInfo *),
   *StereoImage(const Image *,const Image *,ExceptionInfo *),
   *SwirlImage(const Image *,double,ExceptionInfo *),
+  *ThumbnailImage(const Image *,const unsigned long,const unsigned long,
+    ExceptionInfo *),
   *UnsharpMaskImage(const Image *,const double,const double,const double,
     const double,ExceptionInfo *),
   *WaveImage(const Image *,const double,const double,ExceptionInfo *),
@@ -593,12 +1003,7 @@ extern MagickExport ImageType
 
 extern MagickExport IndexPacket
   ConstrainColormapIndex(Image *,const unsigned long),
-  *GetIndexes(const Image *),
-  ValidateColormapIndex(Image *,const unsigned long);
-
-extern MagickExport int
-  GetImageGeometry(const Image *,const char *,const unsigned int,
-    RectangleInfo *);
+  *GetIndexes(const Image *);
 
 extern MagickExport MontageInfo
   *CloneMontageInfo(const ImageInfo *,const MontageInfo *);
@@ -629,10 +1034,11 @@ extern MagickExport unsigned int
   EqualizeImage(Image *),
   FuzzyColorMatch(const PixelPacket *,const PixelPacket *,const double),
   GammaImage(Image *,const char *),
-  GetNumberScenes(const Image *),
+  GetGeometry(const char *,long *,long *,unsigned long *,unsigned long *),
   GradientImage(Image *,const PixelPacket *,const PixelPacket *),
+  IsGeometry(const char *),
   IsGrayImage(const Image *,ExceptionInfo *),
-  IsImagesEqual(Image *,Image *),
+  IsImagesEqual(Image *,const Image *),
   IsTaintImage(const Image *),
   IsMagickConflict(const char *),
   IsMonochromeImage(const Image *,ExceptionInfo *),
@@ -640,6 +1046,8 @@ extern MagickExport unsigned int
   IsPaletteImage(const Image *,ExceptionInfo *),
   IsSubimage(const char *,const unsigned int),
   LevelImage(Image *,const char *),
+  LevelImageChannel(Image *,const ChannelType,const double,const double,
+    const double),
   ListColorInfo(FILE *,ExceptionInfo *),
   ListMagickInfo(FILE *,ExceptionInfo *),
   ModulateImage(Image *,const char *),
@@ -647,10 +1055,16 @@ extern MagickExport unsigned int
   MogrifyImages(const ImageInfo *,const int,char **,Image **),
   NegateImage(Image *,const unsigned int),
   NormalizeImage(Image *),
-  OpaqueImage(Image *,const PixelPacket,const PixelPacket),
-  PlasmaImage(Image *,const SegmentInfo *,int,int),
+  ParseGeometry(const char *,GeometryInfo *),
+  ParseAbsoluteGeometry(const char *,RectangleInfo *),
+  ParseGravityGeometry(const Image *,const char *,RectangleInfo *),
+  ParseMetaGeometry(const char *,long *,long *,unsigned long *,unsigned long *),
+  ParsePageGeometry(const Image *,const char *,RectangleInfo *),
+  ParseSizeGeometry(const Image *,const char *,RectangleInfo *),
+  PlasmaImage(Image *,const SegmentInfo *,unsigned long,unsigned long),
   PopImagePixels(const Image *,const QuantumType,unsigned char *),
-  ProfileImage(Image *,const char *,const unsigned char *,const size_t),
+  ProfileImage(Image *,const char *,const unsigned char *,const size_t,
+    unsigned int),
   PushImagePixels(Image *,const QuantumType,const unsigned char *),
   QueryColorDatabase(const char *,PixelPacket *,ExceptionInfo *),
   QueryColorname(const Image *,const PixelPacket *,const ComplianceType,char *,
@@ -664,13 +1078,12 @@ extern MagickExport unsigned int
   SortColormapByIntensity(Image *),
   SyncImagePixels(Image *),
   ThresholdImage(Image *,const double),
+  ThresholdImageChannel(Image *,const char *),
   TransformRGBImage(Image *,const ColorspaceType),
-  TransparentImage(Image *,const PixelPacket,const unsigned int),
   UnregisterMagickInfo(const char *),
   WriteImage(const ImageInfo *,Image *),
   WriteImages(ImageInfo *,Image *,const char *,ExceptionInfo *),
-  WriteStream(const ImageInfo *,Image *,
-    int (*)(const Image *,const void *,const size_t));
+  WriteStream(const ImageInfo *,Image *,StreamHandler);
 
 extern MagickExport unsigned long
   GetImageDepth(const Image *,ExceptionInfo *),
@@ -685,270 +1098,25 @@ extern MagickExport void
   DestroyImage(Image *),
   DestroyImageInfo(ImageInfo *),
   DestroyImagePixels(Image *),
-  DestroyImages(Image *),
   DestroyMagick(void),
   DestroyMagickInfo(void),
   DestroyMontageInfo(MontageInfo *),
-	GetImageException(Image *,ExceptionInfo *),
+  GetColorTuple(const PixelPacket *,const unsigned int,const unsigned int,
+    const unsigned int,char *),
+  GetImageException(Image *,ExceptionInfo *),
   GetImageInfo(ImageInfo *),
   GetMontageInfo(const ImageInfo *,MontageInfo *),
   InitializeMagick(const char *),
   ModifyImage(Image **,ExceptionInfo *),
-  RegisterARTImage(void),
-  RegisterAVIImage(void),
-  RegisterAVSImage(void),
-  RegisterBIEImage(void),
-  RegisterBMPImage(void),
-  RegisterCAPTIONImage(void),
-  RegisterCMYKImage(void),
-  RegisterCUTImage(void),
-  RegisterDCMImage(void),
-  RegisterDCXImage(void),
-  RegisterDIBImage(void),
-  RegisterDPSImage(void),
-  RegisterDPXImage(void),
-  RegisterEPDFImage(void),
-  RegisterEPIImage(void),
-  RegisterEPSImage(void),
-  RegisterEPS2Image(void),
-  RegisterEPSFImage(void),
-  RegisterEPSIImage(void),
-  RegisterEPTImage(void),
-  RegisterFAXImage(void),
-  RegisterFITSImage(void),
-  RegisterFPXImage(void),
-  RegisterG3Image(void),
-  RegisterGIFImage(void),
-  RegisterGIF87Image(void),
-  RegisterGRADIENTImage(void),
-  RegisterGRANITEImage(void),
-  RegisterGRAYImage(void),
-  RegisterHImage(void),
-  RegisterHDFImage(void),
-  RegisterHISTOGRAMImage(void),
-  RegisterHTMImage(void),
-  RegisterHTMLImage(void),
-  RegisterICBImage(void),
-  RegisterICONImage(void),
-  RegisterJBGImage(void),
-  RegisterJBIGImage(void),
-  RegisterJPGImage(void),
-  RegisterJPEGImage(void),
-  RegisterJP2Image(void),
-  RegisterLABELImage(void),
-  RegisterLOGOImage(void),
-  RegisterMAPImage(void),
-  RegisterMETAImage(void),
-  RegisterMATImage(void),
-  RegisterMATTEImage(void),
-  RegisterMIFFImage(void),
-  RegisterMNGImage(void),
-  RegisterMONOImage(void),
-  RegisterMPCImage(void),
-  RegisterMPEGImage(void),
-  RegisterMPRImage(void),
-  RegisterMSLImage(void),
-  RegisterMTVImage(void),
-  RegisterMVGImage(void),
-  RegisterNETSCAPEImage(void),
-  RegisterNULLImage(void),
-  RegisterP7Image(void),
-  RegisterPBMImage(void),
-  RegisterOTBImage(void),
-  RegisterPALMImage(void),
-  RegisterPCDImage(void),
-  RegisterPCDSImage(void),
-  RegisterPCLImage(void),
-  RegisterPCTImage(void),
-  RegisterPCXImage(void),
-  RegisterPDBImage(void),
-  RegisterPDFImage(void),
-  RegisterPICImage(void),
-  RegisterPICTImage(void),
-  RegisterPIXImage(void),
-  RegisterPLASMAImage(void),
-  RegisterPGMImage(void),
-  RegisterPMImage(void),
-  RegisterPNGImage(void),
-  RegisterPNMImage(void),
-  RegisterPPMImage(void),
-  RegisterPREVIEWImage(void),
-  RegisterPSImage(void),
-  RegisterPS2Image(void),
-  RegisterPS3Image(void),
-  RegisterPSDImage(void),
-  RegisterPTIFImage(void),
-  RegisterPWPImage(void),
-  RegisterRASImage(void),
-  RegisterRGBImage(void),
-  RegisterRGBAImage(void),
-  RegisterRLAImage(void),
-  RegisterRLEImage(void),
-  RegisterSCTImage(void),
-  RegisterSFWImage(void),
-  RegisterSGIImage(void),
-  RegisterSHTMLImage(void),
-  RegisterSTEGANOImage(void),
-  RegisterSUNImage(void),
-  RegisterSVGImage(void),
-  RegisterTEXTImage(void),
-  RegisterTGAImage(void),
-  RegisterTIFImage(void),
-  RegisterTIFFImage(void),
-  RegisterTILEImage(void),
-  RegisterTIMImage(void),
-  RegisterTTFImage(void),
-  RegisterTXTImage(void),
-  RegisterUILImage(void),
-  RegisterURLImage(void),
-  RegisterUYVYImage(void),
-  RegisterVDAImage(void),
-  RegisterVICARImage(void),
-  RegisterVIDImage(void),
-  RegisterVIFFImage(void),
-  RegisterVSTImage(void),
-  RegisterWBMPImage(void),
-  RegisterWMFImage(void),
-  RegisterWPGImage(void),
-  RegisterXImage(void),
-  RegisterXBMImage(void),
-  RegisterXCImage(void),
-  RegisterXCFImage(void),
-  RegisterXPMImage(void),
-  RegisterXVImage(void),
-  RegisterXWDImage(void),
-  RegisterYUVImage(void),
+  SetGeometry(const Image *,RectangleInfo *),
+  SetGeometryInfo(GeometryInfo *),
   SetImage(Image *,const Quantum),
   SetImageOpacity(Image *,const unsigned int),
   SetImageType(Image *,const ImageType),
   SolarizeImage(Image *,const double),
   SyncImage(Image *),
   TextureImage(Image *,const Image *),
-  TransformImage(Image **,const char *,const char *),
-  UnregisterARTImage(void),
-  UnregisterAVIImage(void),
-  UnregisterAVSImage(void),
-  UnregisterBIEImage(void),
-  UnregisterBMPImage(void),
-  UnregisterCAPTIONImage(void),
-  UnregisterCMYKImage(void),
-  UnregisterCUTImage(void),
-  UnregisterDCMImage(void),
-  UnregisterDCXImage(void),
-  UnregisterDIBImage(void),
-  UnregisterDPSImage(void),
-  UnregisterDPXImage(void),
-  UnregisterEPDFImage(void),
-  UnregisterEPIImage(void),
-  UnregisterEPSImage(void),
-  UnregisterEPS2Image(void),
-  UnregisterEPSFImage(void),
-  UnregisterEPSIImage(void),
-  UnregisterEPTImage(void),
-  UnregisterFAXImage(void),
-  UnregisterFITSImage(void),
-  UnregisterFPXImage(void),
-  UnregisterG3Image(void),
-  UnregisterGIFImage(void),
-  UnregisterGIF87Image(void),
-  UnregisterGRADIENTImage(void),
-  UnregisterGRANITEImage(void),
-  UnregisterGRAYImage(void),
-  UnregisterHImage(void),
-  UnregisterHDFImage(void),
-  UnregisterHISTOGRAMImage(void),
-  UnregisterHTMImage(void),
-  UnregisterHTMLImage(void),
-  UnregisterICBImage(void),
-  UnregisterICONImage(void),
-  UnregisterJBGImage(void),
-  UnregisterJBIGImage(void),
-  UnregisterJPGImage(void),
-  UnregisterJPEGImage(void),
-  UnregisterJP2Image(void),
-  UnregisterLABELImage(void),
-  UnregisterLOGOImage(void),
-  UnregisterMAPImage(void),
-  UnregisterMATImage(void),
-  UnregisterMATTEImage(void),
-  UnregisterMETAImage(void),
-  UnregisterMIFFImage(void),
-  UnregisterMNGImage(void),
-  UnregisterMONOImage(void),
-  UnregisterMPCImage(void),
-  UnregisterMPEGImage(void),
-  UnregisterMPRImage(void),
-  UnregisterMSLImage(void),
-  UnregisterMTVImage(void),
-  UnregisterMVGImage(void),
-  UnregisterNETSCAPEImage(void),
-  UnregisterNULLImage(void),
-  UnregisterP7Image(void),
-  UnregisterPBMImage(void),
-  UnregisterOTBImage(void),
-  UnregisterPALMImage(void),
-  UnregisterPCDImage(void),
-  UnregisterPCDSImage(void),
-  UnregisterPCLImage(void),
-  UnregisterPCTImage(void),
-  UnregisterPCXImage(void),
-  UnregisterPDBImage(void),
-  UnregisterPDFImage(void),
-  UnregisterPICImage(void),
-  UnregisterPICTImage(void),
-  UnregisterPIXImage(void),
-  UnregisterPLASMAImage(void),
-  UnregisterPGMImage(void),
-  UnregisterPMImage(void),
-  UnregisterPNGImage(void),
-  UnregisterPNMImage(void),
-  UnregisterPPMImage(void),
-  UnregisterPREVIEWImage(void),
-  UnregisterPSImage(void),
-  UnregisterPS2Image(void),
-  UnregisterPS3Image(void),
-  UnregisterPSDImage(void),
-  UnregisterPTIFImage(void),
-  UnregisterPWPImage(void),
-  UnregisterRASImage(void),
-  UnregisterRGBImage(void),
-  UnregisterRGBAImage(void),
-  UnregisterRLAImage(void),
-  UnregisterRLEImage(void),
-  UnregisterSCTImage(void),
-  UnregisterSFWImage(void),
-  UnregisterSGIImage(void),
-  UnregisterSHTMLImage(void),
-  UnregisterSTEGANOImage(void),
-  UnregisterSUNImage(void),
-  UnregisterSVGImage(void),
-  UnregisterTEXTImage(void),
-  UnregisterTGAImage(void),
-  UnregisterTIFImage(void),
-  UnregisterTIFFImage(void),
-  UnregisterTILEImage(void),
-  UnregisterTIMImage(void),
-  UnregisterTTFImage(void),
-  UnregisterTXTImage(void),
-  UnregisterUILImage(void),
-  UnregisterURLImage(void),
-  UnregisterUYVYImage(void),
-  UnregisterVDAImage(void),
-  UnregisterVICARImage(void),
-  UnregisterVIDImage(void),
-  UnregisterVIFFImage(void),
-  UnregisterVSTImage(void),
-  UnregisterWBMPImage(void),
-  UnregisterWMFImage(void),
-  UnregisterWPGImage(void),
-  UnregisterXImage(void),
-  UnregisterXBMImage(void),
-  UnregisterXCImage(void),
-  UnregisterXCFImage(void),
-  UnregisterXPMImage(void),
-  UnregisterXVImage(void),
-  UnregisterXWDImage(void),
-  UnregisterYUVImage(void);
+  TransformImage(Image **,const char *,const char *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
