@@ -15,10 +15,10 @@
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
-%                                January 2003                                 %
+%                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,6 +54,54 @@
   Include declarations.
 */
 #include "studio.h"
+#include "list.h"
+#include "blob.h"
+#include "utility.h"
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   A p p e n d I m a g e T o L i s t                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AppendImageToList() appends an image to the end of the list.
+%
+%  The format of the AppendImageToList method is:
+%
+%      AppendImageToList(Image *images,Image *image)
+%
+%  A description of each parameter follows:
+%
+%    o images: The image list.
+%
+%    o image: The image.
+%
+%
+*/
+MagickExport void AppendImageToList(Image **images,Image *image)
+{
+  register Image
+    *p;
+
+  assert(images != (Image **) NULL);
+  if (image == (Image *) NULL)
+    return;
+  assert(image->signature == MagickSignature);
+  if ((*images) == (Image *) NULL)
+    {
+      *images=image;
+      return;
+    }
+  assert((*images)->signature == MagickSignature);
+  for (p=(*images); p->next != (Image *) NULL; p=p->next);
+  p->next=image;
+  image->previous=p;
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -83,34 +131,38 @@
 MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
 {
   Image
-    *clone_images,
+    *clone,
     *image;
 
+  register Image
+    *p;
+
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  clone_images=(Image *) NULL;
-  for ( ; images != (Image *) NULL; images=images->next)
+  while (images->previous != (Image *) NULL)
+    images=images->previous;
+  image=(Image *) NULL;
+  for (p=(Image *) NULL; images != (Image *) NULL; images=images->next)
   {
-    image=CloneImage(images,0,0,True,exception);
-    if (image == (Image *) NULL)
+    clone=CloneImage(images,0,0,True,exception);
+    if (clone == (Image *) NULL)
       {
-        if (clone_images != (Image *) NULL)
-          DestroyImageList(clone_images);
+        if (image != (Image *) NULL)
+          DestroyImageList(image);
         return((Image *) NULL);
       }
-    if (clone_images == (Image *) NULL)
+    if (image == (Image *) NULL)
       {
-        clone_images=image;
+        image=clone;
+        p=image;
         continue;
       }
-    image->previous=clone_images;
-    clone_images->next=image;
-    clone_images=clone_images->next;
+    p->next=clone;
+    clone->previous=p;
+    p=p->next;
   }
-  while (clone_images->previous != (Image *) NULL)
-    clone_images=clone_images->previous;
-  return(clone_images);
+  return(image);
 }
 
 /*
@@ -118,52 +170,50 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   D e l e t e I m a g e L i s t                                             %
+%   D e l e t e I m a g e F r o m L i s t                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DeleteImageList() deletes an image at the specified position in the list.
+%  DeleteImageFromList() deletes an image from the list.
 %
-%  The format of the DeleteImageList method is:
+%  The format of the DeleteImageFromList method is:
 %
-%      unsigned int DeleteImageList(Image *images,const unsigned long offset)
+%      DeleteImageFromList(Image **images)
 %
 %  A description of each parameter follows:
 %
 %    o images: The image list.
 %
-%    o offset: The position within the list.
-%
 %
 */
-MagickExport unsigned int DeleteImageList(Image *images,
-  const unsigned long offset)
+MagickExport void DeleteImageFromList(Image **images)
 {
-  Image
-    *image;
+  register Image
+    *p;
 
-  register long
-    i;
-
-  if (images == (Image *) NULL)
-    return(False);
-  assert(images->signature == MagickSignature);
-  while (images->previous != (Image *) NULL)
-    images=images->previous;
-  for (i=0; images != (Image *) NULL; images=images->next)
-    if (i++ == (long) offset)
-      break;
-  if (images == (Image *) NULL)
-    return(False);
-  image=images;
-  if (images->previous != (Image *) NULL)
-    images->previous->next=images->next;
-  if (images->next != (Image *) NULL)
-    images->next->previous=images->previous;
-  DestroyImage(image);
-  return(True);
+  assert(images != (Image **) NULL);
+  if ((*images) == (Image *) NULL)
+    return;
+  assert((*images)->signature == MagickSignature);
+  p=(*images);
+  if ((p->previous == (Image *) NULL) && (p->next == (Image *) NULL))
+    *images=(Image *) NULL;
+  else
+    {
+      if (p->previous != (Image *) NULL)
+        {
+          p->previous->next=p->next;
+          *images=p->previous;
+        }
+      if (p->next != (Image *) NULL)
+        {
+          p->next->previous=p->previous;
+          *images=p->next;
+        }
+    }
+  DestroyImage(p);
 }
 
 /*
@@ -189,26 +239,19 @@ MagickExport unsigned int DeleteImageList(Image *images,
 %
 %
 */
-
-MagickExport void DestroyImages(Image *image)
-{
-  DestroyImageList(image);
-}
-
 MagickExport void DestroyImageList(Image *images)
 {
-  Image
-    *image;
+  register Image
+    *p;
 
   if (images == (Image *) NULL)
     return;
   assert(images->signature == MagickSignature);
-  while (images->previous != (Image *) NULL)
-    images=images->previous;
-  for (image=images; images != (Image *) NULL; image=images)
+  for (p=images; p->previous != (Image *) NULL; p=p->previous);
+  for (images=p; p != (Image *) NULL; images=p)
   {
-    images=images->next;
-    DestroyImage(image);
+    p=p->next;
+    DestroyImage(images);
   }
 }
 
@@ -217,18 +260,17 @@ MagickExport void DestroyImageList(Image *images)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e t I m a g e L i s t                                                   %
+%   G e t I m a g e F r o m L i s t                                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetImageList() returns an image at the specified position in the list.
+%  GetImageFromList() returns an image at the specified offset from the list.
 %
-%  The format of the GetImageList method is:
+%  The format of the GetImageFromList method is:
 %
-%      Image *GetImageList(const Image *images,const unsigned long offset,
-%        ExceptionInfo *exception)
+%      Image *GetImageFromList(const Image *images,const long offset)
 %
 %  A description of each parameter follows:
 %
@@ -236,28 +278,26 @@ MagickExport void DestroyImageList(Image *images)
 %
 %    o offset: The position within the list.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
-%
 %
 */
-MagickExport Image *GetImageList(const Image *images,const unsigned long offset,
-  ExceptionInfo *exception)
+MagickExport Image *GetImageFromList(const Image *images,const long offset)
 {
+  register const Image
+    *p;
+
   register long
     i;
 
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  while (images->previous != (Image *) NULL)
-    images=images->previous;
-  for (i=0; images != (Image *) NULL; images=images->next)
-    if (i++ == (long) offset)
+  for (p=images; p->previous != (Image *) NULL; p=p->previous);
+  for (i=0; p != (Image *) NULL; p=p->next)
+    if (i++ == offset)
       break;
-  if (images == (Image *) NULL)
+  if (p == (Image *) NULL)
     return((Image *) NULL);
-  return(CloneImage(images,0,0,True,exception));
+  return((Image *) p);
 }
 
 /*
@@ -265,18 +305,17 @@ MagickExport Image *GetImageList(const Image *images,const unsigned long offset,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e t I m a g e L i s t I n d e x                                         %
+%   G e t I m a g e I n d e x I n L i s t                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetImageListIndex() returns the position in the list of the specified
-%  image.
+%  GetImageIndexInList() returns the offset in the list of the specified image.
 %
-%  The format of the GetImageListIndex method is:
+%  The format of the GetImageIndexInList method is:
 %
-%      size_t GetImageListIndex(const Image *images)
+%      long GetImageIndexInList(const Image *images)
 %
 %  A description of each parameter follows:
 %
@@ -284,13 +323,13 @@ MagickExport Image *GetImageList(const Image *images,const unsigned long offset,
 %
 %
 */
-MagickExport unsigned long GetImageListIndex(const Image *images)
+MagickExport long GetImageIndexInList(const Image *images)
 {
   register long
     i;
 
-  if (images == (Image *) NULL)
-    return(0);
+  if (images == (const Image *) NULL)
+    return(-1);
   assert(images->signature == MagickSignature);
   for (i=0; images->previous != (Image *) NULL; i++)
     images=images->previous;
@@ -302,17 +341,18 @@ MagickExport unsigned long GetImageListIndex(const Image *images)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e t I m a g e L i s t S i z e                                           %
+%   G e t I m a g e L i s t L e n g t h                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetImageListSize() returns the number of images in the list.
+%  GetImageListLength() returns the length of the list (the number of images in
+%  the list).
 %
-%  The format of the GetImageListSize method is:
+%  The format of the GetImageListLength method is:
 %
-%      size_t GetImageListSize(const Image *images)
+%      unsigned long GetImageListLength(const Image *images)
 %
 %  A description of each parameter follows:
 %
@@ -320,15 +360,9 @@ MagickExport unsigned long GetImageListIndex(const Image *images)
 %
 %
 */
-
-MagickExport unsigned int GetNumberScenes(const Image *image)
+MagickExport unsigned long GetImageListLength(const Image *images)
 {
-  return(GetImageListSize(image));
-}
-
-MagickExport size_t GetImageListSize(const Image *images)
-{
-  register size_t
+  register long
     i;
 
   if (images == (Image *) NULL)
@@ -346,17 +380,17 @@ MagickExport size_t GetImageListSize(const Image *images)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e t N e x t I m a g e                                                   %
+%   G e t N e x t I m a g e I n L i s t                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetNextImage() returns the next image in a list.
+%  GetNextImageInList() returns the next image in the list.
 %
-%  The format of the GetNextImage method is:
+%  The format of the GetNextImageInList method is:
 %
-%      Image *GetNextImage(Image *images)
+%      Image *GetNextImageInList(const Image *images)
 %
 %  A description of each parameter follows:
 %
@@ -364,18 +398,11 @@ MagickExport size_t GetImageListSize(const Image *images)
 %
 %
 */
-MagickExport Image *GetNextImage(Image *images)
+MagickExport Image *GetNextImageInList(const Image *images)
 {
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  if (images->next == (Image *) NULL)
-    return((Image *) NULL);
-  if (images->blob != images->next->blob)
-    {
-      DestroyBlobInfo(images->next->blob);
-      images->next->blob=ReferenceBlob(images->blob);
-    }
   return(images->next);
 }
 
@@ -383,6 +410,37 @@ MagickExport Image *GetNextImage(Image *images)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
+%   G e t P r e v i o u s I m a g e I n L i s t                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetPreviousImageInList() returns the previous image in the list.
+%
+%  The format of the GetPreviousImageInList method is:
+%
+%      Image *GetPreviousImageInList(const Image *images)
+%
+%  A description of each parameter follows:
+%
+%    o images: The image list.
+%
+%
+*/
+MagickExport Image *GetPreviousImageInList(const Image *images)
+{
+  if (images == (Image *) NULL)
+    return((Image *) NULL);
+  assert(images->signature == MagickSignature);
+  return(images->previous);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
 %     I m a g e L i s t T o A r r a y                                         %
 %                                                                             %
 %                                                                             %
@@ -420,15 +478,17 @@ MagickExport Image **ImageListToArray(const Image *images,
   if (images == (Image *) NULL)
     return((Image **) NULL);
   assert(images->signature == MagickSignature);
-  group=(Image **) AcquireMemory(GetImageListSize(images)*sizeof(Image *));
+  group=(Image **) AcquireMemory(GetImageListLength(images)*sizeof(Image *));
   if (group == (Image **) NULL)
     {
-      ThrowException(exception,ResourceLimitError,
-        "Unable to freate image group","Memory allocation failed");
+      ThrowException(exception,ResourceLimitError,"MemoryAllocationFailed",
+        "UnableToCreateImageGroup");
       return((Image **) NULL);
     }
-  for (i=0; i < (long) GetImageListSize(images); i++)
-    group[i]=GetImageList(images,i,exception);
+  while (images->previous != (Image *) NULL)
+    images=images->previous;
+  for (i=0; images != (Image *) NULL; images=images->next)
+    group[i++]=(Image *) images;
   return(group);
 }
 
@@ -437,6 +497,49 @@ MagickExport Image **ImageListToArray(const Image *images,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I n s e r t I m a g e I n L i s t                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  InsertImageInList() inserts an image in the list.
+%
+%  The format of the InsertImageInList method is:
+%
+%      InsertImageInList(Image **images,Image *image)
+%
+%  A description of each parameter follows:
+%
+%    o images: The image list.
+%
+%    o image: The image.
+%
+%
+*/
+MagickExport void InsertImageInList(Image **images,Image *image)
+{
+  Image
+    *split;
+
+  assert(images != (Image **) NULL);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if ((*images) == (Image *) NULL)
+    return;
+  assert((*images)->signature == MagickSignature);
+  split=SplitImageList(*images);
+  if (split == (Image *) NULL)
+    return;
+  AppendImageToList(images,image);
+  AppendImageToList(images,split);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   N e w I m a g e L i s t                                                   %
 %                                                                             %
 %                                                                             %
@@ -460,17 +563,47 @@ MagickExport Image *NewImageList(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   P o p I m a g e L i s t                                                   %
+%   P r e p e n d I m a g e T o L i s t                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  PopImageList() removes the last image in the list.
+%  PrependImageToList() prepends the image to the beginning of the list.
 %
-%  The format of the PopImageList method is:
+%  The format of the PrependImageToList method is:
 %
-%      Image *PopImageList(Image **images)
+%      PrependImageToList(Image *images,Image *image)
+%
+%  A description of each parameter follows:
+%
+%    o images: The image list.
+%
+%    o image: The image.
+%
+%
+*/
+MagickExport void PrependImageToList(Image **images,Image *image)
+{
+  AppendImageToList(&image,*images);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   R e m o v e F i r s t I m a g e F r o m L i s t                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  RemoveFirstImageFromList() removes an image from the beginning of the list.
+%
+%  The format of the RemoveFirstImageFromList method is:
+%
+%      Image *RemoveFirstImageFromList(Image **images)
 %
 %  A description of each parameter follows:
 %
@@ -478,7 +611,7 @@ MagickExport Image *NewImageList(void)
 %
 %
 */
-MagickExport Image *PopImageList(Image **images)
+MagickExport Image *RemoveFirstImageFromList(Image **images)
 {
   Image
     *image;
@@ -487,13 +620,16 @@ MagickExport Image *PopImageList(Image **images)
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
-  while ((*images)->next != (Image *) NULL)
-    (*images)=(*images)->next;
   image=(*images);
-  *images=(*images)->previous;
-  if ((*images) != (Image *) NULL)
-    (*images)->next=(Image *) NULL;
-  image->previous=(Image *) NULL;
+  while (image->previous != (Image *) NULL)
+    image=image->previous;
+  if (image == *images)
+    *images=(*images)->next;
+  if (image->next != (Image *) NULL)
+    {
+      image->next->previous=(Image *) NULL;
+      image->next=(Image *) NULL;
+    }
   return(image);
 }
 
@@ -502,51 +638,44 @@ MagickExport Image *PopImageList(Image **images)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   P u s h I m a g e L i s t                                                 %
+%   R e m o v e L a s t I m a g e F r o m L i s t                             %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  PushImageList() add an image to the end of the list.
+%  RemoveLastImageFromList() removes the last image from the list.
 %
-%  The format of the PushImageList method is:
+%  The format of the RemoveLastImageFromList method is:
 %
-%      unsigned int PushImageList(Image *images,const Image *image,
-%        ExceptionInfo *exception)
+%      Image *RemoveLastImageFromList(Image **images)
 %
 %  A description of each parameter follows:
 %
 %    o images: The image list.
 %
-%    o image: The image.
-%
-%    o exception: Return any errors or warnings in this structure.
-%
 %
 */
-MagickExport unsigned int PushImageList(Image **images,const Image *image,
-  ExceptionInfo *exception)
+MagickExport Image *RemoveLastImageFromList(Image **images)
 {
   Image
-    *next;
+    *image;
 
   assert(images != (Image **) NULL);
-  if (image == (Image *) NULL)
-    return(False);
-  assert(image->signature == MagickSignature);
   if ((*images) == (Image *) NULL)
-    {
-      *images=CloneImageList(image,exception);
-      return(*images != (Image *) NULL);
-    }
+    return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
-  for (next=(*images); next->next != (Image *) NULL; next=next->next);
-  next->next=CloneImageList(image,exception);
-  if (next->next == (Image *) NULL)
-    return(False);
-  next->next->previous=next;
-  return(True);
+  image=(*images);
+  while (image->next != (Image *) NULL)
+    image=image->next;
+  if (image == *images)
+    *images=(*images)->previous;
+  if (image->previous != (Image *) NULL)
+    {
+      image->previous->next=(Image *) NULL;
+      image->previous=(Image *) NULL;
+    }
+  return(image);
 }
 
 /*
@@ -564,115 +693,34 @@ MagickExport unsigned int PushImageList(Image **images,const Image *image,
 %
 %  The format of the ReverseImageList method is:
 %
-%      Image *ReverseImageList(const Image *images,ExceptionInfo *exception)
+%       ReverseImageList(const Image **images)
 %
 %  A description of each parameter follows:
 %
 %    o images: The image list.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
 %
 */
-MagickExport Image *ReverseImageList(const Image *images,
-  ExceptionInfo *exception)
-{
-  Image
-    *reverse_images,
-    *image;
-
-  register long
-    i;
-
-  if (images == (Image *) NULL)
-    return((Image *) NULL);
-  assert(images->signature == MagickSignature);
-  reverse_images=(Image *) NULL;
-  for (i=0; i < (long) GetImageListSize(images); i++)
-  {
-    image=GetImageList(images,GetImageListSize(images)-i-1,exception);
-    if (image == (Image *) NULL)
-      {
-        if (reverse_images != (Image *) NULL)
-          DestroyImageList(reverse_images);
-        break;
-      }
-    if (reverse_images == (Image *) NULL)
-      {
-        reverse_images=image;
-        continue;
-      }
-    image->previous=reverse_images;
-    reverse_images->next=image;
-    reverse_images=reverse_images->next;
-  }
-  while (reverse_images->previous != (Image *) NULL)
-    reverse_images=reverse_images->previous;
-  return(reverse_images);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   S e t I m a g e L i s t                                                   %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  SetImageList() inserts an image into the list at the specified position.
-%
-%  The format of the SetImageList method is:
-%
-%      unsigned int SetImageList(Image *images,const Image *image,
-%        const unsigned long offset,ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o images: The image list.
-%
-%    o image: The image.
-%
-%    o offset: The position within the list.
-%
-%    o exception: Return any errors or warnings in this structure.
-%
-%
-*/
-MagickExport unsigned int SetImageList(Image **images,const Image *image,
-  const unsigned long offset,ExceptionInfo *exception)
+MagickExport void ReverseImageList(Image **images)
 {
   Image
     *next;
 
-  register long
-    i;
+  register Image
+    *p;
 
   assert(images != (Image **) NULL);
-  if (image == (Image *) NULL)
-    return(False);
-  assert(image->signature == MagickSignature);
   if ((*images) == (Image *) NULL)
-    {
-      if (offset > 0)
-        return(False);
-      *images=CloneImageList(image,exception);
-      return(*images != (Image *) NULL);
-    }
+    return;
   assert((*images)->signature == MagickSignature);
-  for (next=(*images); next->next != (Image *) NULL; next=next->next);
-  for (i=0; next != (Image *) NULL; next=next->next)
-    if (i++ == (long) offset)
-      break;
-  if (next == (Image *) NULL)
-    return(False);
-  next->next=CloneImageList(image,exception);
-  if (next->next == (Image *) NULL)
-    return(False);
-  next->next->previous=next;
-  return(True);
+  for (p=(*images); p->next != (Image *) NULL; p=p->next);
+  *images=p;
+  for ( ; p != (Image *) NULL; p=p->next)
+  {
+    next=p->next;
+    p->next=p->previous;
+    p->previous=next;
+  }
 }
 
 /*
@@ -680,130 +728,52 @@ MagickExport unsigned int SetImageList(Image **images,const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S h i f t I m a g e L i s t                                               %
+%   S p l i c e I m a g e I n t o L i s t                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ShiftImageList() removes an image from the beginning of the list.
+%  SpliceImageIntoList() removes 'length' images from the list and replaces
+%  them with the specified splice.
 %
-%  The format of the ShiftImageList method is:
+%  The format of the SpliceImageIntoList method is:
 %
-%      Image *ShiftImageList(Image **images)
+%      SpliceImageIntoList(Image **images,const unsigned long length,
+%        Image *splice)
 %
 %  A description of each parameter follows:
 %
 %    o images: The image list.
 %
-%
-*/
-MagickExport Image *ShiftImageList(Image **images)
-{
-  Image
-    *image;
-
-  assert(images != (Image **) NULL);
-  if ((*images) == (Image *) NULL)
-    return((Image *) NULL);
-  assert((*images)->signature == MagickSignature);
-  while ((*images)->previous != (Image *) NULL)
-    (*images)=(*images)->previous;
-  image=(*images);
-  *images=(*images)->next;
-  if ((*images) != (Image *) NULL)
-    (*images)->previous=(Image *) NULL;
-  image->next=(Image *) NULL;
-  return(image);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   S p l i c e I m a g e L i s t                                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  SpliceImageList() removes the images designated by offset and length from
-%  the list and replaces them with the specified list.
-%
-%  The format of the SpliceImageList method is:
-%
-%      Image *SpliceImageList(Image *images,const unsigned long offset,
-%        const unsigned long length,const Image *splices,
-%        ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o images: The image list.
-%
-%    o offset: The position within the list.
-%
-%    o offset: The length of the image list to remove.
+%    o length: The length of the image list to remove.
 %
 %    o splice: Replace the removed image list with this list.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
-%
 %
 */
-MagickExport Image *SpliceImageList(Image *images,const unsigned long offset,
-  const unsigned long length,const Image *splices,ExceptionInfo *exception)
+MagickExport void SpliceImageIntoList(Image **images,const unsigned long length,
+  Image *splice)
 {
   Image
-    *splice;
+    *split;
 
   register long
     i;
-
-  register Image
-    *p,
-    *q;
-
-  if (images == (Image *) NULL)
-    return((Image *) NULL);
-  assert(images->signature == MagickSignature);
-  while (images->previous != (Image *) NULL)
-    images=images->previous;
-  for (i=0; images != (Image *) NULL; images=images->next)
-    if (i++ == (long) offset)
-      break;
-  if (images == (Image *) NULL)
-    return((Image *) NULL);
-  p=images;
-  for (i=1; images != (Image *) NULL; images=images->next)
-    if (i++ == (long) length)
-      break;
-  if (images == (Image *) NULL)
-    return((Image *) NULL);
-  q=images;
-  splice=CloneImageList(splices,exception);
-  if (splice == (Image *) NULL)
-    return((Image *) NULL);
-  if (p->previous != (Image *) NULL)
-    {
-      p->previous->next=splice;
-      if (splice != (Image *) NULL)
-        splice->previous=p->previous;
-    }
-  p->previous=(Image *) NULL;
-  if (q->next != (Image *) NULL)
-    {
-      q->next->previous=splice;
-      if (splice != (Image *) NULL)
-        {
-          while (splice->next != (Image *) NULL)
-            splice=splice->next;
-          splice->next=q->next;
-        }
-    }
-  q->next=(Image *) NULL;
-  return(p);
+  
+  assert(images != (Image **) NULL);
+  assert(splice != (Image *) NULL);
+  assert(splice->signature == MagickSignature);
+  if ((*images) == (Image *) NULL)
+    return;
+  assert((*images)->signature == MagickSignature);
+  split=SplitImageList(*images);
+  if (split == (Image *) NULL)
+    return;
+  AppendImageToList(images,splice);
+  for (i=0; (i < (long) length) && (split != (Image *) NULL); i++)
+    (void) DeleteImageFromList(&split);
+  AppendImageToList(images,split);
 }
 
 /*
@@ -811,54 +781,69 @@ MagickExport Image *SpliceImageList(Image *images,const unsigned long offset,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   U n s h i f t I m a g e L i s t                                           %
+%   S p l i t I m a g e L i s t                                               %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  UnshiftImageList() adds the image to the beginning of the list.
+%  SplitImageList() splits an image into two lists.
 %
-%  The format of the UnshiftImageList method is:
+%  The format of the SplitImageList method is:
 %
-%      unsigned int UnshiftImageList(Image *images,const Image *image,
-%        ExceptionInfo *exception)
+%      Image *SplitImageList(Image *images)
 %
 %  A description of each parameter follows:
 %
 %    o images: The image list.
 %
-%    o image: The image.
 %
-%    o exception: Return any errors or warnings in this structure.
+*/
+MagickExport Image *SplitImageList(Image *images)
+{
+  if ((images == (Image *) NULL) || (images->next == (Image *) NULL))
+    return((Image *) NULL);
+  images=images->next;
+  images->previous->next=(Image *) NULL;
+  images->previous=(Image *) NULL;
+  return(images);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   S y n c N e x t I m a g e I n L i s t                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SyncNextImageInList() returns the next image in the list after the blob
+%  referenced is synchronized with the current image.
+%
+%  The format of the SyncNextImageInList method is:
+%
+%      Image *SyncNextImageInList(const Image *images)
+%
+%  A description of each parameter follows:
+%
+%    o images: The image list.
 %
 %
 */
-MagickExport unsigned int UnshiftImageList(Image **images,const Image *image,
-  ExceptionInfo *exception)
+MagickExport Image *SyncNextImageInList(const Image *images)
 {
-  Image
-    *next;
-
-  assert(images != (Image **) NULL);
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if ((*images) == (Image *) NULL)
+  if (images == (Image *) NULL)
+    return((Image *) NULL);
+  assert(images->signature == MagickSignature);
+  if (images->next == (Image *) NULL)
+    return((Image *) NULL);
+  if (images->blob != images->next->blob)
     {
-      *images=CloneImageList(image,exception);
-      return(*images != (Image *) NULL);
+      DestroyBlobInfo(images->next->blob);
+      images->next->blob=ReferenceBlob(images->blob);
     }
-  assert((*images)->signature == MagickSignature);
-  while ((*images)->previous != (Image *) NULL)
-    (*images)=(*images)->previous;
-  next=CloneImageList(image,exception);
-  if (next == (Image *) NULL)
-    return(False);
-  while (next->next != (Image *) NULL)
-    next=next->next;
-  (*images)->previous=next;
-  (*images)->previous->next=(*images);
-  while ((*images)->previous != (Image *) NULL)
-    (*images)=(*images)->previous;
-  return(True);
+  return(images->next);
 }
