@@ -17,7 +17,7 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -52,8 +52,11 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "magic.h"
+#include "utility.h"
 
-/*  
+/*
   Define declarations.
 */
 #define MagicFilename  "magic.mgk"
@@ -62,7 +65,7 @@
   Declare color map.
 */
 static char
-  *MagicMap =
+  *MagicMap = (char *)
     "<?xml version=\"1.0\"?>"
     "<magicmap>"
     "  <magic stealth=\"True\" />"
@@ -71,17 +74,17 @@ static char
 /*
   Static declarations.
 */
-static MagicInfo
-  *magic_list = (MagicInfo *) NULL;
-
 static SemaphoreInfo
   *magic_semaphore = (SemaphoreInfo *) NULL;
+
+static MagicInfo
+  *magic_list = (MagicInfo *) NULL;
 
 /*
   Forward declarations.
 */
 static unsigned int
-  ReadConfigurationFile(const char *,ExceptionInfo *);
+  ReadConfigureFile(const char *,const unsigned long,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -115,8 +118,8 @@ MagickExport void DestroyMagicInfo(void)
   {
     magic_info=p;
     p=p->next;
-    if (magic_info->filename != (char *) NULL)
-      LiberateMemory((void **) &magic_info->filename);
+    if (magic_info->path != (char *) NULL)
+      LiberateMemory((void **) &magic_info->path);
     if (magic_info->name != (char *) NULL)
       LiberateMemory((void **) &magic_info->name);
     if (magic_info->target != (char *) NULL)
@@ -169,12 +172,15 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   register MagicInfo
     *p;
 
-  AcquireSemaphoreInfo(&magic_semaphore);
   if (magic_list == (MagicInfo *) NULL)
-    (void) ReadConfigurationFile(MagicFilename,exception);
-  LiberateSemaphoreInfo(&magic_semaphore);
+    {
+      AcquireSemaphoreInfo(&magic_semaphore);
+      if (magic_list == (MagicInfo *) NULL)
+        (void) ReadConfigureFile(MagicFilename,0,exception);
+      LiberateSemaphoreInfo(&magic_semaphore);
+    }
   if ((magic == (const unsigned char *) NULL) || (length == 0))
-    return(magic_list);
+    return((const MagicInfo *) magic_list);
   /*
     Search for requested magic.
   */
@@ -198,7 +204,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
         magic_list=p;
       }
   LiberateSemaphoreInfo(&magic_semaphore);
-  return(p);
+  return((const MagicInfo *) p);
 }
 
 /*
@@ -241,11 +247,12 @@ MagickExport unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
   for (p=magic_list; p != (MagicInfo *) NULL; p=p->next)
   {
     if ((p->previous == (MagicInfo *) NULL) ||
-        (LocaleCompare(p->filename,p->previous->filename) != 0))
+        (LocaleCompare(p->path,p->previous->path) != 0))
       {
         if (p->previous != (MagicInfo *) NULL)
           (void) fprintf(file,"\n");
-        (void) fprintf(file,"Filename: %.1024s\n\n",p->filename);
+        if (p->path != (char *) NULL)
+          (void) fprintf(file,"Path: %.1024s\n\n",p->path);
         (void) fprintf(file,"Name      Offset Target\n");
         (void) fprintf(file,"-------------------------------------------------"
           "------------------------------\n");
@@ -270,38 +277,39 @@ MagickExport unsigned int ListMagicInfo(FILE *file,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   R e a d C o n f i g u r a t i o n F i l e                                 %
++   R e a d C o n f i g u r e F i l e                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadConfigurationFile reads the color configuration file which maps
+%  Method ReadConfigureFile reads the color configuration file which maps
 %  color strings with a particular image format.
 %
-%  The format of the ReadConfigurationFile method is:
+%  The format of the ReadConfigureFile method is:
 %
-%      unsigned int ReadConfigurationFile(const char *basename,
-%        ExceptionInfo *exception)
+%      unsigned int ReadConfigureFile(const char *basename,
+%        const unsigned long depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method ReadConfigurationFile returns True if at least one color
+%    o status: Method ReadConfigureFile returns True if at least one color
 %      is defined otherwise False.
 %
 %    o basename:  The color configuration filename.
 %
+%    o depth: depth of <include /> statements.
+%
 %    o exception: Return any errors or warnings in this structure.
 %
 %
 */
-static unsigned int ReadConfigurationFile(const char *basename,
-  ExceptionInfo *exception)
+static unsigned int ReadConfigureFile(const char *basename,
+  const unsigned long depth,ExceptionInfo *exception)
 {
   char
-    filename[MaxTextExtent],
     keyword[MaxTextExtent],
-    *path,
+    path[MaxTextExtent],
     *q,
     *token,
     *xml;
@@ -310,16 +318,13 @@ static unsigned int ReadConfigurationFile(const char *basename,
     length;
 
   /*
-    Read the magic configuration file.
+    Read the magic configure file.
   */
-  FormatString(filename,"%.1024s",basename);
-  path=GetMagickConfigurePath(filename,exception);
-  if (path != (char *) NULL)
-    {
-      FormatString(filename,"%.1024s",path);
-      LiberateMemory((void **) &path);
-    }
-  xml=(char *) FileToBlob(filename,&length,exception);
+  (void) strcpy(path,basename);
+  if (depth == 0)
+    xml=(char *) GetConfigureBlob(basename,path,&length,exception);
+  else
+    xml=(char *) FileToBlob(basename,&length,exception);
   if (xml == (char *) NULL)
     xml=AllocateString(MagicMap);
   token=AllocateString(xml);
@@ -331,33 +336,48 @@ static unsigned int ReadConfigurationFile(const char *basename,
     GetToken(q,&q,token);
     if (*token == '\0')
       break;
-    FormatString(keyword,"%.1024s",token);
-    if (LocaleCompare(keyword,"<!") == 0)
+    (void) strncpy(keyword,token,MaxTextExtent-1);
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
-          Comment.
+          Comment element.
         */
-        while ((*token != '>') && (*q != '\0'))
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
           GetToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
       {
         /*
-          Include.
+          Include element.
         */
         while ((*token != '>') && (*q != '\0'))
         {
-          strncpy(keyword,token,MaxTextExtent-1);
+          (void) strncpy(keyword,token,MaxTextExtent-1);
           GetToken(q,&q,token);
           if (*token != '=')
             continue;
           GetToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              (void) ReadConfigurationFile(token,exception);
-              while (magic_list->next != (MagicInfo *) NULL)
-                magic_list=magic_list->next;
+              if (depth > 200)
+                ThrowException(exception,ConfigureError,
+                  "IncludeElementNestedTooDeeply",path);
+              else
+                {
+                  char
+                    filename[MaxTextExtent];
+
+                  GetPathComponent(path,HeadPath,filename);
+                  if (*filename != '\0')
+                    (void) strcat(filename,DirectorySeparator);
+                  (void) strncat(filename,token,MaxTextExtent-
+                    strlen(filename)-1);
+                  (void) ReadConfigureFile(filename,depth+1,exception);
+                }
+              if (magic_list != (MagicInfo *) NULL)
+                while (magic_list->next != (MagicInfo *) NULL)
+                  magic_list=magic_list->next;
             }
         }
         continue;
@@ -372,10 +392,10 @@ static unsigned int ReadConfigurationFile(const char *basename,
         */
         magic_info=(MagicInfo *) AcquireMemory(sizeof(MagicInfo));
         if (magic_info == (MagicInfo *) NULL)
-          MagickFatalError(ResourceLimitFatalError,"Unable to allocate magics",
-            "Memory allocation failed");
+          MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+            "UnableToAllocateMagicInfo");
         (void) memset(magic_info,0,sizeof(MagicInfo));
-        magic_info->filename=AcquireString(filename);
+        magic_info->path=AcquireString(path);
         magic_info->signature=MagickSignature;
         if (magic_list == (MagicInfo *) NULL)
           {
@@ -394,7 +414,7 @@ static unsigned int ReadConfigurationFile(const char *basename,
       continue;
     GetToken(q,&q,token);
     GetToken(q,&q,token);
-    switch (*keyword) 
+    switch (*keyword)
     {
       case 'N':
       case 'n':
