@@ -19,7 +19,7 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,15 +54,28 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "cache.h"
+#include "delegate.h"
+#include "log.h"
+#include "magic.h"
+#include "magick.h"
+#include "module.h"
+#include "registry.h"
+#include "resource.h"
+#include "render.h"
+#include "signature.h"
+#include "utility.h"
 
 /*
   Global declarations.
 */
+static SemaphoreInfo
+  *magick_semaphore = (SemaphoreInfo *) NULL;
+
 static MagickInfo
   *magick_list = (MagickInfo *) NULL;
 
-static SemaphoreInfo
-  *magick_semaphore = (SemaphoreInfo *) NULL;
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -91,10 +104,16 @@ MagickExport void DestroyMagick(void)
   DestroyModuleInfo();
   DestroyMagicInfo();
   DestroyMagickInfo();
+  DestroyLogInfo();
   DestroyConstitute();
   DestroyMagickRegistry();
-  DestroyCache();
+  DestroyMagickResources();
+  DestroyRandomReservoir();
   DestroySemaphore();
+#if defined(WIN32)
+  DestroyTracingCriticalSection();
+  NTGhostscriptUnLoadDLL();
+#endif
 }
 
 /*
@@ -134,6 +153,8 @@ MagickExport void DestroyMagickInfo(void)
       LiberateMemory((void **) &magick_info->description);
     if (magick_info->version != (char *) NULL)
       LiberateMemory((void **) &magick_info->version);
+    if (magick_info->note != (char *) NULL)
+      LiberateMemory((void **) &magick_info->note);
     if (magick_info->module != (char *) NULL)
       LiberateMemory((void **) &magick_info->module);
     LiberateMemory((void **) &magick_info);
@@ -153,9 +174,8 @@ MagickExport void DestroyMagickInfo(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetImageMagick() searches for an image format that matches the
-%  specified magick string.  If one is found the name is returned otherwise
-%  NULL.
+%  GetImageMagick() searches for an image format that matches the specified
+%  magick string.  If one is found the name is returned otherwise NULL.
 %
 %  The format of the GetImageMagick method is:
 %
@@ -192,155 +212,6 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e t M a g i c k C o n f i g u r e P a t h                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  GetMagickConfigurePath() searches a number of pre-defined locations
-%  for the specified ImageMagick configuration file and returns the path.
-%  The search order follows:
-%
-%             <current directory>/
-%             <client path>/
-%             $MAGICK_HOME/
-%             $HOME/.magick/
-%             $MAGICK_FONT_PATH
-%             MagickLibPath
-%             MagickModulesPath
-%             MagickSharePath
-%             Windows Resource
-%
-%  The format of the GetMagickConfigurePath method is:
-%
-%      char *GetMagickConfigurePath(const char *filename,
-%        ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o path:  Method GetMagickConfigurePath returns the path if the
-%      configuration file is found, otherwise NULL is returned.
-%
-%    o filename: A character string representing the desired configuration
-%      file.
-%
-%    o exception: Return any errors or warnings in this structure.
-%
-%
-*/
-MagickExport char *GetMagickConfigurePath(const char *filename,
-  ExceptionInfo *exception)
-{
-  char
-    *path,
-    *search_path;
-
-  assert(filename != (const char *) NULL);
-  assert(exception != (ExceptionInfo *) NULL);
-  path=AllocateString(filename);
-  if (IsAccessible(path))
-    return(path);
-  search_path=AllocateString(path);
-  FormatString(path,"%.1024s%s%.1024s",SetClientPath((char *) NULL),
-    DirectorySeparator,filename);
-  if (IsAccessible(path))
-    {
-      LiberateMemory((void **) &search_path);
-      return(path);
-    }
-  ConcatenateString(&search_path,"; Client Path:");
-  ConcatenateString(&search_path,path);
-  if (getenv("MAGICK_HOME") != (char *) NULL)
-    {
-      FormatString(path,"%.1024s%s%.1024s",getenv("MAGICK_HOME"),
-        DirectorySeparator,filename);
-      if (IsAccessible(path))
-        {
-          LiberateMemory((void **) &search_path);
-          return(path);
-        }
-      ConcatenateString(&search_path,"; MAGICK_HOME:");
-      ConcatenateString(&search_path,path);
-    }
-  if (getenv("HOME") != (char *) NULL)
-    {
-      FormatString(path,"%.1024s%s%s%s%.1024s",getenv("HOME"),
-        *getenv("HOME") == '/' ? "/.magick" : "",DirectorySeparator,
-        DirectorySeparator,filename);
-      if (IsAccessible(path))
-        {
-          LiberateMemory((void **) &search_path);
-          return(path);
-        }
-      ConcatenateString(&search_path,"; HOME:");
-      ConcatenateString(&search_path,path);
-    }
-  if (getenv("MAGICK_FONT_PATH") != (char *) NULL)
-    {
-      FormatString(path,"%.1024s%s%.1024s",getenv("MAGICK_FONT_PATH"),
-        DirectorySeparator,filename);
-      if (IsAccessible(path))
-        {
-          LiberateMemory((void **) &search_path);
-          return(path);
-        }
-      ConcatenateString(&search_path,"; MAGICK_FONT_PATH:");
-      ConcatenateString(&search_path,path);
-    }
-  FormatString(path,"%.1024s%.1024s",MagickLibPath,filename);
-  if (IsAccessible(path))
-    {
-      LiberateMemory((void **) &search_path);
-      return(path);
-    }
-  ConcatenateString(&search_path,"; MagickLibPath:");
-  ConcatenateString(&search_path,path);
-  FormatString(path,"%.1024s%.1024s",MagickModulesPath,filename);
-  if (IsAccessible(path))
-    {
-      LiberateMemory((void **) &search_path);
-      return(path);
-    }
-  ConcatenateString(&search_path,"; MagickModulesPath:");
-  ConcatenateString(&search_path,path);
-  FormatString(path,"%.1024s%.1024s",MagickSharePath,filename);
-  if (IsAccessible(path))
-    {
-      LiberateMemory((void **) &search_path);
-      return(path);
-    }
-  ConcatenateString(&search_path,"; MagickSharePath:");
-  ConcatenateString(&search_path,path);
-#if defined(WIN32)
-  {
-    void
-      *blob;
-
-    FormatString(path,"%.1024s",filename);
-    blob=NTResourceToBlob(path);
-    if (blob != (unsigned char *) NULL)
-      {
-        LiberateMemory((void **) &blob);
-        LiberateMemory((void **) &search_path);
-        return(path);
-      }
-    ConcatenateString(&search_path,"; Windows Registry:");
-    ConcatenateString(&search_path,path);
-  }
-#endif
-  ThrowException(exception,ConfigurationError,
-    "Unable to open configuration file",search_path);
-  LiberateMemory((void **) &search_path);
-  LiberateMemory((void **) &path);
-  return((char *) NULL);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   G e t M a g i c k I n f o                                                 %
 %                                                                             %
 %                                                                             %
@@ -389,7 +260,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
       (void) GetModuleInfo((char *) NULL,exception);
     }
   if ((name == (const char *) NULL) ||  (LocaleCompare(name,"*") == 0))
-    return(magick_list);
+    return((const MagickInfo *) magick_list);
   /*
     Find name in list
   */
@@ -399,17 +270,17 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
       break;
   if (p == (MagickInfo *) NULL)
     {
-      LiberateSemaphoreInfo(&magick_semaphore);
       if (*name != '\0')
         {
           ExceptionInfo
             module_exception;
 
+          LiberateSemaphoreInfo(&magick_semaphore);
           GetExceptionInfo(&module_exception);
           (void) OpenModule(name,&module_exception);
           DestroyExceptionInfo(&module_exception);
+          AcquireSemaphoreInfo(&magick_semaphore);
         }
-      AcquireSemaphoreInfo(&magick_semaphore);
       for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
         if (LocaleCompare(p->name,name) == 0)
           break;
@@ -430,37 +301,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
         magick_list=p;
       }
   LiberateSemaphoreInfo(&magick_semaphore);
-  return(p);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   G e t M a g i c k V e r s i o n                                           %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  GetMagickVersion() returns the ImageMagick API version as a string and
-%  as a number.
-%
-%  The format of the GetMagickVersion method is:
-%
-%      const char *GetMagickVersion(unsigned int *version)
-%
-%  A description of each parameter follows:
-%
-%    o version: The ImageMagick version is returned as a number.
-%
-*/
-MagickExport const char *GetMagickVersion(unsigned int *version)
-{
-  assert(version != (unsigned int *) NULL);
-  *version=MagickLibVersion;
-  return(MagickVersion);
+  return((const MagickInfo *) p);
 }
 
 /*
@@ -489,43 +330,78 @@ MagickExport const char *GetMagickVersion(unsigned int *version)
 MagickExport void InitializeMagick(const char *path)
 {
   char
-    directory[MaxTextExtent],
-    filename[MaxTextExtent];
+    execution_path[MaxTextExtent];
 
-  InitializeSemaphore();
+  /*
+    Initialize the Magick environment.
+  */
   (void) setlocale(LC_ALL,"");
   (void) setlocale(LC_NUMERIC,"C");
-  (void) getcwd(directory,MaxTextExtent);
-  (void) SetClientPath(directory);
-  if (path != (const char *) NULL)
+  InitializeSemaphore();
+  InitializeMagickResources();
+  (void) SetLogEventMask(getenv("MAGICK_DEBUG"));
+  *execution_path='\0';
+#if !defined(UseInstalledImageMagick)
+#if defined(POSIX) || defined(WIN32)
+  if ((path == (const char *) NULL) ||
+      (strchr(path,*DirectorySeparator) == (char *) NULL) || (strlen(path) <= 2))
+    (void) GetExecutionPath(execution_path);
+  else
     {
-      GetPathComponent(path,HeadPath,filename);
+#if defined(POSIX)
+      if (*path == *DirectorySeparator)
+#elif defined(WIN32)
+      if (((*path == *DirectorySeparator) &&
+           (*(path+1) == *DirectorySeparator)) || (*(path+1) == ':'))
+#else
+      if (1)
+#endif
+        (void) strncpy(execution_path,path,MaxTextExtent-1);
+      else
+        {
+          (void) getcwd(execution_path,MaxTextExtent-2);
+          (void) strcat(execution_path,DirectorySeparator);
+          if((*path == '.') && (*(path+1) == *DirectorySeparator))
+            (void) strncat(execution_path,path+2,MaxTextExtent-
+              strlen(execution_path)-1);
+          else
+            (void) strncat(execution_path,path,MaxTextExtent-
+              strlen(execution_path)-1);
+        }
+    }
+#endif
+#endif
+  if (*execution_path == '\0')
+    {
+      if (path != (char *) NULL)
+        (void) SetClientName(path);
+    }
+  else
+    {
+      char
+        filename[MaxTextExtent];
+
+      GetPathComponent(execution_path,HeadPath,filename);
       (void) SetClientPath(filename);
-      GetPathComponent(path,BasePath,filename);
+      GetPathComponent(execution_path,TailPath,filename);
       (void) SetClientName(filename);
     }
 #if defined(WIN32)
-  {
-    char
-      *execution_path;
-
+    InitializeTracingCriticalSection();
 #if defined(_DEBUG)
-    int
-      debug;
+    if (IsEventLogging())
+      {
+        int
+          debug;
 
-    debug=_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
-    debug|=_CRTDBG_CHECK_ALWAYS_DF;
-    debug|=_CRTDBG_DELAY_FREE_MEM_DF;
-    debug|=_CRTDBG_LEAK_CHECK_DF;
-    // debug=_CrtSetDbgFlag(debug);
+        debug=_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
+        debug|=_CRTDBG_CHECK_ALWAYS_DF;
+        debug|=_CRTDBG_DELAY_FREE_MEM_DF;
+        debug|=_CRTDBG_LEAK_CHECK_DF;
+        // debug=_CrtSetDbgFlag(debug);
+        // _ASSERTE(_CrtCheckMemory());
+      }
 #endif
-    execution_path=NTGetExecutionPath();
-    GetPathComponent(execution_path,HeadPath,filename);
-    (void) SetClientPath(filename);
-    GetPathComponent(execution_path,TailPath,filename);
-    (void) SetClientName(filename);
-    LiberateMemory((void **) &execution_path);
-  }
 #endif
 }
 
@@ -565,7 +441,7 @@ MagickExport int unsigned IsMagickConflict(const char *magick)
 #if defined(vms)
   return(VMSIsMagickConflict(magick));
 #endif
-#if defined(WIN32)
+#if defined(WIN32) || defined(__CYGWIN__)
   return(NTIsMagickConflict(magick));
 #endif
   return(False);
@@ -598,31 +474,33 @@ MagickExport int unsigned IsMagickConflict(const char *magick)
 */
 MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
 {
-  char
-    filename[MaxTextExtent],
-    *module_file,
-    *path;
-
-  ExceptionInfo
-    path_exception;
-
   register const MagickInfo
     *p;
 
   if (file == (FILE *) NULL)
     file=stdout;
   (void) GetMagickInfo("*",exception);
-  module_file=TagToModule("MIFF");
-  GetExceptionInfo(&path_exception);
-  path=GetMagickConfigurePath(module_file,&path_exception);
-  DestroyExceptionInfo(&path_exception);
-  if (path != (char *) NULL)
-    {
-      GetPathComponent(path,HeadPath,filename);
-      (void) fprintf(file,"Module path: %.1024s\n\n",filename);
-      LiberateMemory((void **) &path);
-    }
-  LiberateMemory((void **) &module_file);
+#if defined(HasMODULES)
+  {
+    char
+      *module_file,
+      path[MaxTextExtent];
+
+    size_t
+      length;
+
+    void
+      *blob;
+
+    module_file=TagToModule("MIFF");
+    blob=GetModuleBlob(module_file,path,&length,exception);
+    if (blob != (void *) NULL)
+      LiberateMemory((void **) &blob);
+    LiberateMemory((void **) &module_file);
+    GetPathComponent(path,HeadPath,path);
+    (void) fprintf(file,"Path: %.1024s\n\n",path);
+  }
+#endif
   (void) fprintf(file,"   Format  Mode  Description\n");
   (void) fprintf(file,"--------------------------------------------------------"
     "-----------------------\n");
@@ -639,6 +517,25 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
     if (p->version != (char *) NULL)
       (void) fprintf(file," (%.1024s)",p->version);
     (void) fprintf(file,"\n");
+    if (p->note != (char *) NULL)
+      {
+        char
+          **text;
+
+        text=StringToList(p->note);
+        if (text != (char **) NULL)
+          {
+            register long
+              i;
+
+            for (i=0; text[i] != (char *) NULL; i++)
+	    			{
+              (void) fprintf(file,"            %.1024s\n",text[i]);
+              LiberateMemory((void **) &text[i]);
+    				}
+            LiberateMemory((void **) &text);
+          }
+      }
   }
   (void) fprintf(file,"\n* native blob support\n\n");
   (void) fflush(file);
@@ -846,8 +743,8 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
   assert(name != (const char *) NULL);
   magick_info=(MagickInfo *) AcquireMemory(sizeof(MagickInfo));
   if (magick_info == (MagickInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"Unable to allocate image",
-      "Memory allocation failed");
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateMagickInfo");
   (void) memset(magick_info,0,sizeof(MagickInfo));
   magick_info->name=AcquireString(name);
   magick_info->adjoin=True;
@@ -887,6 +784,9 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 MagickExport unsigned int UnregisterMagickInfo(const char *name)
 {
   MagickInfo
+    *magick_info;
+
+  register MagickInfo
     *p;
 
   unsigned int
@@ -905,14 +805,17 @@ MagickExport unsigned int UnregisterMagickInfo(const char *name)
       p->previous->next=p->next;
     else
       magick_list=p->next;
-    LiberateMemory((void **) &p->name);
-    if (p->description != (char *) NULL)
-      LiberateMemory((void **) &p->description);
-    if (p->version != (char *) NULL)
-      LiberateMemory((void **) &p->version);
-    if (p->module != (char *) NULL)
-      LiberateMemory((void **) &p->module);
-    LiberateMemory((void **) &p);
+    magick_info=p;
+    LiberateMemory((void **) &magick_info->name);
+    if (magick_info->description != (char *) NULL)
+      LiberateMemory((void **) &magick_info->description);
+    if (magick_info->version != (char *) NULL)
+      LiberateMemory((void **) &magick_info->version);
+    if (magick_info->note != (char *) NULL)
+      LiberateMemory((void **) &magick_info->note);
+    if (magick_info->module != (char *) NULL)
+      LiberateMemory((void **) &magick_info->module);
+    LiberateMemory((void **) &magick_info);
     status=True;
     break;
   }
