@@ -8,23 +8,33 @@
 extern "C" {
 #endif
 
+typedef Image
+  *(*DecoderHandler)(const ImageInfo *,ExceptionInfo *);
+
+typedef unsigned int
+  (*EncoderHandler)(const ImageInfo *,Image *),
+  (*MagickHandler)(const unsigned char *,const size_t);
+
 typedef struct _MagickInfo
 {
   const char
     *name,
     *description,
     *version,
+    *note,
     *module;
 
   ImageInfo
     *image_info;
 
-  Image
-    *(*decoder)(const ImageInfo *,ExceptionInfo *);
+  DecoderHandler
+    decoder;
 
-  unsigned int
-    (*encoder)(const ImageInfo *,Image *),
-    (*magick)(const unsigned char *,const size_t);
+  EncoderHandler
+    encoder;
+
+  MagickHandler
+    magick;
 
   void
     *client_data;
@@ -33,6 +43,7 @@ typedef struct _MagickInfo
     adjoin,
     raw,
     stealth,
+    seekable_stream,
     blob_support,
     thread_support;
 
