@@ -18,7 +18,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,8 +54,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "magic.h"
+#include "magick.h"
+#include "module.h"
+#include "utility.h"
 #if defined(HasLTDL)
-#  include "ltdl/ltdl.h"
+#  include "ltdl.h"
    typedef lt_dlhandle ModuleHandle;
 #else
    typedef void *ModuleHandle;
@@ -79,7 +84,7 @@
   Declare module map.
 */
 static char
-  *ModuleMap =
+  *ModuleMap = (char *)
     "<?xml version=\"1.0\"?>"
     "<modulemap>"
     "  <module />"
@@ -88,14 +93,14 @@ static char
 /*
   Global declarations.
 */
+static SemaphoreInfo
+  *module_semaphore = (SemaphoreInfo *) NULL;
+
 static CoderInfo
   *coder_list = (CoderInfo *) NULL;
 
 static ModuleInfo
   *module_list = (ModuleInfo *) NULL;
-
-static SemaphoreInfo
-  *module_semaphore = (SemaphoreInfo *) NULL;
 
 /*
   Forward declarations.
@@ -108,7 +113,7 @@ static CoderInfo
   *SetCoderInfo(const char *);
 
 static unsigned int
-  ReadConfigurationFile(const char *,ExceptionInfo *),
+  ReadConfigureFile(const char *,const unsigned long,ExceptionInfo *),
   UnloadModule(const CoderInfo *,ExceptionInfo *),
   UnregisterModule(const char *,ExceptionInfo *);
 
@@ -154,7 +159,7 @@ static void lt_dlclose(void *handle)
 
 static const char *lt_dlerror(void)
 {
-  return((const char *) NULL);
+  return("");
 }
 
 static void *lt_dlsym(void *handle,char *symbol)
@@ -198,8 +203,8 @@ MagickExport void DestroyModuleInfo(void)
   {
     module_info=q;
     q=q->next;
-    if (module_info->filename != (char *) NULL)
-      LiberateMemory((void **) &module_info->filename);
+    if (module_info->path != (char *) NULL)
+      LiberateMemory((void **) &module_info->path);
     if (module_info->magick != (char *) NULL)
       LiberateMemory((void **) &module_info->magick);
     if (module_info->name != (char *) NULL)
@@ -275,10 +280,9 @@ MagickExport unsigned int ExecuteModuleProcess(const char *tag,Image **image,
           char
             message[MaxTextExtent];
 
-          FormatString(message,"failed to load module \"%.1024s\"",
-            module_name);
-          ThrowException(&(*image)->exception,MissingDelegateError,message,
-            lt_dlerror());
+          FormatString(message,"\"%.1024s: %.1024s\"",module_name,lt_dlerror());
+          ThrowException(&(*image)->exception,ModuleError,"UnableToLoadModule",
+            message);
           LiberateMemory((void **) &module_name);
           return(status);
         }
@@ -338,12 +342,12 @@ MagickExport const CoderInfo *GetCoderInfo(const char *tag,
   (void) GetMagicInfo((unsigned char *) NULL,0,exception);
   (void) GetModuleInfo(tag,exception);
   if ((tag == (const char *) NULL) || (LocaleCompare(tag,"*") == 0))
-    return(coder_list);
+    return((const CoderInfo *) coder_list);
   for (p=coder_list; p != (CoderInfo *) NULL; p=p->next)
     if (LocaleCompare(p->tag,tag) == 0)
       break;
   if (p == (CoderInfo *) NULL)
-    ThrowException(exception,OptionWarning,"Unrecognized module",tag);
+    ThrowException(exception,ModuleError,"UnrecognizedModule",tag);
   else
     if (p != coder_list)
       {
@@ -359,7 +363,7 @@ MagickExport const CoderInfo *GetCoderInfo(const char *tag,
         coder_list->previous=p;
         coder_list=p;
       }
-  return(p);
+  return((const CoderInfo *) p);
 }
 
 /*
@@ -391,9 +395,8 @@ MagickExport const CoderInfo *GetCoderInfo(const char *tag,
 static char **GetModuleList(ExceptionInfo *exception)
 {
   char
-    filename[MaxTextExtent],
     **modules,
-    *path;
+    path[MaxTextExtent];
 
   DIR
     *directory;
@@ -412,12 +415,8 @@ static char **GetModuleList(ExceptionInfo *exception)
   if (modules == (char **) NULL)
     return((char **) NULL);
   *modules=(char *) NULL;
-  path=GetMagickConfigurePath(ModuleFilename,exception);
-  if (path == (char *) NULL)
-    return((char **) NULL);
-  GetPathComponent(path,HeadPath,filename);
-  LiberateMemory((void **) &path);
-  directory=opendir(filename);
+  GetPathComponent(module_list->path,HeadPath,path);
+  directory=opendir(path);
   if (directory == (DIR *) NULL)
     return((char **) NULL);
   i=0;
@@ -494,21 +493,24 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *name,
   register ModuleInfo
     *p;
 
-  AcquireSemaphoreInfo(&module_semaphore);
   if (module_list == (const ModuleInfo *) NULL)
     {
-      /*
-        Read modules.
-      */
-      if (lt_dlinit() != 0)
-        MagickFatalError(DelegateFatalError,
-          "Unable to initialize module loader",lt_dlerror());
-      OpenStaticModules();
-      (void) ReadConfigurationFile(ModuleFilename,exception);
+      AcquireSemaphoreInfo(&module_semaphore);
+      if (module_list == (const ModuleInfo *) NULL)
+        {
+          /*
+            Read modules.
+          */
+          if (lt_dlinit() != 0)
+            MagickFatalError(ModuleFatalError,"UnableToInitializeModuleLoader",
+              lt_dlerror());
+          RegisterStaticModules();
+          (void) ReadConfigureFile(ModuleFilename,0,exception);
+        }
+      LiberateSemaphoreInfo(&module_semaphore);
     }
-  LiberateSemaphoreInfo(&module_semaphore);
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    return(module_list);
+    return((const ModuleInfo *) module_list);
   AcquireSemaphoreInfo(&module_semaphore);
   for (p=module_list; p != (ModuleInfo *) NULL; p=p->next)
     if (LocaleCompare(p->name,name) == 0)
@@ -528,7 +530,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *name,
       module_list=p;
     }
   LiberateSemaphoreInfo(&module_semaphore);
-  return(p);
+  return((const ModuleInfo *) p);
 }
 
 /*
@@ -571,11 +573,12 @@ MagickExport unsigned int ListModuleInfo(FILE *file,ExceptionInfo *exception)
   for (p=module_list; p != (ModuleInfo *) NULL; p=p->next)
   {
     if ((p->previous == (ModuleInfo *) NULL) ||
-        (LocaleCompare(p->filename,p->previous->filename) != 0))
+        (LocaleCompare(p->path,p->previous->path) != 0))
       {
         if (p->previous != (ModuleInfo *) NULL)
           (void) fprintf(file,"\n");
-        (void) fprintf(file,"Filename: %.1024s\n\n",p->filename);
+        if (p->path != (char *) NULL)
+          (void) fprintf(file,"Path: %.1024s\n\n",p->path);
         (void) fprintf(file,"Magick      Module\n");
         (void) fprintf(file,"-------------------------------------------------"
           "------------------------------\n");
@@ -667,81 +670,87 @@ static void ModuleToTag(const char *filename,const char *format,char *module)
 MagickExport unsigned int OpenModule(const char *module,
   ExceptionInfo *exception)
 {
-  char
-    message[MaxTextExtent],
-    *module_file,
-    module_name[MaxTextExtent],
-    name[MaxTextExtent],
-    *path;
+#if defined(HasMODULES)
+  {
+    char
+      message[MaxTextExtent],
+      *module_file,
+      module_name[MaxTextExtent],
+      name[MaxTextExtent],
+      path[MaxTextExtent];
 
-  CoderInfo
-    *coder_info;
+    CoderInfo
+      *coder_info;
 
-  ModuleHandle
-    handle;
+    ModuleHandle
+      handle;
 
-  register ModuleInfo
-    *p;
+    register ModuleInfo
+      *p;
 
-  void
-    (*method)(void);
+    void
+      (*method)(void);
 
-  /*
-    Assign module name from alias.
-  */
-  assert(module != (const char *) NULL);
-  (void) strncpy(module_name,module,MaxTextExtent-1);
-  if (module_list != (ModuleInfo *) NULL)
-    for (p=module_list; p != (ModuleInfo *) NULL; p=p->next)
-      if (LocaleCompare(p->magick,module) == 0)
-        {
-          (void) strncpy(module_name,p->name,MaxTextExtent-1);
-          break;
-        }
-  /*
-    Load module file.
-  */
-  handle=(ModuleHandle) NULL;
-  module_file=TagToModule(module_name);
-  path=GetMagickConfigurePath(module_file,exception);
-  if (path != (char *) NULL)
-    {
-      handle=lt_dlopen(path);
-      if (handle == (ModuleHandle) NULL)
-        {
-          FormatString(message,"failed to load module \"%.1024s\"",path);
-          ThrowException(exception,MissingDelegateError,message,lt_dlerror());
-        }
-      LiberateMemory((void **) &path);
-    }
-  LiberateMemory((void **) &module_file);
-  if (handle == (ModuleHandle) NULL)
-    return(False);
-  /*
-    Add module to module list.
-  */
-  coder_info=SetCoderInfo(module_name);
-  if (coder_info == (CoderInfo*) NULL)
-    {
-      (void) lt_dlclose(handle);
+    /*
+      Assign module name from alias.
+    */
+    assert(module != (const char *) NULL);
+    (void) strncpy(module_name,module,MaxTextExtent-1);
+    if (module_list != (ModuleInfo *) NULL)
+      for (p=module_list; p != (ModuleInfo *) NULL; p=p->next)
+        if (LocaleCompare(p->magick,module) == 0)
+          {
+            (void) strncpy(module_name,p->name,MaxTextExtent-1);
+            break;
+          }
+    /*
+      Load module file.
+    */
+    handle=(ModuleHandle) NULL;
+    module_file=TagToModule(module_name);
+    *path='\0';
+    if ((module_list != (ModuleInfo *) NULL) &&
+        (module_list->path != (char *) NULL))
+      GetPathComponent(module_list->path,HeadPath,path);
+    (void) strcat(path,DirectorySeparator);
+    (void) strncat(path,module_file,MaxTextExtent-strlen(path)-1);
+    handle=lt_dlopen(path);
+    if (handle == (ModuleHandle) NULL)
+      {
+        FormatString(message,"\"%.1024s: %.1024s\"",path,lt_dlerror());
+        ThrowException(exception,ModuleError,"UnableToLoadModule",message);
+      }
+    LiberateMemory((void **) &module_file);
+    if (handle == (ModuleHandle) NULL)
       return(False);
-    }
-  coder_info->handle=handle;
-  (void) time(&coder_info->load_time);
-  if (!RegisterModule(coder_info,exception))
-    return(False);
-  /*
-    Locate and execute RegisterFORMATImage function
-  */
-  ModuleToTag(module_name,"Register%sImage",name);
-  method=(void (*)(void)) lt_dlsym(handle,name);
-  if (method == (void (*)(void)) NULL)
-    {
-      ThrowException(exception,MissingDelegateError,"failed to find symbol",
-        lt_dlerror());
+    /*
+      Add module to module list.
+    */
+    coder_info=SetCoderInfo(module_name);
+    if (coder_info == (CoderInfo*) NULL)
+      {
+        (void) lt_dlclose(handle);
+        return(False);
+      }
+    coder_info->handle=handle;
+    (void) time(&coder_info->load_time);
+    if (!RegisterModule(coder_info,exception))
       return(False);
-    }
-  method();
+    /*
+      Locate and execute RegisterFORMATImage function
+    */
+    ModuleToTag(module_name,"Register%sImage",name);
+    method=(void (*)(void)) lt_dlsym(handle,name);
+    if (method == (void (*)(void)) NULL)
+      {
+        FormatString(message,"\"%.1024s: %.1024s\"",module_name,lt_dlerror());
+        ThrowException(exception,ModuleError,"UnableToRegisterImageFormat",
+          message);
+        return(False);
+      }
+    method();
+  }
+#endif
   return(True);
 }
 
@@ -808,39 +817,40 @@ MagickExport unsigned int OpenModules(ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   R e a d C o n f i g u r a t i o n F i l e                                 %
++   R e a d C o n f i g u r e F i l e                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadConfigurationFile reads the color configuration file which maps
+%  Method ReadConfigureFile reads the color configuration file which maps
 %  color strings with a particular image format.
 %
-%  The format of the ReadConfigurationFile method is:
+%  The format of the ReadConfigureFile method is:
 %
-%      unsigned int ReadConfigurationFile(const char *basename,
-%        ExceptionInfo *exception)
+%      unsigned int ReadConfigureFile(const char *basename,
+%        const unsigned long depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o status: ReadConfigurationFile() returns True if at least one color is
+%    o status: ReadConfigureFile() returns True if at least one color is
 %      defined otherwise False.
 %
 %    o basename:  The color configuration filename.
 %
+%    o depth: depth of <include /> statements.
+%
 %    o exception: Return any errors or warnings in this structure.
 %
 %
 */
-static unsigned int ReadConfigurationFile(const char *basename,
-  ExceptionInfo *exception)
+static unsigned int ReadConfigureFile(const char *basename,
+  const unsigned long depth,ExceptionInfo *exception)
 {
 #if defined(HasMODULES)
   char
-    filename[MaxTextExtent],
     keyword[MaxTextExtent],
-    *path,
+    path[MaxTextExtent],
     *q,
     *token,
     *xml;
@@ -849,16 +859,13 @@ static unsigned int ReadConfigurationFile(const char *basename,
     length;
 
   /*
-    Read the module configuration file.
+    Read the module configure file.
   */
-  FormatString(filename,"%.1024s",basename);
-  path=GetMagickConfigurePath(filename,exception);
-  if (path != (char *) NULL)
-    {
-      FormatString(filename,"%.1024s",path);
-      LiberateMemory((void **) &path);
-    }
-  xml=(char *) FileToBlob(filename,&length,exception);
+  (void) strcpy(path,basename);
+  if (depth == 0)
+    xml=(char *) GetModuleBlob(basename,path,&length,exception);
+  else
+    xml=(char *) FileToBlob(basename,&length,exception);
   if (xml == (char *) NULL)
     xml=AllocateString(ModuleMap);
   token=AllocateString(xml);
@@ -870,20 +877,20 @@ static unsigned int ReadConfigurationFile(const char *basename,
     GetToken(q,&q,token);
     if (*token == '\0')
       break;
-    FormatString(keyword,"%.1024s",token);
-    if (LocaleCompare(keyword,"<!") == 0)
+    (void) strncpy(keyword,token,MaxTextExtent-1);
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
-          Comment.
+          Comment element.
         */
-        while ((*token != '>') && (*q != '\0'))
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
           GetToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
       {
         /*
-          Include.
+          Include element.
         */
         while ((*token != '>') && (*q != '\0'))
         {
@@ -894,9 +901,24 @@ static unsigned int ReadConfigurationFile(const char *basename,
           GetToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              (void) ReadConfigurationFile(token,exception);
-              while (module_list->next != (ModuleInfo *) NULL)
-                module_list=module_list->next;
+              if (depth > 200)
+                ThrowException(exception,ConfigureError,
+                  "IncludeElementNestedTooDeeply",path);
+              else
+                {
+                  char
+                    filename[MaxTextExtent];
+
+                  GetPathComponent(path,HeadPath,filename);
+                  if (*filename != '\0')
+                    (void) strcat(filename,DirectorySeparator);
+                  (void) strncat(filename,token,MaxTextExtent-
+                    strlen(filename)-1);
+                  (void) ReadConfigureFile(filename,depth+1,exception);
+                }
+              if (module_list != (ModuleInfo *) NULL)
+                while (module_list->next != (ModuleInfo *) NULL)
+                  module_list=module_list->next;
             }
         }
         continue;
@@ -911,10 +933,10 @@ static unsigned int ReadConfigurationFile(const char *basename,
         */
         module_info=(ModuleInfo *) AcquireMemory(sizeof(ModuleInfo));
         if (module_info == (ModuleInfo *) NULL)
-          MagickFatalError(ResourceLimitFatalError,
-            "Unable to allocate module magick","Memory allocation failed");
+          MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+            "UnableToAllocateModuleInfo");
         (void) memset(module_info,0,sizeof(ModuleInfo));
-        module_info->filename=AcquireString(filename);
+        module_info->path=AcquireString(path);
         module_info->signature=MagickSignature;
         if (module_list == (ModuleInfo *) NULL)
           {
@@ -933,7 +955,7 @@ static unsigned int ReadConfigurationFile(const char *basename,
       continue;
     GetToken(q,&q,token);
     GetToken(q,&q,token);
-    switch (*keyword) 
+    switch (*keyword)
     {
       case 'M':
       case 'm':
@@ -1018,7 +1040,6 @@ static CoderInfo *RegisterModule(CoderInfo *entry,ExceptionInfo *exception)
   assert(entry != (CoderInfo *) NULL);
   assert(entry->signature == MagickSignature);
   AcquireSemaphoreInfo(&module_semaphore);
-  (void) UnregisterModule(entry->tag,exception);
   entry->previous=(CoderInfo *) NULL;
   entry->next=(CoderInfo *) NULL;
   if (coder_list == (CoderInfo *) NULL)
@@ -1036,6 +1057,14 @@ static CoderInfo *RegisterModule(CoderInfo *entry,ExceptionInfo *exception)
   for (p=coder_list; p->next != (CoderInfo *) NULL; p=p->next)
     if (LocaleCompare(p->tag,entry->tag) >= 0)
       break;
+  if (LocaleCompare(p->tag,entry->tag) == 0)
+    {
+      /*
+        Module already registered.
+      */
+      LiberateSemaphoreInfo(&module_semaphore);
+      return(entry);
+    }
   if (LocaleCompare(p->tag,entry->tag) < 0)
     {
       /*
@@ -1099,8 +1128,8 @@ static CoderInfo *SetCoderInfo(const char *tag)
   assert(tag != (const char *) NULL);
   entry=(CoderInfo *) AcquireMemory(sizeof(CoderInfo));
   if (entry == (CoderInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"Unable to allocate module info",
-      "Memory allocation failed");
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateModuleInfo");
   (void) memset(entry,0,sizeof(CoderInfo));
   entry->tag=AcquireString(tag);
   entry->signature=MagickSignature;
@@ -1112,6 +1141,57 @@ static CoderInfo *SetCoderInfo(const char *tag)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%  T a g T o M o d u l e                                                      %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TagToModule() takes a module "name" and returnes a complete file system
+%  dynamic module name.
+%
+%  The format of the TagToModule method is:
+%
+%      char *TagToModule(const char *tag)
+%
+%  A description of each parameter follows:
+%
+%    o tag: a character string that represents the name of the particular
+%      module.
+%
+*/
+MagickExport char *TagToModule(const char *tag)
+{
+  char
+    *module_name;
+
+  assert(tag != (char *) NULL);
+  module_name=AllocateString(tag);
+#if defined(HasLTDL)
+  (void) FormatString(module_name,"%.1024s.la",tag);
+  (void) LocaleLower(module_name);
+#else
+#if defined(WIN32)
+  if (LocaleNCompare("IM_MOD_",tag,7) == 0)
+    (void) strncpy(module_name,tag,MaxTextExtent-1);
+  else
+    {
+#if defined(_DEBUG)
+      FormatString(module_name,"IM_MOD_DB_%.1024s_.dll",tag);
+#else
+      FormatString(module_name,"IM_MOD_RL_%.1024s_.dll",tag);
+#endif
+    }
+#endif
+#endif
+  return(module_name);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  T a g T o P r o c e s s                                                    %
 %                                                                             %
 %                                                                             %
@@ -1152,55 +1232,6 @@ static char *TagToProcess(const char *tag)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%  T a g T o M o d u l e                                                      %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  TagToModule() takes a module "name" and returnes a complete file system
-%  dynamic module name.
-%
-%  The format of the TagToModule method is:
-%
-%      char *TagToModule(const char *tag)
-%
-%  A description of each parameter follows:
-%
-%    o tag: a character string that represents the name of the particular
-%      module.
-%
-*/
-MagickExport char *TagToModule(const char *tag)
-{
-  char
-    *module_name;
-
-  assert(tag != (char *) NULL);
-  module_name=AllocateString("tag");
-#if defined(HasLTDL)
-  (void) FormatString(module_name,"%.1024s.la",tag);
-  (void) LocaleLower(module_name);
-#else
-  if (LocaleNCompare("IM_MOD_",tag,7) == 0)
-    (void) strncpy(module_name,tag,MaxTextExtent-1);
-  else
-    {
-#  if defined(_DEBUG)
-      FormatString(module_name,"IM_MOD_DB_%.1024s_.dll",tag);
-#  else
-      FormatString(module_name,"IM_MOD_RL_%.1024s_.dll",tag);
-#  endif
-    }
-#endif
-  return(module_name);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   U n l o a d M o d u l e                                                   %
 %                                                                             %
 %                                                                             %
@@ -1239,7 +1270,7 @@ static unsigned int UnloadModule(const CoderInfo *coder_info,
   ModuleToTag(coder_info->tag,"Unregister%sImage",name);
   method=(void (*)(void)) lt_dlsym((ModuleHandle) coder_info->handle,name);
   if (method == (void (*)(void)) NULL)
-    MagickError(DelegateError,"failed to find symbol",lt_dlerror());
+    MagickError(ModuleError,"Failedto find symbol",lt_dlerror());
   else
     method();
   (void) lt_dlclose((ModuleHandle) coder_info->handle);
@@ -1289,7 +1320,7 @@ static unsigned int UnregisterModule(const char *tag,ExceptionInfo *exception)
   {
     if (LocaleCompare(p->tag,tag) != 0)
       continue;
-    (void) UnloadModule(p,exception);
+    (void) UnloadModule((CoderInfo *) p,exception);
     LiberateMemory((void **) &p->tag);
     if (p->previous != (CoderInfo *) NULL)
       p->previous->next=p->next;
