@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,6 +54,12 @@
   Include declarations.
 */
 #include "studio.h"
+#include "attribute.h"
+#include "gem.h"
+#include "list.h"
+#include "monitor.h"
+#include "render.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -91,8 +97,8 @@ MagickExport MontageInfo *CloneMontageInfo(const ImageInfo *image_info,
 
   clone_info=(MontageInfo *) AcquireMemory(sizeof(MontageInfo));
   if (clone_info == (MontageInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"Unable to clone montage info",
-      "Memory allocation failed");
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateMontageInfo");
   GetMontageInfo(image_info,clone_info);
   if (montage_info == (MontageInfo *) NULL)
     return(clone_info);
@@ -263,8 +269,8 @@ static int SceneCompare(const void *x,const void *y)
 MagickExport Image *MontageImages(const Image *images,
   const MontageInfo *montage_info,ExceptionInfo *exception)
 {
-#define MontageImageText  "  Create visual image directory...  "
-#define TileImageText  "  Create image tiles...  "
+#define MontageImageTag  "Montage/Image"
+#define TileImageTag  "Tile/Image"
 
   char
     tile_geometry[MaxTextExtent],
@@ -286,14 +292,11 @@ MagickExport Image *MontageImages(const Image *images,
     *montage,
     *texture,
     *tile_image,
-    *zoom_image;
+    *thumbnail;
 
   ImageInfo
     *image_info;
 
-  int
-    flags;
-
   long
     x,
     x_offset,
@@ -321,7 +324,8 @@ MagickExport Image *MontageImages(const Image *images,
     metrics;
 
   unsigned int
-    concatenate;
+    concatenate,
+    flags;
 
   unsigned long
     bevel_width,
@@ -349,31 +353,27 @@ MagickExport Image *MontageImages(const Image *images,
   assert(montage_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  number_images=GetImageListSize(images);
+  number_images=GetImageListLength(images);
   master_list=ImageListToArray(images,exception);
   if (master_list == (Image **) NULL)
-    ThrowImageException(ResourceLimitError,"Unable to create montage",
-      "Memory allocation failed")
+    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToCreateImageMontage");
   image_list=master_list;
   for (i=0; i < (long) number_images; i++)
   {
     handler=SetMonitorHandler((MonitorHandler) NULL);
     image=image_list[i];
-    SetGeometry(image,&geometry);
-    flags=ParseImageGeometry(montage_info->geometry,&geometry.x,&geometry.y,
-      &geometry.width,&geometry.height);
-    zoom_image=ZoomImage(image,geometry.width,geometry.height,exception);
-    if (zoom_image == (Image *) NULL)
+    flags=ParseSizeGeometry(image,montage_info->geometry,&geometry);
+    thumbnail=ZoomImage(image,geometry.width,geometry.height,exception);
+    if (thumbnail == (Image *) NULL)
       break;
-    DestroyImage(image);
-    image_list[i]=zoom_image;
+    image_list[i]=thumbnail;
     (void) SetMonitorHandler(handler);
-    MagickMonitor(TileImageText,i,number_images);
+    if (!MagickMonitor(TileImageTag,i,number_images,&image->exception))
+      break;
   }
   if (i < (long) number_images)
     {
-      for (i=0; i < (long) number_images; i++)
-        DestroyImage(master_list[i]);
       LiberateMemory((void **) &master_list);
       return((Image *) NULL);
     }
@@ -418,7 +418,7 @@ MagickExport Image *MontageImages(const Image *images,
       flags=GetGeometry(montage_info->geometry,&tile_info.x,&tile_info.y,
         &tile_info.width,&tile_info.height);
       if ((tile_info.x == 0) && (tile_info.y == 0))
-        concatenate=!((flags & WidthValue) || (flags & HeightValue));
+        concatenate=!((flags & RhoValue) || (flags & SigmaValue));
     }
   border_width=montage_info->border_width;
   bevel_width=0;
@@ -431,13 +431,13 @@ MagickExport Image *MontageImages(const Image *images,
       frame_info.width=tile_info.width;
       frame_info.height=tile_info.height;
       FormatString(absolute_geometry,"%.1024s!",montage_info->frame);
-      flags=ParseImageGeometry(absolute_geometry,&frame_info.outer_bevel,
+      flags=ParseMetaGeometry(absolute_geometry,&frame_info.outer_bevel,
         &frame_info.inner_bevel,&frame_info.width,&frame_info.height);
       if ((flags & HeightValue) == 0)
         frame_info.height=frame_info.width;
-      if ((flags & XValue) == 0)
+      if ((flags & XiValue) == 0)
         frame_info.outer_bevel=(long) (frame_info.width >> 2)+1;
-      if ((flags & YValue) == 0)
+      if ((flags & PsiValue) == 0)
         frame_info.inner_bevel=frame_info.outer_bevel;
       frame_info.x=(long) frame_info.width;
       frame_info.y=(long) frame_info.height;
@@ -479,8 +479,8 @@ MagickExport Image *MontageImages(const Image *images,
   title=TranslateText(image_info,image_list[0],montage_info->title);
   title_offset=0;
   if (montage_info->title != (char *) NULL)
-    title_offset=2*(metrics.ascent-metrics.descent)*MultilineCensus(title)+
-      2*tile_info.y;
+    title_offset=(unsigned long) (2*(metrics.ascent-metrics.descent)*
+      MultilineCensus(title)+2*tile_info.y);
   number_lines=0;
   for (i=0; i < (long) number_images; i++)
   {
@@ -522,15 +522,14 @@ MagickExport Image *MontageImages(const Image *images,
         {
           x_offset=0;
           height=concatenate ? max_height : tile_info.height;
-          y_offset+=height+(tile_info.y+border_width)*2+(metrics.ascent-
-            metrics.descent+4)*number_lines+(montage_info->shadow ? 4 : 0)+
-            (concatenate ? 0 : 2);
+          y_offset+=(unsigned long) (height+(tile_info.y+border_width)*2+
+            (metrics.ascent-metrics.descent+4)*number_lines+
+            (montage_info->shadow ? 4 : 0));
           if (y_offset > (long) bounds.height)
             bounds.height=y_offset;
           max_height=0;
         }
     }
-    bounds.height-=(concatenate ? 0 : 4);
     /*
       Initialize montage image.
     */
@@ -548,8 +547,8 @@ MagickExport Image *MontageImages(const Image *images,
     montage->directory=(char *) AcquireMemory(count);
     if ((montage->montage == (char *) NULL) ||
         (montage->directory == (char *) NULL))
-      ThrowImageException(ResourceLimitError,"Unable to create montage",
-        "Memory allocation failed");
+      ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
+        "UnableToCreateImageMontage");
     x_offset=0;
     y_offset=(long) title_offset;
     FormatString(montage->montage,"%ldx%ld%+ld%+ld",
@@ -634,9 +633,9 @@ MagickExport Image *MontageImages(const Image *images,
       if (image->gravity != ForgetGravity)
         tile_image->gravity=image->gravity;
       FormatString(tile_geometry,"%lux%lu+0+0",image->columns,image->rows);
-      flags=GetImageGeometry(tile_image,tile_geometry,False,&geometry);
-      x=geometry.x+border_width;
-      y=geometry.y+border_width;
+      flags=ParseGravityGeometry(tile_image,tile_geometry,&geometry);
+      x=(long) (geometry.x+border_width);
+      y=(long) (geometry.y+border_width);
       if ((montage_info->frame != (char *) NULL) && (bevel_width != 0))
         {
           FrameInfo
@@ -653,8 +652,8 @@ MagickExport Image *MontageImages(const Image *images,
           tile_info.height=height+2*frame_info.height;
           attribute=GetImageAttribute(image,"label");
           if (attribute != (const ImageAttribute *) NULL)
-            tile_info.height+=(metrics.ascent-metrics.descent+4)*
-              MultilineCensus(attribute->value);
+            tile_info.height+=(unsigned long) ((metrics.ascent-metrics.descent+
+              4)*MultilineCensus(attribute->value));
           frame_image=FrameImage(image,&tile_info,exception);
           if (frame_image != (Image *) NULL)
             {
@@ -734,13 +733,15 @@ MagickExport Image *MontageImages(const Image *images,
       if (((tile+1) == tiles_per_page) || (((tile+1) % tiles_per_row) == 0))
         {
           x_offset=tile_info.x;
-          y_offset+=height+(tile_info.y+border_width)*2+(metrics.ascent-
-            metrics.descent+4)*number_lines+(montage_info->shadow ? 4 : 0);
+          y_offset+=(unsigned long) (height+(tile_info.y+border_width)*2+
+            (metrics.ascent-metrics.descent+4)*number_lines+
+            (montage_info->shadow ? 4 : 0));
           max_height=0;
         }
       DestroyImage(image);
       (void) SetMonitorHandler(handler);
-      MagickMonitor(MontageImageText,tiles,total_tiles);
+      if (!MagickMonitor(MontageImageTag,tiles,total_tiles,&image->exception))
+        break;
       tiles++;
     }
     if ((i+1) < (long) images_per_page)
@@ -762,8 +763,6 @@ MagickExport Image *MontageImages(const Image *images,
   DestroyImage(tile_image);
   if (texture != (Image *) NULL)
     LiberateMemory((void **) &texture);
-  for (i=0; i < (long) number_images; i++)
-    DestroyImage(master_list[i]);
   LiberateMemory((void **) &master_list);
   DestroyDrawInfo(draw_info);
   DestroyImageInfo(image_info);
