@@ -39,21 +39,25 @@ typedef struct _QuantizeInfo
 /*
   Quantization utilities methods.
 */
+extern MagickExport DoublePixelPacket
+  GetImageDynamicThreshold(const Image *,const double,const double,
+    ExceptionInfo *);
+
 extern MagickExport QuantizeInfo
   *CloneQuantizeInfo(const QuantizeInfo *);
 
 extern MagickExport unsigned int
+  GetImageQuantizeError(Image *),
   MapImage(Image *,const Image *,const unsigned int),
   MapImages(Image *,const Image *,const unsigned int),
   OrderedDitherImage(Image *),
-  QuantizationError(Image *),
   QuantizeImage(const QuantizeInfo *,Image *),
   QuantizeImages(const QuantizeInfo *,Image *),
   SegmentImage(Image *,const ColorspaceType,const unsigned int,const double,
     const double);
 
 extern MagickExport void
-  CompressColormap(Image *),
+  CompressImageColormap(Image *),
   DestroyQuantizeInfo(QuantizeInfo *),
   GetQuantizeInfo(QuantizeInfo *);
 
