@@ -18,7 +18,7 @@
 %                                October 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,18 +53,21 @@
   Include declarations.
 */
 #include "studio.h"
+#include "list.h"
+#include "registry.h"
+#include "utility.h"
 
 /*
   Global declarations.
 */
+static SemaphoreInfo
+  *registry_semaphore = (SemaphoreInfo *) NULL;
+
 static long
   id = 0;
 
 static RegistryInfo
   *registry_list = (RegistryInfo *) NULL;
-
-static SemaphoreInfo
-  *registry_semaphore = (SemaphoreInfo *) NULL;
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -93,12 +96,12 @@ static SemaphoreInfo
 */
 MagickExport unsigned int DeleteMagickRegistry(const long id)
 {
-  RegistryInfo
-    *registry_info;
-
   register RegistryInfo
     *p;
 
+  RegistryInfo
+    *registry_info;
+
   AcquireSemaphoreInfo(&registry_semaphore);
   for (p=registry_list; p != (RegistryInfo *) NULL; p=p->next)
   {
@@ -158,12 +161,12 @@ MagickExport unsigned int DeleteMagickRegistry(const long id)
 */
 MagickExport void DestroyMagickRegistry(void)
 {
-  RegistryInfo
-    *registry_info;
-
   register RegistryInfo
     *p;
 
+  RegistryInfo
+    *registry_info;
+
   AcquireSemaphoreInfo(&registry_semaphore);
   for (p=registry_list; p != (RegistryInfo *) NULL; )
   {
@@ -247,7 +250,7 @@ MagickExport Image *GetImageFromMagickRegistry(const char *name,long *id,
   }
   LiberateSemaphoreInfo(&registry_semaphore);
   if (image == (Image *) NULL)
-    ThrowException(exception,RegistryError,"Unable to locate image",name);
+    ThrowException(exception,RegistryError,"UnableToLocateImage",name);
   return(image);
 }
 
@@ -288,12 +291,12 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
   register RegistryInfo
     *p;
 
-  RegistryInfo
-    *registry_info;
-
   void
     *blob;
 
+  RegistryInfo
+    *registry_info;
+
   blob=(void *) NULL;
   *type=UndefinedRegistryType;
   *length=0;
@@ -328,8 +331,8 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
         blob=(void *) AcquireMemory(registry_info->length);
         if (blob == (void *) NULL)
           {
-            ThrowException(exception,RegistryError,"Unable to get registry",
-              "Memory allocation failed");
+            ThrowException(exception,ResourceLimitError,
+              "MemoryAllocationFailed","UnableToGetFromRegistry");
             break;
           }
         (void) memcpy(blob,registry_info->blob,registry_info->length);
@@ -347,7 +350,7 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
         description[MaxTextExtent];
 
       FormatString(description,"id=%ld",id);
-      ThrowException(exception,RegistryError,"Unable to locate registry id",
+      ThrowException(exception,RegistryError,"UnableToLocateRegistryID",
         description);
     }
   return(blob);
@@ -404,14 +407,14 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
       image=(Image *) blob;
       if (length != sizeof(Image))
         {
-          ThrowException(exception,RegistryError,"Unable to set registry",
-            "Structure size mismatch");
+          ThrowException(exception,RegistryError,"UnableToSetRegistry",
+            "StructureSizeMismatch");
           return(-1);
         }
       if (image->signature != MagickSignature)
         {
-          ThrowException(exception,RegistryError,"Unable to set registry",
-            "Image expected");
+          ThrowException(exception,RegistryError,"UnableToSetRegistry",
+            "ImageExpected");
           return(-1);
         }
       clone_blob=(void *) CloneImageList(image,exception);
@@ -427,14 +430,14 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
       image_info=(ImageInfo *) blob;
       if (length != sizeof(ImageInfo))
         {
-          ThrowException(exception,RegistryError,"Unable to set registry",
-            "Structure size mismatch");
+          ThrowException(exception,RegistryError,"UnableToSetRegistry",
+            "StructureSizeMismatch");
           return(-1);
         }
       if (image_info->signature != MagickSignature)
         {
-          ThrowException(exception,RegistryError,"Unable to set registry",
-            "Image info expected");
+          ThrowException(exception,RegistryError,"UnableToSetRegistry",
+            "ImageInfoExpected");
           return(-1);
         }
       clone_blob=(void *) CloneImageInfo(image_info);
@@ -452,8 +455,8 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
   }
   registry_info=(RegistryInfo *) AcquireMemory(sizeof(RegistryInfo));
   if (registry_info == (RegistryInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"Unable to allocate registry",
-      "Memory allocation failed");
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateRegistryInfo");
   (void) memset(registry_info,0,sizeof(RegistryInfo));
   registry_info->type=type;
   registry_info->blob=clone_blob;
