@@ -14,10 +14,11 @@
 %                                                                             %
 %                              Software Design                                %
 %                             William Radcliffe                               %
+%                                John Cristy                                  %
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,15 +54,13 @@
   Include declarations.
 */
 #include "studio.h"
+#include "utility.h"
 #if defined(HasPTHREADS)
 #include <pthread.h>
 #endif
 #if defined(WIN32)
 #include <windows.h>
 #endif
-#if defined(_MT)
-#define MAXSEMLEN  1
-#endif
 
 /*
   Struct declaractions.
@@ -70,33 +69,32 @@ struct SemaphoreInfo
 {
 #if defined(HasPTHREADS)
   pthread_mutex_t
-    id;
+    mutex;
 #endif
-
 #if defined(WIN32)
-  HANDLE
-    id;
+  CRITICAL_SECTION
+    mutex;
 #endif
 
-  unsigned int
+  unsigned long
     signature;
 };
 
 /*
   Static declaractions.
 */
-#if defined(WIN32)
-static CRITICAL_SECTION
-  critical_section;
-
-static unsigned int
-  active_critical_section = False;
-#endif
-
 #if defined(HasPTHREADS)
 static pthread_mutex_t
   semaphore_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
+
+#if defined(WIN32)
+static CRITICAL_SECTION
+  semaphore_mutex;
+
+static unsigned int
+  active_semaphore = False;
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -109,7 +107,7 @@ static pthread_mutex_t
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method AcquireSemaphoreInfo acquires a semaphore.
+%  AcquireSemaphoreInfo() acquires a semaphore.
 %
 %  The format of the AcquireSemaphoreInfo method is:
 %
@@ -119,8 +117,6 @@ static pthread_mutex_t
 %
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
-%    o exit: call this method on termination of the program.
-%
 %
 */
 MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
@@ -130,19 +126,20 @@ MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
   (void) pthread_mutex_lock(&semaphore_mutex);
 #endif
 #if defined(WIN32)
-  if (active_critical_section)
-    EnterCriticalSection(&critical_section);
+  if (!active_semaphore)
+    InitializeCriticalSection(&semaphore_mutex);
+  active_semaphore=True;
+  EnterCriticalSection(&semaphore_mutex);
 #endif
   if (*semaphore_info == (SemaphoreInfo *) NULL)
     *semaphore_info=AllocateSemaphoreInfo();
-  (void) LockSemaphoreInfo(*semaphore_info);
 #if defined(HasPTHREADS)
   (void) pthread_mutex_unlock(&semaphore_mutex);
 #endif
 #if defined(WIN32)
-  if (active_critical_section)
-    LeaveCriticalSection(&critical_section);
+  LeaveCriticalSection(&semaphore_mutex);
 #endif
+  (void) LockSemaphoreInfo(*semaphore_info);
 }
 
 /*
@@ -179,9 +176,9 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
   */
   semaphore_info=(SemaphoreInfo *) AcquireMemory(sizeof(SemaphoreInfo));
   if (semaphore_info == (SemaphoreInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,
-      "Unable to allocate semaphore info","Memory allocation failed");
-  semaphore_info->signature=MagickSignature;
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateSemaphoreInfo");
+  memset(semaphore_info,0,sizeof(SemaphoreInfo));
   /*
     Initialize the semaphore.
   */
@@ -190,7 +187,7 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
     int
       status;
 
-    status=pthread_mutex_init(&semaphore_info->id,
+    status=pthread_mutex_init(&semaphore_info->mutex,
       (const pthread_mutexattr_t *) NULL);
     if (status != 0)
       {
@@ -199,22 +196,10 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
       }
   }
 #endif
-#if defined(WIN32) && defined(_MT)
-  {
-    SECURITY_ATTRIBUTES
-      security;
-
-    security.nLength=sizeof(security);
-    security.lpSecurityDescriptor=NULL;
-    security.bInheritHandle=TRUE;
-    semaphore_info->id=CreateSemaphore(&security,1,MAXSEMLEN,NULL);
-    if (semaphore_info->id == (HANDLE) NULL)
-      {
-        LiberateMemory((void **) &semaphore_info);
-        return((SemaphoreInfo *) NULL);
-      }
-  }
+#if defined(WIN32)
+  InitializeCriticalSection(&semaphore_info->mutex);
 #endif
+  semaphore_info->signature=MagickSignature;
   return(semaphore_info);
 }
 
@@ -229,9 +214,9 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method DestroySemaphore destroys the semaphore environment.
+%  DestroySemaphore() destroys the semaphore environment.
 %
-%  The format of the InitializeSemaphore method is:
+%  The format of the DestroySemaphore method is:
 %
 %      DestroySemaphore(void)
 %
@@ -243,11 +228,9 @@ MagickExport void DestroySemaphore(void)
   (void) pthread_mutex_destroy(&semaphore_mutex);
 #endif
 #if defined(WIN32)
-  if (active_critical_section)
-    {
-      DeleteCriticalSection(&critical_section);
-      active_critical_section=False;
-    }
+  if (active_semaphore)
+    DeleteCriticalSection(&semaphore_mutex);
+  active_semaphore=False;
 #endif
 }
 
@@ -277,30 +260,31 @@ MagickExport void DestroySemaphore(void)
 MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo **) NULL);
+  if (*semaphore_info == (SemaphoreInfo *) NULL)
+    return;
   assert((*semaphore_info)->signature == MagickSignature);
+  (void) UnlockSemaphoreInfo(*semaphore_info);
 #if defined(HasPTHREADS)
   (void) pthread_mutex_lock(&semaphore_mutex);
 #endif
 #if defined(WIN32)
-  if (active_critical_section)
-    EnterCriticalSection(&critical_section);
+  if (!active_semaphore)
+    InitializeCriticalSection(&semaphore_mutex);
+  active_semaphore=True;
+  EnterCriticalSection(&semaphore_mutex);
 #endif
-  if (*semaphore_info == (SemaphoreInfo *) NULL)
-    return;
-  (void) UnlockSemaphoreInfo(*semaphore_info);
 #if defined(HasPTHREADS)
-  (void) pthread_mutex_destroy(&(*semaphore_info)->id);
+  (void) pthread_mutex_destroy(&(*semaphore_info)->mutex);
 #endif
-#if defined(WIN32) && defined(_MT)
-  CloseHandle((*semaphore_info)->id);
+#if defined(WIN32)
+  DeleteCriticalSection(&(*semaphore_info)->mutex);
 #endif
-  LiberateMemory((void **) semaphore_info);
+  LiberateMemory((void **) &(*semaphore_info));
 #if defined(HasPTHREADS)
   (void) pthread_mutex_unlock(&semaphore_mutex);
 #endif
 #if defined(WIN32)
-  if (active_critical_section)
-    LeaveCriticalSection(&critical_section);
+  LeaveCriticalSection(&semaphore_mutex);
 #endif
 }
 
@@ -325,9 +309,13 @@ MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 */
 MagickExport void InitializeSemaphore(void)
 {
+#if defined(HasPTHREADS)
+  (void) pthread_mutex_init(&semaphore_mutex,
+    (const pthread_mutexattr_t *) NULL);
+#endif
 #if defined(WIN32)
-  InitializeCriticalSection(&critical_section);
-  active_critical_section=True;
+  InitializeCriticalSection(&semaphore_mutex);
+  active_semaphore=True;
 #endif
 }
 
@@ -357,9 +345,9 @@ MagickExport void InitializeSemaphore(void)
 MagickExport void LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo **) NULL);
-  assert((*semaphore_info)->signature == MagickSignature);
   if (*semaphore_info == (SemaphoreInfo *) NULL)
     return;
+  assert((*semaphore_info)->signature == MagickSignature);
   (void) UnlockSemaphoreInfo(*semaphore_info);
 }
 
@@ -391,21 +379,14 @@ MagickExport void LiberateSemaphoreInfo(SemaphoreInfo **semaphore_info)
 */
 MagickExport unsigned int LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
-  int
-    status;
-
   assert(semaphore_info != (SemaphoreInfo *) NULL);
   assert(semaphore_info->signature == MagickSignature);
-  status=False;
-#if defined(WIN32) && defined(_MT)
-  status=WaitForSingleObject(semaphore_info->id,INFINITE);
-  if (status == WAIT_FAILED)
+#if defined(HasPTHREADS)
+  if (pthread_mutex_lock(&semaphore_info->mutex))
     return(False);
 #endif
-#if defined(HasPTHREADS)
-  status=pthread_mutex_lock(&semaphore_info->id);
-  if (status != 0)
-    return(False);
+#if defined(WIN32)
+  EnterCriticalSection(&semaphore_info->mutex);
 #endif
   return(True);
 }
@@ -438,21 +419,14 @@ MagickExport unsigned int LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 */
 MagickExport unsigned int UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
-  int
-    status;
-
   assert(semaphore_info != (SemaphoreInfo *) NULL);
   assert(semaphore_info->signature == MagickSignature);
-  status=False;
 #if defined(HasPTHREADS)
-  status=pthread_mutex_unlock(&semaphore_info->id);
-  if (status != 0)
+  if (pthread_mutex_unlock(&semaphore_info->mutex))
     return(False);
 #endif
-#if defined(WIN32) && defined(_MT)
-  status=ReleaseSemaphore(semaphore_info->id,1,NULL);
-  if (status == FALSE)
-    return(False);
+#if defined(WIN32)
+  LeaveCriticalSection(&semaphore_info->mutex);
 #endif
   return(True);
 }
