@@ -18,7 +18,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -59,6 +59,9 @@
   Include declarations.
 */
 #include "studio.h"
+#include "list.h"
+#include "monitor.h"
+#include "render.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -128,8 +131,8 @@ MagickExport Image *AffineTransformImage(const Image *image,
   extent[3].y=image->rows;
   for (i=0; i < 4; i++)
   {
-    x=extent[i].x+0.5;
-    y=extent[i].y+0.5;
+    x=(long) (extent[i].x+0.5);
+    y=(long) (extent[i].y+0.5);
     extent[i].x=x*affine->sx+y*affine->ry+affine->tx;
     extent[i].y=x*affine->rx+y*affine->sy+affine->ty;
   }
@@ -268,9 +271,9 @@ static void CropToFitImage(Image **image,const double x_shear,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IntegralRotateImage rotates the image an integral of 90 degrees.
-%  It allocates the memory necessary for the new Image structure and returns
-%  a pointer to the rotated image.
+%  IntegralRotateImage()  rotates the image an integral of 90 degrees.  It
+%  allocates the memory necessary for the new Image structure and returns a
+%  pointer to the rotated image.
 %
 %  The format of the IntegralRotateImage method is:
 %
@@ -291,7 +294,7 @@ static void CropToFitImage(Image **image,const double x_shear,
 static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
   ExceptionInfo *exception)
 {
-#define RotateImageText  "  Rotate image...  "
+#define RotateImageTag  "Rotate/Image"
 
   Image
     *rotate_image;
@@ -353,7 +356,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         if (!SyncImagePixels(rotate_image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(RotateImageText,y,image->rows);
+          if (!MagickMonitor(RotateImageTag,y,image->rows,exception))
+            break;
       }
       break;
     }
@@ -379,7 +383,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         if (!SyncImagePixels(rotate_image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(RotateImageText,y,image->rows);
+          if (!MagickMonitor(RotateImageTag,y,image->rows,exception))
+            break;
       }
       Swap(page.width,page.height);
       Swap(page.x,page.y);
@@ -410,7 +415,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         if (!SyncImagePixels(rotate_image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(RotateImageText,y,image->rows);
+          if (!MagickMonitor(RotateImageTag,y,image->rows,exception))
+            break;
       }
       page.x=(long) (page.width-rotate_image->columns-page.x);
       page.y=(long) (page.height-rotate_image->rows-page.y);
@@ -439,7 +445,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned int rotations,
         if (!SyncImagePixels(rotate_image))
           break;
         if (QuantumTick(y,image->rows))
-          MagickMonitor(RotateImageText,y,image->rows);
+          if (!MagickMonitor(RotateImageTag,y,image->rows,exception))
+            break;
       }
       Swap(page.width,page.height);
       Swap(page.x,page.y);
@@ -494,16 +501,15 @@ static inline PixelPacket BlendComposite(const PixelPacket *p,
   PixelPacket
     composite;
 
-  color=(double) (p->red*(MaxRGB-alpha)+q->red*alpha)/MaxRGB;
+  color=((double) p->red*(MaxRGB-alpha)+q->red*alpha)/MaxRGB;
   composite.red=(Quantum)
     ((color < 0) ? 0 : (color > MaxRGB) ? MaxRGB : color+0.5);
-  color=(double) (p->green*(MaxRGB-alpha)+q->green*alpha)/MaxRGB;
+  color=((double) p->green*(MaxRGB-alpha)+q->green*alpha)/MaxRGB;
   composite.green=(Quantum)
     ((color < 0) ? 0 : (color > MaxRGB) ? MaxRGB : color+0.5);
-  color=(double) (p->blue*(MaxRGB-alpha)+q->blue*alpha)/MaxRGB;
+  color=((double) p->blue*(MaxRGB-alpha)+q->blue*alpha)/MaxRGB;
   composite.blue=(Quantum)
     ((color < 0) ? 0 : (color > MaxRGB) ? MaxRGB : color+0.5);
-  color=(double) (p->opacity*(MaxRGB-alpha)+q->opacity*alpha)/MaxRGB;
   composite.opacity=p->opacity;
   return(composite);
 }
@@ -512,7 +518,7 @@ static void XShearImage(Image *image,const double degrees,
   const unsigned long width,const unsigned long height,const long x_offset,
   long y_offset)
 {
-#define XShearImageText  "  X Shear image...  "
+#define XShearImageTag  "XShear/Image"
 
   double
     alpha,
@@ -551,7 +557,7 @@ static void XShearImage(Image *image,const double degrees,
         direction=LEFT;
       }
     step=(long) floor(displacement+0.5);
-    alpha=MaxRGB*(displacement-step);
+    alpha=(double) MaxRGB*(displacement-step);
     if (alpha == 0.0)
       {
         /*
@@ -661,7 +667,8 @@ static void XShearImage(Image *image,const double degrees,
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,height))
-      MagickMonitor(XShearImageText,y,height);
+      if (!MagickMonitor(XShearImageTag,y,height,&image->exception))
+        break;
   }
 }
 
@@ -703,7 +710,7 @@ static void YShearImage(Image *image,const double degrees,
   const unsigned long width,const unsigned long height,long x_offset,
   const long y_offset)
 {
-#define YShearImageText  "  Y Shear image...  "
+#define YShearImageTag  "YShear/Image"
 
   double
     alpha,
@@ -742,7 +749,7 @@ static void YShearImage(Image *image,const double degrees,
         direction=UP;
       }
     step=(long) floor(displacement+0.5);
-    alpha=MaxRGB*(displacement-step);
+    alpha=(double) MaxRGB*(displacement-step);
     if (alpha == 0.0)
       {
         /*
@@ -852,7 +859,8 @@ static void YShearImage(Image *image,const double degrees,
     if (!SyncImagePixels(image))
       break;
     if (QuantumTick(y,width))
-      MagickMonitor(YShearImageText,y,width);
+      if (!MagickMonitor(YShearImageTag,y,width,&image->exception))
+        break;
   }
 }
 
@@ -867,16 +875,16 @@ static void YShearImage(Image *image,const double degrees,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RotateImage creates a new image that is a rotated copy of an
-%  existing one.  Positive angles rotate counter-clockwise (right-hand rule),
-%  while negative angles rotate clockwise.  Rotated images are usually larger
-%  than the originals and have 'empty' triangular corners.  X axis.  Empty
-%  triangles left over from shearing the image are filled with the color
-%  defined by the pixel at location (0,0).  RotateImage allocates the memory
-%  necessary for the new Image structure and returns a pointer to the new
-%  image.
+%  RotateImage() creates a new image that is a rotated copy of an existing
+%  one.  Positive angles rotate counter-clockwise (right-hand rule), while
+%  negative angles rotate clockwise.  Rotated images are usually larger than
+%  the originals and have 'empty' triangular corners.  X axis.  Empty
+%  triangles left over from shearing the image are filled with the background
+%  color defined by member 'background_color' of the image.  RotateImage
+%  allocates the memory necessary for the new Image structure and returns a
+%  pointer to the new image.
 %
-%  Method RotateImage is based on the paper "A Fast Algorithm for General
+%  RotateImage() is based on the paper "A Fast Algorithm for General
 %  Raster Rotatation" by Alan W. Paeth.  RotateImage is adapted from a similar
 %  method based on the Paeth paper written by Michael Halle of the Spatial
 %  Imaging Group, MIT Media Lab.
@@ -920,11 +928,9 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   RectangleInfo
     border_info;
 
-  unsigned int
-    rotations;
-
   unsigned long
     height,
+    rotations,
     width,
     y_width;
 
@@ -946,11 +952,9 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   */
   integral_image=IntegralRotateImage(image,rotations,exception);
   if (integral_image == (Image *) NULL)
-    ThrowImageException(ResourceLimitError,"Unable to rotate image",
-      "Memory allocation failed");
-  SetImageType(integral_image,integral_image->background_color.opacity !=
-    OpaqueOpacity ? TrueColorMatteType : TrueColorType);
-  shear.x=(-tan(DegreesToRadians(angle)/2));
+    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToRotateImage");
+  shear.x=(-tan(DegreesToRadians(angle)/2.0));
   shear.y=sin(DegreesToRadians(angle));
   if ((shear.x == 0.0) || (shear.y == 0.0))
     return(integral_image);
@@ -964,9 +968,9 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
       width=image->rows;
       height=image->columns;
     }
-  y_width=(unsigned long) (width+ceil(fabs(shear.x)*height-0.5));
-  x_offset=(long) ceil(2*fabs(shear.x)*height-0.5);
-  y_offset=(long) ceil(fabs(shear.y)*y_width-0.5);
+  x_offset=(long) ceil(fabs(2.0*height*shear.y)-0.5);
+  y_width=(unsigned long) floor(fabs(height*shear.x)+width+0.5);
+  y_offset=(long) ceil(fabs(y_width*shear.y)-0.5);
   /*
     Surround image with a border.
   */
@@ -976,11 +980,13 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
   rotate_image=BorderImage(integral_image,&border_info,exception);
   DestroyImage(integral_image);
   if (rotate_image == (Image *) NULL)
-    ThrowImageException(ResourceLimitError,"Unable to rotate image",
-      "Memory allocation failed");
+    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToRotateImage");
   /*
     Rotate the image.
   */
+  SetImageType(rotate_image,rotate_image->background_color.opacity !=
+    OpaqueOpacity ? TrueColorMatteType : TrueColorType);
   XShearImage(rotate_image,shear.x,width,height,x_offset,
     (long) (rotate_image->rows-height)/2);
   YShearImage(rotate_image,shear.y,y_width,height,
@@ -1003,19 +1009,19 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ShearImage creates a new image that is a shear_image copy of an
-%  existing one.  Shearing slides one edge of an image along the X or Y
-%  axis, creating a parallelogram.  An X direction shear slides an edge
-%  along the X axis, while a Y direction shear slides an edge along the Y
-%  axis.  The amount of the shear is controlled by a shear angle.  For X
-%  direction shears, x_shear is measured relative to the Y axis, and
-%  similarly, for Y direction shears y_shear is measured relative to the
-%  X axis.  Empty triangles left over from shearing the image are filled
-%  with the color defined by the pixel at location (0,0).  ShearImage
-%  allocates the memory necessary for the new Image structure and returns
-%  a pointer to the new image.
+%  ShearImage() creates a new image that is a shear_image copy of an existing
+%  one.  Shearing slides one edge of an image along the X or Y axis, creating
+%  a parallelogram.  An X direction shear slides an edge along the X axis,
+%  while a Y direction shear slides an edge along the Y axis.  The amount of
+%  the shear is controlled by a shear angle.  For X direction shears, 
+%  x_shear is measured relative to the Y axis, and similarly, for Y direction
+%  shears y_shear is measured relative to the X axis.  Empty triangles left
+%  over from shearing the image are filled with the background color defined
+%  by member 'background_color' of the image..  ShearImage() allocates the
+%  memory necessary for the new Image structure and returns a pointer to the
+%  new image.
 %
-%  Method ShearImage is based on the paper "A Fast Algorithm for General
+%  ShearImage() is based on the paper "A Fast Algorithm for General
 %  Raster Rotatation" by Alan W. Paeth.
 %
 %  The format of the ShearImage method is:
@@ -1062,27 +1068,24 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if ((x_shear == 180.0) || (y_shear == 180.0))
-    ThrowImageException(OptionError,"Unable to shear image",
-      "angle is discontinuous");
+    ThrowImageException(ImageError,"UnableToShearImage","AngleIsDiscontinuous");
   /*
     Initialize shear angle.
   */
   integral_image=IntegralRotateImage(image,0,exception);
   if (integral_image == (Image *) NULL)
-    ThrowImageException(ResourceLimitError,"Unable to shear image",
-      "Memory allocation failed");
-  SetImageType(integral_image,integral_image->background_color.opacity !=
-    OpaqueOpacity ?  TrueColorMatteType : TrueColorType);
-  shear.x=(-tan(DegreesToRadians(x_shear)/2));
+    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToShearImage");
+  shear.x=(-tan(DegreesToRadians(x_shear)/2.0));
   shear.y=sin(DegreesToRadians(y_shear));
   if ((shear.x == 0.0) || (shear.y == 0.0))
     return(integral_image);
   /*
     Compute image size.
   */
-  y_width=(unsigned long) (image->columns+ceil(fabs(shear.x)*image->rows-0.5));
-  x_offset=(long) ceil(2*fabs(shear.x)*image->rows-0.5);
-  y_offset=(long) ceil(fabs(shear.y)*y_width-0.5);
+  x_offset=(long) ceil(fabs(2.0*image->rows*shear.x)-0.5);
+  y_width=(unsigned long) floor(fabs(image->rows*shear.x)+image->columns+0.5);
+  y_offset=(long) ceil(fabs(y_width*shear.y)-0.5);
   /*
     Surround image with border.
   */
@@ -1091,12 +1094,14 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   border_info.height=y_offset;
   shear_image=BorderImage(integral_image,&border_info,exception);
   if (shear_image == (Image *) NULL)
-    ThrowImageException(ResourceLimitError,"Unable to shear image",
-      "Memory allocation failed");
+    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToShearImage");
   DestroyImage(integral_image);
   /*
     Shear the image.
   */
+  SetImageType(shear_image,shear_image->background_color.opacity !=
+    OpaqueOpacity ?  TrueColorMatteType : TrueColorType);
   XShearImage(shear_image,shear.x,image->columns,image->rows,x_offset,
     (long) (shear_image->rows-image->rows)/2);
   YShearImage(shear_image,shear.y,y_width,image->rows,
