@@ -18,7 +18,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,6 +54,9 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "cache.h"
+#include "utility.h"
 
 /*
   Typedef declaractions.
@@ -61,21 +64,38 @@
 typedef CacheInfo StreamInfo;
 
 /*
-  Declare pixel stream interfaces.
+  Declare pixel cache interfaces.
 */
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
 static const PixelPacket
   *AcquirePixelStream(const Image *,const long,const long,const unsigned long,
     const unsigned long,ExceptionInfo *);
 
+static IndexPacket
+  *GetIndexesFromStream(const Image *);
+
 static PixelPacket
+  AcquireOnePixelFromStream(const Image *,const long,const long,
+    ExceptionInfo *),
+  GetOnePixelFromStream(Image *,const long,const long),
   *GetPixelStream(Image *,const long,const long,const unsigned long,
     const unsigned long),
+  *GetPixelsFromStream(const Image *),
   *SetPixelStream(Image *,const long,const long,const unsigned long,
     const unsigned long);
 
 static unsigned int
   SyncPixelStream(Image *);
-
+
+static void
+  DestroyPixelStream(Image *);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -163,15 +183,15 @@ static const PixelPacket *AcquirePixelStream(const Image *image,const long x,
   const long y,const unsigned long columns,const unsigned long rows,
   ExceptionInfo *exception)
 {
-  off_t
-    offset;
+  ExtendedSignedIntegralType
+    number_pixels;
+
+  size_t
+    length;
 
   StreamInfo
     *stream_info;
 
-  unsigned long
-    number_pixels;
-
   /*
     Validate pixel cache geometry.
   */
@@ -179,35 +199,35 @@ static const PixelPacket *AcquirePixelStream(const Image *image,const long x,
   if ((x < 0) || (y < 0) || ((x+(long) columns) > (long) image->columns) ||
       ((y+(long) rows) > (long) image->rows) || (columns == 0) || (rows == 0))
     {
-      ThrowException(exception,StreamError,"Unable to acquire pixel stream",
-        "image does not contain the stream geometry");
+      ThrowException(exception,StreamError,"UnableToAcquirePixelStream",
+        "ImageDoesNotContainTheStreamGeometry");
       return((PixelPacket *) NULL);
     }
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
   if (stream_info->type == UndefinedCache)
     {
-      ThrowException(exception,CacheError,"Pixel cache is not open",
+      ThrowException(exception,StreamError,"PixelCacheIsNotOpen",
         image->filename);
       return((PixelPacket *) NULL);
     }
   /*
     Pixels are stored in a temporary buffer until they are synced to the cache.
   */
-  number_pixels=columns*rows;
-  offset=number_pixels*sizeof(PixelPacket);
+  number_pixels=(ExtendedSignedIntegralType) columns*rows;
+  length=number_pixels*sizeof(PixelPacket);
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
-    offset+=number_pixels*sizeof(IndexPacket);
+    length+=number_pixels*sizeof(IndexPacket);
   if (stream_info->pixels == (PixelPacket *) NULL)
-    stream_info->pixels=(PixelPacket *) AcquireMemory(offset);
+    stream_info->pixels=(PixelPacket *) AcquireMemory(length);
   else
-    if (offset != stream_info->length)
-      ReacquireMemory((void **) &stream_info->pixels,offset);
+    if (length != stream_info->length)
+      ReacquireMemory((void **) &stream_info->pixels,length);
   if (stream_info->pixels == (void *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"Memory allocation failed",
-      "unable to allocate cache info");
-  stream_info->length=offset;
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateCacheInfo");
+  stream_info->length=length;
   stream_info->indexes=(IndexPacket *) NULL;
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
@@ -258,6 +278,8 @@ static void DestroyPixelStream(Image *image)
   LiberateSemaphoreInfo(&stream_info->semaphore);
   if (stream_info->pixels != (PixelPacket *) NULL)
     LiberateMemory((void **) &stream_info->pixels);
+  if (stream_info->semaphore != (SemaphoreInfo *) NULL)
+    DestroySemaphoreInfo(&stream_info->semaphore);
   LiberateMemory((void **) &stream_info);
 }
 
@@ -443,8 +465,7 @@ static PixelPacket *GetPixelsFromStream(const Image *image)
 %
 %  The format of the ReadStream() method is:
 %
-%      Image *ReadStream(const ImageInfo *image_info,
-%        void (*Stream)(const Image *,const void *,const size_t),
+%      Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -457,8 +478,8 @@ static PixelPacket *GetPixelsFromStream(const Image *image)
 %
 %
 */
-MagickExport Image *ReadStream(const ImageInfo *image_info,
-  int (*fifo)(const Image *,const void *,const size_t),ExceptionInfo *exception)
+MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
+  ExceptionInfo *exception)
 {
   Image
     *image;
@@ -473,14 +494,14 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  SetPixelCacheMethods(AcquirePixelStream,GetPixelStream,SetPixelStream,
-    SyncPixelStream,GetPixelsFromStream,GetIndexesFromStream,
-    AcquireOnePixelFromStream,GetOnePixelFromStream,DestroyPixelStream);
   clone_info=CloneImageInfo(image_info);
-  clone_info->fifo=fifo;
+  GetCacheInfo(&clone_info->cache);
+  SetPixelCacheMethods(clone_info->cache,AcquirePixelStream,GetPixelStream,
+    SetPixelStream,SyncPixelStream,GetPixelsFromStream,GetIndexesFromStream,
+    AcquireOnePixelFromStream,GetOnePixelFromStream,DestroyPixelStream);
+  clone_info->stream=stream;
   image=ReadImage(clone_info,exception);
   DestroyImageInfo(clone_info);
-  ResetPixelCacheMethods();
   return(image);
 }
 
@@ -521,15 +542,15 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,
 static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
   const unsigned long columns,const unsigned long rows)
 {
-  off_t
-    offset;
+  ExtendedSignedIntegralType
+    number_pixels;
+
+  size_t
+    length;
 
   StreamInfo
     *stream_info;
 
-  unsigned long
-    number_pixels;
-
   /*
     Validate pixel cache geometry.
   */
@@ -537,15 +558,14 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
   if ((x < 0) || (y < 0) || ((x+(long) columns) > (long) image->columns) ||
       ((y+(long) rows) > (long) image->rows) || (columns == 0) || (rows == 0))
     {
-      ThrowException(&image->exception,StreamError,
-        "Unable to set pixel stream","image does not contain the geometry");
+      ThrowException(&image->exception,StreamError,"UnableToSetPixelStream",
+        "ImageDoesNotContainTheStreamGeometry");
       return((PixelPacket *) NULL);
     }
-  if (image->blob->fifo ==
-       (int (*)(const Image *,const void *,const size_t)) NULL)
+  if (image->blob->stream == (StreamHandler) NULL)
     {
-      ThrowException(&image->exception,StreamError,
-        "Unable to set pixel stream","no fifo is defined");
+      ThrowException(&image->exception,StreamError,"UnableToSetPixelStream",
+        "NoStreamHandlerIsDefined");
       return((PixelPacket *) NULL);
     }
   stream_info=(StreamInfo *) image->cache;
@@ -554,7 +574,8 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
       (image->colorspace != GetCacheColorspace(image->cache)))
     {
       if (GetCacheClass(image->cache) == UndefinedClass)
-        (void) image->blob->fifo(image,(const void *) NULL,stream_info->columns);
+        (void) image->blob->stream(image,(const void *) NULL,
+          stream_info->columns);
       stream_info->storage_class=image->storage_class;
       stream_info->colorspace=image->colorspace;
       stream_info->columns=image->columns;
@@ -564,20 +585,27 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
   /*
     Pixels are stored in a temporary buffer until they are synced to the cache.
   */
-  number_pixels=columns*rows;
-  offset=number_pixels*sizeof(PixelPacket);
+  stream_info->columns=columns;
+  stream_info->rows=rows;
+  number_pixels=(ExtendedSignedIntegralType) columns*rows;
+  length=number_pixels*sizeof(PixelPacket);
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
-    offset+=number_pixels*sizeof(IndexPacket);
+    length+=number_pixels*sizeof(IndexPacket);
   if (stream_info->pixels == (PixelPacket *) NULL)
-    stream_info->pixels=(PixelPacket *) AcquireMemory(offset);
+    {
+      stream_info->pixels=(PixelPacket *) AcquireMemory(length);
+      stream_info->length=length;
+    }
   else
-    if (offset != stream_info->length)
-      ReacquireMemory((void **) &stream_info->pixels,offset);
+    if (stream_info->length < length)
+      {
+        ReacquireMemory((void **) &stream_info->pixels,length);
+        stream_info->length=length;
+      }
   if (stream_info->pixels == (void *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"Memory allocation failed",
-      "unable to allocate cache info");
-  stream_info->length=offset;
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+      "UnableToAllocateImagePixels");
   stream_info->indexes=(IndexPacket *) NULL;
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
@@ -621,7 +649,13 @@ static unsigned int SyncPixelStream(Image *image)
   assert(image->signature == MagickSignature);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
-  return(image->blob->fifo(image,stream_info->pixels,stream_info->columns));
+  if (image->blob->stream == (StreamHandler) NULL)
+    {
+      ThrowException(&image->exception,StreamError,"UnableToSyncPixelStream",
+        "NoStreamHandlerIsDefined");
+      return(False);
+    }
+  return(image->blob->stream(image,stream_info->pixels,stream_info->columns));
 }
 
 /*
@@ -642,7 +676,7 @@ static unsigned int SyncPixelStream(Image *image)
 %  The format of the WriteStream() method is:
 %
 %      unsigned int WriteStream(const ImageInfo *image_info,Image *,
-%        int (*Stream)(const Image *,const void *,const size_t))
+%        StreamHandler stream)
 %
 %  A description of each parameter follows:
 %
@@ -653,7 +687,7 @@ static unsigned int SyncPixelStream(Image *image)
 %
 */
 MagickExport unsigned int WriteStream(const ImageInfo *image_info,Image *image,
-  int (*fifo)(const Image *,const void *,const size_t))
+  StreamHandler stream)
 {
   ImageInfo
     *clone_info;
@@ -666,7 +700,7 @@ MagickExport unsigned int WriteStream(const ImageInfo *image_info,Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   clone_info=CloneImageInfo(image_info);
-  clone_info->fifo=fifo;
+  clone_info->stream=stream;
   status=WriteImage(clone_info,image);
   DestroyImageInfo(clone_info);
   return(status);
