@@ -8,11 +8,19 @@
 extern "C" {
 #endif
 
+#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+# define POSIX
+#endif
+
 /*
   System include declarations.
 */
-#define _XOPEN_SOURCE  500
+#define __EXTENSIONS__  1
+#define _GNU_SOURCE  1
+#define _ISOC99_SOURCE  1
 #define _POSIX_C_SOURCE  199506L
+#define _XOPEN_SOURCE  500 
+#define _LARGEFILE64_SOURCE  1
 
 #if !defined(_MAGICK_CONFIG_H)
 # define _MAGICK_CONFIG_H
@@ -26,8 +34,72 @@ extern "C" {
 # endif
 #endif
 
+#if defined(WIN32) && !defined(__CYGWIN__)
+# if defined(_MT) && defined(_DLL) && !defined(_MAGICKDLL_) && !defined(_LIB)
+#  define _MAGICKDLL_
+# endif
+# if defined(_MAGICKDLL_)
+#  if defined(_VISUALC_)
+#   pragma warning( disable: 4273 )  /* Disable the dll linkage warnings */
+#  endif
+#  if !defined(_MAGICKLIB_)
+#   define MagickExport  __declspec(dllimport)
+#   if defined(_VISUALC_)
+#    pragma message( "Magick lib DLL import interface" )
+#   endif
+#  else
+#   define MagickExport  __declspec(dllexport)
+#   if defined(_VISUALC_)
+#    pragma message( "Magick lib DLL export interface" )
+#   endif
+#  endif
+# else
+#  define MagickExport
+#  if defined(_VISUALC_)
+#   pragma message( "Magick lib static interface" )
+#  endif
+# endif
+
+# if defined(_DLL) && !defined(_LIB)
+#  define ModuleExport  __declspec(dllexport)
+#  if defined(_VISUALC_)
+#   pragma message( "Magick module DLL export interface" ) 
+#  endif
+# else
+#  define ModuleExport
+#  if defined(_VISUALC_)
+#   pragma message( "Magick module static interface" ) 
+#  endif
+
+# endif
+# define MagickGlobal __declspec(thread)
+# if defined(_VISUALC_)
+#  pragma warning(disable : 4018)
+#  pragma warning(disable : 4244)
+#  pragma warning(disable : 4142)
+#  pragma warning(disable : 4800)
+#  pragma warning(disable : 4786)
+# endif
+#else
+# define MagickExport
+# define ModuleExport
+# define MagickGlobal
+#endif
+
+#if defined(__cplusplus) || defined(c_plusplus)
+# define storage_class  c_class
+#else
+# define storage_class  class
+#endif
+
+#define MagickSignature  0xabacadabUL
+#define MaxTextExtent  2053
+
 #include <stdarg.h>
 #include <stdio.h>
+#if defined(WIN32) && defined(_DEBUG)
+#define _CRTDBG_MAP_ALLOC
+#endif
 #include <stdlib.h>
 #if !defined(WIN32)
 # include <unistd.h>
@@ -38,6 +110,10 @@ extern "C" {
 # endif
 #endif
 
+#if defined(HAVE_INTTYPES_H)
+# include <inttypes.h>
+#endif
+
 #if defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64)
 # define fseek  fseeko
 # define ftell  ftello
@@ -54,10 +130,13 @@ extern "C" {
 #include <signal.h>
 #include <assert.h>
 
-#if !defined(vms) && !defined(macintosh)
+#if defined(WIN32) || defined(POSIX)
 # include <sys/types.h>
 # include <sys/stat.h>
-# if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+# if defined(HAVE_FTIME)
+# include <sys/timeb.h>
+# endif
+# if defined(POSIX)
 #  if defined(HAVE_SYS_NDIR_H) || defined(HAVE_SYS_DIR_H) || defined(HAVE_NDIR_H)
 #   define dirent direct
 #   define NAMLEN(dirent) (dirent)->d_namlen
@@ -79,10 +158,18 @@ extern "C" {
 # if !defined(S_ISDIR)
 #  define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
 # endif
-# include "magick/api.h"
+# if !defined(S_ISREG)
+#  define S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
+# endif
+# include "magick/image.h"
 # if !defined(WIN32)
 #  include <sys/time.h>
+#if defined(HAVE_SYS_TIMES_H)
 #  include <sys/times.h>
+#endif
+#if defined(HAVE_SYS_RESOURCE_H)
+#  include <sys/resource.h>
+#endif
 # endif
 #else
 # include <types.h>
@@ -92,11 +179,14 @@ extern "C" {
 #  include <console.h>
 #  include <unix.h>
 # endif
-# include "api.h"
+# include "image.h"
 #endif
 
 #if defined(WIN32)
-# include "nt.h"
+# include "nt_base.h"
+#endif
+#if defined(WIN32) || defined(__CYGWIN__)
+# include "nt_feature.h"
 #endif
 #if defined(macintosh)
 # include "mac.h"
@@ -120,31 +210,12 @@ extern "C" {
 /*
   Review these platform specific definitions.
 */
-#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
-# if !defined(ApplicationDefaults)
-#  define ApplicationDefaults  "/usr/X11R6/lib/X11/app-defaults/"
-# endif
+#if defined(POSIX)
 # define DirectorySeparator  "/"
 # define DirectoryListSeparator  ':'
 # define EditorOptions  " -title \"Edit Image Comment\" -e vi"
 # define Exit  exit
 # define IsBasenameSeparator(c)  ((c) == '/')
-# define IsGlob(text) \
-  ((strchr(text,'*') != (char *) NULL) || \
-   (strchr(text,'?') != (char *) NULL) || \
-   (strchr(text,'{') != (char *) NULL) || \
-   (strchr(text,'}') != (char *) NULL) || \
-   (strchr(text,'[') != (char *) NULL) || \
-   (strchr(text,']') != (char *) NULL))
-# if !defined(MagickLibPath)
-#  define MagickLibPath  "/usr/local/lib/ImageMagick/"
-# endif
-# if !defined(MagickModulesPath)
-#  define MagickModulesPath  "/usr/local/lib/ImageMagick/modules/"
-# endif
-# if !defined(MagickSharePath)
-#  define MagickSharePath  "/usr/local/share/ImageMagick/"
-# endif
 # define PreferencesDefaults  "~/."
 # define ProcessPendingEvents(text)
 # define ReadCommandlLine(argc,argv)
@@ -158,11 +229,6 @@ extern "C" {
 #  define EditorOptions  ""
 #  define Exit  exit
 #  define IsBasenameSeparator(c)  (((c) == ']') || ((c) == ':') || ((c) == '/'))
-#  define IsGlob(text) \
-  ((strchr(text,'*') != (char *) NULL) || \
-   (strchr(text,'?') != (char *) NULL) || \
-   (strchr(text,'{') != (char *) NULL) || \
-   (strchr(text,'}') != (char *) NULL))
 #  define MagickLibPath  "sys$login:"
 #  define MagickModulesPath  "sys$login:"
 #  define MagickSharePath  "sys$login:"
@@ -178,13 +244,6 @@ extern "C" {
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions ""
 #  define IsBasenameSeparator(c)  ((c) == ':')
-#  define IsGlob(text) \
-    ((strchr(text,'*') != (char *) NULL) || \
-     (strchr(text,'?') != (char *) NULL) || \
-     (strchr(text,'{') != (char *) NULL) || \
-     (strchr(text,'}') != (char *) NULL) || \
-     (strchr(text,'[') != (char *) NULL) || \
-     (strchr(text,']') != (char *) NULL))
 #  define MagickLibPath  ""
 #  define MagickModulesPath  ""
 #  define MagickSharePath  ""
@@ -196,29 +255,14 @@ extern "C" {
 # endif
 
 # if defined(WIN32)
-#  define ApplicationDefaults  "c:\\ImageMagick\\"
 #  define DirectorySeparator  "\\"
 #  define DirectoryListSeparator  ';'
 #  define EditorOptions ""
 #  define IsBasenameSeparator(c)  (((c) == '/') || ((c) == '\\'))
-#  define IsGlob(text) \
-    ((strchr(text,'*') != (char *) NULL) || \
-     (strchr(text,'?') != (char *) NULL) || \
-     (strchr(text,'{') != (char *) NULL) || \
-     (strchr(text,'}') != (char *) NULL) || \
-     (strchr(text,'[') != (char *) NULL) || \
-     (strchr(text,']') != (char *) NULL))
-#  if !defined(MagickLibPath)
-#   define MagickLibPath  "c:\\ImageMagick\\"
-#  endif
-#  if !defined(MagickModulesPath)
-#   define MagickModulesPath  "c:\\ImageMagick\\"
-#  endif
-#  if !defined(MagickSharePath)
-#   define MagickSharePath  "c:\\ImageMagick\\"
-#  endif
-#  define PreferencesDefaults  "~/."
 #  define ProcessPendingEvents(text)
+#if !defined(PreferencesDefaults)
+#  define PreferencesDefaults  "~\\."
+#endif /* PreferencesDefaults */
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers \
     SetErrorHandler(NTErrorHandler); \
@@ -238,34 +282,29 @@ extern "C" {
 #define AbsoluteValue(x)  ((x) < 0 ? -(x) : (x))
 #define False  0
 #define DegreesToRadians(x) (MagickPI*(x)/180.0)
-#define IsFaxImage(color) \
-  (IsMonochromeImage(image) && ((image)->columns <= 2560))
 #define IsGray(color)  \
   (((color).red == (color).green) && ((color).green == (color).blue))
-#define IsMono(color)  \
-  (IsGray(color) && ((Intensity(color) == 0) || (Intensity(color) == MaxRGB)))
 #define MagickIncarnate(x)  InitializeMagick(x)
 #define MagickEpsilon  1.0e-12
 #define MagickPI  3.14159265358979323846264338327950288419716939937510
+#define MagickSQ2PI 2.50662827463100024161235523934010416269302368164062
 #define Max(x,y)  (((x) > (y)) ? (x) : (y))
 #define Min(x,y)  (((x) < (y)) ? (x) : (y))
-#define QuantumTick(i,span) \
-  (((~((span)-i-1) & ((span)-i-2))+1) == ((span)-i-1))
+#define QuantumTick(i,span) ((((i) & 0xff) == 0) || (i == ((span)-1)))
 #define RadiansToDegrees(x) (180.0*(x)/MagickPI)
 #define ScaleColor5to8(x)  ((x) << 3)
 #define ScaleColor6to8(x)  ((x) << 2)
 #define Swap(x,y) ((x)^=(y), (y)^=(x), (x)^=(y))
 #define True  1
-#define TypeFilename  "type.mgk"
 
 /*
   3D effects.
 */
-#define AccentuateModulate  Upscale(80)
-#define HighlightModulate  Upscale(125)
-#define ShadowModulate  Upscale(135)
-#define DepthModulate  Upscale(185)
-#define TroughModulate  Upscale(110)
+#define AccentuateModulate  ScaleCharToQuantum(80)
+#define HighlightModulate  ScaleCharToQuantum(125)
+#define ShadowModulate  ScaleCharToQuantum(135)
+#define DepthModulate  ScaleCharToQuantum(185)
+#define TroughModulate  ScaleCharToQuantum(110)
 
 /*
   Define system symbols if not already defined.
@@ -285,6 +324,15 @@ extern "C" {
 #if defined(HasLTDL) || defined(_MAGICKMOD_)
 #define HasMODULES
 #endif
+
+/*
+  I/O defines.
+*/
+#if defined(WIN32) && !defined(Windows95) && defined(_VISUALC_)
+#define MagickSeek(file,offset,whence)  _lseeki64(file,offset,whence)
+#else
+#define MagickSeek(file,offset,whence)  lseek(file,offset,whence)
+#endif
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
