@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -54,6 +54,9 @@
   Include declarations.
 */
 #include "studio.h"
+#include "monitor.h"
+#include "list.h"
+#include "utility.h"
 #if defined(HasLCMS)
 #if defined(HAVE_LCMS_LCMS_H)
 #include <lcms/lcms.h>
@@ -94,7 +97,7 @@
 MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   ExceptionInfo *exception)
 {
-#define ChopImageText  "  Chop image...  "
+#define ChopImageTag  "Chop/Image"
 
   Image
     *chop_image;
@@ -132,8 +135,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       ((chop_info->y+(long) chop_info->height) < 0) ||
       (chop_info->x > (long) image->columns) ||
       (chop_info->y > (long) image->rows))
-    ThrowImageException(OptionError,"Unable to chop image",
-      "geometry does not contain image");
+    ThrowImageException(OptionError,"GeometryDoesNotContainImage",
+      "UnableToChopImage");
   clone_info=(*chop_info);
   if ((clone_info.x+(long) clone_info.width) > (long) image->columns)
     clone_info.width=(unsigned long) ((long) image->columns-clone_info.x);
@@ -184,7 +187,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     if (!SyncImagePixels(chop_image))
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(ChopImageText,y,image->rows);
+      if (!MagickMonitor(ChopImageTag,y,image->rows,exception))
+        break;
   }
   /*
     Extract chop image.
@@ -213,7 +217,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     if (!SyncImagePixels(chop_image))
       break;
     if (QuantumTick(i,image->rows))
-      MagickMonitor(ChopImageText,i,image->rows);
+      if (!MagickMonitor(ChopImageTag,i,image->rows,exception))
+        break;
   }
   return(chop_image);
 }
@@ -249,7 +254,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
 MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
 {
   Image
-    *coalesce_image;
+    *coalesce_image,
+    *previous_image;
 
   register const Image
     *next;
@@ -262,8 +268,8 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if (image->next == (Image *) NULL)
-    ThrowImageException(OptionError,"Unable to coalesce image",
-      "image sequence required");
+    ThrowImageException(ImageError,"ImageSequenceIsRequired",
+      "UnableToCoalesceImage");
   /*
     Clone first image in sequence.
   */
@@ -271,12 +277,36 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
   if (coalesce_image == (Image *) NULL)
     return((Image *) NULL);
   (void) memset(&coalesce_image->page,0,sizeof(RectangleInfo));
+  previous_image=coalesce_image;
   /*
     Coalesce image.
   */
   for (next=image->next; next != (Image *) NULL; next=next->next)
   {
-    coalesce_image->next=CloneImage(coalesce_image,0,0,True,exception);
+    switch (next->dispose)
+    {
+      case UndefinedDispose:
+      case NoneDispose:
+      {
+        coalesce_image->next=CloneImage(coalesce_image,0,0,True,exception);
+        if (coalesce_image->next != (Image *) NULL)
+          previous_image=coalesce_image;
+        break;
+      }
+      case BackgroundDispose:
+      {
+        coalesce_image->next=CloneImage(coalesce_image,0,0,True,exception);
+        if (coalesce_image->next != (Image *) NULL)
+          SetImage(coalesce_image->next,OpaqueOpacity);
+        break;
+      }
+      case PreviousDispose:
+      default:
+      {
+        coalesce_image->next=CloneImage(previous_image,0,0,True,exception);
+        break;
+      }
+    }
     if (coalesce_image->next == (Image *) NULL)
       {
         DestroyImageList(coalesce_image);
@@ -327,7 +357,7 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
 MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   ExceptionInfo *exception)
 {
-#define CropImageText  "  Crop image...  "
+#define CropImageTag  "Crop/Image"
 
   Image
     *crop_image;
@@ -362,8 +392,8 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
           ((geometry->y+(long) geometry->height) < 0) ||
           (geometry->x >= (long) image->columns) ||
           (geometry->y >= (long) image->rows))
-        ThrowImageException(OptionError,"Unable to crop image",
-          "geometry does not contain any part of the image");
+        ThrowImageException(OptionError,"GeometryDoesNotContainImage",
+          "UnableToCropImage");
     }
   page=(*geometry);
   if ((page.width != 0) || (page.height != 0))
@@ -399,12 +429,12 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
         page.y=0;
       if ((((long) page.width+page.x) > (long) image->columns) ||
           (((long) page.height+page.y) > (long) image->rows))
-        ThrowImageException(OptionError,"Unable to crop image",
-          "geometry does not contain image");
+        ThrowImageException(OptionError,"GeometryDoesNotContainImage",
+          "UnableToCropImage");
     }
   if ((page.width == 0) || (page.height == 0))
-    ThrowImageException(OptionError,"Unable to crop image",
-      "geometry dimensions are zero");
+    ThrowImageException(OptionError,"GeometryDimensionsAreZero",
+      "UnableToCropImage");
   if ((page.width == image->columns) && (page.height == image->rows) &&
       (page.x == 0) && (page.y == 0))
     return(CloneImage(image,0,0,True,exception));
@@ -414,12 +444,11 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   crop_image=CloneImage(image,page.width,page.height,True,exception);
   if (crop_image == (Image *) NULL)
     return((Image *) NULL);
+  crop_image->page.x=page.x;
+  crop_image->page.y=page.y;
   /*
     Extract crop image.
   */
-  crop_image->page=page;
-  if ((geometry->width == 0) || (geometry->height == 0))
-    memset(&crop_image->page,0,sizeof(RectangleInfo));
   for (y=0; y < (long) crop_image->rows; y++)
   {
     p=AcquireImagePixels(image,page.x,page.y+y,crop_image->columns,1,exception);
@@ -436,7 +465,8 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     if (!SyncImagePixels(crop_image))
       break;
     if (QuantumTick(y,crop_image->rows))
-      MagickMonitor(CropImageText,y,crop_image->rows-1);
+      if (!MagickMonitor(CropImageTag,y,crop_image->rows-1,exception))
+        break;
   }
   if (y < (long) crop_image->rows)
     {
@@ -503,25 +533,25 @@ MagickExport Image *DeconstructImages(const Image *image,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if (image->next == (Image *) NULL)
-    ThrowImageException(OptionError,"Unable to deconstruct image sequence",
-      "image sequence required");
+    ThrowImageException(ImageError,"ImageSequenceIsRequired",
+      "UnableToDeconstructImageSequence");
   /*
     Ensure the image are the same size.
   */
   for (next=image; next != (Image *) NULL; next=next->next)
   {
     if ((next->columns != image->columns) || (next->rows != image->rows))
-      ThrowImageException(OptionError,"Unable to deconstruct image sequence",
-        "image are not the same size");
+      ThrowImageException(OptionError,"ImagesAreNotTheSameSize",
+        "UnableToDeconstructImageSequence");
   }
   /*
     Allocate memory.
   */
   bounds=(RectangleInfo *)
-    AcquireMemory(GetImageListSize(image)*sizeof(RectangleInfo));
+    AcquireMemory(GetImageListLength(image)*sizeof(RectangleInfo));
   if (bounds == (RectangleInfo *) NULL)
-    ThrowImageException(OptionError,"Unable to deconstruct image sequence",
-      "Memory allocation failed");
+    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
+      "UnableToDeconstructImageSequence");
   /*
     Compute the bounding box for each next in the sequence.
   */
@@ -678,8 +708,8 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if (image->next == (Image *) NULL)
-    ThrowImageException(OptionError,"Unable to flatten image",
-      "image sequence required");
+    ThrowImageException(ImageError,"ImageSequenceIsRequired",
+      "UnableToFlattenImage");
   /*
     Clone first image in sequence.
   */
@@ -723,7 +753,7 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
 {
-#define FlipImageText  "  Flip image...  "
+#define FlipImageTag  "Flip/Image"
 
   Image
     *flip_image;
@@ -774,7 +804,8 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
     if (status == False)
       break;
     if (QuantumTick(y,flip_image->rows))
-      MagickMonitor(FlipImageText,y,flip_image->rows);
+      if (!MagickMonitor(FlipImageTag,y,flip_image->rows,exception))
+        break;
   }
   return(flip_image);
 }
@@ -807,7 +838,7 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 {
-#define FlopImageText  "  Flop image...  "
+#define FlopImageTag  "Flop/Image"
 
   Image
     *flop_image;
@@ -866,7 +897,8 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
     if (status == False)
       break;
     if (QuantumTick(y,flop_image->rows))
-      MagickMonitor(FlopImageText,y,flop_image->rows);
+      if (!MagickMonitor(FlopImageTag,y,flop_image->rows,exception))
+        break;
   }
   return(flop_image);
 }
@@ -899,7 +931,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 */
 MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
 {
-#define MosaicImageText  "  Create an image mosaic...  "
+#define MosaicImageTag  "Mosaic/Image"
 
   Image
     *mosaic_image;
@@ -911,7 +943,8 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
     *next;
 
   unsigned int
-    scene;
+    scene,
+    status;
 
   /*
     Determine mosaic bounding box.
@@ -920,9 +953,6 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
   assert(image->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (image->next == (Image *) NULL)
-    ThrowImageException(OptionError,"Unable to create image mosaic",
-      "image sequence required");
   page.width=image->columns;
   page.height=image->rows;
   page.x=0;
@@ -957,7 +987,10 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
   {
     (void) CompositeImage(mosaic_image,CopyCompositeOp,next,next->page.x,
       next->page.y);
-    MagickMonitor(MosaicImageText,scene++,GetImageListSize(image));
+    status=MagickMonitor(MosaicImageTag,scene++,GetImageListLength(image),
+      exception);
+    if (status == False)
+      break;
   }
   return(mosaic_image);
 }
@@ -976,12 +1009,14 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
 %  ProfileImage() adds or removes a ICM, IPTC, or generic profile from an
 %  image.  If the profile is NULL, it is removed from the image otherwise
 %  added.  Use a name of '*' and a profile of NULL to remove all profiles
-%  from the image.
+%  from the image. Ownership of the profile is transferred to ImageMagick
+%  (it should not be altered or deallocated) unless the clone option is set
+%  to True.
 %
 %  The format of the ProfileImage method is:
 %
 %      unsigned int ProfileImage(Image *image,const char *name,
-%        const unsigned char *profile,const size_t length)
+%        const unsigned char *profile,const size_t length,unsigned int clone)
 %
 %  A description of each parameter follows:
 %
@@ -993,10 +1028,13 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
 %
 %    o length: The length of the profile.
 %
+%    o clone: If set True, then copy the profile rather than taking
+%             ownership of it.
+%
 %
 */
 MagickExport unsigned int ProfileImage(Image *image,const char *name,
-  const unsigned char *profile,const size_t length)
+  const unsigned char *profile,const size_t length,unsigned int clone)
 {
   register long
     i,
@@ -1005,9 +1043,9 @@ MagickExport unsigned int ProfileImage(Image *image,const char *name,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (name == (const char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Unable to add/remove profile",
-      "no profile name was given");
-  if ((profile == (const void *) NULL) || (length == 0))
+    ThrowBinaryException(OptionError,"NoProfileNameWasGiven",
+      "UnableToAddOrRemoveProfile");
+  if ((profile == (const unsigned char *) NULL) || (length == 0))
     {
       /*
         Remove an ICM, IPTC, or generic profile from the image.
@@ -1048,12 +1086,20 @@ MagickExport unsigned int ProfileImage(Image *image,const char *name,
     {
       if (image->iptc_profile.length != 0)
         LiberateMemory((void **) &image->iptc_profile.info);
-      image->iptc_profile.info=(unsigned char *) AcquireMemory(length);
-      if (image->iptc_profile.info == (unsigned char *) NULL)
-        ThrowBinaryException(ResourceLimitError,"Unable to add IPTC profile",
-          "Memory allocation failed");
-      image->iptc_profile.length=length;
-      (void) memcpy(image->iptc_profile.info,profile,length);
+      if (clone)
+        {
+          image->iptc_profile.info=(unsigned char *) AcquireMemory(length);
+          if (image->iptc_profile.info == (unsigned char *) NULL)
+            ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+              "UnableToAddIPTCProfile");
+          image->iptc_profile.length=length;
+          (void) memcpy(image->iptc_profile.info,profile,length);
+        }
+      else
+        {
+          image->iptc_profile.length=length;
+          image->iptc_profile.info=(unsigned char *) profile;
+        }
       return(True);
     }
   if (LocaleCompare("icm",name) == 0)
@@ -1105,8 +1151,8 @@ MagickExport unsigned int ProfileImage(Image *image,const char *name,
             cmsOpenProfileFromMem((unsigned char *) profile,length);
           if ((image_profile == (cmsHPROFILE) NULL) ||
               (transform_profile == (cmsHPROFILE) NULL))
-            ThrowBinaryException(ResourceLimitError,"Unable to manage color",
-              "failed to open color profiles");
+            ThrowBinaryException(ResourceLimitError,"UnableToManageColor",
+              "UnableToOpenColorProfile");
           switch (cmsGetColorSpace(transform_profile))
           {
             case icSigCmykData: colorspace=CMYKColorspace; break;
@@ -1143,10 +1189,8 @@ MagickExport unsigned int ProfileImage(Image *image,const char *name,
                   transform_profile,TYPE_RGBA_16,intent,0);
             }
           if (transform == (cmsHTRANSFORM) NULL)
-            ThrowBinaryException(ResourceLimitError,"Unable to manage color",
-              "failed to create color transform");
-          if (image->colorspace == CMYKColorspace)
-            SetImageType(image,ColorSeparationMatteType);
+            ThrowBinaryException(ResourceLimitError,"UnableToManageColor",
+              "UnableToCreateColorTransform");
           for (y=0; y < (long) image->rows; y++)
           {
             q=GetImagePixels(image,0,y,image->columns,1);
@@ -1154,34 +1198,44 @@ MagickExport unsigned int ProfileImage(Image *image,const char *name,
               break;
             for (x=0; x < (long) image->columns; x++)
             {
-              alpha.red=XUpscale(q->red);
-              alpha.green=XUpscale(q->green);
-              alpha.blue=XUpscale(q->blue);
-              alpha.opacity=XUpscale(q->opacity);
+              alpha.red=ScaleQuantumToShort(q->red);
+              alpha.green=ScaleQuantumToShort(q->green);
+              alpha.blue=ScaleQuantumToShort(q->blue);
+              alpha.opacity=ScaleQuantumToShort(q->opacity);
               cmsDoTransform(transform,&alpha,&beta,1);
-              q->red=XDownscale(beta.red);
-              q->green=XDownscale(beta.green);
-              q->blue=XDownscale(beta.blue);
-              q->opacity=XDownscale(beta.opacity);
+              q->red=ScaleShortToQuantum(beta.red);
+              q->green=ScaleShortToQuantum(beta.green);
+              q->blue=ScaleShortToQuantum(beta.blue);
+              q->opacity=ScaleShortToQuantum(beta.opacity);
               q++;
             }
             if (!SyncImagePixels(image))
               break;
           }
-          if (image->colorspace == CMYKColorspace)
-            image->matte=False;
           cmsDeleteTransform(transform);
           cmsCloseProfile(image_profile);
           cmsCloseProfile(transform_profile);
+          if (colorspace == CMYKColorspace)
+            image->colorspace=CMYKColorspace;
+          else
+            image->colorspace=RGBColorspace;
 #endif
           LiberateMemory((void **) &image->color_profile.info);
         }
-      image->color_profile.info=(unsigned char *) AcquireMemory(length);
-      if (image->color_profile.info == (unsigned char *) NULL)
-        ThrowBinaryException(ResourceLimitError,"Unable to add ICM profile",
-          "Memory allocation failed");
-      image->color_profile.length=length;
-      (void) memcpy(image->color_profile.info,profile,length);
+      if (clone)
+        {
+          image->color_profile.info=(unsigned char *) AcquireMemory(length);
+          if (image->color_profile.info == (unsigned char *) NULL)
+            ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+              "UnableToAddICMProfile");
+          image->color_profile.length=length;
+          (void) memcpy(image->color_profile.info,profile,length);
+        }
+      else
+        {
+          image->color_profile.length=length;
+          image->color_profile.info=(unsigned char *) profile;
+        }
       return(True);
     }
   for (i=0; i < (long) image->generic_profiles; i++)
@@ -1189,21 +1243,36 @@ MagickExport unsigned int ProfileImage(Image *image,const char *name,
       break;
   if (i == (long) image->generic_profiles)
     {
-      image->generic_profile=(ProfileInfo *)
-        AcquireMemory((i+1)*sizeof(ProfileInfo));
       if (image->generic_profile == (ProfileInfo *) NULL)
-        ThrowBinaryException(ResourceLimitError,"Memory allocation failed",
+        image->generic_profile=(ProfileInfo *)
+          AcquireMemory((i+1)*sizeof(ProfileInfo));
+      else
+        ReacquireMemory((void **) &image->generic_profile,
+          (i+1)*sizeof(ProfileInfo));
+      if (image->generic_profile == (ProfileInfo *) NULL)
+        ThrowBinaryException(ResourceLimitWarning,"MemoryAllocationFailed",
           (char *) NULL)
       image->generic_profiles++;
+      image->generic_profile[i].length=0;
+      image->generic_profile[i].info=(unsigned char *) NULL;
+      image->generic_profile[i].name=AllocateString(name);
     }
   if (image->generic_profile[i].length != 0)
     LiberateMemory((void **) &image->generic_profile[i].info);
-  image->generic_profile[i].info=(unsigned char *) AcquireMemory(length);
-  if (image->generic_profile[i].info == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"Unable to add profile",
-      "Memory allocation failed");
-  image->generic_profile[i].length=length;
-  (void) memcpy(image->generic_profile[i].info,profile,length);
+  if (clone)
+    {
+      image->generic_profile[i].info=(unsigned char *) AcquireMemory(length);
+      if (image->generic_profile[i].info == (unsigned char *) NULL)
+        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+          "UnableToAddGenericProfile");
+      image->generic_profile[i].length=length;
+      (void) memcpy(image->generic_profile[i].info,profile,length);
+    }
+  else
+    {
+      image->generic_profile[i].length=length;
+      image->generic_profile[i].info=(unsigned char *) profile;
+    }
   return(True);
 }
 
@@ -1240,7 +1309,7 @@ MagickExport unsigned int ProfileImage(Image *image,const char *name,
 MagickExport Image *RollImage(const Image *image,const long x_offset,
   const long y_offset,ExceptionInfo *exception)
 {
-#define RollImageText  "  Roll image...  "
+#define RollImageTag  "Roll/Image"
 
   Image
     *roll_image;
@@ -1261,7 +1330,7 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
   register PixelPacket
     *q;
 
-  PointInfo
+  RectangleInfo
     offset;
 
   /*
@@ -1277,12 +1346,16 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
   /*
     Roll image.
   */
-  offset.x=x_offset % image->columns;
-  offset.y=y_offset % image->rows;
-  while (offset.x < 0.0)
+  offset.x=x_offset;
+  offset.y=y_offset;
+  while (offset.x < 0)
     offset.x+=image->columns;
-  while (offset.y < 0.0)
+  while (offset.x >= (long) image->columns)
+    offset.x-=image->columns;
+  while (offset.y < 0)
     offset.y+=image->rows;
+  while (offset.y >= (long) image->rows)
+    offset.y-=image->rows;
   for (y=0; y < (long) image->rows; y++)
   {
     /*
@@ -1308,7 +1381,8 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
         break;
     }
     if (QuantumTick(y,image->rows))
-      MagickMonitor(RollImageText,y,image->rows);
+      if (!MagickMonitor(RollImageTag,y,image->rows,exception))
+        break;
   }
   return(roll_image);
 }
@@ -1356,8 +1430,8 @@ MagickExport Image *ShaveImage(const Image *image,
 
   if (((2*shave_info->width) >= image->columns) ||
       ((2*shave_info->height) >= image->rows))
-    ThrowImageException(OptionError,"Unable to shave image",
-      "geometry does not contain image");
+    ThrowImageException(OptionError,"GeometryDoesNotContainImage",
+      "UnableToShaveImage");
   SetGeometry(image,&geometry);
   geometry.width-=2*shave_info->width;
   geometry.height-=2*shave_info->height;
@@ -1405,12 +1479,12 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
     *resize_image,
     *transform_image;
 
-  int
-    flags;
-
   RectangleInfo
     geometry;
 
+  unsigned int
+    flags;
+
   assert(image != (Image **) NULL);
   assert((*image)->signature == MagickSignature);
   transform_image=(*image);
@@ -1426,9 +1500,9 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
         Crop image to a user specified size.
       */
       crop_image=(Image *) NULL;
-      flags=GetImageGeometry(transform_image,crop_geometry,False,&geometry);
+      flags=ParseGravityGeometry(transform_image,crop_geometry,&geometry);
       if ((geometry.width == 0) || (geometry.height == 0) ||
-          ((flags & XValue) != 0) || ((flags & YValue) != 0) ||
+          ((flags & XiValue) != 0) || ((flags & PsiValue) != 0) ||
           (flags & PercentValue))
         crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
       else
@@ -1460,8 +1534,7 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
                 geometry.height=height;
                 geometry.x=x;
                 geometry.y=y;
-                next=CropImage(transform_image,&geometry,
-                  &(*image)->exception);
+                next=CropImage(transform_image,&geometry,&(*image)->exception);
                 if (next == (Image *) NULL)
                   break;
                 if (crop_image == (Image *) NULL)
@@ -1491,15 +1564,10 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
   /*
     Scale image to a user specified size.
   */
-  SetGeometry(transform_image,&geometry);
-  flags=GetMagickGeometry(image_geometry,&geometry.x,&geometry.y,
-    &geometry.width,&geometry.height);
+  flags=ParseSizeGeometry(transform_image,image_geometry,&geometry);
   if ((transform_image->columns == geometry.width) &&
       (transform_image->rows == geometry.height))
     return;
-  /*
-    Resize image.
-  */
   resize_image=ZoomImage(transform_image,geometry.width,geometry.height,
     &(*image)->exception);
   if (resize_image == (Image *) NULL)
