@@ -18,7 +18,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -53,16 +53,20 @@
   Include declarations.
 */
 #include "studio.h"
+#include "blob.h"
+#include "render.h"
+#include "utility.h"
 
 /*
   Define declarations.
 */
+#define TypeFilename  "type.mgk"
 
 /*
   Declare type map.
 */
 static char
-  *TypeMap =
+  *TypeMap = (char *)
     "<?xml version=\"1.0\"?>"
     "<typemap>"
     "  <type stealth=\"True\" />"
@@ -81,7 +85,7 @@ static TypeInfo
   Forward declarations.
 */
 static unsigned int
-  ReadConfigurationFile(const char *,ExceptionInfo *);
+  ReadConfigureFile(const char *,const unsigned long,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -115,8 +119,8 @@ MagickExport void DestroyTypeInfo(void)
   {
     type_info=p;
     p=p->next;
-    if (type_info->filename != (char *) NULL)
-      LiberateMemory((void **) &type_info->filename);
+    if (type_info->path != (char *) NULL)
+      LiberateMemory((void **) &type_info->path);
     if (type_info->name != (char *) NULL)
       LiberateMemory((void **) &type_info->name);
     if (type_info->description != (char *) NULL)
@@ -174,12 +178,39 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
   register TypeInfo
     *p;
 
-  AcquireSemaphoreInfo(&type_semaphore);
   if (type_list == (TypeInfo *) NULL)
-    (void) ReadConfigurationFile(TypeFilename,exception);
-  LiberateSemaphoreInfo(&type_semaphore);
+    {
+      AcquireSemaphoreInfo(&type_semaphore);
+      if (type_list == (TypeInfo *) NULL)
+        {
+          (void) ReadConfigureFile(TypeFilename,0,exception);
+#if defined(WIN32) || defined(__CYGWIN__)
+          {
+            TypeInfo
+              *type_info;
+
+            type_info=NTGetTypeList();
+            if (type_info != (TypeInfo *) NULL)
+              {
+                if (type_list == (TypeInfo *) NULL)
+                  type_list=type_info;
+                else
+                  {
+                    while (type_list->next != (TypeInfo *) NULL)
+                      type_list=type_list->next;
+                    type_list->next=type_info;
+                    type_info->previous=type_list;
+                    while (type_list->previous != (TypeInfo *) NULL)
+                      type_list=type_list->previous;
+                  }
+              }
+          }
+#endif
+        }
+      LiberateSemaphoreInfo(&type_semaphore);
+    }
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    return(type_list);
+    return((const TypeInfo *) type_list);
   /*
     Search for requested type.
   */
@@ -203,7 +234,7 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
         type_list=p;
       }
   LiberateSemaphoreInfo(&type_semaphore);
-  return(p);
+  return((const TypeInfo *) p);
 }
 
 /*
@@ -271,10 +302,14 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   static Fontmap
     fontmap[] =
     {
-      { "fixed", "courier" }, { "modern","courier" },
-      { "monotype corsiva", "courier" }, { "news gothic", "helvetica" },
-      { "system", "courier" }, { "terminal", "courier" },
-      { "wingdings", "symbol" }, { (char *) NULL, (char *) NULL }
+      { (char *) "fixed", (char *) "courier" },
+      { (char *) "modern",(char *) "courier" },
+      { (char *) "monotype corsiva", (char *) "courier" },
+      { (char *) "news gothic", (char *) "helvetica" },
+      { (char *) "system", (char *) "courier" },
+      { (char *) "terminal", (char *) "courier" },
+      { (char *) "wingdings", (char *) "symbol" },
+      { (char *) NULL, (char *) NULL }
     };
 
   unsigned long
@@ -306,7 +341,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
       continue;
     if ((weight != 0) && (p->weight != weight))
       continue;
-    return(p);
+    return((TypeInfo *) p);
   }
   /*
     Check for types in the same family.
@@ -353,7 +388,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
       }
   }
   if (type_info != (TypeInfo *) NULL)
-    return(type_info);
+    return((TypeInfo *) type_info);
   /*
     Check for table-based substitution match.
   */
@@ -374,14 +409,82 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   }
   if (type_info != (TypeInfo *) NULL)
     {
-      ThrowException(exception,TypeError,"Font substitution required",
+      ThrowException(exception,TypeError,"FontSubstitutionRequired",
         type_info->family);
-      return(type_info);
+      return((TypeInfo *) type_info);
     }
   if (family != (const char *) NULL)
     type_info=GetTypeInfoByFamily((const char *) NULL,style,stretch,weight,
       exception);
-  return(type_info);
+  return((TypeInfo *) type_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t T y p e l i s t                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetTypeList() returns any types that match the specified pattern.
+%
+%  The format of the GetTypeList function is:
+%
+%      char **GetTypeList(const char *pattern,unsigned long *number_types)
+%
+%  A description of each parameter follows:
+%
+%    o pattern: Specifies a pointer to a text string containing a pattern.
+%
+%    o number_types:  This integer returns the number of types in the list.
+%
+%
+*/
+MagickExport char **GetTypeList(const char *pattern,unsigned long *number_types)
+{
+  char
+    **typelist;
+
+  ExceptionInfo
+    exception;
+
+  register const TypeInfo
+    *p;
+
+  register long
+    i;
+
+  /*
+    Allocate type list.
+  */
+  assert(pattern != (char *) NULL);
+  assert(number_types != (unsigned long *) NULL);
+  *number_types=0;
+  GetExceptionInfo(&exception);
+  p=GetTypeInfo("*",&exception);
+  DestroyExceptionInfo(&exception);
+  if (p == (const TypeInfo *) NULL)
+    return((char **) NULL);
+  i=0;
+  for (p=type_list; p != (const TypeInfo *) NULL; p=p->next)
+    i++;
+  typelist=(char **) AcquireMemory(i*sizeof(char *));
+  if (typelist == (char **) NULL)
+    return((char **) NULL);
+  i=0;
+  for (p=type_list; p != (const TypeInfo *) NULL; p=p->next)
+  {
+    if (p->stealth)
+      continue;
+    if (GlobExpression(p->name,pattern))
+      typelist[i++]=AllocateString(p->name);
+  }
+  *number_types=i;
+  return(typelist);
 }
 
 /*
@@ -409,10 +512,49 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
 %
 %
 */
+
+static const char *StretchTypeToString(StretchType stretch)
+{
+  switch(stretch)
+  {
+    case NormalStretch: return("normal");
+    case UltraCondensedStretch: return("ultra-condensed");
+    case ExtraCondensedStretch: return("extra-condensed");
+    case CondensedStretch: return("condensed");
+    case SemiCondensedStretch: return("semi-condensed");
+    case SemiExpandedStretch: return("semi-expanded");
+    case ExpandedStretch: return("expanded");
+    case ExtraExpandedStretch: return("extra-expanded");
+    case UltraExpandedStretch: return("ultra-expanded");
+    case AnyStretch: return("any");
+    default: break;
+  }
+  return("unknown");
+}
+
+static const char *StyleTypeToString(StyleType style)
+{
+  switch(style)
+  {
+    case NormalStyle: return("normal");
+    case ItalicStyle: return("italic");
+    case ObliqueStyle: return("oblique");
+    case AnyStyle: return("any");
+    default:  break;
+  }
+  return("unknown");
+}
+
 MagickExport unsigned int ListTypeInfo(FILE *file,ExceptionInfo *exception)
 {
-  register long
-    i;
+  char
+    weight[MaxTextExtent];
+
+  const char
+    *family,
+    *name,
+    *stretch,
+    *style;
 
   register const TypeInfo
     *p;
@@ -424,23 +566,30 @@ MagickExport unsigned int ListTypeInfo(FILE *file,ExceptionInfo *exception)
   for (p=type_list; p != (const TypeInfo *) NULL; p=p->next)
   {
     if ((p->previous == (TypeInfo *) NULL) ||
-        (LocaleCompare(p->filename,p->previous->filename) != 0))
+        (LocaleCompare(p->path,p->previous->path) != 0))
       {
         if (p->previous != (TypeInfo *) NULL)
           (void) fprintf(file,"\n");
-        (void) fprintf(file,"Filename: %.1024s\n\n",p->filename);
-        (void) fprintf(file,"Name                             Description\n");
-        (void) fprintf(file,"-------------------------------------------------"
+        if (p->path != (char *) NULL)
+          (void) fprintf(file,"Path: %.1024s\n\n",p->path);
+        (void) fprintf(file,"%-32.32s %-23.23s %-7.7s %-8s %-3s\n",
+          "Name","Family","Style","Stretch","Weight");
+        (void) fprintf(file,"--------------------------------------------------"
           "------------------------------\n");
       }
     if (p->stealth)
       continue;
-    (void) fprintf(file,"%.1024s",p->name);
-    for (i=(long) strlen(p->name); i <= 32; i++)
-      (void) fprintf(file," ");
-    if (p->description != (char *) NULL)
-      (void) fprintf(file," %.1024s",p->description);
-    (void) fprintf(file,"\n");
+    name="unknown";
+    if (p->name != (char *) NULL)
+      name=p->name;
+    family="unknown";
+    if (p->family != (char *) NULL)
+      family=p->family;
+    style=StyleTypeToString(p->style);
+    stretch=StretchTypeToString(p->stretch);
+    FormatString(weight,"%lu",p->weight);
+    (void) fprintf(file,"%-32.32s %-23.23s %-7.7s %-9s %-3s\n",
+      name,family,style,stretch,weight);
   }
   (void) fflush(file);
   LiberateSemaphoreInfo(&type_semaphore);
@@ -452,38 +601,39 @@ MagickExport unsigned int ListTypeInfo(FILE *file,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   R e a d C o n f i g u r a t i o n F i l e                                 %
++   R e a d C o n f i g u r e F i l e                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadConfigurationFile() reads the type configuration file which provides
+%  ReadConfigureFile() reads the type configuration file which provides
 %  a mapping between type attributes and font files.
 %
-%  The format of the ReadConfigurationFile method is:
+%  The format of the ReadConfigureFile method is:
 %
-%      unsigned int ReadConfigurationFile(const char *basename,
-%        ExceptionInfo *exception)
+%      unsigned int ReadConfigureFile(const char *basename,
+%        const unsigned long depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o status: ReadConfigurationFile() returns True if at least one entry
+%    o status: ReadConfigureFile() returns True if at least one entry
 %      is read, otherwise False is returned.
 %
 %    o basename:  The type configuration filename.
 %
+%    o depth: depth of <include /> statements.
+%
 %    o exception: Return any errors or warnings in this structure.
 %
 %
 */
-static unsigned int ReadConfigurationFile(const char *basename,
-  ExceptionInfo *exception)
+static unsigned int ReadConfigureFile(const char *basename,
+  const unsigned long depth,ExceptionInfo *exception)
 {
   char
-    filename[MaxTextExtent],
     keyword[MaxTextExtent],
-    *path,
+    path[MaxTextExtent],
     *q,
     *token,
     *xml;
@@ -492,16 +642,13 @@ static unsigned int ReadConfigurationFile(const char *basename,
     length;
 
   /*
-    Read the type configuration file.
+    Read the type configure file.
   */
-  FormatString(filename,"%.1024s",basename);
-  path=GetMagickConfigurePath(filename,exception);
-  if (path != (char *) NULL)
-    {
-      FormatString(filename,"%.1024s",path);
-      LiberateMemory((void **) &path);
-    }
-  xml=(char *) FileToBlob(filename,&length,exception);
+  (void) strcpy(path,basename);
+  if (depth == 0)
+    xml=(char *) GetTypeBlob(basename,path,&length,exception);
+  else
+    xml=(char *) FileToBlob(basename,&length,exception);
   if (xml == (char *) NULL)
     xml=AllocateString(TypeMap);
   token=AllocateString(xml);
@@ -513,33 +660,48 @@ static unsigned int ReadConfigurationFile(const char *basename,
     GetToken(q,&q,token);
     if (*token == '\0')
       break;
-    strncpy(keyword,token,MaxTextExtent-1);
-    if (LocaleCompare(keyword,"<!") == 0)
+    (void) strncpy(keyword,token,MaxTextExtent-1);
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
-          Comment.
+          Comment element.
         */
-        while ((*token != '>') && (*q != '\0'))
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
           GetToken(q,&q,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
       {
         /*
-          Include.
+          Include element.
         */
         while ((*token != '>') && (*q != '\0'))
         {
-          strncpy(keyword,token,MaxTextExtent-1);
+          (void) strncpy(keyword,token,MaxTextExtent-1);
           GetToken(q,&q,token);
           if (*token != '=')
             continue;
           GetToken(q,&q,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              (void) ReadConfigurationFile(token,exception);
-              while (type_list->next != (TypeInfo *) NULL)
-                type_list=type_list->next;
+              if (depth > 200)
+                ThrowException(exception,ConfigureError,
+                  "IncludeElementNestedTooDeeply",path);
+              else
+                {
+                  char
+                    filename[MaxTextExtent];
+
+                  GetPathComponent(path,HeadPath,filename);
+                  if (*filename != '\0')
+                    (void) strcat(filename,DirectorySeparator);
+                  (void) strncat(filename,token,MaxTextExtent-
+                    strlen(filename)-1);
+                  (void) ReadConfigureFile(filename,depth+1,exception);
+                }
+              if (type_list != (TypeInfo *) NULL)
+                while (type_list->next != (TypeInfo *) NULL)
+                  type_list=type_list->next;
             }
         }
         continue;
@@ -554,10 +716,10 @@ static unsigned int ReadConfigurationFile(const char *basename,
         */
         type_info=(TypeInfo *) AcquireMemory(sizeof(TypeInfo));
         if (type_info == (TypeInfo *) NULL)
-          MagickFatalError(ResourceLimitFatalError,"Unable to allocate fonts",
-            "Memory allocation failed");
+          MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
+            "UnableToAllocateTypeInfo");
         (void) memset(type_info,0,sizeof(TypeInfo));
-        type_info->filename=AcquireString(filename);
+        type_info->path=AcquireString(path);
         type_info->signature=MagickSignature;
         if (type_list == (TypeInfo *) NULL)
           {
@@ -618,7 +780,23 @@ static unsigned int ReadConfigurationFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"glyphs") == 0)
           {
-            type_list->glyphs=AcquireString(token);
+            char
+              *glyphs;
+
+            glyphs=(char *) NULL;
+            CloneString(&glyphs,token);
+#if defined(WIN32)
+            if (strchr(glyphs,'@') != (char *) NULL)
+              {
+                char
+                  path[MaxTextExtent];
+
+                NTGhostscriptFonts(path,MaxTextExtent-2);
+                (void) strcat(path,DirectorySeparator);
+                SubstituteString(&glyphs,"@ghostscript_font_dir@",path);
+              }
+#endif
+            type_list->glyphs=glyphs;
             break;
           }
         break;
@@ -628,7 +806,23 @@ static unsigned int ReadConfigurationFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"metrics") == 0)
           {
-            type_list->metrics=AcquireString(token);
+            char
+              *metrics;
+
+            metrics=(char *) NULL;
+            CloneString(&metrics,token);
+#if defined(WIN32)
+            if (strchr(metrics,'@') != (char *) NULL)
+              {
+                char
+                  path[MaxTextExtent];
+
+                NTGhostscriptFonts(path,MaxTextExtent-2);
+                (void) strcat(path,DirectorySeparator);
+                SubstituteString(&metrics,"@ghostscript_font_dir@",path);
+              }
+#endif
+            type_list->metrics=metrics;
             break;
           }
         break;
