@@ -8,7 +8,18 @@
 extern "C" {
 #endif
 
-
+/*
+  Enum declarations.
+*/
+typedef enum
+{
+  RootPath,
+  HeadPath,
+  TailPath,
+  BasePath,
+  ExtensionPath
+} PathType;
+
 /*
   Typedef declarations.
 */
@@ -34,30 +45,24 @@ extern MagickExport char
   *AllocateString(const char *),
   *Base64Encode(const unsigned char *,const size_t,size_t *),
   *EscapeString(const char *,const char),
-  *GetExecutionPath(const char *),
-  *GetPageGeometry(const char *),
   **ListFiles(const char *,const char *,long *),
-  *PostscriptGeometry(const char *),
   *SetClientName(const char *),
-  *SetClientPath(const char *),
   **StringToArgv(const char *,int *),
   **StringToList(const char *),
   *TranslateText(const ImageInfo *,Image *,const char *);
 
 extern MagickExport const char
-  *AcquireString(const char *);
+  *AcquireString(const char *),
+  *SetClientPath(const char *);
 
 extern MagickExport double
   StringToDouble(const char *,const double);
 
 extern MagickExport int
-  GetGeometry(const char *,long *,long *,unsigned long *,unsigned long *),
   GlobExpression(const char *,const char *),
   LocaleNCompare(const char *,const char *,const size_t),
   LocaleCompare(const char *,const char *),
-  GetMagickGeometry(const char *,long *,long *,unsigned long *,unsigned long *),
-  ParseImageGeometry(const char *,long *,long *,unsigned long *,
-    unsigned long *),
+  SubstituteString(char **,const char*,const char *),
   SystemCommand(const unsigned int,const char *),
   Tokenizer(TokenInfo *,unsigned,char *,size_t,char *,char *,char *,char *,
     char,char *,int *,char *);
@@ -69,8 +74,9 @@ extern MagickExport unsigned int
   CloneString(char **,const char *),
   ConcatenateString(char **,const char *),
   ExpandFilenames(int *,char ***),
+  GetExecutionPath(char *),
   IsAccessible(const char *),
-  IsGeometry(const char *);
+  IsGlob(const char *);
 
 extern MagickExport unsigned long
   MultilineCensus(const char *);
@@ -80,6 +86,7 @@ extern MagickExport void
   AppendImageFormat(const char *,char *),
   *CloneMemory(void *,const void *,const size_t),
   ExpandFilename(char *),
+  FormatSize(const ExtendedSignedIntegralType,char *),
   GetPathComponent(const char *,PathType,char *),
   GetToken(const char *,char **,char *),
   LiberateMemory(void **),
@@ -87,7 +94,6 @@ extern MagickExport void
   LocaleUpper(char *),
   ReacquireMemory(void **,const size_t),
   Strip(char *),
-  SetGeometry(const Image *,RectangleInfo *),
   TemporaryFilename(char *);
 
 extern MagickExport void
