@@ -18,7 +18,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -66,8 +66,15 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/xwindow.h"
+#include "studio.h"
+#include "cache.h"
+#include "command.h"
+#include "list.h"
+#include "log.h"
+#include "resource.h"
+#include "utility.h"
+#include "version.h"
+#include "xwindow.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -103,22 +110,24 @@ static void AnimateUsage(void)
     },
     *options[]=
     {
+      "-authenticate value  decrypt image with this password",
       "-backdrop            display image centered on a backdrop",
-      "-cache threshold     megabytes of memory available to the pixel cache",
       "-colormap type       Shared or Private",
       "-colors value        preferred number of colors in the image",
       "-colorspace type     alternate image colorspace",
       "-crop geometry       preferred size and location of the cropped image",
-      "-debug               display copious debugging information",
+      "-debug events        display copious debugging information",
       "-delay value         display the next image after pausing",
       "-density geometry    horizontal and vertical density of the image",
-      "-depth value         depth of the image",
+      "-depth value         image depth",
       "-display server      display image to this X server",
       "-dither              apply Floyd/Steinberg error diffusion to image",
       "-gamma value         level of gamma correction",
       "-geometry geometry   preferred size and location of the Image window",
       "-help                print program options",
       "-interlace type      None, Line, Plane, or Partition",
+      "-limit type value    Disk, Map, or Memory resource limit",
+      "-log format          format of debugging information",
       "-matte               store matte channel if the image has one",
       "-map type            display image using this Standard Colormap",
       "-monochrome          transform image to black and white",
@@ -126,23 +135,23 @@ static void AnimateUsage(void)
       "-pause               seconds to pause before reanimating",
       "-remote command      execute a command in an remote display process",
       "-rotate degrees      apply Paeth rotation to the image",
-      "-sampling_factor geometry",
+      "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
-      "-scenes range         image scene range",
+      "-scenes range        image scene range",
       "-size geometry       width and height of image",
-      "-treedepth value     depth of the color tree",
+      "-treedepth value     color tree depth",
       "-trim                trim image edges",
       "-verbose             print detailed information about the image",
+      "-version             print version information",
       "-visual type         display image using this visual type",
+      "-virtual-pixel method",
+      "                     Constant, Edge, Mirror, or Tile",
       "-window id           display image to background of this window",
       (char *) NULL
     };
 
-  unsigned int
-    version;
-
-  (void) printf("Version: %.1024s\n",GetMagickVersion(&version));
-  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
+  (void) printf("Version: %.1024s\n",GetMagickVersion((unsigned long *) NULL));
+  (void) printf("Copyright: %.1024s\n\n",GetMagickCopyright());
   (void) printf("Usage: %.1024s [options ...] file [ [options ...] file ...]\n",
     SetClientName((char *) NULL));
   (void) printf("\nWhere options include: \n");
@@ -154,7 +163,7 @@ static void AnimateUsage(void)
     "resources as command line options:  -background, -bordercolor,\n");
   (void) printf(
     "-borderwidth, -font, -foreground, -iconGeometry, -iconic, -name,\n");
-  (void) printf("-mattecolor, -shared_memory, or -title.\n");
+  (void) printf("-mattecolor, -shared-memory, or -title.\n");
   (void) printf(
     "\nBy default, the image format of `file' is determined by its magic\n");
   (void) printf(
@@ -211,9 +220,6 @@ int main(int argc,char **argv)
   Display
     *display;
 
-  double
-    sans;
-
   ExceptionInfo
     exception;
 
@@ -255,11 +261,11 @@ int main(int argc,char **argv)
   /*
     Initialize command line arguments.
   */
-  ReadCommandlLine(argc,&argv);
   InitializeMagick(*argv);
+  ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == False)
-    MagickFatalError(ResourceLimitFatalError,"Memory allocation failed",
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
       (char *) NULL);
   /*
     Set defaults.
@@ -291,19 +297,26 @@ int main(int argc,char **argv)
         */
         i++;
         if (i == argc)
-          MagickFatalError(OptionFatalError,"Missing server name",option);
+          MagickFatalError(OptionFatalError,"MissingServerName",option);
         server_name=argv[i];
         break;
       }
     if (LocaleCompare("help",option+1) == 0)
       AnimateUsage();
+    if (LocaleCompare("version",option+1) == 0)
+      {
+        (void) fprintf(stdout,"Version: %.1024s\n",
+          GetMagickVersion((unsigned long *) NULL));
+        (void) fprintf(stdout,"Copyright: %.1024s\n\n",GetMagickCopyright());
+        Exit(0);
+      }
   }
   /*
     Get user defaults from X resource database.
   */
   display=XOpenDisplay(server_name);
   if (display == (Display *) NULL)
-    MagickFatalError(XServerFatalError,"Unable to connect to X server",
+    MagickFatalError(XServerFatalError,"UnableToOpenXServer",
       XDisplayName(server_name));
  (void) XSetErrorHandler(XError);
   client_name=SetClientName((char *) NULL);
@@ -327,7 +340,7 @@ int main(int argc,char **argv)
   if (LocaleCompare("Partition",resource_value) == 0)
     image_info->interlace=PartitionInterlace;
   if (image_info->interlace == UndefinedInterlace)
-    MagickError(OptionError,"Unrecognized interlace type",resource_value);
+    MagickError(OptionFatalError,"InvalidImageInterlace",resource_value);
   resource_value=
     XGetResourceInstance(resource_database,client_name,"verbose","False");
   image_info->verbose=IsTrue(resource_value);
@@ -381,8 +394,7 @@ int main(int argc,char **argv)
           image_info->dither=quantize_info->dither;
           next_image=ReadImage(image_info,&exception);
           if (exception.severity != UndefinedException)
-            MagickError(exception.severity,exception.reason,
-              exception.description);
+            CatchException(&exception);
           status&=next_image != (Image *) NULL;
           if (next_image == (Image *) NULL)
             continue;
@@ -404,13 +416,29 @@ int main(int argc,char **argv)
       {
         status&=MogrifyImages(image_info,i-j,argv+j,&image);
         (void) CatchImageException(image);
-        PushImageList(&image_list,image,&exception);
-        DestroyImageList(image);
+        AppendImageToList(&image_list,image);
         image=(Image *) NULL;
         j=k+1;
       }
     switch (*(option+1))
     {
+      case 'a':
+      {
+        if (LocaleCompare("authenticate",option+1) == 0)
+          {
+            (void) CloneString(&image_info->authenticate,(char *) NULL);
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingPassword",option);
+                (void) CloneString(&image_info->authenticate,argv[i]);
+              }
+            break;
+          }
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
+        break;
+      }
       case 'b':
       {
         if (LocaleCompare("backdrop",option+1) == 0)
@@ -424,7 +452,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing color",option);
+                  MagickFatalError(OptionFatalError,"MissingColor",option);
                 resource_info.background_color=argv[i];
                 (void) QueryColorDatabase(argv[i],&image_info->background_color,
                   &exception);
@@ -437,7 +465,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing color",option);
+                  MagickFatalError(OptionFatalError,"MissingColor",option);
                 resource_info.border_color=argv[i];
                 (void) QueryColorDatabase(argv[i],&image_info->border_color,
                   &exception);
@@ -451,25 +479,25 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing width",option);
+                  MagickFatalError(OptionFatalError,"MissingWidth",option);
                 resource_info.border_width=atoi(argv[i]);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'c':
       {
         if (LocaleCompare("cache",option+1) == 0)
           {
-            SetCacheThreshold(0);
             if (*option == '-')
               {
                 i++;
-                if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
-                  MagickFatalError(OptionFatalError,"Missing threshold",option);
-                SetCacheThreshold(atol(argv[i]));
+                if ((i == argc) || !sscanf(argv[i],"%ld",&x))
+                  MagickFatalError(OptionFatalError,"MissingThreshold",option);
+                SetMagickResourceLimit(MemoryResource,atol(argv[i]));
+                SetMagickResourceLimit(MapResource,2*atol(argv[i]));
               }
             break;
           }
@@ -480,7 +508,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 resource_info.colormap=UndefinedColormap;
                 if (LocaleCompare("private",option) == 0)
@@ -488,7 +516,7 @@ int main(int argc,char **argv)
                 if (LocaleCompare("shared",option) == 0)
                   resource_info.colormap=SharedColormap;
                 if (resource_info.colormap == UndefinedColormap)
-                  MagickFatalError(OptionFatalError,"Invalid colormap type",
+                  MagickFatalError(OptionFatalError,"InvalidColormapType",
                     option);
               }
             break;
@@ -500,7 +528,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing colors",option);
+                  MagickFatalError(OptionFatalError,"MissingColors",option);
                 quantize_info->number_colors=atol(argv[i]);
               }
             break;
@@ -512,7 +540,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 quantize_info->colorspace=UndefinedColorspace;
                 if (LocaleCompare("cmyk",option) == 0)
@@ -544,7 +572,7 @@ int main(int argc,char **argv)
                 if (LocaleCompare("yuv",option) == 0)
                   quantize_info->colorspace=YUVColorspace;
                 if (quantize_info->colorspace == UndefinedColorspace)
-                  MagickFatalError(OptionFatalError,"Invalid colorspace type",
+                  MagickFatalError(OptionFatalError,"InvalidColorspaceType",
                     option);
               }
             break;
@@ -555,18 +583,26 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'd':
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            resource_info.debug=(*option == '-');
+            (void) SetLogEventMask("None");
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingEventMask",
+                    option);
+                (void) SetLogEventMask(argv[i]);
+              }
             break;
           }
         if (LocaleCompare("delay",option+1) == 0)
@@ -575,7 +611,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing seconds",option);
+                  MagickFatalError(OptionFatalError,"MissingSeconds",option);
               }
             break;
           }
@@ -586,7 +622,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 (void) CloneString(&image_info->density,argv[i]);
               }
             break;
@@ -598,8 +634,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing image depth",
-                    option);
+                  MagickFatalError(OptionFatalError,"MissingImageDepth",option);
                 image_info->depth=atol(argv[i]);
               }
             break;
@@ -611,8 +646,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing server name",
-                    option);
+                  MagickFatalError(OptionFatalError,"MissingServerName",option);
                 (void) CloneString(&image_info->server_name,argv[i]);
               }
             break;
@@ -622,7 +656,7 @@ int main(int argc,char **argv)
             quantize_info->dither=(*option == '-');
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'f':
@@ -634,7 +668,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing font name",option);
+                  MagickFatalError(OptionFatalError,"MissingFontName",option);
                 (void) CloneString(&image_info->font,argv[i]);
               }
             if ((image_info->font == (char *) NULL) ||
@@ -648,13 +682,13 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing foreground",
+                  MagickFatalError(OptionFatalError,"MissingForeground",
                     option);
                 resource_info.foreground_color=argv[i];
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'g':
@@ -662,7 +696,7 @@ int main(int argc,char **argv)
         if (LocaleCompare("gamma",option+1) == 0)
           {
             i++;
-            if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
+            if ((i == argc) || !sscanf(argv[i],"%ld",&x))
               MagickFatalError(OptionFatalError,"Missing value",option);
             break;
           }
@@ -673,12 +707,12 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 resource_info.image_geometry=argv[i];
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'h':
@@ -688,7 +722,7 @@ int main(int argc,char **argv)
             AnimateUsage();
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'i':
@@ -700,7 +734,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 resource_info.icon_geometry=argv[i];
               }
             break;
@@ -716,7 +750,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 image_info->interlace=UndefinedInterlace;
                 if (LocaleCompare("None",option) == 0)
@@ -728,12 +762,58 @@ int main(int argc,char **argv)
                 if (LocaleCompare("Partition",option) == 0)
                   image_info->interlace=PartitionInterlace;
                 if (image_info->interlace == UndefinedInterlace)
-                  MagickFatalError(OptionFatalError,"Invalid interlace type",
+                  MagickFatalError(OptionFatalError,"InvalidInterlaceType",
                     option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
+        break;
+      }
+      case 'l':
+      {
+        if (LocaleCompare("limit",option+1) == 0)
+          {
+            if (*option == '-')
+              {
+                char
+                  *type;
+
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingResourceType",
+                    option);
+                type=argv[i];
+                i++;
+                if ((i == argc) || !sscanf(argv[i],"%ld",&x))
+                  MagickFatalError(OptionFatalError,"MissingResourceLimit",
+                    option);
+                if (LocaleCompare("disk",type) == 0)
+                  SetMagickResourceLimit(DiskResource,atol(argv[i]));
+                else
+                  if (LocaleCompare("map",type) == 0)
+                    SetMagickResourceLimit(MapResource,atol(argv[i]));
+                  else
+                    if (LocaleCompare("memory",type) == 0)
+                      SetMagickResourceLimit(MemoryResource,atol(argv[i]));
+                    else
+                      MagickFatalError(OptionFatalError,
+                        "UnrecognizedResourceType",type);
+              }
+            break;
+          }
+        if (LocaleCompare("log",option+1) == 0)
+          {
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingLogFormat",option);
+                (void) SetLogFormat(argv[i]);
+              }
+            break;
+          }
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'm':
@@ -746,7 +826,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing map type",option);
+                  MagickFatalError(OptionFatalError,"MissingMapType",option);
                 resource_info.map_type=argv[i];
               }
             break;
@@ -759,7 +839,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing color",option);
+                  MagickFatalError(OptionFatalError,"MissingColor",option);
                 resource_info.matte_color=argv[i];
                 (void) QueryColorDatabase(argv[i],&image_info->matte_color,
                   &exception);
@@ -777,7 +857,7 @@ int main(int argc,char **argv)
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'n':
@@ -789,14 +869,14 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing name",option);
+                  MagickFatalError(OptionFatalError,"MissingName",option);
                 resource_info.name=argv[i];
               }
             break;
           }
         if (LocaleCompare("noop",option+1) == 0)
           break;
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'p':
@@ -808,7 +888,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing seconds",option);
+                  MagickFatalError(OptionFatalError,"MissingSeconds",option);
               }
             resource_info.pause=atoi(argv[i]);
             break;
@@ -821,30 +901,30 @@ int main(int argc,char **argv)
           {
             i++;
             if (i == argc)
-              MagickFatalError(OptionFatalError,"Missing command",option);
-            XRemoteCommand(display,resource_info.window_id,argv[i]);
-            Exit(0);
+              MagickFatalError(OptionFatalError,"MissingCommand",option);
+            status=XRemoteCommand(display,resource_info.window_id,argv[i]);
+            Exit(!status);
           }
         if (LocaleCompare("rotate",option+1) == 0)
           {
             i++;
             if ((i == argc) || !IsGeometry(argv[i]))
-              MagickFatalError(OptionFatalError,"Missing degrees",option);
+              MagickFatalError(OptionFatalError,"MissingDegrees",option);
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 's':
       {
-        if (LocaleCompare("sampling_factor",option+1) == 0)
+        if (LocaleCompare("sampling-factor",option+1) == 0)
           {
             (void) CloneString(&image_info->sampling_factor,(char *) NULL);
             if (*option == '-')
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 (void) CloneString(&image_info->sampling_factor,argv[i]);
               }
             break;
@@ -857,7 +937,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing scene number",
+                  MagickFatalError(OptionFatalError,"MissingSceneNumber",
                     option);
                 first_scene=atol(argv[i]);
                 last_scene=first_scene;
@@ -865,7 +945,7 @@ int main(int argc,char **argv)
               }
             break;
           }
-        if (LocaleCompare("shared_memory",option+1) == 0)
+        if (LocaleCompare("shared-memory",option+1) == 0)
           {
             resource_info.use_shared_memory=(*option == '-');
             break;
@@ -877,24 +957,24 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 (void) CloneString(&image_info->size,argv[i]);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 't':
       {
-        if (LocaleCompare("text_font",option+1) == 0)
+        if (LocaleCompare("text-font",option+1) == 0)
           {
             resource_info.text_font=(char *) NULL;
             if (*option == '-')
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing font name",option);
+                  MagickFatalError(OptionFatalError,"MissingFontName",option);
                 resource_info.text_font=argv[i];
               }
             break;
@@ -906,7 +986,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing title",option);
+                  MagickFatalError(OptionFatalError,"MissingTitle",option);
                 resource_info.title=argv[i];
               }
             break;
@@ -918,14 +998,14 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing depth",option);
+                  MagickFatalError(OptionFatalError,"MissingDepth",option);
                 quantize_info->tree_depth=atoi(argv[i]);
               }
             break;
           }
         if (LocaleCompare("trim",option+1) == 0)
           break;
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'v':
@@ -935,6 +1015,35 @@ int main(int argc,char **argv)
             image_info->verbose=(*option == '-');
             break;
           }
+        if (LocaleCompare("version",option+1) == 0)
+          break;
+        if (LocaleCompare("virtual-pixel",option+1) == 0)
+          {
+            if (*option == '-')
+              {
+                VirtualPixelMethod
+                  virtual_pixel_method;
+
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingVirtualPixelMethod",
+                    option);
+                option=argv[i];
+                virtual_pixel_method=UndefinedVirtualPixelMethod;
+                if (LocaleCompare("Constant",option) == 0)
+                  virtual_pixel_method=ConstantVirtualPixelMethod;
+                if (LocaleCompare("Edge",option) == 0)
+                  virtual_pixel_method=EdgeVirtualPixelMethod;
+                if (LocaleCompare("Mirror",option) == 0)
+                  virtual_pixel_method=MirrorVirtualPixelMethod;
+                if (LocaleCompare("Tile",option) == 0)
+                  virtual_pixel_method=TileVirtualPixelMethod;
+                if (virtual_pixel_method == UndefinedVirtualPixelMethod)
+                  MagickFatalError(OptionFatalError,
+                    "UnrecognizedVirtualPixelMethod",option);
+              }
+            break;
+          }
         if (LocaleCompare("visual",option+1) == 0)
           {
             resource_info.visual_type=(char *) NULL;
@@ -942,13 +1051,13 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing visual class",
+                  MagickFatalError(OptionFatalError,"MissingVisualClass",
                     option);
                 resource_info.visual_type=argv[i];
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'w':
@@ -960,13 +1069,13 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,
-                    "Missing id, name, or 'root'",option);
+                  MagickFatalError(OptionFatalError,"MissingIDNameOrRoot",
+                    option);
                 resource_info.window_id=argv[i];
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case '?':
@@ -976,15 +1085,14 @@ int main(int argc,char **argv)
       }
       default:
       {
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
     }
   }
   i--;
   if ((image == (Image *) NULL) && (image_list == (Image *) NULL))
-    MagickFatalError(OptionFatalError,"Missing an image file name",
-      (char *) NULL);
+    MagickFatalError(OptionFatalError,"MissingAnImageFilename",(char *) NULL);
   if (image == (Image *) NULL)
     {
       status&=MogrifyImages(image_info,i-j,argv+j,&image_list);
@@ -994,8 +1102,7 @@ int main(int argc,char **argv)
     {
       status&=MogrifyImages(image_info,i-j,argv+j,&image);
       (void) CatchImageException(image);
-      PushImageList(&image_list,image,&exception);
-      DestroyImageList(image);
+      AppendImageToList(&image_list,image);
     }
   if (resource_info.window_id != (char *) NULL)
     XAnimateBackgroundImage(display,&resource_info,image_list);
@@ -1013,9 +1120,14 @@ int main(int argc,char **argv)
       }
     }
   DestroyImageList(image_list);
-  LiberateMemory((void **) &argv);
   DestroyMagick();
+  for (i=0; i < argc; i++)
+    LiberateMemory((void **) &argv[i]);
+  LiberateMemory((void **) &argv);
   Exit(!status);
+#else
+  MagickFatalError(MissingDelegateError,"XWindowLibraryIsNotAvailable",
+    (char *) NULL);
 #endif
   return(False);
 }
