@@ -17,7 +17,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -68,7 +68,13 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
+#include "studio.h"
+#include "attribute.h"
+#include "command.h"
+#include "list.h"
+#include "log.h"
+#include "version.h"
+#include "utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -93,20 +99,19 @@ static void ConjureUsage(void)
   static const char
     *options[]=
     {
-      "-debug    display copious debugging information",
-      "-help     print program options",
-      "-verbose  print detailed information about the image",
+      "-debug events        display copious debugging information",
+      "-help                print program options",
+      "-log format          format of debugging information",
+      "-verbose             print detailed information about the image",
+      "-version             print version information",
       (char *) NULL
     };
 
   const char
     **p;
 
-  unsigned int
-    version;
-
-  (void) printf("Version: %.1024s\n",GetMagickVersion(&version));
-  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
+  (void) printf("Version: %.1024s\n",GetMagickVersion((unsigned long *) NULL));
+  (void) printf("Copyright: %.1024s\n\n",GetMagickCopyright());
   (void) printf("Usage: %.1024s [options ...] file [ [options ...] file ...]\n",
     SetClientName((char *) NULL));
   (void) printf("\nWhere options include:\n");
@@ -151,14 +156,11 @@ int main(int argc,char **argv)
   unsigned int
     status;
 
-  /*
-    Set defaults.
-  */
-  ReadCommandlLine(argc,&argv);
   InitializeMagick(*argv);
+  ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == False)
-    MagickFatalError(ResourceLimitFatalError,"Memory allocation failed",
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
       (char *) NULL);
   if (argc < 2)
     ConjureUsage();
@@ -173,7 +175,13 @@ int main(int argc,char **argv)
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            image_info->debug=(*option == '-');
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingEventMask",option);
+                (void) SetLogEventMask(argv[i]);
+              }
             continue;
           }
         if (LocaleCompare("help",option+1) == 0)
@@ -182,19 +190,38 @@ int main(int argc,char **argv)
               ConjureUsage();
             continue;
           }
+        if (LocaleCompare("log",option+1) == 0)
+          {
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingLogFormat",option);
+                (void) SetLogFormat(argv[i]);
+              }
+            continue;
+          }
         if (LocaleCompare("verbose",option+1) == 0)
           {
             image_info->verbose=(*option == '-');
             continue;
           }
+        if (LocaleCompare("version",option+1) == 0)
+          {
+            (void) fprintf(stdout,"Version: %.1024s\n",
+              GetMagickVersion((unsigned long *) NULL));
+            (void) fprintf(stdout,"Copyright: %.1024s\n\n",
+              GetMagickCopyright());
+            Exit(0);
+            continue;
+          }
         /*
           Persist key/value pair.
         */
         (void) SetImageAttribute(image_info->attributes,option+1,(char *) NULL);
         status&=SetImageAttribute(image_info->attributes,option+1,argv[i+1]);
         if (status == False)
-          MagickFatalError(ResourceLimitFatalError,"Unable to persist key",
-            option);
+          MagickFatalError(ImageFatalError,"UnableToPersistKey",option);
         i++;
         continue;
       }
@@ -204,17 +231,19 @@ int main(int argc,char **argv)
     (void) SetImageAttribute(image_info->attributes,"filename",(char *) NULL);
     status&=SetImageAttribute(image_info->attributes,"filename",argv[i]);
     if (status == False)
-      MagickFatalError(ResourceLimitFatalError,"Unable to persist key",argv[i]);
+      MagickFatalError(ImageFatalError,"UnableToPersistKey",argv[i]);
     (void) FormatString(image_info->filename,"msl:%.1024s",argv[i]);
     image=ReadImage(image_info,&exception);
     if (exception.severity != UndefinedException)
-      MagickError(exception.severity,exception.reason,exception.description);
+      CatchException(&exception);
     status&=image != (Image *) NULL;
     if (image != (Image *) NULL)
       DestroyImageList(image);
   }
   DestroyImageInfo(image_info);
   DestroyMagick();
+  for (i=0; i < argc; i++)
+    LiberateMemory((void **) &argv[i]);
   LiberateMemory((void **) &argv);
   Exit(!status);
   return(False);
