@@ -13,13 +13,12 @@
 %               Import image to a machine independent format.                 %
 %                                                                             %
 %                                                                             %
-%                                                                             %
 %                           Software Design                                   %
 %                             John Cristy                                     %
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
+%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
 %  to making software imaging solutions freely available.                     %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
@@ -61,8 +60,15 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/xwindow.h"
+#include "studio.h"
+#include "cache.h"
+#include "command.h"
+#include "list.h"
+#include "log.h"
+#include "resource.h"
+#include "utility.h"
+#include "version.h"
+#include "xwindow.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -93,54 +99,56 @@ static void ImportUsage(void)
     {
       "-adjoin              join images into a single multi-image file",
       "-border              include image borders in the output image",
-      "-cache threshold     megabytes of memory available to the pixel cache",
       "-colors value        preferred number of colors in the image",
       "-colorspace type     alternate image colorspace",
       "-comment string      annotate image with comment",
-      "-compress type       type of image compression",
+      "-compress type       image compression tyhpe",
       "-crop geometry       preferred size and location of the cropped image",
-      "-debug               display copious debugging information",
+      "-debug events        display copious debugging information",
       "-delay value         display the next image after pausing",
       "-density geometry    horizontal and vertical density of the image",
-      "-depth value         depth of the image",
+      "-depth value         image depth",
       "-descend             obtain image by descending window hierarchy",
       "-display server      X server to contact",
-      "-dispose method      GIF disposal method",
+      "-dispose method      Undefined, None, Background, Previous",
       "-dither              apply Floyd/Steinberg error diffusion to image",
       "-frame               include window manager frame",
+      "-encoding type       text encoding type",
       "-endian type         LSB or MSB",
       "-geometry geometry   perferred size or location of the image",
       "-interlace type      None, Line, Plane, or Partition",
       "-help                print program options",
       "-label name          assign a label to an image",
+      "-limit type value    Disk, Map, or Memory resource limit",
+      "-log format          format of debugging information",
       "-monochrome          transform image to black and white",
       "-negate              replace every pixel with its complementary color ",
       "-page geometry       size and location of an image canvas",
       "-pause value         seconds delay between snapshots",
-      "-pointsize value     pointsize of Postscript font",
+      "-pointsize value     font point size",
       "-quality value       JPEG/MIFF/PNG compression level",
       "-resize geometry     resize the image",
       "-rotate degrees      apply Paeth rotation to the image",
-      "-sampling_factor geometry",
+      "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-scene value         image scene number",
       "-screen              select image from root window",
       "-silent              operate silently, i.e. don't ring any bells ",
       "-snaps value         number of screen snapshots",
       "-transparent color   make this color transparent within the image",
-      "-treedepth value     depth of the color tree",
+      "-treedepth value     color tree depth",
       "-trim                trim image edges",
       "-type type           image type",
       "-verbose             print detailed information about the image",
+      "-version             print version information",
+      "-virtual-pixel method",
+      "                     Constant, Edge, Mirror, or Tile",
       "-window id           select window with this id or name",
       (char *) NULL
     };
 
-  unsigned int
-    version;
-
-  (void) printf("Version: %.1024s\n",GetMagickVersion(&version));
-  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
+  (void) printf("Version: %.1024s\n",GetMagickVersion((unsigned long *) NULL));
+  (void) printf("Copyright: %.1024s\n\n",GetMagickCopyright());
   (void) printf("Usage: %.1024s [options ...] [ file ]\n",
     SetClientName((char *) NULL));
   (void) printf("\nWhere options include:\n");
@@ -185,9 +193,6 @@ int main(int argc,char **argv)
   Display
     *display;
 
-  double
-    sans;
-
   ExceptionInfo
     exception;
 
@@ -220,14 +225,11 @@ int main(int argc,char **argv)
   XrmDatabase
     resource_database;
 
-  /*
-    Initialize command line arguments.
-  */
-  ReadCommandlLine(argc,&argv);
   InitializeMagick(*argv);
+  ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == False)
-    MagickFatalError(ResourceLimitFatalError,"Memory allocation failed",
+    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
       (char *) NULL);
   /*
     Check for server name specified on the command line.
@@ -248,13 +250,19 @@ int main(int argc,char **argv)
         */
         i++;
         if (i == argc)
-          MagickFatalError(OptionFatalError,"Missing server name on -display",
-            option);
+          MagickFatalError(OptionFatalError,"MissingServerName",option);
         server_name=argv[i];
         break;
       }
     if (LocaleCompare("help",option+1) == 0)
       ImportUsage();
+    if (LocaleCompare("version",option+1) == 0)
+      {
+        (void) fprintf(stdout,"Version: %.1024s\n",
+          GetMagickVersion((unsigned long *) NULL));
+        (void) fprintf(stdout,"Copyright: %.1024s\n\n",GetMagickCopyright());
+        Exit(0);
+      }
   }
   /*
     Get user defaults from X resource database.
@@ -262,7 +270,7 @@ int main(int argc,char **argv)
   SetNotifyHandlers;
   display=XOpenDisplay(server_name);
   if (display == (Display *) NULL)
-    MagickFatalError(OptionFatalError,"Unable to connect to X server",
+    MagickFatalError(OptionFatalError,"UnableToOpenXServer",
       XDisplayName(server_name));
   (void) XSetErrorHandler(XError);
   client_name=SetClientName((char *) NULL);
@@ -343,7 +351,7 @@ int main(int argc,char **argv)
             image_info->adjoin=(*option == '-');
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'b':
@@ -359,27 +367,27 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing border color",
+                  MagickFatalError(OptionFatalError,"MissingBorderColor",
                     option);
                 (void) QueryColorDatabase(argv[i],&image_info->border_color,
                   &exception);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'c':
       {
         if (LocaleCompare("cache",option+1) == 0)
           {
-            SetCacheThreshold(0);
             if (*option == '-')
               {
                 i++;
-                if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
-                  MagickFatalError(OptionFatalError,"Missing threshold",option);
-                SetCacheThreshold(atol(argv[i]));
+                if ((i == argc) || !sscanf(argv[i],"%ld",&x))
+                  MagickFatalError(OptionFatalError,"MissingThreshold",option);
+                SetMagickResourceLimit(MemoryResource,atol(argv[i]));
+                SetMagickResourceLimit(MapResource,2*atol(argv[i]));
               }
             break;
           }
@@ -390,7 +398,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing colors",option);
+                  MagickFatalError(OptionFatalError,"MissingColors",option);
                 quantize_info->number_colors=atol(argv[i]);
               }
             break;
@@ -402,7 +410,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 quantize_info->colorspace=UndefinedColorspace;
                 if (LocaleCompare("cmyk",option) == 0)
@@ -434,7 +442,7 @@ int main(int argc,char **argv)
                 if (LocaleCompare("yuv",option) == 0)
                   quantize_info->colorspace=YUVColorspace;
                 if (quantize_info->colorspace == UndefinedColorspace)
-                  MagickFatalError(OptionFatalError,"Invalid colorspace type",
+                  MagickFatalError(OptionFatalError,"InvalidColorspaceType",
                     option);
               }
             break;
@@ -456,7 +464,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 image_info->compression=UndefinedCompression;
                 if (LocaleCompare("None",option) == 0)
@@ -474,12 +482,12 @@ int main(int argc,char **argv)
                 if (LocaleCompare("LZW",option) == 0)
                   image_info->compression=LZWCompression;
                 if (LocaleCompare("RLE",option) == 0)
-                  image_info->compression=RunlengthEncodedCompression;
+                  image_info->compression=RLECompression;
                 if (LocaleCompare("Zip",option) == 0)
                   image_info->compression=ZipCompression;
                 if (image_info->compression == UndefinedCompression)
-                  MagickFatalError(OptionFatalError,"Invalid compression type",
-                    option);
+                  MagickFatalError(OptionFatalError,
+                    "UnrecognizedImageCompressionType",option);
               }
             break;
           }
@@ -489,18 +497,25 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'd':
       {
         if (LocaleCompare("debug",option+1) == 0)
           {
-            image_info->debug=(*option == '-');
+            (void) SetLogEventMask("None");
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingEventMask",option);
+                (void) SetLogEventMask(argv[i]);
+              }
             break;
           }
         if (LocaleCompare("delay",option+1) == 0)
@@ -509,7 +524,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing seconds",option);
+                  MagickFatalError(OptionFatalError,"MissingSeconds",option);
               }
             break;
           }
@@ -520,7 +535,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 (void) CloneString(&image_info->density,argv[i]);
               }
             break;
@@ -532,8 +547,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing image depth",
-                    option);
+                  MagickFatalError(OptionFatalError,"MissingImageDepth",option);
                 image_info->depth=atol(argv[i]);
               }
             break;
@@ -550,7 +564,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing server name",
+                  MagickFatalError(OptionFatalError,"MissingServerName",
                     option);
                 (void) CloneString(&image_info->server_name,argv[i]);
               }
@@ -561,8 +575,19 @@ int main(int argc,char **argv)
             if (*option == '-')
               {
                 i++;
-                if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing method",option);
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingDisposeMethod",
+                    option);
+                if ((LocaleCompare("0",option) != 0) &&
+                    (LocaleCompare("1",option) != 0) &&
+                    (LocaleCompare("2",option) != 0) &&
+                    (LocaleCompare("3",option) != 0) &&
+                    (LocaleCompare("Undefined",option) != 0) &&
+                    (LocaleCompare("None",option) != 0) &&
+                    (LocaleCompare("Background",option) != 0) &&
+                    (LocaleCompare("Previous",option) != 0))
+                  MagickFatalError(OptionFatalError,"UnrecognizedDisposeMethod",
+                    option);
               }
             break;
           }
@@ -571,11 +596,22 @@ int main(int argc,char **argv)
             quantize_info->dither=(*option == '-');
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'e':
       {
+        if (LocaleCompare("encoding",option+1) == 0)
+          {
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingEncodingType",
+                    option);
+              }
+            break;
+          }
         if (LocaleCompare("endian",option+1) == 0)
           {
             image_info->endian=UndefinedEndian;
@@ -583,7 +619,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 image_info->endian=UndefinedEndian;
                 if (LocaleCompare("LSB",option) == 0)
@@ -591,12 +627,11 @@ int main(int argc,char **argv)
                 if (LocaleCompare("MSB",option) == 0)
                   image_info->endian=MSBEndian;
                 if (image_info->endian == UndefinedEndian)
-                  MagickFatalError(OptionFatalError,"Invalid endian type",
-                    option);
+                  MagickFatalError(OptionFatalError,"InvalidEndianType",option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'f':
@@ -607,7 +642,7 @@ int main(int argc,char **argv)
             ximage_info.frame=(*option == '-');
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'g':
@@ -618,11 +653,11 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'h':
@@ -632,7 +667,7 @@ int main(int argc,char **argv)
             ImportUsage();
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'i':
@@ -644,7 +679,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 image_info->interlace=UndefinedInterlace;
                 if (LocaleCompare("No",option) == 0)
@@ -656,12 +691,12 @@ int main(int argc,char **argv)
                 if (LocaleCompare("Partition",option) == 0)
                   image_info->interlace=PartitionInterlace;
                 if (image_info->interlace == UndefinedInterlace)
-                  MagickFatalError(OptionFatalError,"Invalid interlace type",
+                  MagickFatalError(OptionFatalError,"InvalidInterlaceType",
                     option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'l':
@@ -672,12 +707,52 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing label name",
-                    option);
+                  MagickFatalError(OptionFatalError,"MissingLabelName",option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        if (LocaleCompare("limit",option+1) == 0)
+          {
+            if (*option == '-')
+              {
+                char
+                  *type;
+
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingResourceType",
+                    option);
+                type=argv[i];
+                i++;
+                if ((i == argc) || !sscanf(argv[i],"%ld",&x))
+                  MagickFatalError(OptionFatalError,"MissingResourceLimit",
+                    option);
+                if (LocaleCompare("disk",type) == 0)
+                  SetMagickResourceLimit(DiskResource,atol(argv[i]));
+                else
+                  if (LocaleCompare("map",type) == 0)
+                    SetMagickResourceLimit(MapResource,atol(argv[i]));
+                  else
+                    if (LocaleCompare("memory",type) == 0)
+                      SetMagickResourceLimit(MemoryResource,atol(argv[i]));
+                    else
+                      MagickFatalError(OptionFatalError,
+                        "UnrecognizedResourceType",type);
+              }
+            break;
+          }
+        if (LocaleCompare("log",option+1) == 0)
+          {
+            if (*option == '-')
+              {
+                i++;
+                if (i == argc)
+                  MagickFatalError(OptionFatalError,"MissingLogFormat",option);
+                (void) SetLogFormat(argv[i]);
+              }
+            break;
+          }
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'm':
@@ -693,14 +768,14 @@ int main(int argc,char **argv)
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'n':
       {
         if (LocaleCompare("negate",option+1) == 0)
           break;
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'p':
@@ -712,7 +787,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing page geometry",
+                  MagickFatalError(OptionFatalError,"MissingPageGeometry",
                     option);
                 image_info->page=GetPageGeometry(argv[i]);
               }
@@ -725,7 +800,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing seconds",option);
+                  MagickFatalError(OptionFatalError,"MissingSeconds",option);
                 resource_info.pause=atoi(argv[i]);
               }
             break;
@@ -742,29 +817,30 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing size",option);
+                  MagickFatalError(OptionFatalError,"MissingSize",option);
                 image_info->pointsize=atof(argv[i]);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'q':
       {
         if (LocaleCompare("quality",option+1) == 0)
           {
-            image_info->quality=0;
+            image_info->quality=DefaultCompressionQuality;
             if (*option == '-')
               {
                 i++;
-                if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing quality",option);
+                if ((i == argc) || !IsGeometry(argv[i]))
+                  MagickFatalError(OptionFatalError,"MissingCompressionQuality",
+                    option);
                 image_info->quality=atol(argv[i]);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'r':
@@ -775,7 +851,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
               }
             break;
           }
@@ -783,22 +859,22 @@ int main(int argc,char **argv)
           {
             i++;
             if ((i == argc) || !IsGeometry(argv[i]))
-               MagickFatalError(OptionFatalError,"Missing degrees",option);
+               MagickFatalError(OptionFatalError,"MissingDegrees",option);
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 's':
       {
-        if (LocaleCompare("sampling_factor",option+1) == 0)
+        if (LocaleCompare("sampling-factor",option+1) == 0)
           {
             (void) CloneString(&image_info->sampling_factor,(char *) NULL);
             if (*option == '-')
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 (void) CloneString(&image_info->sampling_factor,argv[i]);
               }
             break;
@@ -809,7 +885,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing scene number",
+                  MagickFatalError(OptionFatalError,"MissingSceneNumber",
                     option);
               }
             break;
@@ -831,7 +907,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !IsGeometry(argv[i]))
-                  MagickFatalError(OptionFatalError,"Missing geometry",option);
+                  MagickFatalError(OptionFatalError,"MissingGeometry",option);
                 (void) CloneString(&image_info->size,argv[i]);
               }
             break;
@@ -841,11 +917,11 @@ int main(int argc,char **argv)
             (void) strcpy(argv[i]+1,"sans");
             i++;
             if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-              MagickFatalError(OptionFatalError,"Missing snaps value",option);
+              MagickFatalError(OptionFatalError,"MissingSnaps",option);
             snapshots=atol(argv[i]);
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 't':
@@ -856,7 +932,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing color",option);
+                  MagickFatalError(OptionFatalError,"MissingColor",option);
               }
             break;
           }
@@ -867,7 +943,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if ((i == argc) || !sscanf(argv[i],"%ld",&x))
-                  MagickFatalError(OptionFatalError,"Missing depth",option);
+                  MagickFatalError(OptionFatalError,"MissingDepth",option);
                 quantize_info->tree_depth=atoi(argv[i]);
               }
             break;
@@ -881,7 +957,7 @@ int main(int argc,char **argv)
               {
                 i++;
                 if (i == argc)
-                  MagickFatalError(OptionFatalError,"Missing type",option);
+                  MagickFatalError(OptionFatalError,"MissingType",option);
                 option=argv[i];
                 if (LocaleCompare("Bilevel",option) == 0)
                   image_info->type=BilevelType;
@@ -904,20 +980,18 @@ int main(int argc,char **argv)
                 if (LocaleCompare("Optimize",option) == 0)
                   image_info->type=OptimizeType;
                 if (image_info->type == UndefinedType)
-                  MagickFatalError(OptionFatalError,"Invalid image type",
-                    option);
+                  MagickFatalError(OptionFatalError,"InvalidImageType",option);
               }
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case 'w':
       {
         i++;
         if (i == argc)
-          MagickFatalError(OptionFatalError,"Missing id, name, or 'root'",
-            option);
+          MagickFatalError(OptionFatalError,"MissingIDNameOrRoot",option);
         (void) CloneString(&target_window,argv[i]);
         break;
       }
@@ -928,7 +1002,9 @@ int main(int argc,char **argv)
             image_info->verbose=(*option == '-');
             break;
           }
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        if (LocaleCompare("version",option+1) == 0)
+          break;
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
       case '?':
@@ -938,7 +1014,7 @@ int main(int argc,char **argv)
       }
       default:
       {
-        MagickFatalError(OptionFatalError,"Unrecognized option",option);
+        MagickFatalError(OptionFatalError,"UnrecognizedOption",option);
         break;
       }
     }
@@ -970,8 +1046,7 @@ int main(int argc,char **argv)
     image=next_image;
   }
   if (image == (Image *) NULL)
-    MagickFatalError(OptionFatalError,"Missing an image file name",
-      (char *) NULL);
+    MagickFatalError(OptionFatalError,"MissingAnImageFilename",(char *) NULL);
   while (image->previous != (Image *) NULL)
     image=image->previous;
   status&=MogrifyImages(image_info,argc-1,argv,&image);
@@ -979,8 +1054,13 @@ int main(int argc,char **argv)
   status&=WriteImages(image_info,image,filename,&image->exception);
   DestroyImageList(image);
   DestroyMagick();
+  for (i=0; i < argc; i++)
+    LiberateMemory((void **) &argv[i]);
   LiberateMemory((void **) &argv);
   Exit(!status);
+#else
+  MagickFatalError(MissingDelegateError,"XWindowLibraryIsNotAvailable",
+    (char *) NULL);
 #endif
   return(False);
 }
